unit untDecayProtection;
{**************************************************************************************}
{**************************** This file is part of SoilGen ****************************}
{****************************SoilGen is free software:*********************************}
{you can redistribute it and/or modify it under the terms******************************}
{of the GNU General Public License as published by the Free Software Foundation,*******}
{either version 3 of the License, or (at your option) any later version.***************}
{  It would be polite to refer to the SoilGen model by mentioning the documentation:  }
{  Peter Finke. 2024. Modelling soil development under global change.                 }
{  SpringerBriefs in Earth System Sciences 10032. ISBN: 978-3-031-55582-4.            }
{  https://doi.org/10.1007/978-3-031-55583-1                                          }
{**************************************************************************************}
{SoilGen is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;**}
{without even the implied warranty of MERCHANTABILITY or ******************************}
{FITNESS FOR A PARTICULAR PURPOSE.*****************************************************}
{See the GNU General Public License for more details.**********************************}
{**************************************************************************************}
{You should have received a copy of the GNU General Public License along with SoilGen.*}
{If not, see <https://www.gnu.org/licenses/>.******************************************}
{$mode Delphi}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, Grids, Buttons,
  StdCtrls, ExtCtrls;

type

  { TfrmDecayProtection }

  TfrmDecayProtection = class(TForm)
    btbttnClose: TBitBtn;
    chckbxF1: TCheckBox;
    chckbxF2: TCheckBox;
    chckbxF3: TCheckBox;
    chckbxF4: TCheckBox;
    edtA1: TEdit;
    edtA3a: TEdit;
    edtA3b: TEdit;
    edtA4: TEdit;
    edtA2: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    mmReport: TMemo;
    rdgrpFcombi: TRadioGroup;
    strgrdOptions: TStringGrid;
    procedure btbttnCloseClick(Sender: TObject);
    procedure chckbxF1Click(Sender: TObject);
    procedure chckbxF2Click(Sender: TObject);
    procedure chckbxF3Click(Sender: TObject);
    procedure chckbxF4Click(Sender: TObject);
    procedure edtA1Change(Sender: TObject);
    procedure edtA2Change(Sender: TObject);
    procedure edtA3aChange(Sender: TObject);
    procedure edtA3bChange(Sender: TObject);
    procedure edtA4Change(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure strgrdOptionsPrepareCanvas(Sender: TObject; aCol, aRow: Integer;
      aState: TGridDrawState);
  private

  public
    procedure UpdateGrid;
  end;

var
  frmDecayProtection: TfrmDecayProtection;
  A1,A2,A3a,A3b,A4:real;

implementation

{$R *.lfm}
uses untGUI;

{ TfrmDecayProtection }

procedure TfrmDecayProtection.FormActivate(Sender: TObject);
var code: integer;
begin
  strgrdOptions.RowHeights[0]:=40;
  {fill table describing the options: strgrdOptions}
  strgrdOptions.Cells[0,0]:='Mechanism';
  strgrdOptions.Cells[1,0]:='Involved SoilGen variables';
  strgrdOptions.Cells[2,0]:='Rate reduction factor (multiplier F): max..none = 0..1';
  strgrdOptions.Cells[0,1]:='Bonding clay mineral surface-cations-OM, where strong bonding (high-activity clay minerals) =reduced decay.';
  strgrdOptions.Cells[0,2]:='Bonding on metal surfaces, where strong bonding (high metal concentrations) =reduced decay.';
  strgrdOptions.Cells[0,3]:='Aggregation driven by soil chemical properties determines the accessibility of OM to micro-organisms,'+
                            'where strong aggregation = reduced decay.';
  strgrdOptions.Cells[0,4]:='Porosity determines the accessibility of OM to micro-organisms, '+
                            'where micro- and mesopores affect the accessibility and thus the decay.';
  strgrdOptions.Cells[1,1]:='Mass fractions (-) of kaolinite, montmorillonite, illite and chlorite';
  strgrdOptions.Cells[1,2]:='Concentration of Al+3 and Fe+2 (mmol dm-3)';
  strgrdOptions.Cells[1,3]:='Mass fraction (-) of calcite and Base Saturation (%)';
  strgrdOptions.Cells[1,4]:='Volume fractions (-) occupied by mesopores and micropores';
  if chckbxF1.checked then strgrdOptions.Cells[2,1]:='F1=A1+(1-A1)*kaol/ (kaol+mont+ill+chlor)'
                      else strgrdOptions.Cells[2,1]:='F1=1';
  if chckbxF2.checked then strgrdOptions.Cells[2,2]:='F2=exp(-A2*(Al+Fe))'
                      else strgrdOptions.Cells[2,2]:='F2=1' ;
  if chckbxF3.checked then strgrdOptions.Cells[2,3]:='F3a=1-A3a*BS/100 | F3b=1-A3b*Calcitefraction | F3=max(F3a,F3b)'
                      else strgrdOptions.Cells[2,3]:='F3=1' ;
  if chckbxF4.checked then strgrdOptions.Cells[2,4]:='F4=A4+(1-A4)*meso/(meso+micro)'
                      else strgrdOptions.Cells[2,4]:='F4=1' ;
  strgrdOptions.Height:=strgrdOptions.RowHeights[0]+strgrdOptions.RowHeights[1]+strgrdOptions.RowHeights[2]+
                        strgrdOptions.RowHeights[3]+strgrdOptions.RowHeights[4]+5;
  strgrdOptions.Refresh;
  val(edtA1.Text,A1,code);
  val(edtA2.Text,A2,code);
  val(edtA3a.Text,A3a,code);
  val(edtA3a.Text,A3b,code);
  val(edtA4.Text,A4,code);
end;

procedure TfrmDecayProtection.strgrdOptionsPrepareCanvas(Sender: TObject; aCol,
  aRow: Integer; aState: TGridDrawState);
var
  ATextStyle: TTextStyle;
begin
  ATextStyle := strgrdOptions.Canvas.TextStyle;
  ATextStyle.SingleLine := false;
  ATextStyle.Wordbreak := true;
  strgrdOptions.Canvas.TextStyle := ATextStyle;
end;

procedure TfrmDecayProtection.btbttnCloseClick(Sender: TObject);
var col,row,code:integer;
begin
  val(edtA1.Text,A1,code); 
  val(edtA2.Text,A2,code);
  val(edtA3a.Text,A3a,code);
  val(edtA3a.Text,A3b,code);
  val(edtA4.Text,A4,code);
  mmReport.Clear;

  mmReport.Lines.add(strgrdOptions.Cells[0,1]+' | '+strgrdOptions.Cells[2,1]+' | ');
  if chckbxF1.checked then mmReport.Lines.add('A1='+edtA1.Text);
  mmReport.Lines.add(strgrdOptions.Cells[0,2]+' | '+strgrdOptions.Cells[2,2]+' | ');
  if chckbxF2.checked then mmReport.Lines.add('A2='+edtA2.Text);
  mmReport.Lines.add(strgrdOptions.Cells[0,3]+' | '+strgrdOptions.Cells[2,3]+' | ');
  if chckbxF3.checked then mmReport.Lines.add('A3a='+edtA3a.Text+', A3b='+edtA3b.Text);
  mmReport.Lines.add(strgrdOptions.Cells[0,4]+' | '+strgrdOptions.Cells[2,4]+' | ');
  if chckbxF4.checked then mmReport.Lines.add('A4='+edtA4.Text);
  mmReport.Lines.add(rdgrpFcombi.Items[rdgrpFcombi.itemindex]);
  Close;
end;

procedure TfrmDecayProtection.chckbxF1Click(Sender: TObject);
begin
  UpdateGrid;
  if chckbxF1.checked then edtA1.Enabled:=true else edtA1.Enabled:=false ;
  frmDecayProtection.Refresh;
end;

procedure TfrmDecayProtection.chckbxF2Click(Sender: TObject);
begin
  UpdateGrid;  
  if chckbxF2.checked then edtA2.Enabled:=true else edtA2.Enabled:=false ;
  frmDecayProtection.Refresh;
end;

procedure TfrmDecayProtection.chckbxF3Click(Sender: TObject);
begin
  UpdateGrid; 
  if chckbxF3.checked then
  begin
    edtA3a.Enabled:=true;   edtA3b.Enabled:=true
  end
  else
  begin
    edtA3a.Enabled:=false ; edtA3b.Enabled:=false ;
  end; 
  frmDecayProtection.Refresh;
end;

procedure TfrmDecayProtection.chckbxF4Click(Sender: TObject);
begin
  UpdateGrid;
  if chckbxF4.checked then edtA4.Enabled:=true else edtA4.Enabled:=false ;
  frmDecayProtection.Refresh;
end;

procedure TfrmDecayProtection.edtA1Change(Sender: TObject);
var code:integer;
begin
  val(edtA1.Text,A1,code);
  if code>0 then
  begin
    showmessage(edtA1.Text+' is not a number!'); edtA1.Text:='0.8'; A1:=0.8; 
    frmDecayProtection.Refresh;
  end;
end;

procedure TfrmDecayProtection.edtA2Change(Sender: TObject);
var code:integer;
begin
  val(edtA2.Text,A2,code);
  if code>0 then
  begin
    showmessage(edtA2.Text+' is not a number!'); edtA2.Text:='5'; A2:=5; 
    frmDecayProtection.Refresh;
  end;
end;

procedure TfrmDecayProtection.edtA3aChange(Sender: TObject);
var code:integer;
begin
  val(edtA3a.Text,A3a,code);
  if code>0 then
  begin
    showmessage(edtA3a.Text+' is not a number!'); edtA3a.Text:='0.2'; A3a:=0.2; 
    frmDecayProtection.Refresh;
  end;
end;

procedure TfrmDecayProtection.edtA3bChange(Sender: TObject);
var code:integer;
begin
  val(edtA3b.Text,A3b,code);
  if code>0 then
  begin
    showmessage(edtA3b.Text+' is not a number!'); edtA3b.Text:='0.2'; A3b:=0.2;
    frmDecayProtection.Refresh;
  end;
end;

procedure TfrmDecayProtection.edtA4Change(Sender: TObject);
var code:integer;
begin
  val(edtA4.Text,A4,code);
  if code>0 then
  begin
    showmessage(edtA4.Text+' is not a number!'); edtA4.Text:='0.8'; A4:=0.8;
    frmDecayProtection.Refresh;
  end;
end;

procedure TfrmDecayProtection.UpdateGrid;
begin
  if chckbxF1.checked then strgrdOptions.Cells[2,1]:='F1=A1+(1-A1)*kaol/ (kaol+mont+ill+chlor)'
                      else strgrdOptions.Cells[2,1]:='F1=1';
  if chckbxF2.checked then strgrdOptions.Cells[2,2]:='F2=exp(-A2*(Al+Fe))'
                      else strgrdOptions.Cells[2,2]:='F2=1' ;
  if chckbxF3.checked then strgrdOptions.Cells[2,3]:='F3a=1-A3a*BS/100 | F3b=1-A3b*Calcitefraction | F3=max(F3a,F3b)'
                      else strgrdOptions.Cells[2,3]:='F3=1' ;
  if chckbxF4.checked then strgrdOptions.Cells[2,4]:='F4=A4+(1-A4)*meso/(meso+micro)'
                      else strgrdOptions.Cells[2,4]:='F4=1' ;
  strgrdOptions.Height:=strgrdOptions.RowHeights[0]+strgrdOptions.RowHeights[1]+strgrdOptions.RowHeights[2]+
                        strgrdOptions.RowHeights[3]+strgrdOptions.RowHeights[4]+5;
  strgrdOptions.Refresh;
end;

end.

