unit untEditBedrock;
{**************************************************************************************}
{**************************** This file is part of SoilGen ****************************}
{****************************SoilGen is free software:*********************************}
{you can redistribute it and/or modify it under the terms******************************}
{of the GNU General Public License as published by the Free Software Foundation,*******}
{either version 3 of the License, or (at your option) any later version.***************}
{  It would be polite to refer to the SoilGen model by mentioning the documentation:  }
{  Peter Finke. 2024. Modelling soil development under global change.                 }
{  SpringerBriefs in Earth System Sciences 10032. ISBN: 978-3-031-55582-4.            }
{  https://doi.org/10.1007/978-3-031-55583-1                                          }
{**************************************************************************************}
{SoilGen is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;**}
{without even the implied warranty of MERCHANTABILITY or ******************************}
{FITNESS FOR A PARTICULAR PURPOSE.*****************************************************}
{See the GNU General Public License for more details.**********************************}
{**************************************************************************************}
{You should have received a copy of the GNU General Public License along with SoilGen.*}
{If not, see <https://www.gnu.org/licenses/>.******************************************}
{$mode Delphi}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, Grids, StdCtrls,
  Buttons, ExtCtrls,lazfileutils;

type

  { TfrmEditBedrock }

  TfrmEditBedrock = class(TForm)
    btbttnQuitNoChange: TBitBtn;
    btbttnQuitWithChanges: TBitBtn;
    edtRockDensity: TEdit;
    edtTargetRockPorosity: TEdit;
    lblTargetRockPorosity: TLabel;
    lblRockDensity: TLabel;
    rdgrpChoose: TRadioGroup;
    StaticText1: TStaticText;
    strgrdRockProps: TStringGrid;
    procedure btbttnQuitNoChangeClick(Sender: TObject);
    procedure btbttnQuitWithChangesClick(Sender: TObject);
    procedure edtRockDensityExit(Sender: TObject);
    procedure edtTargetRockPorosityExit(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure rdgrpChooseClick(Sender: TObject);
    procedure strgrdRockPropsValidateEntry(Sender: TObject; aCol,
      aRow: Integer; const OldValue: string; var NewValue: String);
  private

  public
    procedure readRockprops;
    procedure fillstrgrdRockprops;
    procedure calcsumproduct;
  end;

var
  frmEditBedrock: TfrmEditBedrock;
  sumproduct,value:real;
  code:integer;
  txt:string;

implementation

{$R *.lfm}
uses untGUI;

{ TfrmEditBedrock }

procedure TfrmEditBedrock.FormActivate(Sender: TObject);
begin
  if FileExistsUTF8('Bedrock.txt') { *Converted from FileExists*  }
  then ReadRockprops;
  fillstrgrdRockprops;
  rdgrpChooseClick(Sender);
end;

procedure TfrmEditBedrock.rdgrpChooseClick(Sender: TObject);
begin
  case rdgrpChoose.ItemIndex of
      0: begin
           edtRockDensity.Visible:=true; edtTargetRockPorosity.visible:=false;
           lblTargetRockPorosity.visible:=true; lblRockDensity.visible:=false;
         end;
      1: begin
           edtRockDensity.Visible:=false; edtTargetRockPorosity.visible:=true;
           lblTargetRockPorosity.visible:=false; lblRockDensity.visible:=true;
         end;
  end;
end;

procedure TfrmEditBedrock.edtRockDensityExit(Sender: TObject);
begin
  calcsumproduct;
  val(edtRockDensity.text,Value,code);
  if code<>0 then
  begin
    showmessage('You did not enter a number!');
    edtRockDensity.text:='3.5000000';
    lblRockDensity.Caption:=edtRockDensity.text;
  end
  else
  begin
    rock.rockdensity:=value;
    str(rock.rockdensity:10:7,txt);
    lblRockDensity.caption:=txt;
    lblRockDensity.Refresh;
    {if rock.rockdensity<sumproduct then}
    begin
      rock.rockporosityperc:=100*((sumproduct/rock.rockdensity)-1);
      str(rock.rockporosityperc:10:7,txt);
      edtTargetRockPorosity.Text:=txt;   edtTargetRockPorosity.refresh;
      lblTargetRockPorosity.Caption:=txt;lblTargetRockPorosity.Refresh;
    end; 
    btbttnQuitWithChanges.Enabled:=(rock.rockporosityperc>0);
  end;
end;

procedure TfrmEditBedrock.edtTargetRockPorosityExit(Sender: TObject);
begin
  calcsumproduct;
  val(edtTargetRockPorosity.text,value,code);
  if code<>0 then
   begin
     showmessage('You did not enter a number!');
     edtTargetRockPorosity.text:='5.0000000';
     lblTargetRockPorosity.caption:=edtTargetRockPorosity.text;
   end
  else
  begin
    rock.rockporosityperc:=value;
    str(rock.rockporosityperc:10:7,txt);
    lblTargetRockporosity.caption:=txt;
    lblTargetRockPorosity.Refresh;
    if rock.rockporosityperc>0 then
    begin
      rock.rockdensity:=sumproduct/((100+rock.rockporosityperc)/100);
      str(rock.rockdensity:10:7,txt);
      edtRockDensity.text:=txt;    edtRockDensity.refresh;
      lblRockDensity.Caption:=txt; lblRockDensity.Refresh
    end;
    btbttnQuitWithChanges.Enabled:=(rock.rockporosityperc>0);
  end;
end;

procedure TfrmEditBedrock.btbttnQuitNoChangeClick(Sender: TObject);
begin
  if FileExistsUTF8('Bedrock.txt') { *Converted from FileExists*  } then DeleteFileUTF8('Bedrock.txt'); { *Converted from DeleteFile*  }
  untGUI.frmMAIN.rdgrpChooseDisplay.Columns:=5;    {was 4}
  untGUI.frmMAIN.rdgrpChooseDisplay.ItemIndex:=0;  {was 3}
  Rock.RockBottomLayers:=0;
  close;
end;

procedure TfrmEditBedrock.btbttnQuitWithChangesClick(Sender: TObject);
var sum:real; row,col:integer; f:textfile;
begin
  {recalc mass fractions to sum up to 1}
  with Rock do
  begin
    sum:=rockAlbite[3]+rockKFeldspar[3]+rockMuscovite[3]+rockBiotite[3]+
         rockAnorthite[3]+rockForsterite[3]+rockAugite[3]+rockHornblende[3]+
         rockFayalite[3]+rockQuartz[3]+rockChlorite[3]+rockKaolinite[3]+
         rockIllite[3]+rockMontmorillonite[3]+rockAmorphite[3]+rockOtherite[3]+
         rockGibbsite[3]+rockCalcite[3]+rockGypsum[3];
    rockAlbite[3]:=rockAlbite[3]/sum;
    rockKFeldspar[3]:=rockKFeldspar[3]/sum;
    rockMuscovite[3]:=rockMuscovite[3]/sum;
    rockBiotite[3]:=rockBiotite[3]/sum;
    rockAnorthite[3]:=rockAnorthite[3]/sum;
    rockForsterite[3]:=rockForsterite[3]/sum;
    rockAugite[3]:=rockAugite[3]/sum;
    rockHornblende[3]:=rockHornblende[3]/sum;
    rockFayalite[3]:=rockFayalite[3]/sum;
    rockQuartz[3]:=rockQuartz[3]/sum;
    rockChlorite[3]:=rockChlorite[3]/sum;
    rockKaolinite[3]:=rockKaolinite[3]/sum;
    rockIllite[3]:=rockIllite[3]/sum;
    rockMontmorillonite[3]:=rockMontmorillonite[3]/sum;
    rockAmorphite[3]:=rockAmorphite[3]/sum;
    rockOtherite[3]:=rockOtherite[3]/sum;
    rockGibbsite[3]:=rockGibbsite[3]/sum;
    rockCalcite[3]:=rockCalcite[3]/sum;
    rockGypsum[3]:=rockGypsum[3]/sum;
  end;
  fillstrgrdRockprops;
  {save to file: rockalbite[1]..[2]..rockgypsum[1]..[2] and rockdensity}
  system.assign(f,'Bedrock.txt'); system.rewrite(f);
  for row:=0 to 19 do
  begin
    for col:=1 to 2 do system.write(f,strgrdRockProps.Cells[col,row],' ');
    system.writeln(f,strgrdRockProps.Cells[0,row]);
  end;
  system.Writeln(f,edtRockDensity.Text,' Rock density (kg dm-3)');
  closefile(f);
  untGUI.frmMAIN.mmBedrock.Lines.Loadfromfile('Bedrock.txt');
  untGUI.frmMAIN.rdgrpChooseDisplay.Columns:=5;
  untGUI.frmMAIN.rdgrpChooseDisplay.ItemIndex:=4;
  Rock.RockBottomLayers:=untGUI.rbl;
  close;
end;

procedure TfrmEditBedrock.strgrdRockPropsValidateEntry(Sender: TObject; aCol,
  aRow: Integer; const OldValue: string; var NewValue: String);
var code,acol2:integer; value:real;
begin
   val(strgrdRockProps.Cells[aCol,aRow],Value,code);
   if code<>0 then
   begin
     strgrdRockProps.Cells[aCol,aRow]:=OldValue;
     showmessage('You did not enter a number!');
   end
   else
   begin
     strgrdRockProps.Cells[aCol,aRow]:=NewValue;
     {put new value in data record as well}
     acol2:=acol+1;
     case aRow of
         1: Rock.rockAlbite[acol2]:=value;
         2: Rock.rockKFeldspar[acol2]:=value;
         3: Rock.rockMuscovite[acol2]:=value;
         4: Rock.rockBiotite[acol2]:=value;
         5: Rock.rockAnorthite[acol2]:=value;
         6: Rock.rockForsterite[acol2]:=value;
         7: Rock.rockAugite[acol2]:=value;
         8: Rock.rockHornblende[acol2]:=value;
         9: Rock.rockFayalite[acol2]:=value;
         10: Rock.rockQuartz[acol2]:=value;
         11: Rock.rockChlorite[acol2]:=value;
         12: Rock.rockKaolinite[acol2]:=value;
         13: Rock.rockIllite[acol2]:=value;
         14: Rock.rockMontmorillonite[acol2]:=value;
         15: Rock.rockAmorphite[acol2]:=value;
         16: Rock.rockOtherite[acol2]:=value;
         17: Rock.rockGibbsite[acol2]:=value;
         18: Rock.rockCalcite[acol2]:=value;
         19: Rock.rockGypsum[aCol2]:=value;
     end;
     calcsumproduct;
   end;
end;

procedure TfrmEditBedrock.ReadRockprops;
var f:textfile;   txt1,txt2:string;
begin
  system.assign(f,'Bedrock.txt'); system.reset(f);
  system.readln(f);
  with strgrdRockProps do
  begin
    system.readln(f,Rock.rockAlbite[1],Rock.rockAlbite[3]);
    str(Rock.rockAlbite[1]:5:3,txt1); str(Rock.rockAlbite[3]:7:5,txt2);
    Cells[0,1]:='Albite';            Cells[1,1]:=txt1; cells[2,1]:=txt2;

    system.readln(f,Rock.rockKFeldspar[1],Rock.rockKFeldspar[3]);
    str(Rock.rockKFeldspar[1]:5:3,txt1); str(Rock.rockKFeldspar[3]:7:5,txt2);
    Cells[0,2]:='K-Feldspar';        Cells[1,2]:=txt1; cells[2,2]:=txt2;  

    system.readln(f,Rock.rockMuscovite[1],Rock.rockMuscovite[3]);
    str(Rock.rockMuscovite[1]:5:3,txt1); str(Rock.rockMuscovite[3]:7:5,txt2);
    Cells[0,3]:='Muscovite';         Cells[1,3]:=txt1; cells[2,3]:=txt2; 

    system.readln(f,Rock.rockBiotite[1],Rock.rockBiotite[3]);
    str(Rock.rockBiotite[1]:5:3,txt1); str(Rock.rockBiotite[3]:7:5,txt2);
    Cells[0,4]:='Biotite';           Cells[1,4]:=txt1; cells[2,4]:=txt2;  

    system.readln(f,Rock.rockAnorthite[1],Rock.rockAnorthite[3]);
    str(Rock.rockAnorthite[1]:5:3,txt1); str(Rock.rockAnorthite[3]:7:5,txt2);
    Cells[0,5]:='Anorthite';         Cells[1,5]:=txt1; cells[2,5]:=txt2;

    system.readln(f,Rock.rockForsterite[1],Rock.rockForsterite[3]);
    str(Rock.rockForsterite[1]:5:3,txt1); str(Rock.rockForsterite[3]:7:5,txt2);
    Cells[0,6]:='Forsterite';        Cells[1,6]:=txt1; cells[2,6]:=txt2; 

    system.readln(f,Rock.rockAugite[1],Rock.rockAugite[3]);
    str(Rock.rockAugite[1]:5:3,txt1); str(Rock.rockAugite[3]:7:5,txt2);
    Cells[0,7]:='Augite';            Cells[1,7]:=txt1; cells[2,7]:=txt2; 

    system.readln(f,Rock.rockHornblende[1],Rock.rockHornblende[3]);
    str(Rock.rockHornblende[1]:5:3,txt1); str(Rock.rockHornblende[3]:7:5,txt2);
    Cells[0,8]:='Hornblende';        Cells[1,8]:=txt1; cells[2,8]:=txt2;  

    system.readln(f,Rock.rockFayalite[1],Rock.rockFayalite[3]);
    str(Rock.rockFayalite[1]:5:3,txt1); str(Rock.rockFayalite[3]:7:5,txt2);
    Cells[0,9]:='Fayalite';          Cells[1,9]:=txt1; cells[2,9]:=txt2;  

    system.readln(f,Rock.rockQuartz[1],Rock.rockQuartz[3]);
    str(Rock.rockQuartz[1]:5:3,txt1); str(Rock.rockQuartz[3]:7:5,txt2);
    Cells[0,10]:='Quartz';           Cells[1,10]:=txt1; cells[2,10]:=txt2; 

    system.readln(f,Rock.rockChlorite[1],Rock.rockChlorite[3]);
    str(Rock.rockChlorite[1]:5:3,txt1); str(Rock.rockChlorite[3]:7:5,txt2);
    Cells[0,11]:='Chlorite';         Cells[1,11]:=txt1; cells[2,11]:=txt2;   

    system.readln(f,Rock.rockKaolinite[1],Rock.rockKaolinite[3]);
    str(Rock.rockKaolinite[1]:5:3,txt1); str(Rock.rockKaolinite[3]:7:5,txt2);
    Cells[0,12]:='Kaolinite';        Cells[1,12]:=txt1; cells[2,12]:=txt2;  

    system.readln(f,Rock.rockIllite[1],Rock.rockIllite[3]);
    str(Rock.rockIllite[1]:5:3,txt1); str(Rock.rockIllite[3]:7:5,txt2);
    Cells[0,13]:='Illite';           Cells[1,13]:=txt1; cells[2,13]:=txt2; 

    system.readln(f,Rock.rockMontmorillonite[1],Rock.rockMontmorillonite[3]);
    str(Rock.rockMontmorillonite[1]:5:3,txt1); str(Rock.rockMontmorillonite[3]:7:5,txt2);
    Cells[0,14]:='Montmorillonite';  Cells[1,14]:=txt1; cells[2,14]:=txt2;   

    system.readln(f,Rock.rockAmorphite[1],Rock.rockAmorphite[3]);
    str(Rock.rockAmorphite[1]:5:3,txt1); str(Rock.rockAmorphite[3]:7:5,txt2);
    Cells[0,15]:='Amorphite';        Cells[1,15]:=txt1; cells[2,15]:=txt2;   

    system.readln(f,Rock.rockOtherite[1],Rock.rockOtherite[3]);
    str(Rock.rockOtherite[1]:5:3,txt1); str(Rock.rockOtherite[3]:7:5,txt2);
    Cells[0,16]:='Otherite';         Cells[1,16]:=txt1; cells[2,16]:=txt2;  

    system.readln(f,Rock.rockGibbsite[1],Rock.rockGibbsite[3]);
    str(Rock.rockGibbsite[1]:5:3,txt1); str(Rock.rockGibbsite[3]:7:5,txt2);
    Cells[0,17]:='Gibbsite';         Cells[1,17]:=txt1; cells[2,17]:=txt2;   

    system.readln(f,Rock.rockCalcite[1],Rock.rockCalcite[3]);
    str(Rock.rockCalcite[1]:5:3,txt1); str(Rock.rockCalcite[3]:7:5,txt2);
    Cells[0,18]:='Calcite';          Cells[1,18]:=txt1; cells[2,18]:=txt2;  

    system.readln(f,Rock.rockGypsum[1],Rock.rockGypsum[3]);
    str(Rock.rockGypsum[1]:5:3,txt1); str(Rock.rockGypsum[3]:7:5,txt2);
    Cells[0,19]:='Gypsum';           Cells[1,19]:=txt1; cells[2,19]:=txt2;
  end;
  system.readln(f,Rock.rockdensity);
  str(Rock.rockdensity:5:3,txt1);
  edtRockdensity.Text:=txt1; lblRockDensity.Caption:=txt1;
  calcsumproduct;
  rock.rockporosityperc:=100*((sumproduct/rock.rockdensity)-1);
  str(rock.rockporosityperc:10:7,txt);
  edtTargetRockPorosity.Text:=txt;   edtTargetRockPorosity.refresh;
  lblTargetRockPorosity.Caption:=txt;lblTargetRockPorosity.Refresh;
  closefile(f);
end;

procedure  TfrmEditBedrock.fillstrgrdRockprops;
var txt1,txt2:string;
begin
  with strgrdRockProps do
  begin
    ColWidths[0]:=120;  ColWidths[1]:=150;  ColWidths[2]:=150;
    Cells[0,0]:='mineral';  Cells[1,0]:='specific_gravity_(kg dm-3)';
    Cells[2,0]:='initial_mass_fraction_(-)   ';
    str(Rock.rockAlbite[1]:5:3,txt1); str(Rock.rockAlbite[3]:7:5,txt2);
    Cells[0,1]:='Albite';            Cells[1,1]:=txt1; Cells[2,1]:=txt2;
    str(Rock.rockKFeldspar[1]:5:3,txt1); str(Rock.rockKFeldspar[3]:7:5,txt2);
    Cells[0,2]:='K-Feldspar';        Cells[1,2]:=txt1; Cells[2,2]:=txt2;
    str(Rock.rockMuscovite[1]:5:3,txt1); str(Rock.rockMuscovite[3]:7:5,txt2);
    Cells[0,3]:='Muscovite';         Cells[1,3]:=txt1; Cells[2,3]:=txt2;
    str(Rock.rockBiotite[1]:5:3,txt1); str(Rock.rockBiotite[3]:7:5,txt2);
    Cells[0,4]:='Biotite';           Cells[1,4]:=txt1; Cells[2,4]:=txt2;
    str(Rock.rockAnorthite[1]:5:3,txt1); str(Rock.rockAnorthite[3]:7:5,txt2);
    Cells[0,5]:='Anorthite';         Cells[1,5]:=txt1; Cells[2,5]:=txt2;
    str(Rock.rockForsterite[1]:5:3,txt1); str(Rock.rockForsterite[3]:7:5,txt2);
    Cells[0,6]:='Forsterite';        Cells[1,6]:=txt1; Cells[2,6]:=txt2;
    str(Rock.rockAugite[1]:5:3,txt1); str(Rock.rockAugite[3]:7:5,txt2);
    Cells[0,7]:='Augite';            Cells[1,7]:=txt1; Cells[2,7]:=txt2;
    str(Rock.rockHornblende[1]:5:3,txt1); str(Rock.rockHornblende[3]:7:5,txt2);
    Cells[0,8]:='Hornblende';        Cells[1,8]:=txt1; Cells[2,8]:=txt2;
    str(Rock.rockFayalite[1]:5:3,txt1); str(Rock.rockFayalite[3]:7:5,txt2);
    Cells[0,9]:='Fayalite';          Cells[1,9]:=txt1; Cells[2,9]:=txt2;
    str(Rock.rockQuartz[1]:5:3,txt1); str(Rock.rockQuartz[3]:7:5,txt2);
    Cells[0,10]:='Quartz';           Cells[1,10]:=txt1; Cells[2,10]:=txt2;
    str(Rock.rockChlorite[1]:5:3,txt1); str(Rock.rockChlorite[3]:7:5,txt2);
    Cells[0,11]:='Chlorite';         Cells[1,11]:=txt1; Cells[2,11]:=txt2;
    str(Rock.rockKaolinite[1]:5:3,txt1); str(Rock.rockKaolinite[3]:7:5,txt2);
    Cells[0,12]:='Kaolinite';        Cells[1,12]:=txt1; Cells[2,12]:=txt2;
    str(Rock.rockIllite[1]:5:3,txt1); str(Rock.rockIllite[3]:7:5,txt2);
    Cells[0,13]:='Illite';           Cells[1,13]:=txt1; Cells[2,13]:=txt2;
    str(Rock.rockMontmorillonite[1]:5:3,txt1); str(Rock.rockMontmorillonite[3]:7:5,txt2);
    Cells[0,14]:='Montmorillonite';  Cells[1,14]:=txt1; Cells[2,14]:=txt2;
    str(Rock.rockAmorphite[1]:5:3,txt1); str(Rock.rockAmorphite[3]:7:5,txt2);
    Cells[0,15]:='Amorphite';        Cells[1,15]:=txt1; Cells[2,15]:=txt2;
    str(Rock.rockOtherite[1]:5:3,txt1); str(Rock.rockOtherite[3]:7:5,txt2);
    Cells[0,16]:='Otherite';         Cells[1,16]:=txt1; Cells[2,16]:=txt2;
    str(Rock.rockGibbsite[1]:5:3,txt1); str(Rock.rockGibbsite[3]:7:5,txt2);
    Cells[0,17]:='Gibbsite';         Cells[1,17]:=txt1; Cells[2,17]:=txt2;
    str(Rock.rockCalcite[1]:5:3,txt1); str(Rock.rockCalcite[3]:7:5,txt2);
    Cells[0,18]:='Calcite';          Cells[1,18]:=txt1; Cells[2,18]:=txt2;
    str(Rock.rockGypsum[1]:5:3,txt1); str(Rock.rockGypsum[3]:7:5,txt2);
    Cells[0,19]:='Gypsum';           Cells[1,19]:=txt1; Cells[2,19]:=txt2;
  end;
  calcsumproduct;
  strgrdRockProps.AutoSizeColumns;
end;

procedure TfrmEditBedrock.calcsumproduct;
begin
     {calculate/update minbaseddensity and rockporosityperc
      using rockdensity and mineral specific weights*massfractions }
     sumproduct:=0;
     sumproduct:=sumproduct+Rock.rockAlbite[1]*Rock.rockAlbite[3];
     sumproduct:=sumproduct+Rock.rockKFeldspar[1]*Rock.rockKFeldspar[3];
     sumproduct:=sumproduct+Rock.rockMuscovite[1]*Rock.rockMuscovite[3];
     sumproduct:=sumproduct+Rock.rockBiotite[1]*Rock.rockBiotite[3];
     sumproduct:=sumproduct+Rock.rockAnorthite[1]*Rock.rockAnorthite[3];
     sumproduct:=sumproduct+Rock.rockForsterite[1]*Rock.rockForsterite[3];
     sumproduct:=sumproduct+Rock.rockAugite[1]*Rock.rockAugite[3];
     sumproduct:=sumproduct+Rock.rockHornblende[1]*Rock.rockHornblende[3];
     sumproduct:=sumproduct+Rock.rockFayalite[1]*Rock.rockFayalite[3];
     sumproduct:=sumproduct+Rock.rockQuartz[1]*Rock.rockQuartz[3];
     sumproduct:=sumproduct+Rock.rockChlorite[1]*Rock.rockChlorite[3];
     sumproduct:=sumproduct+Rock.rockKaolinite[1]*Rock.rockKaolinite[3];
     sumproduct:=sumproduct+Rock.rockIllite[1]*Rock.rockIllite[3];
     sumproduct:=sumproduct+Rock.rockMontmorillonite[1]*Rock.rockMontmorillonite[3];
     sumproduct:=sumproduct+Rock.rockAmorphite[1]*Rock.rockAmorphite[3];
     sumproduct:=sumproduct+Rock.rockOtherite[1]*Rock.rockOtherite[3];
     sumproduct:=sumproduct+Rock.rockGibbsite[1]*Rock.rockGibbsite[3];
     sumproduct:=sumproduct+Rock.rockCalcite[1]*Rock.rockCalcite[3];
     sumproduct:=sumproduct+Rock.rockGypsum[1]*Rock.rockGypsum[3];
end;

end.

