unit untGUI;
{**************************************************************************************}
{**************************** This file is part of SoilGen ****************************}
{****************************SoilGen is free software:*********************************}
{you can redistribute it and/or modify it under the terms******************************}
{of the GNU General Public License as published by the Free Software Foundation,*******}
{either version 3 of the License, or (at your option) any later version.***************}
{  It would be polite to refer to the SoilGen model by mentioning the documentation:  }
{  Peter Finke. 2024. Modelling soil development under global change.                 }
{  SpringerBriefs in Earth System Sciences 10032. ISBN: 978-3-031-55582-4.            }
{  https://doi.org/10.1007/978-3-031-55583-1                                          }
{**************************************************************************************}
{SoilGen is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;**}
{without even the implied warranty of MERCHANTABILITY or ******************************}
{FITNESS FOR A PARTICULAR PURPOSE.*****************************************************}
{See the GNU General Public License for more details.**********************************}
{**************************************************************************************}
{You should have received a copy of the GNU General Public License along with SoilGen.*}
{If not, see <https://www.gnu.org/licenses/>.******************************************}
{$MODE Delphi}


interface

uses
  {$IFDEF UNIX}{$IFDEF UseCThreads}
  cthreads,
  {$ENDIF}{$ENDIF}
  Classes,sysutils                              { FPC trunk fileinfo reads exe resources as long as you register the appropriate units }
  , fileinfo
  , winpeimagereader {need this for reading exe info}
  , elfreader {needed for reading ELF executables}
  , machoreader {needed for reading MACH-O executables}
  , LCLIntf, LCLType, LMessages, Messages, Variants, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, Buttons, Math, ExtCtrls, Spin, FileUtil, lazfileutils,
  untProcesses,untBioturbation,untClimate,untNonFatal,untEditSoilData,
  untImportFertilizerData, Menus, untAbout, untEvents, DateUtils, untMemDump,
  untSoftConstants, untDecayProtection,untEditChemConstants,untEditCinputs,untEditBedRock, Types,untSoilGenConst;

Const VersionID='SoilGen 3.8';
      {for special version functionality}
      WritePSD=true;    {if true AND PhysWeathAllowed: PSD in all 16 fractions written (top layer) annually}
      DoSplitCEC=true;  {should be true, allows splitting up the CEC in mineral amnd Organic part}
      OK14C=true;       {should be true, for radiocarbon calcs}
      MinSplashableSoilAreareduction=0.0; {0: no reduction of splashable soil area;
                                         0.75: only 25% of the area can have rainsplash}
                                        {actual area can be even further reduced
                                         by crop cover and ectorganic material}
      {CheckBoxMonthlyData set to true if P, PE and T are to be corrected with monthly data. You need the input for each simulation year!
                        If set to false: correct with annual data (climate file)}
      CheckBoxMonthlyData=false;
      {porosity_threshold and phweathloss_threshold are used to decide when soil production will produce a saprolite layer}
      porosity_threshold=15;   {porosity (volume%) that muast be added by weathering for a rock compartment to become a saprolite compartment}
      phweathloss_threshold=30; {minimal loss of rock (stones) (mass%) to smaller texture fractions for a rock compartment to become a saprolite compartment}
      def_rootcounterthreshold=0.015; {RPM threshold (Mg ha-1 in a compartment) to identify stone shattering by root action}
      rbl=60; {rock bottom layers, the maximum number of potential saprolite compartments. Only relevant if highly weatherable rock}
      {sevaral regular constants used in more units are declared in untSoilGenConst}
      MaxNoPrints=5;
      MaxTDD=100; {max number of time-depth diagrams}
      MaxK=49;    {max number of equilibrium constants}
      TypeChem=2; {1: "old" Calcite/Gypsum chemistry; 2: new Calcite/Gypsum/Gibbsite chemistry}
      EditAllowed=true;           {toggle for allowing editing of inputfiles}

      PW_M=7;     {physical weathering: number of splits to reduce a cube
                   to 8 equal cubes of half the ribbon size}
      seedmineral=0.002; {value for newforming minerals should not be 0, thus a small value instead}
      rhothreshold=1.6; {maximal accepted value for bulkdensity in input file with initial soil data}
      {Default values of constants that can be set via SoftConstants.txt}
      {First block=RothC softconstants}
      def_k_DPM=10.0;
      def_k_RPM=0.2;
      def_k_BIO=0.66;
      def_k_HUM=0.004;
      def_DPM2RPM_agriculture=1.44;
      def_DPM2RPM_grassscrub=0.67;
      def_DPM2RPM_deciduouswood=0.25;
      def_DPM2RPM_coniferouswood=0.10;
      {Values below derived from Kononova, 1975 (see VanBreemen&Buurman, 2002. p84)
       belowground prod=0.7*litterfall aboveground > 42 belowground vs 58% aboveground}
      def_frEctoLitter_grassscrub=0.58;
      def_frEndoLitter_grassscrub=0.42;
      def_frEctoLitter_deciduouswood=0.85; {calibrated Zonian Forest, 2016}
      def_frEndoLitter_deciduouswood=0.15; {calibrated Zonian Forest, 2016}
      def_frEctoLitter_coniferouswood=0.95; {assumed }
      def_frEndoLitter_coniferouswood=0.05; {assumed }
      {Values below are estimates: summerwheat stubble mass approx. equals root mass.
      Based on Campbell et al., 1977. Can. J. Soil Sci. 57:289-310.
       Associated assumption (for total litter input) is that grain and straw are harvested}
      def_frEctoLitter_agriculture=0.41;
      def_frEndoLitter_agriculture=0.59;
      def_facscaleCO2BIOHUM=1.67;
      def_BIO2HUMratio=0.8518518518518520; {0.46/0.54}
      {Values that estimate the Interception Evaporation as fraction of Precipitation,
       per vegetation type}
      def_frPINT_agriculture=0;
      def_frPINT_grassscrub=0;
      def_frPINT_deciduouswood=0.08;  {calibrated N slope Zonian Forest, starting from Dolman et al., 2000}
      def_frPINT_coniferouswood=0.12; {calibrated N slope Zonian Forest, starting from Dolman et al., 2000}
      {Values relevant for clay migration}
      def_clay_kd=0.8;
      def_clay_kr=0.1;
      def_clay_hthetamacro=-1.7*10.19744289; {converted to mm for internal use} {calibrated 2 plots N France}
      def_CM_n=0.075;                 {calibrated 2 plots N France}
      def_CM_fref=2;
      def_CM_vref=0.1;
      def_clay_EctoBD=0.5;            {estimated bulkdensity ectoorganic layers; originally set to 0.8}
      def_clay_EctoThick=10;          {average thickness (mm) of litter layer at which no more splash occurs}
      {Values relevant for physical weathering}
      def_PW_PMax=0.0000016;          {max split probability}
      def_PW_dTdtPmax=0.950;          {hourly temp gradient, calibrated 2 plots N France. }
      def_PW_frStoneSplit=0.02;       {fraction of stone-sizes mass lost to gravel in case of frost spalling or root cracking event}
      {Default value for pCO2}
      def_pCO2atm=0.00035;
      {Cation uptake target values in vegetation}
      def_relCa_Grassscrub=0.271;     def_relMg_Grassscrub=0.164;
      def_relNa_Grassscrub=0.004;     def_relK_Grassscrub=0.560;
      def_relAl_Grassscrub=0.001;
      def_relCa_Deciduouswood=0.481;  def_relMg_Deciduouswood=0.226;
      def_relNa_Deciduouswood=0.025;  def_relK_Deciduouswood=0.265;
      def_relAl_Deciduouswood=0.004;
      def_relCa_Coniferouswood=0.752; def_relMg_Coniferouswood=0.128;
      def_relNa_Coniferouswood=0.004; def_relK_Coniferouswood=0.093;
      def_relAl_Coniferouswood=0.023;
      def_relCa_Agriculture=0.132;    def_relMg_Agriculture=0.075;
      def_relNa_Agriculture=0.013;    def_relK_Agriculture=0.759;
      def_relAl_Agriculture=0.021;
      {for delta14C}
      {yearAD=1950.0; no constant but variable}
      {for weathering of minerals}
      nrminerals=17;    {nr of minerals; also a constant in untSoftConstants+ untProcesses!}
      nrparams=6;       {nr of parameters describing weathering; also a constant in untSoftConstants+ untGUI!}
      {default weathering parameters; can be overriden via SoftConstants}
      def_Albite_logkh25=-9.7;   def_Albite_kEah=66.944;
      def_Albite_logkoh25=-9.95;  def_Albite_kEaoh=60;
      def_Albite_n=0.5;    def_Albite_m=0.5;
      def_KFeldspar_logkh25= -9.9;  def_KFeldspar_kEah= 50.208;
      def_KFeldspar_logkoh25= -10.7; def_KFeldspar_kEaoh= 50;
      def_KFeldspar_n=0.4; def_KFeldspar_m=0.3;
      def_Muscovite_logkh25= -11.8 ;  def_Muscovite_kEah= 54.392;
      def_Muscovite_logkoh25= -11.7; def_Muscovite_kEaoh= 22;
      def_Muscovite_n=0.14; def_Muscovite_m=0.16;
      def_Biotite_logkh25= -10.88;  def_Biotite_kEah= 35;
      def_Biotite_logkoh25= -999; def_Biotite_kEaoh= 0;
      def_Biotite_n=0.32;   def_Biotite_m=0;
      def_Quartz_logkh25= -999;  def_Quartz_kEah= 0;
      def_Quartz_logkoh25= -11;  def_Quartz_kEaoh= 85;
      def_Quartz_n=0;    def_Quartz_m=0.25;
      def_Chlorite_logkh25= -10.02; def_Chlorite_kEah= 40;
      def_Chlorite_logkoh25= -11.8; def_Chlorite_kEaoh= 40;
      def_Chlorite_n=0.5;  def_Chlorite_m=0.2;
      def_Anorthite_logkh25= -5.9; def_Anorthite_kEah= 79.496;
      def_Anorthite_logkoh25= -999;  def_Anorthite_kEaoh= 0;
      def_Anorthite_n=0.9; def_Anorthite_m=0;
      def_Forsterite_logkh25= -6.7;  def_Forsterite_kEah= 41.84;
      def_Forsterite_logkoh25= -999; def_Forsterite_kEaoh= 0;
      def_Forsterite_n=0.49; def_Forsterite_m=0;
      def_Augite_logkh25= -6.7;  def_Augite_kEah= 78;
      def_Augite_logkoh25= -999; def_Augite_kEaoh= 0;
      def_Augite_n=0.85;    def_Augite_m=0;
      def_Kaolinite_logkh25= -12.45; def_Kaolinite_kEah= 50;
      def_Kaolinite_logkoh25= -10.74;  def_Kaolinite_kEaoh= 40;
      def_Kaolinite_n=0.28; def_Kaolinite_m=0.73;
      def_Illite_logkh25= -11.47;  def_Illite_kEah= 46;
      def_Illite_logkoh25= -12.3;  def_Illite_kEaoh= 67;
      def_Illite_n=0.6;    def_Illite_m=0.6;
      def_Montmorrillonite_logkh25= -9.8; def_Montmorrillonite_kEah= 48;
      def_Montmorrillonite_logkoh25= -999;  def_Montmorrillonite_kEaoh= 0;
      def_Montmorrillonite_n=0.38; def_Montmorrillonite_m=0;
      def_Hornblende_logkh25= -10.2;  def_Hornblende_kEah= 50;
      def_Hornblende_logkoh25= -999; def_Hornblende_kEaoh= 0;
      def_Hornblende_n=0.55; def_Hornblende_m=0;
      def_Fayalite_logkh25= -6.0;  def_Fayalite_kEah= 94.4;
      def_Fayalite_logkoh25= -999; def_Fayalite_kEaoh= 0;
      def_Fayalite_n=0.69;  def_Fayalite_m=0;
      {Amorphite is a ficticious mineral representing amorphous, highly weatherable substances
       its elemental composition is input (total analysis) and its weathering speed is high}
      def_Amorphite_logkh25= -7.0;  def_Amorphite_kEah= 41.84;   {prone to calibration...}
      def_Amorphite_logkoh25= -999; def_Amorphite_kEaoh= 0;      {prone to calibration...}
      def_Amorphite_n=0.49;  def_Amorphite_m=0;                  {prone to calibration...}
      {Otherite is a ficticious mineral representing another weatherable but XRD crystalline substance
       its elemental composition is input (total analysis) and its weathering speed has to be set}
      def_Otherite_logkh25= -6.0;  def_Otherite_kEah= 94.4;   {from Fayalite, but prone to calibration...}
      def_Otherite_logkoh25= -999; def_Otherite_kEaoh= 0;     {from Fayalite, but prone to calibration...}
      def_Otherite_n=0.69;  def_Otherite_m=0.0;               {from Fayalite, but prone to calibration...}

type
  {real=double; }
  Profile = array[1..MaxLayers] of real ;
  iProfile = array[1..LL] of INT64;
  Mindat = array[1..nrminerals] of real;
  BProfile= array[1..maxlayers] of boolean;
  BPd14C=array [-MaxNoYears..MaxNoYears]of real;
  {for the C-submodel based on RothC-26.3}
  Monthly = array[1..12] of real;
  RothCGenRec=record {general data for the C-submodel}
    GENpoolpercs: array[1..5] of real;
    GENgrasRDEPTH,GENdeciRDEPTH,GENconiRDEPTH,GENagriRDEPTH:real;
    GENgrasPlant, GENgrasMan, GENdeciPlant, GENdeciMan, GENconiPlant, GENconiMan,
    GENresiPlant, GENresiMan, GENagriPlant, GENagriMan: monthly;
  end;
  RothCRec=Record {year/month/depth data to calculate C-pools and 14C pools of the C-submodel}
       Rainfall,EvapoPan,SoilCover,RDepth,
       totalaccTSMD,totalPlantResiduesIN,totalManureIN: Monthly;
       Temperatures,PlantresiduesIN,ManureIN: array[1..12] of profile;
       ClayPerc,fracManureIN,fracPlantIN,x2,x3: profile;
       DPM2RPM,ThicknessLayer,frEctoLitter,frEndoLitter,PlantResSUM,ManureSUM,
       totalmaxTSMD:real;
       DPMpool,RPMpool,BIOpool,HUMpool,IOMpool,CO2pool,accTSMD,pCO2,
       DPMXpool,RPMXpool,BIOXpool,HUMXpool,IOMXpool,CO2Xpool: profile;
       ectoDPMpool,ectoRPMpool,ectoBIOpool,ectoHUMpool,ectoIOMpool,ectoCO2pool,
       ectoDPMXpool,ectoRPMXpool,ectoBIOXpool,ectoHUMXpool,ectoIOMXpool:real;
       vegetatie:string[15];
  end;
  WeatheringGenRec=record {general weathering data for all minerals}
    MinDisPar: array[1..nrparams,1..nrminerals] of real;
    {param1=log KH25;  param2=kEaH (activ.Energ.acid);     param3=reaction order acid (n);
     param4=log KOH25; param5=kEaOH (activ.Energ.alkalic); param6=reaction order alkalic (m)}
    MinNames: array[1..nrminerals] of string;
  end;
  WeatheringRec=record {pools of every mineral and the cation pools over these minerals}
    Gibbsitepool,WE_Capool,WE_Mgpool,WE_Kpool,WE_Napool,WE_Sipool,WE_Fepool,WE_Alpool,
    WE_FePedogenic,WE_SiPedogenic: profile;
    MinPools: array [1..nrminerals+1] of profile;
  end;
  RockRec=record {data in rotting bedrock}
    rockAlbite,rockKFeldspar,rockMuscovite,rockBiotite,rockAnorthite,rockForsterite,
    rockAugite,rockHornblende,rockFayalite,rockQuartz,rockChlorite,rockKaolinite,
    rockIllite,rockMontmorillonite,rockAmorphite,rockOtherite,rockGibbsite,
    rockCalcite,rockGypsum : array[1..4] of real;
    {[1]=specific weight; [2]=mass ; [3]=mass fraction, per mineral in parent rock; [4]=mass lost relative to inital amount}
    weathCalcite,weathGibbsite,weathGypsum: array [1..8] of real;
    {these are the 6 weathering params and 2 dissolution params for 3 minerals}
    rockdensity,minbaseddensity,rockporosityperc,rockmasssum,rockstonesperc,massclayminerals:real;
    RockBottomLayers:integer;
    saproPW_G : array[-4..11] of real;
    saproPW_K,saproPW_S: array[-4..11]  of INT64;
    saprofreezecounter,saprorootcounter,saprosandm,saprosand,saprofinesandm,saprosiltm,saprosilt,
      saproclaym,saproclay,saprogravelm,saprogravel,saprostonesm,saprostones,phweathloss:
    real;
  end;

  DataRec =Record {data used throughout the simulations}
    idate,day1,CUMT1, NYC, NoLayers, BBC, IFLOW,IFILE, UNITS, R54, PRNTOP2, PRINT2,
      NPRINT, PRNTAB, INITFLG, nd1, nd2, nd3, nd4, NDUM,  KLPLT, NCROP, NN1,
      ITERCH,IFR, Icult , NIRRIG, ch_irr_flag, NWEEK, PRIOR
              : Integer;
    DELM, PDEPTH,DLZ,ABBPOT,THETA1,FWBTC, BCAM1, CMAT1,HMAT1, RP1, ALAMB1,
      ADDFC, ADDTH,RHOS2,RHOS1,RHOS3,RHO1,parden1,AEV1, CELLW,DIFO,rocn2,roslope,
      HDRY, HLOW, AK2, RRES, DUM, NSUM, unfrozen, WaterTableDepth,
      slope,upslopebearing,downwindbearing,latitude
                    : real;
    CLAY,SILT,SAND,OC,GRAVEL,STONES,AW,AH,RDF,ATEM,AAEV,ABCAM,RHO,CMATCH,HMATCH,RP,LAMBDA,APH,W,
      fc,d,CHARGE,SC1,SC2,SC3,SC4,SC5,SC6,SC7,SC8,SC9,SC10,SC11,
      SC12,SC13,SC14,SC15,APCO2,
      oldCALCITEA,CALCITEA,oldGYPA,GYPA,
      ACEC,ACEC_oc,ACEC_clay,alkalinity,BS,CO3,HCO3,EC,SAR,ESP,ET,PH,FLUX,
      DISSOLVEDCATEQ,DELGIBB,DELGYP,DECALCITE,OSPOT,delco2,
      parden, porosity, ClayDispInd, TEM, TEMPGEM, klei2,silt2, zand2, volpercclay,
      DOC,DOCperc,CLAYm,SILTm,SANDm,fineSANDm, OCm, gravelm,stonesm,totalm,       {mass 1 sq m, for compartment thickness}
      ClayFrCEC, OCFrCEC, FacCEC, FacCEC_OC, CEC_empirical,stoneperc,   {2-domain CEC}
      SUM21,MM21                   {percentages 2:1 clay minerals and MontMorillonite in clay fraction}
              : Profile;
    freezecounter,rootcounter: Profile;
    KNAMES    : array[1..maxK] of string[46];
    KValues   : array[1..maxK,1..2] of real; {input: contains eq constants at 25oC and delta-H}
    KValue   : array[1..maxK] of real;       {calc:  contains eq constants at current temp}
    KValueTable : array[1..maxK+3,-10..40] of real; {calc: contains eq constants at temperatures between whole-degree values -10 to +40 oC}
    CMINER    : array[1..maxChem] of real;
    MODRET    : array[1..MaxLayers] of integer;
    igrowth,icut,rtype,PDAY,ESTART,RDFDAY,ESTOP,DAYHAR
              : array[1..MaxNoCrops] of integer;
    RDEP,AK1, PANFAC  : array[1..MaxNoCrops] of real;
    DFER,INCOR
              : array[1..MaxNoChemApp] of integer;
    Dcult     : array[1..MaxNoCultiv] of integer;
    FRAC1Print: array[1..MaxNoPrints] of real;
    PRTIME    : array[1..MaxNoPrints] of integer;
    AIRRBG    : array[1..MaxNoIrrig] of integer;
    SEA       : array[1..MaxChem] of profile;
    AXSE,AXSE_oc,AXSE_clay      : array[1..MaxChem] of profile;
    CHEMDPMpool,CHEMRPMpool,
    CHEMBIOpool,CHEMHUMpool,
    CHEMIOMpool,CHEMCO2pool
              : array[1..MaxChem,0..MaxLayers] of real;
    FER       : array[1..MaxChem,1..MaxNoChemApp] of real;
    cultdep   : array[1..MaxNoCultiv] of real;
    FRAC1,IRRIGA,IRATEA
              : array[1..MaxNoIrrig] of real;
    ARSE      : array[1..MaxChem,1..MaxNoIrrig] of real;
    PAN,WTDEP,AVETEM,AMP
              : array[1..MaxNoWeeks] of real;
    tblock    : array[1..133]of string[153];
    RothCGen  : RothCGenRec;
    RothC     : RothCRec;
    WEGen     : WeatheringGenRec;
    WE        : WeatheringRec;
    firstyear :boolean;
    VGNmeasured:BProfile;
  end;
  supersede=record
        GENpoolpercs: array[1..5] of real;
        GENgrasRDEPTH,GENdeciRDEPTH,GENconiRDEPTH,GENagriRDEPTH:real;
        GENgrasPlant,GENgrasMan,GENdeciPlant,GENdeciMan,GENconiPlant,GENconiMan,
        GENresiPlant,GENresiMan,GENagriPlant,GENagriMan: monthly;
        NCROP,Icult:integer;
        igrowth,icut,rtype,PDAY,ESTART,RDFDAY,ESTOP,DAYHAR: array[1..MaxNoCrops] of integer;
        RDEP,AK1,PANFAC: array[1..MaxNoCrops] of real;
        Dcult: array[1..MaxNoCultiv] of integer;
        cultdep: array[1..MaxNoCultiv] of real;

  end;

    { TfrmMAIN }

    TfrmMAIN = class(TForm)
    btbttnBioTurbation: TBitBtn;
    btbttnClimateChange: TBitBtn;
    btbttnCO2: TBitBtn;
    btbttnd14C: TBitBtn;
    btbttnEditBedrock: TBitBtn;
    btbttnEditCTurnover: TBitBtn;
    btbttnEditEquilibria: TBitBtn;
    btbttnEditSoilData: TBitBtn;
    btbttnEditWeathering: TBitBtn;
    btbttnEventsChange: TBitBtn;
    btbttnFertilizationChange: TBitBtn;
    btbttnOpenDefaultCModel: TBitBtn;
    btbttnOpenDefaultLeachC: TBitBtn;
    btbttnOpenEquilibrium: TBitBtn;
    btbttnOpenWeathering: TBitBtn;
    btbttnOpenSoilProduction: TBitBtn;
    btbttnProtection: TBitBtn;
    btbttnRemove1: TBitBtn;
    btbttnSelect1: TBitBtn;
    bttnErrorInputs: TButton;
    bttnRun: TButton;
    bttnAutoCreate: TButton;
    chckbxBioturbation: TCheckBox;
    chckbxCapacity: TCheckBox;
    chckbxClayMigration: TCheckBox;
    chckbxClimate: TCheckBox;
    chckbxDeleteMonit: TCheckBox;
    chckbxDOS: TCheckBox;
    chckbxEvents: TCheckBox;
    chckbxFertilization: TCheckBox;
    chckbxInitMinerals: TCheckBox;
    chckbxEquilibriums: TCheckBox;
    chckbxCTurnover: TCheckBox;
    chckbxKeepFiles: TCheckBox;
    chckbxDailyTDD: TCheckBox;
    chckbxPhysWeath: TCheckBox;
    chckbxShow: TCheckBox;
    chckbxTurningPlow: TCheckBox;
    chckbxSpecialClimate: TCheckBox;
    chckbxInitSoil: TCheckBox;
    cmbbxScreenVariables: TComboBox;
    cmbbxTimeDepth: TComboBox;
    edtMonitName: TEdit;
    ggProgressMultipleYears: TProgressBar;
    ggProgressCurrentYear: TProgressBar;
    grpbxOnStart: TGroupBox;
    grpbxEditData: TGroupBox;
    grpbxEvents: TGroupBox;
    grpbxInitialRunData: TGroupBox;
    grpbxM1: TGroupBox;
    grpbxM2: TGroupBox;
    grpbxM3: TGroupBox;
    grpbxM4: TGroupBox;
    grpbxM5: TGroupBox;
    grpbxM6: TGroupBox;
    grpbxMultipleRunData: TGroupBox;
    grpbxScreenOutput: TGroupBox;
    Image1: TImage;
    Image2: TImage;
    Image3: TImage;
    Image4: TImage;
    Image5: TImage;
    Image6: TImage;
    Label1: TLabel;
    Label10: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    Label18: TLabel;
    Label19: TLabel;
    Label2: TLabel;
    Label20: TLabel;
    Label21: TLabel;
    Label22: TLabel;
    Label23: TLabel;
    Label24: TLabel;
    Label25: TLabel;
    Label26: TLabel;
    Label27: TLabel;
    lblAmorphite: TLabel;
    lblDefC: TLabel;
    lblDefSoil: TLabel;
    lblDefMin: TLabel;
    lblDefEq: TLabel;
    lblOtherite: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    lstbxScreenVariables: TListBox;
    mmAmorphite: TMemo;
    mmBedrock: TMemo;
    mmInitMineralogy: TMemo;
    mmCturnover: TMemo;
    mmEquilibriums: TMemo;
    mmOtherite: TMemo;
    mmDOS: TMemo;
    mmDefaultCModel: TMemo;
    mmDefaultLeachC: TMemo;
    mmEquilibrium: TMemo;
    mmHeaders: TMemo;
    mmInitSoil: TMemo;
    mmWeatheringData: TMemo;
    pndlgOPENspecialclimate: TOpenDialog;
    rdgrpBBC: TRadioGroup;
    rdgrpChooseDisplay: TRadioGroup;
    rdgrpSALTLOSS: TRadioGroup;
    spndtLatitude: TSpinEdit;
    spndtMultipleYears: TSpinEdit;
    spndtSlopeAngle: TSpinEdit;
    spndtSlopeBearing: TSpinEdit;
    spndtWindBearing: TSpinEdit;
    tbcntrlSHELL: TTabControl;
    pndlgOPEN: TOpenDialog;
    svdlgSAVE: TSaveDialog;
    lblProgress: TLabel;
    Label6: TLabel;
    chckbxAbortNOW: TCheckBox;
    lblNOTE: TLabel;
    mainmenu: TMainMenu;
    About1: TMenuItem;
    Settings1: TMenuItem;
    Label11: TLabel;
    lblLeftTime: TLabel;
    Showmemorydump1: TMenuItem;
    rdgrpTrace: TRadioGroup;
    spndtTraceStart: TSpinEdit;
    lblTraceStart: TLabel;
    procedure btbttnd14CClick(Sender: TObject);          {import d14C values for user-chosen years (will be interpolated)}
    procedure btbttnEditBedrockClick(Sender: TObject);   {define, erase or edit bedrock mineralogical composition}
    procedure btbttnEditCTurnoverClick(Sender: TObject); {opens the window to allow editing the file with C-turnover data}
    procedure btbttnEditEquilibriaClick(Sender: TObject);{opens the window to allow editing the file with chemical equilibrium constants}
    procedure btbttnEditWeatheringClick(Sender: TObject);{void; editing mineralogical inputs via untEditSoilData}
    procedure btbttnProtectionClick(Sender: TObject);    {opens a window to select and quantify protection agains decay of organic C pools}
    procedure bttnAutoCreateClick(Sender: TObject);      {autocreates the inputfiles selected in a previous screen grpbxOnStart}
    procedure bttnRunClick(Sender: TObject);             {Starts a model run using all previously defined settings and selected inputfiles}
    procedure chckbxCTurnoverClick(Sender: TObject);     {Identifies if a file with C-turnover data is available for reading}
    procedure chckbxEquilibriumsClick(Sender: TObject);  {Identifies if a file with chemical equilibrium data is available for reading}
    procedure chckbxInitMineralsClick(Sender: TObject);  {Identifies if a file with mineralogical data is available for reading}
    procedure chckbxInitSoilClick(Sender: TObject);      {Identifies if a file with initial soil data is available for reading}
    procedure chckbxSpecialClimateChange(Sender: TObject); {only active if CheckBoxMonthlyData=true (usually not true) to invoke import of Monthly Climate data}
    procedure tbcntrlSHELLChange(Sender: TObject);       {identifies if another tab sheet on the main window is activated and displays it}
    procedure btbttnOpenDefaultLeachCClick(Sender: TObject); {starts the selection of a file with initial soil data}
    procedure FormActivate(Sender: TObject);             {initialization}
    procedure spndtMultipleYearsChange(Sender: TObject); {if the number of simulation years of a scenario is changed in the main window, the new value is passed to other windows}
    procedure edtMonitNameChange(Sender: TObject);       {displays the name of the log-file}
    procedure btbttnSelect1Click(Sender: TObject);       {select 1 soil variable to be depicted in the (max 6) dynamic time-depth diagrams during runtime}
    procedure btbttnRemove1Click(Sender: TObject);       {remove 1 soil variable to be depicted in the (max 6) dynamic time-depth diagrams during runtime}
    procedure btbttnBioTurbationClick(Sender: TObject);  {opens the window to enter/edit the bioturbation levels and depths over the simulation time}
    procedure mmDOSClick(Sender: TObject);               {void: was used to open an older MSDOS version of SoilGen}
    procedure btbttnOpenDefaultCModelClick(Sender: TObject); {opens the file with C-turnover data }
    procedure chckbxShowClick(Sender: TObject);          {determines if the input files can be shown as text in a Memo-box}
    procedure btbttnOpenWeatheringClick(Sender: TObject);{opens the file with initial mineralogy input}
    procedure rdgrpChooseDisplayClick(Sender: TObject);  {determines which of the input files is shown as text in a Memo-box}
    procedure btbttnClimateChangeClick(Sender: TObject); {opens the window to allow entering/editing the file with climate data and saving it to the correct format}
    procedure chckbxClimateClick(Sender: TObject);       {identifies if a correctly formatted and named climate file is available}
    procedure chckbxBioturbationClick(Sender: TObject);  {identifies if a correctly formatted and named bioturbation file is available}
    procedure btbttnOpenEquilibriumClick(Sender: TObject);{opens the window to allow editing the file with chemical equilibrium constants data and saving it to the correct format}
    procedure FormClose(Sender: TObject; var Action: TCloseAction);{closes the main window, on fatal error or on program termination}
    procedure bttnErrorInputsClick(Sender: TObject);     {shows the contents of the file Messages.txt (containing non-fatal input errors}
    procedure btbttnEditSoilDataClick(Sender: TObject);  {opens the window to enter and edit initial soil data and mineralogical data for all profile layers}
    procedure btbttnFertilizationChangeClick(Sender: TObject);{opens the window to allow entering/editing the file with fertilization data and saving it to the correct format}
    procedure chckbxFertilizationClick(Sender: TObject); {identifies if a correctly formatted and named fertilization file is available}
    procedure chckbxClayMigrationClick(Sender: TObject); {determines if clay migration is to be simulated, always TRUE and the checkbox is not visible at present}
    procedure FormCreate(Sender: TObject);               {some initializations when the program starts and the main form is created}
    procedure About1Click(Sender: TObject);              {shows the window with "About"  information}
    procedure chckbxEventsClick(Sender: TObject);        {identifies if a correctly formatted and named events file is available}
    procedure btbttnEventsChangeClick(Sender: TObject);  {opens the window to enter/edit the various possible "events" that can be invoked over the simulation time and saves to the correct format}
    procedure Showmemorydump1Click(Sender: TObject);     {(via pulldown menu): if a previous run stopped with a crash, this functionality depicts the status of model variables via a new window}
    procedure rdgrpTraceClick(Sender: TObject);          {allows to trace which procedures/functions were visited during runtime, to roughly identify at what point a crash occurs}
    procedure chckbxPhysWeathClick(Sender: TObject);     {determines if physical weathering is to be simulated, always TRUE and the checkbox is not visible at present}
    procedure Settings1Click(Sender: TObject);           {opens a window to be used to adapt the SofConstants (and saves the result to SoftConstants.txt)}
    procedure btbttnCO2Click(Sender: TObject);           {import pCO2 values for user-chosen years (will be interpolated)}
  private
    { Private declarations }
  public
    { Public declarations }
    procedure RunLeachC(FileNameRunData:string);{now obsolete: DOS-version no longer used}
    Procedure HandleIt1;                        {in case of Fatal error: EAccessViolation; invalid memory access error}
    Procedure HandleIt2;                        {in case of Fatal error: EConvertError; string or object conversion error}
    Procedure HandleIt3;                        {in case of Fatal error: EDivByZero: integer divide-by-zero error}
    Procedure HandleIt4;                        {in case of Fatal error: EExternalException: unrecognized exception code}
    Procedure HandleIt5;                        {in case of Fatal error: EInOutError: file I/O error}
    Procedure HandleIt6;                        {in case of Fatal error: ERangeError: integer too large for the declared type to which it is assigned}
    Procedure HandleIt7;                        {in case of Fatal error: EZeroDivide: floating-point divide-by-zero error}
    Procedure HandleIt;                         {in case of Fatal error: save continuation files, a memory dump and basic files that may serve as input}
    Procedure writeDump;                        {saves variable status at time of crash to "dump.txt"}
    procedure readdump;                         {reads back "dump.txt"; only used in untMemDump}
    procedure ReadData(FileNameData1:string);   {reads the soil input file}
    procedure WriteData(FileNameData2:string);  {writes a soil input file valid for the end of the current/last simulation year}
    procedure ReadOut(FileNameOut:string);      {reads soil output file "LC*"; now obsolete: no longer used}
    FUNCTION JULIAN(idate,date:integer):integer;{calculates Julian day number for a given date}
    procedure GeneralRefresh;                   {refreshes the GUI-screen(s)}
    procedure CleanupDisk;                      {deletes temporary files}
    procedure ScreenOutput0;                    {initialising screen time-depth diagrams of max 6 soil variables}
    procedure FillScreenData;                   {select depth data for the current year of max 6 soil variables}
    procedure ScreenOutput1(CurrentYear,cor:integer);  {draw screen time-depth diagrams of max 6 soil variables}
    procedure Turbation(dtop,dmax,dbot,fractop,fracmax,fracbot: real); {turbation over the soil layers between dtop and dbot mm for all soil properties}
    procedure mixtexture(dtop,dmax,dbot,fractop,fracmax,fracbot: real);{apply turbation for clay%, silt%, oc%, volumetric water content and bulk density}
    procedure MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot:real;     {apply turbation for all chemicals on a mass basis}
                      var ChemMass:Profile);
    procedure WriteTimeDepth(year,cor: integer);{write the time-depth diagram of all (well, 100+) soil variables to file *.tdd for the current year}
    procedure EffectSaltLoss;                   {calculates the effect of loss/addition of calcite and gypsum on texture}
    procedure EffectSaltLossTexture;            {calculates the effect of loss/gains of calcite and gypsum on the mass of clay/silt/sand and bulk density per compartment }
    procedure Beschrijf_scenario;               {writes the settings for the current scenario to a log file with default name "SoilGen.txt"}
    procedure ReadCheckClimate;                 {reads the climate file "climate.txt" and produces climateINT.txt with annual values by interpolation}
    procedure WriteFileHeader(var ff:textfile; txt:string); {write headers of yearly time-depth diagrams with info on time-depth gridcell sizes}
    procedure ReadDataC(FileName:string);       {reads initial distribution of OC over various pools by depth (ectorganic layer set to 0), and
                                                 distribution of inputs of plantresidues and manure over the months for all 4 defined vegetation types}
    procedure ReadDataWE(FileName:string);      {reads the data file with mineral mass fractions, stoichiometry of Amorphite and Otherite, and calculates mass (per compartment)}
    procedure WriteDataC(FileName:string);      {gets the annual input data for RothC-based C-routine and writes the inputfile "RC*." to be called for the current year}
    procedure WriteDataWE(FileName:string);     {gets the annual input data for the weathering routines and writes the inputfile "WE*." to be called for the current year}
    procedure ReadOutC(FileNameOut:string);     {reads the annual outputfile from RothC-based C-routine and stores the pool-data (etc) in the record RothCRec}
    procedure ReadOutWE(FileNameOut:string);    {reads annual output data from the weathering routines and stores the pool-data in the record WE}
    Procedure ReCalcOC;                         {calculates updated %OC, clay, silt (sand), calcite, gypsum, RHO and volumetric water content for the soil profile using decomposition routine output}
    procedure InitChem;                         {initializes CEC, exchangeable cations, Gapon coefficients, pCO2, calcite and gypsum contents. To be updated after reading inputfiles}
    procedure ReadCheckFertil;                  {reads the fertilization file "fertilization.txt" and produces "fertilINT.txt" with annual values, NO interpolation (no data=values of 0)}
    procedure debuglog(txt:string);             {writes a (non-fatal) input error to 'Messages.txt'}
    procedure ReadDataEQ(filename:string);      {reads the datafile with chemical equilibrium constants and dH-values for Arrhenius temperature correction}
    procedure ReadDataRock(FileName:string);    {reads the datafile "bedrock.txt" with density and mass fraction per mineral and also the rock density}
    procedure dshowmessage(txt:string);         {same as debuglog, used also for fatal errors}
    procedure ShowWindowHeader;                 {writes a context-specific line at the top of the GUI-screen ending with "Note: semi-permanent beta-version"}
    procedure ShiftCompartments(nr:integer);    {shifts the full profile and all its variables nr compartments upwards (negative) or downwards (positive) to respond to erosion/deposition events}
    procedure Shift1profile(var soilprop:Profile; nr:integer); {shift 1 property of the profile upwards or downwards}
    procedure ReadCheckEvents;                  {reads the events file "events.txt" and produces "EventsINT.txt" with annual values for various possible events, NO interpolation (no data=values of 0)}
    procedure LeesWoord(var f:textfile; var woord:string); {reads one word (string between spaces or at beginning/end of line) from a text file}
    procedure CalcSlopeEffects(var slopewindfacP,slopewindfacPE:real); {calculates the effect of slope, its exposition, wind direction and latitude on correction factors for P and PE}
    procedure singletracewriteln(txt:string);   {stores <txt> in a global variable <singletracetext>, where <txt> indicates the execution point of the program. For debugging purposes.}
    procedure SplitCEC;                         {split up the CEC and exchangeable cations into clay and OC-contributions using the regression function in Foth and Ellis (1996)}
    procedure AdjustCECAXSE2OC;                 {adjust the OC-contributions of exchangeable cations from/to the soil solution. Note: for clay this is done via the clay migration routine}
    procedure BatchMode;                        {reading model inputs using the contents of "SoilGenBatch.txt" (only if this file exists)}
    function IncRad(inclination,aspect,latitude:real):real;  {algorithm from Swift, 1976 to calculate solar radiation on mountain slopes}
    procedure CheckSoftConstants;               {initialize all process constants and overwrite with values found in "SoftConstants.txt" (if this file is found)}
    function getal(txt:string):real;            {converts a string to a real number}
    procedure Prepare_Continuation;             {create a folder with the files needed to continue a finished run and puts all necessary files (incl. full soil variable status) in there}
    procedure ReadContinuation;                 {read the files in the continuation folder and initializes the model variables}
    procedure CopyRecToFrom(var TargetRec,SourceRec:DataRec); {copy a data record: SourceRec to TargetRec}
    procedure ContinueStartStatus;              {reads a data record "continue2.rec" from the continuation folder and reports its contents in "ContinueReport.txt"}
    procedure syswT(s:string);                  {writes a line of input text to outputfile <fouttxt>}
    procedure syswlI(i:integer;s:string);       {writes an integer and explanatory text to outputfile <fouttxt>}
    procedure syswlR(r:real;s:string);          {writes a real and explanatory text to outputfile <fouttxt>}
    procedure Decay;                            {radioactive decay over 1 year of all 14C pools}
    procedure WeatheringIndices(var CIA,IndexB,CIW,PIA,WI,CPA:Profile); {calculate weathering indices at the end of the current year}
    procedure KeepSelectedData;                 {for continuation run: store some variables that may override values from continue.rec by values from soil and turnover input files}
    procedure OverrideSelectedData;             {for continuation run: override the records for the data that must be kept from the Read Default Inputfiles}
    procedure EmulateTurningPlow(plowedcomp:integer); {mimick a turning plow by inverting the layers, repeated for all properties, over the plowed depth}
    procedure Turnplow1profile(var soilprop:Profile; plowedcomp:integer);  {mimick a turning plow by inverting the layers for 1 soil property}
    procedure WriteFileHeaderDaily(var ff:textfile; txt:string);  {write headers of (max 6 selceted) daily time-depth diagrams with info on time-depth gridcell sizes}
    procedure AssignDailyTDD(CurrentYear:integer);  {open (max 6 selected) daily time-depth diagrams}
    procedure CloseDailyTDD;                    {close (max 6 selected) daily time-depth diagrams}
    procedure CheckNonsense;                    {check on nonsense inputs in the soil data file: 1. Do clay+silt+sand+1.7*OC sum up to 100 and are all these positive? ;
                                                 2. Do exchangeable cations sum up to the CEC? 3. Is the charge balance of the solution neutral?
                                                 4. Is the charge balance of the rain water composition neutral?  5. Are the values for calcite and gypsum in range [0;1>}
    procedure CheckStartFromScratch;            {In case of a start of SoilGen without input files: check for presence of amorphite.txt and otherite.txt}
    procedure MVAR(year:integer; txt:string; getal:real); {not active now; allows monitoring for RHO at start+end of various functions and procedures}
    Function HYPRESRHO(clayperc,omperc:real):Real; {pedotransfer function to estimate RHO, the bulk density, from clay and OM. Only called when inputs (soil+events) are changed via the GUI}
  end;

var
  frmMAIN: TfrmMAIN;
  filename,txt,FirstLeachCFileName,FirstRothCFileName,FirstWeatherFileName,BedrockFileName,currentyear,
  ResultTxt,EQFileName,CO2FileName,d14CFileName,
  monthlyTEMPname,monthlyPRECname,monthlyEVAPname,startdir: string;
  findata,foutdata,fexp,fmonit,ftmp,
  f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f11,f12,f13,f14,f15,f16,f17,f18,f19,f20,
  f21,f22,f23,f24,f25,f26,f27,f28,f29,f30,f31,f32,f33,f34,f35,f36,f37,f38,f39,f40,
  f41,f42,f43,f44,f45,f46,f47,f48,f49,f50,f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,
  f61,f62,f63,f64,f65,f66,f67,f68,f69,f70,f71,f72,f73,f74,f75,f76,f77,f78,f79,f80,
  f81,f82,f83,f84,f85,f86,f87,f88,f89,f90,f91,f92,f93,f94,f95,f96,f97,f98,f99,f100,
  f4a,f14a,f44a,f87a,f88a,{calcite,gypsum,gibbsite,PedogenicFe and pedogenicSi in kg m-2}
  ftddtest,fmasstemp,CO2file,d14Cfile,fouttxt,fvgn,fmonthlyEVAP,fmonthlyPREC,fmonthlyTEMP,
  fdaily1,fdaily2,fdaily3,fdaily4,fdaily5,fdaily6:textfile;
  DataInit,DataYearIn,dataYearOut,DataCont,OnDisplay:DataRec;
  mon,monl:   array[1..12]of integer;
  kleur:      array[1..11]of TColor;
  images:     array[1..6] of TImage;
  dataimage:  array[1..6] of Profile;
  dminvalue,dmaxvalue: array[1..6] of real;
  minnen,maxen: array[1..MaxTDD] of real;
  eenheden: array[1..MaxTDD] of string;
  vdist,hdist,dtop,dmax,dbot,fractop,fracmax,fracbot:real;
  largeDOS,bioturbfile,climatefile,fertilfile,eventsfile,
  readdataOK,readdatacOK,readweatherOK,readbedrockOK,
  ReadDataEQOK,ALLOWED,BatchModeYN,writeDaily,warningsissued,startscreen,othererror: boolean;
  zerosimage: array[1..6] of boolean;
  defaultP,defaultPE,corP,corPE,frPINT,cor2:real;
  Pjfmond,Pamjjas:real; {precipitation in months1-3+9-12 and 4-8 to serve as winter/summer precip later}
  corT: array[1..MaxNoWeeks] of real;
  MacroMicroMM:real; {hydraulic head (mm water) at which macropores empty}
  PW_Pmax,PW_dTdtPmax,PW_E,PW_P,PW_frStoneSplit:real;   {parameters for physical weathering, expl. in untProcesses}
  PW_X:     array[-4..11] of INT64; {physical weathering:
                                    parameters per size class, expl. in untProcesses}
  PW_PARDEN,PW_VOL:array[-4..11] of real;
  PW_S,PW_K:array[-4..11,1..MaxLayers] of INT64; {physical weathering:
                                         parameters per size class+depth class, expl. in untProcesses}
  PW_G:array[-4..11,1..MaxLayers] of real;            {physical weathering:
                                         parameters per size class, expl. in untProcesses}
  mass: array[-4..11,1..MaxLayers] of real;  {physical weathering:
                                         mass in 100m2 over dlz mm depth of 11 texture fractions}
  ClayMigrationAllowed,PhysWeathAllowed,Chemstarted:boolean;  {toggle for simulation of clay migration}
  MaxPlusCompartments,iYear,yearBP,cor:integer;
  pCO2atm:real;
  SeriespCO2:array[1..MaxNoyears] of real;
  VGNal,VGNtr,VGNts,VGNks,VGNen,VGNel,VGNem,AvailableWater,phlab :Profile;
  {crash management}
  GUIlast100:array[1..100] of string; {trace list}
  GUIcount10,GUItrace:integer;                 {trace list counter}
  Crashtext,singletracetext:string;
  GlobalTime,GlobalDepth:integer;
  CurrentPlowMixFraction:real;
  {added to make continuation possible}
  frec: TSearchRec;
  bestandsnaam:string;
  DataRecGrootte:integer;
  keep:supersede; {data that should NOT be read from continuation files but
                   be read from the default soil (etc) inputfiles}
  {for 14C age calculation}
  d14C:BPd14C;
  yearAD:integer;
  Fk_DPM,Fk_RPM,Fk_BIO,Fk_HUM,Fk_IOM:Profile;
  {weathering indices}
  CIA,IndexB,CIW,PIA,WI,al2o3init,caoinit,na2oinit,CPA,TRB:profile;
  TurningPlow:boolean; {true: plowing = inversion of the involved compartments
                              followed by harrowing over the upper 100 mm
                              the latter is implemented as bioturbation over this depth
                              with <CurrentPlowMixFraction> intensity (input)}
  {for the case that monthly data are read}
  PtotMonthly, DefaultPtotMonthly, PEtotMonthly, DefaultPEtotMonthly,
  AvTempMonthly,DefaultAvTempMonthly: Monthly;
  {special minerals: Relative Formula Mass, RFM and relative contributions of elements}
  RFMamorphite,RFMotherite:real;
  amorp_O,amorp_Al,amorp_Ca,amorp_Mg,amorp_K,amorp_Na,amorp_Fe,amorp_Si,amorp_H,amorp_Cl,amorp_C,amorp_S,
  other_O,other_Al,other_Ca,other_Mg,other_K,other_Na,other_Fe,other_Si,other_H,other_Cl,other_C,other_S:
  real;
  {to accomodate effect of TotalReserveofBases on Organic matter degradation rates}
  MineralFactorDPM,MineralFactorRPM,MineralFactorBIO,MineralFactorHUM:real;
  totvol,totvol2,initMINvol,initOMvol:real;
  {bedrock}
  rock, rock_original:Rockrec;
  rootcounterthreshold,porositythreshold:real;
  pastyear:integer;

implementation

{$R *.lfm}


procedure TfrmMAIN.bttnRunClick(Sender: TObject);
{Starts a model run using all previously defined settings and selected inputfiles}
var i,i1,j,j1,nocompb,newcompartments,plowedcomp,kchem,month,ii,mm: integer;
    vegperiodtxt,si,sid1,txt,txtevent,txtleft,txttmp,
    txtScreenOutput,fnVGN:string;
    year,specialclimateyear,Ptot,PEtot,JanT,JulT,JanFileGem,JulFileGem,kHenry,Cinput,sumCO2,
    Ca,Mg,Na,K,Cl,SO4,HCO3,CO3,
    dCLAY,dSILT,dSAND,dOC1,dSTONES,dGRAVEL,dRHO,dAW,dCALCITEA,dAGYPS,dAPCO2,dCEC,dSC1,dSC2,dSC3,dSC7,dSC8,
    dSEA1,dSEA2,dSEA3,dSEA4,dSEA5,dSEA6,dSEA7,dSEA12,dSEA13,dSEA14,dSEA15,
    dSEA16,dAXSE1,dAXSE2,dAXSE3,dAXSE4,dAXSE10,dAXSE12,dfrAlbite,dfrKfeldspar,dfrMuscovite,
    dfrBiotite,dfrQuartz,dfrChlorite,dfrAnorthite,dfrFosterite,dfrAugite,dfrKaolinite,dfrIllite,
    dfrMontmorillonite,dfrHornblende,dfrFayalite,dfrGibbsite,dfrAmorphite,dfrOtherite,totalmass,
    slopewindfacP,slopewindfacPE,
    clost2atm,clost2iom,clost2hum,closttot,catlosttot,tempsum,
    gemPDstones,gemPDclay,gemPDsilt,gemPDsand,gemPDgravel: real;
    fturb,fclim,ffert,fevent,fectoout,fVGN:textfile;
    test:boolean;
    StartTime:TDateTime;
    LeftHours:double;
    tmpMinPool:Mindat;
    fbatch:textfile;
begin
Try
   { 1 Conditionally: Check if run=continuation and then read soil variables;
                      Check if capacity flow and then set some parameters}
{at this point: if this run is a continuation of a previous run,
   then read the necessary values of the soil variables at the end of the preceding run}
  if fileexists('continue.rec') then
  begin
    ReadContinuation; ContinueStartStatus;
  end;
  if chckbxCapacity.Checked
  then
  begin
    DataInit.IFLOW:=2; DataInit.ITERCH:=1;
    DataYearIn.IFLOW:=2; DataYearIn.ITERCH:=1;
  end
  else
  begin
    DataInit.IFLOW:=1; {DataInit.ITERCH=input value in file};
    DataYearIn.IFLOW:=1; {DataYearIn.ITERCH=input value in file};
  end;
  writeDaily:=chckbxDailyTDD.Checked;
  {hourly gradient>dailygradient}
   PW_dTdtPmax:=PW_dTdtPmax*24;
   { 2 Set presence of read VGN params to false}
   {initialisation if measured VanGenuchten Parameters are present}
   for i:=1 to MaxLayers do
   begin
     DataInit.VGNmeasured[i]:=false; DataYearIn.VGNmeasured[i]:=false;
     dataYearOut.VGNmeasured[i]:=false;
   end;
   untProcesses.externalcall:=true;
   showmemorydump1.Visible:=false;
   { 3 Write header to 'PSD.txt' (particle size distribution top compartment)}
   if WritePSD then
   begin
     system.assign(fmasstemp,'PSD.txt'); system.rewrite(fmasstemp);
     system.writeln(fmasstemp,'mass_fractions_and_number_of_grains_per-particle_size_class_in_top_compartment_(incl_gravel_and_stone-sized_particles');
     system.Writeln(fmasstemp,'year >32768mm 16.384-32.768mm 8.192-16.384mm 4.096-8.192mm 2.048-4.096mm 1.024-2.048mm ',
                              '0.512-1.024mm 0.256-0.512mm 0.128-0.256mm 0.064-0.128mm 0.032-0.064mm 0.016-0.032mm ',
                              '0.008-0.016mm 0.004-0.008mm 0.002-0.004mm 0-0.002mm >32768mm 16.384-32.768mm 8.192-16.384mm 4.096-8.192mm 2.048-4.096mm 1.024-2.048mm ',
                              '0.512-1.024mm 0.256-0.512mm 0.128-0.256mm 0.064-0.128mm 0.032-0.064mm 0.016-0.032mm ',
                              '0.008-0.016mm 0.004-0.008mm 0.002-0.004mm 0-0.002mm');
   end;
   {If GUItrace=1 then wwwriteln} singletracewriteln('Entering bttnRunClick');
   { 4 Disable buttons in user interface}
   Chemstarted:=false;
   chckbxAbortNOW.Visible:=true;
   frmMain.BorderIcons:=[biMinimize];
   untSoftConstants.frmSoftConstants.btbttnSoftConstantSave.enabled:=false;
   Cinput:=0;
   TurningPlow:=chckbxTurningPlow.checked;
   {disable buttons}
   chckbxTurningPlow.enabled:=false;
   chckbxCapacity.enabled:=false;
   spndtMultipleYears.Enabled:=false;
   chckbxPhysWeath.Enabled:=false;
   rdgrpBBC.Enabled:=false;
   btbttnBioTurbation.Enabled:=false;        chckbxBioturbation.Enabled:=false;
   btbttnClimateChange.Enabled:=false;       chckbxClimate.Enabled:=false;
   btbttnCO2.Enabled:=false;                 btbttnd14C.enabled:=false;
   btbttnFertilizationChange.Enabled:=false; chckbxfertilization.Enabled:=false;
   btbttnEventsChange.Enabled:=false;        chckbxEvents.Enabled:=false;
   btbttnSelect1.Enabled:=false;             btbttnRemove1.Enabled:=false;
   bttnRun.Enabled:=false;                   rdgrpSaltLoss.Enabled:=false;
   edtMonitName.Enabled:=false;              chckbxDeleteMonit.Enabled:=false;
   chckbxClayMigration.Enabled:=false;
   chckbxFertilization.Enabled:=false;
   chckbxKeepFiles.Enabled:=false;
   btbttnOpenDefaultLeachC.Enabled:=false;
   btbttnOpenDefaultCModel.Enabled:=false;
   btbttnOpenEquilibrium.Enabled:=false;
   btbttnOpenWeathering.Enabled:=false;
   btbttnEditSoilData.Enabled:=false;
   btbttnProtection.Enabled:=false;
   {bttnErrorInputs.Enabled:=false;}
   grpbxEditData.Enabled:=false;
   frmMain.about1.visible:=false;
   frmMain.settings1.visible:=false;
   chckbxDailyTDD.enabled:=true;
   {chckbxOnOff.Enabled:=false;}
   {progress bar settigs}
   ggProgressMultipleYears.Min:=0;
   ggProgressMultipleYears.Max:=spndtMultipleYears.Value;
   { 5 Conditionally: exclude bioturbation, climate change, fertilization, events depending on GUI-settings}
   {consequences of checked boxes }
   if not(chckbxBioturbation.Checked) then {user chose to exclude bioturbation}
      if fileexists('bioturbINT.txt') then deletefile('bioturbINT.txt');
   if not(chckbxClimate.Checked) then {user chose to exclude climate change}
   begin
      if fileexists('climate.txt') then deletefile('climate.txt');
      if fileexists('climateINT.txt') then deletefile('climateINT.txt');
   end;
   if not(chckbxFertilization.Checked) then {user chose to exclude fertilization}
   begin
      if fileexists('fertilINT.txt') then deletefile('fertilINT.txt');
   end;
   if not(chckbxEvents.Checked) then {user chose to exclude Erosion/deposition events}
   begin
      if fileexists('Events.txt') then deletefile('Events.txt');
   end;
   {legacy option: screen output is function of running DOS-executable LeachCPF or unit Chem}
   if chckbxDOS.Checked then
   begin
     ggProgressCurrentYear.Visible:=false;
     label6.Visible:=false;
     mmDOS.Visible:=false;
   end
   else
   begin
     ggProgressCurrentYear.Visible:=true;
     label6.Visible:=true;
     mmDOS.Visible:=false;
   end;
   { 6 call ReadCheckClimate (interpolate climate input to annual climate values)}
  {Reads Climate.txt and selects the relevant period.
   This period is put into climateINT.txt}
   ReadCheckClimate; {in: period-data; out: year-specific data}
  {check if climateINT.txt exists}
   climatefile:=fileexists('climateINT.txt');
  {check if bioturbINT exists}
   bioturbfile:=fileexists('bioturbINT.txt');
   { 7 Initialize physical weathering}
   if (PhysWeathAllowed) and (not(fileexists('continue.rec'))) then
   begin
    {initialize Physical Weathering}
    {average grain diameters (micrometers) for each one of 15 size classes (index -4..11)}
    PW_X[-4]:=49152;                                     {stones, only 1 class recognized}
    PW_X[-3]:=24576; PW_X[-2]:=12288; PW_X[-1]:=6144;  PW_X[0]:=3072;             {gravel}
    PW_X[1]:=1536; PW_X[2]:=768;  PW_X[3]:=384;  PW_X[4]:=192; PW_X[5]:=96;        {sand}
    PW_X[6]:=48;   PW_X[7]:=24;   PW_X[8]:=12;   PW_X[9]:=6;    PW_X[10]:=3;       {silt}
    PW_X[11]:=1;                                                                   {clay}
    {initial mass fraction, split grains and grains present per size class and depth class}
    for i:=1 to DataInit.NoLayers do
    begin
      freezecounter[i]:=0; {how often is the freezing point passed: initialization}
      rootcounter[i]:=0;   {how often may roots cause cracks in stones: initialization}
      {sand - silt - clay % sum up to 100-1.7*OC,
       these must be expressed as fraction summing up to 1
       and be evenly distributed over subfractions}
      for j:=1 to 5 do
      begin
        PW_G[j,i]:=0.2*((100-DataInit.GRAVEL[i]-DataInit.STONES[i])/100)
                      *(100-DataInit.CLAY[i]-DataInit.SILT[i]-1.7*DataInit.OC[i])
                      /(100-1.7*DataInit.OC[i]) {markOC};
        PW_PARDEN[j]:=DataInit.RHOS1;
      end;
      for j:=6 to 10 do
      begin
         PW_G[j,i]:=0.2*((100-DataInit.GRAVEL[i]-DataInit.STONES[i])/100)
                       *DataInit.SILT[i]/(100-1.7*DataInit.OC[i]); {markOC}
         PW_PARDEN[j]:=DataInit.RHOS1;
      end;
      PW_G[11,i]:=((100-DataInit.GRAVEL[i]-DataInit.STONES[i])/100)*DataInit.CLAY[i]/(100-1.7*DataInit.OC[i]);    {markOC}
      PW_PARDEN[11]:=DataInit.RHOS2;
      for j:=-3 to 0 do {gravel-classes}
      begin
         PW_G[j,i]:=0.25*DataInit.GRAVEL[i]/100;
         PW_PARDEN[j]:=DataInit.RHOS1;
      end;
      {stone-class}
      PW_G[-4,i]:=DataInit.STONES[i]/100;  PW_PARDEN[-4]:=DataInit.RHOS1;
      {init: no grains split}
      for j:=-4 to 11 do PW_S[j,i]:=0;
      {number of particles PW_K[] for each particle size class}
      dataInit.STONESm[i]:=(dataInit.STONES[i]/100)*(dataInit.SANDm[i]+dataInit.SILTm[i]+dataInit.CLAYm[i]+1.7*dataInit.OCm[i])*(100/(100-DataInit.STONES[i]-DataInit.GRAVEL[i]));
      PW_K[-4,i]:=round(100*dataInit.STONESm[i]/(power((PW_X[-4]/100000),3)*PW_PARDEN[-4]));
      dataInit.GRAVELm[i]:=(dataInit.GRAVEL[i]/100)*(dataInit.SANDm[i]+dataInit.SILTm[i]+dataInit.CLAYm[i]+1.7*dataInit.OCm[i])*(100/(100-DataInit.STONES[i]-DataInit.GRAVEL[i]));
      for j:=-3 to 0 do PW_K[j,i]:=round(0.25*100*dataInit.Gravelm[i]/(power((PW_X[j]/100000),3)*PW_PARDEN[j]));
      for j:=1 to 5  do PW_K[j,i]:=round(0.20*100*dataInit.SANDm[i]/(power((PW_X[j]/100000),3)*PW_PARDEN[j]));
      for j:=6 to 10 do PW_K[j,i]:=round(0.20*100*dataInit.SILTm[i]/(power((PW_X[j]/100000),3)*PW_PARDEN[j]));
      PW_K[11,i]:=round(100*dataInit.CLAYm[i]/(power((PW_X[11]/100000),3)*PW_PARDEN[11]));
    end;
    { 8 Conditionally: If bedrock then initialize grainsize data in bedrock layer}
    {only in case of bedrock}
    if rock.RockBottomLayers>0 then
    begin;
     {in separate variables}
     for j:=-3 to 11 do
     begin
       rock.saprofreezecounter:=0;  rock.saprorootcounter:=0;
       rock.saproPW_G[j]:=0;
       rock.saproPW_K[j]:=0;
     end;
     {also regular variables}
     for i:=1 to rbl do
     begin
      for j:=-3 to 11 do
      begin
        freezecounter[DataInit.NoLayers+i]:=0;  rootcounter[DataInit.NoLayers+i]:=0;
        rock.saproPW_G[j]:=0;  PW_G[j,DataInit.NoLayers+i]:=0;
        rock.saproPW_K[j]:=0;  PW_K[j,DataInit.NoLayers+i]:=0;
      end;
     end;
    end;
   end;
   { 9 call ReadCheckFertil (construct annual fertilization data from input)+
       call ReadCheckEvents (construct annual events data from input)+
       call Beschrijf_scenario (write scenario to logfile) }
  {reads inorganic fertilization data from fertilization.txt
   and select the relevant period.
   This period is put into fertilINT.txt}
    ReadCheckFertil; {in: period-data; out: year-specific data}
  {check if inorganic fertilization}
    fertilfile:=fileexists('fertilINT.txt');
  {reads event data from Events.txt and select the relevant period.
   This period is put into EventsINT.txt}
    ReadCheckEvents; {in: period-data; out: year-specific data}
  {check if Events}
    Eventsfile:=fileexists('EventsINT.txt');
  {delete/create or append existing monitoringfile and describe scenario}
    if (fileexists(edtMonitName.Text)and chckbxDeleteMonit.Checked)
    then
    begin
      system.assign(fmonit,edtMonitName.Text); system.rewrite(fmonit);
    end
    else if (fileexists(edtMonitName.Text)) then
         begin  {file exists but cannot be overwritten}
           system.assign(fmonit,edtMonitName.Text); system.append(fmonit);
         end
         else  {file does not exist}
         begin
           system.assign(fmonit,edtMonitName.Text); system.rewrite(fmonit);
         end;
    Beschrijf_scenario;
  { 10 Write header to 'ectorganic.out'}
  {make file containing output of ecto-organic pools and total CO2-emission from profile}
  system.assign(fectoout,'Ectorganic.out'); system.Rewrite(fectoout);
  system.Writeln(fectoout,'Year      Litterfall    Decomposable  Resistant     Biomass       ',
                               'Humus         Inert         CO2 emission');
  system.writeln(fectoout,'         (cumulative)   plant pool    plant pool    pool          ',
                               'pool          pool         (cumulative)');
  system.writeln(fectoout,'          ton C/ha      ton C/ha      ton C/ha      ton C/ha      ',
                               'ton C/ha      ton C/ha      ton C/ha');
  { 11 Create time depth diagram files (*.tdd) and write headers.}
  {TimeDepth diagrams: make files}
    txt:=cmbbxTimeDepth.Items[0]; system.assign(f1,txt+'.tdd'); system.rewrite(f1);
    txt:=cmbbxTimeDepth.Items[1]; system.assign(f2,txt+'.tdd'); system.rewrite(f2);
    txt:=cmbbxTimeDepth.Items[2]; system.assign(f3,txt+'.tdd'); system.rewrite(f3);
    txt:=cmbbxTimeDepth.Items[3]; system.assign(f4,txt+'.tdd'); system.rewrite(f4);
    system.assign(f4a,'Calcite(kgCaCO3perm2).tdd'); system.rewrite(f4a);
    txt:=cmbbxTimeDepth.Items[4]; system.assign(f5,txt+'.tdd'); system.rewrite(f5);
    txt:=cmbbxTimeDepth.Items[5]; system.assign(f6,txt+'.tdd'); system.rewrite(f6);
    txt:=cmbbxTimeDepth.Items[6]; system.assign(f7,txt+'.tdd'); system.rewrite(f7);
    txt:=cmbbxTimeDepth.Items[7]; system.assign(f8,txt+'.tdd'); system.rewrite(f8);
    txt:=cmbbxTimeDepth.Items[8]; system.assign(f9,txt+'.tdd'); system.rewrite(f9);
    txt:=cmbbxTimeDepth.Items[9]; system.assign(f10,txt+'.tdd'); system.rewrite(f10);
    txt:=cmbbxTimeDepth.Items[10]; system.assign(f11,txt+'.tdd'); system.rewrite(f11);
    txt:=cmbbxTimeDepth.Items[11]; system.assign(f12,txt+'.tdd'); system.rewrite(f12);
    txt:=cmbbxTimeDepth.Items[12]; system.assign(f13,txt+'.tdd'); system.rewrite(f13);
    txt:=cmbbxTimeDepth.Items[13]; system.assign(f14,txt+'.tdd'); system.rewrite(f14);
    system.assign(f14a,'Gypsum(kgCaSO4perm2).tdd'); system.rewrite(f14a);
    txt:=cmbbxTimeDepth.Items[14]; system.assign(f15,txt+'.tdd'); system.rewrite(f15);
    txt:=cmbbxTimeDepth.Items[15]; system.assign(f16,txt+'.tdd'); system.rewrite(f16);
    txt:=cmbbxTimeDepth.Items[16]; system.assign(f17,txt+'.tdd'); system.rewrite(f17);
    txt:=cmbbxTimeDepth.Items[17]; system.assign(f18,txt+'.tdd'); system.rewrite(f18);
    txt:=cmbbxTimeDepth.Items[18]; system.assign(f19,txt+'.tdd'); system.rewrite(f19);
    txt:=cmbbxTimeDepth.Items[19]; system.assign(f20,txt+'.tdd'); system.rewrite(f20);
    txt:=cmbbxTimeDepth.Items[20]; system.assign(f21,txt+'.tdd'); system.rewrite(f21);
    txt:=cmbbxTimeDepth.Items[21]; system.assign(f22,txt+'.tdd'); system.rewrite(f22);
    txt:=cmbbxTimeDepth.Items[22]; system.assign(f23,txt+'.tdd'); system.rewrite(f23);
    txt:=cmbbxTimeDepth.Items[23]; system.assign(f24,txt+'.tdd'); system.rewrite(f24);
    txt:=cmbbxTimeDepth.Items[24]; system.assign(f25,txt+'.tdd'); system.rewrite(f25);
    txt:=cmbbxTimeDepth.Items[25]; system.assign(f26,txt+'.tdd'); system.rewrite(f26);
    txt:=cmbbxTimeDepth.Items[26]; system.assign(f27,txt+'.tdd'); system.rewrite(f27);
    txt:=cmbbxTimeDepth.Items[27]; system.assign(f28,txt+'.tdd'); system.rewrite(f28);
    txt:=cmbbxTimeDepth.Items[28]; system.assign(f29,txt+'.tdd'); system.rewrite(f29);
    txt:=cmbbxTimeDepth.Items[29]; system.assign(f30,txt+'.tdd'); system.rewrite(f30);
    txt:=cmbbxTimeDepth.Items[30]; system.assign(f31,txt+'.tdd'); system.rewrite(f31);
    txt:=cmbbxTimeDepth.Items[31]; system.assign(f32,txt+'.tdd'); system.rewrite(f32);
    txt:=cmbbxTimeDepth.Items[32]; system.assign(f33,txt+'.tdd'); system.rewrite(f33);
    txt:=cmbbxTimeDepth.Items[33]; system.assign(f34,txt+'.tdd'); system.rewrite(f34);
    txt:=cmbbxTimeDepth.Items[34]; system.assign(f35,txt+'.tdd'); system.rewrite(f35);
    txt:=cmbbxTimeDepth.Items[35]; system.assign(f36,txt+'.tdd'); system.rewrite(f36);
    txt:=cmbbxTimeDepth.Items[36]; system.assign(f37,txt+'.tdd'); system.rewrite(f37);
    txt:=cmbbxTimeDepth.Items[37]; system.assign(f38,txt+'.tdd'); system.rewrite(f38);
    txt:=cmbbxTimeDepth.Items[38]; system.assign(f39,txt+'.tdd'); system.rewrite(f39);
    txt:=cmbbxTimeDepth.Items[39]; system.assign(f40,txt+'.tdd'); system.rewrite(f40);
    txt:=cmbbxTimeDepth.Items[40]; system.assign(f41,txt+'.tdd'); system.rewrite(f41);
    txt:=cmbbxTimeDepth.Items[41]; system.assign(f42,txt+'.tdd'); system.rewrite(f42);
    txt:=cmbbxTimeDepth.Items[42]; system.assign(f43,txt+'.tdd'); system.rewrite(f43);
    txt:=cmbbxTimeDepth.Items[43]; system.assign(f44,txt+'.tdd'); system.rewrite(f44);
    system.assign(f44a,'Gibbsite(kgAlOH3perm2).tdd'); system.rewrite(f44a);
    txt:=cmbbxTimeDepth.Items[44]; system.assign(f45,txt+'.tdd'); system.rewrite(f45);
  txt:=cmbbxTimeDepth.Items[45]; system.assign(f46,txt+'.tdd'); system.rewrite(f46);
  txt:=cmbbxTimeDepth.Items[46]; system.assign(f47,txt+'.tdd'); system.rewrite(f47);
  txt:=cmbbxTimeDepth.Items[47]; system.assign(f48,txt+'.tdd'); system.rewrite(f48);
  txt:=cmbbxTimeDepth.Items[48]; system.assign(f49,txt+'.tdd'); system.rewrite(f49);
  txt:=cmbbxTimeDepth.Items[49]; system.assign(f50,txt+'.tdd'); system.rewrite(f50);
  txt:=cmbbxTimeDepth.Items[50]; system.assign(f51,txt+'.tdd'); system.rewrite(f51);
  txt:=cmbbxTimeDepth.Items[51]; system.assign(f52,txt+'.tdd'); system.rewrite(f52);
  txt:=cmbbxTimeDepth.Items[52]; system.assign(f53,txt+'.tdd'); system.rewrite(f53);
  txt:=cmbbxTimeDepth.Items[53]; system.assign(f54,txt+'.tdd'); system.rewrite(f54);
  txt:=cmbbxTimeDepth.Items[54]; system.assign(f55,txt+'.tdd'); system.rewrite(f55);
  txt:=cmbbxTimeDepth.Items[55]; system.assign(f56,txt+'.tdd'); system.rewrite(f56);
  txt:=cmbbxTimeDepth.Items[56]; system.assign(f57,txt+'.tdd'); system.rewrite(f57);
  txt:=cmbbxTimeDepth.Items[57]; system.assign(f58,txt+'.tdd'); system.rewrite(f58);
  txt:=cmbbxTimeDepth.Items[58]; system.assign(f59,txt+'.tdd'); system.rewrite(f59);
  txt:=cmbbxTimeDepth.Items[59]; system.assign(f60,txt+'.tdd'); system.rewrite(f60);
  txt:=cmbbxTimeDepth.Items[60]; system.assign(f61,txt+'.tdd'); system.rewrite(f61);
  txt:=cmbbxTimeDepth.Items[61]; system.assign(f62,txt+'.tdd'); system.rewrite(f62);
  txt:=cmbbxTimeDepth.Items[62]; system.assign(f63,txt+'.tdd'); system.rewrite(f63);
  txt:=cmbbxTimeDepth.Items[63]; system.assign(f64,txt+'.tdd'); system.rewrite(f64);
  txt:=cmbbxTimeDepth.Items[64]; system.assign(f65,txt+'.tdd'); system.rewrite(f65);
  txt:=cmbbxTimeDepth.Items[65]; system.assign(f66,txt+'.tdd'); system.rewrite(f66);
  txt:=cmbbxTimeDepth.Items[66]; system.assign(f67,txt+'.tdd'); system.rewrite(f67);
  txt:=cmbbxTimeDepth.Items[67]; system.assign(f68,txt+'.tdd'); system.rewrite(f68);
  txt:=cmbbxTimeDepth.Items[68]; system.assign(f69,txt+'.tdd'); system.rewrite(f69);
  txt:=cmbbxTimeDepth.Items[69]; system.assign(f70,txt+'.tdd'); system.rewrite(f70);
  txt:=cmbbxTimeDepth.Items[70]; system.assign(f71,txt+'.tdd'); system.rewrite(f71);
  txt:=cmbbxTimeDepth.Items[71]; system.assign(f72,txt+'.tdd'); system.rewrite(f72);
  txt:=cmbbxTimeDepth.Items[72]; system.assign(f73,txt+'.tdd'); system.rewrite(f73);
  txt:=cmbbxTimeDepth.Items[73]; system.assign(f74,txt+'.tdd'); system.rewrite(f74);
  txt:=cmbbxTimeDepth.Items[74]; system.assign(f75,txt+'.tdd'); system.rewrite(f75);
  txt:=cmbbxTimeDepth.Items[75]; system.assign(f76,txt+'.tdd'); system.rewrite(f76);
  txt:=cmbbxTimeDepth.Items[76]; system.assign(f77,txt+'.tdd'); system.rewrite(f77);
  txt:=cmbbxTimeDepth.Items[77]; system.assign(f78,txt+'.tdd'); system.rewrite(f78);
  txt:=cmbbxTimeDepth.Items[78]; system.assign(f79,txt+'.tdd'); system.rewrite(f79);
  txt:=cmbbxTimeDepth.Items[79]; system.assign(f80,txt+'.tdd'); system.rewrite(f80);
  txt:=cmbbxTimeDepth.Items[80]; system.assign(f81,txt+'.tdd'); system.rewrite(f81);
  txt:=cmbbxTimeDepth.Items[81]; system.assign(f82,txt+'.tdd'); system.rewrite(f82);
  txt:=cmbbxTimeDepth.Items[82]; system.assign(f83,txt+'.tdd'); system.rewrite(f83);
  txt:=cmbbxTimeDepth.Items[83]; system.assign(f84,txt+'.tdd'); system.rewrite(f84);
  txt:=cmbbxTimeDepth.Items[84]; system.assign(f85,txt+'.tdd'); system.rewrite(f85);
  txt:=cmbbxTimeDepth.Items[85]; system.assign(f86,txt+'.tdd'); system.rewrite(f86);
  txt:=cmbbxTimeDepth.Items[86]; system.assign(f87,txt+'.tdd'); system.rewrite(f87);
  system.assign(f87a,'Pedogenic-Fe(kgFeOOHperm2).tdd'); system.rewrite(f87a);
  txt:=cmbbxTimeDepth.Items[87]; system.assign(f88,txt+'.tdd'); system.rewrite(f88);
  system.assign(f88a,'Pedogenic-Si(kgH4SiO4perm2).tdd'); system.rewrite(f88a);
  txt:=cmbbxTimeDepth.Items[88]; system.assign(f89,txt+'.tdd'); system.rewrite(f89);
  txt:=cmbbxTimeDepth.Items[89]; system.assign(f90,txt+'.tdd'); system.rewrite(f90);
  txt:=cmbbxTimeDepth.Items[90]; system.assign(f91,txt+'.tdd'); system.rewrite(f91);
  txt:=cmbbxTimeDepth.Items[91]; system.assign(f92,txt+'.tdd'); system.rewrite(f92);
  txt:=cmbbxTimeDepth.Items[92]; system.assign(f93,txt+'.tdd'); system.rewrite(f93);
  txt:=cmbbxTimeDepth.Items[93]; system.assign(f94,txt+'.tdd'); system.rewrite(f94);
  txt:=cmbbxTimeDepth.Items[94]; system.assign(f95,txt+'.tdd'); system.rewrite(f95);   
  txt:=cmbbxTimeDepth.Items[95]; system.assign(f96,txt+'.tdd'); system.rewrite(f96);
  txt:=cmbbxTimeDepth.Items[96]; system.assign(f97,txt+'.tdd'); system.rewrite(f97);  
  txt:=cmbbxTimeDepth.Items[97]; system.assign(f98,txt+'.tdd'); system.rewrite(f98);
  txt:=cmbbxTimeDepth.Items[98]; system.assign(f99,txt+'.tdd'); system.rewrite(f99);
  txt:=cmbbxTimeDepth.Items[99]; system.assign(f100,txt+'.tdd'); system.rewrite(f100);
    {write headers of time-depth diagrams with info on time-depth gridcell size
     correct number of depth-gridcells for max number of deposition compartments
     that may occur relative to the original number of compartments; this number was
     deduced from Events.txt in ReadCheckEvents and put in the global variable MaxPlusCompartments}
    WriteFileHeader(f1,eenheden[1]);
    WriteFileHeader(f2,eenheden[2]);
    WriteFileHeader(f3,eenheden[3]);
    WriteFileHeader(f4,eenheden[4]);  writefileheader(f4a,'(kgCaCO3perm2)');
    WriteFileHeader(f5,eenheden[5]);   WriteFileHeader(f6,eenheden[6]);
    WriteFileHeader(f7,eenheden[7]);  WriteFileHeader(f8,eenheden[8]);   WriteFileHeader(f9,eenheden[9]);
    WriteFileHeader(f10,eenheden[10]); WriteFileHeader(f11,eenheden[11]);  WriteFileHeader(f12,eenheden[12]);
    WriteFileHeader(f13,eenheden[13]);
    WriteFileHeader(f14,eenheden[14]); WriteFileHeader(f14a,'(kgCaSO4perm2)');
    WriteFileHeader(f15,eenheden[15]);
    WriteFileHeader(f16,eenheden[16]); WriteFileHeader(f17,eenheden[17]);  WriteFileHeader(f18,eenheden[18]);
    WriteFileHeader(f19,eenheden[19]); WriteFileHeader(f20,eenheden[20]);  WriteFileHeader(f21,eenheden[21]);
    WriteFileHeader(f22,eenheden[22]); WriteFileHeader(f23,eenheden[23]);  WriteFileHeader(f24,eenheden[24]);
    WriteFileHeader(f25,eenheden[25]); WriteFileHeader(f26,eenheden[26]);  WriteFileHeader(f27,eenheden[27]);
    WriteFileHeader(f28,eenheden[28]); WriteFileHeader(f29,eenheden[29]);  WriteFileHeader(f30,eenheden[30]);
    WriteFileHeader(f31,eenheden[31]); WriteFileHeader(f32,eenheden[32]);  WriteFileHeader(f33,eenheden[33]);
    WriteFileHeader(f34,eenheden[34]); WriteFileHeader(f35,eenheden[35]);  WriteFileHeader(f36,eenheden[36]);
    WriteFileHeader(f37,eenheden[37]); WriteFileHeader(f38,eenheden[38]);  WriteFileHeader(f39,eenheden[39]);
    WriteFileHeader(f40,eenheden[40]); WriteFileHeader(f41,eenheden[41]);  WriteFileHeader(f42,eenheden[42]);
    WriteFileHeader(f43,eenheden[43]);
    WriteFileHeader(f44,eenheden[44]); WriteFileHeader(f44a,'(kgAlOH3perm2)');
    WriteFileHeader(f45,eenheden[45]);
    WriteFileHeader(f46,eenheden[46]); WriteFileHeader(f47,eenheden[47]);
    WriteFileHeader(f48,eenheden[48]); WriteFileHeader(f49,eenheden[49]);
    WriteFileHeader(f50,eenheden[50]); WriteFileHeader(f51,eenheden[51]);
    WriteFileHeader(f52,eenheden[52]); WriteFileHeader(f53,eenheden[53]);
    WriteFileHeader(f54,eenheden[54]); WriteFileHeader(f55,eenheden[55]);
    WriteFileHeader(f56,eenheden[56]); WriteFileHeader(f57,eenheden[57]);
    WriteFileHeader(f58,eenheden[58]); WriteFileHeader(f59,eenheden[59]);
    WriteFileHeader(f60,eenheden[60]); WriteFileHeader(f61,eenheden[61]);
    WriteFileHeader(f62,eenheden[62]); WriteFileHeader(f63,eenheden[63]);
    WriteFileHeader(f64,eenheden[64]); WriteFileHeader(f65,eenheden[65]);
    WriteFileHeader(f66,eenheden[66]); WriteFileHeader(f67,eenheden[67]);
    WriteFileHeader(f68,eenheden[68]); WriteFileHeader(f69,eenheden[69]);
    WriteFileHeader(f70,eenheden[70]); WriteFileHeader(f71,eenheden[71]);
    WriteFileHeader(f72,eenheden[72]); WriteFileHeader(f73,eenheden[73]);
    WriteFileHeader(f74,eenheden[74]); WriteFileHeader(f75,eenheden[75]);
    WriteFileHeader(f76,eenheden[76]); WriteFileHeader(f77,eenheden[77]);
    WriteFileHeader(f78,eenheden[78]); WriteFileHeader(f79,eenheden[79]);
    WriteFileHeader(f80,eenheden[80]);
    WriteFileHeader(f81,eenheden[81]); WriteFileHeader(f82,eenheden[82]);
    WriteFileHeader(f83,eenheden[83]); WriteFileHeader(f84,eenheden[84]);
    WriteFileHeader(f85,eenheden[85]); WriteFileHeader(f86,eenheden[86]);
    WriteFileHeader(f87,eenheden[87]); WriteFileHeader(f87a,'(kgFeOOHperm2)');
    WriteFileHeader(f88,eenheden[88]); WriteFileHeader(f88a,'(kgH4SiO4perm2)');
    WriteFileHeader(f89,eenheden[89]); WriteFileHeader(f90,eenheden[90]);
    WriteFileHeader(f91,eenheden[91]); WriteFileHeader(f92,eenheden[92]);
    WriteFileHeader(f93,eenheden[93]); WriteFileHeader(f94,eenheden[94]);
    WriteFileHeader(f95,eenheden[95]); WriteFileHeader(f96,eenheden[96]);
    WriteFileHeader(f97,eenheden[97]); WriteFileHeader(f98,eenheden[98]);     
    WriteFileHeader(f99,eenheden[99]); WriteFileHeader(f100,eenheden[100]);
  { 12 Initialize 14C activity data}
  yearBP:=spndtMultipleYears.Value-i+1; yearAD:=1950-yearBP;
  if not(fileexists('continue.rec')) then
  begin
    DataInit.RothC.ectoDPMXpool:=0; DataInit.RothC.ectoRPMXpool:=0; DataInit.RothC.ectoBIOXpool:=0;
    DataInit.RothC.ectoHUMXpool:=0; DataInit.RothC.ectoIOMXpool:=0;
    for i:=1 to dataInit.Nolayers do
    begin
      DataInit.RothC.DPMXpool[i]:=DataInit.RothC.DPMpool[i]*d14C[yearBP];
      DataInit.RothC.RPMXpool[i]:=DataInit.RothC.RPMpool[i]*d14C[yearBP];
      DataInit.RothC.BIOXpool[i]:=DataInit.RothC.BIOpool[i]*d14C[yearBP];
      DataInit.RothC.HUMXpool[i]:=DataInit.RothC.HUMpool[i]*d14C[yearBP];
      DataInit.RothC.IOMXpool[i]:=DataInit.RothC.IOMpool[i]*d14C[yearBP];
    end;
  end;
  {At this point (on tabsheet 1), the default input files have been read with ReadData, and edited,
   and were (max 6) graphical output variables selected}
  tbcntrlSHELL.TabIndex:=2; tbcntrlSHELLChange(Sender);
  { 13 call Screenoutput0 (initialize screen with 6 tdd graphs)}
  ScreenOutput0;
  newcompartments:=DataInit.NoLayers;
  { 14 call CopyRecToFrom (set all data in record DataYearIn equal to DataInit)}
  CopyRecToFrom(DataYearIn,DataInit);     {copy records basic inputfile to DataYearIn}
  for i:=1 to MaxLayers do DataYearIn.MODRET[i]:=5;
  {soil physical characteristics are always estimated with Hypres-PTF.
   Thus the effect of bioturbation on changing
   soil physical characteristics can be accounted for.}
   { 15 Couple bioturbation, climate, fertilization and events files, if any.}
  {initialisation turbationparameters}
  if bioturbfile then
  begin system.Assign(fturb,'bioturbINT.txt'); system.Reset(fturb); end;
  if climatefile then
  begin system.Assign(fclim,'climateINT.txt'); system.Reset(fclim); end;

  {Determine parameters influenced by slope and wind direction}
  {first, assign values to slope,upslopebearing,downwindbearing,latitude if not done so yet
   in batchmode}
  if not(batchmodeYN) then
  begin
    datayearin.slope:=spndtSlopeAngle.Value;
    datayearin.upslopebearing:=spndtSlopeBearing.Value;
    datayearin.downwindbearing:=spndtWindBearing.Value;
    datayearin.latitude:=spndtLatitude.Value;
  end;
  { 16 call CalcSlopeEffects (to get correction factors for P and PE based on slope, wind, latitude)}
  {From here slopewindfacP and slopewindfacPE can be set later; currently set to value}
  CalcSlopeEffects(slopewindfacP,slopewindfacPE);
  {assume slope given in via GUI as constant, but convert to percentage
   for calculations of runoff like in LEACHC. Also curve number must be calculated
   according to Soil Conservation Service 1972}
  DataInit.roslope:=tan(pi*datainit.slope/180)*100;
  DataInit.rocn2:=75; {leave as it is, since no precise determination available}
  DataYearIn.roslope:=DataInit.roslope;
  DataYearIn.rocn2:=DataInit.rocn2;
  { 15 Ctd Couple bioturbation, climate, fertilization and events files, if any.}
  if fertilfile then
  begin system.Assign(ffert,'fertilINT.txt'); system.Reset(ffert); end;
  if eventsfile then
  begin system.Assign(fevent,'EventsINT.txt'); system.Reset(fevent); end;
  { 17 Conditionally: call ReadDataRock (bedrock mineral props) + convert to mass +
                      calculate rock porosity + initialize grain size fractions.
                      Create 'Bedrock.out’ + write initial mass per mineral}
  if FileExistsUTF8('Bedrock.txt') { *Converted from FileExists*  } then
  begin
     {inlezen}
     ReadDataRock('Bedrock.txt');
     BedrockFileName:='Bedrock.txt';
     rdgrpChooseDisplay.columns:=5;
     Rock.RockBottomLayers:=rbl;
     Rock.phweathloss:=0;
     Rock.minbaseddensity:=0;
     Rock.minbaseddensity:=Rock.minbaseddensity+Rock.rockAlbite[1]*Rock.rockAlbite[3];
     Rock.minbaseddensity:=Rock.minbaseddensity+Rock.rockKFeldspar[1]*Rock.rockKFeldspar[3];
     Rock.minbaseddensity:=Rock.minbaseddensity+Rock.rockMuscovite[1]*Rock.rockMuscovite[3];
     Rock.minbaseddensity:=Rock.minbaseddensity+Rock.rockBiotite[1]*Rock.rockBiotite[3];
     Rock.minbaseddensity:=Rock.minbaseddensity+Rock.rockAnorthite[1]*Rock.rockAnorthite[3];
     Rock.minbaseddensity:=Rock.minbaseddensity+Rock.rockForsterite[1]*Rock.rockForsterite[3];
     Rock.minbaseddensity:=Rock.minbaseddensity+Rock.rockAugite[1]*Rock.rockAugite[3];
     Rock.minbaseddensity:=Rock.minbaseddensity+Rock.rockHornblende[1]*Rock.rockHornblende[3];
     Rock.minbaseddensity:=Rock.minbaseddensity+Rock.rockFayalite[1]*Rock.rockFayalite[3];
     Rock.minbaseddensity:=Rock.minbaseddensity+Rock.rockQuartz[1]*Rock.rockQuartz[3];
     Rock.minbaseddensity:=Rock.minbaseddensity+Rock.rockChlorite[1]*Rock.rockChlorite[3];
     Rock.minbaseddensity:=Rock.minbaseddensity+Rock.rockKaolinite[1]*Rock.rockKaolinite[3];
     Rock.minbaseddensity:=Rock.minbaseddensity+Rock.rockIllite[1]*Rock.rockIllite[3];
     Rock.minbaseddensity:=Rock.minbaseddensity+Rock.rockMontmorillonite[1]*Rock.rockMontmorillonite[3];
     Rock.minbaseddensity:=Rock.minbaseddensity+Rock.rockAmorphite[1]*Rock.rockAmorphite[3];
     Rock.minbaseddensity:=Rock.minbaseddensity+Rock.rockOtherite[1]*Rock.rockOtherite[3];
     Rock.minbaseddensity:=Rock.minbaseddensity+Rock.rockGibbsite[1]*Rock.rockGibbsite[3];
     Rock.minbaseddensity:=Rock.minbaseddensity+Rock.rockCalcite[1]*Rock.rockCalcite[3];
     Rock.minbaseddensity:=Rock.minbaseddensity+Rock.rockGypsum[1]*Rock.rockGypsum[3];
{    previously, rock porosity was calculated based on the mass fractions per mineral.
     Changed to calc based on mass to have compatible calcs for output to bedrock.out}
{     Rock.rockporosityperc:=0;
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockAlbite[3]/Rock.rockAlbite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockKFeldspar[3]/Rock.rockKFeldspar[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockMuscovite[3]/Rock.rockMuscovite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockBiotite[3]/Rock.rockBiotite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockAnorthite[3]/Rock.rockAnorthite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockForsterite[3]/Rock.rockForsterite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockAugite[3]/Rock.rockAugite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockHornblende[3]/Rock.rockHornblende[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockFayalite[3]/Rock.rockFayalite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockQuartz[3]/Rock.rockQuartz[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockChlorite[3]/Rock.rockChlorite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockKaolinite[3]/Rock.rockKaolinite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockIllite[3]/Rock.rockIllite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockMontmorillonite[3]/Rock.rockMontmorillonite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockAmorphite[3]/Rock.rockAmorphite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockOtherite[3]/Rock.rockOtherite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockGibbsite[3]/Rock.rockGibbsite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockCalcite[3]/Rock.rockCalcite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockGypsum[3]/Rock.rockGypsum[1];
     Rock.rockporosityperc:=100*(1-Rock.rockdensity*Rock.rockporosityperc);}

     {mass fractions [3] converted to kg and put in variabele [2]}
     Rock.rockAlbite[2]:=    Rock.rockAlbite[3]*Rock.rockdensity*dataInit.DLZ ;
     Rock.rockKFeldspar[2]:= Rock.rockKFeldspar[3]*Rock.rockdensity*dataInit.DLZ ;
     Rock.rockMuscovite[2]:= Rock.rockMuscovite[3]*Rock.rockdensity*dataInit.DLZ;
     Rock.rockBiotite[2]:=   Rock.rockBiotite[3]*Rock.rockdensity*dataInit.DLZ ;
     Rock.rockQuartz[2]:=    Rock.rockQuartz[3]*Rock.rockdensity*dataInit.DLZ ;
     Rock.rockChlorite[2]:=  Rock.rockChlorite[3]*Rock.rockdensity*dataInit.DLZ ;
     Rock.rockAnorthite[2]:= Rock.rockAnorthite[3]*Rock.rockdensity*dataInit.DLZ ;
     Rock.rockForsterite[2]:=Rock.rockForsterite[3]*Rock.rockdensity*dataInit.DLZ ;
     Rock.rockAugite[2]:=    Rock.rockAugite[3]*Rock.rockdensity*dataInit.DLZ ;
     Rock.rockKaolinite[2]:= Rock.rockKaolinite[3]*Rock.rockdensity*dataInit.DLZ ;
     Rock.rockIllite[2]:=    Rock.rockIllite[3]*Rock.rockdensity*dataInit.DLZ ;
     Rock.rockMontmorillonite[2]:= Rock.rockMontmorillonite[3]*Rock.rockdensity*dataInit.DLZ ;
     Rock.rockHornblende[2]:=Rock.rockHornblende[3]*Rock.rockdensity*dataInit.DLZ ;
     Rock.rockFayalite[2]:=  Rock.rockFayalite[3]*Rock.rockdensity*dataInit.DLZ ;
     Rock.rockAmorphite[2]:= Rock.rockAmorphite[3]*Rock.rockdensity*dataInit.DLZ ;
     Rock.rockOtherite[2]:=  Rock.rockOtherite[3]*Rock.rockdensity*dataInit.DLZ ;
    {added}
     Rock.rockGibbsite[2]:=  Rock.rockGibbsite[3]*Rock.rockdensity*dataInit.DLZ ;
     Rock.rockCalcite[2]:=   Rock.rockCalcite[3]*Rock.rockdensity*dataInit.DLZ ;
     Rock.rockGypsum[2]:=    Rock.rockGypsum[3]*Rock.rockdensity*dataInit.DLZ ;
    {calc rockporosity based on mass per mineral and specific gravity}
     Rock.rockporosityperc:=0;
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockAlbite[2]/Rock.rockAlbite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockKFeldspar[2]/Rock.rockKFeldspar[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockMuscovite[2]/Rock.rockMuscovite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockBiotite[2]/Rock.rockBiotite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockAnorthite[2]/Rock.rockAnorthite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockForsterite[2]/Rock.rockForsterite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockAugite[2]/Rock.rockAugite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockHornblende[2]/Rock.rockHornblende[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockFayalite[2]/Rock.rockFayalite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockQuartz[2]/Rock.rockQuartz[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockChlorite[2]/Rock.rockChlorite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockKaolinite[2]/Rock.rockKaolinite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockIllite[2]/Rock.rockIllite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockMontmorillonite[2]/Rock.rockMontmorillonite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockAmorphite[2]/Rock.rockAmorphite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockOtherite[2]/Rock.rockOtherite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockGibbsite[2]/Rock.rockGibbsite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockCalcite[2]/Rock.rockCalcite[1];
     Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockGypsum[2]/Rock.rockGypsum[1];
     Rock.rockporosityperc:=100*(1-Rock.rockporosityperc/DataInit.DLZ);

     {initilally, no mass lost yet}
     Rock.rockAlbite[4]:=    0;
     Rock.rockKFeldspar[4]:= 0;
     Rock.rockMuscovite[4]:= 0;
     Rock.rockBiotite[4]:=   0 ;
     Rock.rockQuartz[4]:=    0 ;
     Rock.rockChlorite[4]:=  0 ;
     Rock.rockAnorthite[4]:= 0 ;
     Rock.rockForsterite[4]:=0 ;
     Rock.rockAugite[4]:=    0 ;
     Rock.rockKaolinite[4]:= 0 ;
     Rock.rockIllite[4]:=    0 ;
     Rock.rockMontmorillonite[4]:= 0 ;
     Rock.rockHornblende[4]:=0 ;
     Rock.rockFayalite[4]:=  0 ;
     Rock.rockAmorphite[4]:= 0 ;
     Rock.rockOtherite[4]:=  0 ;
     Rock.rockGibbsite[4]:=  0 ;
     Rock.rockCalcite[4]:=   0 ;
     Rock.rockGypsum[4]:=    0 ;
     for i:=1 to rbl do
     begin
       rock.saprosandm:=0;     dataInit.SANDm[DataInit.NoLayers+i]:=rock.saprosandm;
       rock.saprofinesandm:=0; dataInit.fineSANDm[DataInit.NoLayers+i]:=rock.saprofinesandm;
       rock.saprosiltm:=0;     dataInit.SILTm[DataInit.NoLayers+i]:=rock.saprosiltm;
       rock.saproclaym:=0;     dataInit.CLAYm[DataInit.NoLayers+i]:=rock.saproclaym;
       rock.saprogravelm:=0;   dataInit.GRAVELm[DataInit.NoLayers+i]:= rock.saprogravelm;
       rock.saprostonesm:=rock.rockdensity*DataInit.DLZ; dataInit.STONESm[DataInit.NoLayers+i]:=rock.saprostonesm;
       rock.saproPW_G[-4]:=1;  PW_G[-4,DataInit.NoLayers+i]:=1;
       rock.saproPW_K[-4]:=round(100*rock.saproSTONESm/(power((PW_X[-4]/100000),3)*PW_PARDEN[-4]));
       PW_K[-4,DataInit.NoLayers+i]:=rock.saproPW_K[-4];
     end;
     with Rock do
     begin
       system.assign(ftmp,'Bedrock.out'); system.rewrite(ftmp);
       System.writeln(ftmp,'Mass per mineral');
       system.writeln(ftmp,'Year   ','Albite          ','K-Feldspar      ',
       'Muscovite       ','Biotite         ','Anorthite       ',
       'Forsterite      ','Augite          ','Hornblende      ',
       'Fayalite        ','Quartz          ','Chlorite        ',
       'Kaolinite       ','Illite          ','Montmorillonite ',
       'Amorphite       ','Otherite        ','Gibbsite        ',
       'Calcite         ','Gypsum         ','Porosity PhWeathL% ',
       'saproCL  saproSI  saproSA  saproGR saproST');
       system.write(ftmp,'0     ');
       system.write(ftmp,rockAlbite[2]:15:12,' ');
       system.write(ftmp,rockKFeldspar[2]:15:12,' ');
       system.write(ftmp,rockMuscovite[2]:15:12,' ');
       system.write(ftmp,rockBiotite[2]:15:12,' ');
       system.write(ftmp,rockAnorthite[2]:15:12,' ');
       system.write(ftmp,rockForsterite[2]:15:12,' ');
       system.write(ftmp,rockAugite[2]:15:12,' ');
       system.write(ftmp,rockHornblende[2]:15:12,' ');
       system.write(ftmp,rockFayalite[2]:15:12,' ');
       system.write(ftmp,rockQuartz[2]:15:12,' ');
       system.write(ftmp,rockChlorite[2]:15:12,' ');
       system.write(ftmp,rockKaolinite[2]:15:12,' ');
       system.write(ftmp,rockIllite[2]:15:12,' ');
       system.write(ftmp,rockMontmorillonite[2]:15:12,' ');
       system.write(ftmp,rockAmorphite[2]:15:12,' ');
       system.write(ftmp,rockOtherite[2]:15:12,' ');
       system.write(ftmp,rockGibbsite[2]:15:12,' ');
       system.write(ftmp,rockCalcite[2]:15:12,' ');
       system.write(ftmp,rockGypsum[2]:15:12,' ');
       system.write(ftmp,Rock.rockporosityperc:8:5,' ');
       system.write(ftmp,rock.phweathloss:8:5,' ');
       system.write(ftmp,rock.saproclaym:8:5,' ');       
       system.write(ftmp,rock.saprosiltm:8:5,' ');
       system.write(ftmp,rock.saprosandm:8:5,' ');
       system.write(ftmp,rock.saprogravelm:8:5,' ');
       system.writeln(ftmp,rock.saprostonesm:8:5);
       closefile(ftmp);
     end;

     Rock_original:=Rock;
  end
  else
  begin
     rdgrpChooseDisplay.columns:=5; {was 4}
     Rock.RockBottomLayers:=0;
  end;
  { 18 call CopyRecToFrom (set all data in record DataYearIn equal to DataInit)}
  {testje}CopyRecToFrom(DataYearIn,DataInit);     {copy records basic inputfile to DataYearIn}
  StartTime:=Now;
  {If GUItrace=1 then wwwriteln} singletracewriteln('bttnRunClick: starting annual cycle loop');
  for i:=1 to spndtMultipleYears.Value do
  begin
    { 19 Conditionally: if daily output tdd's are requested then call AssignDailyTDD}
    writeDaily:=chckbxDailyTDD.Checked;
    if writeDaily then AssignDailyTDD(i);
    { 22 Assign pCO2 for the current year}
    pCO2atm:=SeriespCO2[spndtMultipleYears.Value-i+1];
    iyear:=i; GlobalTime:=iyear; yearBP:=spndtMultipleYears.Value-i+1;
    yearAD:=1950-yearBP;
    DataYearIn.WaterTableDepth:=DataYearIn.DLZ*MaxLayers; {default: very deep water table}
    { 20 Conditionally: if tracing option applies for this year then set an indicator variable}
    if iyear>=spndtTraceStart.Value
    then begin untProcesses.TRACE:=rdgrpTrace.ItemIndex; GUItrace:=rdgrpTrace.ItemIndex; end
    else begin untProcesses.TRACE:=0; GUItrace:=0; end;
    rdgrpTrace.Color:=clBtnFace;
    spndtTraceStart.Color:=clWindow;
    { 21 Conditionally: if the program abortion checkbox was checked then
                        call WriteData, call WriteDataC, call WriteDataWE, write logfile, raise EInOutError}
    if chckbxAbortNOW.Checked
    then
    begin
      {make annual input file for the case that simulations are to be continued later}
      WriteData('LC2BCont.');
      {make RothC input file for the case that simulations are to be continued later}
      WriteDataC('RC2BCont.');  {invoerbestand RothC}
      {make Weathering input file for the case that simulations are to be continued later}
      WriteDataWE('WE2BCont.'); {input file Weathering routines}
      system.Writeln(fmonit);
      system.writeln(fmonit,'Program aborted by user');
      system.writeln(fmonit,'The following files were saved:');
      system.Writeln(fmonit,' <LC2BCont.> as new soil & water input file;');
      system.Writeln(fmonit,' <RC2BCont.> as C-turnover status file;');
      system.Writeln(fmonit,' <WE2BCont.> as weathering status file.');
      raise EInOutError.Create('Aborted by user!');
    end;
    str(i,si); str(i-1,sid1);
    currentyear:='Simulation year '+si;
    DataYearIn.firstyear:=(i=1);
    lblProgress.Caption:='Progress (ready: year 1-'+sid1+').';
    str(i+1000000,si); si:='LC'+copy(si,2,6);
    { 23 Read climate data for this year
         (or for 12 months in this year; if special option chckbxSpecialClimate activated)}
    {if necessary, apply climate corrections to P and PE data}
    if climatefile then
    begin
      system.Readln(fclim,year,Ptot,PEtot,JanT,JulT,
                          DataYearIn.RothC.PlantResSUM,DataYearIn.RothC.manureSUM,
                          vegperiodtxt);
      {if monthly data for P, EP, Temp: then read these values}
      if chckbxSpecialClimate.checked
      then
      begin
        system.read(fmonthlyPREC,specialclimateyear);
        if round(spndtMultipleYears.Value-year+1)<>round(specialclimateyear)
        then dshowmessage('Year '+currentyear+'; Fatal error: synchronisation error monthly precipitation file')
        else
        begin
          for i1:=1 to 12 do system.read(fmonthlyPREC,PtotMonthly[i1]);
          system.readln(fmonthlyPREC);
        end;
        system.read(fmonthlyEVAP,specialclimateyear);
        if round(spndtMultipleYears.Value-year+1)<>round(specialclimateyear)
        then dshowmessage('Year '+currentyear+'; Fatal error: synchronisation error monthly evaporation file')
        else
        begin
          for i1:=1 to 12 do system.read(fmonthlyEVAP,PEtotMonthly[i1]);
          system.readln(fmonthlyEVAP);
        end;
        system.read(fmonthlyTEMP,specialclimateyear);
        if round(spndtMultipleYears.Value-year+1)<>round(specialclimateyear)
        then dshowmessage('Year '+currentyear+'; Fatal error: synchronisation error monthly temperature file')
        else
        begin
          for i1:=1 to 12 do system.read(fmonthlyTEMP,AvTempMonthly[i1]);
          system.readln(fmonthlyTEMP);
        end;
      end;
      { 24 Calculate correction factor for P and PE data in default year
           using the actual values from the climate file}
      {correction factor for yearly P and PE in default-file to actuel P and PE}
      corP:=Ptot/defaultP; corPE:=PEtot/defaultPE;
      { 25 Calculate correction factor for P and PE for slope, wind, latitude}
      {additional correction factor if profile is located on a slope;
       these factors can be calculated with known slope angle, slope orientation, wind direction, wind velocity}
      corP:=corP*slopewindfacP;
      {corPE:=corPE*slopewindfacPE correction conditional to presence of forest; later};
      { 26 Convert daily temperature data in default year using the actual values from the climate file}
      {convert temperature data in defaultfile using the actual JanT and JulT}
      JanFileGem:=(DataInit.avetem[1]+DataInit.avetem[2]+DataInit.avetem[3]+DataInit.avetem[4])/4;
      JulFileGem:=(DataInit.avetem[27]+DataInit.avetem[28]+DataInit.avetem[29]+DataInit.avetem[30])/4;
      {correction temperature (shift) to be fixed in 3rd week january and 3rd week july}
      corT[3]:=JanT-JanFileGem; corT[29]:=JulT-JulFileGem;
      {other weeks: interpolate correction temperatures}
      for i1:=4 to 28 do corT[i1]:=corT[3]+(corT[29]-corT[3])*(i1-3)/26;
      for i1:=30 to 53 do corT[i1]:=corT[29]+(corT[3]-corT[29])*(i1-29)/27;
      for i1:=1 to 2 do corT[i1]:=corT[29]+(corT[3]-corT[29])*(i1+53)/56;
      {vegetation type and climatic period are to be separated (as text)}
      {remove trailing spaces}
      while vegperiodtxt[1]=' ' do delete(vegperiodtxt,1,1);
      DataYearIn.RothC.vegetatie:=copy(vegperiodtxt,1,pos(' ',vegperiodtxt)-1);
      delete(vegperiodtxt,1,pos(' ',vegperiodtxt)-1);
      if (DataYearIn.RothC.vegetatie<>'agriculture')and(DataYearIn.RothC.vegetatie<>'grassscrub')and
         (DataYearIn.RothC.vegetatie<>'deciduouswood')and(DataYearIn.RothC.vegetatie<>'coniferouswood')
      then
        dshowmessage('Year '+currentyear+'; Fatal error: unknown type of vegetation ['+DataYearIn.RothC.vegetatie+'] in climate file');
      { 27 Correct daily P with interception evaporation, which is vegetation dependent and the correction factor}
      {correct precipitation with interception evaporation, which is vegetation dependent}
      if DataYearIn.RothC.vegetatie='agriculture' then frPINT:=untProcesses.frPINT_agriculture
      else if DataYearIn.RothC.vegetatie='grassscrub' then frPINT:=untProcesses.frPINT_grassscrub
           else if DataYearIn.RothC.vegetatie='deciduouswood' then frPINT:=untProcesses.frPINT_deciduouswood
                else if DataYearIn.RothC.vegetatie='coniferouswood' then frPINT:=untProcesses.frPINT_coniferouswood
                     else frPINT:=0;
      corP:=corP*(1-frPINT);
      { 28 Correct weekly PE for for slope, wind, latitude}
      if (DataYearIn.RothC.vegetatie='deciduouswood')or(DataYearIn.RothC.vegetatie='coniferouswood')
      then corPE:=corPE                {no radiation/slope correction for PE
                                        because radiation + temperature are assumed
                                        to be constant at the forest floor even when slope
                                        and aspect differ}
      else corPE:=corPE*slopewindfacPE; {apply radiation correction for slope+aspect}
      if round(year)<>i then dshowmessage('Year '+currentyear+'; Fatal error: synchronisation error climate file');
      { 29 Conditionally:  If chckbxSpecialClimate activated:
                           correct monthly P for interception + monthly P and PE with correction factor}
      {at this point the climate data can be corrected using
       1. either the annual correction factor (default, chckbxSpecialClimate.checked=false), or
       2. or the monthly correction factor (when chckbxSpecialClimate.checked=true)}
      if chckbxSpecialClimate.checked=false
      then
      begin
        for i1:=1 to DataYearIn.NIRRIG do DataYearIn.IRRIGA[i1]:=DataInit.IRRIGA[i1]*corP;
        for i1:=1 to DataYearIn.NWEEK do
        begin
          DataYearIn.PAN[i1]:=DataInit.PAN[i1]*corPE;
          DataYearIn.AVETEM[i1]:=DataInit.AVETEM[i1]+corT[i1];
        end;
        end
      else {there are monthly values for P, PE and Temp}
      begin
        {generic corrections for slope/exposition}
        corP:=slopewindfacP*(1-frPINT);
        if (DataYearIn.RothC.vegetatie='deciduouswood')or(DataYearIn.RothC.vegetatie='coniferouswood')
        then corPE:=1
        else corPE:=slopewindfacPE;
        for i1:=1 to DataYearIn.NIRRIG do
        begin
          {make monthly correction factors: first select the month for any Julian day i1}
                if (i1<=31)  then month:=1
           else if (i1<=59)  then month:=2
           else if (i1<=90)  then month:=3
           else if (i1<=120) then month:=4
           else if (i1<=151) then month:=5
           else if (i1<=181) then month:=6
           else if (i1<=212) then month:=7
           else if (i1<=243) then month:=8
           else if (i1<=273) then month:=9
           else if (i1<=304) then month:=10
           else if (i1<=334) then month:=11
           else if (i1<=365) then month:=12 ;
           {second: correct the daily value for precipitation using the ratio default/this_year for the current month}
           DataYearIn.IRRIGA[i1]:=DataInit.IRRIGA[i1]*PtotMonthly[month]*corP/max(DefaultPtotMonthly[month],0.1);
        end;
        for i1:=1 to DataYearIn.NWEEK do
        begin
          month:=-1;
          case i1 of 1 : month:=1;  2: month:=1;  3:month:=1;  4:month:=1;
                     6 : month:=2;  7: month:=2;  8:month:=2;
                     10: month:=3; 11: month:=3; 12:month:=3;
                     14: month:=4; 15: month:=4; 16:month:=4; 17:month:=4;
                     19: month:=5; 20: month:=5; 21:month:=5;
                     23: month:=6; 24: month:=6; 25:month:=6;
                     27: month:=7; 28: month:=7; 29:month:=7; 30:month:=7;
                     32: month:=8; 33: month:=8; 34:month:=8;
                     36: month:=9; 37: month:=9; 38:month:=9; 39:month:=9;
                     40: month:=10;41: month:=10;42:month:=10;43:month:=10;
                     45: month:=11;46: month:=11;47:month:=11;
                     49: month:=12;50: month:=12;51:month:=12;52:month:=12; 53:month:=12;
          end;
          if month>0 {week falls fully within a month}
          then
          begin
             DataYearIn.PAN[i1]:=DataInit.PAN[i1]*corPE*PEtotmonthly[month]/max(defaultPEtotmonthly[month],0.1);
             DataYearIn.AVETEM[i1]:=DataInit.AVETEM[i1]+AVtempMonthly[month]-defaultAvTempMonthly[month];
          end
          else  {week falls partly in 2 months}
          begin
            if i1=5
            then begin DataYearIn.PAN[i1]:=DataInit.PAN[i1]*corPE*
                                     ((3*PEtotmonthly[1]/max(defaultPEtotmonthly[1],0.1))+
                                     (4*PEtotmonthly[2]/max(defaultPEtotmonthly[2],0)))/7;
                       DataYearIn.AVETEM[i1]:=DataInit.AVETEM[i1]+(3/7)*(AVtempMonthly[1]-defaultAvTempMonthly[1])+
                                                                  (4/7)*(AVtempMonthly[2]-defaultAvTempMonthly[2]);
                 end
            else if i1=9
            then begin DataYearIn.PAN[i1]:=DataInit.PAN[i1]*corPE*
                                     ((3*PEtotmonthly[2]/max(defaultPEtotmonthly[2],0.1))+
                                     (4*PEtotmonthly[3]/max(defaultPEtotmonthly[3],0.1)))/7;
                       DataYearIn.AVETEM[i1]:=DataInit.AVETEM[i1]+(3/7)*(AVtempMonthly[2]-defaultAvTempMonthly[2])+
                                                                  (4/7)*(AVtempMonthly[3]-defaultAvTempMonthly[3]);
                 end
            else if i1=13
            then begin DataYearIn.PAN[i1]:=DataInit.PAN[i1]*corPE*
                                     ((6*PEtotmonthly[3]/max(defaultPEtotmonthly[3],0.1))+
                                     (1*PEtotmonthly[4]/max(defaultPEtotmonthly[4],0.1)))/7;
                       DataYearIn.AVETEM[i1]:=DataInit.AVETEM[i1]+(6/7)*(AVtempMonthly[3]-defaultAvTempMonthly[3])+
                                                                  (1/7)*(AVtempMonthly[4]-defaultAvTempMonthly[4]);
                 end
            else if i1=18
            then begin DataYearIn.PAN[i1]:=DataInit.PAN[i1]*corPE*
                                     ((1*PEtotmonthly[4]/max(defaultPEtotmonthly[4],0.1))+
                                     (6*PEtotmonthly[5]/max(defaultPEtotmonthly[5],0.1)))/7;
                       DataYearIn.AVETEM[i1]:=DataInit.AVETEM[i1]+(1/7)*(AVtempMonthly[4]-defaultAvTempMonthly[4])+
                                                                  (6/7)*(AVtempMonthly[5]-defaultAvTempMonthly[5]);
                 end
            else if i1=22
            then begin DataYearIn.PAN[i1]:=DataInit.PAN[i1]*corPE*
                                     ((4*PEtotmonthly[5]/max(defaultPEtotmonthly[5],0.1))+
                                     (3*PEtotmonthly[6]/max(defaultPEtotmonthly[6],0.1)))/7 ;
                       DataYearIn.AVETEM[i1]:=DataInit.AVETEM[i1]+(4/7)*(AVtempMonthly[5]-defaultAvTempMonthly[5])+
                                                                  (3/7)*(AVtempMonthly[6]-defaultAvTempMonthly[6]);
                 end
            else if i1=26
            then begin DataYearIn.PAN[i1]:=DataInit.PAN[i1]*corPE*
                                     ((6*PEtotmonthly[6]/max(defaultPEtotmonthly[6],0.1))+
                                     (1*PEtotmonthly[7]/max(defaultPEtotmonthly[7],0.1)))/7;
                       DataYearIn.AVETEM[i1]:=DataInit.AVETEM[i1]+(6/7)*(AVtempMonthly[6]-defaultAvTempMonthly[6])+
                                                                  (1/7)*(AVtempMonthly[7]-defaultAvTempMonthly[7]);
                 end
            else if i1=31
            then begin DataYearIn.PAN[i1]:=DataInit.PAN[i1]*corPE*
                                     ((2*PEtotmonthly[7]/max(defaultPEtotmonthly[7],0.1))+
                                     (5*PEtotmonthly[8]/max(defaultPEtotmonthly[8],0.1)))/7 ;
                       DataYearIn.AVETEM[i1]:=DataInit.AVETEM[i1]+(2/7)*(AVtempMonthly[7]-defaultAvTempMonthly[7])+
                                                                  (5/7)*(AVtempMonthly[8]-defaultAvTempMonthly[8]);
                 end
            else if i1=35
            then begin DataYearIn.PAN[i1]:=DataInit.PAN[i1]*corPE*
                                     ((5*PEtotmonthly[8]/max(defaultPEtotmonthly[8],0.1))+
                                     (2*PEtotmonthly[9]/max(defaultPEtotmonthly[9],0.1)))/7 ;
                       DataYearIn.AVETEM[i1]:=DataInit.AVETEM[i1]+(5/7)*(AVtempMonthly[8]-defaultAvTempMonthly[8])+
                                                                  (2/7)*(AVtempMonthly[9]-defaultAvTempMonthly[9]);
                 end
            else  if i1=44
            then begin DataYearIn.PAN[i1]:=DataInit.PAN[i1]*corPE*
                                     ((3*PEtotmonthly[10]/max(defaultPEtotmonthly[10],0.1))+
                                     (4*PEtotmonthly[11]/max(defaultPEtotmonthly[11],0.1)))/7;
                       DataYearIn.AVETEM[i1]:=DataInit.AVETEM[i1]+(3/7)*(AVtempMonthly[10]-defaultAvTempMonthly[10])+
                                                                  (4/7)*(AVtempMonthly[1]-defaultAvTempMonthly[11]);
                 end
            else  if i1=48
            then begin DataYearIn.PAN[i1]:=DataInit.PAN[i1]*corPE*
                                     ((5*PEtotmonthly[11]/max(defaultPEtotmonthly[11],0.1))+
                                     (2*PEtotmonthly[12]/max(defaultPEtotmonthly[12],0.1)))/7;
                       DataYearIn.AVETEM[i1]:=DataInit.AVETEM[i1]+(5/7)*(AVtempMonthly[11]-defaultAvTempMonthly[11])+
                                                                  (2/7)*(AVtempMonthly[12]-defaultAvTempMonthly[12]);
                 end
            end;
        end;
      end;
      {If GUItrace=1 then wwwriteln} singletracewriteln('bttnRunClick: climate variables downscaled for current year');
    end
    else {assign some data to prevent runtime errors}
    if i=1 then begin
      dshowmessage('Year '+currentyear+'; Not fatal: No climate file: Setting vegetation type to deciduouswood with annual litter input of 18 ton C/ha,yr');
      DataYearIn.RothC.vegetatie:='deciduouswood';
      DataYearIn.RothC.PlantResSUM:=18.00;
      DataYearIn.RothC.manureSUM:=0.00;
      corP:=Ptot/defaultP;
      corPE:=PEtot/defaultPE;
      corP:=corP*slopewindfacP;
      {correct precipitation with interception evaporation, which is vegetation dependent}
      frPINT:=untProcesses.frPINT_deciduouswood ;
      corP:=corP*(1-frPINT);
    end;
    { 30 Read fertilization data for this year (if any)}
    if fertilfile then
    begin
      system.Readln(ffert,year,Ca,Mg,Na,K,Cl,SO4,HCO3,CO3);
      {mol/m2 fertilizer on oct 1 of Ca,Mg,Na,K,Cl,SO4,HCO3,CO3 respectivily}
      DataYearIn.FER[1,274]:=Ca;    DataYearIn.FER[2,274]:=Mg;
      DataYearIn.FER[3,274]:=Na;    DataYearIn.FER[4,274]:=K;
      DataYearIn.FER[5,274]:=Cl;    DataYearIn.FER[6,274]:=SO4;
      DataYearIn.FER[7,274]:=HCO3;  DataYearIn.FER[8,274]:=CO3;
      if round(year)<>i then dshowmessage('Year '+currentyear+'; Fatal error: synchronisation error fertilization file');
      {If GUItrace=1 then wwwriteln} singletracewriteln('bttnRunClick: fertilization variables calculated for current year');
    end;
    untProcesses.WaterTableInRootingZone:=false;
    { 31 Read events for this year (if any) and read per type of event the descriptive data.
         Put data in DataInit, DataYearIn and DataYearOut after initializations}
    if eventsfile then
    begin
     DataYearIn.WaterTableDepth:=DataYearIn.DLZ*MaxLayers; {default: very deep water table}
     system.Read(fevent,year);
     leeswoord(fevent,txtevent);
     system.Read(fevent,nocompb);
     {If GUItrace=1 then wwwriteln} singletracewriteln('bttnRunClick: event read for current year');
     if txtevent='Erosion'
     then begin newcompartments:=newcompartments-nocompb; system.readln(fevent);
                ShiftCompartments(-nocompb); {shift non-eroded compartments upward}
          end
     else if txtevent='Deposition'
     then begin newcompartments:=newcompartments+nocompb;
                system.read(fevent,dCLAY,dSILT,dOC1,dRHO,dAW,dCALCITEA,dAGYPS,
                                     dAPCO2,dSEA1,dSEA2,dSEA3,dSEA4,dSEA5,
                                     dSEA6,dSEA7,dSEA12,dSEA13,dSEA14,dSEA15,
                                     dSEA16,dAXSE1,dAXSE2,dAXSE3,dAXSE4,
                                     dAXSE10,dAXSE12,dCEC,dSC1,dSC2,dSC3,dSC7,dSC8,
                                     dfrAlbite,dfrKfeldspar,dfrMuscovite,dfrBiotite,
                                     dfrQuartz,dfrChlorite,dfrAnorthite,dfrFosterite,
                                     dfrAugite,dfrKaolinite,dfrIllite,dfrMontmorillonite,
                                     dfrHornblende,dfrFayalite,dfrGibbsite,dfrAmorphite,dfrOtherite);
                if (eoln(fevent))
                then
                begin
                  dSTONES:=0; dGRAVEL:=0; system.readln(fevent);
                end
                else
                begin
                  system.readln(fevent,dGRAVEL,dSTONES);
                end;
                {Rescale mineral fractions so that these sum up to 1}
                tempsum:=dfrAlbite+dfrKfeldspar+dfrMuscovite+dfrBiotite+
                                     dfrQuartz+dfrChlorite+dfrAnorthite+dfrFosterite+
                                     dfrAugite+dfrKaolinite+dfrIllite+dfrMontmorillonite+
                                     dfrHornblende+dfrFayalite+dfrGibbsite+dfrAmorphite+dfrOtherite;
                dfrAlbite:=dfrAlbite/tempsum;        dfrKfeldspar:=dfrKfeldspar/tempsum;  dfrMuscovite:=dfrMuscovite/tempsum;
                dfrBiotite:=dfrBiotite/tempsum;      dfrQuartz:=dfrQuartz/tempsum;        dfrChlorite:=dfrChlorite/tempsum;
                dfrAnorthite:=dfrAnorthite/tempsum;  dfrFosterite:=dfrFosterite/tempsum;  dfrAugite:=dfrAugite/tempsum;
                dfrKaolinite:=dfrKaolinite/tempsum;  dfrIllite:=dfrIllite/tempsum;        dfrMontmorillonite:=dfrMontmorillonite/tempsum;
                dfrHornblende:=dfrHornblende/tempsum;dfrFayalite:=dfrFayalite/tempsum;    dfrGibbsite:=dfrGibbsite/tempsum;
                dfrAmorphite:=dfrAmorphite/tempsum;  dfrOtherite:=dfrOtherite/tempsum;

                {undone: force RHO to be estimated by the PTF to prevent mass balance problems later}
                {dRHO:=HYPRESRHO(100*dCLAY/(100-1.7*dOC1),dOC1);}
                {pf23: convert to mass (kg/compartment. dCALCITEA and dAGYPS inputs are fraction of fine earth,
                       and we assume nothing is present in stone+gravel fractions, so total mass of CaCO3 and CaSO4 in compartment
                       needs reductive correction by /(1+(dSTONES+dGRAVEL)/100)}
                dCALCITEA:=dCALCITEA*dRHO*DataInit.DLZ/(1+(dSTONES+dGRAVEL)/100);
                dAGYPS:=dAGYPS*dRHO*DataInit.DLZ/(1+(dSTONES+dGRAVEL)/100);
                ShiftCompartments(nocompb); {shift old compartments downward
                                             and copy data from old top compartment
                                             into newly added compartments}
                for i1:=1 to newcompartments do
                begin  {MODRET should always be 0; set here because routines are for reals only}
                  dataYearIn.MODRET[i1]:=5;   dataYearOut.MODRET[i1]:=5;
                end;
                for i1:=1 to nocompb do {overwrite new compartments with data
                                         read from EventsINT.txt}
                begin
                  {for DataInit do}
                  begin
                   DataInit.CLAY[i1]:=dCLAY; DataInit.SILT[i1]:=dSILT; DataInit.OC[i1]:=max(1e-5,dOC1);
                   dSAND:=100-dCLAY-dSILT-1.7*dOC1; DataInit.SAND[i1]:=(100-DataInit.CLAY[i1]-DataInit.SILT[i1]-1.7*DataInit.OC[i1]);
                   DataInit.GRAVEL[i1]:=dGRAVEL; DataInit.STONES[i1]:=dSTONES;
                   DataInit.RHO[i1]:=dRHO;   DataInit.AW[i1]:=dAW;     DataInit.TEM[i1]:=DataInit.AVETEM[1];
                   DataInit.CLAYm[i1]:=((100-DataInit.GRAVEL[i1]-DataInit.STONES[i1])/100)*DataInit.CLAY[i1]*DataInit.DLZ*DataInit.RHO[i1]/100;
                   DataInit.SILTm[i1]:=((100-DataInit.GRAVEL[i1]-DataInit.STONES[i1])/100)*DataInit.SILT[i1]*DataInit.DLZ*DataInit.RHO[i1]/100;
                   DataInit.SANDm[i1]:=((100-DataInit.GRAVEL[i1]-DataInit.STONES[i1])/100)*DataInit.SAND[i1]*DataInit.DLZ*DataInit.RHO[i1]/100;
                   DataInit.fineSANDm[i1]:=DataInit.SANDm[i1]*0.4; {assuming 40% is <256 microns}
                   DataInit.OCm[i1]:=((100-DataInit.GRAVEL[i1]-DataInit.STONES[i1])/100)*DataInit.OC[i1]*DataInit.DLZ*DataInit.RHO[i1]/100;
                   DataInit.ATEM[i1]:=DataInit.AVETEM[1]; DataInit.Freezecounter[i1]:=0; DataInit.Rootcounter[i1]:=0;
                   DataInit.CALCITEA[i1]:=dCALCITEA; DataInit.oldCALCITEA[i1]:=dCALCITEA;
                   DataInit.GYPA[i1]:=dAGYPS;     DataInit.oldGYPA[i1]:=dAGYPS;  DataInit.APCO2[i1]:=dAPCO2;
                   DataInit.SEA[1][i1]:=dSEA1;    DataInit.SEA[2][i1]:=dSEA2;    DataInit.SEA[3][i1]:=dSEA3;
                   DataInit.SEA[4][i1]:=dSEA4;    DataInit.SEA[5][i1]:=dSEA5;    DataInit.SEA[6][i1]:=dSEA6;
                   DataInit.SEA[7][i1]:=dSEA7;    DataInit.SEA[12][i1]:=dSEA12;  DataInit.SEA[13][i1]:=dSEA13;
                   DataInit.SEA[14][i1]:=dSEA14;  DataInit.SEA[15][i1]:=dSEA15;  DataInit.SEA[16][i1]:=dSEA16;
                   DataInit.AXSE[1][i1]:=dAXSE1;  DataInit.AXSE[2][i1]:=dAXSE2;  DataInit.AXSE[3][i1]:=dAXSE3;
                   DataInit.AXSE[4][i1]:=dAXSE4;  DataInit.AXSE[10][i1]:=dAXSE10;DataInit.AXSE[12][i1]:=dAXSE12;
                   DataInit.ACEC[i1]:=dCEC;       DataInit.SC1[i1]:=dSC1;        DataInit.SC2[i1]:=dSC2;
                   DataInit.SC3[i1]:=dSC3;        DataInit.SC7[i1]:=dSC7;        DataInit.SC8[i1]:=dSC8;
                   {2-domain CEC  and AXSE}
                    {1} DataInit.CEC_empirical[i1]:=32+36.7*DataInit.OC[i1]+1.96*DataInit.clay[i1];
                    {2} DataInit.FacCEC[i1]:=DataInit.ACEC[i1]/DataInit.CEC_empirical[i1];
                    {3} DataInit.FacCEC_OC[i1]:=36.7*DataInit.FacCEC[i1];  {FacCEC_OC contains adjusted regression coefficient}
                    {4+7} DataInit.ACEC_clay[i1]:=DataInit.ACEC[i1]-DataInit.FacCEC_OC[i1]*DataInit.OC[i1];
                          DataInit.ACEC_OC[i1]:=DataInit.ACEC[i1]-DataInit.ACEC_clay[i1];
                    {5} DataInit.ClayFrCEC[i1]:=DataInit.ACEC_clay[i1]/DataInit.ACEC[i1];
                    {6} DataInit.OCFrCEC[i1]:=DataInit.ACEC_OC[i1]/DataInit.ACEC[i1];
                    {7} for j1:=1 to MaxChem do
                    begin
                      DataInit.AXSE_oc[j1,i1]:=DataInit.AXSE[j1,i1]*DataInit.OCFrCEC[i1];
                      {_oc values will be adapted each year when the OC is adapted in RECALCOC;
                       _clay values will be adapted in the CHEM.claymigration procedures and
                                                    in the CHEM.physicalweathering procedure}
                      DataInit.AXSE_clay[j1,i1]:=DataInit.AXSE[j1,i1]*DataInit.ClayFrCEC[i1];
                    end;
                   {add: weathering data and rothc-data. Assuming in new layers:
                   a. same mineral composition as original parent material;
                   b. empty C-pools}
                   {a1. mass fractions}
                   DataInit.WE.Gibbsitepool[i1]:=dfrGibbsite;
                   DataInit.WE.MinPools[1][i1]:= dfrAlbite;
                   DataInit.WE.MinPools[2][i1]:= dfrKfeldspar;
                   DataInit.WE.MinPools[3][i1]:= dfrMuscovite;
                   DataInit.WE.MinPools[4][i1]:= dfrBiotite;
                   DataInit.WE.MinPools[5][i1]:= dfrQuartz;
                   DataInit.WE.MinPools[6][i1]:= dfrChlorite;
                   DataInit.WE.MinPools[7][i1]:= dfrAnorthite;
                   DataInit.WE.MinPools[8][i1]:= dfrFosterite;
                   DataInit.WE.MinPools[9][i1]:= dfrAugite;
                   DataInit.WE.MinPools[10][i1]:=dfrKaolinite;
                   DataInit.WE.MinPools[11][i1]:=dfrIllite;
                   DataInit.WE.MinPools[12][i1]:=dfrMontmorillonite;
                   DataInit.WE.MinPools[13][i1]:=dfrHornblende;
                   DataInit.WE.MinPools[14][i1]:=dfrFayalite;
                   DataInit.WE.MinPools[15][i1]:=dfrAmorphite;
                   DataInit.WE.MinPools[16][i1]:=dfrOtherite;
                   {a2. convert to mass}
                   totalmass:=dRHO*DataInit.DLZ*(100-dSTONES-dGRAVEL)/100;          {kg dry matter over DLZ mm compartment thickness and 1 m2 area, excluding stones+gravel}
                   totalmass:=totalmass-dCALCITEA-dAGYPS-1.7*DataYearIn.OCm[i1];    {but we subtract mass of gypsum, calcite and OC, all assumed part of fine earth}
                   totalmass:=totalmass+((dSTONES+dGRAVEL)/100)*DataInit.DLZ *2.65; {..raised by the mass in stones and gravel, with specific weight assumed to be 2.65 kg dm-3}
                   {a3. now totalmass is mass associated with the 14 minerals + gibbsite}
                   DataInit.WE.Gibbsitepool[i1]:=DataInit.WE.Gibbsitepool[i1]*totalmass;
                   for j:=1 to nrminerals do
                     DataInit.WE.MinPools[j][i1]:=DataInit.WE.MinPools[j][i1]*totalmass;
                   {a4. convert the mineral pools (kg) to elemental pools in minerals (mol)}
                   {first convert kg mineral to mol mineral by using the
                   Relative Formula Mass, RFM, per mineral from elemental weights and stoichiometric coeficients.
                   These are calculated constants in untProcesses
                   "1000" is used to convert kmol>mol}
                    tmpMinPool[1]:=1000*DataInit.WE.MinPools[1][i1]/untProcesses.RFMalbite;
                    tmpMinPool[2]:=1000*DataInit.WE.MinPools[2][i1]/untProcesses.RFMkfeldspar;
                    tmpMinPool[3]:=1000*DataInit.WE.MinPools[3][i1]/untProcesses.RFMmuscovite;
                    tmpMinPool[4]:=1000*DataInit.WE.MinPools[4][i1]/untProcesses.RFMbiotite;
                    tmpMinPool[5]:=1000*DataInit.WE.MinPools[5][i1]/untProcesses.RFMquartz;
                    tmpMinPool[6]:=1000*DataInit.WE.MinPools[6][i1]/untProcesses.RFMchlorite;
                    tmpMinPool[7]:=1000*DataInit.WE.MinPools[7][i1]/untProcesses.RFManorthite;
                    tmpMinPool[8]:=1000*DataInit.WE.MinPools[8][i1]/untProcesses.RFMForsterite;
                    tmpMinPool[9]:=1000*DataInit.WE.MinPools[9][i1]/untProcesses.RFMaugite;
                    tmpMinPool[10]:=1000*DataInit.WE.MinPools[10][i1]/untProcesses.RFMkaolinite;
                    tmpMinPool[11]:=1000*DataInit.WE.MinPools[11][i1]/untProcesses.RFMillite;
                    tmpMinPool[12]:=1000*DataInit.WE.MinPools[12][i1]/untProcesses.RFMmontmorillonite;
                    tmpMinPool[13]:=1000*DataInit.WE.MinPools[13][i1]/untProcesses.RFMhornblende;
                    tmpMinPool[14]:=1000*DataInit.WE.MinPools[14][i1]/untProcesses.RFMfayalite;
                    if RFMamorphite>0
                       then tmpMinPool[15]:=1000*DataInit.WE.MinPools[15][i1]/untProcesses.RFMamorphite
                       else tmpMinPool[15]:=0;
                    if RFMotherite>0
                       then tmpMinPool[16]:=1000*DataInit.WE.MinPools[16][i1]/untProcesses.RFMotherite
                       else tmpMinPool[16]:=0;
                    {tmpGibbsite:=1000*DataInit.WE.Gibbsitepool[i1]/untProcesses.RFMgibbsite; }
                    {tmpGibbsite was never used. Line above this one therefore commented.
                     Line below this line converts initial Gibbsite Al(OH)3 (kmol) to mol Al in Gibbsite}
                    DataInit.WE.Gibbsitepool[i1]:=1000*DataInit.WE.Gibbsitepool[i1]/untProcesses.RFMgibbsite;
                    {a5. then use the stoichiometric composition of each mineral to obtain mmoles of element:
                    {1 albite Na Al Si3 O8}
                    {2 K-feldspar K Al Si3 O8}
                    {3 muscovite K Al3 Si3 O10 (OH)2}
                    {4 biotite K Mg1.5 Fe1.5 Al Si3 O10 (OH)2}
                    {5 Quartz Si O2}
                    {6 Clorite Clinochlore Mg5 Al2 Si3 O10 (OH)8 }
                    {7 anorthite Ca Al2 Si2 O8 }
                    {8 Forsterite Mg2 Si O4}
                    {9 augite orthoferrosilite Fe Si O3 }
                    {10 kaolinite Al2 Si2 O5 (OH)4}
                    {11 Illite Si3.4 O10 (OH)2 Mg0.02 Al2.38 Ca0.01 K0.44, Godderis et al 2006}
                    {12 Montmorillonite Si4 O10 (OH)2 Mg0.33 Al1.67 Ca0.04125 Mg0.04125 K0.0825 Na0.0825 }
                    {13 hornblende Ca2 Mg4 Al Si6.7 Al O22.4 from Profile model }
                    {14 fayalite Fe2 Si O4}
                    {15 amorphite}
                    {16 otherite}
                    DataInit.WE.WE_Capool[i1]:=0*tmpMinPool[1]+0*tmpMinPool[2]+0*tmpMinPool[3]+
                              0*tmpMinPool[4]+0*tmpMinPool[5]+0*tmpMinPool[6]+
                              1*tmpMinPool[7]+0*tmpMinPool[8]+0*tmpMinPool[9]+
                              0*tmpMinPool[10]+0.01*tmpMinPool[11]+0.04125*tmpMinPool[12]+
                              2*tmpMinPool[13]+0*tmpMinPool[14]+amorp_Ca*tmpMinPool[15]+
                              other_Ca*tmpMinPool[16];
                    DataInit.WE.WE_Mgpool[i1]:=0*tmpMinPool[1]+0*tmpMinPool[2]+0*tmpMinPool[3]+
                              1.5*tmpMinPool[4]+0*tmpMinPool[5]+5*tmpMinPool[6]+
                              0*tmpMinPool[7]+2*tmpMinPool[8]+0*tmpMinPool[9]+
                              0*tmpMinPool[10]+0.02*tmpMinPool[11]+(0.33+0.04125)*tmpMinPool[12]+
                              4*tmpMinPool[13]+0*tmpMinPool[14]+amorp_Mg*tmpMinPool[15]+
                              other_Mg*tmpMinPool[16];
                    DataInit.WE.WE_Napool[i1]:=1*tmpMinPool[1]+0*tmpMinPool[2]+0*tmpMinPool[3]+
                              0*tmpMinPool[4]+0*tmpMinPool[5]+0*tmpMinPool[6]+
                              0*tmpMinPool[7]+0*tmpMinPool[8]+0*tmpMinPool[9]+
                              0*tmpMinPool[10]+0*tmpMinPool[11]+0.0825*tmpMinPool[12]+
                              0*tmpMinPool[13]+0*tmpMinPool[14];
                    DataInit.WE.WE_Kpool[i1]:=0*tmpMinPool[1]+1*tmpMinPool[2]+1*tmpMinPool[3]+
                              1*tmpMinPool[4]+0*tmpMinPool[5]+0*tmpMinPool[6]+
                              0*tmpMinPool[7]+0*tmpMinPool[8]+0*tmpMinPool[9]+
                              0*tmpMinPool[10]+0.44*tmpMinPool[11]+0.0825*tmpMinPool[12]+
                              0*tmpMinPool[13]+0*tmpMinPool[14]+amorp_K*tmpMinPool[15]+
                              other_K*tmpMinPool[16];
                    DataInit.WE.WE_Sipool[i1]:=3*tmpMinPool[1]+3*tmpMinPool[2]+3*tmpMinPool[3]+
                              3*tmpMinPool[4]+1*tmpMinPool[5]+3*tmpMinPool[6]+
                              2*tmpMinPool[7]+1*tmpMinPool[8]+1*tmpMinPool[9]+
                              2*tmpMinPool[10]+3.4*tmpMinPool[11]+4*tmpMinPool[12]+
                              6.7*tmpMinPool[13]+1*tmpMinPool[14];
                    DataInit.WE.WE_Fepool[i1]:=0*tmpMinPool[1]+0*tmpMinPool[2]+0*tmpMinPool[3]+
                              1.5*tmpMinPool[4]+0*tmpMinPool[5]+0*tmpMinPool[6]+
                              0*tmpMinPool[7]+0*tmpMinPool[8]+1*tmpMinPool[9]+
                              0*tmpMinPool[10]+0*tmpMinPool[11]+0*tmpMinPool[12]+
                              0*tmpMinPool[13]+2*tmpMinPool[14]+amorp_Fe*tmpMinPool[15]+
                              other_Fe*tmpMinPool[16];
                    DataInit.WE.WE_Alpool[i1]:=1*tmpMinPool[1]+1*tmpMinPool[2]+3*tmpMinPool[3]+
                              1*tmpMinPool[4]+0*tmpMinPool[5]+2*tmpMinPool[6]+
                              2*tmpMinPool[7]+0*tmpMinPool[8]+0*tmpMinPool[9]+
                              2*tmpMinPool[10]+2.38*tmpMinPool[11]+1.67*tmpMinPool[12]+
                              1*tmpMinPool[13]+0*tmpMinPool[14]+amorp_Al*tmpMinPool[15]+
                              other_Al*tmpMinPool[16];
                   {b. Organic matter}
                   DataInit.RothC.ClayPerc[i1]:=dCLAY*(dCLAY+dSILT+dSAND+1.7*dOC1)/(dCLAY+dSILT+dSAND);
                   DataInit.RothC.DPMpool[i1]:=0;  DataInit.RothC.RPMpool[i1]:=0;
                   DataInit.RothC.BIOpool[i1]:=0;  DataInit.RothC.HUMpool[i1]:=0;
                   DataInit.RothC.IOMpool[i1]:=0;  DataInit.RothC.CO2pool[i1]:=0;
                   DataInit.RothC.accTSMD[i1]:=0;  DataInit.RothC.pCO2[i1]:=pCO2atm;
                   {now distribute the incoming OC over the RothCpools in the same
                    way as the initial C (see ReadDataC-procedure)}
                   DataInit.RothC.DPMpool[i1]:=DataInit.RothCGEN.GENPoolPercs[1]
                                               *DataInit.OC[i1]*DataInit.RHO[i1]*DataInit.dlz/1000;
                   DataInit.RothC.RPMpool[i1]:=DataInit.RothCGEN.GENPoolPercs[2]
                                               *DataInit.OC[i1]*DataInit.RHO[i1]*DataInit.dlz/1000;
                   DataInit.RothC.BIOpool[i1]:=DataInit.RothCGEN.GENPoolPercs[3]
                                               *DataInit.OC[i1]*DataInit.RHO[i1]*DataInit.dlz/1000;
                   DataInit.RothC.HUMpool[i1]:=DataInit.RothCGEN.GENPoolPercs[4]
                                               *DataInit.OC[i1]*DataInit.RHO[i1]*DataInit.dlz/1000;
                   DataInit.RothC.IOMpool[i1]:=DataInit.RothCGEN.GENPoolPercs[5]
                                               *DataInit.OC[i1]*DataInit.RHO[i1]*DataInit.dlz/1000;
                   {delta14C pool corrected; d14C[year] taken from a table for all yearBP (relative to 1950) unless otherwise decided}
                   DataInit.RothC.DPMXpool[i1]:=DataInit.RothC.DPMpool[i1]*d14C[yearBP];
                   DataInit.RothC.RPMXpool[i1]:=DataInit.RothC.RPMpool[i1]*d14C[yearBP];
                   DataInit.RothC.BIOXpool[i1]:=DataInit.RothC.BIOpool[i1]*d14C[yearBP];
                   DataInit.RothC.HUMXpool[i1]:=DataInit.RothC.HUMpool[i1]*d14C[yearBP];
                   DataInit.RothC.IOMXpool[i1]:=DataInit.RothC.IOMpool[i1]*d14C[yearBP];
                   for j:=1 to MaxChem do DataInit.CHEMDPMpool[j][i1]:=0;
                   for j:=1 to MaxChem do DataInit.CHEMRPMpool[j][i1]:=0;
                   for j:=1 to MaxChem do DataInit.CHEMBIOpool[j][i1]:=0;
                   for j:=1 to MaxChem do DataInit.CHEMHUMpool[j][i1]:=0;
                   for j:=1 to MaxChem do DataInit.CHEMIOMpool[j][i1]:=0;
                   for j:=1 to MaxChem do DataInit.CHEMCO2pool[j][i1]:=0;
                  end;
                 {for DataYearIn do}
                 begin
                  DataYearIn.CLAY[i1]:=dCLAY; DataYearIn.SILT[i1]:=dSILT; DataYearIn.OC[i1]:=max(1e-5,dOC1);
                  dSAND:=100-dCLAY-dSILT-1.7*dOC1; DataYearIn.SAND[i1]:=(100-DataYearIn.CLAY[i1]-DataYearIn.SILT[i1]-1.7*DataYearIn.OC[i1]);
                  DataYearIn.GRAVEL[i1]:=dGRAVEL; DataYearIn.STONES[i1]:=dSTONES;
                  DataYearIn.RHO[i1]:=dRHO;   DataYearIn.AW[i1]:=dAW;     DataYearIn.TEM[i1]:=DataInit.AVETEM[1];
                  DataYearIn.CLAYm[i1]:=((100-DataYearIn.GRAVEL[i1]-DataYearIn.STONES[i1])/100)*DataYearIn.CLAY[i1]*DataYearIn.DLZ*DataYearIn.RHO[i1]/100;
                  DataYearIn.SILTm[i1]:=((100-DataYearIn.GRAVEL[i1]-DataYearIn.STONES[i1])/100)*DataYearIn.SILT[i1]*DataYearIn.DLZ*DataYearIn.RHO[i1]/100;
                  DataYearIn.SANDm[i1]:=((100-DataYearIn.GRAVEL[i1]-DataYearIn.STONES[i1])/100)*DataYearIn.SAND[i1]*DataYearIn.DLZ*DataYearIn.RHO[i1]/100;
                  DataYearIn.fineSANDm[i1]:=DataYearIn.SANDm[i1]*0.4; {assuming 40% is <256 microns}
                  DataYearIn.OCm[i1]:=((100-DataYearIn.GRAVEL[i1]-DataYearIn.STONES[i1])/100)*DataYearIn.OC[i1]*DataYearIn.DLZ*DataYearIn.RHO[i1]/100;
                  DataYearIn.ATEM[i1]:=DataYearIn.AVETEM[1]; DataYearIn.Freezecounter[i1]:=0; DataYearIn.Rootcounter[i1]:=0;
                  DataYearIn.CALCITEA[i1]:=dCALCITEA; DataYearIn.oldCALCITEA[i1]:=dCALCITEA;
                  DataYearIn.GYPA[i1]:=dAGYPS;     DataYearIn.oldGYPA[i1]:=dAGYPS;  DataYearIn.APCO2[i1]:=dAPCO2;
                  DataYearIn.SEA[1][i1]:=dSEA1;    DataYearIn.SEA[2][i1]:=dSEA2;    DataYearIn.SEA[3][i1]:=dSEA3;
                  DataYearIn.SEA[4][i1]:=dSEA4;    DataYearIn.SEA[5][i1]:=dSEA5;    DataYearIn.SEA[6][i1]:=dSEA6;
                  DataYearIn.SEA[7][i1]:=dSEA7;    DataYearIn.SEA[12][i1]:=dSEA12;  DataYearIn.SEA[13][i1]:=dSEA13;
                  DataYearIn.SEA[14][i1]:=dSEA14;  DataYearIn.SEA[15][i1]:=dSEA15;  DataYearIn.SEA[16][i1]:=dSEA16;
                  DataYearIn.AXSE[1][i1]:=dAXSE1;  DataYearIn.AXSE[2][i1]:=dAXSE2;  DataYearIn.AXSE[3][i1]:=dAXSE3;
                  DataYearIn.AXSE[4][i1]:=dAXSE4;  DataYearIn.AXSE[10][i1]:=dAXSE10;DataYearIn.AXSE[12][i1]:=dAXSE12;
                  DataYearIn.ACEC[i1]:=dCEC;       DataYearIn.SC1[i1]:=dSC1;        DataYearIn.SC2[i1]:=dSC2;
                  DataYearIn.SC3[i1]:=dSC3;        DataYearIn.SC7[i1]:=dSC7;        DataYearIn.SC8[i1]:=dSC8;
                  {2-domain CEC  and AXSE}
                   {1} DataYearIn.CEC_empirical[i1]:=32+36.7*DataYearIn.OC[i1]+1.96*DataYearIn.clay[i1];
                   {2} DataYearIn.FacCEC[i1]:=DataYearIn.ACEC[i1]/DataYearIn.CEC_empirical[i1];
                   {3} DataYearIn.FacCEC_OC[i1]:=36.7*DataYearIn.FacCEC[i1];  {FacCEC_OC contains adjusted regression coefficient}
                   {4+7} DataYearIn.ACEC_clay[i1]:=DataYearIn.ACEC[i1]-DataYearIn.FacCEC_OC[i1]*DataYearIn.OC[i1];
                         DataYearIn.ACEC_OC[i1]:=DataYearIn.ACEC[i1]-DataYearIn.ACEC_clay[i1];
                   {5} DataYearIn.ClayFrCEC[i1]:=DataYearIn.ACEC_clay[i1]/DataYearIn.ACEC[i1];
                   {6} DataYearIn.OCFrCEC[i1]:=DataYearIn.ACEC_OC[i1]/DataYearIn.ACEC[i1];
                   {7} for j1:=1 to MaxChem do
                   begin
                     DataYearIn.AXSE_oc[j1,i1]:=DataYearIn.AXSE[j1,i1]*DataYearIn.OCFrCEC[i1];
                     {_oc values will be adapted each year when the OC is adapted in RECALCOC;
                      _clay values will be adapted in the CHEM.claymigration procedures and
                                                   in the CHEM.physicalweathering procedure}
                     DataYearIn.AXSE_clay[j1,i1]:=DataYearIn.AXSE[j1,i1]*DataYearIn.ClayFrCEC[i1];
                   end;
                  {add: weathering data and rothc-data. Assuming in new layers:
                  a. same mineral composition as original parent material;
                  b. empty C-pools}
                  {a1. mass fractions}
                  DataYearIn.WE.Gibbsitepool[i1]:=dfrGibbsite;
                  DataYearIn.WE.MinPools[1][i1]:= dfrAlbite;
                  DataYearIn.WE.MinPools[2][i1]:= dfrKfeldspar;
                  DataYearIn.WE.MinPools[3][i1]:= dfrMuscovite;
                  DataYearIn.WE.MinPools[4][i1]:= dfrBiotite;
                  DataYearIn.WE.MinPools[5][i1]:= dfrQuartz;
                  DataYearIn.WE.MinPools[6][i1]:= dfrChlorite;
                  DataYearIn.WE.MinPools[7][i1]:= dfrAnorthite;
                  DataYearIn.WE.MinPools[8][i1]:= dfrFosterite;
                  DataYearIn.WE.MinPools[9][i1]:= dfrAugite;
                  DataYearIn.WE.MinPools[10][i1]:=dfrKaolinite;
                  DataYearIn.WE.MinPools[11][i1]:=dfrIllite;
                  DataYearIn.WE.MinPools[12][i1]:=dfrMontmorillonite;
                  DataYearIn.WE.MinPools[13][i1]:=dfrHornblende;
                  DataYearIn.WE.MinPools[14][i1]:=dfrFayalite;
                  DataYearIn.WE.MinPools[15][i1]:=dfrAmorphite;
                  DataYearIn.WE.MinPools[16][i1]:=dfrOtherite;
                  {a2. convert to mass}
                  totalmass:=dRHO*DataInit.DLZ*(100-dSTONES-dGRAVEL)/100;          {kg dry matter over DLZ mm compartment thickness and 1 m2 area, excluding stones+gravel}
                  totalmass:=totalmass-dCALCITEA-dAGYPS-1.7*DataYearIn.OCm[i1];    {but we subtract mass of gypsum, calcite and OC, all assumed part of fine earth}
                  totalmass:=totalmass+((dSTONES+dGRAVEL)/100)*DataInit.DLZ *2.65; {..raised by the mass in stones and gravel, with specific weight assumed to be 2.65 kg dm-3}
                  {a3. now totalmass is mass associated with the 14 minerals + gibbsite}
                  DataYearIn.WE.Gibbsitepool[i1]:=DataYearIn.WE.Gibbsitepool[i1]*totalmass;
                  for j:=1 to nrminerals do
                    DataYearIn.WE.MinPools[j][i1]:=DataYearIn.WE.MinPools[j][i1]*totalmass;
                  {a4. convert the mineral pools (kg) to elemental pools in minerals (mol)}
                  {first convert kg mineral to mol mineral by using the
                  Relative Formula Mass, RFM, per mineral from elemental weights and stoichiometric coeficients.
                  These are calculated constants in untProcesses
                  "1000" is used to convert kmol>mol}
                   tmpMinPool[1]:=1000*DataYearIn.WE.MinPools[1][i1]/untProcesses.RFMalbite;
                   tmpMinPool[2]:=1000*DataYearIn.WE.MinPools[2][i1]/untProcesses.RFMkfeldspar;
                   tmpMinPool[3]:=1000*DataYearIn.WE.MinPools[3][i1]/untProcesses.RFMmuscovite;
                   tmpMinPool[4]:=1000*DataYearIn.WE.MinPools[4][i1]/untProcesses.RFMbiotite;
                   tmpMinPool[5]:=1000*DataYearIn.WE.MinPools[5][i1]/untProcesses.RFMquartz;
                   tmpMinPool[6]:=1000*DataYearIn.WE.MinPools[6][i1]/untProcesses.RFMchlorite;
                   tmpMinPool[7]:=1000*DataYearIn.WE.MinPools[7][i1]/untProcesses.RFManorthite;
                   tmpMinPool[8]:=1000*DataYearIn.WE.MinPools[8][i1]/untProcesses.RFMForsterite;
                   tmpMinPool[9]:=1000*DataYearIn.WE.MinPools[9][i1]/untProcesses.RFMaugite;
                   tmpMinPool[10]:=1000*DataYearIn.WE.MinPools[10][i1]/untProcesses.RFMkaolinite;
                   tmpMinPool[11]:=1000*DataYearIn.WE.MinPools[11][i1]/untProcesses.RFMillite;
                   tmpMinPool[12]:=1000*DataYearIn.WE.MinPools[12][i1]/untProcesses.RFMmontmorillonite;
                   tmpMinPool[13]:=1000*DataYearIn.WE.MinPools[13][i1]/untProcesses.RFMhornblende;
                   tmpMinPool[14]:=1000*DataYearIn.WE.MinPools[14][i1]/untProcesses.RFMfayalite;
                   if RFMamorphite>0
                      then tmpMinPool[15]:=1000*DataYearIn.WE.MinPools[15][i1]/untProcesses.RFMamorphite
                      else tmpMinPool[15]:=0;
                   if RFMotherite>0
                      then tmpMinPool[16]:=1000*DataYearIn.WE.MinPools[16][i1]/untProcesses.RFMotherite
                      else tmpMinPool[16]:=0;
                   {tmpGibbsite:=1000*DataYearIn.WE.Gibbsitepool[i1]/untProcesses.RFMgibbsite; }
                   {tmpGibbsite was never used. Line above this one therefore commented.
                    Line below this line converts initial Gibbsite Al(OH)3 (kmol) to mol Al in Gibbsite}
                   DataYearIn.WE.Gibbsitepool[i1]:=1000*DataYearIn.WE.Gibbsitepool[i1]/untProcesses.RFMgibbsite;
                   {a5. then use the stoichiometric composition of each mineral to obtain mmoles of element:
                   {1 albite Na Al Si3 O8}
                   {2 K-feldspar K Al Si3 O8}
                   {3 muscovite K Al3 Si3 O10 (OH)2}
                   {4 biotite K Mg1.5 Fe1.5 Al Si3 O10 (OH)2}
                   {5 Quartz Si O2}
                   {6 Clorite Clinochlore Mg5 Al2 Si3 O10 (OH)8 }
                   {7 anorthite Ca Al2 Si2 O8 }
                   {8 Forsterite Mg2 Si O4}
                   {9 augite orthoferrosilite Fe Si O3 }
                   {10 kaolinite Al2 Si2 O5 (OH)4}
                   {11 Illite Si3.4 O10 (OH)2 Mg0.02 Al2.38 Ca0.01 K0.44, Godderis et al 2006}
                   {12 Montmorillonite Si4 O10 (OH)2 Mg0.33 Al1.67 Ca0.04125 Mg0.04125 K0.0825 Na0.0825 }
                   {13 hornblende Ca2 Mg4 Al Si6.7 Al O22.4 from Profile model }
                   {14 fayalite Fe2 Si O4}
                   {15 amorphite}
                   {16 otherite}
                   DataYearIn.WE.WE_Capool[i1]:=0*tmpMinPool[1]+0*tmpMinPool[2]+0*tmpMinPool[3]+
                             0*tmpMinPool[4]+0*tmpMinPool[5]+0*tmpMinPool[6]+
                             1*tmpMinPool[7]+0*tmpMinPool[8]+0*tmpMinPool[9]+
                             0*tmpMinPool[10]+0.01*tmpMinPool[11]+0.04125*tmpMinPool[12]+
                             2*tmpMinPool[13]+0*tmpMinPool[14]+amorp_Ca*tmpMinPool[15]+
                             other_Ca*tmpMinPool[16];
                   DataYearIn.WE.WE_Mgpool[i1]:=0*tmpMinPool[1]+0*tmpMinPool[2]+0*tmpMinPool[3]+
                             1.5*tmpMinPool[4]+0*tmpMinPool[5]+5*tmpMinPool[6]+
                             0*tmpMinPool[7]+2*tmpMinPool[8]+0*tmpMinPool[9]+
                             0*tmpMinPool[10]+0.02*tmpMinPool[11]+(0.33+0.04125)*tmpMinPool[12]+
                             4*tmpMinPool[13]+0*tmpMinPool[14]+amorp_Mg*tmpMinPool[15]+
                             other_Mg*tmpMinPool[16];
                   DataYearIn.WE.WE_Napool[i1]:=1*tmpMinPool[1]+0*tmpMinPool[2]+0*tmpMinPool[3]+
                             0*tmpMinPool[4]+0*tmpMinPool[5]+0*tmpMinPool[6]+
                             0*tmpMinPool[7]+0*tmpMinPool[8]+0*tmpMinPool[9]+
                             0*tmpMinPool[10]+0*tmpMinPool[11]+0.0825*tmpMinPool[12]+
                             0*tmpMinPool[13]+0*tmpMinPool[14];
                   DataYearIn.WE.WE_Kpool[i1]:=0*tmpMinPool[1]+1*tmpMinPool[2]+1*tmpMinPool[3]+
                             1*tmpMinPool[4]+0*tmpMinPool[5]+0*tmpMinPool[6]+
                             0*tmpMinPool[7]+0*tmpMinPool[8]+0*tmpMinPool[9]+
                             0*tmpMinPool[10]+0.44*tmpMinPool[11]+0.0825*tmpMinPool[12]+
                             0*tmpMinPool[13]+0*tmpMinPool[14]+amorp_K*tmpMinPool[15]+
                             other_K*tmpMinPool[16];
                   DataYearIn.WE.WE_Sipool[i1]:=3*tmpMinPool[1]+3*tmpMinPool[2]+3*tmpMinPool[3]+
                             3*tmpMinPool[4]+1*tmpMinPool[5]+3*tmpMinPool[6]+
                             2*tmpMinPool[7]+1*tmpMinPool[8]+1*tmpMinPool[9]+
                             2*tmpMinPool[10]+3.4*tmpMinPool[11]+4*tmpMinPool[12]+
                             6.7*tmpMinPool[13]+1*tmpMinPool[14];
                   DataYearIn.WE.WE_Fepool[i1]:=0*tmpMinPool[1]+0*tmpMinPool[2]+0*tmpMinPool[3]+
                             1.5*tmpMinPool[4]+0*tmpMinPool[5]+0*tmpMinPool[6]+
                             0*tmpMinPool[7]+0*tmpMinPool[8]+1*tmpMinPool[9]+
                             0*tmpMinPool[10]+0*tmpMinPool[11]+0*tmpMinPool[12]+
                             0*tmpMinPool[13]+2*tmpMinPool[14]+amorp_Fe*tmpMinPool[15]+
                             other_Fe*tmpMinPool[16];
                   DataYearIn.WE.WE_Alpool[i1]:=1*tmpMinPool[1]+1*tmpMinPool[2]+3*tmpMinPool[3]+
                             1*tmpMinPool[4]+0*tmpMinPool[5]+2*tmpMinPool[6]+
                             2*tmpMinPool[7]+0*tmpMinPool[8]+0*tmpMinPool[9]+
                             2*tmpMinPool[10]+2.38*tmpMinPool[11]+1.67*tmpMinPool[12]+
                             1*tmpMinPool[13]+0*tmpMinPool[14]+amorp_Al*tmpMinPool[15]+
                             other_Al*tmpMinPool[16];
                  {b. Organic matter}
                  DataYearIn.RothC.ClayPerc[i1]:=dCLAY*(dCLAY+dSILT+dSAND+1.7*dOC1)/(dCLAY+dSILT+dSAND);
                  DataYearIn.RothC.DPMpool[i1]:=0;  DataYearIn.RothC.RPMpool[i1]:=0;
                  DataYearIn.RothC.BIOpool[i1]:=0;  DataYearIn.RothC.HUMpool[i1]:=0;
                  DataYearIn.RothC.IOMpool[i1]:=0;  DataYearIn.RothC.CO2pool[i1]:=0;
                  DataYearIn.RothC.accTSMD[i1]:=0;  DataYearIn.RothC.pCO2[i1]:=pCO2atm;
                  {now distribute the incoming OC over the RothCpools in the same
                   way as the initial C (see ReadDataC-procedure)}
                  DataYearIn.RothC.DPMpool[i1]:=DataYearIn.RothCGEN.GENPoolPercs[1]
                                              *DataYearIn.OC[i1]*DataYearIn.RHO[i1]*DataYearIn.dlz/1000;
                  DataYearIn.RothC.RPMpool[i1]:=DataYearIn.RothCGEN.GENPoolPercs[2]
                                              *DataYearIn.OC[i1]*DataYearIn.RHO[i1]*DataYearIn.dlz/1000;
                  DataYearIn.RothC.BIOpool[i1]:=DataYearIn.RothCGEN.GENPoolPercs[3]
                                              *DataYearIn.OC[i1]*DataYearIn.RHO[i1]*DataYearIn.dlz/1000;
                  DataYearIn.RothC.HUMpool[i1]:=DataYearIn.RothCGEN.GENPoolPercs[4]
                                              *DataYearIn.OC[i1]*DataYearIn.RHO[i1]*DataYearIn.dlz/1000;
                  DataYearIn.RothC.IOMpool[i1]:=DataYearIn.RothCGEN.GENPoolPercs[5]
                                              *DataYearIn.OC[i1]*DataYearIn.RHO[i1]*DataYearIn.dlz/1000;
                  {delta14C pool corrected; d14C[year] taken from a table for all yearBP (relative to 1950) unless otherwise decided}
                  DataYearIn.RothC.DPMXpool[i1]:=DataYearIn.RothC.DPMpool[i1]*d14C[yearBP];
                  DataYearIn.RothC.RPMXpool[i1]:=DataYearIn.RothC.RPMpool[i1]*d14C[yearBP];
                  DataYearIn.RothC.BIOXpool[i1]:=DataYearIn.RothC.BIOpool[i1]*d14C[yearBP];
                  DataYearIn.RothC.HUMXpool[i1]:=DataYearIn.RothC.HUMpool[i1]*d14C[yearBP];
                  DataYearIn.RothC.IOMXpool[i1]:=DataYearIn.RothC.IOMpool[i1]*d14C[yearBP];
                  for j:=1 to MaxChem do DataYearIn.CHEMDPMpool[j][i1]:=0;
                  for j:=1 to MaxChem do DataYearIn.CHEMRPMpool[j][i1]:=0;
                  for j:=1 to MaxChem do DataYearIn.CHEMBIOpool[j][i1]:=0;
                  for j:=1 to MaxChem do DataYearIn.CHEMHUMpool[j][i1]:=0;
                  for j:=1 to MaxChem do DataYearIn.CHEMIOMpool[j][i1]:=0;
                  for j:=1 to MaxChem do DataYearIn.CHEMCO2pool[j][i1]:=0;
                 end;
                 {for DataYearOut do}
                 begin
                  DataYearOut.CLAY[i1]:=dCLAY; DataYearOut.SILT[i1]:=dSILT; DataYearOut.OC[i1]:=max(1e-5,dOC1);
                  dSAND:=100-dCLAY-dSILT-1.7*dOC1; DataYearOut.SAND[i1]:=(100-DataYearOut.CLAY[i1]-DataYearOut.SILT[i1]-1.7*DataYearOut.OC[i1]);
                  DataYearOut.GRAVEL[i1]:=dGRAVEL; DataYearOut.STONES[i1]:=dSTONES;
                  DataYearOut.RHO[i1]:=dRHO;   DataYearOut.AW[i1]:=dAW;     DataYearOut.TEM[i1]:=DataInit.AVETEM[1];
                  DataYearOut.CLAYm[i1]:=((100-DataYearOut.GRAVEL[i1]-DataYearOut.STONES[i1])/100)*DataYearOut.CLAY[i1]*DataYearOut.DLZ*DataYearOut.RHO[i1]/100;
                  DataYearOut.SILTm[i1]:=((100-DataYearOut.GRAVEL[i1]-DataYearOut.STONES[i1])/100)*DataYearOut.SILT[i1]*DataYearOut.DLZ*DataYearOut.RHO[i1]/100;
                  DataYearOut.SANDm[i1]:=((100-DataYearOut.GRAVEL[i1]-DataYearOut.STONES[i1])/100)*DataYearOut.SAND[i1]*DataYearIn.DLZ*DataYearOut.RHO[i1]/100;
                  DataYearOut.fineSANDm[i1]:=DataYearOut.SANDm[i1]*0.4; {assuming 40% is <256 microns}
                  DataYearOut.OCm[i1]:=((100-DataYearOut.GRAVEL[i1]-DataYearOut.STONES[i1])/100)*DataYearOut.OC[i1]*DataYearOut.DLZ*DataYearOut.RHO[i1]/100;
                  DataYearOut.ATEM[i1]:=DataInit.AVETEM[1]; DataYearOut.Freezecounter[i1]:=0; DataYearOut.Rootcounter[i1]:=0;
                  DataYearOut.CALCITEA[i1]:=dCALCITEA; DataYearOut.oldCALCITEA[i1]:=dCALCITEA;
                  DataYearOut.GYPA[i1]:=dAGYPS;     DataYearOut.oldGYPA[i1]:=dAGYPS;  DataYearOut.APCO2[i1]:=dAPCO2;
                  DataYearOut.SEA[1][i1]:=dSEA1;    DataYearOut.SEA[2][i1]:=dSEA2;    DataYearOut.SEA[3][i1]:=dSEA3;
                  DataYearOut.SEA[4][i1]:=dSEA4;    DataYearOut.SEA[5][i1]:=dSEA5;    DataYearOut.SEA[6][i1]:=dSEA6;
                  DataYearOut.SEA[7][i1]:=dSEA7;    DataYearOut.SEA[12][i1]:=dSEA12;  DataYearOut.SEA[13][i1]:=dSEA13;
                  DataYearOut.SEA[14][i1]:=dSEA14;  DataYearOut.SEA[15][i1]:=dSEA15;  DataYearOut.SEA[16][i1]:=dSEA16;
                  DataYearOut.AXSE[1][i1]:=dAXSE1;  DataYearOut.AXSE[2][i1]:=dAXSE2;  DataYearOut.AXSE[3][i1]:=dAXSE3;
                  DataYearOut.AXSE[4][i1]:=dAXSE4;  DataYearOut.AXSE[10][i1]:=dAXSE10;DataYearOut.AXSE[12][i1]:=dAXSE12;
                  DataYearOut.ACEC[i1]:=dCEC;       DataYearOut.SC1[i1]:=dSC1;        DataYearOut.SC2[i1]:=dSC2;
                  DataYearOut.SC3[i1]:=dSC3;        DataYearOut.SC7[i1]:=dSC7;        DataYearOut.SC8[i1]:=dSC8;
                  {2-domain CEC  and AXSE}
                   {1} DataYearOut.CEC_empirical[i1]:=32+36.7*DataYearOut.OC[i1]+1.96*DataYearOut.clay[i1];
                   {2} DataYearOut.FacCEC[i1]:=DataYearOut.ACEC[i1]/DataYearOut.CEC_empirical[i1];
                   {3} DataYearOut.FacCEC_OC[i1]:=36.7*DataYearOut.FacCEC[i1];  {FacCEC_OC contains adjusted regression coefficient}
                   {4+7} DataYearOut.ACEC_clay[i1]:=DataYearOut.ACEC[i1]-DataYearOut.FacCEC_OC[i1]*DataYearOut.OC[i1];
                         DataYearOut.ACEC_OC[i1]:=DataYearOut.ACEC[i1]-DataYearOut.ACEC_clay[i1];
                   {5} DataYearOut.ClayFrCEC[i1]:=DataYearOut.ACEC_clay[i1]/DataYearOut.ACEC[i1];
                   {6} DataYearOut.OCFrCEC[i1]:=DataYearOut.ACEC_OC[i1]/DataYearOut.ACEC[i1];
                   {7} for j1:=1 to MaxChem do
                   begin
                     DataYearOut.AXSE_oc[j1,i1]:=DataYearOut.AXSE[j1,i1]*DataYearOut.OCFrCEC[i1];
                     {_oc values will be adapted each year when the OC is adapted in RECALCOC;
                      _clay values will be adapted in the CHEM.claymigration procedures and
                                                   in the CHEM.physicalweathering procedure}
                     DataYearOut.AXSE_clay[j1,i1]:=DataYearOut.AXSE[j1,i1]*DataYearOut.ClayFrCEC[i1];
                   end;
                  {add: weathering data and rothc-data. Assuming in new layers:
                  a. same mineral composition as original parent material;
                  b. empty C-pools}
                  {a1. mass fractions}
                  DataYearOut.WE.Gibbsitepool[i1]:=dfrGibbsite;
                  DataYearOut.WE.MinPools[1][i1]:= dfrAlbite;
                  DataYearOut.WE.MinPools[2][i1]:= dfrKfeldspar;
                  DataYearOut.WE.MinPools[3][i1]:= dfrMuscovite;
                  DataYearOut.WE.MinPools[4][i1]:= dfrBiotite;
                  DataYearOut.WE.MinPools[5][i1]:= dfrQuartz;
                  DataYearOut.WE.MinPools[6][i1]:= dfrChlorite;
                  DataYearOut.WE.MinPools[7][i1]:= dfrAnorthite;
                  DataYearOut.WE.MinPools[8][i1]:= dfrFosterite;
                  DataYearOut.WE.MinPools[9][i1]:= dfrAugite;
                  DataYearOut.WE.MinPools[10][i1]:=dfrKaolinite;
                  DataYearOut.WE.MinPools[11][i1]:=dfrIllite;
                  DataYearOut.WE.MinPools[12][i1]:=dfrMontmorillonite;
                  DataYearOut.WE.MinPools[13][i1]:=dfrHornblende;
                  DataYearOut.WE.MinPools[14][i1]:=dfrFayalite;
                  DataYearOut.WE.MinPools[15][i1]:=dfrAmorphite;
                  DataYearOut.WE.MinPools[16][i1]:=dfrOtherite;
                  {a2. convert to mass}
                  totalmass:=dRHO*DataInit.DLZ*(100-dSTONES-dGRAVEL)/100;          {kg dry matter over DLZ mm compartment thickness and 1 m2 area, excluding stones+gravel}
                  totalmass:=totalmass-dCALCITEA-dAGYPS-1.7*DataYearOut.OCm[i1];    {but we subtract mass of gypsum, calcite and OC, all assumed part of fine earth}
                  totalmass:=totalmass+((dSTONES+dGRAVEL)/100)*DataInit.DLZ *2.65; {..raised by the mass in stones and gravel, with specific weight assumed to be 2.65 kg dm-3}
                  {a3. now totalmass is mass associated with the 14 minerals + gibbsite}
                  DataYearOut.WE.Gibbsitepool[i1]:=DataYearOut.WE.Gibbsitepool[i1]*totalmass;
                  for j:=1 to nrminerals do
                    DataYearOut.WE.MinPools[j][i1]:=DataYearOut.WE.MinPools[j][i1]*totalmass;
                  {a4. convert the mineral pools (kg) to elemental pools in minerals (mol)}
                  {first convert kg mineral to mol mineral by using the
                  Relative Formula Mass, RFM, per mineral from elemental weights and stoichiometric coeficients.
                  These are calculated constants in untProcesses
                  "1000" is used to convert kmol>mol}
                   tmpMinPool[1]:=1000*DataYearOut.WE.MinPools[1][i1]/untProcesses.RFMalbite;
                   tmpMinPool[2]:=1000*DataYearOut.WE.MinPools[2][i1]/untProcesses.RFMkfeldspar;
                   tmpMinPool[3]:=1000*DataYearOut.WE.MinPools[3][i1]/untProcesses.RFMmuscovite;
                   tmpMinPool[4]:=1000*DataYearOut.WE.MinPools[4][i1]/untProcesses.RFMbiotite;
                   tmpMinPool[5]:=1000*DataYearOut.WE.MinPools[5][i1]/untProcesses.RFMquartz;
                   tmpMinPool[6]:=1000*DataYearOut.WE.MinPools[6][i1]/untProcesses.RFMchlorite;
                   tmpMinPool[7]:=1000*DataYearOut.WE.MinPools[7][i1]/untProcesses.RFManorthite;
                   tmpMinPool[8]:=1000*DataYearOut.WE.MinPools[8][i1]/untProcesses.RFMForsterite;
                   tmpMinPool[9]:=1000*DataYearOut.WE.MinPools[9][i1]/untProcesses.RFMaugite;
                   tmpMinPool[10]:=1000*DataYearOut.WE.MinPools[10][i1]/untProcesses.RFMkaolinite;
                   tmpMinPool[11]:=1000*DataYearOut.WE.MinPools[11][i1]/untProcesses.RFMillite;
                   tmpMinPool[12]:=1000*DataYearOut.WE.MinPools[12][i1]/untProcesses.RFMmontmorillonite;
                   tmpMinPool[13]:=1000*DataYearOut.WE.MinPools[13][i1]/untProcesses.RFMhornblende;
                   tmpMinPool[14]:=1000*DataYearOut.WE.MinPools[14][i1]/untProcesses.RFMfayalite;
                   if RFMamorphite>0
                     then tmpMinPool[15]:=1000*DataYearOut.WE.MinPools[15][i1]/untProcesses.RFMamorphite
                     else tmpMinPool[15]:=0;
                   if RFMotherite>0
                     then tmpMinPool[16]:=1000*DataYearOut.WE.MinPools[16][i1]/untProcesses.RFMotherite
                     else tmpMinPool[16]:=0;
                   {tmpGibbsite:=1000*DataYearOut.WE.Gibbsitepool[i1]/untProcesses.RFMgibbsite; }
                   {tmpGibbsite was never used. Line above this one therefore commented.
                    Line below this line converts initial Gibbsite Al(OH)3 (kmol) to mol Al in Gibbsite}
                   DataYearOut.WE.Gibbsitepool[i1]:=1000*DataYearOut.WE.Gibbsitepool[i1]/untProcesses.RFMgibbsite;
                   {a5. then use the stoichiometric composition of each mineral to obtain mmoles of element:
                   {1 albite Na Al Si3 O8}
                   {2 K-feldspar K Al Si3 O8}
                   {3 muscovite K Al3 Si3 O10 (OH)2}
                   {4 biotite K Mg1.5 Fe1.5 Al Si3 O10 (OH)2}
                   {5 Quartz Si O2}
                   {6 Clorite Clinochlore Mg5 Al2 Si3 O10 (OH)8 }
                   {7 anorthite Ca Al2 Si2 O8 }
                   {8 Forsterite Mg2 Si O4}
                   {9 augite orthoferrosilite Fe Si O3 }
                   {10 kaolinite Al2 Si2 O5 (OH)4}
                   {11 Illite Si3.4 O10 (OH)2 Mg0.02 Al2.38 Ca0.01 K0.44, Godderis et al 2006}
                   {12 Montmorillonite Si4 O10 (OH)2 Mg0.33 Al1.67 Ca0.04125 Mg0.04125 K0.0825 Na0.0825 }
                   {13 hornblende Ca2 Mg4 Al Si6.7 Al O22.4 from Profile model }
                   {14 fayalite Fe2 Si O4}
                   {15 amorphite}
                   {16 otherite}
                   DataYearOut.WE.WE_Capool[i1]:=0*tmpMinPool[1]+0*tmpMinPool[2]+0*tmpMinPool[3]+
                             0*tmpMinPool[4]+0*tmpMinPool[5]+0*tmpMinPool[6]+
                             1*tmpMinPool[7]+0*tmpMinPool[8]+0*tmpMinPool[9]+
                             0*tmpMinPool[10]+0.01*tmpMinPool[11]+0.04125*tmpMinPool[12]+
                             2*tmpMinPool[13]+0*tmpMinPool[14]+amorp_Ca*tmpMinPool[15]+
                             other_Ca*tmpMinPool[16];
                   DataYearOut.WE.WE_Mgpool[i1]:=0*tmpMinPool[1]+0*tmpMinPool[2]+0*tmpMinPool[3]+
                             1.5*tmpMinPool[4]+0*tmpMinPool[5]+5*tmpMinPool[6]+
                             0*tmpMinPool[7]+2*tmpMinPool[8]+0*tmpMinPool[9]+
                             0*tmpMinPool[10]+0.02*tmpMinPool[11]+(0.33+0.04125)*tmpMinPool[12]+
                             4*tmpMinPool[13]+0*tmpMinPool[14]+amorp_Mg*tmpMinPool[15]+
                             other_Mg*tmpMinPool[16];
                   DataYearOut.WE.WE_Napool[i1]:=1*tmpMinPool[1]+0*tmpMinPool[2]+0*tmpMinPool[3]+
                             0*tmpMinPool[4]+0*tmpMinPool[5]+0*tmpMinPool[6]+
                             0*tmpMinPool[7]+0*tmpMinPool[8]+0*tmpMinPool[9]+
                             0*tmpMinPool[10]+0*tmpMinPool[11]+0.0825*tmpMinPool[12]+
                             0*tmpMinPool[13]+0*tmpMinPool[14];
                   DataYearOut.WE.WE_Kpool[i1]:=0*tmpMinPool[1]+1*tmpMinPool[2]+1*tmpMinPool[3]+
                             1*tmpMinPool[4]+0*tmpMinPool[5]+0*tmpMinPool[6]+
                             0*tmpMinPool[7]+0*tmpMinPool[8]+0*tmpMinPool[9]+
                             0*tmpMinPool[10]+0.44*tmpMinPool[11]+0.0825*tmpMinPool[12]+
                             0*tmpMinPool[13]+0*tmpMinPool[14]+amorp_K*tmpMinPool[15]+
                             other_K*tmpMinPool[16];
                   DataYearOut.WE.WE_Sipool[i1]:=3*tmpMinPool[1]+3*tmpMinPool[2]+3*tmpMinPool[3]+
                             3*tmpMinPool[4]+1*tmpMinPool[5]+3*tmpMinPool[6]+
                             2*tmpMinPool[7]+1*tmpMinPool[8]+1*tmpMinPool[9]+
                             2*tmpMinPool[10]+3.4*tmpMinPool[11]+4*tmpMinPool[12]+
                             6.7*tmpMinPool[13]+1*tmpMinPool[14];
                   DataYearOut.WE.WE_Fepool[i1]:=0*tmpMinPool[1]+0*tmpMinPool[2]+0*tmpMinPool[3]+
                             1.5*tmpMinPool[4]+0*tmpMinPool[5]+0*tmpMinPool[6]+
                             0*tmpMinPool[7]+0*tmpMinPool[8]+1*tmpMinPool[9]+
                             0*tmpMinPool[10]+0*tmpMinPool[11]+0*tmpMinPool[12]+
                             0*tmpMinPool[13]+2*tmpMinPool[14]+amorp_Fe*tmpMinPool[15]+
                             other_Fe*tmpMinPool[16];
                   DataYearOut.WE.WE_Alpool[i1]:=1*tmpMinPool[1]+1*tmpMinPool[2]+3*tmpMinPool[3]+
                             1*tmpMinPool[4]+0*tmpMinPool[5]+2*tmpMinPool[6]+
                             2*tmpMinPool[7]+0*tmpMinPool[8]+0*tmpMinPool[9]+
                             2*tmpMinPool[10]+2.38*tmpMinPool[11]+1.67*tmpMinPool[12]+
                             1*tmpMinPool[13]+0*tmpMinPool[14]+amorp_Al*tmpMinPool[15]+
                             other_Al*tmpMinPool[16];
                  {b. Organic matter}
                  DataYearOut.RothC.ClayPerc[i1]:=dCLAY*(dCLAY+dSILT+dSAND+1.7*dOC1)/(dCLAY+dSILT+dSAND);
                  DataYearOut.RothC.DPMpool[i1]:=0;  DataYearOut.RothC.RPMpool[i1]:=0;
                  DataYearOut.RothC.BIOpool[i1]:=0;  DataYearOut.RothC.HUMpool[i1]:=0;
                  DataYearOut.RothC.IOMpool[i1]:=0;  DataYearOut.RothC.CO2pool[i1]:=0;
                  DataYearOut.RothC.accTSMD[i1]:=0;  DataYearOut.RothC.pCO2[i1]:=pCO2atm;
                  {now distribute the incoming OC over the RothCpools in the same
                   way as the initial C (see ReadDataC-procedure)}
                  DataYearOut.RothC.DPMpool[i1]:=DataYearOut.RothCGEN.GENPoolPercs[1]
                                              *DataYearOut.OC[i1]*DataYearOut.RHO[i1]*DataYearOut.dlz/1000;
                  DataYearOut.RothC.RPMpool[i1]:=DataYearOut.RothCGEN.GENPoolPercs[2]
                                              *DataYearOut.OC[i1]*DataYearOut.RHO[i1]*DataYearOut.dlz/1000;
                  DataYearOut.RothC.BIOpool[i1]:=DataYearOut.RothCGEN.GENPoolPercs[3]
                                              *DataYearOut.OC[i1]*DataYearOut.RHO[i1]*DataYearOut.dlz/1000;
                  DataYearOut.RothC.HUMpool[i1]:=DataYearOut.RothCGEN.GENPoolPercs[4]
                                              *DataYearOut.OC[i1]*DataYearOut.RHO[i1]*DataYearOut.dlz/1000;
                  DataYearOut.RothC.IOMpool[i1]:=DataYearOut.RothCGEN.GENPoolPercs[5]
                                              *DataYearOut.OC[i1]*DataYearOut.RHO[i1]*DataYearOut.dlz/1000;
                  {delta14C pool corrected; d14C[year] taken from a table for all yearBP (relative to 1950) unless otherwise decided}
                  DataYearOut.RothC.DPMXpool[i1]:=DataYearOut.RothC.DPMpool[i1]*d14C[yearBP];
                  DataYearOut.RothC.RPMXpool[i1]:=DataYearOut.RothC.RPMpool[i1]*d14C[yearBP];
                  DataYearOut.RothC.BIOXpool[i1]:=DataYearOut.RothC.BIOpool[i1]*d14C[yearBP];
                  DataYearOut.RothC.HUMXpool[i1]:=DataYearOut.RothC.HUMpool[i1]*d14C[yearBP];
                  DataYearOut.RothC.IOMXpool[i1]:=DataYearOut.RothC.IOMpool[i1]*d14C[yearBP];
                  for j:=1 to MaxChem do DataYearOut.CHEMDPMpool[j][i1]:=0;
                  for j:=1 to MaxChem do DataYearOut.CHEMRPMpool[j][i1]:=0;
                  for j:=1 to MaxChem do DataYearOut.CHEMBIOpool[j][i1]:=0;
                  for j:=1 to MaxChem do DataYearOut.CHEMHUMpool[j][i1]:=0;
                  for j:=1 to MaxChem do DataYearOut.CHEMIOMpool[j][i1]:=0;
                  for j:=1 to MaxChem do DataYearOut.CHEMCO2pool[j][i1]:=0;
                 end;
                 if physweathallowed then
                 begin
                   {Physical weathering: initialise grain pools in new compartments}
                   {sand - silt - clay % sum up to 100-1.7*OC,
                    these must be expressed as fraction summing up to 1
                    and be evenly distributed over subfractions}
                   for j:=1 to 5 do
                   begin
                     PW_G[j,i1]:=0.2*((100-DataYearIn.GRAVEL[i1]-DataYearIn.STONES[i1])/100)
                                    *(100-DataYearIn.CLAY[i1]-DataYearIn.SILT[i1]-1.7*DataYearIn.OC[i1])
                                    /(100-1.7*DataYearIn.OC[i1]);  {markOC}
                   end;
                   for j:=6 to 10 do
                   begin
                      PW_G[j,i1]:=0.2*((100-DataYearIn.GRAVEL[i1]-DataYearIn.STONES[i1])/100)
                                     *DataYearIn.SILT[i1]/(100-1.7*DataYearIn.OC[i1]);  {markOC}
                   end;
                   PW_G[11,i1]:=((100-DataYearIn.GRAVEL[i1]-DataYearIn.STONES[i1])/100)
                                 *DataYearIn.CLAY[i1]/(100-1.7*DataYearIn.OC[i1]);  {markOC} 
                   for j:=-3 to 0 do {gravel-classes}
                      PW_G[j,i1]:=0.25*DataYearIn.GRAVEL[i1]/100;
                   {stone-class}
                   PW_G[-4,i1]:=DataYearIn.STONES[i1]/100;
                   {init: no grains split}
                   for j:=-4 to 11 do PW_S[j,i1]:=0;
                   {number of particles PW_K[] for each particle size class}
                   dataYearIn.STONESm[i1]:=(dataYearIn.STONES[i1]/100)*(dataYearIn.SANDm[i1]+dataYearIn.SILTm[i1]+dataYearIn.CLAYm[i1]+1.7*dataYearIn.OCm[i1])*(100/(100-dataYearIn.STONES[i1]-dataYearIn.GRAVEL[i1]));
                   PW_K[-4,i1]:=round(100*dataYearIn.STONESm[i1]/(power((PW_X[-4]/100000),3)*PW_PARDEN[-4]));
                   dataYearIn.GRAVELm[i1]:=(dataYearIn.GRAVEL[i1]/100)*(dataYearIn.SANDm[i1]+dataYearIn.SILTm[i1]+dataYearIn.CLAYm[i1]+1.7*dataYearIn.OCm[i1])*(100/(100-dataYearIn.STONES[i1]-dataYearIn.GRAVEL[i1]));
                   {nb was it a bug, "(100*dataYearIn.SANDm[i1]+d" instead of "(dataYearIn.SANDm[i1]+d"? guess so}
                   for j:=-3 to 0 do PW_K[j,i1]:=round(0.25*100*dataYearIn.Gravelm[i1]/(power((PW_X[j]/100000),3)*PW_PARDEN[j]));
                   for j:=1 to 5  do PW_K[j,i1]:=round(0.20*100*dataYearIn.SANDm[i1]/(power((PW_X[j]/100000),3)*PW_PARDEN[j]));
                   for j:=6 to 10 do PW_K[j,i1]:=round(0.20*100*dataYearIn.SILTm[i1]/(power((PW_X[j]/100000),3)*PW_PARDEN[j]));
                   PW_K[11,i1]:=round(100*dataYearIn.CLAYm[i1]/(power((PW_X[11]/100000),3)*PW_PARDEN[11]));
                  {now we calculate STONEPERC as a volume percentage}
                  totvol:=0;
                  for j:=-4 to 0 do totvol:=totvol+PW_K[j,i1]*power((PW_X[j]/100000),3); {volume of all gravel+stone sized mineral matter, in dm3 over dlz(mm)*100(m2)}
                  {total soil volume=dlz*100 dm3, thus volume% of gravel+stones=totvol*100/(dlz*100), simplifying to totvol/dlz}
                  DataYearIn.STONEPERC[i1]:=totvol/DataYearIn.dlz;    DataYearOut.STONEPERC[i1]:=totvol/DataYearIn.dlz;
                  DataInit.STONESm[i1]:=(DataInit.STONES[i1]/100)*(DataInit.SANDm[i1]+DataInit.SILTm[i1]+DataInit.CLAYm[i1]+1.7*DataInit.OCm[i1])*(100/(100-DataInit.STONES[i1]-DataInit.GRAVEL[i1]));
                  DataInit.GRAVELm[i1]:=(DataInit.GRAVEL[i1]/100)*(DataInit.SANDm[i1]+DataInit.SILTm[i1]+DataInit.CLAYm[i1]+1.7*DataInit.OCm[i1])*(100/(100-DataInit.STONES[i1]-DataInit.GRAVEL[i1]));

                  DataInit.STONEPERC[i1]:=DataYearIn.STONEPERC[i1]
                 end;
                end;  {for i1:=1 to nocompb do}
          end
      else if txtevent='WaterTable'
           then begin  {nocompb corresponds, rounded to dlz mm, the WaterTable.
                        Below this depth, at BBC=2 the hydraulic conductivity must be reduced.
                        This is done in Chem.HC using a factor <choke>}
                        DataYearIn.WaterTableDepth:=DataYearIn.DLZ*nocompb;
                        if DataYearIn.WaterTableDepth<DatayearIn.RothC.RDepth[1]
                        then untProcesses.WaterTableInRootingZone:=true;
                        system.readln(fevent);
                        if i=1 then
                        begin    {in year 1: set water content read from inputfile to
                                             theta-sat for all compartments below watertable}
                          for j:=1 to datayearin.NoLayers do
                          begin
                             datayearin.PARDEN[j]:=100/(datayearin.CLAY[j]/datayearin.RHOS2+
                                                   (100-datayearin.CLAY[j]-1.7*datayearin.OC[j])/
                                                   datayearin.RHOS1+1.7*datayearin.OC[j]/
                                                   datayearin.RHOS3);
                             if j*datayearin.DLZ>=DataYearIn.WaterTableDepth
                             then datayearin.AW[j]:=1-(datayearin.RHO[j]/datayearin.PARDEN[j])-0.01;
                          end;
                        end;
                end
      else if txtevent='VGNChange'
           then begin {nocompb corresponds to the number of layers for which
                       Van Genuchten parameters will be input (via file fnVGN).
                       The other (deeper) layers will get estimated
                       Van Genuchten parameters using
                       the Hypress functions in untProcesses.predictVGN}
                       system.readln(fevent,fnVGN);
                       fnVGN:=trim(fnVGN);
                       if fileexists(fnVGN) then
                       begin
                         for j:=1 to nocompb do DataYearIn.VGNmeasured[j]:=true;
                         for j:=nocompb+1 to DataYearIn.NoLayers do DataYearIn.VGNmeasured[j]:=false;
                         {now read Van Genuchten parameters from fnVGN}
                         system.Assign(fVGN,fnVGN); system.Reset(fVGN);
                         system.Readln(fVGN); {skip header line}
                         for j:=1 to nocompb do
                         begin
                           {read data}
                           system.Readln(fVGN,VGNts[j],VGNtr[j],VGNal[j],VGNen[j],VGNel[j],VGNks[j]);
                           {adapt theta-sat so that it reflects the porosity}
                           DataYearIn.PARDEN[j]:=100/(datayearin.CLAY[j]/datayearin.RHOS2+
                                                     (100-datayearin.CLAY[j]-1.7*datayearin.OC[j])/
                                                     datayearin.RHOS1+1.7*datayearin.OC[j]/
                                                     datayearin.RHOS3);
                           VGNts[j]:=min(max(1-DataYearIn.RHO[j]/DataYearIn.parden[j],0.05),VGNts[j]);
                           {adapt theta-sat to porosity calculated by particle density and bulk density,
                           but only if this value is smaller than the Read theta-sat}
                           VGNtr[j]:=min(VGNtr[j],VGNts[j]-0.01); {check that theta-res <=theta-sat}
                           {calc "m" from "n"}
                           VGNem[j]:=1-1/VGNen[j];
                         end;
                         closefile(fVGN);
                       end
                       else dshowmessage('Year '+currentyear+'; Not fatal: '+fnVGN+' not found; VGNChange ignored.');
                end
      else if txtevent='Plowing'
                 then begin
                        DataYearIn.cultdep[1]:=nocompb*dataYearIn.DLZ;
                        system.readln(fevent,CurrentPlowMixFraction);
                        DataYearOut.cultdep[1]:=nocompb*DataYearOut.DLZ;
                      end
            else
            begin
                 {No plowing: make sure that cultivation depth is reset to initially read value}
                 DataYearIn.cultdep[1]:=dataInit.cultdep[1];
                 DataYearOut.cultdep[1]:=dataYearIn.cultdep[1];
                 if txtevent='SlashBurn'
                 then begin
                        system.readln(fevent,clost2atm,clost2iom,clost2hum);
                        {literature values for clost2atm=0.7; clost2iom=0.03; clost2hum should be the rest}
                        {See Seiler and Crutzen 1980 Climate Change 2: 207-247 and its citation in the review paper
                         by Glaser, Lehmann and Zech, 2002, Biol.Fertil.Soils 35: 219-230}
                        closttot:=clost2atm+clost2iom+clost2hum;
                        if closttot<>1 then  {make the fractions sum up to 1!}
                        begin
                           clost2atm:=clost2atm/closttot;
                           clost2iom:=clost2iom/closttot;
                           clost2hum:=clost2hum/closttot;
                        end;
                        {transfer C and associated ions to CO2pool (factor clost2atm lost to atmosphere)
                                                        to IOMpool (factor closttoIOM is charred)
                                                        to HUMpool (factor clost2HUM to a slowly decomposable pool)
                         Do this for ectorganic material and the nocompb soil compartments affected by burning.
                         NB: nocompb may be =0!}
                         {burn the ectorganic matter}
                         closttot:=DataYearIn.RothC.ectoDPMpool+DataYearIn.RothC.ectoRPMpool+
                                   DataYearIn.RothC.ectoBIOpool+DataYearIn.RothC.ectoHUMpool;
                         DataYearIn.RothC.ectoDPMpool:=0;
                         DataYearIn.RothC.ectoRPMpool:=0;
                         DataYearIn.RothC.ectoBIOpool:=0;
                         DataYearIn.RothC.ectoHUMpool:=clost2hum*closttot;
                         DataYearIn.RothC.ectoCO2pool:=DataYearIn.RothC.ectoCO2pool+clost2atm*closttot;
                         DataYearIn.RothC.ectoIOMpool:=DataYearIn.RothC.ectoIOMpool+clost2IOM*closttot;
                         {equally burn the Delta14C-related pools}
                         closttot:=DataYearIn.RothC.ectoDPMXpool+DataYearIn.RothC.ectoRPMXpool+
                                   DataYearIn.RothC.ectoBIOXpool+DataYearIn.RothC.ectoHUMXpool;
                         DataYearIn.RothC.ectoDPMXpool:=0;
                         DataYearIn.RothC.ectoRPMXpool:=0;
                         DataYearIn.RothC.ectoBIOXpool:=0;
                         DataYearIn.RothC.ectoHUMXpool:=clost2hum*closttot;
                         DataYearIn.RothC.ectoIOMXpool:=DataYearIn.RothC.ectoIOMXpool+clost2IOM*closttot;
                         {equally burn the pools with associated ions}
                         for kchem:=1 to maxChem do
                         begin
                          catlosttot:=DataYearIn.CHEMDPMpool[kchem,0]+DataYearIn.CHEMRPMpool[kchem,0]+
                                      DataYearIn.CHEMBIOpool[kchem,0]+DataYearIn.CHEMHUMpool[kchem,0];
                          DataYearIn.CHEMDPMpool[kchem,0]:=0;
                          DataYearIn.CHEMRPMpool[kchem,0]:=0;
                          DataYearIn.CHEMBIOpool[kchem,0]:=0;
                          DataYearIn.CHEMHUMpool[kchem,0]:=clost2hum*catlosttot;
                          DataYearIn.CHEMIOMpool[kchem,0]:=DataYearIn.CHEMIOMpool[kchem,0]+clost2iom*catlosttot;
                          DataYearIn.CHEMCO2pool[kchem,0]:=DataYearIn.CHEMCO2pool[kchem,0]+clost2atm*catlosttot;;
                         end;
                         {burn the involved soil layers and also incorporate the burnt ectorganic layer in the first compartment}
                         for j:=1 to nocompb do
                         begin
                            closttot:=DataYearIn.RothC.DPMpool[j]+DataYearIn.RothC.RPMpool[j]+
                                      DataYearIn.RothC.BIOpool[j]+DataYearIn.RothC.HUMpool[j];
                            DataYearIn.RothC.DPMpool[j]:=0;
                            DataYearIn.RothC.RPMpool[j]:=0;
                            DataYearIn.RothC.BIOpool[j]:=0;
                            DataYearIn.RothC.HUMpool[j]:=clost2hum*closttot+DataYearIn.RothC.ectoHUMpool;
                            DataYearIn.RothC.CO2pool[j]:=DataYearIn.RothC.CO2pool[j]+clost2atm*closttot+DataYearIn.RothC.ectoCO2pool;
                            DataYearIn.RothC.IOMpool[j]:=DataYearIn.RothC.IOMpool[j]+clost2IOM*closttot+DataYearIn.RothC.ectoIOMpool;
                            {and set ectorganic C to 0}
                            DataYearIn.RothC.ectoHUMpool:=0;
                            DataYearIn.RothC.ectoCO2pool:=0;
                            DataYearIn.RothC.ectoIOMpool:=0;
                            {update OCmass-variable}
                            dataYearIn.OCm[j]:=(dataYearIn.RothC.DPMpool[j]+dataYearIn.RothC.RPMpool[j]+
                                dataYearIn.RothC.BIOpool[j]+dataYearIn.RothC.HUMpool[j]+
                                dataYearIn.RothC.IOMpool[j])/10; {1 ton C/ha =0.1 kg/m2}
                            closttot:=(dataYearIn.CLAYm[j]+dataYearIn.SILTm[j]+1.7*dataYearIn.OCm[j]+dataYearIn.SANDm[j])/dataYearIn.dlz;
                            {closttot contains now temporarily the value for the new bulk density}
                            dataYearIn.OC[j]:=100*dataYearIn.OCm[i]/(closttot*dataYearIn.dlz);
                            {the organic matter percentage is now also updated}
                            {equally burn the Delta14C-related pools}
                            closttot:=DataYearIn.RothC.DPMXpool[j]+DataYearIn.RothC.RPMXpool[j]+
                                      DataYearIn.RothC.BIOXpool[j]+DataYearIn.RothC.HUMXpool[j];
                            DataYearIn.RothC.DPMXpool[j]:=0;
                            DataYearIn.RothC.RPMXpool[j]:=0;
                            DataYearIn.RothC.BIOXpool[j]:=0;
                            DataYearIn.RothC.HUMXpool[j]:=clost2hum*closttot+DataYearIn.RothC.ectoHUMXpool;;
                            DataYearIn.RothC.IOMXpool[j]:=DataYearIn.RothC.IOMXpool[j]+clost2IOM*closttot+DataYearIn.RothC.ectoIOMXpool;
                            {and set Delta14C related pools to 0}
                            DataYearIn.RothC.HUMXpool[j]:=0;
                            DataYearIn.RothC.IOMXpool[j]:=0;
                            {equally burn the pools with associated ions}
                            for kchem:=1 to maxChem do
                            begin
                              catlosttot:=DataYearIn.CHEMDPMpool[kchem,j]+DataYearIn.CHEMRPMpool[kchem,j]+
                                          DataYearIn.CHEMBIOpool[kchem,j]+DataYearIn.CHEMHUMpool[kchem,j];
                              DataYearIn.CHEMDPMpool[kchem,j]:=0;
                              DataYearIn.CHEMRPMpool[kchem,j]:=0;
                              DataYearIn.CHEMBIOpool[kchem,j]:=0;
                              DataYearIn.CHEMHUMpool[kchem,j]:=clost2hum*catlosttot;
                              DataYearIn.CHEMIOMpool[kchem,j]:=DataYearIn.CHEMIOMpool[kchem,j]+clost2iom*catlosttot;
                              DataYearIn.CHEMCO2pool[kchem,j]:=DataYearIn.CHEMCO2pool[kchem,j]+clost2atm*catlosttot;
                            end;
                            {add ectorganic cation pools to layer 1 and set the ectorganic cation pools to 0}
                            begin
                              DataYearIn.CHEMHUMpool[kchem,1]:=DataYearIn.CHEMHUMpool[kchem,1]+DataYearIn.CHEMHUMpool[kchem,0];
                              DataYearIn.CHEMIOMpool[kchem,1]:=DataYearIn.CHEMIOMpool[kchem,1]+DataYearIn.CHEMIOMpool[kchem,0];
                              DataYearIn.CHEMCO2pool[kchem,1]:=DataYearIn.CHEMCO2pool[kchem,1]+DataYearIn.CHEMCO2pool[kchem,0];
                              DataYearIn.CHEMHUMpool[kchem,0]:=0;
                              DataYearIn.CHEMIOMpool[kchem,0]:=0;
                              DataYearIn.CHEMCO2pool[kchem,0]:=0;
                            end;
                         end;
                      end
                 else system.readln(fevent);
            end;
       {If GUItrace=1 then wwwriteln} singletracewriteln('bttnRunClick: event translated to boundary conditions for current year');
    end;
    { 32 Set plant growth data for this year, can be "constants" for some vegetations}
    {set plant growth data for SoilGen crop growth module}
    if DataYearIn.RothC.vegetatie='grassscrub'
    then begin DataYearIn.igrowth[1]:=1; DataYearIn.icut[1]:=1; DataYearIn.rtype[1]:=1; DataYearIn.pday[1]:=10199;
               DataYearIn.estart[1]:=30199; DataYearIn.rdfday[1]:=40199; DataYearIn.estop[1]:=50199;
               DataYearIn.dayhar[1]:=123199; DataYearIn.rdep[1]:=1; DataYearIn.ak1[1]:=1; DataYearIn.panfac[1]:=1;
         end
    else if DataYearIn.RothC.vegetatie='coniferouswood'
         then
         begin DataYearIn.igrowth[1]:=1; DataYearIn.icut[1]:=1; DataYearIn.rtype[1]:=1; DataYearIn.pday[1]:=10199;
               DataYearIn.estart[1]:=30199; DataYearIn.rdfday[1]:=40199; DataYearIn.estop[1]:=50199;
               DataYearIn.dayhar[1]:=123199; DataYearIn.rdep[1]:=1; DataYearIn.ak1[1]:=1; DataYearIn.panfac[1]:=1;
         end
    else if DataYearIn.RothC.vegetatie='deciduouswood'
         then
         begin DataYearIn.igrowth[1]:=1; DataYearIn.icut[1]:=1; DataYearIn.rtype[1]:=1; DataYearIn.pday[1]:=10199;
               DataYearIn.estart[1]:=30199; DataYearIn.rdfday[1]:=40199; DataYearIn.estop[1]:=50199;
               DataYearIn.dayhar[1]:=123199; DataYearIn.rdep[1]:=1; DataYearIn.ak1[1]:=1; DataYearIn.panfac[1]:=1;
         end
    else if DataYearIn.RothC.vegetatie='agriculture'
         then
         begin DataYearIn.igrowth[1]:=2; DataYearIn.icut[1]:=2; DataYearIn.rtype[1]:=1;
               DataYearIn.pday[1]:=DataInit.pday[1];
               DataYearIn.estart[1]:=DataInit.estart[1]; DataYearIn.rdfday[1]:=DataInit.rdfday[1];
               DataYearIn.estop[1]:=DataInit.estop[1];   DataYearIn.dayhar[1]:=DataInit.dayhar[1];
               DataYearIn.rdep[1]:=1; DataYearIn.ak1[1]:=1; DataYearIn.panfac[1]:=1;
               {Thus, the date read from the first soil data input file remain valid!}
         end;
    { 33 call WriteDataC, WriteDataWE, WriteData (inputfiles for this year)}
    WriteDataC('RC'+copy(si,3,length(si)-2));  {inputfile RothC}
    WriteDataWE('WE'+copy(si,3,length(si)-2)); {inputfile Weathering routines}
    WriteData(si);                             {inputfile Soil submodel}
    { 34 Calculate and report to the GUI some progress variabes}
    frmMAIN.Caption:=versionID+' running inputfile '+AppendPathDelim(startdir)+si;
    {If GUItrace=1 then wwwriteln} singletracewriteln('bttnRunClick: input files for C, Weathering and soil props created for current year');
    {header in output screen}
    str(i:5,txtScreenOutput); txtScreenOutput:=trim(txtScreenOutput);
    if chckbxSpecialClimate.checked {sum up annual input values Precipitation}
    then begin
           tempsum:=0;
           for i1:=1 to 12 do tempsum:=tempsum+PtotMonthly[i1];
           str(tempsum*corP/(1-frPINT):4:0,txttmp);
         end
    else str(PTOT*slopewindfacP:4:0,txttmp);
    txt:=trim(txttmp);
    txtScreenOutput:='Year='+txtScreenOutput+'| P='+txttmp+' mm ';
    if chckbxSpecialClimate.checked {sum up annual input values Potential Evaporation}
    then begin
           tempsum:=0;
           for i1:=1 to 12 do tempsum:=tempsum+PEtotMonthly[i1];
           str(tempsum*corPE:4:0,txttmp);
         end
    else str(PETOT*slopewindfacPE:4:0,txttmp);
    txt:=trim(txttmp);  txtScreenOutput:=txtScreenOutput+'| PE='+txttmp+' mm ';
    if chckbxSpecialClimate.checked {report january input value temperature}
    then str(AvTempMonthly[1]:5:1,txttmp)
    else str(JanT:5:1,txttmp);
    txt:=trim(txttmp);  txtScreenOutput:=txtScreenOutput+'| JanT/JulT ='+txttmp+'/';
    if chckbxSpecialClimate.checked {report july input value temperature}
    then str(AvTempMonthly[7]:5:1,txttmp)
    else str(JulT:5:1,txttmp);
    txt:=trim(txttmp);  txtScreenOutput:=txtScreenOutput+txttmp+' °C ';
    txtScreenOutput:=txtScreenOutput+'| Veg='+DataYearIn.RothC.vegetatie+' ';
    if DataYearIn.WaterTableDepth<=DataYearIn.PDEPTH
    then begin str(DataYearIn.WaterTableDepth:4:0,txttmp); txttmp:='| perched WT: '+txttmp+' mm '; end
    else txttmp:='';
    if BBC=2 then txttmp:=txttmp+'| free drainage '
             else if BBC=3 then txttmp:=txttmp+'| 0-drainage ';
    txt:=trim(txttmp); txtScreenOutput:=txtScreenOutput+' '+txttmp;
    if (eventsfile)and(txtevent<>'non') then txtScreenOutput:=txtScreenoutput+'| Event: '+txtevent;
    grpbxScreenOutput.Caption:=' Screen TDD (blue=air; purple=rock). '+ txtScreenOutput;

    GeneralRefresh;
    untProcesses.yearBP:=spndtMultipleYears.Value-i+1; untProcesses.d14Cvalue:=d14C[untProcesses.yearBP];
    { 35   Start the run inside the year (untProcesses.StartProcesses)}
    {If GUItrace=1 then wwwriteln} singletracewriteln('bttnRunClick: about to enter Process Unit for current year');
    if chckbxDOS.Checked then
    begin
      RunLeachC(si);      {run LEACHC as external DOS-application, disactivated}
    end
    else
    begin
      mmDOS.Clear;
      untProcesses.file0:=si;
      untProcesses.fileWE:='WE'+copy(si,3,length(si)-2);
      MVAR(GlobalTime*10000+Datayearin.nolayers*100,' Voor untProcesses: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
      MVAR(GlobalTime*10000+Datayearin.nolayers*100,' Voor untProcesses: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);
      untProcesses.StartProcesses; {run the internal SoilGen code}
      MVAR(GlobalTime*10000+Datayearin.nolayers*100,' Na untProcesses: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
      MVAR(GlobalTime*10000+Datayearin.nolayers*100,' Na untProcesses: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);
    end;
    {If GUItrace=1 then wwwriteln} singletracewriteln('bttnRunClick: just ended Process Units for current year');
    { 36 Conditionally:  If bedrock then write mass per mineral and rockporosity and weathering loss
                         at end of current year}
    if FileExistsUTF8('Bedrock.txt') { *Converted from FileExists*  } then
      {output to file of rock properties}
    with Rock do
    begin
        system.assign(ftmp,'Bedrock.out'); system.append(ftmp);
        str(GlobalTime:5,txt);
        system.write(ftmp,txt,' ');
        system.write(ftmp,rockAlbite[2]:15:12,' ');
        system.write(ftmp,rockKFeldspar[2]:15:12,' ');
        system.write(ftmp,rockMuscovite[2]:15:12,' ');
        system.write(ftmp,rockBiotite[2]:15:12,' ');
        system.write(ftmp,rockAnorthite[2]:15:12,' ');
        system.write(ftmp,rockForsterite[2]:15:12,' ');
        system.write(ftmp,rockAugite[2]:15:12,' ');
        system.write(ftmp,rockHornblende[2]:15:12,' ');
        system.write(ftmp,rockFayalite[2]:15:12,' ');
        system.write(ftmp,rockQuartz[2]:15:12,' ');
        system.write(ftmp,rockChlorite[2]:15:12,' ');
        system.write(ftmp,rockKaolinite[2]:15:12,' ');
        system.write(ftmp,rockIllite[2]:15:12,' ');
        system.write(ftmp,rockMontmorillonite[2]:15:12,' ');
        system.write(ftmp,rockAmorphite[2]:15:12,' ');
        system.write(ftmp,rockOtherite[2]:15:12,' ');
        system.write(ftmp,rockGibbsite[2]:15:12,' ');
        system.write(ftmp,rockCalcite[2]:15:12,' ');
        system.write(ftmp,rockGypsum[2]:15:12,' ');
      {calculation of new porosity}
      minbaseddensity:=0;  rockmasssum:=0;
      {first with new mineral weights}
      minbaseddensity:=minbaseddensity+rockAlbite[1]*rockAlbite[2];
      rockmasssum:=rockmasssum+rockAlbite[2];
      minbaseddensity:=minbaseddensity+rockKFeldspar[1]*rockKFeldspar[2];  
      rockmasssum:=rockmasssum+rockKFeldspar[2];
      minbaseddensity:=minbaseddensity+rockMuscovite[1]*rockMuscovite[2];  
      rockmasssum:=rockmasssum+rockMuscovite[2];
      minbaseddensity:=minbaseddensity+rockBiotite[1]*rockBiotite[2]; 
      rockmasssum:=rockmasssum+rockBiotite[2];
      minbaseddensity:=minbaseddensity+rockAnorthite[1]*rockAnorthite[2]; 
      rockmasssum:=rockmasssum+rockAnorthite[2];
      minbaseddensity:=minbaseddensity+rockForsterite[1]*rockForsterite[2];  
      rockmasssum:=rockmasssum+rockForsterite[2];
      minbaseddensity:=minbaseddensity+rockAugite[1]*rockAugite[2]; 
      rockmasssum:=rockmasssum+rockAugite[2];
      minbaseddensity:=minbaseddensity+rockHornblende[1]*rockHornblende[2]; 
      rockmasssum:=rockmasssum+rockHornblende[2];
      minbaseddensity:=minbaseddensity+rockFayalite[1]*rockFayalite[2]; 
      rockmasssum:=rockmasssum+rockFayalite[2];
      minbaseddensity:=minbaseddensity+rockQuartz[1]*rockQuartz[2];  
      rockmasssum:=rockmasssum+rockQuartz[2];
      minbaseddensity:=minbaseddensity+rockChlorite[1]*rockChlorite[2]; 
      rockmasssum:=rockmasssum+rockChlorite[2];
      minbaseddensity:=minbaseddensity+rockKaolinite[1]*rockKaolinite[2]; 
      rockmasssum:=rockmasssum+rockKaolinite[2];
      minbaseddensity:=minbaseddensity+rockIllite[1]*rockIllite[2]; 
      rockmasssum:=rockmasssum+rockIllite[2];
      minbaseddensity:=minbaseddensity+rockMontmorillonite[1]*rockMontmorillonite[2];
      rockmasssum:=rockmasssum+rockMontmorillonite[2];
      minbaseddensity:=minbaseddensity+rockAmorphite[1]*rockAmorphite[2];  
      rockmasssum:=rockmasssum+rockAmorphite[2];
      minbaseddensity:=minbaseddensity+rockOtherite[1]*rockOtherite[2]; 
      rockmasssum:=rockmasssum+rockOtherite[2];
      minbaseddensity:=minbaseddensity+rockGibbsite[1]*rockGibbsite[2]; 
      rockmasssum:=rockmasssum+rockGibbsite[2];
      minbaseddensity:=minbaseddensity+rockCalcite[1]*rockCalcite[2];  
      rockmasssum:=rockmasssum+rockCalcite[2];
      minbaseddensity:=minbaseddensity+rockGypsum[1]*rockGypsum[2];  
      rockmasssum:=rockmasssum+rockGypsum[2];
      {regels hieronder kloppen niet!!!, nakijken.
      {then make these weight fractions relative to the original rock mass}
      minbaseddensity:=minbaseddensity/(Rock.rockdensity*dataInit.DLZ);
      Rock.rockporosityperc:=0;
      Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockAlbite[2]/Rock.rockAlbite[1];
      Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockKFeldspar[2]/Rock.rockKFeldspar[1];
      Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockMuscovite[2]/Rock.rockMuscovite[1];
      Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockBiotite[2]/Rock.rockBiotite[1];
      Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockAnorthite[2]/Rock.rockAnorthite[1];
      Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockForsterite[2]/Rock.rockForsterite[1];
      Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockAugite[2]/Rock.rockAugite[1];
      Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockHornblende[2]/Rock.rockHornblende[1];
      Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockFayalite[2]/Rock.rockFayalite[1];
      Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockQuartz[2]/Rock.rockQuartz[1];
      Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockChlorite[2]/Rock.rockChlorite[1];
      Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockKaolinite[2]/Rock.rockKaolinite[1];
      Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockIllite[2]/Rock.rockIllite[1];
      Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockMontmorillonite[2]/Rock.rockMontmorillonite[1];
      Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockAmorphite[2]/Rock.rockAmorphite[1];
      Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockOtherite[2]/Rock.rockOtherite[1];
      Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockGibbsite[2]/Rock.rockGibbsite[1];
      Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockCalcite[2]/Rock.rockCalcite[1];
      Rock.rockporosityperc:=Rock.rockporosityperc+Rock.rockGypsum[2]/Rock.rockGypsum[1];
      Rock.rockporosityperc:=100*(1-Rock.rockporosityperc/DataInit.DLZ);
      system.write(ftmp,Rock.rockporosityperc:8:5,' ');
      system.write(ftmp,rock.phweathloss:8:5,' ');
      system.write(ftmp,rock.saproclaym:8:5,' ');
      system.write(ftmp,rock.saprosiltm:8:5,' ');
      system.write(ftmp,rock.saprosandm:8:5,' ');
      system.write(ftmp,rock.saprogravelm:8:5,' ');
      system.writeln(ftmp,rock.saprostonesm:8:5);
      closefile(ftmp);
    end;
    { 37 Conditionally: if daily output tdd's are requested then call CloseDailyTDD}
    if writeDaily then CloseDailyTDD;
    si:=si+'.out';
    {ReadOut(si);}       {read the annual outputfile inherited from LEACHC,
                         including the selectivity coefficients
                         that were calculated in year 1
                         and store the data in dataYearOut;
                         currently done via untProcesses.passDataOut (memory) to gain speed}
    { 38 Call ReadOutC, Call ReadOutWE (yearly outputs for C and minerals)}
    ReadOutC('RC'+copy(si,3,length(si)-2)); {read annual outputfile of RothC}
    ReadOutWE('WE'+copy(si,3,length(si)-2)); {read annual outputfile with weathering data}
    {If GUItrace=1 then wwwriteln} singletracewriteln('bttnRunClick: read C and Weathering outputfiles for current year');
    {adaptations after first, initialisation year}
    if i=1 then
    begin
      dataYearOut.NN1:=0; {fix selectivity coefficients after the first run}
      dataYearOut.PRIOR:=2; {from now on pass pressure heads to next run;
                         to prevent problems with vol.moisture contents exceeding theta-sat
                         As theta-sat is calculated using a PTF based on
                         OC, BD, CS and as these may be changed since the previous
                         run of SoilGen as result of bioturbation etc.
                         Problem mainly occurs in frozen soils}
    end;
    { 39 Update mass per textural class}
    for i1:=1 to dataYearOut.NoLayers do
    begin
      DataYearOut.gravelm[i1]:=(PW_K[-3,i1]*power((PW_X[-3]/100000),3)*PW_PARDEN[-3]/100)+{eqvmass[-3,i]}
                              (PW_K[-2,i1]*power((PW_X[-2]/100000),3)*PW_PARDEN[-2]/100)+{eqvmass[-2,i]}
                              (PW_K[-1,i1]*power((PW_X[-1]/100000),3)*PW_PARDEN[-1]/100)+{eqvmass[-1,i]}
                              (PW_K[0,i1]*power((PW_X[0]/100000),3)*PW_PARDEN[0]/100);{eqvmass[0,i]};
      DataYearOut.stonesm[i1]:=(PW_K[-4,i1]*power((PW_X[-4]/100000),3)*PW_PARDEN[-4]/100);
      {DataYearOut.CLAYm[i1]:=(PW_K[11,i1]*power((PW_X[11]/100000),3)*PW_PARDEN[11]/100); niet helemaal hetzelfde als in untProcesses aangepaste waarde}
      DataYearOut.SILTm[i1]:=(PW_K[6,i1]*power((PW_X[6]/100000),3)*PW_PARDEN[6]/100)+
                              (PW_K[7,i1]*power((PW_X[7]/100000),3)*PW_PARDEN[7]/100)+
                              (PW_K[8,i1]*power((PW_X[8]/100000),3)*PW_PARDEN[8]/100)+
                              (PW_K[9,i1]*power((PW_X[9]/100000),3)*PW_PARDEN[9]/100)+
                              (PW_K[10,i1]*power((PW_X[10]/100000),3)*PW_PARDEN[10]/100);
      DataYearOut.SANDm[i1]:=(PW_K[1,i1]*power((PW_X[1]/100000),3)*PW_PARDEN[1]/100)+
                             (PW_K[2,i1]*power((PW_X[2]/100000),3)*PW_PARDEN[2]/100)+
                             (PW_K[3,i1]*power((PW_X[3]/100000),3)*PW_PARDEN[3]/100)+
                             (PW_K[4,i1]*power((PW_X[4]/100000),3)*PW_PARDEN[4]/100)+
                             (PW_K[5,i1]*power((PW_X[5]/100000),3)*PW_PARDEN[5]/100);
      DataYearOut.totalm[i1] := DataYearOut.CLAYm[i1] + DataYearOut.SILTm[i1] + DataYearOut.SANDm[i1] +
                               1.7 * DataYearOut.OCm[i1] + DataYearOut.Gravelm[i1] + DataYearOut.stonesm[i1];

      DataYearOut.gravel[i1]:=100*DataYearOut.Gravelm[i1]/DataYearOut.totalm[i1];
      DataYearOut.stones[i1]:=100*DataYearOut.stonesm[i1]/DataYearOut.totalm[i1];
      {now we calculate STONEPERC as a volume percentage}
      totvol:=0;
      for j:=-4 to 0 do totvol:=totvol+PW_K[j,i1]*power((PW_X[j]/100000),3); {volume of all gravel+stone sized mineral matter, in dm3 over dlz(mm)*100(m2)}
      {total soil volume=dlz*100 dm3, thus volume% of gravel+stones=totvol*100/(dlz*100), simplifying to totvol/dlz}
      DataYearOut.STONEPERC[i1]:=totvol/DataYearOut.dlz;
    end;
    { 40 call ReCalcOC (from mass per OC-pool to over-all OC mass, also include effect of physical weathering)}
    {calc the OC using the results of the C model; include the effect of physical weathering}
    ReCalcOC;
    {If GUItrace=1 then wwwriteln} singletracewriteln('bttnRunClick: just ended ReCalcOC for current year');
    { 41 call AdjustCECAXSE2OC (adjust the CEC to the changed OC-content)}
    {adjust the CEC to the changed OC-content}
    AdjustCECAXSE2OC;
    {If GUItrace=1 then wwwriteln} singletracewriteln('bttnRunClick: just ended AdjustCECAXSE2OC for current year');
    { 42 call Turbation (apply bioturbation, if any, to the indicated toplayers)}
    {if applicable, apply the set bioturbation for the current year}
    if bioturbfile then
    begin
      system.Readln(fturb,year,dtop,dmax,dbot,fractop,fracmax,fracbot);
      if round(year)<>i then dshowmessage('Year '+currentyear+'; Fatal error: synchronisation error bioturbation file');
      {limit bioturbation depths to unfrozen part of the profile above the water table}
      dtop:=min((min(dtop,DataYearIn.unfrozen)),DataYearIn.watertabledepth);
      dmax:=min((min(dmax,DataYearIn.unfrozen)),DataYearIn.watertabledepth);
      dbot:=min((min(dbot,DataYearIn.unfrozen)),DataYearIn.watertabledepth);
      if (dbot>0) then Turbation(dtop,dmax,dbot,fractop,fracmax,fracbot); {this includes another call to AdjustCECAXSE2OC}
    end;
    {If GUItrace=1 then wwwriteln} singletracewriteln('bttnRunClick: just ended (natural) bioturbation routine for current year');
    { 43 Conditionally:  if agriculture then call Turbation to emulate harrowing; if set: first apply turning plow}
    {if applicable, apply max bioturbation (eqv plowing) when land use = agriculture}
    if DataYearOut.RothC.vegetatie='agriculture'
    then
    begin
      {first add all ectorganic material -should not be a lot- to the first compartment}
      dataYearOut.RothC.DPMpool[1]:=dataYearOut.RothC.DPMpool[1]+dataYearOut.RothC.ectoDPMpool;
      dataYearOut.RothC.ectoDPMpool:=0;
      dataYearOut.RothC.RPMpool[1]:=dataYearOut.RothC.RPMpool[1]+dataYearOut.RothC.ectoRPMpool;
      dataYearOut.RothC.ectoRPMpool:=0;
      dataYearOut.RothC.BIOpool[1]:=dataYearOut.RothC.BIOpool[1]+dataYearOut.RothC.ectoBIOpool;
      dataYearOut.RothC.ectoBIOpool:=0;
      dataYearOut.RothC.HUMpool[1]:=dataYearOut.RothC.HUMpool[1]+dataYearOut.RothC.ectoHUMpool;
      dataYearOut.RothC.ectoHUMpool:=0;
      dataYearOut.RothC.IOMpool[1]:=dataYearOut.RothC.IOMpool[1]+dataYearOut.RothC.ectoIOMpool;
      dataYearOut.RothC.ectoIOMpool:=0;
      {14Cspecies}
      if OK14c then
      begin
      dataYearOut.RothC.DPMXpool[1]:=dataYearOut.RothC.DPMXpool[1]+dataYearOut.RothC.ectoDPMXpool;
      dataYearOut.RothC.ectoDPMXpool:=0;
      dataYearOut.RothC.RPMXpool[1]:=dataYearOut.RothC.RPMXpool[1]+dataYearOut.RothC.ectoRPMXpool;
      dataYearOut.RothC.ectoRPMXpool:=0;
      dataYearOut.RothC.BIOXpool[1]:=dataYearOut.RothC.BIOXpool[1]+dataYearOut.RothC.ectoBIOXpool;
      dataYearOut.RothC.ectoBIOXpool:=0;
      dataYearOut.RothC.HUMXpool[1]:=dataYearOut.RothC.HUMXpool[1]+dataYearOut.RothC.ectoHUMXpool;
      dataYearOut.RothC.ectoHUMXpool:=0;
      dataYearOut.RothC.IOMXpool[1]:=dataYearOut.RothC.IOMXpool[1]+dataYearOut.RothC.ectoIOMXpool;
      dataYearOut.RothC.ectoIOMXpool:=0;
      end;
      for j:=1 to Maxchem do
      begin {add all chemicals stored in ectorganic materials}
         dataYearOut.CHEMDPMpool[j,1]:=dataYearOut.CHEMDPMpool[j,1]+dataYearOut.CHEMDPMpool[j,0];
         dataYearOut.CHEMDPMpool[j,0]:=0;
         dataYearOut.CHEMRPMpool[j,1]:=dataYearOut.CHEMRPMpool[j,1]+dataYearOut.CHEMRPMpool[j,0];
         dataYearOut.CHEMRPMpool[j,0]:=0;
         dataYearOut.CHEMBIOpool[j,1]:=dataYearOut.CHEMBIOpool[j,1]+dataYearOut.CHEMBIOpool[j,0];
         dataYearOut.CHEMBIOpool[j,0]:=0;
         dataYearOut.CHEMHUMpool[j,1]:=dataYearOut.CHEMHUMpool[j,1]+dataYearOut.CHEMHUMpool[j,0];
         dataYearOut.CHEMHUMpool[j,0]:=0;
         dataYearOut.CHEMIOMpool[j,1]:=dataYearOut.CHEMIOMpool[j,1]+dataYearOut.CHEMIOMpool[j,0];
         dataYearOut.CHEMIOMpool[j,0]:=0;
         dataYearOut.CHEMCO2pool[j,1]:=dataYearOut.CHEMCO2pool[j,1]+dataYearOut.CHEMCO2pool[j,0];
         dataYearOut.CHEMCO2pool[j,0]:=0;
      end;
      if not(TurningPlow)
      then
      begin
        {subsequently apply "plowing turbation" over the cultdep[1] top mm of the soil.
         It is assumed a gradient zone between max% biot and zero biot
         between cultivationdepth-layerthickness and cultivationdepth+layerthickness.
         A default value of 50% is taken from:
         Ullrich, A. and M. Volk. 2009. Application of the Soil and Water Assessment Tool (SWAT)
         to predict the impact of alternative management practices on water quality and quantity.
         Agricultural Water Management, 96 (8)pp. 1207-1217
         Via the file Events.txt other values may be set for cultivation depth and plowing turbation}
       if dataYearOut.cultdep[1]>0
       then Turbation(0,max(0,dataYearOut.cultdep[1]-DataYearIn.dlz),
                      min(dataYearOut.PDEPTH,dataYearOut.cultdep[1]+DataYearIn.dlz),
                      CurrentPlowMixFraction,CurrentPlowMixFraction,0);
      end
      else  {in case of turning plow}
      begin
        {First invert the top compartments to mimick plowing}
        plowedcomp:=round(dataYearOut.cultdep[1]/DataYearIn.dlz);
        EmulateTurningPlow(plowedcomp);
        {subsequently apply "turbation" by harrowing over the upper 100 mm of the soil.
         It is assumed a gradient zone between max% biot and zero biot between 50 and 100 mm.
         A default value of 50% is taken from:
         Ullrich, A. and M. Volk. 2009. Application of the Soil and Water Assessment Tool (SWAT)
         to predict the impact of alternative management practices on water quality and quantity.
         Agricultural Water Management, 96 (8)pp. 1207-1217
         Via the file Events.txt other values may be set for cultivation depth and harrowing turbation}
        Turbation(0,50,100,CurrentPlowMixFraction,CurrentPlowMixFraction,0);
      end;
      {If GUItrace=1 then wwwriteln} singletracewriteln('bttnRunClick: just ended plowing routine (agriculture) for current year');
    end;
    { 44 Update 'ectorganic.out' at the end of this year}
    {write ecto-organic status. NB: size of CO2-emission is calculated by
     (i) summing up the CO2-pools of all compartiments + ectoorganic CO2-pool
     (ii)compensating this value for the CO2 still present in the soil (using pCO2)}
    SumCO2:=0;
    for i1:=1 to dataYearOut.NoLayers do
    begin
      kHenry:=29.41*exp(-2400*(1/(dataYearOut.tem[i1]+273)-1/298));
      SumCO2:=SumCO2+dataYearOut.RothC.CO2pool[i1]-
                     untProcesses.pco2_to_tonha((dataYearOut.apCO2[i1]-pCO2atm),
                                            kHenry,
                                            dataYearOut.dlz*dataYearOut.porosity[i1]);
    end;
    sumCO2:=dataYearOut.RothC.ectoCO2pool+sumCO2;
    Cinput:=Cinput+dataYearOut.RothC.PlantResSUM*untProcesses.RothC.frectolitter+dataYearOut.RothC.manureSUM;
    system.writeln(fectoout,i:5,' ',Cinput:13:5,
                    ' ',dataYearOut.RothC.ectoDPMpool:13:5,' ',
                    dataYearOut.RothC.ectoRPMpool:13:5,' ',dataYearOut.RothC.ectoBIOpool:13:5,' ',
                    dataYearOut.RothC.ectoHUMpool:13:5,' ',dataYearOut.RothC.ectoIOMpool:13:5,' ',
                    sumCO2:13:5);
    ggProgressMultipleYears.Position:=i;
    ggProgressCurrentYear.Position:=0;
    {next 2 lines to allow for calculation of the effect of mass loss on texture}
    dataYearOut.oldCALCITEA:=DataYearIn.CALCITEA;
    dataYearOut.oldGYPA:=DataYearIn.GYPA;
    { 45 call CopyRecToFrom (copy record DataYearOut to DataYearIn, for next simulation year)}
    CopyRecToFrom(DataYearIn,dataYearOut); {put the complete record with new input data in DataYearIn}
    { 46 call EffectSaltLoss (CaCO3 and CaSO4 mass-loss/-gain on texture) +
         call WeatheringIndices +
         call Decay (14C decay)}
    {take into account the CaCO3 and CaSO4 mass-loss/-gain on texture as set in the input}
    if (rdgrpSALTLOSS.ItemIndex<=2)and(i>1) then EffectSaltLoss;
    {calculate weathering indices}
    WeatheringIndices(CIA,IndexB,CIW,PIA,WI,CPA);
    {If GUItrace=1 then wwwriteln} singletracewriteln('bttnRunClick: just calculated weathering indices for current year');
    {finally, decay 14C pools by radioactive decay. Uses the *poolsX in DataYearIn and copies the result to dataYearOut !}
    if OK14c then Decay;
    cor:=0; {used to show or not the newly added layer}
    { 47 Conditionally:  if bedrock and rockporosity>=threshold and weatheringloss>=threshold
                         then convert bedrock compartment to saprolite (=soil) compartment}
    {Here: check if the porosity in the upper bedrock layer of DLZ mm is now more than a threshold value.
     Only if yes: add one compartment below the current profile}
    if FileExistsUTF8('Bedrock.txt') { *Converted from FileExists*  } then
    begin  {if (i) the porosity has increased by 15%, and (ii) more than 30% of the mass has been physically weathered
            then transform the rock compartment to a saprolite compartment and start the weatheriig of a new rock compartment}
      if (Rock.rockporosityperc>(porosity_threshold+rock_original.rockporosityperc))and(rock.phweathloss>phweathloss_threshold) then
      begin
        cor:=1;
       {1. add compartment with properties of the saprolite layer below the current profile, and
        2. give the bedrock below the saprolite the orginal bedrock properties and make it 1 compartment thinner}
        {ad 1 add compartment with properties of the saprolite layer below the current profile}
        {the bottom soil compartment index is raised by 1}
        DataYearIn.NoLayers:=DataYearIn.NoLayers+1;
        DataYearIn.PDEPTH:=dataYearIn.PDEPTH+DataYearIn.DLZ;

         MVAR(GlobalTime*10000+Datayearin.nolayers*100,' MAIN, voor porositythresholdcheck=POS: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
         MVAR(GlobalTime*10000+Datayearin.nolayers*100,' MAIN, voor porositythresholdcheck=POS: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);
        {for DataYearIn do}
        begin
         {distribution over fractions based on physical weathering :}
         {already calculated in untProcesses (at end each year, PassDataOut):
          a. untGUI.rock.saproPW_K[-4]: the number of stones in the rock compartment;
          b. untGUI.rock.saproStonesm:= (untLEACHcShell.rock.saproPW_K[-4] * power((untLEACHcShell.PW_X[-4] / 100000), 3) * untLEACHcShell.PW_PARDEN[-4] / 100):
             the mass of stones in the rock compartent;
          c. untGUI.rock.phweathloss:=100*(untLEACHcShell.rock_original.saproStonesm-untLEACHcShell.rock.saproStonesm)/untLEACHcShell.rock_original.saproStonesm:
             the loss by physical weathering, as a mass% of the original rock mass.
         Calculations at this point in the code are needed to initialize the mass and nr of paticles in clay-silt-sand and their grainsize subfractions.}
         {1a : we assume that each mineral that is still present (i.e. not chemically weathered) will physically weather into sand, silt or clay
              as a function of the mho's hardness, with as threshold values: }
         {    Mho [0;3> = clay (Montmorillonite, Gypsum, Chlorite, Kaolinite, Muscovite, Biotite, Illite, Gibbsite) }
         {    Mho [3,6] = silt (Calcite, Hornblende, Augite, K-Feldspar, Anorthite, Amorphite, Otherite) }
         {    Mho <6;>  = sand (Fayalite, Albite, Quartz, Forsterite) }
         {1b: we multiply the remaining mass converted to texture fraction (using Mho), by the mass fraction lost by physical weathering =(rock.phweathloss/100).
              We assume the gravel mass =half of the mass lost by physical weathering}
         {update rock.saprostonesm using rock.phweathloss}
         rock.saprostonesm:=(1-rock.phweathloss/100)*(rock.rockMontmorillonite[2]+rock.rockGypsum[2]+rock.rockChlorite[2]+rock.rockKaolinite[2]+
                          rock.rockMuscovite[2]+rock.rockBiotite[2]+rock.rockIllite[2]+rock.rockGibbsite[2]+rock.rockCalcite[2]+rock.rockHornblende[2]+
                          rock.rockAugite[2]+rock.rockKFeldspar[2]+rock.rockAnorthite[2]+rock.rockAmorphite[2]+rock.rockOtherite[2]+
                          rock.rockFayalite[2]+rock.rockAlbite[2]+rock.rockQuartz[2]+rock.rockForsterite[2]);
         {calculate mass in clay-silt-sand-gravel fraction using the physically weathered part}
         rock.saproclaym:=0.5*(rock.phweathloss/100)*(rock.rockMontmorillonite[2]+rock.rockGypsum[2]+rock.rockChlorite[2]+rock.rockKaolinite[2]+
                          rock.rockMuscovite[2]+rock.rockBiotite[2]+rock.rockIllite[2]+rock.rockGibbsite[2]);
         rock.saprosiltm:=0.5*(rock.phweathloss/100)*(rock.rockCalcite[2]+rock.rockHornblende[2]+rock.rockAugite[2]+rock.rockKFeldspar[2]+
                          rock.rockAnorthite[2]+rock.rockAmorphite[2]+rock.rockOtherite[2]);
         rock.saprosandm:=0.5*(rock.phweathloss/100)*(rock.rockFayalite[2]+rock.rockAlbite[2]+rock.rockQuartz[2]+rock.rockForsterite[2]);
         rock.saprogravelm:=rock.saproclaym+rock.saprosiltm+rock.saprosandm;
         {here saproclaym+saprosiltm+saprosandm+saprogravelm equals rockpweathloss as a mass}
         {1c: These masses are determined according to the specific gravities of the composing minerals.
              Adaptation to the particle densities used in SoilGen (RHOS1 and RHOS2) is needed because otherwise the bulk densities will be structurally
              too high (some mineras have particle densities >>2.65).
              For this, first the particle density per texture class is determined by gemPD=SUM[1..n](massi*SGi)/SUM(massi)
              and then the masses are rock.sapro*m are corrected by multiplication by the ratio RHOS1/gemPD (but RHOS2/gemPD for clay)}
         gemPDstones:={SUM[1..n](massi*SGi}((1-rock.phweathloss/100)*
         ((rock.rockMontmorillonite[2]*rock.rockMontmorillonite[1])+(rock.rockGypsum[2]*rock.rockGypsum[1])+
          (rock.rockChlorite[2]*rock.rockChlorite[1])+(rock.rockKaolinite[2]*rock.rockKaolinite[1])+
          (rock.rockMuscovite[2]*rock.rockMuscovite[1])+(rock.rockBiotite[2]*rock.rockBiotite[1])+
          (rock.rockIllite[2]*rock.rockIllite[1])+(rock.rockGibbsite[2]*rock.rockGibbsite[1])+
          (rock.rockCalcite[2]*rock.rockCalcite[1])+(rock.rockHornblende[2]*rock.rockHornblende[1])+
          (rock.rockAugite[2]*rock.rockAugite[1])+(rock.rockKFeldspar[2]*rock.rockKFeldspar[1])+
          (rock.rockAnorthite[2]*rock.rockAnorthite[1])+(rock.rockAmorphite[2]*rock.rockAmorphite[1])+
          (rock.rockOtherite[2]*rock.rockOtherite[1])+ (rock.rockFayalite[2]*rock.rockFayalite[1])+
          (rock.rockAlbite[2]*rock.rockAlbite[1])+(rock.rockQuartz[2]*rock.rockQuartz[1])+
          (rock.rockForsterite[2]*rock.rockForsterite[1]))
          {/SUM(massi)}/rock.saprostonesm);

         gemPDclay:={SUM[1..n](massi*SGi}(0.5*(rock.phweathloss/100)*
         ((rock.rockMontmorillonite[2]*rock.rockMontmorillonite[1])+(rock.rockGypsum[2]*rock.rockGypsum[1])+
          (rock.rockChlorite[2]*rock.rockChlorite[1])+(rock.rockKaolinite[2]*rock.rockKaolinite[1])+
          (rock.rockMuscovite[2]*rock.rockMuscovite[1])+(rock.rockBiotite[2]*rock.rockBiotite[1])+
          (rock.rockIllite[2]*rock.rockIllite[1])+(rock.rockGibbsite[2]*rock.rockGibbsite[1]))
          {/SUM(massi)}/rock.saproclaym);

         gemPDsilt:={SUM[1..n](massi*SGi}(0.5*(rock.phweathloss/100)*
         ((rock.rockCalcite[2]*rock.rockCalcite[1])+(rock.rockHornblende[2]*rock.rockHornblende[1])+
          (rock.rockAugite[2]*rock.rockAugite[1])+(rock.rockKFeldspar[2]*rock.rockKFeldspar[1])+
          (rock.rockAnorthite[2]*rock.rockAnorthite[1])+(rock.rockAmorphite[2]*rock.rockAmorphite[1])+
          (rock.rockOtherite[2]*rock.rockOtherite[1]))
          {/SUM(massi)}/rock.saprosiltm);

         gemPDsand:={SUM[1..n](massi*SGi}(0.5*(rock.phweathloss/100)*
         ((rock.rockFayalite[2]*rock.rockFayalite[1])+(rock.rockAlbite[2]*rock.rockAlbite[1])+
          (rock.rockQuartz[2]*rock.rockQuartz[1])+(rock.rockForsterite[2]*rock.rockForsterite[1]))
          {/SUM(massi)}/rock.saprosandm);

         gemPDgravel:={SUM[1..n](massi*SGi}(0.5*(rock.phweathloss/100)*
         ((rock.rockMontmorillonite[2]*rock.rockMontmorillonite[1])+(rock.rockGypsum[2]*rock.rockGypsum[1])+
          (rock.rockChlorite[2]*rock.rockChlorite[1])+(rock.rockKaolinite[2]*rock.rockKaolinite[1])+
          (rock.rockMuscovite[2]*rock.rockMuscovite[1])+(rock.rockBiotite[2]*rock.rockBiotite[1])+
          (rock.rockIllite[2]*rock.rockIllite[1])+(rock.rockGibbsite[2]*rock.rockGibbsite[1])+
          (rock.rockCalcite[2]*rock.rockCalcite[1])+(rock.rockHornblende[2]*rock.rockHornblende[1])+
          (rock.rockAugite[2]*rock.rockAugite[1])+(rock.rockKFeldspar[2]*rock.rockKFeldspar[1])+
          (rock.rockAnorthite[2]*rock.rockAnorthite[1])+(rock.rockAmorphite[2]*rock.rockAmorphite[1])+
          (rock.rockOtherite[2]*rock.rockOtherite[1])+ (rock.rockFayalite[2]*rock.rockFayalite[1])+
          (rock.rockAlbite[2]*rock.rockAlbite[1])+(rock.rockQuartz[2]*rock.rockQuartz[1])+
          (rock.rockForsterite[2]*rock.rockForsterite[1]))
          {/SUM(massi)}/(rock.saproclaym+rock.saprosiltm+rock.saprosandm));

         {now correct saproclaym, saprosiltm, saprosandm, saprogravelm and saprostonesm by the ratio RHOS*/gemPD
         This uses RHOS*, the SoilGen particle density, where clay=RHOS2, other are RHOS1.
         Thus, differences in particle densities between silt-sand-gravel and stones dissappear}
         cor2:=rock.saprostonesm; rock.saprostonesm:=(cor2)*datainit.RHOS1/gemPDStones;
         cor2:=rock.saproclaym;   rock.saproclaym:=(cor2)*datainit.RHOS2/gemPDclay;
         cor2:=rock.saprosiltm;   rock.saprosiltm:=(cor2)*datainit.RHOS1/gemPDsilt;
         cor2:=rock.saprosandm;   rock.saprosandm:=(cor2)*datainit.RHOS1/gemPDsand;
         cor2:=rock.saprogravelm; rock.saprogravelm:=(cor2)*datainit.RHOS1/gemPDgravel;

         {1d: Calculate STONEPERC as a volume percentage, soilgen particle densities to convert mass to volume}
         rock.rockstonesperc:=rock.saproclaym/datainit.RHOS2 + (rock.saprosiltm+rock.saprosandm+rock.saprogravelm+rock.saproStonesm)/datainit.RHOS1;
             {now it is a volume, in dm3, in all remaining minerals over all grain sizes,
              this has to be converted to volume percentage still in stones and gravel}
             {volume in stones is used to recalculate Rock.saproPW_K[-4]}
         Rock.saproPW_K[-4]:=round(rock.rockstonesperc*(1-rock.phweathloss/100)/power((untGUI.PW_X[-4]/100000),3));
             {now calculate the stoneperc as well}
         rock.rockstonesperc:=rock.rockstonesperc*((100-0.5*rock.phweathloss))/datainit.dlz;
         DataYearIn.STONEPERC[DataYearIn.NoLayers]:=rock.rockstonesperc;

         {2: we now convert the saproclaym, saprosiltm and saprosandm to the number of grains rock.saproPW_K[1..11].
             Again we assume the gravel grains =half of the mass lost by physical weathering}
         Rock.saproPW_K[-3]:=round(0.25*Rock.saprogravelm/(power((untGUI.PW_X[-3]/100000),3)*untGUI.PW_PARDEN[-3]/100));
         Rock.saproPW_K[-2]:=round(0.25*Rock.saprogravelm/(power((untGUI.PW_X[-2]/100000),3)*untGUI.PW_PARDEN[-2]/100));
         Rock.saproPW_K[-1]:=round(0.25*Rock.saprogravelm/(power((untGUI.PW_X[-1]/100000),3)*untGUI.PW_PARDEN[-1]/100));
         Rock.saproPW_K[0]:=round(0.25*Rock.saprogravelm/(power((untGUI.PW_X[0]/100000),3)*untGUI.PW_PARDEN[0]/100));
         {sand grains}
         Rock.saproPW_K[1]:=round(0.2*Rock.saproSANDm/(power((untGUI.PW_X[1]/100000),3)*untGUI.PW_PARDEN[1]/100));
         Rock.saproPW_K[2]:=round(0.2*Rock.saproSANDm/(power((untGUI.PW_X[2]/100000),3)*untGUI.PW_PARDEN[2]/100));
         Rock.saproPW_K[3]:=round(0.2*Rock.saproSANDm/(power((untGUI.PW_X[3]/100000),3)*untGUI.PW_PARDEN[3]/100));
         Rock.saproPW_K[4]:=round(0.2*Rock.saproSANDm/(power((untGUI.PW_X[4]/100000),3)*untGUI.PW_PARDEN[4]/100));
         Rock.saproPW_K[5]:=round(0.2*Rock.saproSANDm/(power((untGUI.PW_X[5]/100000),3)*untGUI.PW_PARDEN[5]/100));
         {silt grains}
         Rock.saproPW_K[6]:=round(0.2*Rock.saproSILTm/(power((untGUI.PW_X[6]/100000),3)*untGUI.PW_PARDEN[6]/100));
         Rock.saproPW_K[7]:=round(0.2*Rock.saproSILTm/(power((untGUI.PW_X[7]/100000),3)*untGUI.PW_PARDEN[7]/100));
         Rock.saproPW_K[8]:=round(0.2*Rock.saproSILTm/(power((untGUI.PW_X[8]/100000),3)*untGUI.PW_PARDEN[8]/100));
         Rock.saproPW_K[9]:=round(0.2*Rock.saproSILTm/(power((untGUI.PW_X[9]/100000),3)*untGUI.PW_PARDEN[9]/100));
         Rock.saproPW_K[10]:=round(0.2*Rock.saproSILTm/(power((untGUI.PW_X[10]/100000),3)*untGUI.PW_PARDEN[10]/100));
         {clay grains}
         Rock.saproPW_K[11]:=round(Rock.saproCLAYm/(power((untGUI.PW_X[11]/100000),3)*untGUI.PW_PARDEN[11]/100));
         {3: We use the saproPW_K[-4..11] (including stones) to initialize
          3a: rock.sapro* (the mass percentages stones-gravel-sand-silt-clay)
          3b: DataYearIn.CLAYm[DataYearIn.NoLayers] (and silt-sand-gravel-stones mass)
          3c: DataYearIn.CLAY[DataYearIn.NoLayers]  (and silt-sand-gravel-stones mass percentages. Note: OC and OCm depend on overlying layer)    
          3d: DataYearIn.RHO[DataYearIn.NoLayers]   (using the PTF and mass% clay and OC)
          3e: PW_K[-4..11,DataYearIn.NoLayers] (the number of grains)   }

         {3a}
         Rock.saprostones:=100*Rock.saproStonesm/(Rock.saproCLAYm + Rock.saproSILTm + Rock.saproSANDm + Rock.saproGravelm + Rock.saproStonesm);
         Rock.saprogravel:=100*Rock.saproGravelm/(Rock.saproCLAYm + Rock.saproSILTm + Rock.saproSANDm + Rock.saproGravelm + Rock.saproStonesm);
         Rock.saprosilt:=100*Rock.saproSiltm/(Rock.saproCLAYm + Rock.saproSILTm + Rock.saproSANDm + Rock.saproGravelm + Rock.saproStonesm); 
         Rock.saprosand:=100*Rock.saproSandm/(Rock.saproCLAYm + Rock.saproSILTm + Rock.saproSANDm + Rock.saproGravelm + Rock.saproStonesm);
         Rock.saproclay:=100*Rock.saproClaym/(Rock.saproCLAYm + Rock.saproSILTm + Rock.saproSANDm + Rock.saproGravelm + Rock.saproStonesm);
         {3b}
         DataYearIn.CLAYm[DataYearIn.NoLayers]:=Rock.saproclaym;
         DataYearIn.SILTm[DataYearIn.NoLayers]:=Rock.saprosiltm;
         DataYearIn.SANDm[DataYearIn.NoLayers]:=Rock.saprosandm;
         DataYearIn.fineSANDm[i1]:=DataYearIn.SANDm[i1]*0.4; {assuming 40% is <256 microns}
         DataYearIn.GRAVELm[DataYearIn.NoLayers]:=Rock.saprogravelm;
         DataYearIn.STONESm[DataYearIn.NoLayers]:=Rock.saprostonesm;
         {3c}
         DataYearIn.OC[DataYearIn.NoLayers]:=DataYearIn.OC[DataYearIn.NoLayers-1];   {initial value: copy from layer above}
         DataYearIn.CLAY[DataYearIn.NoLayers]:=max(1,Rock.saproclaym*100/(Rock.saproclaym+Rock.saprosiltm+Rock.saprosandm));
         DataYearIn.SILT[DataYearIn.NoLayers]:=max(1,Rock.saprosiltm*100/(Rock.saproclaym+Rock.saprosiltm+Rock.saprosandm));
         DataYearIn.SAND[DataYearIn.NoLayers]:=max(1,Rock.saprosandm*100/(Rock.saproclaym+Rock.saprosiltm+Rock.saprosandm));
         {rescale to sum to 100%, in case the max-function led to 1 somewhere}
         DataYearIn.CLAY[DataYearIn.NoLayers]:=DataYearIn.CLAY[DataYearIn.NoLayers]*100/
                                              (DataYearIn.CLAY[DataYearIn.NoLayers]+DataYearIn.SILT[DataYearIn.NoLayers]+DataYearIn.SAND[DataYearIn.NoLayers]);  
         DataYearIn.SILT[DataYearIn.NoLayers]:=DataYearIn.SILT[DataYearIn.NoLayers]*100/
                                              (DataYearIn.CLAY[DataYearIn.NoLayers]+DataYearIn.SILT[DataYearIn.NoLayers]+DataYearIn.SAND[DataYearIn.NoLayers]);
         DataYearIn.SAND[DataYearIn.NoLayers]:=DataYearIn.SAND[DataYearIn.NoLayers]*100/
                                              (DataYearIn.CLAY[DataYearIn.NoLayers]+DataYearIn.SILT[DataYearIn.NoLayers]+DataYearIn.SAND[DataYearIn.NoLayers]);
         DataYearIn.GRAVEL[DataYearIn.NoLayers]:=Rock.saprogravelm*100/(Rock.saproclaym+Rock.saprosiltm+Rock.saprosandm+Rock.saprogravelm+Rock.saprostonesm);
         DataYearIn.STONES[DataYearIn.NoLayers]:=Rock.saprostonesm*100/(Rock.saproclaym+Rock.saprosiltm+Rock.saprosandm+Rock.saprogravelm+Rock.saprostonesm);
         {3d: by lack of something better: empirical estimation of initial RHO (Hypres)}
         {DataYearIn.RHO[DataYearIn.NoLayers]:=HYPRESRHO(DataYearIn.CLAY[DataYearIn.NoLayers],1.7*DataYearIn.OC[DataYearIn.NoLayers]);}
         {RHO can be done better: ρ=mass(s+s+c)/(T*volfr(s+s+c+pores) equivalent to ρ=mass(s+s+c)/(T*volfr(1-gr-st))  }
         DataYearIn.RHO[DataYearIn.NoLayers]:=(DataYearIn.CLAYm[DataYearIn.NoLayers]+DataYearIn.SILTm[DataYearIn.NoLayers]+DataYearIn.SANDm[DataYearIn.NoLayers])/
                                              (DataYearIn.dlz*DataYearIn.STONEPERC[DataYearIn.NoLayers]/100);
         DataYearIn.OCm[DataYearIn.NoLayers]:=((100-DataYearIn.GRAVEL[DataYearIn.NoLayers]-DataYearIn.STONES[DataYearIn.NoLayers])/100)*DataYearIn.OC[DataYearIn.NoLayers]*DataYearIn.RHO[DataYearIn.NoLayers]/100;
         {3e: set the texture class grain counts PW_K equal to the subfractions in sapro*}
         for j:=-4 to 11 do PW_K[j,DataYearIn.NoLayers]:=rock.saproPW_K[j];

         {Other initial data come from the layer above}
         DataYearIn.AAEV[DataYearIn.NoLayers]:=DataYearIn.AAEV[DataYearIn.Nolayers-1];
         DataYearIn.ABCAM[DataYearIn.NoLayers]:=DataYearIn.ABCAM[DataYearIn.Nolayers-1];
         DataYearIn.CMATCH[DataYearIn.NoLayers]:=DataYearIn.CMATCH[DataYearIn.Nolayers-1];
         DataYearIn.HMATCH[DataYearIn.NoLayers]:=DataYearIn.HMATCH[DataYearIn.Nolayers-1];
         DataYearIn.RP[DataYearIn.NoLayers]:=DataYearIn.RP[DataYearIn.Nolayers-1];
         DataYearIn.LAMBDA[DataYearIn.NoLayers]:=DataYearIn.LAMBDA[DataYearIn.Nolayers-1];
         DataYearIn.fc[DataYearIn.NoLayers]:=DataYearIn.fc[DataYearIn.Nolayers-1];
         DataYearIn.d[DataYearIn.NoLayers]:=DataYearIn.d[DataYearIn.Nolayers-1];
         DataYearIn.MODRET[DataYearIn.NoLayers]:=DataYearIn.MODRET[DataYearIn.Nolayers-1];
         DataYearIn.AH[DataYearIn.NoLayers]:=DataYearIn.AH[DataYearIn.NoLayers-1];

         DataYearIn.AW[DataYearIn.NoLayers]:=min(DataYearIn.AW[DataYearIn.NoLayers-1],rock.rockporosityperc/100);
         DataYearIn.TEM[DataYearIn.NoLayers]:=DataYearIn.TEM[DataYearIn.NoLayers-1];
         DataYearIn.ATEM[DataYearIn.NoLayers]:=DataYearIn.ATEM[DataYearIn.NoLayers-1];
         DataYearIn.SAR[DataYearIn.NoLayers]:=DataYearIn.SAR[DataYearIn.NoLayers]-1;
         DataYearIn.pH[DataYearIn.NoLayers]:=DataYearIn.pH[DataYearIn.NoLayers]-1;
         DataYearIn.CALCITEA[DataYearIn.NoLayers]:=(rock.rockcalcite[2]/rock.rockmasssum)*DataYearIn.RHO[DataYearIn.NoLayers]*DataYearIn.DLZ;
         DataYearIn.oldCALCITEA[DataYearIn.NoLayers]:=DataYearIn.CALCITEA[DataYearIn.NoLayers];
         DataYearIn.GYPA[DataYearIn.NoLayers]:=(rock.rockgypsum[2]/rock.rockmasssum)*DataYearIn.RHO[DataYearIn.NoLayers]*DataYearIn.DLZ;
         DataYearIn.oldGYPA[DataYearIn.NoLayers]:=DataYearIn.GYPA[DataYearIn.NoLayers];
         DataYearIn.APCO2[DataYearIn.NoLayers]:=DataYearIn.APCO2[DataYearIn.NoLayers-1];
         DataYearIn.SEA[1][DataYearIn.NoLayers]:=DataYearIn.SEA[1][DataYearIn.NoLayers-1];
         DataYearIn.SEA[2][DataYearIn.NoLayers]:=DataYearIn.SEA[2][DataYearIn.NoLayers-1];
         DataYearIn.SEA[3][DataYearIn.NoLayers]:=DataYearIn.SEA[3][DataYearIn.NoLayers-1];
         DataYearIn.SEA[4][DataYearIn.NoLayers]:=DataYearIn.SEA[4][DataYearIn.NoLayers-1];
         DataYearIn.SEA[5][DataYearIn.NoLayers]:=DataYearIn.SEA[5][DataYearIn.NoLayers-1];
         DataYearIn.SEA[6][DataYearIn.NoLayers]:=DataYearIn.SEA[6][DataYearIn.NoLayers-1];
         DataYearIn.SEA[7][DataYearIn.NoLayers]:=DataYearIn.SEA[7][DataYearIn.NoLayers-1];
         DataYearIn.SEA[12][DataYearIn.NoLayers]:=DataYearIn.SEA[12][DataYearIn.NoLayers-1];
         DataYearIn.SEA[13][DataYearIn.NoLayers]:=DataYearIn.SEA[13][DataYearIn.NoLayers-1];
         DataYearIn.SEA[14][DataYearIn.NoLayers]:=DataYearIn.SEA[14][DataYearIn.NoLayers-1];
         DataYearIn.SEA[15][DataYearIn.NoLayers]:=DataYearIn.SEA[15][DataYearIn.NoLayers-1];
         DataYearIn.SEA[16][DataYearIn.NoLayers]:=DataYearIn.SEA[16][DataYearIn.NoLayers-1];
         DataYearIn.AXSE[1][DataYearIn.NoLayers]:=DataYearIn.AXSE[1][DataYearIn.NoLayers-1];
         DataYearIn.AXSE[2][DataYearIn.NoLayers]:=DataYearIn.AXSE[2][DataYearIn.NoLayers-1];
         DataYearIn.AXSE[3][DataYearIn.NoLayers]:=DataYearIn.AXSE[3][DataYearIn.NoLayers-1];
         DataYearIn.AXSE[4][DataYearIn.NoLayers]:=DataYearIn.AXSE[4][DataYearIn.NoLayers-1];
         DataYearIn.AXSE[10][DataYearIn.NoLayers]:=DataYearIn.AXSE[10][DataYearIn.NoLayers-1];
         DataYearIn.AXSE[12][DataYearIn.NoLayers]:=DataYearIn.AXSE[12][DataYearIn.NoLayers-1];
         DataYearIn.ACEC[DataYearIn.NoLayers]:=DataYearIn.ACEC[DataYearIn.NoLayers-1];
         DataYearIn.SC1[DataYearIn.NoLayers]:=DataYearIn.SC1[DataYearIn.NoLayers-1];
         DataYearIn.SC2[DataYearIn.NoLayers]:=DataYearIn.SC2[DataYearIn.NoLayers-1];
         DataYearIn.SC3[DataYearIn.NoLayers]:=DataYearIn.SC3[DataYearIn.NoLayers-1];
         DataYearIn.SC7[DataYearIn.NoLayers]:=DataYearIn.SC7[DataYearIn.NoLayers-1];
         DataYearIn.SC8[DataYearIn.NoLayers]:=DataYearIn.SC8[DataYearIn.NoLayers-1];
         {2-domain CEC  and AXSE}
          {1} DataYearIn.CEC_empirical[DataYearIn.NoLayers]:=32+36.7*DataYearIn.OC[DataYearIn.NoLayers]+1.96*DataYearIn.clay[DataYearIn.NoLayers];
          {2} DataYearIn.FacCEC[DataYearIn.NoLayers]:=DataYearIn.ACEC[DataYearIn.NoLayers]/DataYearIn.CEC_empirical[DataYearIn.NoLayers];
          {3} DataYearIn.FacCEC_OC[DataYearIn.NoLayers]:=36.7*DataYearIn.FacCEC[DataYearIn.NoLayers];  {FacCEC_OC contains adjusted regression coefficient}
          {4+7} DataYearIn.ACEC_clay[DataYearIn.NoLayers]:=DataYearIn.ACEC[DataYearIn.NoLayers]-DataYearIn.FacCEC_OC[DataYearIn.NoLayers]*DataYearIn.OC[DataYearIn.NoLayers];
                DataYearIn.ACEC_OC[DataYearIn.NoLayers]:=DataYearIn.ACEC[DataYearIn.NoLayers]-DataYearIn.ACEC_clay[DataYearIn.NoLayers];
          {5} DataYearIn.ClayFrCEC[DataYearIn.NoLayers]:=DataYearIn.ACEC_clay[DataYearIn.NoLayers]/DataYearIn.ACEC[DataYearIn.NoLayers];
          {6} DataYearIn.OCFrCEC[DataYearIn.NoLayers]:=DataYearIn.ACEC_OC[DataYearIn.NoLayers]/DataYearIn.ACEC[DataYearIn.NoLayers];
          {7} for j1:=1 to MaxChem do
          begin
            DataYearIn.AXSE_oc[j1,DataYearIn.NoLayers]:=DataYearIn.AXSE[j1,DataYearIn.NoLayers]*DataYearIn.OCFrCEC[DataYearIn.NoLayers];
            {_oc values will be adapted each year when the OC is adapted in RECALCOC;
             _clay values will be adapted in the CHEM.claymigration procedures and
                      in the CHEM.physicalweathering procedure}
            DataYearIn.AXSE_clay[j1,DataYearIn.NoLayers]:=DataYearIn.AXSE[j1,DataYearIn.NoLayers]*DataYearIn.ClayFrCEC[DataYearIn.NoLayers];
          end;

         {add: weathering data and rothc-data. Assuming in new layers:
         a. same mineral composition as original parent material;
         b. empty C-pools}
         {a1. mass }
         DataYearIn.WE.Gibbsitepool[DataYearIn.NoLayers]:=rock.rockGibbsite[2];
         DataYearIn.WE.MinPools[1][DataYearIn.NoLayers]:= rock.rockAlbite[2];
         DataYearIn.WE.MinPools[2][DataYearIn.NoLayers]:= rock.rockKFeldspar[2];
         DataYearIn.WE.MinPools[3][DataYearIn.NoLayers]:= rock.rockMuscovite[2];
         DataYearIn.WE.MinPools[4][DataYearIn.NoLayers]:= rock.rockBiotite[2];
         DataYearIn.WE.MinPools[5][DataYearIn.NoLayers]:= rock.rockQuartz[2];
         DataYearIn.WE.MinPools[6][DataYearIn.NoLayers]:= rock.rockChlorite[2];
         DataYearIn.WE.MinPools[7][DataYearIn.NoLayers]:= rock.rockAnorthite[2];
         DataYearIn.WE.MinPools[8][DataYearIn.NoLayers]:= rock.rockForsterite[2];
         DataYearIn.WE.MinPools[9][DataYearIn.NoLayers]:= rock.rockAugite[2];
         DataYearIn.WE.MinPools[10][DataYearIn.NoLayers]:=rock.rockKaolinite[2];
         DataYearIn.WE.MinPools[11][DataYearIn.NoLayers]:=rock.rockIllite[2];
         DataYearIn.WE.MinPools[12][DataYearIn.NoLayers]:=rock.rockMontmorillonite[2];
         DataYearIn.WE.MinPools[13][DataYearIn.NoLayers]:=rock.rockHornblende[2];
         DataYearIn.WE.MinPools[14][DataYearIn.NoLayers]:=rock.rockFayalite[2];
         DataYearIn.WE.MinPools[15][DataYearIn.NoLayers]:=rock.rockAmorphite[2];
         DataYearIn.WE.MinPools[16][DataYearIn.NoLayers]:=rock.rockOtherite[2];
         {a2. convert the mineral pools (kg) to elemental pools in minerals (mol)}
         {first convert kg mineral to mol mineral by using the
         Relative Formula Mass, RFM, per mineral from elemental weights and stoichiometric coeficients.
         These are calculated constants in untProcesses
         "1000" is used to convert kmol>mol}
          tmpMinPool[1]:=1000*DataYearIn.WE.MinPools[1][DataYearIn.NoLayers]/untProcesses.RFMalbite;
          tmpMinPool[2]:=1000*DataYearIn.WE.MinPools[2][DataYearIn.NoLayers]/untProcesses.RFMkfeldspar;
          tmpMinPool[3]:=1000*DataYearIn.WE.MinPools[3][DataYearIn.NoLayers]/untProcesses.RFMmuscovite;
          tmpMinPool[4]:=1000*DataYearIn.WE.MinPools[4][DataYearIn.NoLayers]/untProcesses.RFMbiotite;
          tmpMinPool[5]:=1000*DataYearIn.WE.MinPools[5][DataYearIn.NoLayers]/untProcesses.RFMquartz;
          tmpMinPool[6]:=1000*DataYearIn.WE.MinPools[6][DataYearIn.NoLayers]/untProcesses.RFMchlorite;
          tmpMinPool[7]:=1000*DataYearIn.WE.MinPools[7][DataYearIn.NoLayers]/untProcesses.RFManorthite;
          tmpMinPool[8]:=1000*DataYearIn.WE.MinPools[8][DataYearIn.NoLayers]/untProcesses.RFMForsterite;
          tmpMinPool[9]:=1000*DataYearIn.WE.MinPools[9][DataYearIn.NoLayers]/untProcesses.RFMaugite;
          tmpMinPool[10]:=1000*DataYearIn.WE.MinPools[10][DataYearIn.NoLayers]/untProcesses.RFMkaolinite;
          tmpMinPool[11]:=1000*DataYearIn.WE.MinPools[11][DataYearIn.NoLayers]/untProcesses.RFMillite;
          tmpMinPool[12]:=1000*DataYearIn.WE.MinPools[12][DataYearIn.NoLayers]/untProcesses.RFMmontmorillonite;
          tmpMinPool[13]:=1000*DataYearIn.WE.MinPools[13][DataYearIn.NoLayers]/untProcesses.RFMhornblende;
          tmpMinPool[14]:=1000*DataYearIn.WE.MinPools[14][DataYearIn.NoLayers]/untProcesses.RFMfayalite;
          if RFMamorphite>0
             then tmpMinPool[15]:=1000*DataYearIn.WE.MinPools[15][DataYearIn.NoLayers]/untProcesses.RFMamorphite
             else tmpMinPool[15]:=0;
          if RFMotherite>0
             then tmpMinPool[16]:=1000*DataYearIn.WE.MinPools[16][DataYearIn.NoLayers]/untProcesses.RFMotherite
             else tmpMinPool[16]:=0;
          {tmpGibbsite:=1000*DataYearIn.WE.Gibbsitepool[DataYearIn.NoLayers]/untProcesses.RFMgibbsite; }
          {tmpGibbsite was never used. Line above this one therefore commented.
           Line below this line converts initial Gibbsite Al(OH)3 (kmol) to mol Al in Gibbsite}
          DataYearIn.WE.Gibbsitepool[DataYearIn.NoLayers]:=1000*DataYearIn.WE.Gibbsitepool[DataYearIn.NoLayers]/untProcesses.RFMgibbsite;
          {a3. then use the stoichiometric composition of each mineral to obtain mmoles of element:
          {1 albite Na Al Si3 O8}
          {2 K-feldspar K Al Si3 O8}
          {3 muscovite K Al3 Si3 O10 (OH)2}
          {4 biotite K Mg1.5 Fe1.5 Al Si3 O10 (OH)2}
          {5 Quartz Si O2}
          {6 Clorite Clinochlore Mg5 Al2 Si3 O10 (OH)8 }
          {7 anorthite Ca Al2 Si2 O8 }
          {8 Forsterite Mg2 Si O4}
          {9 augite orthoferrosilite Fe Si O3 }
          {10 kaolinite Al2 Si2 O5 (OH)4}
          {11 Illite Si3.4 O10 (OH)2 Mg0.02 Al2.38 Ca0.01 K0.44, Godderis et al 2006}
          {12 Montmorillonite Si4 O10 (OH)2 Mg0.33 Al1.67 Ca0.04125 Mg0.04125 K0.0825 Na0.0825 }
          {13 hornblende Ca2 Mg4 Al Si6.7 Al O22.4 from Profile model }
          {14 fayalite Fe2 Si O4}
          {15 amorphite}
          {16 otherite}
          DataYearIn.WE.WE_Capool[DataYearIn.NoLayers]:=0*tmpMinPool[1]+0*tmpMinPool[2]+0*tmpMinPool[3]+
                    0*tmpMinPool[4]+0*tmpMinPool[5]+0*tmpMinPool[6]+
                    1*tmpMinPool[7]+0*tmpMinPool[8]+0*tmpMinPool[9]+
                    0*tmpMinPool[10]+0.01*tmpMinPool[11]+0.04125*tmpMinPool[12]+
                    2*tmpMinPool[13]+0*tmpMinPool[14]+amorp_Ca*tmpMinPool[15]+
                    other_Ca*tmpMinPool[16];
          DataYearIn.WE.WE_Mgpool[DataYearIn.NoLayers]:=0*tmpMinPool[1]+0*tmpMinPool[2]+0*tmpMinPool[3]+
                    1.5*tmpMinPool[4]+0*tmpMinPool[5]+5*tmpMinPool[6]+
                    0*tmpMinPool[7]+2*tmpMinPool[8]+0*tmpMinPool[9]+
                    0*tmpMinPool[10]+0.02*tmpMinPool[11]+(0.33+0.04125)*tmpMinPool[12]+
                    4*tmpMinPool[13]+0*tmpMinPool[14]+amorp_Mg*tmpMinPool[15]+
                    other_Mg*tmpMinPool[16];
          DataYearIn.WE.WE_Napool[DataYearIn.NoLayers]:=1*tmpMinPool[1]+0*tmpMinPool[2]+0*tmpMinPool[3]+
                    0*tmpMinPool[4]+0*tmpMinPool[5]+0*tmpMinPool[6]+
                    0*tmpMinPool[7]+0*tmpMinPool[8]+0*tmpMinPool[9]+
                    0*tmpMinPool[10]+0*tmpMinPool[11]+0.0825*tmpMinPool[12]+
                    0*tmpMinPool[13]+0*tmpMinPool[14];
          DataYearIn.WE.WE_Kpool[DataYearIn.NoLayers]:=0*tmpMinPool[1]+1*tmpMinPool[2]+1*tmpMinPool[3]+
                    1*tmpMinPool[4]+0*tmpMinPool[5]+0*tmpMinPool[6]+
                    0*tmpMinPool[7]+0*tmpMinPool[8]+0*tmpMinPool[9]+
                    0*tmpMinPool[10]+0.44*tmpMinPool[11]+0.0825*tmpMinPool[12]+
                    0*tmpMinPool[13]+0*tmpMinPool[14]+amorp_K*tmpMinPool[15]+
                    other_K*tmpMinPool[16];
          DataYearIn.WE.WE_Sipool[DataYearIn.NoLayers]:=3*tmpMinPool[1]+3*tmpMinPool[2]+3*tmpMinPool[3]+
                    3*tmpMinPool[4]+1*tmpMinPool[5]+3*tmpMinPool[6]+
                    2*tmpMinPool[7]+1*tmpMinPool[8]+1*tmpMinPool[9]+
                    2*tmpMinPool[10]+3.4*tmpMinPool[11]+4*tmpMinPool[12]+
                    6.7*tmpMinPool[13]+1*tmpMinPool[14];
          DataYearIn.WE.WE_Fepool[DataYearIn.NoLayers]:=0*tmpMinPool[1]+0*tmpMinPool[2]+0*tmpMinPool[3]+
                    1.5*tmpMinPool[4]+0*tmpMinPool[5]+0*tmpMinPool[6]+
                    0*tmpMinPool[7]+0*tmpMinPool[8]+1*tmpMinPool[9]+
                    0*tmpMinPool[10]+0*tmpMinPool[11]+0*tmpMinPool[12]+
                    0*tmpMinPool[13]+2*tmpMinPool[14]+amorp_Fe*tmpMinPool[15]+
                    other_Fe*tmpMinPool[16];
          DataYearIn.WE.WE_Alpool[DataYearIn.NoLayers]:=1*tmpMinPool[1]+1*tmpMinPool[2]+3*tmpMinPool[3]+
                    1*tmpMinPool[4]+0*tmpMinPool[5]+2*tmpMinPool[6]+
                    2*tmpMinPool[7]+0*tmpMinPool[8]+0*tmpMinPool[9]+
                    2*tmpMinPool[10]+2.38*tmpMinPool[11]+1.67*tmpMinPool[12]+
                    1*tmpMinPool[13]+0*tmpMinPool[14]+amorp_Al*tmpMinPool[15]+
                    other_Al*tmpMinPool[16];
         {b. Organic matter}
         DataYearIn.RothC.ClayPerc[DataYearIn.NoLayers]:=DataYearIn.CLAY[DataYearIn.NoLayers];
         DataYearIn.RothC.DPMpool[DataYearIn.NoLayers]:=0;  DataYearIn.RothC.RPMpool[DataYearIn.NoLayers]:=0;
         DataYearIn.RothC.BIOpool[DataYearIn.NoLayers]:=0;  DataYearIn.RothC.HUMpool[DataYearIn.NoLayers]:=0;
         DataYearIn.RothC.IOMpool[DataYearIn.NoLayers]:=0;  DataYearIn.RothC.CO2pool[DataYearIn.NoLayers]:=0;
         DataYearIn.RothC.accTSMD[DataYearIn.NoLayers]:=0;  DataYearIn.RothC.pCO2[DataYearIn.NoLayers]:=DataYearIn.RothC.pCO2[DataYearIn.NoLayers-1];
         {now distribute the incoming OC over the RothCpools in the same
          way as the initial C (see ReadDataC-procedure)}
         DataYearIn.RothC.DPMpool[DataYearIn.NoLayers]:=DataYearIn.RothCGEN.GENPoolPercs[1]
                           *((100-DataYearIn.GRAVEL[DataYearIn.NoLayers]-DataYearIn.STONES[DataYearIn.NoLayers])/100)
                           *DataYearIn.OC[DataYearIn.NoLayers]*DataYearIn.RHO[DataYearIn.NoLayers]*DataYearIn.dlz/1000;
         DataYearIn.RothC.RPMpool[DataYearIn.NoLayers]:=DataYearIn.RothCGEN.GENPoolPercs[2] 
                           *((100-DataYearIn.GRAVEL[DataYearIn.NoLayers]-DataYearIn.STONES[DataYearIn.NoLayers])/100)
                           *DataYearIn.OC[DataYearIn.NoLayers]*DataYearIn.RHO[DataYearIn.NoLayers]*DataYearIn.dlz/1000;
         DataYearIn.RothC.BIOpool[DataYearIn.NoLayers]:=DataYearIn.RothCGEN.GENPoolPercs[3] 
                           *((100-DataYearIn.GRAVEL[DataYearIn.NoLayers]-DataYearIn.STONES[DataYearIn.NoLayers])/100)
                           *DataYearIn.OC[DataYearIn.NoLayers]*DataYearIn.RHO[DataYearIn.NoLayers]*DataYearIn.dlz/1000;
         DataYearIn.RothC.HUMpool[DataYearIn.NoLayers]:=DataYearIn.RothCGEN.GENPoolPercs[4] 
                           *((100-DataYearIn.GRAVEL[DataYearIn.NoLayers]-DataYearIn.STONES[DataYearIn.NoLayers])/100)
                           *DataYearIn.OC[DataYearIn.NoLayers]*DataYearIn.RHO[DataYearIn.NoLayers]*DataYearIn.dlz/1000;
         DataYearIn.RothC.IOMpool[DataYearIn.NoLayers]:=DataYearIn.RothCGEN.GENPoolPercs[5] 
                           *((100-DataYearIn.GRAVEL[DataYearIn.NoLayers]-DataYearIn.STONES[DataYearIn.NoLayers])/100)
                           *DataYearIn.OC[DataYearIn.NoLayers]*DataYearIn.RHO[DataYearIn.NoLayers]*DataYearIn.dlz/1000;
         {delta14C pool corrected; d14C[year] taken from a table for all yearBP (relative to 1950) unless otherwise decided}
         DataYearIn.RothC.DPMXpool[DataYearIn.NoLayers]:=DataYearIn.RothC.DPMpool[DataYearIn.NoLayers]*d14C[yearBP];
         DataYearIn.RothC.RPMXpool[DataYearIn.NoLayers]:=DataYearIn.RothC.RPMpool[DataYearIn.NoLayers]*d14C[yearBP];
         DataYearIn.RothC.BIOXpool[DataYearIn.NoLayers]:=DataYearIn.RothC.BIOpool[DataYearIn.NoLayers]*d14C[yearBP];
         DataYearIn.RothC.HUMXpool[DataYearIn.NoLayers]:=DataYearIn.RothC.HUMpool[DataYearIn.NoLayers]*d14C[yearBP];
         DataYearIn.RothC.IOMXpool[DataYearIn.NoLayers]:=DataYearIn.RothC.IOMpool[DataYearIn.NoLayers]*d14C[yearBP];
         for j:=1 to MaxChem do DataYearIn.CHEMDPMpool[j][DataYearIn.NoLayers]:=0;
         for j:=1 to MaxChem do DataYearIn.CHEMRPMpool[j][DataYearIn.NoLayers]:=0;
         for j:=1 to MaxChem do DataYearIn.CHEMBIOpool[j][DataYearIn.NoLayers]:=0;
         for j:=1 to MaxChem do DataYearIn.CHEMHUMpool[j][DataYearIn.NoLayers]:=0;
         for j:=1 to MaxChem do DataYearIn.CHEMIOMpool[j][DataYearIn.NoLayers]:=0;
         for j:=1 to MaxChem do DataYearIn.CHEMCO2pool[j][DataYearIn.NoLayers]:=0;
         {there may be more vars to update...}
         { 48 with derived  soil data. call CopyRecToFrom (copy record DataYearOut to DataYearIn, for next simulation year)}
         copyrectofrom(DataYearOut,DataYearIn);
        end;
        {ad 2 give the bedrock below the saprolite the orginal bedrock properties and make it 1 compartment thinner}
        rock_original.RockBottomLayers:=max(0,rock_original.RockBottomLayers-1); rock:=rock_original;
        {begin var-specific conversion}
        for j:=1 to 4 do
    {[1]=specific weight; [2]=mass ; [3]=mass fraction per mineral in parent rock; [4]=mass lost relative to inital amount}
          begin
             rock.rockAlbite[j]:=rock_original.rockAlbite[j];
             rock.rockKFeldspar[j]:=rock_original.rockKFeldspar[j];
             rock.rockMuscovite[j]:=rock_original.rockMuscovite[j];
             rock.rockBiotite[j]:=rock_original.rockBiotite[j];
             rock.rockAnorthite[j]:=rock_original.rockAnorthite[j];
             rock.rockForsterite[j]:=rock_original.rockForsterite[j];
             rock.rockAugite[j]:=rock_original.rockAugite[j];
             rock.rockHornblende[j]:=rock_original.rockHornblende[j];
             rock.rockFayalite[j]:=rock_original.rockFayalite[j];
             rock.rockQuartz[j]:=rock_original.rockQuartz[j];
             rock.rockChlorite[j]:=rock_original.rockChlorite[j];
             rock.rockKaolinite[j]:=rock_original.rockKaolinite[j];
             rock.rockIllite[j]:=rock_original.rockIllite[j];
             rock.rockMontmorillonite[j]:=rock_original.rockMontmorillonite[j];
             rock.rockAmorphite[j]:=rock_original.rockAmorphite[j];
             rock.rockOtherite[j]:=rock_original.rockOtherite[j];
             rock.rockGibbsite[j]:=rock_original.rockGibbsite[j];
             rock.rockCalcite[j]:=rock_original.rockCalcite[j];
             rock.rockGypsum[j]:=rock_original.rockGypsum[j];
          end;
        for j:=1 to 8 do
    {these are the 6 weathering params and 2 dissolution params for 3 minerals}
          begin
            rock.weathCalcite[j]:=rock_original.weathCalcite[j];
            rock.weathGibbsite[j]:=rock_original.weathGibbsite[j];
            rock.weathGypsum[j]:=rock_original.weathGypsum[j]
          end;
        rock.rockdensity:=rock_original.rockdensity;
        rock.minbaseddensity:=rock_original.minbaseddensity;
        rock.rockporosityperc:=rock_original.rockporosityperc;
        rock.rockmasssum:=rock_original.rockmasssum;
        rock.rockstonesperc:=rock_original.rockstonesperc;
        rock.massclayminerals:=rock_original.massclayminerals;
        rock.RockBottomLayers:=rock_original.RockBottomLayers;
        for j:=-4 to 11 do
          begin
           rock.saproPW_G[j]:=rock_original.saproPW_G[j];
           rock.saproPW_K[j]:=rock_original.saproPW_K[j];
           rock.saproPW_S[j]:=rock_original.saproPW_S[j];
          end;
        rock.saprofreezecounter:=rock_original.saprofreezecounter;
        rock.saprorootcounter:=rock_original.saprorootcounter;
        rock.saprosandm:=rock_original.saprosandm;
        rock.saprosand:=rock_original.saprosand;
        rock.saprofinesandm:=rock_original.saprofinesandm;
        rock.saprosiltm:=rock_original.saprosiltm;
        rock.saprosilt:=rock_original.saprosilt;
        rock.saproclaym:=rock_original.saproclaym;
        rock.saproclay:=rock_original.saproclay;
        rock.saprogravelm:=rock_original.saprogravel;
        rock.saprogravel:=rock_original.saprogravel;
        rock.saprostonesm:=rock_original.saprostonesm;
        rock.saprostones:=rock_original.saprostones;
        {end var-specific copying}
         MVAR(GlobalTime*10000+Datayearin.nolayers*100,' MAIN, na porositythresholdcheck=POS: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
         MVAR(GlobalTime*10000+Datayearin.nolayers*100,' MAIN, na porositythresholdcheck=POS: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);
      end;
    end;
    { 49 call ScreenOutput1 (update screen with 6 tdd graphs)}
    {write selected graphs to screen}
    ScreenOutput1(i,cor);
    { 50 call WriteTimeDepth (write soil data at end of year to *.tdd-files)}
    {write time-depth diagrams to file}
    WriteTimeDepth(i,cor);
    {If GUItrace=1 then wwwriteln} singletracewriteln('bttnRunClick: just wrote TDDs to file for current year');
    if not(chckbxKeepFiles.Checked) then
    begin {do not keep *. and *.out }
      test:=DeleteFile('LC'+copy(si,3,length(si)-5)); if not(test) then debuglog('cannot delete LC*.');
      test:=DeleteFile('LC'+copy(si,3,length(si)-2)); if not(test) then debuglog('cannot delete LC*.out');
      test:=DeleteFile('RC'+copy(si,3,length(si)-5)); if not(test) then debuglog('cannot delete RC*.');
      test:=DeleteFile('RC'+copy(si,3,length(si)-2)); if not(test) then debuglog('cannot delete RC*.out');
      test:=DeleteFile('WE'+copy(si,3,length(si)-5)); if not(test) then debuglog('cannot delete WE*.');
      test:=DeleteFile('WE'+copy(si,3,length(si)-2)); if not(test) then debuglog('cannot delete WE*.out');
    end;
    LeftHours:=(hourspan(now,starttime)*spndtMultipleYears.Value/iyear)-hourspan(now,starttime);
    str(LeftHours:6:2,txtleft);
    lblLeftTime.Caption:=txtleft;
    {new year}
  end;
  {If GUItrace=1 then wwwriteln} singletracewriteln('bttnRunClick: ending annual cycle loop');
  { 51 Write final Van Genuchten parameters to file}
  system.Assign(fVGN,'FinalVGNparam.txt');
  system.Rewrite(fVGN);
  system.writeln(fVGN,'   Layer  Theta-Sat Theta-Res     alpha(cm-1)   n     gamma       ksat(mm/day)');
  for i1:=1 to dataYearOut.NoLayers do
  begin
    system.writeln(fVGN,' ',i1:8,' ',
                        untProcesses.VGNts[i1+1]:9:5,' ',
                        untProcesses.VGNtr[i1+1]:9:5,' ',
                        untProcesses.VGNal[i1+1]:9:5,' ',
                        untProcesses.VGNen[i1+1]:9:5,' ',
                        untProcesses.VGNel[i1+1]:9:5,' ',
                        untProcesses.VGNks[i1+1]:10:5);
  end;
  closefile(fVGN);
  { 52 call Prepare_Continuation (create subfolder 'continuation' and put in all the necessary files for a subsequent SoilGen run)}
  {create a subfolder [continuation] and put in all the necessary files for a subsequent SoilGen run}
  Prepare_Continuation;
  { 53 call WriteData, call WriteDataC, call WriteDataWE (inputfiles for a future year)}
  {create a Soil inputfile in case simulation series is to be continued later}
  WriteData('LC2BCont.');
  {create a  RothC inputfile in case simulation series is to be continued later}
  WriteDataC('RC2BCont.');  {inputfile RothC}
  {create a  Weathering inputfile in case simulation series is to be continued later}
  WriteDataWE('WE2BCont.'); {inputfile Weathering routines}
  { 54 write closing remarks to logfile + Close all outputfiles and call cleanupdisk (remove temporary files) +
       Make a batch file of the current run if that did not exist yet + Handle runtime errors.}
  system.Writeln(fmonit);
  system.writeln(fmonit,'Program terminated normally.');
  system.Writeln(fmonit,'This run can be continued from folder ',AppendPathDelim(startdir)+'continuation'+PathDelim);
  system.Writeln(fmonit,'This folder contains a complete dump of alle relevant variables.');
  system.writeln(fmonit,'The following files were also saved:');
  system.Writeln(fmonit,' <LC2BCont.> = soil & water input file;');
  system.Writeln(fmonit,' <RC2BCont.> contains status of C-pools ');
  system.Writeln(fmonit,' <WE2BCont.> contains status of Cation pools in primary minerals.');
  system.writeln(fmonit,'This run took ',hourspan(now,starttime):5:1,' hours runtime.');

  lblProgress.Caption:='Multiple years simulation ready.';
  frmMAIN.Caption:=versionid+' ready';
  {clean up}
  closefile(fmonit);
  closefile(fectoout);
  if bioturbfile then closefile(fturb);
  if climatefile then closefile(fclim);
  closefile(f1);    closefile(f2);
  closefile(f3);    closefile(f4);  closefile(f4a);
  closefile(f5);    closefile(f6);
  closefile(f7);    closefile(f8);
  closefile(f9);    closefile(f10);
  closefile(f11);   closefile(f12);
  closefile(f13);   closefile(f14); closefile(f14a);
  closefile(f15);   closefile(f16);
  closefile(f17);   closefile(f18);
  closefile(f19);   closefile(f20);
  closefile(f21);   closefile(f22);
  closefile(f23);   closefile(f24);
  closefile(f25);   closefile(f26);
  closefile(f27);   closefile(f28);
  closefile(f29);   closefile(f30);
  closefile(f31);   closefile(f32);
  closefile(f33);   closefile(f34);
  closefile(f35);   closefile(f36);
  closefile(f37);   closefile(f38);
  closefile(f39);   closefile(f40);
  closefile(f41);   closefile(f42);
  closefile(f43);   closefile(f44); closefile(f44a);
  closefile(f45);
  closefile(f46);   closefile(f47);
  closefile(f48);   closefile(f49);
  closefile(f50);   closefile(f51);
  closefile(f52);   closefile(f53);
  closefile(f54);   closefile(f55);
  closefile(f56);   closefile(f57);
  closefile(f58);   closefile(f59);
  closefile(f60);   closefile(f61);
  closefile(f62);   closefile(f63);
  closefile(f64);   closefile(f65);
  closefile(f66);   closefile(f67);
  closefile(f68);   closefile(f69);
  closefile(f70);   closefile(f71);
  closefile(f72);   closefile(f73);
  closefile(f74);   closefile(f75);
  closefile(f76);   closefile(f77);
  closefile(f78);   closefile(f79);
  closefile(f80);   closefile(f81);
  closefile(f82);   closefile(f83);
  closefile(f84);   closefile(f85);
  closefile(f86);   closefile(f87);  closefile(f87a);
  closefile(f88);   closefile(f88a);
  closefile(f89);
  closefile(f90);   closefile(f91); 
  closefile(f92);   closefile(f93);
  closefile(f94);   closefile(f95);       
  closefile(f96);   closefile(f97);
  closefile(f98);   closefile(f99);
  closefile(f100);
  if WritePSD then closefile(fmasstemp);
  if chckbxSpecialClimate.Checked then
  begin closefile(fmonthlyEVAP);closefile(fmonthlyPREC);closefile(fmonthlyTEMP);end;
  cleanupdisk;
  frmMain.BorderIcons:=[biSystemMenu, biMinimize];
  chckbxAbortNOW.Visible:=false;
  if not(batchmodeYN) then   {summarize the current run in a batch file}
  begin
    system.assign(fbatch,'SoilGenBatch.txt'); system.rewrite(fbatch);
    for ii:=1 to length(FirstLeachCFileName) do if FirstLeachCFileName[ii]='\'then mm:=ii;
    system.writeln(fbatch,copy(FirstLeachCFileName,mm+1,length(FirstLeachCFileName)-mm));
    system.writeln(fbatch,copy(EQFileName,mm+1,length(EQFileName)-mm));
    system.writeln(fbatch,copy(FirstRothCFileName,mm+1,length(FirstRothCFileName)-mm));
    system.writeln(fbatch,copy(FirstWeatherFileName,mm+1,length(FirstWeatherFileName)-mm));
    system.writeln(fbatch,spndtMultipleYears.Value:4);
    system.writeln(fbatch,datayearin.slope:5:1);
    system.writeln(fbatch,datayearin.upslopebearing:5:1);
    system.writeln(fbatch,datayearin.downwindbearing:5:1);
    system.writeln(fbatch,datayearin.latitude:6:3);
    closefile(fbatch);
  end;
  {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving bttnRunClick');
Except
  on EAccessViolation do{Checks for invalid memory access errors}
  HandleIt1;
  on EConvertError do{Indicates string or object conversion errors}
  HandleIt2;
  on EDivByZero do{Catches integer divide-by-zero errors}
  HandleIt3;
  on EExternalException do{Signifies an unrecognized exception code}
  HandleIt4;
  on EInOutError do{Represents a file I/O error}
  HandleIt5;
  on ERangeError do{Indicates an integer value that is too large for the declared type to which it is assigned}
  HandleIt6;
  on EZeroDivide do{Catches floating-point divide-by-zero errors}
  HandleIt7;
Else
  HandleIt;
  {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving bttnRunClick with errors');
end;
end;

procedure TfrmMAIN.chckbxCTurnoverClick(Sender: TObject);
{Identifies if a file with C-turnover data is available for reading}
begin
  lblDefC.visible:=not(chckbxCTurnover.Checked);
end;

procedure TfrmMAIN.chckbxEquilibriumsClick(Sender: TObject);
{Identifies if a file with chemical equilibrium data is available for reading}
begin
  lblDefEq.visible:=not(chckbxEquilibriums.Checked);
end;

procedure TfrmMAIN.chckbxInitMineralsClick(Sender: TObject);
{Identifies if a file with mineralogical data is available for reading}
begin
  lblDefMin.visible:=not(chckbxInitMinerals.Checked);
end;

procedure TfrmMAIN.chckbxInitSoilClick(Sender: TObject);
{Identifies if a file with initial soil data is available for reading}
begin
  lblDefSoil.visible:=not(chckbxInitSoil.Checked);
end;

procedure TfrmMAIN.chckbxSpecialClimateChange(Sender: TObject);
{only active if CheckBoxMonthlyData=true (usually not true) to invoke import of Monthly Climate data}
var header:string;
begin
  if (chckbxSpecialClimate.Checked)and(monthlyTEMPname='') then
  {monhtly climate data are to be given as input and this has not yet
   been done (no name of file has yet been given)}
  begin
    pndlgOpenSpecialClimate.InitialDir:=startdir;
    pndlgOpenSpecialClimate.Title:='Open a monthly climate file: temperature data';
    if pndlgOpenSpecialClimate.Execute
    then begin monthlyTEMPname:=pndlgOpenSpecialClimate.FileName;
               system.assign(fmonthlyTEMP,monthlyTEMPname);
               system.reset(fmonthlyTEMP); system.readln(fmonthlyTEMP,header);
         end
    else monthlyTEMPname:='';
    pndlgOpenSpecialClimate.Title:='Open a monthly climate file: precipitation data';
    if pndlgOpenSpecialClimate.Execute
    then begin monthlyPRECname:=pndlgOpenSpecialClimate.FileName;
               system.assign(fmonthlyPREC,monthlyPRECname);
               system.reset(fmonthlyPREC); system.readln(fmonthlyPREC,header);
         end
    else monthlyPRECname:='';
    pndlgOpenSpecialClimate.Title:='Open a monthly climate file: evaporation data';
    if pndlgOpenSpecialClimate.Execute
    then begin monthlyEVAPname:=pndlgOpenSpecialClimate.FileName;
               system.assign(fmonthlyEVAP,monthlyEVAPname);
               system.reset(fmonthlyEVAP); system.readln(fmonthlyEVAP,header);
         end
    else monthlyEVAPname:='';
  end;
end;

procedure TfrmMAIN.btbttnd14CClick(Sender: TObject);
{import d14C values for user-chosen years (will be interpolated)}
var YearBP,past,future,now,year:integer; getal:real; header:string; klaar:boolean;
begin
   klaar:=false;
   pndlgOPEN.Filter:= 'delta14C input files (*.txt)|*.txt';
   if pndlgOPEN.Execute then
   begin
      for YearBP:=-MaxNoYears to MaxNoyears do d14C[YearBP]:=-9999;
      d14CFileName:=pndlgOpen.FileName;
      system.assign(d14CFile,d14CFileName); system.reset(d14CFile);
      system.readln(d14CFile,header);
      while not(eof(d14CFile)) do
      begin
        system.Readln(d14CFile,YearBP,getal);
        if (YearBP<MaxNoyears)
        then d14C[YearBP]:=(getal/1000)+1;
      end;
      closefile(d14CFile);
      {interpolate missing years}
      {1. First series of years until first input data: make constant}
      if d14C[MaxNoyears]<>-9999
      then past:=MaxNoyears
      else
      begin
        now:=MaxNoYears;
        repeat
          now:=now-1;
        until (d14C[now]<>-9999)or(now=-MaxNoYears);
        past:=now;
        for year:=MaxNoYears downto past+1 do d14C[year]:=d14C[past];
      end;
      if (past=-MaxNoYears) and (d14C[past]=-9999)
      then begin d14C[past]:=0; {default value} klaar:=true end;
      for year:=MaxNoYears downto past+1 do d14C[year]:=d14C[past];
      while not(klaar) do
      begin {2. read other data in blocks and interpolate intermediate years}
        while past>-MaxNoYears do
        begin {seek future}
           future:=past;
           repeat
             future:=future-1;
           until (d14C[future]<>-9999)or(future=-MaxNoYears+1);
           klaar:=(future=-MaxNoYears+1)or(d14C[future]<>-9999);
        {past=oldest year with data; future=youngest year with data. Now interpolate}
           if d14C[future]<>-9999 then
           begin
             for year:=past-1 downto future+1 do
             begin
               d14C[year]:=d14C[past]-
                           (past-year)*(d14C[past]-d14C[future])/(past-future);
             end;
             past:=future;
           end
           else past:=-MaxNoYears+1;
        end;
      end;
      {3. Last series of years from last input data to BPyear=1: make constant}
      past:=-MaxNoYears;
      repeat past:=past+1 until d14C[past]<>-9999;
      for year:=-MaxNoYears to past-1 do d14C[year]:=d14C[past];
   end;
   {tmp:}
   system.assign(d14CFile,'d14Ctemp.txt'); system.rewrite(d14CFile);
   system.writeln(d14CFile,header);
   for yearBP:=MaxNoYears downto -MaxNoYears do
   system.writeln(d14CFile,YearBP,' ',d14C[yearBP]:10:8);
   closefile(d14CFile);
end;

procedure TfrmMAIN.btbttnEditBedrockClick(Sender: TObject);
{define, erase or edit bedrock mineralogical composition}
begin
  {de-activate menu item to display dump.txt. This is because it will fill
  the data fields, which may override the read input}
  showmemorydump1.Visible:=false;
  untEditBedrock.frmEditBedrock.ShowModal;
end;

procedure TfrmMAIN.btbttnEditCTurnoverClick(Sender: TObject);
{opens the window to allow editing the file with C-turnover data}
begin
  {de-activate menu item to display dump.txt. This is because it will fill
  the data fields, which may override the read input}
  showmemorydump1.Visible:=false;
  untEditCinputs.frmEditCinputs.ShowModal;
end;

procedure TfrmMAIN.btbttnEditEquilibriaClick(Sender: TObject);
{opens the window to allow editing the file with chemical equilibrium constants}
begin
  {de-activate menu item to display dump.txt. This is because it will fill
  the data fields, which may override the read input}
  showmemorydump1.Visible:=false;
  untEditChemConstants.frmEditChemConstants.ShowModal;
end;

procedure TfrmMAIN.btbttnEditWeatheringClick(Sender: TObject);
{void; editing mineralogical inputs via untEditSoilData}
begin
   {de-activate menu item to display dump.txt. This is because it will fill
   the data fields, which may override the read input}
   showmemorydump1.Visible:=false;
   {untEditSoilData.frmEditSoilData.ShowModal;}
end;

procedure TfrmMAIN.btbttnProtectionClick(Sender: TObject);
{opens a window to select and quantify protection agains decay of organic C pools}
begin
  untDecayProtection.frmDecayProtection.showmodal;
end;

procedure TfrmMAIN.bttnAutoCreateClick(Sender: TObject);
{autocreates the inputfiles selected in a previous screen grpbxOnStart}
var location,SoilFileName,EquilibriumFileName,CModelFileName,WeatheringFileName:string;
begin
  if not(fileexists('Amorphite.txt')) then mmAmorphite.Lines.SaveToFile('Amorphite.txt'); 
  if not(fileexists('Otherite.txt')) then mmOtherite.Lines.SaveToFile('Otherite.txt');
  if not(chckbxInitSoil.Checked) then mmInitSoil.Lines.SaveToFile('Default_InitSoil.txt');
  if not(chckbxInitMinerals.Checked) then mmInitMineralogy.Lines.SaveToFile('Default_InitMineralogy.txt');
  if not(chckbxEquilibriums.Checked) then mmEquilibriums.Lines.SaveToFile('Default_EquilibriumConstants.txt');
  if not(chckbxCTurnover.Checked) then mmCTurnover.Lines.SaveToFile('Default_CarbonInputs.txt');
  grpbxOnStart.visible:=false;
  startscreen:=false;
  {if all 4 Default_*.txt files were autocreated, then they can be read automatically as well}
  if (chckbxInitSoil.checked=false)and(chckbxInitMinerals.checked=false)and
     (chckbxEquilibriums.checked=false)and(chckbxCTurnover.checked=false)
  then {they were all autocreated and can be auto-read as well}
  begin
   location:=AppendPathDelim(startdir);
   SoilFileName:=location+'Default_InitSoil.txt';
   {read soil file}
     {de-activate menu item to display dump.txt. This is because it will fill
      the data fields, which may override the read input}
      showmemorydump1.Visible:=false;
      {If GUItrace=1 then wwwriteln} singletracewriteln('Entering bttnAutoCreateClick');
      pndlgOPEN.Filter:= 'Initial Soil status input files (*.txt)|*.txt';
         mmDefaultLeachC.Lines.LoadFromFile(SoilFileName);
         mmDefaultLeachC.Enabled:=true;
         InitChem;
         ReadData(SoilFileName);
         FirstLeachCFileName:=SoilFileName;
         {DataYearIn:=DataInit;}  CopyRecToFrom(DataYearIn,DataInit);
         DataYearIn.oldCALCITEA:=DataYearIn.CALCITEA;
         DataYearIn.oldGYPA:=DataYearIn.GYPA;
         if fileexists('Messages.txt')and(warningsissued)
         then bttnErrorInputs.Visible:=true
         else bttnErrorInputs.Visible:=false;
         if ReadDataOK then
         begin
           btbttnOpenEquilibrium.Enabled:=true;
           btbttnEditSoilData.Enabled:=true;
         end;
     {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving bttnAutoCreateClick');

   EquilibriumFileName:=location+'Default_EquilibriumConstants.txt';
   EQFileName:=EquilibriumFileName;
   {read equilibrium file}
     {de-activate menu item to display dump.txt. This is because it will fill
      the data fields, which may override the read input}
      showmemorydump1.Visible:=false;
      {If GUItrace=1 then wwwriteln} singletracewriteln('Entering btbttnOpenEquilibriumClick');
      pndlgOPEN.Filter:= 'Equilibrium constant input files (*.txt)|*.txt';
         mmEquilibrium.Lines.LoadFromFile(EquilibriumFileName);
         mmEquilibrium.Enabled:=true;
         ReadDataEQ(EquilibriumFileName);
         if fileexists('Messages.txt')and(warningsissued)
         then bttnErrorInputs.Visible:=true
         else bttnErrorInputs.Visible:=false;
         if ReadDataEQOK
         then
         begin
           btbttnOpenDefaultCModel.Enabled:=true;
           btbttnEditEquilibria.Enabled:=true;
         end;
      {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving btbttnOpenEquilibriumClick');

   CModelFileName:=location+'Default_CarbonInputs.txt';
   {read C-model inputfile}
     {de-activate menu item to display dump.txt. This is because it will fill
      the data fields, which may override the read input}
      showmemorydump1.Visible:=false;
      {If GUItrace=1 then wwwriteln} singletracewriteln('Entering btbttnOpenDefaultCModelClick');
      pndlgOPEN.Filter:= 'C-input files (*.txt)|*.txt';
         mmDefaultCModel.Lines.LoadFromFile(CModelFileName);
         mmDefaultCModel.Enabled:=true;
         ReadDataC(CModelFileName);
         FirstRothCFileName:=CModelFileName;
         {DataYearIn:=DataInit;}  CopyRecToFrom(DataYearIn,DataInit);
         if fileexists('Messages.txt')and(warningsissued)
         then bttnErrorInputs.Visible:=true
         else bttnErrorInputs.Visible:=false;
         if ReadDataCOK then
         begin
           btbttnOpenWeathering.Enabled:=true;
           btbttnEditCTurnover.Enabled:=true;
         end;
   {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving btbttnOpenDefaultCModelClick');

   WeatheringFileName:=location+'Default_InitMineralogy.txt';
   {read weathering inputfile}
     {de-activate menu item to display dump.txt. This is because it will fill
      the data fields, which may override the read input}
      showmemorydump1.Visible:=false;
      {If GUItrace=1 then wwwriteln} singletracewriteln('Entering btbttnOpenWeatheringClick');
      pndlgOPEN.Filter:= 'Initial mineralogy input files (*.txt)|*.txt';
         mmWeatheringData.Lines.LoadFromFile(WeatheringFileName);
         mmWeatheringData.Enabled:=true;
         ReadDataWE(WeatheringFileName);
         ALLOWED:=readweatherOK;
         if readweatherOK then btbttnEditWeathering.Enabled:=true;
         FirstWeatherFileName:=WeatheringFileName;
         {DataYearIn:=DataInit; }CopyRecToFrom(DataYearIn,DataInit);
         if fileexists('Messages.txt')and(warningsissued)
         then bttnErrorInputs.Visible:=true
         else bttnErrorInputs.Visible:=false;
      {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving btbttnOpenWeatheringClick');

     {soil production: read hardrock data}
    {de-activate menu item to display dump.txt. This is because it will fill
    the data fields, which may override the read input}
    showmemorydump1.Visible:=false;
    Rock.RockBottomLayers:=0;
    if fileexists('Bedrock.txt') then
    begin
       mmBedrock.Lines.LoadFromFile('Bedrock.txt');
       mmBedrock.Enabled:=true;
       ReadDataRock('Bedrock.txt');
       ALLOWED:=readbedrockOK;
       BedrockFileName:='Bedrock.txt';
       rdgrpChooseDisplay.columns:=5;
       Rock.RockBottomLayers:=rbl;
       {DataYearIn:=DataInit;}  CopyRecToFrom(DatayearIn,DataInit);
       if (not(bttnErrorInputs.Visible))and(fileexists('Messages.txt')and(warningsissued))
       then bttnErrorInputs.Visible:=true;
    end;


   tbcntrlSHELL.TabIndex:=1; tbcntrlSHELLChange(Self);{2nd tab sheet}

   lstbxScreenVariables.Clear;
   lstbxScreenVariables.Items.Add(cmbbxScreenVariables.Items[28]);
   lstbxScreenVariables.Items.Add(cmbbxScreenVariables.Items[1]);
   lstbxScreenVariables.Items.Add(cmbbxScreenVariables.Items[3]);
   lstbxScreenVariables.Items.Add(cmbbxScreenVariables.Items[31]);
   lstbxScreenVariables.Items.Add(cmbbxScreenVariables.Items[27]);
   lstbxScreenVariables.Items.Add(cmbbxScreenVariables.Items[21]);
  end;
end;

Procedure TfrmMAIN.HandleIt1; {invalid memory access errors}
{in case of Fatal error: EAccessViolation; invalid memory access error}
begin
 dshowmessage('Year '+currentyear+'; Fatal error: Invalid memory access error!');
 system.Writeln(fmonit);
 system.Writeln(fmonit,'Year '+currentyear+'; Fatal error: Invalid memory access error!');
 HandleIt;
end;

Procedure TfrmMAIN.HandleIt2; {string or object conversion errors}
{in case of Fatal error: EConvertError; string or object conversion error}
begin  
 dshowmessage('Year '+currentyear+'; Fatal error: String or object conversion error!');
 system.Writeln(fmonit);
 system.Writeln(fmonit,'Year '+currentyear+'; Fatal error: String or object conversion error!');
 HandleIt;
end;

Procedure TfrmMAIN.HandleIt3; {integer divide-by-zero errors}
{in case of Fatal error: EDivByZero: integer divide-by-zero error}
begin   
 dshowmessage('Year '+currentyear+'; Fatal error: Integer divide-by-zero error!');
 system.Writeln(fmonit);
 system.Writeln(fmonit,'Year '+currentyear+'; Fatal error: Integer divide-by-zero error!');
 HandleIt;
end;

Procedure TfrmMAIN.HandleIt4; {unrecognized exception code}
{in case of Fatal error: EExternalException: unrecognized exception code}
begin      
 dshowmessage('Year '+currentyear+'; Fatal error: Unrecognized exception code!');
 system.Writeln(fmonit);
 system.Writeln(fmonit,'Year '+currentyear+'; Fatal error: Unrecognized exception code!');
 HandleIt;
end;

Procedure TfrmMAIN.HandleIt5; {file I/O error}
{in case of Fatal error: EInOutError: file I/O error}
begin
 if chckbxAbortNOW.Checked   {EInOutError was raised!}
 then dshowmessage('Year '+currentyear+': Program aborted by user!')
 else
 begin dshowmessage('Year '+currentyear+': EInOutError!');
   system.Writeln(fmonit);
   system.Writeln(fmonit,'Year '+currentyear+': EInOutError!');
 end;
 HandleIt;
end;

Procedure TfrmMAIN.HandleIt6; {integer value that is too large for the declared type to which it is assigned}
{in case of Fatal error: ERangeError: integer too large for the declared type to which it is assigned}
begin
 dshowmessage(currentyear+'Fatal error: Integer value too large!');
 system.Writeln(fmonit);
 system.Writeln(fmonit,currentyear+'Fatal error: Integer value too large!');
 HandleIt;
end;

Procedure TfrmMAIN.HandleIt7; {floating-point divide-by-zero errors}
{in case of Fatal error: EZeroDivide: floating-point divide-by-zero error}
begin
 dshowmessage('Year '+currentyear+'; Fatal error: Floating-point divide-by-zero error!');
 system.Writeln(fmonit);
 system.Writeln(fmonit,'Year '+currentyear+'; Fatal error: Floating-point divide-by-zero error!');
 HandleIt;
end;

Procedure TfrmMAIN.HandleIt;
{in case of Fatal error: save continuation files, a memory dump and basic files that may serve as input}
begin
  untProcesses.TRACE:=0;   GUItrace:=0;
  othererror:=true; {one of the errors EAccessViolation/EConvertError/EDivByZero/EExternalException/EInOutError/ERangeError/EZeroDivide was raised}
  prepare_continuation;
  system.writeln(fmonit,'A set of continuation files were written to a folder with that name.');
  {create an annual inputfile in case simulation series is to be continued later}
  WriteData('LC2BCont.');
  {create a RothC inputfile in case simulation series is to be continued later}
  WriteDataC('RC2BCont.');  {inputfile RothC}
  {create a Weathering inputfile in case simulation series is to be continued later}
  WriteDataWE('WE2BCont.'); {inputfile Weathering routines}
  system.writeln(fmonit,'The following files were saved:');
  system.Writeln(fmonit,' <LC2BCont.> as new soil & water input file;');
  system.Writeln(fmonit,' <RC2BCont.> as C-turnover status file;');
  system.Writeln(fmonit,' <WE2BCont.> as weathering status file.');
  writedump;
  system.Writeln(fmonit,' <dump.txt> contains a memory dump.');
  closefile(fmonit);
  {activate menu item to display dump.txt}
  showmemorydump1.Caption:='memory dump (current run)';
  showmemorydump1.Visible:=true;
  closefile(f1);    closefile(f2);
  closefile(f3);    closefile(f4);  closefile(f4a);
  closefile(f5);    closefile(f6);
  closefile(f7);    closefile(f8);
  closefile(f9);    closefile(f10);
  closefile(f11);   closefile(f12);
  closefile(f13);   closefile(f14); closefile(f14a);
  closefile(f15);   closefile(f16);
  closefile(f17);   closefile(f18);
  closefile(f19);   closefile(f20);
  closefile(f21);   closefile(f22);
  closefile(f23);   closefile(f24);
  closefile(f25);   closefile(f26);
  closefile(f27);   closefile(f28);
  closefile(f29);   closefile(f30);
  closefile(f31);   closefile(f32);
  closefile(f33);   closefile(f34);
  closefile(f35);   closefile(f36);
  closefile(f37);   closefile(f38);
  closefile(f39);   closefile(f40);
  closefile(f41);   closefile(f42);
  closefile(f43);   closefile(f44); closefile(f44a);
  closefile(f45);
  closefile(f46);   closefile(f47);
  closefile(f48);   closefile(f49);
  closefile(f50);   closefile(f51);
  closefile(f52);   closefile(f53);
  closefile(f54);   closefile(f55);
  closefile(f56);   closefile(f57);
  closefile(f58);   closefile(f59);
  closefile(f60);   closefile(f61);
  closefile(f62);   closefile(f63);
  closefile(f64);   closefile(f65);
  closefile(f66);   closefile(f67);
  closefile(f68);   closefile(f69);
  closefile(f70);   closefile(f71);
  closefile(f72);   closefile(f73);
  closefile(f74);   closefile(f75);
  closefile(f76);   closefile(f77);
  closefile(f78);   closefile(f79);
  closefile(f80);   closefile(f81);
  closefile(f82);   closefile(f83);
  closefile(f84);   closefile(f85);
  closefile(f86);   closefile(f87);  closefile(f87a);
  closefile(f88);   closefile(f88a);
  closefile(f89);
  closefile(f90);   closefile(f91);
  closefile(f92);   closefile(f93);
  closefile(f94);   closefile(f95); 
  closefile(f96);   closefile(f97);
  closefile(f98);   closefile(f99);
  closefile(f100);
  if WritePSD then closefile(fmasstemp);
  cleanupdisk;
  close;
end;

procedure TfrmMAIN.writedump;
{saves variable status at time of crash to "dump.txt"}
var dmp:textfile; txt:string; i:integer; calcdum,gypsdum:real;
begin
  assignfile(dmp,'dump.txt');
  system.Rewrite(dmp);
  System.write(dmp,dataYearIn.NoLayers,' Year:',iYear,' ');
  if Chemstarted  {crashed inside Chem unit}
  then System.writeln(dmp,'Time (days):',untProcesses.Time:6:2)
  else System.writeln(dmp);
  {DataInit}
  for i:=1 to DataInit.Nolayers do
  begin
    system.writeln(dmp,DataInit.CLAY[i]);    system.writeln(dmp,DataInit.SILT[i]);
    system.writeln(dmp,DataInit.parden[i]);    system.writeln(dmp,DataInit.porosity[i]);
    system.writeln(dmp,DataInit.ClayDispInd[i]);     system.writeln(dmp,DataInit.OC[i]);
    system.writeln(dmp,DataInit.AW[i]);    system.writeln(dmp,DataInit.W[i]);
    system.writeln(dmp,DataInit.AH[i]);    system.writeln(dmp,DataInit.RDF[i]);
    system.writeln(dmp,DataInit.ATEM[i]);    system.writeln(dmp,DataInit.TEM[i]);
    system.writeln(dmp,DataInit.TEMPGEM[i]);    system.writeln(dmp,DataInit.AAEV[i]);
    system.writeln(dmp,DataInit.ABCAM[i]);    system.writeln(dmp,DataInit.RHO[i]);
    system.writeln(dmp,DataInit.CMATCH[i]);    system.writeln(dmp,DataInit.HMATCH[i]);
    system.writeln(dmp,DataInit.LAMBDA[i]);    system.writeln(dmp,DataInit.APH[i]);
    system.writeln(dmp,DataInit.PH[i]);    system.writeln(dmp,DataInit.APCO2[i]);
    {pf10 convert back to massfractions}
    calcdum:=DataInit.CALCITEA[i]/(DataInit.RHO[i]*(1+(DataInit.stones[i]+DataInit.gravel[i])/100)*DataInit.DLZ);
    gypsdum:=DataInit.GYPA[i]/(DataInit.RHO[i]*(1+(DataInit.stones[i]+DataInit.gravel[i])/100)*DataInit.DLZ);
    system.writeln(dmp,calcdum);    system.writeln(dmp,gypsdum);
    system.writeln(dmp,DataInit.ACEC[i]);    system.writeln(dmp,DataInit.alkalinity[i]);
    system.writeln(dmp,DataInit.BS[i]);    system.writeln(dmp,DataInit.EC[i]);
    system.writeln(dmp,DataInit.SAR[i]);    system.writeln(dmp,DataInit.ESP[i]);
    system.writeln(dmp,DataInit.ET[i]);    system.writeln(dmp,DataInit.FLUX[i]);
    system.writeln(dmp,DataInit.DOC[i]);    system.writeln(dmp,DataInit.SEA[1][i]);
    system.writeln(dmp,DataInit.SEA[2][i]);    system.writeln(dmp,DataInit.SEA[3][i]);
    system.writeln(dmp,DataInit.SEA[4][i]);    system.writeln(dmp,DataInit.SEA[5][i]);
    system.writeln(dmp,DataInit.SEA[6][i]);    system.writeln(dmp,DataInit.SEA[7][i]);
    system.writeln(dmp,DataInit.HCO3[i]);    system.writeln(dmp,DataInit.SEA[8][i]);
    system.writeln(dmp,DataInit.CO3[i]);    system.writeln(dmp,DataInit.SEA[9][i]);
    system.writeln(dmp,DataInit.SEA[10][i]);    system.writeln(dmp,DataInit.SEA[11][i]);
    system.writeln(dmp,DataInit.SEA[12][i]);    system.writeln(dmp,DataInit.SEA[13][i]);
    system.writeln(dmp,DataInit.SEA[14][i]);    system.writeln(dmp,DataInit.SEA[15][i]);
    system.writeln(dmp,DataInit.SEA[16][i]);    system.writeln(dmp,DataInit.SEA[17][i]);
    system.writeln(dmp,DataInit.SEA[18][i]);    system.writeln(dmp,DataInit.SEA[19][i]);
    system.writeln(dmp,DataInit.SEA[20][i]);    system.writeln(dmp,DataInit.SEA[21][i]);
    system.writeln(dmp,DataInit.SEA[22][i]);    system.writeln(dmp,DataInit.SEA[23][i]);
    system.writeln(dmp,DataInit.AXSE[1][i]);    system.writeln(dmp,DataInit.AXSE[2][i]);
    system.writeln(dmp,DataInit.AXSE[3][i]);    system.writeln(dmp,DataInit.AXSE[4][i]);
    system.writeln(dmp,DataInit.SEA[10][i]);    system.writeln(dmp,DataInit.SEA[12][i]);
    system.writeln(dmp,DataInit.RothC.DPMpool[i]);    system.writeln(dmp,DataInit.RothC.RPMpool[i]);
    system.writeln(dmp,DataInit.RothC.BIOpool[i]);    system.writeln(dmp,DataInit.RothC.HUMpool[i]);
    system.writeln(dmp,DataInit.RothC.IOMpool[i]);    system.writeln(dmp,DataInit.RothC.CO2pool[i]);
    system.writeln(dmp,DataInit.RothC.accTSMD[i]);    system.writeln(dmp,DataInit.RothC.pCO2[i]);
    system.writeln(dmp,DataInit.ChemDPMpool[1][i]);    system.writeln(dmp,DataInit.ChemRPMpool[1][i]);
    system.writeln(dmp,DataInit.ChemBIOpool[1][i]);    system.writeln(dmp,DataInit.ChemHUMpool[1][i]);
    system.writeln(dmp,DataInit.ChemIOMpool[1][i]);    system.writeln(dmp,DataInit.ChemCO2pool[1][i]);
    system.writeln(dmp,DataInit.ChemDPMpool[2][i]);    system.writeln(dmp,DataInit.ChemRPMpool[2][i]);
    system.writeln(dmp,DataInit.ChemBIOpool[2][i]);    system.writeln(dmp,DataInit.ChemHUMpool[2][i]);
    system.writeln(dmp,DataInit.ChemIOMpool[2][i]);    system.writeln(dmp,DataInit.ChemCO2pool[2][i]);
    system.writeln(dmp,DataInit.ChemDPMpool[3][i]);    system.writeln(dmp,DataInit.ChemRPMpool[3][i]);
    system.writeln(dmp,DataInit.ChemBIOpool[3][i]);    system.writeln(dmp,DataInit.ChemHUMpool[3][i]);
    system.writeln(dmp,DataInit.ChemIOMpool[3][i]);    system.writeln(dmp,DataInit.ChemCO2pool[3][i]);
    system.writeln(dmp,DataInit.ChemDPMpool[4][i]);    system.writeln(dmp,DataInit.ChemRPMpool[4][i]);
    system.writeln(dmp,DataInit.ChemBIOpool[4][i]);    system.writeln(dmp,DataInit.ChemHUMpool[4][i]);
    system.writeln(dmp,DataInit.ChemIOMpool[4][i]);    system.writeln(dmp,DataInit.ChemCO2pool[4][i]);
    system.writeln(dmp,DataInit.ChemDPMpool[5][i]);    system.writeln(dmp,DataInit.ChemRPMpool[5][i]);
    system.writeln(dmp,DataInit.ChemBIOpool[5][i]);    system.writeln(dmp,DataInit.ChemHUMpool[5][i]);
    system.writeln(dmp,DataInit.ChemIOMpool[5][i]);    system.writeln(dmp,DataInit.ChemCO2pool[5][i]);
    system.writeln(dmp,DataInit.ChemDPMpool[6][i]);    system.writeln(dmp,DataInit.ChemRPMpool[6][i]);
    system.writeln(dmp,DataInit.ChemBIOpool[6][i]);    system.writeln(dmp,DataInit.ChemHUMpool[6][i]);
    system.writeln(dmp,DataInit.ChemIOMpool[6][i]);    system.writeln(dmp,DataInit.ChemCO2pool[6][i]);
    system.writeln(dmp,DataInit.ChemDPMpool[7][i]);    system.writeln(dmp,DataInit.ChemRPMpool[7][i]);
    system.writeln(dmp,DataInit.ChemBIOpool[7][i]);    system.writeln(dmp,DataInit.ChemHUMpool[7][i]);
    system.writeln(dmp,DataInit.ChemIOMpool[7][i]);    system.writeln(dmp,DataInit.ChemCO2pool[7][i]);
    system.writeln(dmp,DataInit.ChemDPMpool[8][i]);    system.writeln(dmp,DataInit.ChemRPMpool[8][i]);
    system.writeln(dmp,DataInit.ChemBIOpool[8][i]);    system.writeln(dmp,DataInit.ChemHUMpool[8][i]);
    system.writeln(dmp,DataInit.ChemIOMpool[8][i]);    system.writeln(dmp,DataInit.ChemCO2pool[8][i]);
    system.writeln(dmp,DataInit.ChemDPMpool[9][i]);    system.writeln(dmp,DataInit.ChemRPMpool[9][i]);
    system.writeln(dmp,DataInit.ChemBIOpool[9][i]);    system.writeln(dmp,DataInit.ChemHUMpool[9][i]);
    system.writeln(dmp,DataInit.ChemIOMpool[9][i]);    system.writeln(dmp,DataInit.ChemCO2pool[9][i]);
    system.writeln(dmp,DataInit.ChemDPMpool[10][i]);    system.writeln(dmp,DataInit.ChemRPMpool[10][i]);
    system.writeln(dmp,DataInit.ChemBIOpool[10][i]);    system.writeln(dmp,DataInit.ChemHUMpool[10][i]);
    system.writeln(dmp,DataInit.ChemIOMpool[10][i]);    system.writeln(dmp,DataInit.ChemCO2pool[10][i]);
    system.writeln(dmp,DataInit.ChemDPMpool[11][i]);    system.writeln(dmp,DataInit.ChemRPMpool[11][i]);
    system.writeln(dmp,DataInit.ChemBIOpool[11][i]);    system.writeln(dmp,DataInit.ChemHUMpool[11][i]);
    system.writeln(dmp,DataInit.ChemIOMpool[11][i]);    system.writeln(dmp,DataInit.ChemCO2pool[11][i]);
    system.writeln(dmp,DataInit.ChemDPMpool[12][i]);    system.writeln(dmp,DataInit.ChemRPMpool[12][i]);
    system.writeln(dmp,DataInit.ChemBIOpool[12][i]);    system.writeln(dmp,DataInit.ChemHUMpool[12][i]);
    system.writeln(dmp,DataInit.ChemIOMpool[12][i]);    system.writeln(dmp,DataInit.ChemCO2pool[12][i]);
    system.writeln(dmp,DataInit.ChemDPMpool[13][i]);    system.writeln(dmp,DataInit.ChemRPMpool[13][i]);
    system.writeln(dmp,DataInit.ChemBIOpool[13][i]);    system.writeln(dmp,DataInit.ChemHUMpool[13][i]);
    system.writeln(dmp,DataInit.ChemIOMpool[13][i]);    system.writeln(dmp,DataInit.ChemCO2pool[13][i]);
    system.writeln(dmp,DataInit.ChemDPMpool[14][i]);    system.writeln(dmp,DataInit.ChemRPMpool[14][i]);
    system.writeln(dmp,DataInit.ChemBIOpool[14][i]);    system.writeln(dmp,DataInit.ChemHUMpool[14][i]);
    system.writeln(dmp,DataInit.ChemIOMpool[14][i]);    system.writeln(dmp,DataInit.ChemCO2pool[14][i]);
    system.writeln(dmp,DataInit.ChemDPMpool[15][i]);    system.writeln(dmp,DataInit.ChemRPMpool[15][i]);
    system.writeln(dmp,DataInit.ChemBIOpool[15][i]);    system.writeln(dmp,DataInit.ChemHUMpool[15][i]);
    system.writeln(dmp,DataInit.ChemIOMpool[15][i]);    system.writeln(dmp,DataInit.ChemDPMpool[16][i]);
    system.writeln(dmp,DataInit.ChemRPMpool[16][i]);    system.writeln(dmp,DataInit.ChemBIOpool[16][i]);
    system.writeln(dmp,DataInit.ChemHUMpool[16][i]);    system.writeln(dmp,DataInit.ChemIOMpool[16][i]);
    system.writeln(dmp,DataInit.ChemCO2pool[16][i]);    system.writeln(dmp,DataInit.WE.Gibbsitepool[i]);
    system.writeln(dmp,DataInit.WE.WE_Capool[i]);    system.writeln(dmp,DataInit.WE.WE_Mgpool[i]);
    system.writeln(dmp,DataInit.WE.WE_Kpool[i]);    system.writeln(dmp,DataInit.WE.WE_Napool[i]);
  end;
  {DataYearIn}
  for i:=1 to dataYearIn.Nolayers do
  begin
    system.writeln(dmp,DataYearIn.CLAY[i]);    system.writeln(dmp,DataYearIn.SILT[i]);
    system.writeln(dmp,DataYearIn.parden[i]);    system.writeln(dmp,DataYearIn.porosity[i]);
    system.writeln(dmp,DataYearIn.ClayDispInd[i]);     system.writeln(dmp,DataYearIn.OC[i]);
    system.writeln(dmp,DataYearIn.AW[i]);    system.writeln(dmp,DataYearIn.W[i]);
    system.writeln(dmp,DataYearIn.AH[i]);    system.writeln(dmp,DataYearIn.RDF[i]);
    system.writeln(dmp,DataYearIn.ATEM[i]);    system.writeln(dmp,DataYearIn.TEM[i]);
    system.writeln(dmp,DataYearIn.TEMPGEM[i]);    system.writeln(dmp,DataYearIn.AAEV[i]);
    system.writeln(dmp,DataYearIn.ABCAM[i]);    system.writeln(dmp,DataYearIn.RHO[i]);
    system.writeln(dmp,DataYearIn.CMATCH[i]);    system.writeln(dmp,DataYearIn.HMATCH[i]);
    system.writeln(dmp,DataYearIn.LAMBDA[i]);    system.writeln(dmp,DataYearIn.APH[i]);
    system.writeln(dmp,DataYearIn.PH[i]);    system.writeln(dmp,DataYearIn.APCO2[i]);
    {pf23 convert back to massfractions of the fine earth, correcting for the stones+gravel mass percentages: mfr=mass/(RHO*DLZ*(1-stonesfr-gravelfr))}
    calcdum:=DataYearIn.CALCITEA[i]/(DataYearIn.RHO[i]*DataYearIn.DLZ*(100-DataYearIn.Gravel[i]-DataYearIn.Stones[i])/100);
    gypsdum:=DataYearIn.GYPA[i]/(DataYearIn.RHO[i]*DataYearIn.DLZ*(100-DataYearIn.Gravel[i]-DataYearIn.Stones[i])/100);
    system.writeln(dmp,calcdum);    system.writeln(dmp,gypsdum);
    system.writeln(dmp,DataYearIn.ACEC[i]);    system.writeln(dmp,DataYearIn.alkalinity[i]);
    system.writeln(dmp,DataYearIn.BS[i]);    system.writeln(dmp,DataYearIn.EC[i]);
    system.writeln(dmp,DataYearIn.SAR[i]);    system.writeln(dmp,DataYearIn.ESP[i]);
    system.writeln(dmp,DataYearIn.ET[i]);    system.writeln(dmp,DataYearIn.FLUX[i]);
    system.writeln(dmp,DataYearIn.DOC[i]);    system.writeln(dmp,DataYearIn.SEA[1][i]);
    system.writeln(dmp,DataYearIn.SEA[2][i]);    system.writeln(dmp,DataYearIn.SEA[3][i]);
    system.writeln(dmp,DataYearIn.SEA[4][i]);    system.writeln(dmp,DataYearIn.SEA[5][i]);
    system.writeln(dmp,DataYearIn.SEA[6][i]);    system.writeln(dmp,DataYearIn.SEA[7][i]);
    system.writeln(dmp,DataYearIn.HCO3[i]);    system.writeln(dmp,DataYearIn.SEA[8][i]);
    system.writeln(dmp,DataYearIn.CO3[i]);    system.writeln(dmp,DataYearIn.SEA[9][i]);
    system.writeln(dmp,DataYearIn.SEA[10][i]);    system.writeln(dmp,DataYearIn.SEA[11][i]);
    system.writeln(dmp,DataYearIn.SEA[12][i]);    system.writeln(dmp,DataYearIn.SEA[13][i]);
    system.writeln(dmp,DataYearIn.SEA[14][i]);    system.writeln(dmp,DataYearIn.SEA[15][i]);
    system.writeln(dmp,DataYearIn.SEA[16][i]);    system.writeln(dmp,DataYearIn.SEA[17][i]);
    system.writeln(dmp,DataYearIn.SEA[18][i]);    system.writeln(dmp,DataYearIn.SEA[19][i]);
    system.writeln(dmp,DataYearIn.SEA[20][i]);    system.writeln(dmp,DataYearIn.SEA[21][i]);
    system.writeln(dmp,DataYearIn.SEA[22][i]);    system.writeln(dmp,DataYearIn.SEA[23][i]);
    system.writeln(dmp,DataYearIn.AXSE[1][i]);    system.writeln(dmp,DataYearIn.AXSE[2][i]);
    system.writeln(dmp,DataYearIn.AXSE[3][i]);    system.writeln(dmp,DataYearIn.AXSE[4][i]);
    system.writeln(dmp,DataYearIn.SEA[10][i]);    system.writeln(dmp,DataYearIn.SEA[12][i]);
    system.writeln(dmp,DataYearIn.RothC.DPMpool[i]);    system.writeln(dmp,DataYearIn.RothC.RPMpool[i]);
    system.writeln(dmp,DataYearIn.RothC.BIOpool[i]);    system.writeln(dmp,DataYearIn.RothC.HUMpool[i]);
    system.writeln(dmp,DataYearIn.RothC.IOMpool[i]);    system.writeln(dmp,DataYearIn.RothC.CO2pool[i]);
    system.writeln(dmp,DataYearIn.RothC.accTSMD[i]);    system.writeln(dmp,DataYearIn.RothC.pCO2[i]);
    system.writeln(dmp,DataYearIn.ChemDPMpool[1][i]);    system.writeln(dmp,DataYearIn.ChemRPMpool[1][i]);
    system.writeln(dmp,DataYearIn.ChemBIOpool[1][i]);    system.writeln(dmp,DataYearIn.ChemHUMpool[1][i]);
    system.writeln(dmp,DataYearIn.ChemIOMpool[1][i]);    system.writeln(dmp,DataYearIn.ChemCO2pool[1][i]);
    system.writeln(dmp,DataYearIn.ChemDPMpool[2][i]);    system.writeln(dmp,DataYearIn.ChemRPMpool[2][i]);
    system.writeln(dmp,DataYearIn.ChemBIOpool[2][i]);    system.writeln(dmp,DataYearIn.ChemHUMpool[2][i]);
    system.writeln(dmp,DataYearIn.ChemIOMpool[2][i]);    system.writeln(dmp,DataYearIn.ChemCO2pool[2][i]);
    system.writeln(dmp,DataYearIn.ChemDPMpool[3][i]);    system.writeln(dmp,DataYearIn.ChemRPMpool[3][i]);
    system.writeln(dmp,DataYearIn.ChemBIOpool[3][i]);    system.writeln(dmp,DataYearIn.ChemHUMpool[3][i]);
    system.writeln(dmp,DataYearIn.ChemIOMpool[3][i]);    system.writeln(dmp,DataYearIn.ChemCO2pool[3][i]);
    system.writeln(dmp,DataYearIn.ChemDPMpool[4][i]);    system.writeln(dmp,DataYearIn.ChemRPMpool[4][i]);
    system.writeln(dmp,DataYearIn.ChemBIOpool[4][i]);    system.writeln(dmp,DataYearIn.ChemHUMpool[4][i]);
    system.writeln(dmp,DataYearIn.ChemIOMpool[4][i]);    system.writeln(dmp,DataYearIn.ChemCO2pool[4][i]);
    system.writeln(dmp,DataYearIn.ChemDPMpool[5][i]);    system.writeln(dmp,DataYearIn.ChemRPMpool[5][i]);
    system.writeln(dmp,DataYearIn.ChemBIOpool[5][i]);    system.writeln(dmp,DataYearIn.ChemHUMpool[5][i]);
    system.writeln(dmp,DataYearIn.ChemIOMpool[5][i]);    system.writeln(dmp,DataYearIn.ChemCO2pool[5][i]);
    system.writeln(dmp,DataYearIn.ChemDPMpool[6][i]);    system.writeln(dmp,DataYearIn.ChemRPMpool[6][i]);
    system.writeln(dmp,DataYearIn.ChemBIOpool[6][i]);    system.writeln(dmp,DataYearIn.ChemHUMpool[6][i]);
    system.writeln(dmp,DataYearIn.ChemIOMpool[6][i]);    system.writeln(dmp,DataYearIn.ChemCO2pool[6][i]);
    system.writeln(dmp,DataYearIn.ChemDPMpool[7][i]);    system.writeln(dmp,DataYearIn.ChemRPMpool[7][i]);
    system.writeln(dmp,DataYearIn.ChemBIOpool[7][i]);    system.writeln(dmp,DataYearIn.ChemHUMpool[7][i]);
    system.writeln(dmp,DataYearIn.ChemIOMpool[7][i]);    system.writeln(dmp,DataYearIn.ChemCO2pool[7][i]);
    system.writeln(dmp,DataYearIn.ChemDPMpool[8][i]);    system.writeln(dmp,DataYearIn.ChemRPMpool[8][i]);
    system.writeln(dmp,DataYearIn.ChemBIOpool[8][i]);    system.writeln(dmp,DataYearIn.ChemHUMpool[8][i]);
    system.writeln(dmp,DataYearIn.ChemIOMpool[8][i]);    system.writeln(dmp,DataYearIn.ChemCO2pool[8][i]);
    system.writeln(dmp,DataYearIn.ChemDPMpool[9][i]);    system.writeln(dmp,DataYearIn.ChemRPMpool[9][i]);
    system.writeln(dmp,DataYearIn.ChemBIOpool[9][i]);    system.writeln(dmp,DataYearIn.ChemHUMpool[9][i]);
    system.writeln(dmp,DataYearIn.ChemIOMpool[9][i]);    system.writeln(dmp,DataYearIn.ChemCO2pool[9][i]);
    system.writeln(dmp,DataYearIn.ChemDPMpool[10][i]);    system.writeln(dmp,DataYearIn.ChemRPMpool[10][i]);
    system.writeln(dmp,DataYearIn.ChemBIOpool[10][i]);    system.writeln(dmp,DataYearIn.ChemHUMpool[10][i]);
    system.writeln(dmp,DataYearIn.ChemIOMpool[10][i]);    system.writeln(dmp,DataYearIn.ChemCO2pool[10][i]);
    system.writeln(dmp,DataYearIn.ChemDPMpool[11][i]);    system.writeln(dmp,DataYearIn.ChemRPMpool[11][i]);
    system.writeln(dmp,DataYearIn.ChemBIOpool[11][i]);    system.writeln(dmp,DataYearIn.ChemHUMpool[11][i]);
    system.writeln(dmp,DataYearIn.ChemIOMpool[11][i]);    system.writeln(dmp,DataYearIn.ChemCO2pool[11][i]);
    system.writeln(dmp,DataYearIn.ChemDPMpool[12][i]);    system.writeln(dmp,DataYearIn.ChemRPMpool[12][i]);
    system.writeln(dmp,DataYearIn.ChemBIOpool[12][i]);    system.writeln(dmp,DataYearIn.ChemHUMpool[12][i]);
    system.writeln(dmp,DataYearIn.ChemIOMpool[12][i]);    system.writeln(dmp,DataYearIn.ChemCO2pool[12][i]);
    system.writeln(dmp,DataYearIn.ChemDPMpool[13][i]);    system.writeln(dmp,DataYearIn.ChemRPMpool[13][i]);
    system.writeln(dmp,DataYearIn.ChemBIOpool[13][i]);    system.writeln(dmp,DataYearIn.ChemHUMpool[13][i]);
    system.writeln(dmp,DataYearIn.ChemIOMpool[13][i]);    system.writeln(dmp,DataYearIn.ChemCO2pool[13][i]);
    system.writeln(dmp,DataYearIn.ChemDPMpool[14][i]);    system.writeln(dmp,DataYearIn.ChemRPMpool[14][i]);
    system.writeln(dmp,DataYearIn.ChemBIOpool[14][i]);    system.writeln(dmp,DataYearIn.ChemHUMpool[14][i]);
    system.writeln(dmp,DataYearIn.ChemIOMpool[14][i]);    system.writeln(dmp,DataYearIn.ChemCO2pool[14][i]);
    system.writeln(dmp,DataYearIn.ChemDPMpool[15][i]);    system.writeln(dmp,DataYearIn.ChemRPMpool[15][i]);
    system.writeln(dmp,DataYearIn.ChemBIOpool[15][i]);    system.writeln(dmp,DataYearIn.ChemHUMpool[15][i]);
    system.writeln(dmp,DataYearIn.ChemIOMpool[15][i]);    system.writeln(dmp,DataYearIn.ChemDPMpool[16][i]);
    system.writeln(dmp,DataYearIn.ChemRPMpool[16][i]);    system.writeln(dmp,DataYearIn.ChemBIOpool[16][i]);
    system.writeln(dmp,DataYearIn.ChemHUMpool[16][i]);    system.writeln(dmp,DataYearIn.ChemIOMpool[16][i]);
    system.writeln(dmp,DataYearIn.ChemCO2pool[16][i]);    system.writeln(dmp,DataYearIn.WE.Gibbsitepool[i]);
    system.writeln(dmp,DataYearIn.WE.WE_Capool[i]);    system.writeln(dmp,DataYearIn.WE.WE_Mgpool[i]);
    system.writeln(dmp,DataYearIn.WE.WE_Kpool[i]);    system.writeln(dmp,DataYearIn.WE.WE_Napool[i]);
  end;
  if length(crashtext)=0 then crashtext:='(none)';
  if (Chemstarted)  {crashed inside Chem unit} then
  begin
    untProcesses.PassDataOUT; txt:='';
    if (rdgrpTrace.ItemIndex=1)
    then
    begin
      for i:=(untProcesses.count10 mod 30)+2 to 30 do txt:=txt+#13#10+untProcesses.last100[i];
      for i:=1 to (untProcesses.count10 mod 30)+1 do txt:=txt+#13#10+untProcesses.last100[i];
      showmessage('Crashed inside SoilGen soil processes unit. '+#13#10+
                  'Last message (may be crash-related):'+#13#10+
                   crashtext+#13#10+#13#10+'Trace list below:'+#13#10+'...'+#13#10+txt
                   +#13#10+'Last routine='+singletracetext);
      system.writeln(fmonit,'Crashed inside SoilGen soil processes unit. '+'Last message (may be crash-related):'+
                   crashtext+'Trace list below:');
      for i:=(untProcesses.count10 mod 30)+2 to 30 do system.writeln(fmonit,untProcesses.last100[i]);
      for i:=1 to (untProcesses.count10 mod 30)+1 do system.writeln(fmonit,untProcesses.last100[i]);
      system.writeln(fmonit,'Last routine='+singletracetext);
    end
    else
    begin
      showmessage('Crashed inside SoilGen soil processes unit. '+#13#10+
                     'Last message (may be crash-related):'+#13#10+
                      crashtext+#13#10+#13#10+'Last routine='+singletracetext);
      system.writeln(fmonit,'Crashed inside SoilGen soil processes unit. '+'Last message (may be crash-related):'+
                   'Last message (may be crash-related):'+crashtext+
                   'Last routine='+singletracetext);
    end;
  end
  else
  begin
    showmessage('Crashed inside SoilGen GUI.');
    txt:='';
    if iyear>=spndtTraceStart.Value
    then begin untProcesses.TRACE:=rdgrpTrace.ItemIndex; GUItrace:=rdgrpTrace.ItemIndex; end
    else begin untProcesses.TRACE:=0; GUItrace:=0; end;
    if (GUItrace=1)
    then
    begin
      for i:=(GUIcount10 mod 30)+2 to 30 do txt:=txt+#13#10+GUIlast100[i];
      for i:=1 to (GUIcount10 mod 30)+1 do txt:=txt+#13#10+GUIlast100[i];
      showmessage('Crashed inside SoilGen GUI. '+#13#10+
                  'Last message (may be crash-related):'+#13#10+
                   crashtext+#13#10+#13#10+'Trace list below:'+#13#10+#13#10+txt
                   +#13#10+'Last routine='+singletracetext);
      system.writeln(fmonit,'Crashed inside SoilGen GUI. '+'Last message (may be crash-related):'+
                   crashtext+'Trace list below:');
      for i:=(untProcesses.count10 mod 30)+2 to 30 do system.writeln(fmonit,GUIlast100[i]);
      for i:=1 to (GUIcount10 mod 30)+1 do system.writeln(fmonit,GUIlast100[i]);
      system.writeln(fmonit,'Last routine='+singletracetext);
    end
    else
    begin
      showmessage('Crashed inside SoilGen GUI. '+#13#10+
                     'Last message (may be crash-related):'+#13#10+
                      crashtext+#13#10+#13#10+'Last routine='+singletracetext);
      system.writeln(fmonit,'Crashed inside SoilGen GUI. '+'Last message (may be crash-related):'+
                   'Last message (may be crash-related):'+crashtext+
                   'Last routine='+singletracetext);
    end;
  end;
  {DataYearOut}
  for i:=1 to dataYearIn.Nolayers do
  begin
    system.writeln(dmp,DataYearOut.CLAY[i]);    system.writeln(dmp,DataYearOut.SILT[i]);
    system.writeln(dmp,DataYearOut.parden[i]);    system.writeln(dmp,DataYearOut.porosity[i]);
    system.writeln(dmp,DataYearOut.ClayDispInd[i]);     system.writeln(dmp,DataYearOut.OC[i]);
    system.writeln(dmp,DataYearOut.AW[i]);    system.writeln(dmp,DataYearOut.W[i]);
    system.writeln(dmp,DataYearOut.AH[i]);    system.writeln(dmp,DataYearOut.RDF[i]);
    system.writeln(dmp,DataYearOut.ATEM[i]);    system.writeln(dmp,DataYearOut.TEM[i]);
    system.writeln(dmp,DataYearOut.TEMPGEM[i]);    system.writeln(dmp,DataYearOut.AAEV[i]);
    system.writeln(dmp,DataYearOut.ABCAM[i]);    system.writeln(dmp,DataYearOut.RHO[i]);
    system.writeln(dmp,DataYearOut.CMATCH[i]);    system.writeln(dmp,DataYearOut.HMATCH[i]);
    system.writeln(dmp,DataYearOut.LAMBDA[i]);    system.writeln(dmp,DataYearOut.APH[i]);
    system.writeln(dmp,DataYearOut.PH[i]);    system.writeln(dmp,DataYearOut.APCO2[i]);
    {pf23 convert back to massfractions of the fine earth, correcting for the stones+gravel mass percentages}
    calcdum:=(1+(DataYearOut.STONES[i]+DataYearout.GRAVEL[i])/100)*DataYearOut.CALCITEA[i]/(DataYearOut.RHO[i]*DataYearOut.DLZ);
    gypsdum:=(1+(DataYearOut.STONES[i]+DataYearout.GRAVEL[i])/100)*DataYearOut.GYPA[i]/(DataYearOut.RHO[i]*DataYearOut.DLZ);
    system.writeln(dmp,calcdum);    system.writeln(dmp,gypsdum);
    system.writeln(dmp,DataYearOut.ACEC[i]);    system.writeln(dmp,DataYearOut.alkalinity[i]);
    system.writeln(dmp,DataYearOut.BS[i]);    system.writeln(dmp,DataYearOut.EC[i]);
    system.writeln(dmp,DataYearOut.SAR[i]);    system.writeln(dmp,DataYearOut.ESP[i]);
    system.writeln(dmp,DataYearOut.ET[i]);    system.writeln(dmp,DataYearOut.FLUX[i]);
    system.writeln(dmp,DataYearOut.DOC[i]);    system.writeln(dmp,DataYearOut.SEA[1][i]);
    system.writeln(dmp,DataYearOut.SEA[2][i]);    system.writeln(dmp,DataYearOut.SEA[3][i]);
    system.writeln(dmp,DataYearOut.SEA[4][i]);    system.writeln(dmp,DataYearOut.SEA[5][i]);
    system.writeln(dmp,DataYearOut.SEA[6][i]);    system.writeln(dmp,DataYearOut.SEA[7][i]);
    system.writeln(dmp,DataYearOut.HCO3[i]);    system.writeln(dmp,DataYearOut.SEA[8][i]);
    system.writeln(dmp,DataYearOut.CO3[i]);    system.writeln(dmp,DataYearOut.SEA[9][i]);
    system.writeln(dmp,DataYearOut.SEA[10][i]);    system.writeln(dmp,DataYearOut.SEA[11][i]);
    system.writeln(dmp,DataYearOut.SEA[12][i]);    system.writeln(dmp,DataYearOut.SEA[13][i]);
    system.writeln(dmp,DataYearOut.SEA[14][i]);    system.writeln(dmp,DataYearOut.SEA[15][i]);
    system.writeln(dmp,DataYearOut.SEA[16][i]);    system.writeln(dmp,DataYearOut.SEA[17][i]);
    system.writeln(dmp,DataYearOut.SEA[18][i]);    system.writeln(dmp,DataYearOut.SEA[19][i]);
    system.writeln(dmp,DataYearOut.SEA[20][i]);    system.writeln(dmp,DataYearOut.SEA[21][i]);
    system.writeln(dmp,DataYearOut.SEA[22][i]);    system.writeln(dmp,DataYearOut.SEA[23][i]);
    system.writeln(dmp,DataYearOut.AXSE[1][i]);    system.writeln(dmp,DataYearOut.AXSE[2][i]);
    system.writeln(dmp,DataYearOut.AXSE[3][i]);    system.writeln(dmp,DataYearOut.AXSE[4][i]);
    system.writeln(dmp,DataYearOut.SEA[10][i]);    system.writeln(dmp,DataYearOut.SEA[12][i]);
    system.writeln(dmp,DataYearOut.RothC.DPMpool[i]);    system.writeln(dmp,DataYearOut.RothC.RPMpool[i]);
    system.writeln(dmp,DataYearOut.RothC.BIOpool[i]);    system.writeln(dmp,DataYearOut.RothC.HUMpool[i]);
    system.writeln(dmp,DataYearOut.RothC.IOMpool[i]);    system.writeln(dmp,DataYearOut.RothC.CO2pool[i]);
    system.writeln(dmp,DataYearOut.RothC.accTSMD[i]);    system.writeln(dmp,DataYearOut.RothC.pCO2[i]);
    system.writeln(dmp,DataYearOut.ChemDPMpool[1][i]);    system.writeln(dmp,DataYearOut.ChemRPMpool[1][i]);
    system.writeln(dmp,DataYearOut.ChemBIOpool[1][i]);    system.writeln(dmp,DataYearOut.ChemHUMpool[1][i]);
    system.writeln(dmp,DataYearOut.ChemIOMpool[1][i]);    system.writeln(dmp,DataYearOut.ChemCO2pool[1][i]);
    system.writeln(dmp,DataYearOut.ChemDPMpool[2][i]);    system.writeln(dmp,DataYearOut.ChemRPMpool[2][i]);
    system.writeln(dmp,DataYearOut.ChemBIOpool[2][i]);    system.writeln(dmp,DataYearOut.ChemHUMpool[2][i]);
    system.writeln(dmp,DataYearOut.ChemIOMpool[2][i]);    system.writeln(dmp,DataYearOut.ChemCO2pool[2][i]);
    system.writeln(dmp,DataYearOut.ChemDPMpool[3][i]);    system.writeln(dmp,DataYearOut.ChemRPMpool[3][i]);
    system.writeln(dmp,DataYearOut.ChemBIOpool[3][i]);    system.writeln(dmp,DataYearOut.ChemHUMpool[3][i]);
    system.writeln(dmp,DataYearOut.ChemIOMpool[3][i]);    system.writeln(dmp,DataYearOut.ChemCO2pool[3][i]);
    system.writeln(dmp,DataYearOut.ChemDPMpool[4][i]);    system.writeln(dmp,DataYearOut.ChemRPMpool[4][i]);
    system.writeln(dmp,DataYearOut.ChemBIOpool[4][i]);    system.writeln(dmp,DataYearOut.ChemHUMpool[4][i]);
    system.writeln(dmp,DataYearOut.ChemIOMpool[4][i]);    system.writeln(dmp,DataYearOut.ChemCO2pool[4][i]);
    system.writeln(dmp,DataYearOut.ChemDPMpool[5][i]);    system.writeln(dmp,DataYearOut.ChemRPMpool[5][i]);
    system.writeln(dmp,DataYearOut.ChemBIOpool[5][i]);    system.writeln(dmp,DataYearOut.ChemHUMpool[5][i]);
    system.writeln(dmp,DataYearOut.ChemIOMpool[5][i]);    system.writeln(dmp,DataYearOut.ChemCO2pool[5][i]);
    system.writeln(dmp,DataYearOut.ChemDPMpool[6][i]);    system.writeln(dmp,DataYearOut.ChemRPMpool[6][i]);
    system.writeln(dmp,DataYearOut.ChemBIOpool[6][i]);    system.writeln(dmp,DataYearOut.ChemHUMpool[6][i]);
    system.writeln(dmp,DataYearOut.ChemIOMpool[6][i]);    system.writeln(dmp,DataYearOut.ChemCO2pool[6][i]);
    system.writeln(dmp,DataYearOut.ChemDPMpool[7][i]);    system.writeln(dmp,DataYearOut.ChemRPMpool[7][i]);
    system.writeln(dmp,DataYearOut.ChemBIOpool[7][i]);    system.writeln(dmp,DataYearOut.ChemHUMpool[7][i]);
    system.writeln(dmp,DataYearOut.ChemIOMpool[7][i]);    system.writeln(dmp,DataYearOut.ChemCO2pool[7][i]);
    system.writeln(dmp,DataYearOut.ChemDPMpool[8][i]);    system.writeln(dmp,DataYearOut.ChemRPMpool[8][i]);
    system.writeln(dmp,DataYearOut.ChemBIOpool[8][i]);    system.writeln(dmp,DataYearOut.ChemHUMpool[8][i]);
    system.writeln(dmp,DataYearOut.ChemIOMpool[8][i]);    system.writeln(dmp,DataYearOut.ChemCO2pool[8][i]);
    system.writeln(dmp,DataYearOut.ChemDPMpool[9][i]);    system.writeln(dmp,DataYearOut.ChemRPMpool[9][i]);
    system.writeln(dmp,DataYearOut.ChemBIOpool[9][i]);    system.writeln(dmp,DataYearOut.ChemHUMpool[9][i]);
    system.writeln(dmp,DataYearOut.ChemIOMpool[9][i]);    system.writeln(dmp,DataYearOut.ChemCO2pool[9][i]);
    system.writeln(dmp,DataYearOut.ChemDPMpool[10][i]);    system.writeln(dmp,DataYearOut.ChemRPMpool[10][i]);
    system.writeln(dmp,DataYearOut.ChemBIOpool[10][i]);    system.writeln(dmp,DataYearOut.ChemHUMpool[10][i]);
    system.writeln(dmp,DataYearOut.ChemIOMpool[10][i]);    system.writeln(dmp,DataYearOut.ChemCO2pool[10][i]);
    system.writeln(dmp,DataYearOut.ChemDPMpool[11][i]);    system.writeln(dmp,DataYearOut.ChemRPMpool[11][i]);
    system.writeln(dmp,DataYearOut.ChemBIOpool[11][i]);    system.writeln(dmp,DataYearOut.ChemHUMpool[11][i]);
    system.writeln(dmp,DataYearOut.ChemIOMpool[11][i]);    system.writeln(dmp,DataYearOut.ChemCO2pool[11][i]);
    system.writeln(dmp,DataYearOut.ChemDPMpool[12][i]);    system.writeln(dmp,DataYearOut.ChemRPMpool[12][i]);
    system.writeln(dmp,DataYearOut.ChemBIOpool[12][i]);    system.writeln(dmp,DataYearOut.ChemHUMpool[12][i]);
    system.writeln(dmp,DataYearOut.ChemIOMpool[12][i]);    system.writeln(dmp,DataYearOut.ChemCO2pool[12][i]);
    system.writeln(dmp,DataYearOut.ChemDPMpool[13][i]);    system.writeln(dmp,DataYearOut.ChemRPMpool[13][i]);
    system.writeln(dmp,DataYearOut.ChemBIOpool[13][i]);    system.writeln(dmp,DataYearOut.ChemHUMpool[13][i]);
    system.writeln(dmp,DataYearOut.ChemIOMpool[13][i]);    system.writeln(dmp,DataYearOut.ChemCO2pool[13][i]);
    system.writeln(dmp,DataYearOut.ChemDPMpool[14][i]);    system.writeln(dmp,DataYearOut.ChemRPMpool[14][i]);
    system.writeln(dmp,DataYearOut.ChemBIOpool[14][i]);    system.writeln(dmp,DataYearOut.ChemHUMpool[14][i]);
    system.writeln(dmp,DataYearOut.ChemIOMpool[14][i]);    system.writeln(dmp,DataYearOut.ChemCO2pool[14][i]);
    system.writeln(dmp,DataYearOut.ChemDPMpool[15][i]);    system.writeln(dmp,DataYearOut.ChemRPMpool[15][i]);
    system.writeln(dmp,DataYearOut.ChemBIOpool[15][i]);    system.writeln(dmp,DataYearOut.ChemHUMpool[15][i]);
    system.writeln(dmp,DataYearOut.ChemIOMpool[15][i]);    system.writeln(dmp,DataYearOut.ChemDPMpool[16][i]);
    system.writeln(dmp,DataYearOut.ChemRPMpool[16][i]);    system.writeln(dmp,DataYearOut.ChemBIOpool[16][i]);
    system.writeln(dmp,DataYearOut.ChemHUMpool[16][i]);    system.writeln(dmp,DataYearOut.ChemIOMpool[16][i]);
    system.writeln(dmp,DataYearOut.ChemCO2pool[16][i]);    system.writeln(dmp,DataYearOut.WE.Gibbsitepool[i]);
    system.writeln(dmp,DataYearOut.WE.WE_Capool[i]);    system.writeln(dmp,DataYearOut.WE.WE_Mgpool[i]);
    system.writeln(dmp,DataYearOut.WE.WE_Kpool[i]);    system.writeln(dmp,DataYearOut.WE.WE_Napool[i]);
  end;
  closefile(dmp);
  untProcesses.TRACE:=0; GUItrace:=0;
end;

procedure TfrmMAIN.readdump;
{reads back "dump.txt"; only used in untMemDump}
var dmp:textfile;  i:integer; txt:string;
begin
  assignfile(dmp,'dump.txt');
  system.Reset(dmp);
  System.readln(dmp,dataYearIn.NoLayers,txt); dataInit.NoLayers:=dataYearIn.NoLayers; datayearout.NoLayers:=dataYearIn.NoLayers;
  untMemDump.frmMemDump.Caption:='Contents of variables at time of crash: '+txt;
  {DataInit}
  for i:=1 to DataInit.Nolayers do
  begin
    system.readln(dmp,DataInit.CLAY[i]);    system.readln(dmp,DataInit.SILT[i]);
    system.readln(dmp,DataInit.parden[i]);    system.readln(dmp,DataInit.porosity[i]);
    system.readln(dmp,DataInit.ClayDispInd[i]);     system.readln(dmp,DataInit.OC[i]);
    system.readln(dmp,DataInit.AW[i]);    system.readln(dmp,DataInit.W[i]);
    system.readln(dmp,DataInit.AH[i]);    system.readln(dmp,DataInit.RDF[i]);
    system.readln(dmp,DataInit.ATEM[i]);    system.readln(dmp,DataInit.TEM[i]);
    system.readln(dmp,DataInit.TEMPGEM[i]);    system.readln(dmp,DataInit.AAEV[i]);
    system.readln(dmp,DataInit.ABCAM[i]);    system.readln(dmp,DataInit.RHO[i]);
    system.readln(dmp,DataInit.CMATCH[i]);    system.readln(dmp,DataInit.HMATCH[i]);
    system.readln(dmp,DataInit.LAMBDA[i]);    system.readln(dmp,DataInit.APH[i]);
    system.readln(dmp,DataInit.PH[i]);    system.readln(dmp,DataInit.APCO2[i]);
    system.readln(dmp,DataInit.CALCITEA[i]);    system.readln(dmp,DataInit.GYPA[i]);
    {note that calcitea and gypa are here read as mass fractions and NOT converted.
     this is allowed because the values will NEVER be used for model calculations!}
    system.readln(dmp,DataInit.ACEC[i]);    system.readln(dmp,DataInit.alkalinity[i]);
    system.readln(dmp,DataInit.BS[i]);    system.readln(dmp,DataInit.EC[i]);
    system.readln(dmp,DataInit.SAR[i]);    system.readln(dmp,DataInit.ESP[i]);
    system.readln(dmp,DataInit.ET[i]);    system.readln(dmp,DataInit.FLUX[i]);
    system.readln(dmp,DataInit.DOC[i]);    system.readln(dmp,DataInit.SEA[1][i]);
    system.readln(dmp,DataInit.SEA[2][i]);    system.readln(dmp,DataInit.SEA[3][i]);
    system.readln(dmp,DataInit.SEA[4][i]);    system.readln(dmp,DataInit.SEA[5][i]);
    system.readln(dmp,DataInit.SEA[6][i]);    system.readln(dmp,DataInit.SEA[7][i]);
    system.readln(dmp,DataInit.HCO3[i]);    system.readln(dmp,DataInit.SEA[8][i]);
    system.readln(dmp,DataInit.CO3[i]);    system.readln(dmp,DataInit.SEA[9][i]);
    system.readln(dmp,DataInit.SEA[10][i]);    system.readln(dmp,DataInit.SEA[11][i]);
    system.readln(dmp,DataInit.SEA[12][i]);    system.readln(dmp,DataInit.SEA[13][i]);
    system.readln(dmp,DataInit.SEA[14][i]);    system.readln(dmp,DataInit.SEA[15][i]);
    system.readln(dmp,DataInit.SEA[16][i]);    system.readln(dmp,DataInit.SEA[17][i]);
    system.readln(dmp,DataInit.SEA[18][i]);    system.readln(dmp,DataInit.SEA[19][i]);
    system.readln(dmp,DataInit.SEA[20][i]);    system.readln(dmp,DataInit.SEA[21][i]);
    system.readln(dmp,DataInit.SEA[22][i]);    system.readln(dmp,DataInit.SEA[23][i]);
    system.readln(dmp,DataInit.AXSE[1][i]);    system.readln(dmp,DataInit.AXSE[2][i]);
    system.readln(dmp,DataInit.AXSE[3][i]);    system.readln(dmp,DataInit.AXSE[4][i]);
    system.readln(dmp,DataInit.SEA[10][i]);    system.readln(dmp,DataInit.SEA[12][i]);
    system.readln(dmp,DataInit.RothC.DPMpool[i]);    system.readln(dmp,DataInit.RothC.RPMpool[i]);
    system.readln(dmp,DataInit.RothC.BIOpool[i]);    system.readln(dmp,DataInit.RothC.HUMpool[i]);
    system.readln(dmp,DataInit.RothC.IOMpool[i]);    system.readln(dmp,DataInit.RothC.CO2pool[i]);
    system.readln(dmp,DataInit.RothC.accTSMD[i]);    system.readln(dmp,DataInit.RothC.pCO2[i]);
    system.readln(dmp,DataInit.ChemDPMpool[1][i]);    system.readln(dmp,DataInit.ChemRPMpool[1][i]);
    system.readln(dmp,DataInit.ChemBIOpool[1][i]);    system.readln(dmp,DataInit.ChemHUMpool[1][i]);
    system.readln(dmp,DataInit.ChemIOMpool[1][i]);    system.readln(dmp,DataInit.ChemCO2pool[1][i]);
    system.readln(dmp,DataInit.ChemDPMpool[2][i]);    system.readln(dmp,DataInit.ChemRPMpool[2][i]);
    system.readln(dmp,DataInit.ChemBIOpool[2][i]);    system.readln(dmp,DataInit.ChemHUMpool[2][i]);
    system.readln(dmp,DataInit.ChemIOMpool[2][i]);    system.readln(dmp,DataInit.ChemCO2pool[2][i]);
    system.readln(dmp,DataInit.ChemDPMpool[3][i]);    system.readln(dmp,DataInit.ChemRPMpool[3][i]);
    system.readln(dmp,DataInit.ChemBIOpool[3][i]);    system.readln(dmp,DataInit.ChemHUMpool[3][i]);
    system.readln(dmp,DataInit.ChemIOMpool[3][i]);    system.readln(dmp,DataInit.ChemCO2pool[3][i]);
    system.readln(dmp,DataInit.ChemDPMpool[4][i]);    system.readln(dmp,DataInit.ChemRPMpool[4][i]);
    system.readln(dmp,DataInit.ChemBIOpool[4][i]);    system.readln(dmp,DataInit.ChemHUMpool[4][i]);
    system.readln(dmp,DataInit.ChemIOMpool[4][i]);    system.readln(dmp,DataInit.ChemCO2pool[4][i]);
    system.readln(dmp,DataInit.ChemDPMpool[5][i]);    system.readln(dmp,DataInit.ChemRPMpool[5][i]);
    system.readln(dmp,DataInit.ChemBIOpool[5][i]);    system.readln(dmp,DataInit.ChemHUMpool[5][i]);
    system.readln(dmp,DataInit.ChemIOMpool[5][i]);    system.readln(dmp,DataInit.ChemCO2pool[5][i]);
    system.readln(dmp,DataInit.ChemDPMpool[6][i]);    system.readln(dmp,DataInit.ChemRPMpool[6][i]);
    system.readln(dmp,DataInit.ChemBIOpool[6][i]);    system.readln(dmp,DataInit.ChemHUMpool[6][i]);
    system.readln(dmp,DataInit.ChemIOMpool[6][i]);    system.readln(dmp,DataInit.ChemCO2pool[6][i]);
    system.readln(dmp,DataInit.ChemDPMpool[7][i]);    system.readln(dmp,DataInit.ChemRPMpool[7][i]);
    system.readln(dmp,DataInit.ChemBIOpool[7][i]);    system.readln(dmp,DataInit.ChemHUMpool[7][i]);
    system.readln(dmp,DataInit.ChemIOMpool[7][i]);    system.readln(dmp,DataInit.ChemCO2pool[7][i]);
    system.readln(dmp,DataInit.ChemDPMpool[8][i]);    system.readln(dmp,DataInit.ChemRPMpool[8][i]);
    system.readln(dmp,DataInit.ChemBIOpool[8][i]);    system.readln(dmp,DataInit.ChemHUMpool[8][i]);
    system.readln(dmp,DataInit.ChemIOMpool[8][i]);    system.readln(dmp,DataInit.ChemCO2pool[8][i]);
    system.readln(dmp,DataInit.ChemDPMpool[9][i]);    system.readln(dmp,DataInit.ChemRPMpool[9][i]);
    system.readln(dmp,DataInit.ChemBIOpool[9][i]);    system.readln(dmp,DataInit.ChemHUMpool[9][i]);
    system.readln(dmp,DataInit.ChemIOMpool[9][i]);    system.readln(dmp,DataInit.ChemCO2pool[9][i]);
    system.readln(dmp,DataInit.ChemDPMpool[10][i]);    system.readln(dmp,DataInit.ChemRPMpool[10][i]);
    system.readln(dmp,DataInit.ChemBIOpool[10][i]);    system.readln(dmp,DataInit.ChemHUMpool[10][i]);
    system.readln(dmp,DataInit.ChemIOMpool[10][i]);    system.readln(dmp,DataInit.ChemCO2pool[10][i]);
    system.readln(dmp,DataInit.ChemDPMpool[11][i]);    system.readln(dmp,DataInit.ChemRPMpool[11][i]);
    system.readln(dmp,DataInit.ChemBIOpool[11][i]);    system.readln(dmp,DataInit.ChemHUMpool[11][i]);
    system.readln(dmp,DataInit.ChemIOMpool[11][i]);    system.readln(dmp,DataInit.ChemCO2pool[11][i]);
    system.readln(dmp,DataInit.ChemDPMpool[12][i]);    system.readln(dmp,DataInit.ChemRPMpool[12][i]);
    system.readln(dmp,DataInit.ChemBIOpool[12][i]);    system.readln(dmp,DataInit.ChemHUMpool[12][i]);
    system.readln(dmp,DataInit.ChemIOMpool[12][i]);    system.readln(dmp,DataInit.ChemCO2pool[12][i]);
    system.readln(dmp,DataInit.ChemDPMpool[13][i]);    system.readln(dmp,DataInit.ChemRPMpool[13][i]);
    system.readln(dmp,DataInit.ChemBIOpool[13][i]);    system.readln(dmp,DataInit.ChemHUMpool[13][i]);
    system.readln(dmp,DataInit.ChemIOMpool[13][i]);    system.readln(dmp,DataInit.ChemCO2pool[13][i]);
    system.readln(dmp,DataInit.ChemDPMpool[14][i]);    system.readln(dmp,DataInit.ChemRPMpool[14][i]);
    system.readln(dmp,DataInit.ChemBIOpool[14][i]);    system.readln(dmp,DataInit.ChemHUMpool[14][i]);
    system.readln(dmp,DataInit.ChemIOMpool[14][i]);    system.readln(dmp,DataInit.ChemCO2pool[14][i]);
    system.readln(dmp,DataInit.ChemDPMpool[15][i]);    system.readln(dmp,DataInit.ChemRPMpool[15][i]);
    system.readln(dmp,DataInit.ChemBIOpool[15][i]);    system.readln(dmp,DataInit.ChemHUMpool[15][i]);
    system.readln(dmp,DataInit.ChemIOMpool[15][i]);    system.readln(dmp,DataInit.ChemDPMpool[16][i]);
    system.readln(dmp,DataInit.ChemRPMpool[16][i]);    system.readln(dmp,DataInit.ChemBIOpool[16][i]);
    system.readln(dmp,DataInit.ChemHUMpool[16][i]);    system.readln(dmp,DataInit.ChemIOMpool[16][i]);
    system.readln(dmp,DataInit.ChemCO2pool[16][i]);    system.readln(dmp,DataInit.WE.Gibbsitepool[i]);
    system.readln(dmp,DataInit.WE.WE_Capool[i]);    system.readln(dmp,DataInit.WE.WE_Mgpool[i]);
    system.readln(dmp,DataInit.WE.WE_Kpool[i]);    system.readln(dmp,DataInit.WE.WE_Napool[i]);
  end;
  {DataYearIn}
  for i:=1 to dataYearIn.Nolayers do
  begin
    system.readln(dmp,DataYearIn.CLAY[i]);    system.readln(dmp,DataYearIn.SILT[i]);
    system.readln(dmp,DataYearIn.parden[i]);    system.readln(dmp,DataYearIn.porosity[i]);
    system.readln(dmp,DataYearIn.ClayDispInd[i]);     system.readln(dmp,DataYearIn.OC[i]);
    system.readln(dmp,DataYearIn.AW[i]);    system.readln(dmp,DataYearIn.W[i]);
    system.readln(dmp,DataYearIn.AH[i]);    system.readln(dmp,DataYearIn.RDF[i]);
    system.readln(dmp,DataYearIn.ATEM[i]);    system.readln(dmp,DataYearIn.TEM[i]);
    system.readln(dmp,DataYearIn.TEMPGEM[i]);    system.readln(dmp,DataYearIn.AAEV[i]);
    system.readln(dmp,DataYearIn.ABCAM[i]);    system.readln(dmp,DataYearIn.RHO[i]);
    system.readln(dmp,DataYearIn.CMATCH[i]);    system.readln(dmp,DataYearIn.HMATCH[i]);
    system.readln(dmp,DataYearIn.LAMBDA[i]);    system.readln(dmp,DataYearIn.APH[i]);
    system.readln(dmp,DataYearIn.PH[i]);    system.readln(dmp,DataYearIn.APCO2[i]);
    system.readln(dmp,DataYearIn.CALCITEA[i]);    system.readln(dmp,DataYearIn.GYPA[i]);
    system.readln(dmp,DataYearIn.ACEC[i]);    system.readln(dmp,DataYearIn.alkalinity[i]);
    system.readln(dmp,DataYearIn.BS[i]);    system.readln(dmp,DataYearIn.EC[i]);
    system.readln(dmp,DataYearIn.SAR[i]);    system.readln(dmp,DataYearIn.ESP[i]);
    system.readln(dmp,DataYearIn.ET[i]);    system.readln(dmp,DataYearIn.FLUX[i]);
    system.readln(dmp,DataYearIn.DOC[i]);    system.readln(dmp,DataYearIn.SEA[1][i]);
    system.readln(dmp,DataYearIn.SEA[2][i]);    system.readln(dmp,DataYearIn.SEA[3][i]);
    system.readln(dmp,DataYearIn.SEA[4][i]);    system.readln(dmp,DataYearIn.SEA[5][i]);
    system.readln(dmp,DataYearIn.SEA[6][i]);    system.readln(dmp,DataYearIn.SEA[7][i]);
    system.readln(dmp,DataYearIn.HCO3[i]);    system.readln(dmp,DataYearIn.SEA[8][i]);
    system.readln(dmp,DataYearIn.CO3[i]);    system.readln(dmp,DataYearIn.SEA[9][i]);
    system.readln(dmp,DataYearIn.SEA[10][i]);    system.readln(dmp,DataYearIn.SEA[11][i]);
    system.readln(dmp,DataYearIn.SEA[12][i]);    system.readln(dmp,DataYearIn.SEA[13][i]);
    system.readln(dmp,DataYearIn.SEA[14][i]);    system.readln(dmp,DataYearIn.SEA[15][i]);
    system.readln(dmp,DataYearIn.SEA[16][i]);    system.readln(dmp,DataYearIn.SEA[17][i]);
    system.readln(dmp,DataYearIn.SEA[18][i]);    system.readln(dmp,DataYearIn.SEA[19][i]);
    system.readln(dmp,DataYearIn.SEA[20][i]);    system.readln(dmp,DataYearIn.SEA[21][i]);
    system.readln(dmp,DataYearIn.SEA[22][i]);    system.readln(dmp,DataYearIn.SEA[23][i]);
    system.readln(dmp,DataYearIn.AXSE[1][i]);    system.readln(dmp,DataYearIn.AXSE[2][i]);
    system.readln(dmp,DataYearIn.AXSE[3][i]);    system.readln(dmp,DataYearIn.AXSE[4][i]);
    system.readln(dmp,DataYearIn.SEA[10][i]);    system.readln(dmp,DataYearIn.SEA[12][i]);
    system.readln(dmp,DataYearIn.RothC.DPMpool[i]);    system.readln(dmp,DataYearIn.RothC.RPMpool[i]);
    system.readln(dmp,DataYearIn.RothC.BIOpool[i]);    system.readln(dmp,DataYearIn.RothC.HUMpool[i]);
    system.readln(dmp,DataYearIn.RothC.IOMpool[i]);    system.readln(dmp,DataYearIn.RothC.CO2pool[i]);
    system.readln(dmp,DataYearIn.RothC.accTSMD[i]);    system.readln(dmp,DataYearIn.RothC.pCO2[i]);
    system.readln(dmp,DataYearIn.ChemDPMpool[1][i]);    system.readln(dmp,DataYearIn.ChemRPMpool[1][i]);
    system.readln(dmp,DataYearIn.ChemBIOpool[1][i]);    system.readln(dmp,DataYearIn.ChemHUMpool[1][i]);
    system.readln(dmp,DataYearIn.ChemIOMpool[1][i]);    system.readln(dmp,DataYearIn.ChemCO2pool[1][i]);
    system.readln(dmp,DataYearIn.ChemDPMpool[2][i]);    system.readln(dmp,DataYearIn.ChemRPMpool[2][i]);
    system.readln(dmp,DataYearIn.ChemBIOpool[2][i]);    system.readln(dmp,DataYearIn.ChemHUMpool[2][i]);
    system.readln(dmp,DataYearIn.ChemIOMpool[2][i]);    system.readln(dmp,DataYearIn.ChemCO2pool[2][i]);
    system.readln(dmp,DataYearIn.ChemDPMpool[3][i]);    system.readln(dmp,DataYearIn.ChemRPMpool[3][i]);
    system.readln(dmp,DataYearIn.ChemBIOpool[3][i]);    system.readln(dmp,DataYearIn.ChemHUMpool[3][i]);
    system.readln(dmp,DataYearIn.ChemIOMpool[3][i]);    system.readln(dmp,DataYearIn.ChemCO2pool[3][i]);
    system.readln(dmp,DataYearIn.ChemDPMpool[4][i]);    system.readln(dmp,DataYearIn.ChemRPMpool[4][i]);
    system.readln(dmp,DataYearIn.ChemBIOpool[4][i]);    system.readln(dmp,DataYearIn.ChemHUMpool[4][i]);
    system.readln(dmp,DataYearIn.ChemIOMpool[4][i]);    system.readln(dmp,DataYearIn.ChemCO2pool[4][i]);
    system.readln(dmp,DataYearIn.ChemDPMpool[5][i]);    system.readln(dmp,DataYearIn.ChemRPMpool[5][i]);
    system.readln(dmp,DataYearIn.ChemBIOpool[5][i]);    system.readln(dmp,DataYearIn.ChemHUMpool[5][i]);
    system.readln(dmp,DataYearIn.ChemIOMpool[5][i]);    system.readln(dmp,DataYearIn.ChemCO2pool[5][i]);
    system.readln(dmp,DataYearIn.ChemDPMpool[6][i]);    system.readln(dmp,DataYearIn.ChemRPMpool[6][i]);
    system.readln(dmp,DataYearIn.ChemBIOpool[6][i]);    system.readln(dmp,DataYearIn.ChemHUMpool[6][i]);
    system.readln(dmp,DataYearIn.ChemIOMpool[6][i]);    system.readln(dmp,DataYearIn.ChemCO2pool[6][i]);
    system.readln(dmp,DataYearIn.ChemDPMpool[7][i]);    system.readln(dmp,DataYearIn.ChemRPMpool[7][i]);
    system.readln(dmp,DataYearIn.ChemBIOpool[7][i]);    system.readln(dmp,DataYearIn.ChemHUMpool[7][i]);
    system.readln(dmp,DataYearIn.ChemIOMpool[7][i]);    system.readln(dmp,DataYearIn.ChemCO2pool[7][i]);
    system.readln(dmp,DataYearIn.ChemDPMpool[8][i]);    system.readln(dmp,DataYearIn.ChemRPMpool[8][i]);
    system.readln(dmp,DataYearIn.ChemBIOpool[8][i]);    system.readln(dmp,DataYearIn.ChemHUMpool[8][i]);
    system.readln(dmp,DataYearIn.ChemIOMpool[8][i]);    system.readln(dmp,DataYearIn.ChemCO2pool[8][i]);
    system.readln(dmp,DataYearIn.ChemDPMpool[9][i]);    system.readln(dmp,DataYearIn.ChemRPMpool[9][i]);
    system.readln(dmp,DataYearIn.ChemBIOpool[9][i]);    system.readln(dmp,DataYearIn.ChemHUMpool[9][i]);
    system.readln(dmp,DataYearIn.ChemIOMpool[9][i]);    system.readln(dmp,DataYearIn.ChemCO2pool[9][i]);
    system.readln(dmp,DataYearIn.ChemDPMpool[10][i]);    system.readln(dmp,DataYearIn.ChemRPMpool[10][i]);
    system.readln(dmp,DataYearIn.ChemBIOpool[10][i]);    system.readln(dmp,DataYearIn.ChemHUMpool[10][i]);
    system.readln(dmp,DataYearIn.ChemIOMpool[10][i]);    system.readln(dmp,DataYearIn.ChemCO2pool[10][i]);
    system.readln(dmp,DataYearIn.ChemDPMpool[11][i]);    system.readln(dmp,DataYearIn.ChemRPMpool[11][i]);
    system.readln(dmp,DataYearIn.ChemBIOpool[11][i]);    system.readln(dmp,DataYearIn.ChemHUMpool[11][i]);
    system.readln(dmp,DataYearIn.ChemIOMpool[11][i]);    system.readln(dmp,DataYearIn.ChemCO2pool[11][i]);
    system.readln(dmp,DataYearIn.ChemDPMpool[12][i]);    system.readln(dmp,DataYearIn.ChemRPMpool[12][i]);
    system.readln(dmp,DataYearIn.ChemBIOpool[12][i]);    system.readln(dmp,DataYearIn.ChemHUMpool[12][i]);
    system.readln(dmp,DataYearIn.ChemIOMpool[12][i]);    system.readln(dmp,DataYearIn.ChemCO2pool[12][i]);
    system.readln(dmp,DataYearIn.ChemDPMpool[13][i]);    system.readln(dmp,DataYearIn.ChemRPMpool[13][i]);
    system.readln(dmp,DataYearIn.ChemBIOpool[13][i]);    system.readln(dmp,DataYearIn.ChemHUMpool[13][i]);
    system.readln(dmp,DataYearIn.ChemIOMpool[13][i]);    system.readln(dmp,DataYearIn.ChemCO2pool[13][i]);
    system.readln(dmp,DataYearIn.ChemDPMpool[14][i]);    system.readln(dmp,DataYearIn.ChemRPMpool[14][i]);
    system.readln(dmp,DataYearIn.ChemBIOpool[14][i]);    system.readln(dmp,DataYearIn.ChemHUMpool[14][i]);
    system.readln(dmp,DataYearIn.ChemIOMpool[14][i]);    system.readln(dmp,DataYearIn.ChemCO2pool[14][i]);
    system.readln(dmp,DataYearIn.ChemDPMpool[15][i]);    system.readln(dmp,DataYearIn.ChemRPMpool[15][i]);
    system.readln(dmp,DataYearIn.ChemBIOpool[15][i]);    system.readln(dmp,DataYearIn.ChemHUMpool[15][i]);
    system.readln(dmp,DataYearIn.ChemIOMpool[15][i]);    system.readln(dmp,DataYearIn.ChemDPMpool[16][i]);
    system.readln(dmp,DataYearIn.ChemRPMpool[16][i]);    system.readln(dmp,DataYearIn.ChemBIOpool[16][i]);
    system.readln(dmp,DataYearIn.ChemHUMpool[16][i]);    system.readln(dmp,DataYearIn.ChemIOMpool[16][i]);
    system.readln(dmp,DataYearIn.ChemCO2pool[16][i]);    system.readln(dmp,DataYearIn.WE.Gibbsitepool[i]);
    system.readln(dmp,DataYearIn.WE.WE_Capool[i]);    system.readln(dmp,DataYearIn.WE.WE_Mgpool[i]);
    system.readln(dmp,DataYearIn.WE.WE_Kpool[i]);    system.readln(dmp,DataYearIn.WE.WE_Napool[i]);
  end;
  {DataYearOut}
  for i:=1 to dataYearIn.Nolayers do
  begin
    system.readln(dmp,DataYearOut.CLAY[i]);    system.readln(dmp,DataYearOut.SILT[i]);
    system.readln(dmp,DataYearOut.parden[i]);    system.readln(dmp,DataYearOut.porosity[i]);
    system.readln(dmp,DataYearOut.ClayDispInd[i]);     system.readln(dmp,DataYearOut.OC[i]);
    system.readln(dmp,DataYearOut.AW[i]);    system.readln(dmp,DataYearOut.W[i]);
    system.readln(dmp,DataYearOut.AH[i]);    system.readln(dmp,DataYearOut.RDF[i]);
    system.readln(dmp,DataYearOut.ATEM[i]);    system.readln(dmp,DataYearOut.TEM[i]);
    system.readln(dmp,DataYearOut.TEMPGEM[i]);    system.readln(dmp,DataYearOut.AAEV[i]);
    system.readln(dmp,DataYearOut.ABCAM[i]);    system.readln(dmp,DataYearOut.RHO[i]);
    system.readln(dmp,DataYearOut.CMATCH[i]);    system.readln(dmp,DataYearOut.HMATCH[i]);
    system.readln(dmp,DataYearOut.LAMBDA[i]);    system.readln(dmp,DataYearOut.APH[i]);
    system.readln(dmp,DataYearOut.PH[i]);    system.readln(dmp,DataYearOut.APCO2[i]);
    system.readln(dmp,DataYearOut.CALCITEA[i]);    system.readln(dmp,DataYearOut.GYPA[i]);
    system.readln(dmp,DataYearOut.ACEC[i]);    system.readln(dmp,DataYearOut.alkalinity[i]);
    system.readln(dmp,DataYearOut.BS[i]);    system.readln(dmp,DataYearOut.EC[i]);
    system.readln(dmp,DataYearOut.SAR[i]);    system.readln(dmp,DataYearOut.ESP[i]);
    system.readln(dmp,DataYearOut.ET[i]);    system.readln(dmp,DataYearOut.FLUX[i]);
    system.readln(dmp,DataYearOut.DOC[i]);    system.readln(dmp,DataYearOut.SEA[1][i]);
    system.readln(dmp,DataYearOut.SEA[2][i]);    system.readln(dmp,DataYearOut.SEA[3][i]);
    system.readln(dmp,DataYearOut.SEA[4][i]);    system.readln(dmp,DataYearOut.SEA[5][i]);
    system.readln(dmp,DataYearOut.SEA[6][i]);    system.readln(dmp,DataYearOut.SEA[7][i]);
    system.readln(dmp,DataYearOut.HCO3[i]);    system.readln(dmp,DataYearOut.SEA[8][i]);
    system.readln(dmp,DataYearOut.CO3[i]);    system.readln(dmp,DataYearOut.SEA[9][i]);
    system.readln(dmp,DataYearOut.SEA[10][i]);    system.readln(dmp,DataYearOut.SEA[11][i]);
    system.readln(dmp,DataYearOut.SEA[12][i]);    system.readln(dmp,DataYearOut.SEA[13][i]);
    system.readln(dmp,DataYearOut.SEA[14][i]);    system.readln(dmp,DataYearOut.SEA[15][i]);
    system.readln(dmp,DataYearOut.SEA[16][i]);    system.readln(dmp,DataYearOut.SEA[17][i]);
    system.readln(dmp,DataYearOut.SEA[18][i]);    system.readln(dmp,DataYearOut.SEA[19][i]);
    system.readln(dmp,DataYearOut.SEA[20][i]);    system.readln(dmp,DataYearOut.SEA[21][i]);
    system.readln(dmp,DataYearOut.SEA[22][i]);    system.readln(dmp,DataYearOut.SEA[23][i]);
    system.readln(dmp,DataYearOut.AXSE[1][i]);    system.readln(dmp,DataYearOut.AXSE[2][i]);
    system.readln(dmp,DataYearOut.AXSE[3][i]);    system.readln(dmp,DataYearOut.AXSE[4][i]);
    system.readln(dmp,DataYearOut.SEA[10][i]);    system.readln(dmp,DataYearOut.SEA[12][i]);
    system.readln(dmp,DataYearOut.RothC.DPMpool[i]);    system.readln(dmp,DataYearOut.RothC.RPMpool[i]);
    system.readln(dmp,DataYearOut.RothC.BIOpool[i]);    system.readln(dmp,DataYearOut.RothC.HUMpool[i]);
    system.readln(dmp,DataYearOut.RothC.IOMpool[i]);    system.readln(dmp,DataYearOut.RothC.CO2pool[i]);
    system.readln(dmp,DataYearOut.RothC.accTSMD[i]);    system.readln(dmp,DataYearOut.RothC.pCO2[i]);
    system.readln(dmp,DataYearOut.ChemDPMpool[1][i]);    system.readln(dmp,DataYearOut.ChemRPMpool[1][i]);
    system.readln(dmp,DataYearOut.ChemBIOpool[1][i]);    system.readln(dmp,DataYearOut.ChemHUMpool[1][i]);
    system.readln(dmp,DataYearOut.ChemIOMpool[1][i]);    system.readln(dmp,DataYearOut.ChemCO2pool[1][i]);
    system.readln(dmp,DataYearOut.ChemDPMpool[2][i]);    system.readln(dmp,DataYearOut.ChemRPMpool[2][i]);
    system.readln(dmp,DataYearOut.ChemBIOpool[2][i]);    system.readln(dmp,DataYearOut.ChemHUMpool[2][i]);
    system.readln(dmp,DataYearOut.ChemIOMpool[2][i]);    system.readln(dmp,DataYearOut.ChemCO2pool[2][i]);
    system.readln(dmp,DataYearOut.ChemDPMpool[3][i]);    system.readln(dmp,DataYearOut.ChemRPMpool[3][i]);
    system.readln(dmp,DataYearOut.ChemBIOpool[3][i]);    system.readln(dmp,DataYearOut.ChemHUMpool[3][i]);
    system.readln(dmp,DataYearOut.ChemIOMpool[3][i]);    system.readln(dmp,DataYearOut.ChemCO2pool[3][i]);
    system.readln(dmp,DataYearOut.ChemDPMpool[4][i]);    system.readln(dmp,DataYearOut.ChemRPMpool[4][i]);
    system.readln(dmp,DataYearOut.ChemBIOpool[4][i]);    system.readln(dmp,DataYearOut.ChemHUMpool[4][i]);
    system.readln(dmp,DataYearOut.ChemIOMpool[4][i]);    system.readln(dmp,DataYearOut.ChemCO2pool[4][i]);
    system.readln(dmp,DataYearOut.ChemDPMpool[5][i]);    system.readln(dmp,DataYearOut.ChemRPMpool[5][i]);
    system.readln(dmp,DataYearOut.ChemBIOpool[5][i]);    system.readln(dmp,DataYearOut.ChemHUMpool[5][i]);
    system.readln(dmp,DataYearOut.ChemIOMpool[5][i]);    system.readln(dmp,DataYearOut.ChemCO2pool[5][i]);
    system.readln(dmp,DataYearOut.ChemDPMpool[6][i]);    system.readln(dmp,DataYearOut.ChemRPMpool[6][i]);
    system.readln(dmp,DataYearOut.ChemBIOpool[6][i]);    system.readln(dmp,DataYearOut.ChemHUMpool[6][i]);
    system.readln(dmp,DataYearOut.ChemIOMpool[6][i]);    system.readln(dmp,DataYearOut.ChemCO2pool[6][i]);
    system.readln(dmp,DataYearOut.ChemDPMpool[7][i]);    system.readln(dmp,DataYearOut.ChemRPMpool[7][i]);
    system.readln(dmp,DataYearOut.ChemBIOpool[7][i]);    system.readln(dmp,DataYearOut.ChemHUMpool[7][i]);
    system.readln(dmp,DataYearOut.ChemIOMpool[7][i]);    system.readln(dmp,DataYearOut.ChemCO2pool[7][i]);
    system.readln(dmp,DataYearOut.ChemDPMpool[8][i]);    system.readln(dmp,DataYearOut.ChemRPMpool[8][i]);
    system.readln(dmp,DataYearOut.ChemBIOpool[8][i]);    system.readln(dmp,DataYearOut.ChemHUMpool[8][i]);
    system.readln(dmp,DataYearOut.ChemIOMpool[8][i]);    system.readln(dmp,DataYearOut.ChemCO2pool[8][i]);
    system.readln(dmp,DataYearOut.ChemDPMpool[9][i]);    system.readln(dmp,DataYearOut.ChemRPMpool[9][i]);
    system.readln(dmp,DataYearOut.ChemBIOpool[9][i]);    system.readln(dmp,DataYearOut.ChemHUMpool[9][i]);
    system.readln(dmp,DataYearOut.ChemIOMpool[9][i]);    system.readln(dmp,DataYearOut.ChemCO2pool[9][i]);
    system.readln(dmp,DataYearOut.ChemDPMpool[10][i]);    system.readln(dmp,DataYearOut.ChemRPMpool[10][i]);
    system.readln(dmp,DataYearOut.ChemBIOpool[10][i]);    system.readln(dmp,DataYearOut.ChemHUMpool[10][i]);
    system.readln(dmp,DataYearOut.ChemIOMpool[10][i]);    system.readln(dmp,DataYearOut.ChemCO2pool[10][i]);
    system.readln(dmp,DataYearOut.ChemDPMpool[11][i]);    system.readln(dmp,DataYearOut.ChemRPMpool[11][i]);
    system.readln(dmp,DataYearOut.ChemBIOpool[11][i]);    system.readln(dmp,DataYearOut.ChemHUMpool[11][i]);
    system.readln(dmp,DataYearOut.ChemIOMpool[11][i]);    system.readln(dmp,DataYearOut.ChemCO2pool[11][i]);
    system.readln(dmp,DataYearOut.ChemDPMpool[12][i]);    system.readln(dmp,DataYearOut.ChemRPMpool[12][i]);
    system.readln(dmp,DataYearOut.ChemBIOpool[12][i]);    system.readln(dmp,DataYearOut.ChemHUMpool[12][i]);
    system.readln(dmp,DataYearOut.ChemIOMpool[12][i]);    system.readln(dmp,DataYearOut.ChemCO2pool[12][i]);
    system.readln(dmp,DataYearOut.ChemDPMpool[13][i]);    system.readln(dmp,DataYearOut.ChemRPMpool[13][i]);
    system.readln(dmp,DataYearOut.ChemBIOpool[13][i]);    system.readln(dmp,DataYearOut.ChemHUMpool[13][i]);
    system.readln(dmp,DataYearOut.ChemIOMpool[13][i]);    system.readln(dmp,DataYearOut.ChemCO2pool[13][i]);
    system.readln(dmp,DataYearOut.ChemDPMpool[14][i]);    system.readln(dmp,DataYearOut.ChemRPMpool[14][i]);
    system.readln(dmp,DataYearOut.ChemBIOpool[14][i]);    system.readln(dmp,DataYearOut.ChemHUMpool[14][i]);
    system.readln(dmp,DataYearOut.ChemIOMpool[14][i]);    system.readln(dmp,DataYearOut.ChemCO2pool[14][i]);
    system.readln(dmp,DataYearOut.ChemDPMpool[15][i]);    system.readln(dmp,DataYearOut.ChemRPMpool[15][i]);
    system.readln(dmp,DataYearOut.ChemBIOpool[15][i]);    system.readln(dmp,DataYearOut.ChemHUMpool[15][i]);
    system.readln(dmp,DataYearOut.ChemIOMpool[15][i]);    system.readln(dmp,DataYearOut.ChemDPMpool[16][i]);
    system.readln(dmp,DataYearOut.ChemRPMpool[16][i]);    system.readln(dmp,DataYearOut.ChemBIOpool[16][i]);
    system.readln(dmp,DataYearOut.ChemHUMpool[16][i]);    system.readln(dmp,DataYearOut.ChemIOMpool[16][i]);
    system.readln(dmp,DataYearOut.ChemCO2pool[16][i]);    system.readln(dmp,DataYearOut.WE.Gibbsitepool[i]);
    system.readln(dmp,DataYearOut.WE.WE_Capool[i]);    system.readln(dmp,DataYearOut.WE.WE_Mgpool[i]);
    system.readln(dmp,DataYearOut.WE.WE_Kpool[i]);    system.readln(dmp,DataYearOut.WE.WE_Napool[i]);
  end;
  closefile(dmp);
end;

procedure TfrmMAIN.RunLeachC(FileNameRunData:string);
{now obsolete: DOS-version no longer used}
{var ProgramHandle:THandle;
    fbatch:textfile;
    txt,batchprog:string;
    count:integer;}
begin    {NB createprocess contains help info for o.a. winexec}
{  batchprog:=startdir+'\exec.bat';
  system.assign(fbatch,batchprog); system.rewrite(fbatch);
  system.writeln(fbatch,'LEACHcPF<'+FileNameRunData);
  system.writeln(fbatch,'if exist title erase title');
  system.writeln(fbatch,'if exist watch.txt erase watch.txt');
  closefile(fbatch);
  copyfile(PChar(startdir+'\exec.bat'),PChar(startdir+'\watch.txt'),true);
  ProgramHandle:=winexec(PChar(batchprog),SW_MINIMIZE);

  count:=0;
  while FileExists(startdir+'\watch.txt') do
  begin
     count:=count+1; str(count,txt);
     if count = 10000
     then begin GeneralRefresh; count:=0; end;
  end;
  if ProgramHandle<=31 then
  begin
    case ProgramHandle of
    0:                    dshowmessage(currentyear+': The system was out of memory or resources');
    ERROR_BAD_FORMAT:     dshowmessage(currentyear+': The .EXE file was invalid (non-Win32 .EXE or error in .EXE image)');
    ERROR_FILE_NOT_FOUND:	dshowmessage(currentyear+': The specified file was not found');
    ERROR_PATH_NOT_FOUND: dshowmessage(currentyear+': The specified path was not found');
    end;
  end; }
end;

procedure TfrmMAIN.tbcntrlSHELLChange(Sender: TObject);
{identifies if another tab sheet on the main window is activated and displays it}
begin
  case tbcntrlSHELL.TabIndex of
  0: begin
       grpbxInitialRunData.Visible:=true;
       grpbxMultipleRunData.Visible:=false;
       grpbxScreenOutput.Visible:=false;
       grpbxOnStart.visible:=false;
     end;
  1: begin
       grpbxInitialRunData.Visible:=false;
       grpbxMultipleRunData.Visible:=ALLOWED;
       grpbxScreenOutput.Visible:=false; 
       grpbxOnStart.visible:=false;
       bttnRUN.enabled:=readdataOK and readdataEQOK and readdataCOK and readweatherOK;
       if (bttnRUN.enabled)and(warningsissued)
       then
       begin
        grpbxMultipleRunData.Caption:=' Multiple years run data (but: warnings on input in <Messages.txt> !)';
        grpbxMultipleRunData.Font.Color:=clRed;
        if (fileexists('Messages.txt'))and(warningsissued)
        then bttnErrorInputs.Visible:=true
        else bttnErrorInputs.Visible:=false;
       end
       else
       begin
        grpbxMultipleRunData.Caption:=' Multiple years run data';
        grpbxMultipleRunData.Font.Color:=clWindowText;
       end;
       if not chckbxAbortNOW.Visible then
       begin {when the second tab sheet is selected AND the simulations
              did NOT yet start}
              {autodetect some files and set some associated checkboxes}
         chckbxClimate.Checked:=fileexists('Climate.txt');
         chckbxClimate.Enabled:=fileexists('Climate.txt');
         btbttnClimateChange.Enabled:=true;
         chckbxBioturbation.Checked:=fileexists('BioturbINT.txt');
         chckbxBioturbation.Enabled:=fileexists('BioturbINT.txt');
         btbttnBioturbation.Enabled:=true;
         chckbxFertilization.Checked:=fileexists('Fertilization.txt');
         chckbxFertilization.Enabled:=fileexists('Fertilization.txt');
         btbttnFertilizationChange.Enabled:=true;  
         chckbxEvents.Checked:=fileexists('Events.txt');
         chckbxEvents.Enabled:=fileexists('Events.txt');
         btbttnEventsChange.Enabled:=true;
         if (chckbxSpecialClimate.Checked)and(monthlyTEMPname='') then
         {monhtly climate data are to be given as input and this has not yet
          been done (no name of file has yet been given)}
         begin
           pndlgOpenSpecialClimate.Title:='Open a monthly climate file: temperature data';
           if pndlgOpenSpecialClimate.Execute
              then monthlyTEMPname:=pndlgOpenSpecialClimate.FileName
              else monthlyTEMPname:='';
           pndlgOpenSpecialClimate.Title:='Open a monthly climate file: precipitation data';
           if pndlgOpenSpecialClimate.Execute
              then monthlyPRECname:=pndlgOpenSpecialClimate.FileName
              else monthlyPRECname:='';
           pndlgOpenSpecialClimate.Title:='Open a monthly climate file: evaporation data';
           if pndlgOpenSpecialClimate.Execute
              then monthlyEVAPname:=pndlgOpenSpecialClimate.FileName
              else monthlyEVAPname:='';
         end;
       end;
     end;
  2: begin
       grpbxInitialRunData.Visible:=false;
       grpbxMultipleRunData.Visible:=false;
       grpbxScreenOutput.Visible:=true;  
       grpbxOnStart.visible:=false;
     end;
  end;
end;

procedure TfrmMAIN.btbttnOpenDefaultLeachCClick(Sender: TObject);
{starts the selection of a file with initial soil data}
begin
  {de-activate menu item to display dump.txt. This is because it will fill
   the data fields, which may override the read input}
   showmemorydump1.Visible:=false;
   {If GUItrace=1 then wwwriteln} singletracewriteln('Entering btbttnOpenDefaultLeachCClick');
   pndlgOPEN.Filter:= 'Initial Soil status input files (*.txt)|*.txt';
   if pndlgOPEN.Execute then
   begin
      mmDefaultLeachC.Lines.LoadFromFile(pndlgOPEN.FileName);
      mmDefaultLeachC.Enabled:=true;
      InitChem;
      ReadData(pndlgOPEN.FileName);
      FirstLeachCFileName:=pndlgOPEN.FileName;
      {DataYearIn:=dataInit;} CopyRecToFrom(DataYearIn,DataInit);
      DataYearIn.oldCALCITEA:=DataYearIn.CALCITEA;
      DataYearIn.oldGYPA:=DataYearIn.GYPA;
      if (fileexists('Messages.txt'))and(warningsissued)
      then bttnErrorInputs.Visible:=true
      else bttnErrorInputs.Visible:=false;
      if ReadDataOK then
      begin
        btbttnOpenEquilibrium.Enabled:=true;
        btbttnEditSoilData.Enabled:=true;
      end;
   end;    
   {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving btbttnOpenDefaultLeachCClick');
end;

procedure TfrmMAIN.ReadData(FileNameData1:string);
{reads the soil input file}
var i,j,y,y1,y2,date3112,extradays,q:integer; txt,msg:string;
    LeachCidentified:boolean;
begin
{If GUItrace=1 then wwwriteln} singletracewriteln('Entering ReadData');
q:=0; LeachCidentified:=false;
try
defaultP:=0; defaultPE:=0;
for i:=1 to 12 do
begin defaultPtotMonthly[i]:=0; defaultPEtotMonthly[i]:=0; defaultavTEMPMonthly[i]:=0 end;
{with DataInit do}
begin
 mmHeaders.Clear;
 system.Assign(findata,FileNameData1); system.Reset(findata);
 for i:=1 to 7 do
 begin
   q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt);
   if pos('LEACHC SALINITY DATA FILE',txt)>0 then LeachCidentified:=true;
 end;
 if not(LeachCidentified) then raise EInOutError.Create('This is no LEACHC-file!');
 q:=q+1; system.readln(findata,DataInit.idate);
 q:=q+1; system.readln(findata,DataInit.day1);
 q:=q+1; system.readln(findata,DataInit.CUMT1);
 {NWEEK uitrekenen met Jday, day1 en cumt1}
 if DataInit.CUMT1<010101
 then DataInit.NWEEK:=((DataInit.CUMT1-DataInit.day1+1) div 7)+1
 else
 begin
   y1:=DataInit.day1-100*trunc(DataInit.day1/100);
   y2:=DataInit.CUMT1-100*trunc(DataInit.CUMT1/100);
   if y2=y1
   then      {begin- and end time are inside the same calender-year}
     DataInit.NWEEK:=((Julian(DataInit.idate,DataInit.CUMT1)-Julian(DataInit.idate,DataInit.day1)+1) div 7)+1
   else      {begin- and end time are not inside the same calender-year}
   begin
     if DataInit.idate=1 then date3112:=1231 else date3112:=3112;
     {first partial year}
     extradays:=1+Julian(DataInit.idate,date3112*100+y1)-Julian(DataInit.idate,DataInit.day1);
     {whole years between day1 and CUMT1}
     for y:=y1+1 to y2-1 do
       extradays:=extradays+1+Julian(DataInit.idate,date3112*100+y)-Julian(DataInit.idate,101*100+y);
     {last partial year}
     extradays:=extradays+1+Julian(DataInit.idate,DataInit.CUMT1)-Julian(DataInit.idate,(10100+y2));
     DataInit.NWEEK:=(extradays div 7)+1;
   end;
 end;
 q:=q+1; system.readln(findata,DataInit.DELM);
 q:=q+1; system.readln(findata,DataInit.NYC);
 q:=q+1; system.readln(findata,DataInit.PDEPTH);
 q:=q+1; system.readln(findata,DataInit.DLZ);
 DataInit.NoLayers:=round(DataInit.PDEPTH/DataInit.DLZ);
 q:=q+1; system.readln(findata,DataInit.BBC);
 q:=q+1; system.readln(findata,DataInit.ABBPOT);
 for i:=8 to 12 do begin q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt); end;
 q:=q+1; system.readln(findata,DataInit.IFLOW);
 q:=q+1; system.readln(findata,DataInit.THETA1);
 for i:=13 to 14 do begin q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt); end;
 q:=q+1; system.readln(findata,DataInit.IFILE);
 for i:=15 to 16 do begin q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt); end;
 q:=q+1; system.readln(findata,DataInit.UNITS);
 q:=q+1; system.readln(findata,DataInit.R54);
 q:=q+1; system.readln(findata,DataInit.PRNTOP2);
 q:=q+1; system.readln(findata,DataInit.PRINT2);
 q:=q+1; system.readln(findata,DataInit.NPRINT);
 q:=q+1; system.readln(findata,DataInit.PRNTAB);
 q:=q+1; system.readln(findata,DataInit.INITFLG);
 for i:=17 to 18 do begin q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt); end;
 q:=q+1; system.readln(findata,DataInit.NSUM);
 q:=q+1; system.readln(findata,DataInit.nd1);
 q:=q+1; system.readln(findata,DataInit.nd2);
 q:=q+1; system.readln(findata,DataInit.nd3);
 q:=q+1; system.readln(findata,DataInit.nd4);
 for i:=19 to 20 do begin q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt); end;
 q:=q+1; system.readln(findata,DataInit.FWBTC);
 for i:=21 to 26 do begin q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt); end;
 for i:=1 to DataInit.NPRINT do begin q:=q+1; system.readln(findata,DataInit.PRTIME[i],DataInit.FRAC1print[i]); end;
 for i:=27 to 38 do begin q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt); end;
 for i:=1 to DataInit.NoLayers do
 begin q:=q+1; system.read(findata,DataInit.NDUM,DataInit.CLAY[i],DataInit.SILT[i],DataInit.OC[i],
                                     DataInit.MODRET[i],DataInit.AW[i],DataInit.AH[i],
                                     DataInit.RDF[i],DataInit.ATEM[i]);
       if not(eoln(findata))
          then readln(findata,DataInit.GRAVEL[i],DataInit.STONES[i])
          else
          begin
            DataInit.GRAVEL[i]:=0; DataInit.STONES[i]:=0;
            readln(findata);
          end;
       DataInit.OC[i]:=max(1e-5,DataInit.OC[i]);
       DataInit.TEM[i]:=DataInit.ATEM[i];
 end;
 {yyy   }
 {look for the depth where in the middle of the summer the soil is still frozen}
 DataInit.unfrozen:=DataInit.PDEPTH;  {in?}
 i:=DataInit.NoLayers;
 repeat
   if DataInit.tem[i]<0 then DataInit.unfrozen:=DataInit.DLZ*i;
   i:=i-1;
 until (DataInit.tem[i]>0) or (i=1);
 q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt);
 q:=q+1; system.readln(findata,DataInit.PRIOR);
 q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt);
 q:=q+1; system.readln(findata,DataInit.RHOS2,DataInit.RHOS1,DataInit.RHOS3);
 for i:=41 to 44 do begin q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt); end;
 q:=q+1; system.readln(findata,DataInit.RHO1,DataInit.parden1);
 q:=q+1; system.readln(findata,DataInit.AEV1);
 q:=q+1; system.readln(findata,DataInit.BCAM1);
 q:=q+1; system.readln(findata,DataInit.CMAT1,DataInit.HMAT1);
 q:=q+1; system.readln(findata,DataInit.RP1);
 q:=q+1; system.readln(findata,DataInit.ALAMB1);
 q:=q+1; system.readln(findata,DataInit.ADDFC);
 q:=q+1; system.readln(findata,DataInit.ADDTH);
 for i:=45 to 50 do begin q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt); end;
 for i:=1 to DataInit.NoLayers do
 begin
   q:=q+1;
   system.readln(findata,DataInit.DUM,DataInit.AAEV[i],DataInit.ABCAM[i],DataInit.RHO[i],
                         DataInit.CMATCH[i],DataInit.HMATCH[i],DataInit.RP[i],
                         DataInit.LAMBDA[i],DataInit.fc[i],DataInit.d[i]);
   {to be decided: should the first, intial RHO be input or estimated by HYPRESRHO(clayperc,omperc:real)?
   For the moment: input. But any modification of OC and clay in editSoilData will lead to te use of the pedotransfer function!}
   if DataInit.RHO[i]>rhothreshold then DataInit.RHO[i]:=rhothreshold; {depending on the value of rhothreshold, larger values for RHO are not kept}
   if DataInit.MODRET[i]<>0
   then
   begin
     {needed to make correct estimates of conductivity: temperature, profile depth and water table depth}
     untProcesses.TEM[i]:=DataInit.ATEM[i];
     untProcesses.PDEPTH:=DataInit.PDEPTH;
     untProcesses.WaterTableDepth:=999; {very deep}
     untProcesses.estimate_KatH(i+1,DataInit.clay[i],DataInit.silt[i],DataInit.rho[i],
                              DataInit.oc[i],DataInit.DUM,DataInit.CMATCH[i],DataInit.HMATCH[i]);
   end;
   {next convert CLAY, SILT, SAND and OC to mass for DLZ mm depth and 1 sq meter. The mass is calculated for the fine-earth fraction only.}
    DataInit.SAND[i]:=(100-DataInit.CLAY[i]-DataInit.SILT[i]-1.7*DataInit.OC[i]);
    DataInit.CLAYm[i]:=((100-DataInit.GRAVEL[i]-DataInit.STONES[i])/100)*DataInit.CLAY[i]*DataInit.DLZ*DataInit.RHO[i]/100;
    DataInit.SILTm[i]:=((100-DataInit.GRAVEL[i]-DataInit.STONES[i])/100)*DataInit.SILT[i]*DataInit.DLZ*DataInit.RHO[i]/100;
    DataInit.SANDm[i]:=((100-DataInit.GRAVEL[i]-DataInit.STONES[i])/100)*DataInit.SAND[i]*DataInit.DLZ*DataInit.RHO[i]/100;
    DataInit.fineSANDm[i]:=DataInit.SANDm[i]*0.4; {assuming 40% is <256 microns}
    DataInit.OCm[i]:=((100-DataInit.GRAVEL[i]-DataInit.STONES[i])/100)*DataInit.OC[i]*DataInit.DLZ*DataInit.RHO[i]/100;
 end;
 for i:=51 to 57 do begin q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt); end;
 q:=q+1; system.readln(findata,DataInit.rocn2);
 q:=q+1; system.readln(findata,DataInit.roslope);
{PHYSICAL PROPERTIES COMPLETE.}
 for i:=58 to 65 do begin q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt); end;
 q:=q+1; system.readln(findata,DataInit.KLPLT);
 q:=q+1; system.readln(findata,DataInit.NCROP);
 q:=q+1; system.readln(findata,DataInit.HDRY);
 q:=q+1; system.readln(findata,DataInit.HLOW);
 q:=q+1; system.readln(findata,DataInit.AK2);
 q:=q+1; system.readln(findata,DataInit.RRES);
 for i:=66 to 70 do begin q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt); end;
 for i:=1 to DataInit.NCROP do
 begin
   q:=q+1;
   system.readln(findata,DataInit.igrowth[i],DataInit.icut[i],DataInit.rtype[i],
                         DataInit.PDAY[i],DataInit.ESTART[i],DataInit.RDFDAY[i],
                         DataInit.ESTOP[i],DataInit.DAYHAR[i],DataInit.RDEP[i],
                         DataInit.AK1[i],DataInit.PANFAC[i]);
 end;
{Crop data system.read}
 for i:=71 to 77 do begin q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt); end;
 for i:=1 to DataInit.NoLayers do
 begin
   q:=q+1;
   system.readln(findata,DataInit.NDUM,DataInit.SEA[1][I],DataInit.SEA[2][I],DataInit.SEA[3][I],
                         DataInit.SEA[4][I],DataInit.SEA[5][I],DataInit.SEA[6][I],DataInit.SEA[7][I],
                         DataInit.SEA[12][i],DataInit.SEA[13][i],DataInit.SEA[14][i],DataInit.SEA[15][i],
                         DataInit.DOCperc[i]);
   for j:=1 to 6 do DataInit.SEA[j][I]:=max(DataInit.SEA[j][I],1E-14);
   for j:=12 to 15 do DataInit.SEA[j][I]:=max(DataInit.SEA[j][I],1E-14);
   DataInit.DOC[i]:=DataInit.DOCperc[i]*DataInit.RHO[i]*1*DataInit.DLZ/10; {here calculated to mmol/m2 but ignored: calc als fraction BIOM}
   DataInit.alkalinity[i]:=DataInit.SEA[7][I];
 end;
 for i:=78 to 79 do begin q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt); end;
 q:=q+1;
 for i:=1 to 7 do begin system.read(findata,DataInit.CHARGE[i]); end;
 for i:=12 to 16 do begin system.read(findata,DataInit.CHARGE[i]); end;
 system.readln(findata,txt); mmHeaders.Lines.Add(txt);
 q:=q+1; system.readln(findata,DataInit.CELLW);
{Soluble cations and anions system.read}
 for i:=81 to 87 do begin q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt); end;
 for i:=1 to DataInit.NoLayers do
 begin
   q:=q+1;
   system.readln(findata,DataInit.NDUM,DataInit.AXSE[1][I],DataInit.AXSE[2][I],DataInit.AXSE[3][I],
                         DataInit.AXSE[4][I],DataInit.ACEC[I],DataInit.AXSE[10][i],DataInit.AXSE[12][i]);
   for j:=1 to 4 do DataInit.AXSE[j][I]:=max(DataInit.AXSE[j][I],1E-14);
   DataInit.AXSE[10][I]:=max(DataInit.AXSE[10][I],1E-14); DataInit.AXSE[12][I]:=max(DataInit.AXSE[12][I],1E-14);
 end;
{Exchangeable cations read.
 Next the AXSE[] and ACEC[i] have to be split up in a part connected to the organic matter
 and a part connected to the clay. This is done in the routine: SplitCEC}
 SplitCEC;
 for i:=88 to 92 do begin q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt); end;
 for i:=1 to DataInit.NoLayers do
 begin
   q:=q+1;
   {SC1=Mg/Ca; SC2=Ca/Na; SC3=Ca/K;  SC4=Mg/K;  SC5= Mg/Na; SC6= K/Na;
    SC7=Ca/Al; SC8=Ca/H;  SC9=Mg/Al; SC10=K/Al; SC11= Na/Al; SC12=Mg/H;
    SC13=K/H;  SC14=Na/H; SC15=Al/H }
   system.readln(findata,DataInit.NDUM,DataInit.SC1[I],DataInit.SC2[I],DataInit.SC3[I],
                         DataInit.APCO2[I],DataInit.CALCITEA[I],DataInit.GYPA[I],
                         DataInit.SC7[i],DataInit.sc8[i]);
   {check on Gapon coefficients too close to 0 and correct to a low value}
   if (DataInit.SC1[I]<0.00000001) then DataInit.SC1[I]:=0.00000001;   
   if (DataInit.SC2[I]<0.00000001) then DataInit.SC2[I]:=0.00000001;
   if (DataInit.SC3[I]<0.00000001) then DataInit.SC3[I]:=0.00000001;
   if (DataInit.SC7[I]<0.00000001) then DataInit.SC7[I]:=0.00000001;
   if (DataInit.SC8[I]<0.00000001) then DataInit.SC8[I]:=0.00000001;
   {pf23 convert Calcite and gypsum from mass fraction to kg/compartment; correcting for gravel+stones (without calcite and gypsum): mass=RHO*DLZ*mfr*(1-stonefr-gravelfr)}
   DataInit.CALCITEA[I]:=DataInit.CALCITEA[I]*DataInit.RHO[i]*DataInit.DLZ*(1-(DataInit.STONES[i]+DataInit.GRAVEL[i])/100);
   DataInit.GYPA[I]:=DataInit.GYPA[I]*DataInit.RHO[i]*DataInit.DLZ*(1-(DataInit.STONES[i]+DataInit.GRAVEL[i])/100);
 end;
{Selectivity coefficients system.read.}
 for i:=93 to 99 do begin q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt); end;
 q:=q+1; system.readln(findata,DataInit.NN1);
 q:=q+1; system.readln(findata,DataInit.ITERCH);
 if DataInit.iflow<>1 then DataInit.ITERCH:=1; {in case of capacity model}
 for i:=100 to 102 do begin q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt); end;
 q:=q+1; system.readln(findata,DataInit.DIFO);
 for i:=103 to 106 do begin q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt); end;
 q:=q+1; system.readln(findata,DataInit.IFR);
 for i:=107 to 110 do begin q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt); end;
 for i:=1 to DataInit.IFR do
 begin
   q:=q+1;
   system.readln(findata,DataInit.DFER[i],DataInit.INCOR[i],DataInit.FER[1,i],DataInit.FER[2,i],
                         DataInit.FER[3,i],DataInit.FER[4,i],DataInit.FER[5,i],DataInit.FER[6,i],
                         DataInit.FER[7,i],DataInit.FER[8,i],DataInit.FER[12,i]);
 end;
{Chemical applications system.read.}
 for i:=111 to 114 do begin q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt); end;
 q:=q+1; system.readln(findata,DataInit.Icult );
 for i:=115 to 118 do begin q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt); end;
 for i:=1 to DataInit.Icult do begin q:=q+1; system.readln(findata,DataInit.Dcult[i],DataInit.cultdep[i]); end;
{Cultivation dates system.read.}
 for i:=119 to 122 do begin q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt); end;
 q:=q+1; system.readln(findata,DataInit.NIRRIG);
 q:=q+1; system.readln(findata,DataInit.ch_irr_flag);
 for i:=123 to 126 do begin q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt); end;
 Pjfmond:=0; Pamjjas:=0;
 for i:=1 to DataInit.NIRRIG do
 begin
   q:=q+1; system.readln(findata,DataInit.AIRRBG[i],DataInit.FRAC1[i],DataInit.IRRIGA[i],
                                 DataInit.IRATEA[i],DataInit.ARSE[1,i],DataInit.ARSE[2,i],
                                 DataInit.ARSE[3,i],DataInit.ARSE[4,i],DataInit.ARSE[5,i],
                                 DataInit.ARSE[6,i],DataInit.ARSE[7,i],DataInit.ARSE[12,i]);
   if (i<=90)or(i>=274)
      then Pjfmond:=Pjfmond+DataInit.IRRIGA[i]  {sum precip over months 1,2,3,10,11,12}
      else Pamjjas:=Pamjjas+DataInit.IRRIGA[i]; {sum precip over months 4,5,6,7,8,9}
   defaultP:=defaultP+DataInit.IRRIGA[i];
   {monthly default precipitation if monthly P-data are read}
   if chckbxSpecialClimate.Checked
   then
   begin
           if (i<=31)  then DefaultPtotMonthly[1]:=DefaultPtotMonthly[1]+DataInit.IRRIGA[i]
      else if (i<=59)  then DefaultPtotMonthly[2]:=DefaultPtotMonthly[2]+DataInit.IRRIGA[i]
      else if (i<=90)  then DefaultPtotMonthly[3]:=DefaultPtotMonthly[3]+DataInit.IRRIGA[i]
      else if (i<=120) then DefaultPtotMonthly[4]:=DefaultPtotMonthly[4]+DataInit.IRRIGA[i]
      else if (i<=151) then DefaultPtotMonthly[5]:=DefaultPtotMonthly[5]+DataInit.IRRIGA[i]
      else if (i<=181) then DefaultPtotMonthly[6]:=DefaultPtotMonthly[6]+DataInit.IRRIGA[i]
      else if (i<=212) then DefaultPtotMonthly[7]:=DefaultPtotMonthly[7]+DataInit.IRRIGA[i]
      else if (i<=243) then DefaultPtotMonthly[8]:=DefaultPtotMonthly[8]+DataInit.IRRIGA[i]
      else if (i<=273) then DefaultPtotMonthly[9]:=DefaultPtotMonthly[9]+DataInit.IRRIGA[i]
      else if (i<=304) then DefaultPtotMonthly[10]:=DefaultPtotMonthly[10]+DataInit.IRRIGA[i]
      else if (i<=334) then DefaultPtotMonthly[11]:=DefaultPtotMonthly[11]+DataInit.IRRIGA[i]
      else if (i<=365) then DefaultPtotMonthly[12]:=DefaultPtotMonthly[12]+DataInit.IRRIGA[i];
   end;
 end;
 for i:=127 to 133 do begin q:=q+1; system.readln(findata,txt); mmHeaders.Lines.Add(txt); end;
 for i:=1 to DataInit.NWEEK do
 begin
   q:=q+1; system.readln(findata,DataInit.NDUM,DataInit.PAN[i],DataInit.WTDEP[i],
                                 DataInit.AVETEM[i],DataInit.AMP[i]);
   defaultPE:=defaultPE+DataInit.PAN[i];
 end;
 {monthly default evaporation if monthly PE-data are read}
 if chckbxSpecialClimate.Checked
 then
 begin
   defaultPEtotMonthly[1]:=DataInit.PAN[1]+DataInit.PAN[2]+DataInit.PAN[3]+DataInit.PAN[4]+
                           DataInit.PAN[5]*3/7;
   defaultPEtotMonthly[2]:=DataInit.PAN[5]*4/7+DataInit.PAN[6]+DataInit.PAN[7]+DataInit.PAN[8]+
                           DataInit.PAN[9]*3/7;
   defaultPEtotMonthly[3]:=DataInit.PAN[9]*4/7+DataInit.PAN[10]+DataInit.PAN[11]+DataInit.PAN[12]+
                           DataInit.PAN[13]*6/7;
   defaultPEtotMonthly[4]:=DataInit.PAN[13]*1/7+DataInit.PAN[14]+DataInit.PAN[15]+DataInit.PAN[16]+
                           DataInit.PAN[17]+DataInit.PAN[18]*1/7;
   defaultPEtotMonthly[5]:=DataInit.PAN[18]*6/7+DataInit.PAN[19]+DataInit.PAN[20]+DataInit.PAN[21]+
                           DataInit.PAN[22]*4/7;
   defaultPEtotMonthly[6]:=DataInit.PAN[22]*3/7+DataInit.PAN[23]+DataInit.PAN[24]+DataInit.PAN[25]+
                           DataInit.PAN[26]*6/7;
   defaultPEtotMonthly[7]:=DataInit.PAN[26]*1/7+DataInit.PAN[27]+DataInit.PAN[28]+DataInit.PAN[29]+
                           DataInit.PAN[30]+DataInit.PAN[31]*2/7;
   defaultPEtotMonthly[8]:=DataInit.PAN[31]*5/7+DataInit.PAN[32]+DataInit.PAN[33]+DataInit.PAN[34]+
                           DataInit.PAN[35]*5/7;
   defaultPEtotMonthly[9]:=DataInit.PAN[35]*2/7+DataInit.PAN[36]+DataInit.PAN[37]+DataInit.PAN[38]+
                           DataInit.PAN[39];
   defaultPEtotMonthly[10]:=DataInit.PAN[40]+DataInit.PAN[41]+DataInit.PAN[42]+DataInit.PAN[43]+
                           DataInit.PAN[44]*3/7;
   defaultPEtotMonthly[11]:=DataInit.PAN[44]*4/7+DataInit.PAN[45]+DataInit.PAN[46]+DataInit.PAN[47]+
                           DataInit.PAN[48]*5/7;
   defaultPEtotMonthly[12]:=DataInit.PAN[48]*2/7+DataInit.PAN[49]+DataInit.PAN[50]+DataInit.PAN[51]+
                           DataInit.PAN[52]+DataInit.PAN[53];
   defaultAvTempMonthly[1]:=(7*DataInit.AVETEM[1]+7*DataInit.AVETEM[2]+7*DataInit.AVETEM[3]+7*DataInit.AVETEM[4]+
                           3*DataInit.AVETEM[5])/31;
   defaultAvTempMonthly[2]:=(4*DataInit.AVETEM[5]+7*DataInit.AVETEM[6]+7*DataInit.AVETEM[7]+7*DataInit.AVETEM[8]+
                           3*DataInit.AVETEM[9])/28;
   defaultAvTempMonthly[3]:=(4*DataInit.AVETEM[9]+7*DataInit.AVETEM[10]+7*DataInit.AVETEM[11]+7*DataInit.AVETEM[12]+
                           6*DataInit.AVETEM[13])/31;
   defaultAvTempMonthly[4]:=(DataInit.AVETEM[13]+7*DataInit.AVETEM[14]+7*DataInit.AVETEM[15]+7*DataInit.AVETEM[16]+
                           7*DataInit.AVETEM[17]+DataInit.AVETEM[18])/30;
   defaultAvTempMonthly[5]:=(6*DataInit.AVETEM[18]+7*DataInit.AVETEM[19]+7*DataInit.AVETEM[20]+7*DataInit.AVETEM[21]+
                           4*DataInit.AVETEM[22])/31;
   defaultAvTempMonthly[6]:=(3*DataInit.AVETEM[22]+7*DataInit.AVETEM[23]+7*DataInit.AVETEM[24]+7*DataInit.AVETEM[25]+
                           6*DataInit.AVETEM[26])/30;
   defaultAvTempMonthly[7]:=(DataInit.AVETEM[26]+7*DataInit.AVETEM[27]+7*DataInit.AVETEM[28]+7*DataInit.AVETEM[29]+
                           7*DataInit.AVETEM[30]+2*DataInit.AVETEM[31])/31;
   defaultAvTempMonthly[8]:=(5*DataInit.AVETEM[31]+7*DataInit.AVETEM[32]+7*DataInit.AVETEM[33]+7*DataInit.AVETEM[34]+
                           5*DataInit.AVETEM[35])/31;
   defaultAvTempMonthly[9]:=(2*DataInit.AVETEM[35]+7*DataInit.AVETEM[36]+7*DataInit.AVETEM[37]+7*DataInit.AVETEM[38]+
                           7*DataInit.AVETEM[39])/30;
   defaultAvTempMonthly[10]:=(7*DataInit.AVETEM[40]+7*DataInit.AVETEM[41]+7*DataInit.AVETEM[42]+7*DataInit.AVETEM[43]+
                           3*DataInit.AVETEM[44])/31;
   defaultAvTempMonthly[11]:=(4*DataInit.AVETEM[44]+7*DataInit.AVETEM[45]+7*DataInit.AVETEM[46]+7*DataInit.AVETEM[47]+
                           5*DataInit.AVETEM[48])/30;
   defaultAvTempMonthly[12]:=(2*DataInit.AVETEM[48]+7*DataInit.AVETEM[49]+7*DataInit.AVETEM[50]+7*DataInit.AVETEM[51]+
                           7*DataInit.AVETEM[52]+DataInit.AVETEM[53])/31;
 end;
{ET and water table data read.}
 closefile(findata);
 {copy headers into tblock-array}
 for i:=1 to 133 do
 begin
   txt:=mmHeaders.Lines[i-1]; DataInit.tblock[i]:=copy(txt,1,min(153,length(txt)));
 end;
 for i:=1 to DataInit.NoLayers do
 begin
   DataInit.MM21[i]:=20; {for the moment fix montmorillonite percentage in clay fraction at 20%}
   DataInit.SUM21[i]:=50;{for the moment fix percentage (montmorillonite+illite) in clay fraction at 50%}
   {eventually, this should become input, also in events.txt!}
 end;
 {stoneperc is here still mass-based, will be recalculated to volume-based later}
 for i:=1 to DataInit.Nolayers do stoneperc[i]:=DataInit.GRAVEL[i]+DataInit.STONES[i];
 {inputs do satisfy the format criteria, but checks on nonsense still have to be done}
 checknonsense;
 ReadDataOK:=true;
 {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving ReadData');
end;
except
  ReadDataOK:=false;
  if not(LeachCidentified) then dshowmessage('Year '+currentyear+'; Fatal error: This is no LEACHC-file!')
  else
  begin
    str(q,Msg); Msg:='Year '+currentyear+'; Fatal error: Error in inputfile '+FileNameData1+' near line '+Msg;
    dshowmessage(Msg);
  end;
  dshowmessage('You will not be able to run SoilGen unless you correct this error and reload.');
  {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving ReadData with errors');
end;
end;

procedure TfrmMAIN.WriteData(FileNameData2:string);
{writes a soil input file valid for the end of the current/last simulation year}
var i,j,cnt:integer; FileNameShort:string;  deficit,calcdum,gypsdum,alkdum:real;
begin
{If GUItrace=1 then wwwriteln} singletracewriteln('Entering WriteData');
{with DataYearIn do}
begin
 FileNameShort:=FileNameData2;
 while pos('\',FileNameShort)>0 do delete(FileNameShort,1,pos('\',FileNameShort));
 while pos('/',FileNameShort)>0 do delete(FileNameShort,1,pos('/',FileNameShort));
 system.Assign(foutdata,FileNameData2); system.Rewrite(foutdata);
 cnt:=length(DataYearIn.tblock[1])-pos('<',DataYearIn.tblock[1]);
 DataYearIn.tblock[1]:=FileNameShort+' <'+copy(DataYearIn.tblock[1],pos('<',DataYearIn.tblock[1])+1,cnt);
 for i:=1 to 7 do system.writeln(foutdata,DataYearIn.tblock[i]);
 system.writeln(foutdata,DataYearIn.idate);
 system.writeln(foutdata,DataYearIn.day1);
 system.writeln(foutdata,DataYearIn.CUMT1);
 system.writeln(foutdata,DataYearIn.DELM:4:2);
 system.writeln(foutdata,DataYearIn.NYC);
 system.writeln(foutdata,DataYearIn.PDEPTH:4:0);
 system.writeln(foutdata,DataYearIn.DLZ:3:0);
 DataYearIn.NoLayers:=round(DataYearIn.PDEPTH/DataYearIn.DLZ);
 {BBC is chosen by evaluating the yearly (P-PE)}
 deficit:=0;
 for i:=1 to DataYearIn.NIRRIG do deficit:=deficit+DataYearIn.IRRIGA[i];
 for i:=1 to DataYearIn.NWEEK do deficit:=deficit-DataYearIn.PAN[i]*DataYearIn.PANFAC[1];
 {BBC type is set via user interface now}
 case rdgrpBBC.ItemIndex of 0: DataYearIn.BBC:=2; {free drainage}
                            1: DataYearIn.BBC:=3; {zero drainage}
                            2: begin  {if precip surplus: free, else zero drainage}
                                 if deficit>0 then DataYearIn.BBC:=2 else DataYearIn.BBC:=3;
                               end;
 end;
 {NB: an option is, in case the watertabledepth is inside the profile,
      that BBC will be reset to BBC=1 in untProcesses.readcnew. Tests however have shown
      numerical problems and this option was removed again}
 system.writeln(foutdata,DataYearIn.BBC);
 system.writeln(foutdata,DataYearIn.ABBPOT:4:0);
 for i:=8 to 12 do system.writeln(foutdata,DataYearIn.tblock[i]);
 system.writeln(foutdata,DataYearIn.IFLOW);
 system.writeln(foutdata,DataYearIn.THETA1:3:1);
 for i:=13 to 14 do system.writeln(foutdata,DataYearIn.tblock[i]);
 system.writeln(foutdata,DataYearIn.IFILE);
 for i:=15 to 16 do system.writeln(foutdata,DataYearIn.tblock[i]);
 system.writeln(foutdata,DataYearIn.UNITS);
 system.writeln(foutdata,DataYearIn.R54);
 system.writeln(foutdata,DataYearIn.PRNTOP2);
 system.writeln(foutdata,DataYearIn.PRINT2);
 system.writeln(foutdata,DataYearIn.NPRINT);
 system.writeln(foutdata,DataYearIn.PRNTAB);
 system.writeln(foutdata,DataYearIn.INITFLG);
 for i:=17 to 18 do system.writeln(foutdata,DataYearIn.tblock[i]);
 system.writeln(foutdata,DataYearIn.NSUM:4:2);
 system.writeln(foutdata,DataYearIn.nd1);
 system.writeln(foutdata,DataYearIn.nd2);
 system.writeln(foutdata,DataYearIn.nd3);
 system.writeln(foutdata,DataYearIn.nd4);
 for i:=19 to 20 do system.writeln(foutdata,DataYearIn.tblock[i]);
 system.writeln(foutdata,DataYearIn.FWBTC:3:1);
 for i:=21 to 26 do system.writeln(foutdata,DataYearIn.tblock[i]);
 for i:=1 to DataYearIn.NPRINT do system.writeln(foutdata,DataYearIn.PRTIME[i],'       ',DataYearIn.FRAC1Print[i]:3:1);
 for i:=27 to 38 do system.writeln(foutdata,DataYearIn.tblock[i]);
 for i:=1 to DataYearIn.NoLayers do system.writeln(foutdata,i:2,'     ',DataYearIn.CLAY[i]:5:2,'  ',
                                                 DataYearIn.SILT[i]:5:2,'  ',DataYearIn.OC[i]:5:2,
                                                 '         ',
                                                 DataYearIn.MODRET[i],'  ',DataYearIn.AW[i]:5:3,'  ',
                                                 DataYearIn.AH[i]:7:1,'       ',
                                                 DataYearIn.RDF[i]:4:2,'       ',
                                                 DataYearIn.TEM[i]:5:1,'       ',
                                                 DataYearIn.GRAVEL[i]:7:3,'       ',
                                                 DataYearIn.STONES[i]:7:3);
 system.writeln(foutdata,DataYearIn.tblock[39]);
 system.writeln(foutdata,DataYearIn.PRIOR);
 system.writeln(foutdata,DataYearIn.tblock[40]);
 system.writeln(foutdata,'                 ',DataYearIn.RHOS2:4:2,'         ',DataYearIn.RHOS1:4:2,
                         '              ',DataYearIn.RHOS3:4:2);
 for i:=41 to 44 do system.writeln(foutdata,DataYearIn.tblock[i]);
 system.writeln(foutdata,DataYearIn.RHO1:4:1,' ',DataYearIn.parden1:4:1);
 system.writeln(foutdata,DataYearIn.AEV1:4:1);
 system.writeln(foutdata,DataYearIn.BCAM1:4:1);
 system.writeln(foutdata,DataYearIn.CMAT1:4:1,' ',DataYearIn.HMAT1:4:1);
 system.writeln(foutdata,DataYearIn.RP1:4:1);
 system.writeln(foutdata,DataYearIn.ALAMB1:4:1);
 system.writeln(foutdata,DataYearIn.ADDFC:4:1);
 system.writeln(foutdata,DataYearIn.ADDTH:4:1);
 for i:=45 to 50 do system.writeln(foutdata,DataYearIn.tblock[i]);
 for i:=1 to DataYearIn.NoLayers do
 begin
       if DataYearIn.MODRET[i]<>0
       then untProcesses.estimate_KatH(i+1,DataYearIn.clay[i],DataYearIn.silt[i],
                                  {correct?}DataYearIn.rho[i],DataYearIn.oc[i],DataYearIn.DUM,
                                  DataYearIn.CMATCH[i],DataYearIn.HMATCH[i]);
       system.writeln(foutdata,' ',i:2,'        ',DataYearIn.AAEV[i]:5:2,
                               '    ',DataYearIn.ABCAM[i]:4:2,'     ',{(100/(100-DataYearIn.STONES[i]-DataYearIn.GRAVEL[i]))* }
                               DataYearIn.RHO[i]:5:3,'     ',DataYearIn.CMATCH[i]:4:0,
                               '     ',DataYearIn.HMATCH[i]:5:1,'     ',
                               DataYearIn.RP[i]:3:1,'     ',DataYearIn.LAMBDA[i]:5:1,
                               '     ',DataYearIn.fc[i]:6:1,'         ',DataYearIn.d[i]:6:1);
 end;
 for i:=51 to 57 do system.writeln(foutdata,DataYearIn.tblock[i]);
 system.writeln(foutdata,DataYearIn.rocn2:2:0);
 system.writeln(foutdata,DataYearIn.roslope:2:0);
{PHYSICAL PROPERTIES COMPLETE.}
 for i:=58 to 65 do system.writeln(foutdata,DataYearIn.tblock[i]);
 system.writeln(foutdata,DataYearIn.KLPLT);
 system.writeln(foutdata,DataYearIn.NCROP);
 system.writeln(foutdata,DataYearIn.HDRY:5:0);
 system.writeln(foutdata,DataYearIn.HLOW:5:0);
 system.writeln(foutdata,DataYearIn.AK2:3:1);
 system.writeln(foutdata,DataYearIn.RRES:4:2);
 for i:=66 to 70 do system.writeln(foutdata,DataYearIn.tblock[i]);
 for i:=1 to DataYearIn.NCROP do system.writeln(foutdata,DataYearIn.igrowth[i]:4,'    ',DataYearIn.icut[i]:4,'    ',
                                              DataYearIn.rtype[i]:4,'         ',DataYearIn.PDAY[i]:6,' ',
                                              DataYearIn.ESTART[i]:6,' ',DataYearIn.RDFDAY[i]:6,' ',
                                              DataYearIn.ESTOP[i]:6,' ',DataYearIn.DAYHAR[i]:6,'  ',
                                              DataYearIn.RDEP[i]:4:2,'   ',DataYearIn.AK1[i]:3:1,'     ',
                                              DataYearIn.PANFAC[i]:4:2);
{Crop data written to file}
 for i:=71 to 77 do system.writeln(foutdata,DataYearIn.tblock[i]);
 for i:=1 to DataYearIn.NoLayers do
 begin
   system.write(foutdata,'    ',i:3,'     ');
   for j:=1 to 7 do system.write(foutdata,DataYearIn.SEA[j][i]:15:10,' ');
   {alkdum:=2*(DataYearIn.SEA[1][i]+DataYearIn.SEA[2][i]+DataYearIn.SEA[14][i]-DataYearIn.SEA[6][i])+
             (DataYearIn.SEA[3][i]+DataYearIn.SEA[4][i]-DataYearIn.SEA[5][i]+DataYearIn.SEA[10][i]-DataYearIn.SEA[11][i])-
           3*(DataYearIn.SEA[12][i]+DataYearIn.SEA[13][i]);
   system.write(foutdata,alkdum:15:10,' '); commented out because this includes effect H+ and OH- while elsewhere alkalinity does not}
   for j:=12 to 15 do system.write(foutdata,DataYearIn.SEA[j][i]:15:10,' ');
   DataYearIn.DOCperc[i]:=DataYearIn.DOC[i]*10/(DataYearIn.DLZ*1*DataYearIn.RHO[i]);
   system.write(foutdata,DataYearIn.DOCperc[i]:10:7);
   system.writeln(foutdata);
 end;
 for i:=78 to 79 do system.writeln(foutdata,DataYearIn.tblock[i]);
 for i:=1 to 7 do system.write(foutdata,DataYearIn.CHARGE[i]:7:3,' ');
 for i:=12 to 16 do system.write(foutdata,DataYearIn.CHARGE[i]:7:3,' ');
 system.writeln(foutdata,DataYearIn.tblock[80]);
 system.writeln(foutdata,DataYearIn.CELLW:4:0);
{Soluble cations and anions written to file}
 for i:=81 to 87 do system.writeln(foutdata,DataYearIn.tblock[i]);
 for i:=1 to DataYearIn.NoLayers do
 begin
    system.write(foutdata,'     ',i:3,'     ');
    for j:=1 to 4 do system.write(foutdata,DataYearIn.AXSE[j][i]:15:10,'  ');
    system.write(foutdata,DataYearIn.ACEC[I]:15:10);
    system.write(foutdata,DataYearIn.AXSE[10][i]:15:10,'  ');
    system.write(foutdata,DataYearIn.AXSE[12][i]:15:10,'  ');
    system.writeln(foutdata);
 end;
{Exchangeable cations written to file}
 for i:=88 to 92 do system.writeln(foutdata,DataYearIn.tblock[i]);
 for i:=1 to DataYearIn.NoLayers do
 begin
   {pf23: convert for writing to leachc input file calcite and gypsum back to mass fractions, correcting for gravel+stones (without calcite and gypsum)mfr=mass/(RHO*DLZ*(1-stonesfr-gravelfr))}
   calcdum:=DataYearIn.CALCITEA[I]/(DataYearIn.RHO[i]*DataYearIn.DLZ*(1-(DataYearIn.STONES[I]+DataYearIn.GRAVEL[I])/100));
   gypsdum:=DataYearIn.GYPA[I]/(DataYearIn.RHO[i]*DataYearIn.DLZ*(1-(DataYearIn.STONES[I]+DataYearIn.GRAVEL[I])/100));
   system.writeln(foutdata,'    ',i:3,'   ',DataYearIn.SC1[I]:15:10,' ',
                           DataYearIn.SC2[I]:15:10,' ',DataYearIn.SC3[I]:15:10,' ',
                           DataYearIn.APCO2[I]:7:5,' ',
                           calcdum:15:10,' ',gypsdum:15:10,' ',
                           DataYearIn.SC7[I]:15:10,' ',DataYearIn.SC8[I]:15:10);
 end;
{Selectivity coefficients written to file}
 for i:=93 to 99 do system.writeln(foutdata,DataYearIn.tblock[i]);
 system.writeln(foutdata,DataYearIn.NN1);
 system.writeln(foutdata,DataYearIn.ITERCH);
 for i:=100 to 102 do system.writeln(foutdata,DataYearIn.tblock[i]);
 system.writeln(foutdata,DataYearIn.DIFO:3:0);
 for i:=103 to 106 do system.writeln(foutdata,DataYearIn.tblock[i]);
 system.writeln(foutdata,DataYearIn.IFR);
 for i:=107 to 110 do system.writeln(foutdata,DataYearIn.tblock[i]);
 for i:=1 to DataYearIn.IFR do
 begin
   system.write(foutdata,'   ',DataYearIn.DFER[i]:6,'         ',DataYearIn.INCOR[i]:2,'          ');
   for j:=1 to 8 do system.write(foutdata,DataYearIn.FER[j,i]:5:2,' ');
   system.write(foutdata,DataYearIn.FER[12,i]:5:2,' ');
   system.writeln(foutdata);
 end;
{Chemical applications written to file}
 for i:=111 to 114 do system.writeln(foutdata,DataYearIn.tblock[i]);
 system.writeln(foutdata,DataYearIn.Icult);
 for i:=115 to 118 do system.writeln(foutdata,DataYearIn.tblock[i]);
 for i:=1 to DataYearIn.Icult do system.writeln(foutdata,DataYearIn.Dcult[i]:6,'       ',DataYearIn.cultdep[i]:4:0);
{Cultivation dates written to file}
 for i:=119 to 122 do system.writeln(foutdata,DataYearIn.tblock[i]);
 system.writeln(foutdata,DataYearIn.NIRRIG);
 system.writeln(foutdata,DataYearIn.ch_irr_flag);
 for i:=123 to 126 do system.writeln(foutdata,DataYearIn.tblock[i]);
 for i:=1 to DataYearIn.NIRRIG do
 begin
   system.write(foutdata,DataYearIn.AIRRBG[i]:6,'      ',DataYearIn.FRAC1[i]:3:1,'   ',
                           DataYearIn.IRRIGA[i]:5:1,'    ',DataYearIn.IRATEA[i]:5:1,'       ');
   for j:=1 to 7 do system.write(foutdata,DataYearIn.ARSE[j,i]:5:3,' ');
   system.write(foutdata,DataYearIn.ARSE[12,i]:5:3,' ');
   system.writeln(foutdata);
 end;
 for i:=127 to 133 do system.writeln(foutdata,DataYearIn.tblock[i]);
 for i:=1 to DataYearIn.NWEEK do
   system.writeln(foutdata,'       ',i:3,'       ',
                           DataYearIn.PAN[i]:4:1,'    ',DataYearIn.WTDEP[i]:4:1,'          ',
                           DataYearIn.AVETEM[i]:4:1,'      ',DataYearIn.AMP[i]:4:1);
{ET and water table data write.}
 closefile(foutdata);
end;
{If GUItrace=1 then wwwriteln} singletracewriteln('Leaving WriteData');
end;

FUNCTION TfrmMAIN.JULIAN(idate,date:integer):integer;
{calculates Julian day number for a given date}
var i,temp,d,m,y:integer;
begin
  y:=date-100*trunc(date/100);
  if idate=2 then
  begin m:=trunc(date/100)-100*trunc(date/10000); d:=trunc(date/10000); end
  else
  begin d:=trunc(date/100)-100*trunc(date/10000); m:=trunc(date/10000); end;
  temp:=0;
  if m=1 then temp:=d
  else
  begin for i:=2 to m do
        if (y mod 4=0) then temp:=temp+monl[i-1] else temp:=temp+mon[i-1];
        temp:=temp+d;
  end;
  julian:=temp;
end;   {FUNCTION JULIAN}

procedure TfrmMAIN.ReadOut(FileNameOut:string);
{reads soil output file "LC*"; now obsolete: no longer used}
{this routine is no longer called}
{1. reads outputfile *.out of SoilGen, legacy from LEACHC,
 2. copies the relevant data into the record dataYearOut,
    which will later be used to create a new input file}
var txt,txt2:string;
    datum,code,i:integer;
    year1:boolean;
begin
{If GUItrace=1 then wwwriteln} singletracewriteln('Entering ReadOut');
{0. set data record dataYearOut equal to DataYearIn as a manner of initialisation
(input LEACHC).  Relevant fields are later overwritten with data read from
  *.OUT file LEACHC}
 {dataYearOut:=DataYearIn;} CopyRecToFrom(DataYearout,DataYearIn);
 year1:=pos('000001',FileNameOut)>0;
 datum:=1; {will be re-initialised later}
 {1 en 2}
 with dataYearOut do
 begin
   system.Assign(fexp,FileNameOut); system.Reset(fexp);
   if (year1)and(nn1<>0) then {read the selectivity coefficients that were calculated during the first year}
   begin
     repeat system.readln(fexp,txt); until pos('Calculated selectivity coefficients',txt)>0;
     system.readln(fexp,txt);
     {SC1=Mg/Ca; SC2=Ca/Na; SC3=Ca/K;  SC4=Mg/K;  SC5= Mg/Na; SC6= K/Na;
      SC7=Ca/Al; SC8=Ca/H;  SC9=Mg/Al; SC10=K/Al; SC11= Na/Al; SC12=Mg/H;
      SC13=K/H;  SC14=Na/H; SC15=Al/H }
     for i:=1 to NoLayers do system.readln(fexp,dum,sc1[i],sc2[i],sc3[i],sc4[i],sc5[i],sc6[i],dum,dum,dum,sc7[i],sc8[i]);
   end;
   {find last output}
   system.reset(fexp);
   Repeat
     system.Readln(fexp,txt);
     if (pos('DATE',txt)>0) then
     begin
       txt2:=txt;
       system.Delete(txt2,POS('DATE',txt2),4);
       while pos(' ',txt2)>0 do delete(txt2,pos(' ',txt2),1);
       while pos('/',txt2)>0 do delete(txt2,pos('/',txt2),1);
       system.val(txt2,datum,code);
     end;
   until (pos('DATE',txt)>0) and (datum=dataYearOut.CUMT1);
   {now start reading data from last simulation date}
   {first series: water, anions, calcite, gypsum}
   repeat
     system.Readln(fexp,txt)
   until pos('Node',txt)>0;
   for i:=1 to 2 do system.readln(fexp);
   for i:=1 to NoLayers do
     system.Readln(fexp,dum,AW[i],AH[i],
                        FLUX[i],ET[i],SEA[5][i],SEA[6][i],
                        HCO3[i],CO3[i],dum,dum,CALCITEA[i],GYPA[i],SAR[i],ESP[i],EC[i],PH[i]);
   {second series: root density and cations (exchangeable+in solution)}
   repeat system.Readln(fexp,txt) until pos('Node',txt)>0;
   for i:=1 to 3 do system.readln(fexp);
   for i:=1 to NoLayers do
   begin
     system.Readln(fexp,dum,RDF[i],
                        AXSE[1][i],SEA[1][i],AXSE[2][i],SEA[2][i],
                        AXSE[3][i],SEA[3][i],AXSE[4][i],SEA[4][i]);
     alkalinity[i]:=SEA[7][i]+2*SEA[8][i]+SEA[11][i]-SEA[10][i];
     DISSOLVEDCATEQ[i]:=2*SEA[1][i]+2*SEA[2][i]+SEA[3][i]+SEA[4][i];
     SEA[7][i]:=alkalinity[i];
     ACEC[i]:=AXSE[1][i]+AXSE[2][i]+AXSE[3][i]+AXSE[4][i]+AXSE[10][i]+AXSE[12][i];
     BS[i]:=100*(AXSE[1][i]+AXSE[2][i]+AXSE[3][i]+AXSE[4][i])/ACEC[i];
   end;
 end;
 closefile(fexp);
{If GUItrace=1 then wwwriteln} singletracewriteln('Leaving ReadOut');
end;

procedure TfrmMAIN.FormActivate(Sender: TObject);
{initialization}
var i:integer; txt:string;
begin
  othererror:=false; {nothing wrong yet}
  MineralFactorDPM:=1;  MineralFactorRPM:=1; MineralFactorBIO:=1; MineralFactorHUM:=1;
  spndtMultipleYears.MaxValue:=MaxNoyears;
  monthlyTEMPname:=''; monthlyPRECname:=''; monthlyEVAPname:='';
  {str(100*(1-MinSplashableSoilAreareduction):5:1,txt);
  showmessage('Special version: bioturbfr for upper 1 mm =0 when "agriculture"'+#13#13+
              'Reduced splash-affected area when "agriculture" to maximally '+txt+'%'); }
  btbttnd14c.caption:='Import delta14C values';
  btbttnd14C.enabled:=OK14c;
  pndlgOpen.InitialDir:=startdir;
  pndlgOPENspecialclimate.InitialDir:=startdir;
  showmemorydump1.Caption:='memory dump (A previous run)';
  showmemorydump1.Visible:=(fileexists('dump.txt'));
  claymigrationallowed:=chckbxClayMigration.Checked;
  showwindowheader;
  if fileexists('Messages.txt') then renamefile('Messages.txt','MessagesOLD.txt');
  currentyear:='When reading initial inputs';
  {copy combobox containing names of screen output variables (cmbbxScreenVariables)
   into combobox with tdd-filenames (cmbbxTimeDepth), while deleting whitespaces from name}
  cmbbxScreenVariables.Text:=cmbbxScreenVariables.Items[0];
  cmbbxTimeDepth.Clear;
  for i:=0 to maxTDD-1 do
  begin
    txt:=cmbbxScreenVariables.Items[i];
    eenheden[i+1]:=copy(txt,pos('(',txt),length(txt)+1-pos('(',txt));
    {delete(txt,pos('(',txt),length(txt)+1-pos('(',txt));}{delete units}
    while pos(' ',txt)>0 do delete(txt,pos(' ',txt),1); {delete spaces}
    cmbbxTimeDepth.Items.Add(txt);
  end;
  {autodetect some files and set some associated checkboxes}
  chckbxClimate.Checked:=fileexists('Climate.txt');
  chckbxClimate.Enabled:=fileexists('Climate.txt');
  chckbxBioturbation.Checked:=fileexists('BioturbINT.txt');
  chckbxBioturbation.Enabled:=fileexists('BioturbINT.txt');
  chckbxFertilization.Checked:=fileexists('Fertilization.txt');
  chckbxFertilization.Enabled:=fileexists('Fertilization.txt');  
  chckbxEvents.Checked:=fileexists('Events.txt');
  chckbxEvents.Enabled:=fileexists('Events.txt');
  {no dos window inside}
  largeDOS:=false;
  {DOS-run only allowed if the executable exists}
  chckbxDOS.enabled:=fileexists('LEACHcPF.exe');
  chckbxDOS.Visible:=chckbxDOS.enabled;
  {years: normal and leap years}
  for i:=1 to 12 do
  begin if i in [1,3,5,7,8,10,12] then begin mon[i]:=31; monl[i]:=31; end
        else  if i<>2 then begin mon[i]:=30; monl[i]:=30; end
              else begin mon[i]:=28; monl[i]:=29; end;
  end;
  {series from blue to almost white in 5 colours}
  kleur[1]:=$00FF0000;  kleur[2]:=$00FF3333; kleur[3]:=$00FF6666;
  kleur[4]:=$00FF9999;  kleur[5]:=$00FFCCCC;
  {series from almost white to red in 5 colours}
  kleur[6]:=$00CCCCFF;  kleur[7]:=$009999FF; kleur[8]:=$006666FF;
  kleur[9]:=$003333FF; kleur[10]:=$000000FF;
  {zwart}
  kleur[11]:=$00000000;
  {initial value range for later display of time-depth diagrams}
  for i:=1 to MaxTDD do begin minnen[i]:=0; maxen[i]:=100; end;
  minnen[1]:=0; maxen[1]:=100 ; {Alkalinity (mmol-/l)}
  minnen[2]:=0;   maxen[2]:= 100; {Base Saturation (%)}
  minnen[3]:=0;   maxen[3]:= 50; {Ca-exch (mmol+/kg soil)}
  minnen[4]:=0;   maxen[4]:= 0.5; {Calcite (mass fraction)}
  minnen[5]:=0;   maxen[5]:= 10; {Ca-solution (mmol/l)}
  minnen[6]:=0;   maxen[6]:= 10; {Cl- (mmol/l)}
  minnen[7]:=0;   maxen[7]:= 10; {CO3-- (mmol/l)}
  minnen[8]:=0;   maxen[8]:=100 ; {Diss. Ca+Mg+Na+K (mmol+/l)}
  minnen[9]:=0;   maxen[9]:= 5; {EC (mS/m)}
  minnen[10]:=0;  maxen[10]:=100 ; {ESP}
  minnen[11]:=0;  maxen[11]:=10 ; {ET (mm)}
  minnen[12]:=0;  maxen[12]:=100; {Exch. Ca+Mg+Na+K (mmol+/kg)}
  minnen[13]:=-20;maxen[13]:= 20; {Flux (mm)}
  minnen[14]:=0;  maxen[14]:= 0.5; {Gypsum (mass fraction)}
  minnen[15]:=0;  maxen[15]:=10 ; {HCO3- (mmol/l)}
  minnen[16]:=0;  maxen[16]:=50 ; {K-exch (mmol+/kg soil)}
  minnen[17]:=0;  maxen[17]:= 10; {K-solution (mmol/l)}
  minnen[18]:=0;  maxen[18]:= 50; {Mg-exch (mmol+/kg soil)}
  minnen[19]:=0;  maxen[19]:=10 ; {Mg-solution (mmol/l)}
  minnen[20]:=0;  maxen[20]:=50 ; {Na-exch (mmol+/kg soil)}
  minnen[21]:=0;  maxen[21]:=10 ; {Na-solution (mmol/l)}
  minnen[22]:=3;  maxen[22]:=10 ; {pH}
  minnen[23]:=-100; maxen[23]:= 0; {Potential (kPa)}
  minnen[24]:=0;  maxen[24]:= 1; {Roots (fraction of total)}
  minnen[25]:=0;  maxen[25]:= 1; {SAR}
  minnen[26]:=0;  maxen[26]:= 10; {SO4-- (mol/l)}
  minnen[27]:=0;  maxen[27]:= 0.60; {Theta (cm.3/cm.3)}
  minnen[28]:=0.5;maxen[28]:= 2; {RHO (kg/dm.3)}
  minnen[29]:=0;  maxen[29]:= 40; {Clay (mass% fine earth)}
  minnen[30]:=0;  maxen[30]:=70 ; {Silt (mass% fine earth)}
  minnen[31]:=0;  maxen[31]:= 70; {Sand (mass% fine earth)}
  minnen[32]:=0;  maxen[32]:=2; {OC (mass% solid fraction)}
  minnen[33]:=0.10; maxen[33]:=0.60; {Porosity (volume fraction)}
  minnen[34]:=0;  maxen[34]:= 40; {Clay (volume%)}
  minnen[35]:=0;  maxen[35]:=1 ; {ClayDispersionIndicator (-)}
  minnen[36]:=-5; maxen[36]:=25 ; {Temperature (oC)}
  minnen[37]:=0;  maxen[37]:=0.5; {DecomposablePlantMaterial (ton/ha)}
  minnen[38]:=0;  maxen[38]:=5 ; {ResistantPlantMaterial (ton/ha)}
  minnen[39]:=0;  maxen[39]:=0.5 ; {Biomass (ton/ha)}
  minnen[40]:=0;  maxen[40]:=0.5 ; {Humus (ton/ha)}
  minnen[41]:=0;  maxen[41]:=5 ; {InertOrganicMatter (ton/ha)}
  minnen[42]:=0;  maxen[42]:=0.02 ; {pCO2 (bar)}
  minnen[43]:=0;  maxen[43]:= 50; {Al-exch (mmol+/kg soil)}
  minnen[44]:=0;  maxen[44]:= 100; {Gibbsite (mol/m2)}
  minnen[45]:=0;  maxen[45]:= 10; {Al-solution (mmol/l)}
  minnen[46]:=0;  maxen[46]:=50; {delta14C, in promilles}
  minnen[47]:=0;  maxen[47]:=500; {age BP}
  minnen[48]:=0;  maxen[48]:=100; {CIA, %}
  minnen[49]:=0;  maxen[49]:=1; {indexB, 0-1}
  minnen[50]:=0;  maxen[50]:=100; {CIW, %}
  minnen[51]:=0;  maxen[51]:=100; {PIA, %}
  minnen[52]:=0;  maxen[52]:=2; {WI, 0-2 (>1=bemesting...)}
  minnen[53]:=0;  maxen[53]:=2; {DPMXpermil/DPM}
  minnen[54]:=0;  maxen[54]:=2; {DPMXpermil/DPM}
  minnen[55]:=0;  maxen[55]:=2; {DPMXpermil/DPM}
  minnen[56]:=0;  maxen[56]:=2; {DPMXpermil/DPM}
  minnen[57]:=0;  maxen[57]:=2; {DPMXpermil/DPM}
  minnen[58]:=0;  maxen[58]:=100; {CPA, %}
  minnen[59]:=0;  maxen[59]:=0.6; {available water capacity}
  minnen[60]:=0;  maxen[60]:=500;
  minnen[61]:=0;  maxen[61]:=500;
  minnen[62]:=0;  maxen[62]:=500;
  minnen[63]:=0;  maxen[63]:=500;
  minnen[64]:=0;  maxen[64]:=500;
  minnen[65]:=0;  maxen[65]:=700;
  minnen[66]:=0;  maxen[66]:=500;
  minnen[67]:=0;  maxen[67]:=75; {albite, all minerals can max have fraction of 1, assuming RHO=1.5 kg/dm3 and DLS=50 mm}
  minnen[68]:=0;  maxen[68]:=75;
  minnen[69]:=0;  maxen[69]:=75;
  minnen[70]:=0;  maxen[70]:=75;
  minnen[71]:=0;  maxen[71]:=75;
  minnen[72]:=0;  maxen[72]:=2*75;  {chlorite, also secondary mineral}
  minnen[73]:=0;  maxen[73]:=75;
  minnen[74]:=0;  maxen[74]:=75;
  minnen[75]:=0;  maxen[75]:=75;
  minnen[76]:=0;  maxen[76]:=2*75;  {kaol, also secondary mineral}
  minnen[77]:=0;  maxen[77]:=2*75;  {ill, also secondary mineral}
  minnen[78]:=0;  maxen[78]:=2*75;  {montm, also secondary mineral}
  minnen[79]:=0;  maxen[79]:=75;
  minnen[80]:=0;  maxen[80]:=75; {fayalite}
  minnen[81]:=3;  maxen[81]:=10; {pH-lab}
  minnen[82]:=0;  maxen[82]:=75; {amorphite}
  minnen[83]:=0;  maxen[83]:=75;
  minnen[84]:=0;  maxen[84]:=500; {TRB}
  minnen[85]:=0;  maxen[85]:=5;   {rate modifier MoistureTemperature}
  minnen[86]:=0;  maxen[86]:=3;   {rate modifier Geochemistry} 
  minnen[87]:=0;  maxen[87]:=75;
  minnen[88]:=0;  maxen[88]:=75;
  minnen[89]:=0;  maxen[89]:=0.05;   {Fe2}
  minnen[90]:=0;  maxen[90]:=0.0005; {Fe3}
  minnen[91]:=0;  maxen[91]:=0.7;    {H4SiO4s}
  minnen[92]:=0;  maxen[92]:=1;
  minnen[93]:=0;  maxen[93]:=1;
  minnen[94]:=0;  maxen[94]:=1;
  minnen[95]:=0;  maxen[95]:=20;
  minnen[96]:=0;  maxen[96]:=20;
  minnen[97]:=0;  maxen[97]:=1;
  {names for chemical equilibrium constants}
  DataInit.knames[1]:='KH  Henrys law constant for CO2';
  DataInit.knames[2]:='KW  Stability constant of water';
  DataInit.knames[3]:='KA1 First dissociation constant of H2CO3';
  DataInit.knames[4]:='KA2 Second dissociation constant of H2CO3';
  DataInit.knames[5]:='KD1 Stability constant of CaCO3';
  DataInit.knames[6]:='KD2 Stability constant of CaHCO3+';
  DataInit.knames[7]:='KD3 Stability constant of CaOH+';
  DataInit.knames[8]:='KD4 Stability constant of CaSO4';
  DataInit.knames[9]:='KD5 Stability constant of MgCO3';
  DataInit.knames[10]:='KD6 Stability constant of MgHCO3+';
  DataInit.knames[11]:='KD7 Stability constant of MgOH+';
  DataInit.knames[12]:='KD8 Stability constant of MgSO4';
  DataInit.knames[13]:='KD9 Stability constant of NaSO4-';
  DataInit.knames[14]:='KD10 Stability constant of NaCO3-';
  DataInit.knames[15]:='KD11 Stability constant of KSO4-';
  DataInit.knames[16]:='SP1 Solubility product of gypsum';
  DataInit.knames[17]:='SP2 Solubility product of calcite';
  DataInit.knames[18]:='KSP Solubility product of Gibbsite';
  DataInit.knames[19]:='KAl1 Stability constant AlOH_2p';
  DataInit.knames[20]:='KAl2 Stability constant AlOH2_p';
  DataInit.knames[21]:='KAl4 Stability constant AlOH4_m ';
  DataInit.knames[22]:='Kf1 Stability constant AlF_2p';
  DataInit.knames[23]:='Kf2 Stability constant AlF2_p';
  DataInit.knames[24]:='Kf3 Stability constant AlF3';
  DataInit.knames[25]:='Kf4 Stability constant AlF4_m';
  DataInit.knames[26]:='Kf5 Stability constant AlF5_2m';
  DataInit.knames[27]:='Ko1 Stability constant AlOrg';
  DataInit.knames[28]:='Ko2 Stability constant AlHOrg_p';
  DataInit.knames[29]:='Kg1 Stability constant H2Org_m';
  DataInit.knames[30]:='Kg2 Stability constant HOrg_2m';
  DataInit.knames[31]:='Kg3 Stability constant Org_3m';
  DataInit.knames[32]:='KHF Dissociation constant HF';
  DataInit.knames[33]:='KCaF Solubility product of CaF2';
  DataInit.knames[34]:='KAlbit   Solubility product of Albite';
  DataInit.knames[35]:='KKFeld   Solubility product of K-Feldspar';
  DataInit.knames[36]:='KMusco   Solubility product of Muscovite';
  DataInit.knames[37]:='KBioti   Solubility product of Biotite';
  DataInit.knames[38]:='KQuart   Solubility product of Quartz';
  DataInit.knames[39]:='KChlor   Solubility product of Chlorite';
  DataInit.knames[40]:='KAnort   Solubility product of Anorthite';
  DataInit.knames[41]:='KForst   Solubility product of Forsterite';
  DataInit.knames[42]:='KAugit   Solubility product of Augite';
  DataInit.knames[43]:='KKaoli   Solubility product of Kaolinite';
  DataInit.knames[44]:='KIllit   Solubility product of Illite';
  DataInit.knames[45]:='KMontm   Solubility product of Montmorillonite';
  DataInit.knames[46]:='KHornb   Solubility product of Hornblende';
  DataInit.knames[47]:='KFayal   Solubility product of Fayalite';
  DataInit.knames[48]:='KAmorp   Solubility product of Amorphite';
  DataInit.knames[49]:='KOther   Solubility product of Otherite';

  for i:=-MaxNoyears to maxNoYears do d14C[i]:=1;
  if not(fileexists(('dump.txt'))) then
    if fileexists('SoilGenBatch.txt')
    then
    begin BatchModeYN:=true; BatchMode end
    else
    begin BatchModeYN:=false; CheckStartFromScratch end;

    with DataInit do with Rock do
    begin
      {specific weights and initial mass fraction in parent rock}
      rockAlbite[1]:=2.62 ; rockAlbite[3]:=0.0539460892620415;
      rockKFeldspar[1]:=2.58 ; rockKFeldspar[3]:=0.0324348210340256;
      rockMuscovite[1]:=2.83 ; rockMuscovite[3]:=0.243040212107821;
      rockBiotite[1]:=3.1 ; rockBiotite[3]:=0;
      rockAnorthite[1]:=2.75 ; rockAnorthite[3]:=0.0231197525408749;
      rockForsterite[1]:=3.27 ; rockForsterite[3]:=0;
      rockAugite[1]:=3.4 ; rockAugite[3]:=0;
      rockHornblende[1]:=3.3 ; rockHornblende[3]:=0.0121961997348652;
      rockFayalite[1]:=4.392 ; rockFayalite[3]:=0;
      rockQuartz[1]:=2.65 ; rockQuartz[3]:=0.329916040653999;
      rockChlorite[1]:=2.95 ; rockChlorite[3]:=0.0828104286345559;
      rockKaolinite[1]:=2.6 ; rockKaolinite[3]:=0.058594785682722;
      rockIllite[1]:=2.75 ; rockIllite[3]:=0;
      rockMontmorillonite[1]:=2.01 ; rockMontmorillonite[3]:=0.0477242598320813;
      rockAmorphite[1]:=3 ; rockAmorphite[3]:=0;
      rockOtherite[1]:=3 ; rockOtherite[3]:=0;
      rockGibbsite[1]:=2.4 ; rockGibbsite[3]:=0;
      rockCalcite[1]:=2.71 ; rockCalcite[3]:=0.114891736632788;
      rockGypsum[1]:=2.31 ; rockGypsum[3]:=0.00132567388422448;
      {now the 6 weathering params and 2 dissolution params for 3 minerals}
      weathCalcite[1]:=-0.3 ;    weathCalcite[3]:=14.4 ;
      weathGibbsite[1]:=-7.65 ;   weathGibbsite[3]:=47.5 ;
      weathGypsum[1]:=-999 ;     weathGypsum[3]:=-999 ;
      weathCalcite[3]:=1 ;    weathCalcite[4]:=-3.48 ;
      weathGibbsite[3]:=0.99 ;   weathGibbsite[4]:=-16.65 ;
      weathGypsum[3]:=-999 ;     weathGypsum[4]:=-999 ;
      weathCalcite[5]:=35.4 ;    weathCalcite[6]:=1 ;
      weathGibbsite[5]:=80.1 ;   weathGibbsite[6]:=-0.78 ;
      weathGypsum[5]:=-999 ;     weathGypsum[6]:=-999 ;
      weathCalcite[7]:=9.61;     weathCalcite[8]:=power(10,-8.48) ;
      weathGibbsite[7]:=-105.30; weathGibbsite[8]:=power(10,7.74) ;
      weathGypsum[7]:=-0.46;     weathGypsum[8]:=power(10,-4.58) ;
      rockdensity:=3.0;
    end;
end;

procedure TfrmMAIN.spndtMultipleYearsChange(Sender: TObject);
{if the number of simulation years of a scenario is changed in the main window,
the new value is passed to other windows}
begin
   ggProgressMultipleYears.Min:=0;
   ggProgressMultipleYears.Max:=spndtMultipleYears.Value;
   if not(batchmodeYN) then
   begin
   untEvents.frmEvents.spndtEventYearBP.MinValue:=1;
   untEvents.frmEvents.spndtEventYearBP.MaxValue:=spndtMultipleYears.Value;
   untEvents.frmEvents.spndtEventSimYear.MinValue:=1;
   untEvents.frmEvents.spndtEventSimYear.MaxValue:=spndtMultipleYears.Value;
   untEvents.frmEvents.spndtEventSimYear.Value:=spndtMultipleYears.Value;
   untEvents.frmEvents.spndtEventYearBP.Value:=1;
   end;
   spndtTraceStart.MaxValue:=spndtMultipleYears.Value;
  {tracing can be set only after time extent is set}
   {rdgrpTrace.Visible:=true;
   rdgrpTrace.ItemIndex:=0;
   spndtTraceStart.Visible:=false;
   lblTraceStart.Visible:=false; }
end;

procedure TfrmMAIN.GeneralRefresh;
{refreshes the GUI-screen(s)}
begin
  {ggProgressMultipleYears.Refresh;
  lblProgress.Refresh;
  image1.Refresh; image2.Refresh; image3.Refresh;
  image4.Refresh; image5.Refresh; image6.Refresh;}
  Application.ProcessMessages;
  frmMAIN.Refresh;
end;

procedure TfrmMAIN.CleanupDisk;
{deletes temporary files}
begin
  if fileexists('exec.bat') then deletefile('exec.bat');
  if fileexists('climateINT.txt') then deletefile('climateINT.txt');
  if fileexists('fertilINT.txt') then deletefile('fertilINT.txt');
  if fileexists('EventsINT.txt') then deletefile('EventsINT.txt');
  if fileexists('tempClimate.txt') then deletefile('tempClimate.txt');
  if fileexists('title.') then deletefile('title.');
  if fileexists('tmpdmp.txt') then deletefile('tmpdmp.txt');
end;

procedure TfrmMAIN.edtMonitNameChange(Sender: TObject);
{displays the name of the log-file}
begin
  chckbxDeleteMonit.Caption:='Delete existing version of '+
                             edtMonitName.Text+' before starting'
end;

procedure TfrmMAIN.btbttnSelect1Click(Sender: TObject);
{select 1 soil variable to be depicted in the (max 6) dynamic time-depth diagrams during runtime}
begin
  if cmbbxScreenVariables.Text<>'(none)'then
  lstbxScreenVariables.Items.Add(cmbbxScreenVariables.Text);
  btbttnSelect1.Enabled:=(lstbxScreenVariables.Items.Count<6);
  btbttnRemove1.Enabled:=(lstbxScreenVariables.Items.Count>0);
end;

procedure TfrmMAIN.btbttnRemove1Click(Sender: TObject);
{remove 1 soil variable to be depicted in the (max 6) dynamic time-depth diagrams during runtime}
var i,weg:integer;
begin
  weg:=-1;
  for i:=0 to lstbxScreenVariables.Items.Count-1 do
   if lstbxScreenVariables.Selected[i] then weg:=i;
  if weg>-1 then lstbxScreenVariables.Items.Delete(weg);
  btbttnSelect1.Enabled:=(lstbxScreenVariables.Items.Count<6);
  btbttnRemove1.Enabled:=(lstbxScreenVariables.Items.Count>0);
end;

procedure TfrmMAIN.ScreenOutput0;
{initialising screen time-depth diagrams of max 6 soil variables}
var i,j:integer; txt:string;
begin 
  {If GUItrace=1 then wwwriteln} singletracewriteln('Entering Screenoutput0 (initialising screen diagrams)');
  images[1]:=image1; images[2]:=image2; images[3]:=image3;
  images[4]:=image4; images[5]:=image5; images[6]:=image6;
  for j:=1 to lstbxScreenVariables.Items.Count do
  begin
    images[j].Parent:=grpbxScreenOutput;
    images[j].Canvas.Pen.Color:=clWhite;
    images[j].Canvas.Brush.Color:=clWhite;
    {wit vlak} images[j].Canvas.Rectangle(0,0,image1.width,image1.height);
    images[j].Canvas.Pen.Color:=clBlack;
    {binnenkader} images[j].Canvas.Rectangle(9,11,image1.width-29,image1.height-11);
    {grayscale legend}
    for i:=30 to image1.Height-50 do
    begin
      images[j].Canvas.Pen.Color:=65793*round(255*(1-(i-30)/(image1.Height-50-30)));
      {images[j].Canvas.MoveTo(211,220-i);}  images[j].Canvas.MoveTo(image1.width-29,image1.height-i);
      {images[j].Canvas.LineTo(221,220-i);}  images[j].Canvas.LineTo(image1.width-19,image1.height-i);
    end;
    images[j].Canvas.Brush.Color:=clWhite;
    str((((MaxPlusCompartments)*DataInit.DLZ)/10):3:0,txt);
    txt:=txt+' cm';
    images[j].Canvas.TextOut(1,2,'0');
    images[j].Canvas.TextOut(0,45,'D');
    images[j].Canvas.TextOut(1,60,'e');
    images[j].Canvas.TextOut(1,75,'p');
    images[j].Canvas.TextOut(1,90,'t');
    images[j].Canvas.TextOut(1,105,'h');
    {images[j].Canvas.TextOut(1,208,txt);}   images[j].Canvas.TextOut(1,image1.height-12,txt);
    str(spndtMultipleYears.Value,txt);
    txt:=txt+' Yrs';
    {images[j].Canvas.TextOut(200,208,txt);} images[j].Canvas.TextOut(image1.width-40,image1.height-12,txt);
    if j<=lstbxScreenVariables.Items.Count then
    begin
      txt:=lstbxScreenVariables.Items[j-1];
      images[j].Canvas.TextOut(10,0,txt);
      images[j].Visible:=true;
    end
    else images[j].Visible:=false;;
    txt:='value';
    {images[j].Canvas.TextOut(212,0,txt);}  images[j].Canvas.TextOut(image1.width-28,0,txt);
  end;
  image1:=images[1]; image2:=images[2]; image3:=images[3];
  image4:=images[4]; image5:=images[5]; image6:=images[6];
  {hdist:=max(1,200 div spndtMultipleYears.Value);}
  {hdist:=(240-(10+30))/spndtMultipleYears.Value;}
  hdist:=(image1.width-(10+30))/spndtMultipleYears.Value;{35-}
  {vdist:=max(1,200 div DataYearIn.NoLayers);}
  {vdist:=(220-(11+10))/MaxPlusCompartments;}
  vdist:=(image1.height-(14+11))/(MaxPlusCompartments);       {35-}
  {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving ScreenOutput0');
end;

procedure TfrmMAIN.FillScreenData;
{select depth data for the current year of max 6 soil variables}
var i,j:integer;
    klei1,zand1,silt1,totmin,ry1,ry2,ry3,ry4,rsy,P1,P2,m:real;
    dum,calcdum,gypsdum:Profile;
begin
{If GUItrace=1 then wwwriteln} singletracewriteln('Entering FillScreenData');
  for i:=1 to lstbxScreenVariables.Items.Count do
  begin
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[0]
     then begin dataimage[i]:=dataYearOut.alkalinity end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[1]
     then begin dataimage[i]:=dataYearOut.BS end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[2]
     then begin dataimage[i]:=dataYearOut.AXSE[1] end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[3]
     then begin  {pf23: convert back to massfraction, correcting for stone+gravel content (without calcite+gypsum) mfr=mass/(RHO*DLZ*(1-stonesfr-gravelfr))}
            for j:=1 to dataYearOut.NoLayers do
            calcdum[j]:=dataYearOut.CALCITEA[j]/(dataYearOut.RHO[j]*dataYearOut.DLZ*(1-(DataYearOut.STONES[j]+DataYearOut.GRAVEL[j])/100));
            dataimage[i]:=calcdum;
          end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[4]
     then begin dataimage[i]:=dataYearOut.SEA[1] end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[5]
     then begin dataimage[i]:=dataYearOut.SEA[5] end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[6]
     then begin dataimage[i]:=dataYearOut.CO3 end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[7]
     then begin dataimage[i]:=dataYearOut.DISSOLVEDCATEQ end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[8]
     then begin dataimage[i]:=dataYearOut.EC end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[9]
     then begin dataimage[i]:=dataYearOut.ESP end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[10]
     then begin dataimage[i]:=dataYearOut.ET end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[11]
     then begin dataimage[i]:=dataYearOut.ACEC end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[12]
     then begin dataimage[i]:=dataYearOut.FLUX end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[13]
     then begin {pf23: convert back to massfraction, correcting for stone+gravel content (without calcite+gypsum) mfr=mass/(RHO*DLZ*(1-stonesfr-gravelfr))}
            for j:=1 to dataYearOut.NoLayers do
            gypsdum[j]:=dataYearOut.GYPA[j]/(dataYearOut.RHO[j]*dataYearOut.DLZ*(1-(DataYearOut.STONES[j]+DataYearOut.GRAVEL[j])/100));
            dataimage[i]:=gypsdum;
          end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[14]
     then begin dataimage[i]:=dataYearOut.HCO3 end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[15]
     then begin dataimage[i]:=dataYearOut.AXSE[4] end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[16]
     then begin dataimage[i]:=dataYearOut.SEA[4] end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[17]
     then begin dataimage[i]:=dataYearOut.AXSE[2] end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[18]
     then begin dataimage[i]:=dataYearOut.SEA[2] end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[19]
     then begin dataimage[i]:=dataYearOut.AXSE[3] end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[20]
     then begin dataimage[i]:=dataYearOut.SEA[3] end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[21]
     then begin dataimage[i]:=dataYearOut.PH end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[22]
     then begin dataimage[i]:=dataYearOut.AH end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[23]
     then begin dataimage[i]:=dataYearOut.RDF end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[24]
     then begin dataimage[i]:=dataYearOut.SAR end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[25]
     then begin dataimage[i]:=dataYearOut.SEA[6] end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[26]
     then begin dataimage[i]:=dataYearOut.AW end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[27]
     then begin dataimage[i]:=dataYearOut.RHO end;
     for j:=1 to dataYearOut.NoLayers do
     begin
       klei1:=dataYearOut.clay[j]; silt1:=dataYearOut.silt[j];
       {old: zand1:=100-1.7*dataYearOut.OC[j]-silt1-klei1;}
       zand1:=100-1.7*dataYearOut.OC[j]-silt1-klei1;  {markOC}
       totmin:=klei1+silt1+zand1;
       dataYearOut.klei2[j]:=100*klei1/totmin;
       dataYearOut.silt2[j]:=100*silt1/totmin;
       dataYearOut.zand2[j]:=100*zand1/totmin;
       {Mean particle density}
       ry1:=1.7*dataYearOut.OC[j]/dataYearOut.RHOS3;
       ry2:=silt1/dataYearOut.RHOS1;
       ry3:=klei1/dataYearOut.RHOS2;
       ry4:=zand1/dataYearOut.RHOS1;
       rsy:=ry1+ry2+ry3+ry4;
       {dataYearOut.PARDEN[j]:=100/(CLAY[I]/RHOS2+(100-CLAY[j]-1.7*OC[j])/RHOS1+1.7*OC[j]/RHOS3); }
       dataYearOut.PARDEN[j]:=100/(ry3+ry2+ry4+ry1);
       {Calculate porosity}
       dataYearOut.porosity[j]:=((100-DataYearOut.stoneperc[j])/100)*(1-dataYearOut.RHO[j]/dataYearOut.PARDEN[j]);
       {calculate volumetric clay percentage}
       dataYearOut.volpercclay[j]:=(100-dataYearOut.porosity[j])*ry3/rsy;
     end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[28]
     then begin dataimage[i]:=dataYearOut.klei2 end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[29]
     then begin dataimage[i]:=dataYearOut.silt2 end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[30]
     then begin dataimage[i]:=dataYearOut.zand2 end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[31]
     then begin dataimage[i]:=dataYearOut.OC end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[32]
     then begin dataimage[i]:=dataYearOut.porosity end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[33]
     then begin dataimage[i]:=dataYearOut.volpercclay end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[34]
     then begin dataimage[i]:=dataYearOut.ClayDispInd end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[35]
     then begin dataimage[i]:=dataYearOut.Tem end;  
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[36]
     then begin dataimage[i]:=dataYearOut.RothC.DPMPool end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[37]
     then begin dataimage[i]:=dataYearOut.RothC.RPMPool end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[38]
     then begin dataimage[i]:=dataYearOut.RothC.BIOPool end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[39]
     then begin dataimage[i]:=dataYearOut.RothC.HUMPool end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[40]
     then begin dataimage[i]:=dataYearOut.RothC.IOMPool end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[41]
     then begin dataimage[i]:=dataYearOut.apCO2 end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[42]
     then begin dataimage[i]:=dataYearOut.AXSE[12] end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[43]
     then begin
            for j:=1 to dataYearOut.NoLayers do
            begin
              dum[j]:=dataYearOut.WE.Gibbsitepool[j]{/1000};
              {was excl 1000 en *77.979/(1000000*dataYearOut.DLZ*dataYearOut.RHO[j])}
            end;
            dataimage[i]:=dum;
          end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[44]
     then begin dataimage[i]:=dataYearOut.SEA[12] end;
     {if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[42]
     then begin dataimage[i]:=dataYearOut.ISS end; }
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[45]
     then
     begin
        for j:=1 to dataYearOut.NoLayers do
            begin
              {delta14C: first calculate it}
              dum[j]:=(dataYearOut.RothC.DPMXpool[j]+dataYearOut.RothC.RPMXpool[j]+
                         dataYearOut.RothC.BIOXpool[j]+dataYearOut.RothC.HUMXpool[j]+
                         dataYearOut.RothC.IOMXpool[j])/
                        (dataYearOut.RothC.DPMpool[j]+dataYearOut.RothC.RPMpool[j]+
                         dataYearOut.RothC.BIOpool[j]+dataYearOut.RothC.HUMpool[j]+
                         dataYearOut.RothC.IOMpool[j]);
            end;
        dataimage[i]:=dum;
     end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[46]
     then
     begin
        for j:=1 to dataYearOut.NoLayers do
            begin
              {delta14C: first calculate it}
              dum[j]:=(dataYearOut.RothC.DPMXpool[j]+dataYearOut.RothC.RPMXpool[j]+
                       dataYearOut.RothC.BIOXpool[j]+dataYearOut.RothC.HUMXpool[j]+
                       dataYearOut.RothC.IOMXpool[j])/
                      (dataYearOut.RothC.DPMpool[j]+dataYearOut.RothC.RPMpool[j]+
                       dataYearOut.RothC.BIOpool[j]+dataYearOut.RothC.HUMpool[j]+
                       dataYearOut.RothC.IOMpool[j]);
              {calculate age; relative to the year of measurement}
              dum[j]:=(-5568/ln(2))*(ln((dum[j]/1000)+1)+((yearAD-1950)/(5730/ln(2))))
            end;
        dataimage[i]:=dum;
     end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[47]
     then begin dataimage[i]:=CIA end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[48]
     then begin dataimage[i]:=indexB end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[49]
     then begin dataimage[i]:=CIW end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[50]
     then begin dataimage[i]:=PIA end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[51]
     then begin dataimage[i]:=WI end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[52]
     then begin dataimage[i]:=Fk_DPM end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[53]
     then begin dataimage[i]:=Fk_RPM end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[54]
     then begin dataimage[i]:=Fk_BIO end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[55]
     then begin dataimage[i]:=Fk_HUM end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[56]
     then begin dataimage[i]:=Fk_IOM end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[57]
     then begin dataimage[i]:=CPA end;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[58]
     then
     begin
       for j:=1 to DatayearOut.NoLayers do
       begin
         P1:=101.9744;    {equivalent in cm water of pF 2.0}
         P2:=16161.85781; {equivalent in cm water of pF4.2}
         m:=1-(1/untProcesses.VGNen[j+1]);
         AvailableWater[j]:=(untProcesses.VGNtr[j+1]+(untProcesses.VGNts[j+1]-untProcesses.VGNtr[j+1])/
                            (power((1+power((untProcesses.VGNal[j+1]*P1),untProcesses.VGNen[j+1])),m)))-
                            (untProcesses.VGNtr[j+1]+(untProcesses.VGNts[j+1]-untProcesses.VGNtr[j+1])/
                            (power((1+power((untProcesses.VGNal[j+1]*P2),untProcesses.VGNen[j+1])),m)));
       end;
       dataimage[i]:=AvailableWater;
     end;

     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[59]
        then dataimage[i]:=DataYearOut.WE.WE_Capool;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[60]
        then dataimage[i]:=DataYearOut.WE.WE_Mgpool;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[61]
        then dataimage[i]:=DataYearOut.WE.WE_Napool;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[62]
        then dataimage[i]:=DataYearOut.WE.WE_Kpool;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[63]
        then dataimage[i]:=DataYearOut.WE.WE_Fepool;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[64]
        then dataimage[i]:=DataYearOut.WE.WE_Sipool;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[65]
        then dataimage[i]:=DataYearOut.WE.WE_Alpool;
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[66]
        then dataimage[i]:=DataYearOut.WE.MinPools[1]; {albite}
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[67]
        then dataimage[i]:=DataYearOut.WE.MinPools[2];
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[68]
        then dataimage[i]:=DataYearOut.WE.MinPools[3];
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[69]
        then dataimage[i]:=DataYearOut.WE.MinPools[4];
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[70]
        then dataimage[i]:=DataYearOut.WE.MinPools[5];
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[71]
        then dataimage[i]:=DataYearOut.WE.MinPools[6];  {chlorite, also secondary mineral}
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[72]
        then dataimage[i]:=DataYearOut.WE.MinPools[7];
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[73]
        then dataimage[i]:=DataYearOut.WE.MinPools[8];
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[74]
        then dataimage[i]:=DataYearOut.WE.MinPools[9];
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[75]
        then dataimage[i]:=DataYearOut.WE.MinPools[10];  {kaol, also secondary mineral}
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[76]
        then dataimage[i]:=DataYearOut.WE.MinPools[11];  {ill, also secondary mineral}
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[77]
        then dataimage[i]:=DataYearOut.WE.MinPools[12];  {montm, also secondary mineral}
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[78]
        then dataimage[i]:=DataYearOut.WE.MinPools[13];
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[79]
        then dataimage[i]:=DataYearOut.WE.MinPools[14]; {fayalite}
     if lstbxScreenVariables.Items[i-1]=cmbbxScreenVariables.Items[80]
        then dataimage[i]:=phlab; {pH-lab}

     {add other tdd's!}
  end;
{If GUItrace=1 then wwwriteln} singletracewriteln('Leaving FillScreenData');
end;

procedure TfrmMAIN.ScreenOutput1(CurrentYear,cor:integer);
{draw screen time-depth diagrams of max 6 soil variables}
var img,lay,toplaycurrentrun,i,X1, Y1, X2, Y2:integer;
    vul:TColor;
    txt:string;
    rcx1,rcy1,rcx2,rcy2,getal:real;
begin  
{If GUItrace=1 then wwwriteln} singletracewriteln('Entering ScreenOutput1 (draw diagrams)');
  FillScreenData;
  Y2:=10; {will be reset later}
  for img:=1 to lstbxScreenVariables.Items.Count do
  begin
    if CurrentYear=1 then
    begin
      for i:=1 to maxTDD do
      if lstbxScreenVariables.Items[img-1]=cmbbxScreenVariables.Items[i-1]
      then
      begin
        dminvalue[img]:=minnen[i];
        dmaxvalue[img]:=maxen[i];
      end;
      {}
      str(dminvalue[img]:4:0,txt);
      {images[img].Canvas.TextOut(210,190,txt);}  images[img].Canvas.TextOut(image1.width-28,image1.height-30,txt);
      if (dmaxvalue[img]<1)and(dmaxvalue[img]>0.009)
      then str(dmaxvalue[img]:4:2,txt)
      else str(dmaxvalue[img]:4:0,txt);
      {images[img].Canvas.TextOut(210,20,txt);}   images[img].Canvas.TextOut(image1.width-28,image1.height-200,txt);
    end;
  end;
  for img:=1 to lstbxScreenVariables.Items.Count do
    if images[img].Visible then
    begin
      rcY2:=14; {dummy initialization, not used}
      for lay:=1 to MaxPlusCompartments do
      begin
         toplaycurrentrun:=maxpluscompartments-DataYearOut.NoLayers-Rock.RockBottomLayers; {RockAddition}
         {determine corner coordinates using profile depth and CurrentYear}
         {rcX1:=10+(240-10-30)*(CurrentYear-1)/spndtMultipleYears.Value;}
         rcX1:=10+(image1.width-10-30)*(CurrentYear-1)/spndtMultipleYears.Value; {-35}
         if lay=1 then rcY1:=14 else rcY1:=rcY2;
         rcX2:=rcX1+hdist; rcY2:=rcY1+vdist;
         X1:=round(rcX1); X2:=round(rcX2); Y1:=round(rcY1); Y2:=round(rcY2);
         {display only current soil profile, so layers not yet added are not shown}
         if (lay>toplaycurrentrun)and(lay<=maxpluscompartments-Rock.RockBottomLayers-cor)
         then {layers inside the current soil profile: grayscale}
         begin
            getal:=dataimage[img][lay-toplaycurrentrun];
            if getal > dmaxvalue[img] then getal:=dmaxvalue[img];
            if getal < dminvalue[img] then getal:=dminvalue[img];
            vul:=65793*round(255*(1-(getal-dminvalue[img])/
                                        (dmaxvalue[img]-dminvalue[img])));
            vul:=max(vul,65793);
            vul:=min(vul,16777215);
         end
         else begin
            if (lay<=toplaycurrentrun) then vul:=clSkyBlue;  {layers on top the current soil profile: blue}
            if (lay>maxpluscompartments-Rock.RockBottomLayers-cor) then vul:=clFuchsia; {layers in hard rock: clFuchsia}
         end;
         {and draw the filled rectangle}
         images[img].Canvas.Pen.Color:=vul;
         images[img].Canvas.Brush.Color:=vul;
         images[img].Canvas.Rectangle(X1, Y1, X2, Y2);
      end;
    end;
  image1:=images[1]; image2:=images[2]; image3:=images[3];
  image4:=images[4]; image5:=images[5]; image6:=images[6];
{If GUItrace=1 then wwwriteln} singletracewriteln('Leaving ScreenOutput1');
end;

procedure TfrmMAIN.Turbation(dtop,dmax,dbot,fractop,fracmax,fracbot: real);
{turbation over the soil layers between dtop and dbot mm for all soil properties}
var I,chem,ii,itemp,j:integer;
    oldRHO,ChemMass,oldAW:Profile;
    dumsum:real;
{turbation:
 1: soil physical mixing (texture, bulk density determined anew)
 2. soil chemical mixing (mixing solid phase, dissolved phase and exchange phase
    and re-equilibrate chemistry}
begin
{If GUItrace=1 then wwwriteln} singletracewriteln('Entering Turbation');
  {1. soil physical turbation}
  {1a. clay content, silt , OC, bulk density and volumetric moisture content
       are determined anew for the layers that are partially mixed}
  oldRHO:=dataYearOut.RHO; oldAW:=dataYearOut.AW;
  MixTexture(dtop,dmax,dbot,fractop,fracmax,fracbot);
  {also the mix texture subfractions : mix only the remaining number of grains per size-class, PW_K[ii,layer]
  We assume that the gravel size classes are not mixed unless by plowing, via the turning plow procedure}
  for ii:=1 to 11 do
  begin
    for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=PW_K[ii,i];
    MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);   {texture fraction ii; mass fraction }
    for I:=1 to dataYearOut.NoLayers do PW_K[ii,i]:=round(ChemMass[i]);
  end;
  {and calculate from the remaining grains the mass fractions (PW_G) and mass over 100m2 (mass)}
  dumsum:=0;
  for i:=1 to dataYearIn.NoLayers do
  begin
    for ii:=-4 to 11 do dumsum:=dumsum+PW_K[ii,i]*power((PW_X[ii]/100000),3)*PW_PARDEN[ii];
    for ii:=-4 to 11 do        {100 converts to grains per 100m2, all size classes but not yet clay}
    begin
      {PW_G[ii,i]:=PW_K[ii,i]/(DataYearOut.RHO[i]*100*DatayearOut.DLZ/PW_PARDEN[ii])/power((PW_X[ii]/100000),3); }
      PW_G[ii,i]:=PW_K[ii,i]*power((PW_X[ii]/100000),3)*PW_PARDEN[ii]/dumsum;
      mass[ii,i]:=PW_K[ii,i]*power((PW_X[ii]/100000),3)*PW_PARDEN[ii]/100;
    end; {now clay }
    mass[11,i]:=claym[i]*100;   {better but still to be tested:}  mass[11,i]:=DatayearOut.claym[i]*100;

  end;
  {1b. adapt soil physical relations pF-theta-K.
       This is done by fitting the Pedotransferfunctions of CLAY,SILT,OC,RHO}
  {This happens automatically via the SoilGen variable: MODRET[1..KK] has value 5, thus the
   continuous PTF are applied before each new (yearly) SoilGen-run.}

  {2. soil chemical mixing.
      activities:
      a. mixing of the soil, dissolved and exchange phases
        (these phases are first converted to mass, than mixed and than reconverted;
         this implies that "on the fly" also the moisture content is mixed!);
      b. the result is brought into chemical equilibrium using the normal routines
         from SoilGen, which were translated + adapted from Fortran (LEACHC) to Pascal.}
   {ad 2a:}
     {2a1: mixing of concentrations. This comprises the variables:
           * SEA[5,i] Cl-concentration in mmol/liter in compartiment i
           * SEA[6,i] SO4
           * HCO3[i]  HCO3
           * CO3[i]   CO3
           * SEA[1,i] Ca
           * SEA[2,i] Mg
           * SEA[3,i] Na
           * SEA[4,i] K
           * SEA[12,i] Al   
           * SEA[13,i] Fe3+
           * SEA[14,i] Fe2+
           * SEA[15,i] H4SiO4
           * SEA[16,i] DOC
           These concentrations are firstly converted to molare mass by CONC*oldAW*DLZ,
           where oldAW="old" volumetric moisture content (dm3 liq/dm3 soil)
           and DLZ is compartment thickness in mm (corresponding to 1 dm3 soil
                                              at 1 m2 soil surface)
           After this: mixing (procedure MixChem)
           Finally backconvert to molare concentrations, but now on basis of
           the new volumetric moisture content AW}
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.SEA[5,i]*oldAW[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);   {Cl}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.SEA[5,i]:=ChemMass[i]/(dataYearOut.AW[i]*dataYearOut.DLZ);
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.SEA[6,i]*oldAW[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);   {SO4}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.SEA[6,i]:=ChemMass[i]/(dataYearOut.AW[i]*dataYearOut.DLZ);
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.HCO3[i]*oldAW[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);   {HCO3}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.HCO3[i]:=ChemMass[i]/(dataYearOut.AW[i]*dataYearOut.DLZ);
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.CO3[i]*oldAW[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);   {CO3}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.CO3[i]:=ChemMass[i]/(dataYearOut.AW[i]*dataYearOut.DLZ);
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.SEA[1,i]*oldAW[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);   {Ca}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.SEA[1,i]:=ChemMass[i]/(dataYearOut.AW[i]*dataYearOut.DLZ);
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.SEA[2,i]*oldAW[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);   {Mg}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.SEA[2,i]:=ChemMass[i]/(dataYearOut.AW[i]*dataYearOut.DLZ);
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.SEA[3,i]*oldAW[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);   {Na}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.SEA[3,i]:=ChemMass[i]/(dataYearOut.AW[i]*dataYearOut.DLZ);
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.SEA[4,i]*oldAW[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);   {K}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.SEA[4,i]:=ChemMass[i]/(dataYearOut.AW[i]*dataYearOut.DLZ);
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.SEA[12,i]*oldAW[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);   {Al}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.SEA[12,i]:=ChemMass[i]/(dataYearOut.AW[i]*dataYearOut.DLZ);   
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.SEA[13,i]*oldAW[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);   {Fe3+}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.SEA[13,i]:=ChemMass[i]/(dataYearOut.AW[i]*dataYearOut.DLZ);
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.SEA[14,i]*oldAW[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);   {Fe2+}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.SEA[14,i]:=ChemMass[i]/(dataYearOut.AW[i]*dataYearOut.DLZ); 
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.SEA[15,i]*oldAW[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);   {H4SiO4}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.SEA[15,i]:=ChemMass[i]/(dataYearOut.AW[i]*dataYearOut.DLZ); 
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.SEA[16,i]*oldAW[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);   {DOC}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.SEA[16,i]:=ChemMass[i]/(dataYearOut.AW[i]*dataYearOut.DLZ);
     {2a2: mixing of mass (kg/compartment). These are the variables:
           * CALCITEA[i] Calcite CaCO3 mass in compartiment i
           * GYPA[i]     Gypsum CaSO4 mass in compartiment i
           * MinPools[nrminerals][i] primary minerals
           * Gibbsite
           * the cations Ca, Mg, Na, K, Si, Fe and Al still fixed in minerals [1..nrminerals]
           Mixing (procedure MixChem)}
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.CALCITEA[i];
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass); {CaCO3}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.CALCITEA[i]:=ChemMass[i];
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.GYPA[i];
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass); {CaSO4}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.GYPA[i]:=ChemMass[i];
     for j:=1 to nrminerals do
     begin
        for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.WE.MinPools[j][i];
        MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass); {mineral j}
        for I:=1 to dataYearOut.NoLayers do dataYearOut.WE.MinPools[j][i]:=ChemMass[i];
     end;
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.WE.Gibbsitepool[i];
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass); {gibbsite}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.WE.Gibbsitepool[i]:=ChemMass[i];

     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.WE.WE_Capool[i];
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass); {Ca}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.WE.WE_Capool[i]:=ChemMass[i];
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.WE.WE_Mgpool[i];
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass); {Mg}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.WE.WE_Mgpool[i]:=ChemMass[i];
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.WE.WE_Kpool[i];
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass); {K}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.WE.WE_Kpool[i]:=ChemMass[i];
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.WE.WE_Napool[i];
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass); {Na}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.WE.WE_Napool[i]:=ChemMass[i];
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.WE.WE_Sipool[i];
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass); {Si}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.WE.WE_Sipool[i]:=ChemMass[i];
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.WE.WE_Fepool[i];
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass); {Fe}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.WE.WE_Fepool[i]:=ChemMass[i];
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.WE.WE_Alpool[i];
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass); {Al}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.WE.WE_Alpool[i]:=ChemMass[i];
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.WE.WE_FePedogenic[i];
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass); {Pedogenic (=from weathering) Fe}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.WE.WE_FePedogenic[i]:=ChemMass[i];
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.WE.WE_SiPedogenic[i];
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass); {Pedogenic (=from weathering) Si}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.WE.WE_SiPedogenic[i]:=ChemMass[i];

     {2a3: mixing of exchange species/amounts. These are the variables:
           * AXSE[1,i] Ca (mmol(+)/kg soil) in compartment i
           * AXSE[2,i] Mg (mmol(+)/kg soil) in compartment i
           * AXSE[3,i] Na (mmol(+)/kg soil) in compartment i
           * AXSE[4,i] K (mmol(+)/kg soil) in compartment i
           * AXSE[10,i] H (mmol(+)/kg soil) in compartment i
           * AXSE[12,i] Al (mmol(+)/kg soil) in compartment i
           These amounts are first converted into molar masses by
           AXSE*VALx*DLZ*oldRHO,
           where
           VALx = valency of the cation x
           oldRHO="old" bulk density (kg soil/dm3 soil)
           and DLZ is compartment thickness in mm (1 mm corresponds to 1 dm3 soil
                                              at 1 m2 soil surface)
           Hereafter mixing (procedure MixChem)
           Finally backconversion to concentration, but now on the basis of the
           new bulk density RHO}
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.AXSE[1,i]*2*oldRHO[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {exch Ca}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.AXSE[1,i]:=ChemMass[i]/(dataYearOut.RHO[i]*2*dataYearOut.DLZ);
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.AXSE[2,i]*2*oldRHO[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {exch Mg}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.AXSE[2,i]:=ChemMass[i]/(dataYearOut.RHO[i]*2*dataYearOut.DLZ);
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.AXSE[3,i]*oldRHO[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {exch Na}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.AXSE[3,i]:=ChemMass[i]/(dataYearOut.RHO[i]*dataYearOut.DLZ);
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.AXSE[4,i]*oldRHO[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {exch K}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.AXSE[4,i]:=ChemMass[i]/(dataYearOut.RHO[i]*dataYearOut.DLZ);
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.AXSE[10,i]*oldRHO[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {exch H}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.AXSE[10,i]:=ChemMass[i]/(dataYearOut.RHO[i]*dataYearOut.DLZ);
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.AXSE[12,i]*oldRHO[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {exch Al}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.AXSE[12,i]:=ChemMass[i]/(dataYearOut.RHO[i]*dataYearOut.DLZ);
     {also the CEC of a compartment changes by the mixing}
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.ACEC[i]*oldRHO[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {CEC}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.ACEC[i]:=ChemMass[i]/(dataYearOut.RHO[i]*dataYearOut.DLZ);
     {repeat for exchangeable stuff on _clay}
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.AXSE_clay[1,i]*2*oldRHO[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {exch Ca }
     for I:=1 to dataYearOut.NoLayers do dataYearOut.AXSE_clay[1,i]:=ChemMass[i]/(dataYearOut.RHO[i]*2*dataYearOut.DLZ);
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.AXSE_clay[2,i]*2*oldRHO[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {exch Mg }
     for I:=1 to dataYearOut.NoLayers do dataYearOut.AXSE_clay[2,i]:=ChemMass[i]/(dataYearOut.RHO[i]*2*dataYearOut.DLZ);
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.AXSE_clay[3,i]*oldRHO[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {exch Na }
     for I:=1 to dataYearOut.NoLayers do dataYearOut.AXSE_clay[3,i]:=ChemMass[i]/(dataYearOut.RHO[i]*dataYearOut.DLZ);
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.AXSE_clay[4,i]*oldRHO[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {exch K }
     for I:=1 to dataYearOut.NoLayers do dataYearOut.AXSE_clay[4,i]:=ChemMass[i]/(dataYearOut.RHO[i]*dataYearOut.DLZ);
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.AXSE_clay[10,i]*oldRHO[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {exch H }
     for I:=1 to dataYearOut.NoLayers do dataYearOut.AXSE_clay[10,i]:=ChemMass[i]/(dataYearOut.RHO[i]*dataYearOut.DLZ);
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.AXSE_clay[12,i]*oldRHO[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {exch Al }
     for I:=1 to dataYearOut.NoLayers do dataYearOut.AXSE_clay[12,i]:=ChemMass[i]/(dataYearOut.RHO[i]*dataYearOut.DLZ);
     {also the CEC_clay of a compartment changes by the mixing}
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.ACEC_clay[i]*oldRHO[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {CEC}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.ACEC_clay[i]:=ChemMass[i]/(dataYearOut.RHO[i]*dataYearOut.DLZ);
     {mix the factor that converts OC to CEC_oc as this may differ by compartment.}
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.FacCEC_OC[i]*oldRHO[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);
     for I:=1 to dataYearOut.NoLayers do dataYearOut.FacCEC_OC[i]:=ChemMass[i]/(dataYearOut.RHO[i]*dataYearOut.DLZ);
     {subdivide again in AXSE_oc and AXSE_clay and ACEC_clay and ACEC_oc.
      hereto the new (mixed) OC is used and also the mixed AXSE and mixed AXSE_clay
      to calculate AXSE_oc and ACEC_oc. Comment the next line and uncomment the _oc lines below if this is to be undone}
     AdjustCECAXSE2OC;
     {commented part below: mixing all separate subcomponents separately
     {repeat for exchangeable stuff on _oc
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.AXSE_oc[1,i]*2*oldRHO[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {exch Ca
     for I:=1 to dataYearOut.NoLayers do dataYearOut.AXSE_oc[1,i]:=ChemMass[i]/(dataYearOut.RHO[i]*2*dataYearOut.DLZ);
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.AXSE_oc[2,i]*2*oldRHO[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {exch Mg
     for I:=1 to dataYearOut.NoLayers do dataYearOut.AXSE_oc[2,i]:=ChemMass[i]/(dataYearOut.RHO[i]*2*dataYearOut.DLZ);
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.AXSE_oc[3,i]*oldRHO[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {exch Na
     for I:=1 to dataYearOut.NoLayers do dataYearOut.AXSE_oc[3,i]:=ChemMass[i]/(dataYearOut.RHO[i]*dataYearOut.DLZ);
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.AXSE_oc[4,i]*oldRHO[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {exch K
     for I:=1 to dataYearOut.NoLayers do dataYearOut.AXSE_oc[4,i]:=ChemMass[i]/(dataYearOut.RHO[i]*dataYearOut.DLZ);
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.AXSE_oc[10,i]*oldRHO[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {exch H
     for I:=1 to dataYearOut.NoLayers do dataYearOut.AXSE_oc[10,i]:=ChemMass[i]/(dataYearOut.RHO[i]*dataYearOut.DLZ);
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.AXSE_oc[12,i]*oldRHO[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {exch Al
     for I:=1 to dataYearOut.NoLayers do dataYearOut.AXSE_oc[12,i]:=ChemMass[i]/(dataYearOut.RHO[i]*dataYearOut.DLZ);
     {also the CEC of a compartment changes by the mixing
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.ACEC_oc[i]*oldRHO[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {CEC
     for I:=1 to dataYearOut.NoLayers do dataYearOut.ACEC_oc[i]:=ChemMass[i]/(dataYearOut.RHO[i]*dataYearOut.DLZ);

     {mixing is ready for the inorganic chemical pools}
     {mixing of the organic + organic chemical pools.
      These are ton/ha cq mmol/m2 and do not need conversion}
     {first add the bioturbated fraction <fractop> of the ectorganic material to the first compartment,
      but not DPM and RPM.
      This is because organisms are expected to incorporate humus and biomass but not litter fragments
      into the soil - may be modified-}
      dataYearOut.RothC.BIOpool[1]:=dataYearOut.RothC.BIOpool[1]+fractop*dataYearOut.RothC.ectoBIOpool;
      dataYearOut.RothC.ectoBIOpool:=(1-fractop)*dataYearOut.RothC.ectoBIOpool;
      dataYearOut.RothC.HUMpool[1]:=dataYearOut.RothC.HUMpool[1]+fractop*dataYearOut.RothC.ectoHUMpool;
      dataYearOut.RothC.ectoHUMpool:=(1-fractop)*dataYearOut.RothC.ectoHUMpool;
      dataYearOut.RothC.IOMpool[1]:=dataYearOut.RothC.IOMpool[1]+fractop*dataYearOut.RothC.ectoIOMpool;
      dataYearOut.RothC.ectoIOMpool:=(1-fractop)*dataYearOut.RothC.ectoIOMpool;
      {SAME FOR 14c SPECIES}
     if OK14c then
     begin
      dataYearOut.RothC.BIOXpool[1]:=dataYearOut.RothC.BIOXpool[1]+fractop*dataYearOut.RothC.ectoBIOXpool;
      dataYearOut.RothC.ectoBIOXpool:=(1-fractop)*dataYearOut.RothC.ectoBIOXpool;
      dataYearOut.RothC.HUMXpool[1]:=dataYearOut.RothC.HUMXpool[1]+fractop*dataYearOut.RothC.ectoHUMXpool;
      dataYearOut.RothC.ectoHUMXpool:=(1-fractop)*dataYearOut.RothC.ectoHUMXpool;
      dataYearOut.RothC.IOMXpool[1]:=dataYearOut.RothC.IOMXpool[1]+fractop*dataYearOut.RothC.ectoIOMXpool;
      dataYearOut.RothC.ectoIOMXpool:=(1-fractop)*dataYearOut.RothC.ectoIOMXpool;
     end;
      for j:=1 to Maxchem do
      begin {add all chemicals stored in BIO, HUM and IOM ectorganic materials}
         dataYearOut.CHEMBIOpool[j,1]:=dataYearOut.CHEMBIOpool[j,1]+fractop*dataYearOut.CHEMBIOpool[j,0];
         dataYearOut.CHEMBIOpool[j,0]:=(1-fractop)*dataYearOut.CHEMBIOpool[j,0];
         dataYearOut.CHEMHUMpool[j,1]:=dataYearOut.CHEMHUMpool[j,1]+fractop*dataYearOut.CHEMHUMpool[j,0];
         dataYearOut.CHEMHUMpool[j,0]:=(1-fractop)*dataYearOut.CHEMHUMpool[j,0];
         dataYearOut.CHEMIOMpool[j,1]:=dataYearOut.CHEMIOMpool[j,1]+fractop*dataYearOut.CHEMIOMpool[j,0];
         dataYearOut.CHEMIOMpool[j,0]:=(1-fractop)*dataYearOut.CHEMIOMpool[j,0];
         dataYearOut.CHEMCO2pool[j,1]:=dataYearOut.CHEMCO2pool[j,1]+fractop*dataYearOut.CHEMCO2pool[j,0];
         dataYearOut.CHEMCO2pool[j,0]:=(1-fractop)*dataYearOut.CHEMCO2pool[j,0];
      end;
     {now do the mixing}
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.RothC.DPMpool[i];
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {DPM}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.RothC.DPMpool[i]:=ChemMass[i]; 
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.RothC.RPMpool[i];
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {RPM}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.RothC.RPMpool[i]:=ChemMass[i];
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.RothC.BIOpool[i];
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {BIO}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.RothC.BIOpool[i]:=ChemMass[i];
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.RothC.HUMpool[i];
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {HUM}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.RothC.HUMpool[i]:=ChemMass[i];
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.RothC.IOMpool[i];
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {IOM}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.RothC.IOMpool[i]:=ChemMass[i];
     {for 14C species}
    if OK14c then
    begin
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.RothC.DPMXpool[i];
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {DPMX}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.RothC.DPMXpool[i]:=ChemMass[i];
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.RothC.RPMXpool[i];
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {RPMX}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.RothC.RPMXpool[i]:=ChemMass[i];
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.RothC.BIOXpool[i];
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {BIOX}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.RothC.BIOXpool[i]:=ChemMass[i];
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.RothC.HUMXpool[i];
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {HUMX}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.RothC.HUMXpool[i]:=ChemMass[i];
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.RothC.IOMXpool[i];
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {IOMX}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.RothC.IOMXpool[i]:=ChemMass[i];
    end;
     for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.DOC[i]*oldAW[i]*dataYearOut.DLZ;
     MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);   {DOC}
     for I:=1 to dataYearOut.NoLayers do dataYearOut.DOC[i]:=ChemMass[i]/(dataYearOut.AW[i]*dataYearOut.DLZ);
     {CO2-pool is not mixed because it is not eaten!}
     for chem:=1 to MaxChem do
     begin
       for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.CHEMDPMpool[chem,i];
       MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {DPM}
       for I:=1 to dataYearOut.NoLayers do dataYearOut.CHEMDPMpool[chem,i]:=ChemMass[i]; 
       for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.CHEMRPMpool[chem,i];
       MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {RPM}
       for I:=1 to dataYearOut.NoLayers do dataYearOut.CHEMRPMpool[chem,i]:=ChemMass[i];
       for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.CHEMBIOpool[chem,i];
       MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {BIO}
       for I:=1 to dataYearOut.NoLayers do dataYearOut.CHEMBIOpool[chem,i]:=ChemMass[i];
       for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.CHEMHUMpool[chem,i];
       MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {HUM}
       for I:=1 to dataYearOut.NoLayers do dataYearOut.CHEMHUMpool[chem,i]:=ChemMass[i];
       for I:=1 to dataYearOut.NoLayers do ChemMass[i]:=dataYearOut.CHEMIOMpool[chem,i];
       MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot,ChemMass);    {IOM}
       for I:=1 to dataYearOut.NoLayers do dataYearOut.CHEMIOMpool[chem,i]:=ChemMass[i];
       {CO2-pool is not mixed because it is not eaten!.
        It's zero anyway at the end of each day}
      end;
     {mixing ready}
   {ad 2b:}
     for I:=1 to dataYearOut.NoLayers do
     begin
        itemp:=min(40,max(-10,round(DataYearOut.tem[i])));
        for ii:=1 to maxK do dataYearOut.KValue[ii]:=DataInit.kvaluetable[ii,itemp];
     end;
{If GUItrace=1 then wwwriteln} singletracewriteln('Leaving Turbation');
end;

procedure TfrmMAIN.mixtexture(dtop,dmax,dbot,fractop,fracmax,fracbot: real);
{apply turbation for clay%, silt%, oc%, volumetric water content and bulk density}
{Homogenises a soil profile using
a. the (input) fraction to be mixed per compartment (vertical homogenisation)
   nb: the remaining fraction wil be homogenised with the "mixing result" horizontally
b. the original clay%, silt%, oc% and bulk density
Result (per compartment):
a. new clay%, silt%, oc%, volumetric water content
b. new bulk density
Assumption: mass conservation and volume conservation}
var topcomp,botcomp,maxcomp,i:integer;
    oldCLAYm,oldSILTm,oldSANDm,oldOCm,oldWATERm,fracmixed,
    mixCLAY,mixSILT,mixSAND,mixOC,mixwater,porosity,water:Profile;
    sumfrac,summixclay,summixsilt,summixsand,summixOC,summixwater,parden,
    totwater,totporosity:real;
    checkvolumeOK:boolean;
begin
{If GUItrace=1 then wwwriteln} singletracewriteln('Entering MixTexture');
{1. Initialisation}
{1a.Determine what compartmenten must be mixed wholly/partially}
   botcomp:=round(dbot/dataYearOut.DLZ)+1;   botcomp:=min(botcomp,datayearout.nolayers);
   topcomp:=round(dtop/dataYearOut.DLZ)+1;   topcomp:=min(topcomp,datayearout.nolayers);
   maxcomp:=round(dmax/dataYearOut.DLZ)+1;   maxcomp:=min(maxcomp,datayearout.nolayers);
   {fraction mixed determined per compartment}
   for i:=1 to MaxLayers do fracmixed[i]:=0;
   for i:=topcomp to botcomp do
   begin
      if (i>=topcomp)and(i<maxcomp)
      then if maxcomp<>topcomp
           then fracmixed[i]:=fractop+(fracmax-fractop)*(i-topcomp)/(maxcomp-topcomp)
           else fracmixed[i]:=(fracmax+fractop)/2
      else
        if (i>=maxcomp)and(i<=botcomp)
        then if maxcomp<>botcomp
        then fracmixed[i]:=fracbot+(fracmax-fracbot)*(i-botcomp)/(maxcomp-botcomp)
        else fracmixed[i]:=(fracbot+fracmax)/2;
   end;
{1b. initialise mixpools}
   sumfrac:=0; summixclay:=0; summixsilt:=0; summixsand:=0; summixOC:=0; summixwater:=0;
{1c.Initialise internal variables. old*m contains * as mass per compartment,m2}
   for i:=topcomp to botcomp do
   begin
     sumfrac:=fracmixed[i]+sumfrac;
     oldCLAYm[i]:=dataYearOut.CLAYm[i];
     oldSILTm[i]:=dataYearOut.SILTm[i];
     oldSANDm[i]:=dataYearOut.SANDm[i];
     oldOCm[i]:=dataYearOut.OCm[i];
     oldWATERm[i]:=dataYearOut.AW[i]*dataYearOut.DLZ;
     summixclay:=summixclay+fracmixed[i]*oldCLAYm[i];
     summixsilt:=summixsilt+fracmixed[i]*oldSILTm[i];
     summixsand:=summixsand+fracmixed[i]*oldSANDm[i];
     summixOC:=summixOC+fracmixed[i]*oldOCm[i];
     summixwater:=summixwater+fracmixed[i]*oldWATERm[i];
   end;
 {2. Mix. It is assumed that the material belonging to <fracmixed>
     is mixed OVER the compartments (vertical mixing),
     whereafter the result is mixed INSIDE the compartments
     with those materials that were NOT part of fracmixed.
     Thus, afterwards there is lateral homogenisation inside each compartment.}
 {2a. First mix and homogenise the mass}
   for i:=topcomp to botcomp do
   begin
      mixCLAY[i]:=(fracmixed[i]*summixclay/sumfrac)+((1-fracmixed[i])*oldCLAYm[i]);
      mixSILT[i]:=(fracmixed[i]*summixsilt/sumfrac)+((1-fracmixed[i])*oldSILTm[i]);
      mixSAND[i]:=(fracmixed[i]*summixsand/sumfrac)+((1-fracmixed[i])*oldSANDm[i]);
      mixOC[i]:=(fracmixed[i]*summixOC/sumfrac)+((1-fracmixed[i])*oldOCm[i]);
      mixWATER[i]:=(fracmixed[i]*summixwater/sumfrac)+((1-fracmixed[i])*oldWATERm[i]);
   end;
 {2b. convert Clay, Silt, OC to weights.
      Then estimate new bulk density, assumping constant volume
      Then backconvert to weightpercentages}

         {MVAR(GlobalTime*10000+Datayearin.nolayers*100,' MAIN, voor Mixtexture: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]); }
         MVAR(GlobalTime*10000+Datayearin.nolayers*100,' MAIN, voor Mixtexture: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);
   for i:=topcomp to botcomp do
   begin
      dataYearOut.RHO[i]:=(mixCLAY[i]+mixSILT[i]+mixSAND[i]+1.7*mixOC[i])/(dataYearOut.DLZ*((100-DataYearOut.Stones[i]-DataYearOut.Gravel[i])/100));
      {undone: DataYearOut.RHO[i]:=HYPRESRHO(100*mixCLAY[i]/(mixCLAY[i]+mixSILT[i]+mixSAND[i]+1.7*mixOC[i]),1.7*100*mixOC[i]/(mixCLAY[i]+mixSILT[i]+mixSAND[i]+1.7*mixOC[i]));}
      {RHO applies to fine earth only, so DLZ as volume estimate must be corrected for volume% stones}
      {dataYearOut.CLAY[i]:=100*mixCLAY[i]/(dataYearOut.RHO[i]*dataYearOut.DLZ);
      dataYearOut.SILT[i]:=100*mixSILT[i]/(dataYearOut.RHO[i]*dataYearOut.DLZ);
      dataYearOut.SAND[i]:=100*mixSAND[i]/(dataYearOut.RHO[i]*dataYearOut.DLZ);
      dataYearOut.OC[i]:=100*mixOC[i]/(dataYearOut.RHO[i]*dataYearOut.DLZ);}  {markOC}
      dataYearOut.CLAY[i]:=100*mixCLAY[i]/(mixCLAY[i]+mixSILT[i]+mixSAND[i]+1.7*mixOC[i]);
      dataYearOut.SILT[i]:=100*mixSILT[i]/(mixCLAY[i]+mixSILT[i]+mixSAND[i]+1.7*mixOC[i]);
      dataYearOut.SAND[i]:=100*mixSAND[i]/(mixCLAY[i]+mixSILT[i]+mixSAND[i]+1.7*mixOC[i]);
      dataYearOut.OC[i]:=100*mixOC[i]/(mixCLAY[i]+mixSILT[i]+mixSAND[i]+1.7*mixOC[i]);
      dataYearOut.CLAYm[i]:=mixCLAY[i]; 
      dataYearOut.SILTm[i]:=mixSILT[i];
      dataYearOut.SANDm[i]:=mixSAND[i];
      dataYearOut.OCm[i]:=mixOC[i];
   end;

            {MVAR(GlobalTime*10000+Datayearin.nolayers*100,' MAIN, na Mixtexture: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);}
            MVAR(GlobalTime*10000+Datayearin.nolayers*100,' MAIN, na Mixtexture: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);
 {2c. convert Water to volume fraction.
      Check if the maximal porosity corresponding to the new RHO
      is not exceeded}
   checkvolumeOK:=true;
   for i:=topcomp to botcomp do
   begin
      parden:=100/((dataYearOut.CLAY[i]/dataYearOut.RHOS2)+
                   ((100-dataYearOut.CLAY[i]-1.7*dataYearOut.OC[i])/dataYearOut.RHOS1)+
                   (1.7*dataYearOut.OC[i]/dataYearOut.RHOS3));
      {same calc of particle density parden as internally in SoilGen}
      porosity[i]:=max(0.1,((100-DataYearOut.stoneperc[i])/100)*(1-(DataYearOut.RHO[i]/parden))); {do not allow negative porosities due to high OM...}
      water[i]:=mixWATER[i]/dataYearOut.DLZ;
      {water[i] now contains volumetric moisture content; still check with porosity}
      if water[i]>porosity[i] then checkvolumeOK:=false;
   end;
   if not(checkvolumeOK) then {porosity in minimally 1 compartment too low for water content}
   begin
     totwater:=0; totporosity:=0;
     for i:=topcomp to botcomp do
     begin totwater:=totwater+water[i]; totporosity:=totporosity+porosity[i]; end;
     if totwater<=totporosity then {there is in total sufficient porosity for all water}
     begin
       {fill in the mixing zone in each compartment the same fraction of the
        pore volume with water so that total mass is conserved}
       for i:=topcomp to botcomp do
       begin
          dataYearOut.AW[i]:=porosity[i]*totwater/totporosity;
       end;
     end
     else {problem: amount of water exceeds porosity}
     begin
        {write a message!}
        dshowmessage('Year '+currentyear+'; Not fatal (bioturbation): Cannot redistribute water; porosity too low!');
        {solution: all pores are filled with water, but surplus of water is not
         redistributed over other profile parts; thus the mass balance error is
         limited to water and does not include chemical masses}
         for i:=topcomp to botcomp do
         begin
           dataYearOut.AW[i]:=porosity[i];
         end;
     end;
   end
   else {porosity-check OK}
   begin
     {distribute the water definitively using the calculated mass distribution}
     for i:=topcomp to botcomp do dataYearOut.AW[i]:=water[i];
   end; 
{If GUItrace=1 then wwwriteln} singletracewriteln('Leaving MixTexture');
end;

procedure TfrmMAIN.MixChem(dtop,dmax,dbot,fractop,fracmax,fracbot:real;
                           var ChemMass:Profile);
{apply turbation for all chemicals on a mass basis}
{homogenises a profile on the basis of
a. the (input) mixing fraction per compartment (vertical homogenisation)
   nb: the remaining fraction wil be homogenised with the "mixing result" horizontally
b. the original mass given in ChemMass[i]
Result (per compartment):  new ChemMass[i]
Assumptions:
* mass conservation
* chemical equilibria are re-calculated after all chemical compounts are mixed}
var topcomp,botcomp,maxcomp,i:integer;
    fracmixed:Profile;
    sumfrac,summixchem:real;
begin
{If GUItrace=1 then wwwriteln} singletracewriteln('Entering MixChem');
{1. Initialisation}
{1a.Determine what compartmenten must be mixed wholly/partially}
   botcomp:=round(dbot/dataYearOut.DLZ)+1;   botcomp:=min(botcomp,datayearout.nolayers);
   topcomp:=round(dtop/dataYearOut.DLZ)+1;   topcomp:=min(topcomp,datayearout.nolayers);
   maxcomp:=round(dmax/dataYearOut.DLZ)+1;   maxcomp:=min(maxcomp,datayearout.nolayers);
   {fraction mixed determined per compartment}
   for i:=1 to MaxLayers do fracmixed[i]:=0;
   for i:=topcomp to botcomp do
   begin
      if (i>=topcomp)and(i<maxcomp)
      then if maxcomp<>topcomp
           then fracmixed[i]:=fractop+(fracmax-fractop)*(i-topcomp)/(maxcomp-topcomp)
           else fracmixed[i]:=(fracmax+fractop)/2
      else
        if (i>=maxcomp)and(i<=botcomp)
        then if maxcomp<>botcomp
        then fracmixed[i]:=fracbot+(fracmax-fracbot)*(i-botcomp)/(maxcomp-botcomp)
        else fracmixed[i]:=(fracbot+fracmax)/2;
   end;
{1b. mixpools initialised}
   sumfrac:=0; summixchem:=0;
{1c.Internal variables initialised. old*m contains * as mass per compartment,m2}
   for i:=topcomp to botcomp do
   begin
     sumfrac:=fracmixed[i]+sumfrac;
     summixchem:=summixchem+fracmixed[i]*ChemMass[i];
   end;
 {2. Mix. It is assumed that the material belonging to <fracmixed>
     is mixied OVER the compartments (vertical mixing),
     whereafter the result is mixed INSIDE the compartments
     with those materials that were NOT part of fracmixed.
     Thus, afterwards there is lateral homogenisation inside each compartment.}
 {2 First mix and homogenise the mass}
   for i:=topcomp to botcomp do
   begin
      ChemMass[i]:=(fracmixed[i]*summixchem/sumfrac)+((1-fracmixed[i])*ChemMass[i]);
   end;
{If GUItrace=1 then wwwriteln} singletracewriteln('Leaving MixChem');
end;

procedure TfrmMAIN.btbttnBioTurbationClick(Sender: TObject);
{opens the window to enter/edit the bioturbation levels and depths over the simulation time}
begin
   untBioturbation.frmBIOTURBATION.ShowModal;
end;

procedure TfrmMAIN.writeTimeDepth(year,cor:integer);
{write the time-depth diagram of all (well, 100+) soil variables to file *.tdd
for the current year}
var i,j,toplaycurrentrun:integer;
    klei1,silt1,zand1,totmin,y1,y2,y3,y4,sy,calcdum,gypsdum,delta14c,age,
    p1,p2,m:real;
begin
{If GUItrace=1 then wwwriteln} singletracewriteln('Entering WriteTimeDepth');
 for i:=1 to maxpluscompartments do
 begin
  toplaycurrentrun:=maxpluscompartments-DataYearOut.NoLayers-Rock.RockBottomLayers; {Rockaddition}
  if (i<=toplaycurrentrun)or(i>maxpluscompartments-Rock.RockBottomLayers-cor)  {outside simulated profile}
  then
  begin
   system.write(f1,'-999 '); system.write(f2,'-999 ');
   system.write(f3,'-999 '); system.write(f4,'-999 ');   system.write(f4a,'-999 ');
   system.write(f5,'-999 '); system.write(f6,'-999 ');
   system.write(f7,'-999 '); system.write(f8,'-999 ');
   system.write(f9,'-999 '); system.write(f10,'-999 ');
   system.write(f11,'-999 '); system.write(f12,'-999 ');
   system.write(f13,'-999 '); system.write(f14,'-999 '); system.write(f14a,'-999 ');
   system.write(f15,'-999 '); system.write(f16,'-999 ');
   system.write(f17,'-999 '); system.write(f18,'-999 ');
   system.write(f19,'-999 '); system.write(f20,'-999 ');
   system.write(f21,'-999 '); system.write(f22,'-999 ');
   system.write(f23,'-999 '); system.write(f24,'-999 ');
   system.write(f25,'-999 '); system.write(f26,'-999 ');
   system.write(f27,'-999 '); system.write(f28,'-999 ');   
   system.write(f29,'-999 '); system.write(f30,'-999 ');
   system.write(f31,'-999 '); system.write(f32,'-999 ');
   system.write(f33,'-999 '); system.write(f34,'-999 ');
   system.write(f35,'-999 '); system.write(f36,'-999 ');
   system.write(f37,'-999 '); system.write(f38,'-999 '); 
   system.write(f39,'-999 '); system.write(f40,'-999 ');
   system.write(f41,'-999 '); system.write(f42,'-999 ');
   system.write(f43,'-999 '); system.write(f44,'-999 '); system.write(f44a,'-999 ');
   system.write(f45,'-999 ');
   system.write(f46,'-999 '); system.write(f47,'-999 ');
   system.write(f48,'-999 '); system.write(f49,'-999 ');
   system.write(f50,'-999 '); system.write(f51,'-999 ');
   system.write(f52,'-999 '); system.write(f53,'-999 ');
   system.write(f54,'-999 '); system.write(f55,'-999 ');
   system.write(f56,'-999 '); system.write(f57,'-999 ');
   system.write(f58,'-999 '); system.write(f59,'-999 ');
   system.write(f60,'-999 '); system.write(f61,'-999 ');
   system.write(f62,'-999 '); system.write(f63,'-999 ');
   system.write(f64,'-999 '); system.write(f65,'-999 ');
   system.write(f66,'-999 '); system.write(f67,'-999 ');
   system.write(f68,'-999 '); system.write(f69,'-999 ');
   system.write(f70,'-999 '); system.write(f71,'-999 ');
   system.write(f72,'-999 '); system.write(f73,'-999 ');
   system.write(f74,'-999 '); system.write(f75,'-999 ');
   system.write(f76,'-999 '); system.write(f77,'-999 ');
   system.write(f78,'-999 '); system.write(f79,'-999 ');
   system.write(f80,'-999 '); system.write(f81,'-999 ');
   system.write(f82,'-999 '); system.write(f83,'-999 ');
   system.write(f84,'-999 '); system.write(f85,'-999 ');
   system.write(f86,'-999 '); system.write(f87,'-999 '); system.write(f87a,'-999 ');
   system.write(f88,'-999 '); system.write(f88a,'-999 ');
   system.write(f89,'-999 ');
   system.write(f90,'-999 '); system.write(f91,'-999 ');  
   system.write(f92,'-999 '); system.write(f93,'-999 ');
   system.write(f94,'-999 '); system.write(f95,'-999 ');  
   system.write(f96,'-999 '); system.write(f97,'-999 ');
   system.write(f98,'-999 '); system.write(f99,'-999 ');
   system.write(f100,'-999 ');
  end
  else
  begin
   j:=i-toplaycurrentrun;  {Rockaddition: check if adaptation needed ???  }
   system.write(f1,dataYearOut.alkalinity[j]:15:8,' ');
   system.write(f2,dataYearOut.BS[j]:15:8,' ');
   system.write(f3,dataYearOut.AXSE[1][j]:15:8,' ');
   {pf23: convert back to massfraction mfr=mass/(RHO*DLZ*(1-stonesfr-gravelfr))}
   calcdum:=dataYearOut.CALCITEA[j]/(dataYearOut.RHO[j]*dataYearOut.DLZ*(1-(dataYearOut.STONES[j]+dataYearOut.GRAVEL[j])/100));
   system.write(f4,calcdum:15:8,' ');  system.write(f4a,(dataYearOut.CALCITEA[j]*dataYearOut.RHO[j]*datayearout.dlz),' ');
   system.write(f5,dataYearOut.SEA[1][j]:15:8,' ');
   system.write(f6,dataYearOut.SEA[5][j]:15:8,' ');
   system.write(f7,dataYearOut.CO3[j]:15:8,' ');
   system.write(f8,dataYearOut.DISSOLVEDCATEQ[j]:15:8,' ');
   system.write(f9,dataYearOut.EC[j]:15:8,' ');
   system.write(f10,dataYearOut.ESP[j]:15:8,' ');
   system.write(f11,dataYearOut.ET[j]:15:8,' ');
   system.write(f12,dataYearOut.ACEC[j]:15:8,' ');
   system.write(f13,dataYearOut.FLUX[j]:15:8,' '); 
   {pf10: convert back to massfraction}
   gypsdum:=dataYearOut.GYPA[j]/(dataYearOut.RHO[j]*dataYearOut.DLZ*(1-(dataYearOut.STONES[j]+dataYearOut.GRAVEL[j])/100));
   system.write(f14,gypsdum:15:8,' ');  system.write(f14a,(dataYearOut.GYPA[j]*dataYearOut.RHO[j]*datayearout.dlz),' ');
   system.write(f15,dataYearOut.HCO3[j]:15:8,' ');
   system.write(f16,dataYearOut.AXSE[4][j]:15:8,' ');
   system.write(f17,dataYearOut.SEA[4][j]:15:8,' ');
   system.write(f18,dataYearOut.AXSE[2][j]:15:8,' ');
   system.write(f19,dataYearOut.SEA[2][j]:15:8,' ');
   system.write(f20,dataYearOut.AXSE[3][j]:15:8,' ');
   system.write(f21,dataYearOut.SEA[3][j]:15:8,' ');
   system.write(f22,dataYearOut.PH[j]:15:8,' ');
   system.write(f23,dataYearOut.AH[j]:15:8,' ');
   system.write(f24,dataYearOut.RDF[j]:15:8,' ');
   system.write(f25,dataYearOut.SAR[j]:15:8,' ');
   system.write(f26,dataYearOut.SEA[6][j]:15:8,' ');
   system.write(f27,dataYearOut.AW[j]:15:8,' ');
   system.write(f28,dataYearOut.RHO[j]:15:8,' ');
   klei1:=dataYearOut.clay[j]; silt1:=dataYearOut.silt[j];
   {old: zand1:=100-1.7*dataYearOut.OC[j]-silt1-klei1; }
   zand1:=100-1.7*dataYearOut.OC[j]-silt1-klei1;  {markOC}
   totmin:=klei1+silt1+zand1;
   dataYearOut.klei2[j]:=100*klei1/totmin;
   dataYearOut.silt2[j]:=100*silt1/totmin;
   dataYearOut.zand2[j]:=100*zand1/totmin;
   system.write(f29,dataYearOut.klei2[j]:15:8,' ');
   system.write(f30,dataYearOut.silt2[j]:15:8,' ');
   system.write(f31,dataYearOut.zand2[j]:15:8,' ');
   system.write(f32,dataYearOut.OC[j]:15:8,' ');
   {Mean particle density}
   y1:=1.7*dataYearOut.OC[j]/dataYearOut.RHOS3;
   y2:=silt1/dataYearOut.RHOS1;
   y3:=klei1/dataYearOut.RHOS2;
   y4:=zand1/dataYearOut.RHOS1;
   sy:=y1+y2+y3+y4;
   dataYearOut.PARDEN[j]:=100/(y3+y2+y4+y1);
   {Calculate porosity}
   dataYearOut.porosity[j]:=((100-DataYearOut.stoneperc[j])/100)*(1-dataYearOut.RHO[j]/dataYearOut.PARDEN[j]);  {correct?}
   system.write(f33,dataYearOut.porosity[j]:15:8,' ');
   {volumetric clay content}
   dataYearOut.volpercclay[j]:=(100-dataYearOut.porosity[j])*y3/sy;
   system.write(f34,dataYearOut.volpercclay[j]:15:8,' ');
   system.write(f35,dataYearOut.ClayDispInd[j]:15:8,' ');
   system.write(f36,dataYearOut.TempGem[j]:15:8,' ');
   system.write(f37,dataYearOut.RothC.DPMpool[j]:15:8,' ');
   system.write(f38,dataYearOut.RothC.RPMpool[j]:15:8,' ');
   system.write(f39,dataYearOut.RothC.BIOpool[j]:15:8,' ');
   system.write(f40,dataYearOut.RothC.HUMpool[j]:15:8,' ');
   system.write(f41,dataYearOut.RothC.IOMpool[j]:15:8,' ');
   system.write(f42,dataYearOut.APCO2[j]:15:8,' ');
   system.write(f43,dataYearOut.AXSE[12][j]:15:8,' ');
   system.write(f44,dataYearOut.WE.Gibbsitepool[j]:15:8,' ');   {/1000 removed}
   system.write(f44a,0.07800*dataYearOut.WE.Gibbsitepool[j],' ');
   {was excl 1000 en *77.979/(1000000*dataYearOut.DLZ*dataYearOut.RHO[j])}
   system.write(f45,dataYearOut.SEA[12][j]:15:8,' ');
   {TESTTDD line below: change tested target variable of type Profile}
   {delta14C: first calculate it}
   if (dataYearOut.RothC.DPMpool[j]+dataYearOut.RothC.RPMpool[j]+
                dataYearOut.RothC.BIOpool[j]+dataYearOut.RothC.HUMpool[j]+
                dataYearOut.RothC.IOMpool[j])<=0
   then delta14c:=0
   else
     delta14c:=(dataYearOut.RothC.DPMXpool[j]+dataYearOut.RothC.RPMXpool[j]+
                dataYearOut.RothC.BIOXpool[j]+dataYearOut.RothC.HUMXpool[j]+
                dataYearOut.RothC.IOMXpool[j])/
               (dataYearOut.RothC.DPMpool[j]+dataYearOut.RothC.RPMpool[j]+
                dataYearOut.RothC.BIOpool[j]+dataYearOut.RothC.HUMpool[j]+
                dataYearOut.RothC.IOMpool[j]);
   system.write(f46,(delta14c-1)*1000:15:8,' '); {includes a re-conversion into "true" delta14c}
   {calculate age; relative to the year of measurement}

   if delta14c=0
   then age:=-999
   else age:=-(5568/LN(2))*LN((delta14c/1000)+1)*EXP((yearAD-1950)/(5730/LN(2)));
   system.write(f47,age:7:1,' ');
   system.write(f48,CIA[j]:6:3,' ');
   system.write(f49,indexB[j]:6:5,' ');
   system.write(f50,CIW[j]:6:3,' ');
   system.write(f51,PIA[j]:6:3,' ');
   system.write(f52,WI[j]:6:5,' ');
   system.write(f53,Fk_DPM[j]:9:4,' ');
   system.write(f54,Fk_RPM[j]:9:4,' ');
   system.write(f55,Fk_BIO[j]:9:4,' ');
   system.write(f56,Fk_HUM[j]:9:4,' ');
   system.write(f57,Fk_IOM[j]:9:4,' ');
   system.write(f58,CPA[j]:6:3,' ');
   {Available Water}
   P1:=101.9744;    {equivalent in cm water of pF 2.0}
   P2:=16161.85781; {equivalent in cm water of pF4.2}
   m:=1-(1/untProcesses.VGNen[j+1]);
   AvailableWater[j]:=(untProcesses.VGNtr[j+1]+(untProcesses.VGNts[j+1]-untProcesses.VGNtr[j+1])/
                      (power((1+power((untProcesses.VGNal[j+1]*P1),untProcesses.VGNen[j+1])),m)))-
                      (untProcesses.VGNtr[j+1]+(untProcesses.VGNts[j+1]-untProcesses.VGNtr[j+1])/
                      (power((1+power((untProcesses.VGNal[j+1]*P2),untProcesses.VGNen[j+1])),m)));
   system.write(f59,AvailableWater[j]:7:5,' ');
   system.write(f60,DataYearOut.WE.WE_Capool[j],' ');
   system.write(f61,DataYearOut.WE.WE_Mgpool[j],' ');
   system.write(f62,DataYearOut.WE.WE_Napool[j],' ');
   system.write(f63,DataYearOut.WE.WE_Kpool[j],' ');
   system.write(f64,DataYearOut.WE.WE_Fepool[j],' ');
   system.write(f65,DataYearOut.WE.WE_Sipool[j],' ');
   system.write(f66,DataYearOut.WE.WE_Alpool[j],' ');
   system.write(f67,DataYearOut.WE.MinPools[1][j],' '); {albite}
   system.write(f68,DataYearOut.WE.MinPools[2][j],' ');
   system.write(f69,DataYearOut.WE.MinPools[3][j],' ');
   system.write(f70,DataYearOut.WE.MinPools[4][j],' ');
   system.write(f71,DataYearOut.WE.MinPools[5][j],' ');
   system.write(f72,DataYearOut.WE.MinPools[6][j],' ');
   system.write(f73,DataYearOut.WE.MinPools[7][j],' ');
   system.write(f74,DataYearOut.WE.MinPools[8][j],' ');
   system.write(f75,DataYearOut.WE.MinPools[9][j],' ');
   system.write(f76,DataYearOut.WE.MinPools[10][j],' ');
   system.write(f77,DataYearOut.WE.MinPools[11][j],' ');
   system.write(f78,DataYearOut.WE.MinPools[12][j],' ');
   system.write(f79,DataYearOut.WE.MinPools[13][j],' ');
   system.write(f80,DataYearOut.WE.MinPools[14][j],' '); {fayalite}
   {calculate pH-H2O in 1:5 dilution as in lab protocol}
   system.write(f81,phlab[j]:15:8,' '); {pH-H2O_lab in 1:5 dilution}
   system.write(f82,DataYearOut.WE.MinPools[15][j],' '); {amorphite}
   system.write(f83,DataYearOut.WE.MinPools[16][j],' '); {otherite}
   system.write(f84,TRB[j],' '); {Total reserve of bases}
   system.write(f85,dataYearOut.rothc.x2[j],' '); {rate modifier MoistureTemperature, 31-12}
   system.write(f86,dataYearOut.rothc.x3[j],' '); {rate modifier Geochemistry, 31-12}
   system.write(f87,dataYearOut.WE.WE_FePedogenic[j],' ');
   system.write(f87a,0.08885*dataYearOut.WE.WE_FePedogenic[j],' ');
   system.write(f88,dataYearOut.WE.WE_SiPedogenic[j],' ');   
   system.write(f88a,0.0280855*dataYearOut.WE.WE_SiPedogenic[j],' ');
   system.write(f89,dataYearOut.SEA[14][j]:15:8,' ');     
   system.write(f90,dataYearOut.SEA[13][j]:15:8,' ');
   system.write(f91,dataYearOut.SEA[15][j]:15:8,' ');  
   system.write(f92,untProcesses.Fp1[j+1],' ');
   system.write(f93,untProcesses.Fp2[j+1],' ');
   system.write(f94,untProcesses.Fp3[j+1],' ');
   system.write(f95,untProcesses.Fp4[j+1],' ');
   system.write(f96,DataYearOut.STONES[j],' ');
   system.write(f97,DataYearOut.GRAVEL[j],' ');
   system.write(f98,DataYearOut.freezecounter[j],' ');    
   system.write(f99,DataYearOut.rootcounter[j],' ');
   system.write(f100,DataYearOut.stoneperc[j],' ');
  end;
 end;
 system.writeln(f1);  system.writeln(f2);
 system.writeln(f3);  system.writeln(f4);  system.writeln(f4a);
 system.writeln(f5);  system.writeln(f6);  system.writeln(f7); system.writeln(f8);
 system.writeln(f9);  system.writeln(f10); system.writeln(f11); system.writeln(f12);
 system.writeln(f13); system.writeln(f14); system.writeln(f14a);
 system.writeln(f15); system.writeln(f16);
 system.writeln(f17); system.writeln(f18); system.writeln(f19); system.writeln(f20);
 system.writeln(f21); system.writeln(f22); system.writeln(f23); system.writeln(f24);
 system.writeln(f25); system.writeln(f26); system.writeln(f27); system.writeln(f28);
 system.writeln(f29); system.writeln(f30); system.writeln(f31); system.writeln(f32);
 system.writeln(f33); system.writeln(f34); system.writeln(f35); system.writeln(f36); 
 system.writeln(f37); system.writeln(f38); system.writeln(f39); system.writeln(f40);
 system.writeln(f41); system.writeln(f42);
 system.writeln(f43); system.writeln(f44); system.writeln(f44a); system.writeln(f45);
 system.writeln(f46); system.writeln(f47);
 system.writeln(f48); system.writeln(f49);
 system.writeln(f50); system.writeln(f51);
 system.writeln(f52); system.writeln(f53);
 system.writeln(f54); system.writeln(f55);
 system.writeln(f56); system.writeln(f57);
 system.writeln(f58); system.writeln(f59);
 system.writeln(f60); system.writeln(f61);
 system.writeln(f62); system.writeln(f63);
 system.writeln(f64); system.writeln(f65);
 system.writeln(f66); system.writeln(f67);
 system.writeln(f68); system.writeln(f69);
 system.writeln(f70); system.writeln(f71);
 system.writeln(f72); system.writeln(f73);
 system.writeln(f74); system.writeln(f75);
 system.writeln(f76); system.writeln(f77);
 system.writeln(f78); system.writeln(f79);
 system.writeln(f80); system.writeln(f81);
 system.writeln(f82); system.writeln(f83);
 system.writeln(f84); system.writeln(f85);
 system.writeln(f86); system.writeln(f87);   system.writeln(f87a);
 system.writeln(f88); system.writeln(f88a);  system.writeln(f89);
 system.writeln(f90); system.writeln(f91); 
 system.writeln(f92); system.writeln(f93);
 system.writeln(f94); system.writeln(f95);    
 system.writeln(f96); system.writeln(f97);
 system.writeln(f98); system.writeln(f99);
 system.writeln(f100);
{If GUItrace=1 then wwwriteln} singletracewriteln('Leaving WriteTimeDepth');
end;

procedure TfrmMAIN.mmDOSClick(Sender: TObject);
{void: was used to open an older MSDOS version of SoilGen}
begin
  largeDOS:=not(largeDOS);
  if largeDOS then
  begin
    mmDOS.Top:=406; mmDOS.Height:=150;
  end
  else
  begin 
    mmDOS.Top:=536; mmDOS.Height:=20;
  end;
end;

procedure TfrmMAIN.EffectSaltLoss;
{calculates the effect of loss/addition of calcite and gypsum on texture}
var oldRHO,oldAW:Profile;
{calculates the effect of loss/addition of calcite and gypsum on texture}
begin
{If GUItrace=1 then wwwriteln} singletracewriteln('Entering EffectSaltLoss');
  oldRHO:=DataYearIn.RHO; oldAW:=DataYearIn.AW;
  EffectSaltLossTexture;
  {1b. adapt soil physical relations pF-theta-K.
       This is done by fitting the Pedotransferfunctions of CLAY,SILT,OC,RHO}
  {This happens automatically via the SoilGen variable: MODRET[1..KK] has value 5, thus the
   continuous PTF by Rawls & Braakensiek 1985 on silt, clay, OC en RHO are applied before each
   new (yearly) SoilGen-run.}
{If GUItrace=1 then wwwriteln} singletracewriteln('Leaving EffectSaltLoss');
end;

procedure TfrmMAIN.EffectSaltLossTexture;
{calculates the effect of loss/gains of calcite and gypsum
on the mass of clay/silt/sand and bulk density per compartment }
{Result (per compartment):
a. new clay%, silt%, oc%, volumetric water content
b. new bulk density
Assumptions: * mass conservation * volume conservation}

var i,j,imem:integer;
    oldCLAYm,oldSILTm,oldSANDm,oldOCm,oldWATERm,
    newCLAY,newSILT,newSAND,newOC,newWATER,porosity,water,delCaCO3m,delCaSO4m,totCSS:Profile;
    parden,totwater,totporosity,testrho,tempCLAY, tempOC, tempPARDEN, tempdeltclay:real;
    checkvolumeOK,checkRHOok,newcompflag:boolean;
    strimem:string;
begin
{If GUItrace=1 then wwwriteln} singletracewriteln('Entering EffectSaltLossTexture');
  {Decide on indicator if in the previous year a rock compartment was added to the soil.
   In such case (newcompflag=true), there is no "old*" value}
  newcompflag:=((FileExistsUTF8('Bedrock.txt')and (Rock.rockporosityperc>porosity_threshold)));
{1. Initialisation}
{1. Internal variabeles are initialised.
    old*m and del*m contain * as mass(-loss) per compartment,m2}
   for i:=1 to DataYearIn.NoLayers do
   begin
     delCaCO3m[i]:=(DataYearIn.oldCALCITEA[i]-DataYearIn.CALCITEA[i]);
     delCaSO4m[i]:=(DataYearIn.oldGYPA[i]-DataYearIn.GYPA[i]);
     {clay,silt,sand,oc are in % whilst Ca* are fractions!}
     oldCLAYm[i]:=DataYearIn.CLAYm[i];
     oldSILTm[i]:=DataYearIn.SILTm[i];
     oldSANDm[i]:=DataYearIn.SANDm[i];
     totCSS[i]:=oldCLAYm[i]+oldSILTm[i]+oldSANDm[i];
     oldOCm[i]:=DataYearIn.OCm[i];
     oldWATERm[i]:=DataYearIn.AW[i]*DataYearIn.DLZ;
   end;
 {2. Correct for lost CaCO3 and CaSO4 (mass}
 {2a. depending on the set option, the mass is corrected}
   for i:=1 to DataYearIn.NoLayers do
   begin
      case rdgrpSALTLOSS.ItemIndex of
      0: begin {all mass loss at the cost of the clay fraction}
           newCLAY[i]:=oldCLAYm[i]-delCaCO3m[i]-delCaSO4m[i];
           newSILT[i]:=oldSILTm[i];
           newSAND[i]:=oldSANDm[i];
         end;
      1: begin {all mass loss at the proportional cost of Clay, silt, sand}
           newCLAY[i]:=oldCLAYm[i]-((delCaSO4m[i]+delCaCO3m[i])*oldCLAYm[i]/totCSS[i]);
           newSILT[i]:=oldSILTm[i]-((delCaSO4m[i]+delCaCO3m[i])*oldSILTm[i]/totCSS[i]);
           newSAND[i]:=oldSANDm[i]-((delCaSO4m[i]+delCaCO3m[i])*oldSANDm[i]/totCSS[i]);
         end;
      2: begin {mass loss at the proportional cost of sand and silt only}
           newCLAY[i]:=oldCLAYm[i];
           newSILT[i]:=oldSILTm[i]-((delCaSO4m[i]+delCaCO3m[i])*oldSILTm[i]/(oldSILTm[i]+oldSANDm[i]));
           newSAND[i]:=oldSANDm[i]-((delCaSO4m[i]+delCaCO3m[i])*oldSANDm[i]/(oldSILTm[i]+oldSANDm[i]));
         end;
      end;
      newOC[i]:=oldOCm[i];
      newWATER[i]:=oldWATERm[i];
   end;
   {2a2: here we added a check: does the new RHO[i] not become too high:
             (RHOmax=(1-0.05)*PARDEN), where 0.05 is the minimal porosity that should be in every compartment
              to allow for the PTF for h-theta-K to be valid.
              In case the check is negative, the extra mass of clay-silt-sand (incremented
              by delCaSO4 or delCaCO3 to give newCLAY, newSAND and newSILT) could be added to the layer above.
              This mimics the upward growth of a calcic/gypsic horizon due to water stagnation below.}
   repeat
      imem:=-999; checkRHOok:=true;
      for i:=1 to DataYearIn.NoLayers do
      begin
        testRHO:=(newCLAY[i]+newSILT[i]+newSAND[i]+1.7*newOC[i])/(DataYearIn.DLZ*(100-dataYearIn.Stones[i]-dataYearIn.Gravel[i])/100);
        {undone: testRHO:=HYPRESRHO(100*newCLAY[i]/(newCLAY[i]+newSILT[i]+newSAND[i]+1.7*newOC[i]),1.7*100*newOC[i]/(newCLAY[i]+newSILT[i]+newSAND[i]+1.7*newOC[i]));  }
        {tempCLAY:=100*newCLAY[i]/(testRHO*DataYearIn.DLZ);
        tempOC:=100*newOC[i]/(testRHO*DataYearIn.DLZ);}
        tempCLAY:=100*newCLAY[i]/(newCLAY[i]+newSILT[i]+newSAND[i]+1.7*newOC[i]);
        tempOC:=1.7*100*newOC[i]/(newCLAY[i]+newSILT[i]+newSAND[i]+1.7*newOC[i]);
        tempPARDEN:=100/((tempCLAY/DataYearIn.RHOS2)+
                     ((100-tempCLAY-1.7*tempOC)/DataYearIn.RHOS1)+
                     (1.7*tempOC/DataYearIn.RHOS3));
        if testRHO>=(1-0.05)*tempPARDEN then
        begin imem:=i; checkRHOok:=false; end;
      end;
      str(imem,strimem);
      {at this point, if the bulk density in any layer > (0.95*particle density)
       then checkRHOok=false and imem is the lowermost layer where this is the case}
      {now correct RHO for the lowermost layer with too high RHO
       by adding the increased mass to the layer above instead}
      if imem>0 then
      begin
       case rdgrpSALTLOSS.ItemIndex of
       0: begin {all mass loss at the cost of the clay fraction}
            newCLAY[imem]:=oldCLAYm[imem];
            newCLAY[imem-1]:=newCLAY[imem-1]-delCaCO3m[imem]-delCaSO4m[imem];
            newSILT[imem]:=oldSILTm[imem];
            newSAND[imem]:=oldSANDm[imem];
          end;
       1: begin {all mass loss at the proportional cost of Clay, silt, sand}
            newCLAY[imem]:=oldCLAYm[imem];
            newCLAY[imem-1]:=newCLAY[imem-1]-((delCaSO4m[imem]+delCaCO3m[imem])*oldCLAYm[imem]/totCSS[imem]);
            newSILT[imem]:=oldSILTm[imem];
            newSILT[imem-1]:=newSILT[imem-1]-((delCaSO4m[imem]+delCaCO3m[imem])*oldSILTm[imem]/totCSS[imem]);
            newSAND[i]:=oldSANDm[i];
            newSAND[imem-1]:=newSAND[imem-1]-((delCaSO4m[imem]+delCaCO3m[imem])*oldSANDm[imem]/totCSS[imem]);
          end;
       2: begin {mass loss at the proportional cost of sand and silt only}
            newCLAY[imem]:=oldCLAYm[imem];
            newSILT[imem]:=oldSILTm[imem];
            newSILT[imem-1]:=newSILT[imem-1]-((delCaSO4m[imem]+delCaCO3m[imem])*oldSILTm[imem]/(oldSILTm[imem]+oldSANDm[imem]));
            newSAND[imem]:=oldSANDm[imem];
            newSAND[imem-1]:=newSAND[imem-1]-((delCaSO4m[imem]+delCaCO3m[imem])*oldSANDm[imem]/(oldSILTm[imem]+oldSANDm[imem]));
          end;
       end;
       {also re-allocate the increments of calcite and gypsum!}
       DataYearIn.CALCITEA[imem]:=  DataYearIn.CALCITEA[imem]+delCaCO3m[imem];
       DataYearIn.CALCITEA[imem-1]:=DataYearIn.CALCITEA[imem-1]-delCaCO3m[imem];
       DataYearIn.GYPA[imem]:=  DataYearIn.GYPA[imem]+delCaSO4m[imem];
       DataYearIn.GYPA[imem-1]:=DataYearIn.GYPA[imem-1]-delCaSO4m[imem];
       {now the bulk density in layer imem can not be too high due to calcite or gypsum,
        but it can still be too high due to clay illuviation}
       {first check again}
        testRHO:=(newCLAY[imem]+newSILT[imem]+newSAND[imem]+1.7*newOC[imem])/(DataYearIn.DLZ*(100-DatayearIn.Stones[i]-dataYearIn.Gravel[i])/100);
        {undone: testRHO:=HYPRESRHO(100*newCLAY[imem]/(newCLAY[imem]+newSILT[imem]+newSAND[imem]+1.7*newOC[imem]),
                           1.7*100*newOC[imem]/(newCLAY[imem]+newSILT[imem]+newSAND[imem]+1.7*newOC[imem]));}
       { tempCLAY:=100*newCLAY[imem]/(testRHO*DataYearIn.DLZ);
        tempOC:=100*newOC[imem]/(testRHO*DataYearIn.DLZ); }
        tempCLAY:=100*newCLAY[imem]/(newCLAY[imem]+newSILT[imem]+newSAND[imem]+1.7*newOC[imem]);
        tempOC:=1.7*100*newOC[i]/(newCLAY[imem]+newSILT[imem]+newSAND[imem]+1.7*newOC[imem]);
        tempPARDEN:=100/((tempCLAY/DataYearIn.RHOS2)+
                     ((100-tempCLAY-1.7*tempOC)/DataYearIn.RHOS1)+
                     (1.7*tempOC/DataYearIn.RHOS3));
        if testRHO>=(1-0.05)*tempPARDEN then checkRHOok:=false else checkRHOok:=true;
       {end of check: if checkRHOok is still false then illuviated clay has filled
        too many pores in compartment <imem>. Re-allocate clay partly to overlying compartment.
        Algorithm: decrease clay mass in imem in steps of 1% of the originall mass,
                   increae clay mass in imem-1 with the same mass
                   check again on testRHO
                   until the check is OK}
        if not(checkRHOok) then
        begin
          tempdeltclay:=0.01*newclay[imem];
          repeat
             newCLAY[imem-1]:=newCLAY[imem-1]+tempdeltclay;
             newCLAY[imem]  :=newCLAY[imem]  -tempdeltclay;
             testRHO:=(newCLAY[imem]+newSILT[imem]+newSAND[imem]+1.7*newOC[imem])/(DataYearIn.DLZ*(100-DatayearIn.Stones[i]-dataYearIn.Gravel[i])/100);
             {undone: testRHO:=HYPRESRHO(100*newCLAY[imem]/(newCLAY[imem]+newSILT[imem]+newSAND[imem]+1.7*newOC[imem]),
                                1.7*100*newOC[imem]/(newCLAY[imem]+newSILT[imem]+newSAND[imem]+1.7*newOC[imem]));  }
             {tempCLAY:=100*newCLAY[imem]/(testRHO*DataYearIn.DLZ);
             tempOC:=100*newOC[imem]/(testRHO*DataYearIn.DLZ);}
             tempCLAY:=100*newCLAY[imem]/(newCLAY[imem]+newSILT[imem]+newSAND[imem]+1.7*newOC[imem]);
             tempOC:=1.7*100*newOC[i]/(newCLAY[imem]+newSILT[imem]+newSAND[imem]+1.7*newOC[imem]);
             tempPARDEN:=100/((tempCLAY/DataYearIn.RHOS2)+
                         ((100-tempCLAY-1.7*tempOC)/DataYearIn.RHOS1)+
                         (1.7*tempOC/DataYearIn.RHOS3));
             if testRHO>=(1-0.05)*tempPARDEN then checkRHOok:=false else checkRHOok:=true;
          until checkRHOok;
        end;
       debuglog('Corrected for too high bulk density in compartment '+strimem);
      end;
       {in the next repeat-until loop the check and possible correction is repeated}
   until checkRHOok;

         MVAR(GlobalTime*10000+Datayearin.nolayers*100,' MAIN, voor EffectSaltLossTexture: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
         {MVAR(GlobalTime*10000+Datayearout.nolayers*100,' MAIN, voor EffectSaltLossTexture: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);}
 {2b. convert Clay, Silt, OC to weight percentages.
      First estimate new bulk density assuming constant volume
      Then convert to weight percentages}
   for i:=1 to DataYearIn.NoLayers do
   begin
      DataYearIn.RHO[i]:=(newCLAY[i]+newSILT[i]+newSAND[i]+1.7*newOC[i])/(DataYearIn.DLZ*(100-DatayearIn.Stones[i]-dataYearIn.Gravel[i])/100);
      {undone: DataYearIn.RHO[i]:=HYPRESRHO(100*newCLAY[i]/(newCLAY[i]+newSILT[i]+newSAND[i]+1.7*newOC[i]),
                                1.7*100*newOC[i]/(newCLAY[i]+newSILT[i]+newSAND[i]+1.7*newOC[i]));  }
      {RHO applies to fine earth only, thus DLZ as estimator for volume must be corrected for stones}
      {DataYearIn.CLAY[i]:=100*newCLAY[i]/(DataYearIn.RHO[i]*DataYearIn.DLZ);}
      DataYearIn.CLAY[i]:=100*newCLAY[i]/(newCLAY[i]+newSILT[i]+newSAND[i]+1.7*newOC[i]);
      {also adapt number of grains per size class: clay}
      if physweathallowed
      then PW_K[11,i]:=round(PW_K[11,i]*newClay[i]/oldClaym[i]);
      CLAYm[i]:=newCLAY[i];
      {DataYearIn.SILT[i]:=100*newSILT[i]/(DataYearIn.RHO[i]*DataYearIn.DLZ);  }
      DataYearIn.SILT[i]:=100*newSILT[i]/(newCLAY[i]+newSILT[i]+newSAND[i]+1.7*newOC[i]);
      { adapt number of grains per size class: 5 silt size classes}
      if physweathallowed
      then for j:=6 to 10 do PW_K[j,i]:=round(PW_K[j,i]*newSilt[i]/oldSiltm[i]);
      SILTm[i]:=newSILT[i]; 
      { adapt number of grains per size class: 5 sand size classes}
      if physweathallowed
      then for j:=1 to 5 do PW_K[j,i]:=round(PW_K[j,i]*newSand[i]/oldSandm[i]);
      SANDm[i]:=newSAND[i];
      DataYearIn.SAND[i]:=100*newSAND[i]/(newCLAY[i]+newSILT[i]+newSAND[i]+1.7*newOC[i]);
      fineSANDm[i]:=(PW_K[4,i]*power((PW_X[4]/100000),3)*PW_PARDEN[4]/100)+ {eqv mass[4,i]}
              (PW_K[5,i]*power((PW_X[5]/100000),3)*PW_PARDEN[5]/100); {eqv mass[5,i]}
      {DataYearIn.OC[i]:=100*newOC[i]/(DataYearIn.RHO[i]*DataYearIn.DLZ); }{markOC}
      DataYearIn.OC[i]:=1.7*100*newOC[i]/(newCLAY[i]+newSILT[i]+newSAND[i]+1.7*newOC[i]);
      OCm[i]:=newOC[i];
   end;

            MVAR(GlobalTime*10000+Datayearin.nolayers*100,' MAIN, na EffectSaltLossTexture: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
            {MVAR(GlobalTime*10000+Datayearout.nolayers*100,' MAIN, na EffectSaltLossTexture: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]); }
 {2c. Convert Water to volume fraction.
      Check if the maximal porosity with new RHO is not exceeded}
   checkvolumeOK:=true;
   for i:=1 to DataYearIn.NoLayers do
   begin
      parden:=100/((DataYearIn.CLAY[i]/DataYearIn.RHOS2)+
                   ((100-DataYearIn.CLAY[i]-1.7*DataYearIn.OC[i])/DataYearIn.RHOS1)+
                   (1.7*DataYearIn.OC[i]/DataYearIn.RHOS3));
      {same calc particle density parden as in SoilGen}
      porosity[i]:=max(0.1,((100-DataYearIn.stoneperc[i])/100)*(1-(DataYearIn.RHO[i]/parden))); {do not allow negative porosities due to high OM...}
      water[i]:=newWATER[i]/DataYearIn.DLZ;
      {water[i] contains now volumetric moisture content ; check with porosity still needed}
      if water[i]>porosity[i] then checkvolumeOK:=false;
   end;
   if not(checkvolumeOK) then {porosity in minimally 1 compartment too low for water content}
   begin
     totwater:=0; totporosity:=0;
     for i:=1 to DataYearIn.NoLayers do
     begin totwater:=totwater+water[i]; totporosity:=totporosity+porosity[i]; end;
     if totwater<=totporosity then {there is enough total porosity for all water}
     begin
       {fill in the mixed zone in each compartment the same fraction of the
        pore volume with water so that mass is conserved}
       for i:=1 to DataYearIn.NoLayers do
       begin
          DataYearIn.AW[i]:=porosity[i]*totwater/totporosity;
       end;
     end
     else {problem: there is more water than can be stored in the pores}
     begin
        {write a message!}
        dshowmessage('Year '+currentyear+'; Not fatal (effect salt gain/loss on texture): Cannot redistribute water; porosity too low!');
        {solution: all pores are filled with water, but surplus of water is not
         redistributed over other profile parts; thus the mass balance error is
         limited tot water and does not include chemical masses}
         for i:=1 to DataYearIn.NoLayers do
         begin
           DataYearIn.AW[i]:=porosity[i];
         end;
     end;
   end
   else {porosity check OK}
   begin
     {redistribute the water definitively with the calculated mass distribution}
     for i:=1 to DataYearIn.NoLayers do DataYearIn.AW[i]:=water[i];
   end;  
{If GUItrace=1 then wwwriteln} singletracewriteln('Leaving EffectSaltLossTexture');
end;

procedure TfrmMAIN.Beschrijf_scenario;
{writes the settings for the current scenario to a log file with default name "SoilGen.txt"}
var fturb,fclim,ffert,fsoft:textfile;
    year,dtop,dmax,dbot,fractop,fracmax,fracbot,dmass,mass,
    Xfac,BPyear,Pb,PEb,JanTb,JulTb,yearb:real;
    i,mm,node:integer;
    txt,txt2:string;
    FileVerInfo: TFileVersionInfo;
begin
  for i:=1 to length(FirstLeachCFileName) do
    if FirstLeachCFileName[i]='\' then mm:=i;
  for i:=1 to length(FirstLeachCFileName) do
    if FirstLeachCFileName[i]='/' then mm:=i;
  FileVerInfo:=TFileVersionInfo.Create(nil);
  try
    FileVerInfo.filename :=AppendPathDelim(startdir)+'SoilGen.exe';
    txt:=FileVerInfo.VersionStrings.Values['FileVersion'];
  finally
    FileVerInfo.Free;
  end;
  if length(txt)>0 then txt:=' version: '+txt;
  system.writeln(fmonit,versionid+' '+txt+'.');
  system.writeln(fmonit,'Reference for SoilGen1:'+
                        ' Finke, P.A. and J.L. Hutson, 2008.'+
                        ' Modelling Soil Genesis in Calcareous Loess.'+
                        ' Geoderma 145: 462-479.');
  system.writeln(fmonit,'Reference for SoilGen2:'+
                        ' Finke, P.A., 2012.'+
                        ' Modeling the genesis of Luvisols as a function '+
                        ' of topographic position in loess parent material. '+
                        ' Quaternary International 265: 3-17');
  system.writeln(fmonit,'Reference for SoilGen3:'+
                        ' Peter Finke. 2024. Modelling soil development under global change.'+
                        ' SpringerBriefs in Earth System Sciences 10032.'+
                        ' ISBN: 978-3-031-55582-4. https://doi.org/10.1007/978-3-031-55583-1 ');
  system.writeln(fmonit);
  if DataInit.IFLOW=1
  then system.writeln(fmonit,'SoilGen simulates water flow with the Richards Equation.')
  else system.writeln(fmonit,'SoilGen simulates water flow with the Addiscott tipping bucket model.');
  system.writeln(fmonit,rdgrpBBC.Caption,' ',rdgrpBBC.Items[rdgrpBBC.ItemIndex]); 
  system.writeln(fmonit);
  system.writeln(fmonit,'**** Scenario ****');
  system.writeln(fmonit,'Inputfile Year1 soil chemistry  : ',FirstLeachCFileName);
  system.Writeln(fmonit,'Chemical equilibrium constants  : ',EQFileName);
  system.Writeln(fmonit,'General data for weathering submodel: ',FirstWeatherFileName);
  system.writeln(fmonit,'Inputfile Year1 weathering submodel : ',
                         copy(FirstLeachCFileName,1,mm),'WE000001');
  system.writeln(fmonit,'General data for soil C submodel: ',FirstRothCFileName);
  system.writeln(fmonit,'Inputfile Year1 soil C submodel : ',
                         copy(FirstLeachCFileName,1,mm),'RC000001');
  system.writeln(fmonit);
  if fileexists('continue.rec') then
  begin
    system.writeln(fmonit,'The above files were only used for initialization;');
    system.Writeln(fmonit,'This is a continued run and the final data of the preceding run were read from <continue.rec>');
    system.writeln(fmonit,'Scenario specifications of the new run are given below:');
  end;
  system.writeln(fmonit,'Simulation period: ',spndtMultipleYears.Value,'BP - present.');
  system.writeln(fmonit);
  if fileexists('InitEcto.txt') then
  begin
    system.writeln(fmonit,'Intial amount of ectorganic C over the various pools read from InitEcto.txt');
    system.writeln(fmonit);
  end;
  {bioturbation}
  if bioturbfile then
  begin
    system.writeln(fmonit,'Bioturbation   : YES (specified hereunder):');
    system.assign(fturb,'bioturbINT.txt'); system.reset(fturb);
    system.writeln(fmonit,'YearBP   -- depth of * turbation (mm) --   --- annual mass fraction turbated ---  Mg/ha,yr');
    system.writeln(fmonit,'         1:shallow  2:maximal  3:deepest   (depth 1)     (depth 2)     (depth 3)');
    repeat
       system.Readln(fturb,year,dtop,dmax,dbot,fractop,fracmax,fracbot,txt2);
       BPYear:=spndtMultipleYears.Value-year+1;
       if BPyear>=0 then
       BEGIN
        if (BatchModeYN) {frac* already fractions if bioturbINT.txt is offered to program in batchmode}
           and (pos('SET',txt2)>0)
        then system.Write(fmonit,BPyear:5:0,'    ',dtop:5:0,'      ',dmax:5:0,'      ',dbot:5:0,'       ',
                                 fractop:7:4,'       ',fracmax:7:4,'       ',fracbot:7:4,'     ');
        if not(BatchModeYN)
        then system.Write(fmonit,BPyear:5:0,'    ',dtop:5:0,'      ',dmax:5:0,'      ',dbot:5:0,'       ',
                                 fractop/1000:7:4,'       ',fracmax/1000:7:4,'       ',fracbot/1000:7:4,'     ');
        mass:=0;
        if batchmodeYN then Xfac:=1 else Xfac:=1000;
        for mm:=round(dtop) to round(dbot) do
        begin
         if mm<=round(dmax)
         then
         begin
          if round(dmax)<>round(dtop)
          then dmass:=FRACTOP+(FRACMAX-FRACTOP)*(dmax-mm)/(dmax-dtop)
          else dmass:=(FRACMAX+FRACTOP)/2
         end
         else
         begin
          if round(dmax)<>round(dbot)
          then dmass:=FRACMAX+(FRACBOT-FRACMAX)*(dbot-mm)/(dbot-dmax)
          else dmass:=(FRACBOT+FRACMAX)/2
         end;
         node:=trunc(mm/dataInit.DLZ)+1;
         mass:=mass+dmass*untGUI.DataYearIn.RHO[node]*10/Xfac;
        end;
        if batchmodeYN and (pos('SET',txt2)>0) then system.Writeln(fmonit,mass:5:1);
        if  not(BatchModeYN) then system.Writeln(fmonit,mass:5:1);

       END;
    until eof(fturb);
    system.Writeln(fmonit,'Nonspecified years: linear interpolation or constant continuation.');
    system.Writeln(fmonit,'Attention: This scenario covers ',spndtMultipleYears.Value,' years BP - present!');
    closefile(fturb);
  end
  else system.writeln(fmonit,'Bioturbation   : NO.');
  writeln(fmonit);
  {climate correction}
  if climatefile then
  begin
    system.writeln(fmonit,'Climate variation: YES (specified hereunder):');
    system.assign(fclim,'Climate.txt'); system.reset(fclim);
    system.Readln(fclim); {header weg}
    system.writeln(fmonit,'YearBP  SimYear     P     EP   JanT  JulT  PlantsInC(t/ha,y) ManureInC(t/ha,y)    Vegetation      Period');
    repeat
        system.Readln(fclim,BPyear,Pb,PEb,JanTb,JulTb,txt);
        yearb:=spndtMultipleYears.Value-BPyear;
        str(yearb:6:0,txt2);
        if yearb<0
        then txt2:='No sim'
        else system.Writeln(fmonit,BPyear:6:0,'   ',txt2,' ',Pb:6:1,' ',PEb:6:1,' ',JanTb:5:1,' ',JulTb:5:1,' ',txt);
    until eof(fclim);
    system.Writeln(fmonit,'Nonspecified years: linear interpolation or constant continuation.');
    system.Writeln(fmonit,'Attention: This scenario covers ',spndtMultipleYears.Value,' years BP - present!');
    closefile(fclim);
    if chckbxSpecialClimate.Checked
       then
       begin
         system.Writeln(fmonit,'Monthly average temperatures read from ',monthlyTEMPname);
         system.Writeln(fmonit,'Monthly precipitation sums read from ',monthlyPRECname);
         system.Writeln(fmonit,'Monthly evaporation sums read from ',monthlyEVAPname);
       end;
  end
  else system.writeln(fmonit,'Climate variation: NO.');
  writeln(fmonit);
  {fertilization}
  if fertilfile then
  begin
    system.writeln(fmonit,'Inorganic fertilizers: YES (specified hereunder):');
    system.assign(ffert,'fertilization.txt'); system.reset(ffert);
    system.readln(ffert,txt);
    system.Writeln(fmonit,txt);
    repeat
        system.Readln(ffert,BPyear,txt);
        yearb:=spndtMultipleYears.Value-BPyear;
        str(BPyear:6:0,txt2);
        if yearb<0
        then txt2:='No sim'
        else system.Writeln(fmonit,txt2,' ',txt);
    until eof(ffert);
    system.Writeln(fmonit,'Nonspecified years: NO FERTILIZATION.');
    system.Writeln(fmonit,'Attention: This scenario covers ',spndtMultipleYears.Value,' years BP - present!');
    closefile(ffert);
  end
  else system.writeln(fmonit,'Inorganic fertilizers: NO.');
  writeln(fmonit);
  writeln(fmonit,'Protection mechanisms against organic matter decay:');
  for i:=0 to untDecayProtection.frmDecayProtection.mmReport.Lines.Count-1 do
    writeln(fmonit,frmDecayProtection.mmReport.Lines[i]);
  writeln(fmonit);
  if FileExistsUTF8('Bedrock.txt') { *Converted from FileExists*  } then
  begin
    writeln(fmonit,'Bedrock weathering (Soil production) simulated');
    for i:=0 to mmBedrock.Lines.Count-1 do
      writeln(fmonit,mmBedrock.Lines[i]);
    writeln(fmonit);
  end;
  {Events}
  if EventsFile then
  begin
    system.writeln(fmonit,'Erosion/deposition/shallow water table/VGNChange events: YES (specified hereunder):');
    system.writeln(fmonit,'Note that deposition is assumed to be gradual in reality when it only 1 layer is added and then will not bury the ectorganic layer');
    system.assign(ffert,'Events.txt'); system.reset(ffert);
    system.readln(ffert,txt);
    txt:='BPYear'+copy(txt,15,length(txt)-14);
    if length(txt)>75 then txt:=copy(txt,1,75)+' etc...';
    system.Writeln(fmonit,txt);
    repeat
        if not(eof(ffert))
        then
        begin
         system.Readln(ffert,yearb,txt);
         if length(txt)>75 then txt:=copy(txt,1,75)+' etc...(see file Events.txt)';
         BPyear:=spndtMultipleYears.Value-yearb+1;
         str(BPyear:6:0,txt2);
         end
        else begin BPyear:=0; txt2:=' '; txt:='No data in Events file!'; end;
        if BPyear<0
        then txt2:='No sim'
        else system.Writeln(fmonit,txt2,' ',txt);
    until eof(ffert);
    system.Writeln(fmonit,'Nonspecified years: NO EVENTS.');
    system.Writeln(fmonit,'Attention: This scenario covers ',spndtMultipleYears.Value,' years BP - present!');
    closefile(ffert);
  end
  else system.writeln(fmonit,'Events: NO.');
  system.writeln(fmonit);
  system.writeln(fmonit,'Site characteristics:');
  str(DataInit.slope,txt);   txt:='Slope angle (degrees)     : '+txt;
  system.writeln(fmonit,txt);
  str(DataInit.upSlopeBearing,txt); txt:='Upslope bearing (degrees) : '+txt;
  system.writeln(fmonit,txt);
  str(DataInit.downWindBearing,txt);  txt:='Downwind bearing (degrees): '+txt;
  system.writeln(fmonit,txt);
  str(DataInit.Latitude,txt);     txt:='Latitude (degrees)        : '+txt;
  system.writeln(fmonit,txt);

  system.writeln(fmonit);
  { Loss of calcite, gypsum will affect: }
  system.write(fmonit,'Loss of calcite, gypsum will affect: ');
  case rdgrpSALTLOSS.ItemIndex of
  0: system.writeln(fmonit,'Clay % only.');
  1: system.writeln(fmonit,'Clay, Silt and Sand percentages proportionally.');
  2: system.writeln(fmonit,'Silt and Sand % only.');
  end;
  writeln(fmonit);
  { Is clay migration simulated, and what setting was used }
  if chckbxClayMigration.Checked
  then  system.writeln(fmonit,'Clay migration simulated.')
  else system.writeln(fmonit,'No clay migration simulated.');
  writeln(fmonit);
  { Is physical weathering simulated, and what settings were used }
  if chckbxPhysWeath.Checked
  then system.writeln(fmonit,'Physical weathering (due to temperature fluctuations) simulated.')
  else system.writeln(fmonit,'No physical weathering simulated.');
  writeln(fmonit);
  if fileexists(AppendPathDelim(startdir)+'SoftConstants.txt') then
  begin
    writeln(fmonit,'Soft Constants (SoftConstants.txt):');
    system.assign(fsoft,AppendPathDelim(startdir)+'SoftConstants.txt'); system.reset(fsoft);
    system.readln(fsoft); {skip first line}
    repeat
      system.readln(fsoft,txt); system.Writeln(fmonit,txt);
    until eof(fsoft);
    closefile(fsoft);
  end
  else writeln(fmonit,'Soft Constants (SoftConstants.txt): No such file found so default values used.');
  writeln(fmonit);
  writeln(fmonit,'*** RESULTS ***');
  str(1000000+spndtMultipleYears.Value,txt);
  txt:=copy(txt,2,6);
  writeln(fmonit,'Model input per year : ');
  writeln(fmonit,'         ','files LC000001. to LC'+txt+'. (chemistry)');
  writeln(fmonit,'         ','files WE000001. to WE'+txt+'. (weathering)');
  writeln(fmonit,'         ','files RC000001. to RC'+txt+'. (C-pools)');
  writeln(fmonit,'Model output per year: ');
  writeln(fmonit,'         ','files LC000001.out to LC'+txt+'.out (chemistry)');
  writeln(fmonit,'         ','files WE000001.out to WE'+txt+'.out (weathering)');
  writeln(fmonit,'         ','files RC000001.out to RC'+txt+'.out (C-pools)');
  writeln(fmonit,'Ectorganic pools and CO2-emission: ');
  writeln(fmonit,'         file <Ectorganic.out>');
  writeln(fmonit,'Time-depth diagrams  :');
  for i:=0 to maxTDD-1 do
    if pos('none',cmbbxTimeDepth.Items[i])=0
    then writeln(fmonit,'         ',cmbbxTimeDepth.Items[i]+'.tdd');
end;

procedure TfrmMAIN.ReadCheckClimate;
{reads the climate file "climate.txt" and produces climateINT.txt with annual values by interpolation}
var BPyear,year,yeara,yearb,startBPYearSim,dy:integer;
    fclim,fclim2:textfile;
    P,PE,JanT,JulT,PlantIN,ManIN,Pa,Pb,PEa,PEb,JanTa,JanTb,JulTa,JulTb,PlantINa,PlantINb,
    ManINa,ManINb:real;
    txta,txtb: string;
begin
   {If GUItrace=1 then wwwriteln} singletracewriteln('Entering ReadCheckClimate');
   startBPYearSim:=spndtMultipleYears.Value;
   if fileexists('Climate.txt') then
   begin
     system.Assign(fclim2,'climateINT.txt'); system.Rewrite(fclim2);
     system.Assign(fclim,'Climate.txt');  system.Reset(fclim);
     system.Readln(fclim); {skip header line}
     system.Readln(fclim,BPyear,Pb,PEb,JanTb,JulTb,PlantInb,ManINb,txtb);
     yearb:=startBPYearSim-BPyear+1;
     {if the first input year is not the starting year (1) of the simulation:
      fill the gap with constant values}
     if yearb>1 then
     begin
       for year:=1 to min(startBPYearSim,yearb-1) do
       system.writeln(fclim2,year,' ',Pb:5:2,' ',PEb:5:2,' ',JanTb:5:1,' ',JulTb:5:1,' ',
                             PlantINb:4:1,' ',ManINb:4:1,' ',txtb);
     end;
     if not (eof(fclim)) then
     repeat
       {set *a variables equal to already read *b variables}
       yeara:=yearb; Pa:=Pb; PEa:=PEb; JanTa:=JanTb; JulTa:=JulTb; txta:=txtb;
       PlantINa:=PlantINb; ManINa:=ManINb;
       {write the *a variables while converting percentages to fractions}
       if (yeara<=startBPYearSim)and(yeara>0)
       then system.writeln(fclim2,yeara,' ',Pa:5:2,' ',PEa:5:2,' ',JanTa:5:1,' ',
                                  JulTa:5:1,' ',PlantINa:4:1,' ',ManINa:4:1,' ',txta);
       {read new set *.b variables}
       system.Readln(fclim,BPyear,Pb,PEb,JanTb,JulTb,PlantInb,ManINb,txtb);
       yearb:=startBPYearSim-BPyear+1;
       {if more than 1 year between *a and *b : interpolate linearly by year}
       if yearb-yeara>1 then
       begin
         dy:=yearb-yeara;
         for year:=min(startBPYearSim,yeara)+1
             to    min(startBPYearSim,yearb-1)
             do
         begin
           P:={round}(Pa+(year-yeara)*(Pb-Pa)/dy);
           PE:={round}(PEa+(year-yeara)*(PEb-PEa)/dy);
           JanT:={round}(JanTa+(year-yeara)*(JanTb-JanTa)/dy);
           JulT:={round}(JulTa+(year-yeara)*(JulTb-JulTa)/dy);
           PlantIN:=PlantINa+(year-yeara)*(PlantINb-PlantINa)/dy;
           ManIN:=ManINa+(year-yeara)*(ManINb-ManINa)/dy;
           if (year>0)
           then system.writeln(fclim2,year,' ',P:5:2,' ',PE:5:2,' ',JanT:5:1,' ',JulT:5:1,
                                      ' ',PlantIN:4:1,' ',ManIn:4:1,' ',txta);
         end;
       end;
     until (eof(fclim));
     {write last year}
     if (yearb<=startBPYearSim)and(yearb>0)
     then system.writeln(fclim2,yearb,' ',Pb:5:2,' ',PEb:5:2,' ',JanTb:5:1,' ',JulTb:5:1,' ',
                             PlantINb:4:1,' ',ManINb:4:1,' ',txtb);
     {if last input year is not final simulation year:
      fill the gap with constant values}
     if yearb<startBPYearSim then
     begin
       for year:=yearb+1 to startBPYearSim do
       if (year>0)
       then system.writeln(fclim2,year,' ',Pb:5:2,' ',PEb:5:2,' ',JanTb:5:1,' ',JulTb:5:1,' ',
                             PlantINb:4:1,' ',ManINb:4:1,' ',txtb);
     end;
     closefile(fclim);
     closefile(fclim2);
   end;
   {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving ReadCheckClimate');
end;

procedure TfrmMAIN.WriteFileHeader(var ff:textfile; txt:string);
{write headers of (max 6 selected) daily time-depth diagrams with info on time-depth gridcell sizes}
begin
    system.Writeln(ff,MaxPlusCompartments:6,' (ncols, no. compartments) UNITS='+txt);
    system.Writeln(ff,spndtMultipleYears.Value:6,' (nrows, no. years)');
    system.Writeln(ff,dataInit.DLZ:6:0,' (colsize [mm per compartments])');
end;

procedure TfrmMAIN.btbttnOpenDefaultCModelClick(Sender: TObject);
{opens the file with C-turnover data }
begin
  {de-activate menu item to display dump.txt. This is because it will fill
   the data fields, which may override the read input}
   showmemorydump1.Visible:=false;
   {If GUItrace=1 then wwwriteln} singletracewriteln('Entering btbttnOpenDefaultCModelClick');
   pndlgOPEN.Filter:= 'C-input files (*.txt)|*.txt';
   if pndlgOPEN.Execute then
   begin
      mmDefaultCModel.Lines.LoadFromFile(pndlgOPEN.FileName);
      mmDefaultCModel.Enabled:=true;
      ReadDataC(pndlgOPEN.FileName);
      FirstRothCFileName:=pndlgOPEN.FileName;
      {DataYearIn:=dataInit; }CopyRecToFrom(DataYearIn,dataInit);
      if (not(bttnErrorInputs.Visible))and(fileexists('Messages.txt')and(warningsissued))
      then bttnErrorInputs.Visible:=true;
      if ReadDataCOK then
      begin
        btbttnOpenWeathering.Enabled:=true;
        btbttnEditCTurnover.Enabled:=true;
      end;
   end; 
{If GUItrace=1 then wwwriteln} singletracewriteln('Leaving btbttnOpenDefaultCModelClick');
end;

procedure TfrmMAIN.ReadDataC(FileName:string);
{Initialises all data for RothC and reads the general inputs like
 * initial distribution of OC over various pools by depth (ectorganic layer set to 0)
 * distribution of inputs of plantresidues and manure over the months for
   all defined vegetation types grassscrub, deciduouswood, coniferouswood
   and agriculture}
var fRothC,finitecto:textfile; i,maxq,m,chem,q:integer; txt,Msg:string;
    poolsum:real;
begin
{If GUItrace=1 then wwwriteln} singletracewriteln('Entering ReadDataC');
  {initialisation}
  maxq:=mmDefaultCModel.Lines.Count;
  {fill DataInit.RothC}
  begin
    DataInit.RothC.DPM2RPM:=0; DataInit.RothC.ThicknessLayer:=DataInit.dlz;      {layer thickness obtained from SoilGen}
    for m:=1 to 12 do
    begin
      DataInit.RothC.totalaccTSMD[m]:=0; {accumulated moisture deficit}
      DataInit.RothC.totalPlantResiduesIN[m]:=0;
      DataInit.RothC.totalManureIN[m]:=0;
    end;
    for i:=1 to DataInit.NoLayers+1 do
    begin
       DataInit.RothC.DPMpool[i]:=0; DataInit.RothC.RPMpool[i]:=0; DataInit.RothC.BIOpool[i]:=0;
       DataInit.RothC.HUMpool[i]:=0; DataInit.RothC.IOMpool[i]:=0; DataInit.RothC.CO2pool[i]:=0;
       DataInit.RothC.ClayPerc[i]:=0;
       DataInit.RothC.totalaccTSMD[1]:=0; DataInit.RothC.accTSMD[i]:=0;
       for chem:=1 to untProcesses.nchem do
       begin
         DataInit.CHEMDPMpool[chem,i-1]:=0; DataInit.CHEMRPMpool[chem,i-1]:=0;
         DataInit.CHEMBIOpool[chem,i-1]:=0; DataInit.CHEMHUMpool[chem,i-1]:=0;
         DataInit.CHEMIOMpool[chem,i-1]:=0; DataInit.CHEMCO2pool[chem,i-1]:=0;
       end;
       for m:=1 to 12 do
       begin
          DataInit.RothC.SoilCover[m]:=0;          DataInit.RothC.RDepth[m]:=0;
          DataInit.RothC.Rainfall[m]:=0;           DataInit.RothC.EvapoPan[m]:=0;
          DataInit.RothC.PlantresiduesIN[m][i]:=0; DataInit.RothC.ManureIN[m][i]:=0;
          DataInit.RothC.Temperatures[m][i]:=0;
       end;
    end;
  end;
  q:=0;
  try
  {read the datafile with General data for RothC}
  {fill DataInit.RothCGen}
  begin
    system.Assign(fRothC,FileName); system.reset(fRothC);
    repeat q:=q+1; system.readln(fRothC,txt)
    until (pos('[initial profile data',txt)>0)or(q=maxq);
    if q=maxq then raise EInOutError.Create('File incomplete');
    q:=q+1; system.readln(fRothC,txt);
    poolsum:=0;
    for i:=1 to 5 do
    begin
      q:=q+1; system.Read(fRothC,DataInit.RothCGen.GENPoolPercs[i]);
      poolsum:=poolsum+DataInit.RothCGen.GENPoolPercs[i];
    end;
    if abs(poolsum)<0.1 then {all organic matter in Inert Organic Matter pool}
    begin DataInit.RothCGen.GENPoolPercs[5]:=100; poolsum:=100; end;
    if abs(poolsum-100)>0.001 then {summing error}
    begin for i:=1 to 5 do DataInit.RothCGen.GENPoolPercs[i]:=100*DataInit.RothCGen.GENPoolPercs[i]/poolsum; end;
    system.reset(fRothC); q:=0;
    repeat q:=q+1; system.readln(fRothC,txt)
    until (pos('grassscrub',txt)>0)or(q=maxq);
    if q=maxq then raise EInOutError.Create('File incomplete');;
    for m:=1 to 12 do
    begin
      q:=q+1; system.Readln(fRothC,i,DataInit.RothCGen.GENgrasPlant[m],DataInit.RothCGen.GENgrasMan[m]);
    end;
    q:=q+1; system.readln(fRothC,DataInit.RothCGen.GENgrasRDEPTH);
    if DataInit.RothCGen.GENgrasRDEPTH>(DataInit.dlz*DataInit.NoLayers)
    then
    begin
       DataInit.RothCGen.GENgrasRDEPTH:=(DataInit.dlz*DataInit.NoLayers);
       debuglog(currentyear+': Non-fatal input error after reading C-turnoverfile: ');
       debuglog('Max rooting depth <grassscrub> cannot exceed profile depth.');
       debuglog('Change Max rooting depth (C-turnover file) or change profile depth (soil data file)!');
       debuglog('If you do nothing then SoilGen sets Max rooting depth to profile depth.');
       debuglog('***');
    end;
    system.reset(fRothC); q:=0;
    repeat q:=q+1; system.readln(fRothC,txt) until pos('deciduouswood',txt)>0;
    for m:=1 to 12 do
    begin
      q:=q+1; system.Readln(fRothC,i,DataInit.RothCGen.GENdeciPlant[m],DataInit.RothCGen.GENdeciMan[m]);
    end;
    q:=q+1; system.readln(fRothC,DataInit.RothCGen.GENdeciRDEPTH);
    if DataInit.RothCGen.GENdeciRDEPTH>(DataInit.dlz*DataInit.NoLayers)
    then
    begin
       DataInit.RothCGen.GENdeciRDEPTH:=(DataInit.dlz*DataInit.NoLayers);
       debuglog(currentyear+': Non-fatal input error after reading C-turnoverfile: ');
       debuglog('Max rooting depth <deciduouswood> cannot exceed profile depth.');
       debuglog('Change Max rooting depth (C-turnover file) or change profile depth (soil data file)!');
       debuglog('If you do nothing then SoilGen sets Max rooting depth to profile depth.'); 
       debuglog('***');
    end;
    system.reset(fRothC); q:=0;
    repeat q:=q+1; system.readln(fRothC,txt)
    until (pos('coniferouswood',txt)>0)or(q=maxq);
    if q=maxq then raise EInOutError.Create('File incomplete');;
    for m:=1 to 12 do
    begin
      q:=q+1; system.Readln(fRothC,i,DataInit.RothCGen.GENconiPlant[m],DataInit.RothCGen.GENconiMan[m]);
    end;
    q:=q+1; system.readln(fRothC,DataInit.RothCGen.GENconiRDEPTH);
    if DataInit.RothCGen.GENconiRDEPTH>(DataInit.dlz*DataInit.NoLayers)
    then
    begin
       DataInit.RothCGen.GENconiRDEPTH:=(DataInit.dlz*DataInit.NoLayers);
       debuglog(currentyear+': Non-fatal input error after reading C-turnoverfile: ');
       debuglog('Max rooting depth <coniferouswood> cannot exceed profile depth.');
       debuglog('Change Max rooting depth (C-turnover file) or change profile depth (soil data file)!');
       debuglog('If you do nothing then SoilGen sets Max rooting depth to profile depth.'); 
       debuglog('***'); 
    end;
    system.reset(fRothC); q:=0;
    repeat q:=q+1; system.readln(fRothC,txt)
    until (pos('agriculture',txt)>0)or(q=maxq);
    if q=maxq then raise EInOutError.Create('File incomplete');;
    for m:=1 to 12 do
    begin
      q:=q+1; system.Readln(fRothC,i,DataInit.RothCGen.GENagriPlant[m],DataInit.RothCGen.GENagriMan[m]);
    end;
    q:=q+1; system.readln(fRothC,DataInit.RothCGen.GENagriRDEPTH);
    if DataInit.RothCGen.GENagriRDEPTH>(DataInit.dlz*DataInit.NoLayers)
    then
    begin
       DataInit.RothCGen.GENagriRDEPTH:=(DataInit.dlz*DataInit.NoLayers);
       debuglog(currentyear+': Non-fatal input error after reading C-turnoverfile: ');
       debuglog('Max rooting depth <agriculture> cannot exceed profile depth.');
       debuglog('Change Max rooting depth (C-turnover file) or change profile depth (soil data file)!');
       debuglog('If you do nothing then SoilGen sets Max rooting depth to profile depth.'); 
       debuglog('***');
    end;
    closefile(fRothC);
  end;
  {determining the initial C-pools by converting the OC%, using the bulkdensity
   (both are input for SoilGen) to the pools DPM, RPM, BIOm HUM and IOM
   (units: ton C/ha for dlz thickness).
   Hereto, the distribution coefficients are applied that were just (this routine) read
   into GENPoolPercs[i].}
   {fill DataInit}
   begin
    DataInit.RothC.ectoDPMpool:=0; DataInit.RothC.ectoRPMpool:=0; DataInit.RothC.ectoBIOpool:=0;
    DataInit.RothC.ectoHUMpool:=0; DataInit.RothC.ectoIOMpool:=0; DataInit.RothC.ectoCO2pool:=0;
    if fileexists('InitEcto.txt') then
    begin
     system.assign(finitecto,'InitEcto.txt'); system.reset(finitecto);
     system.readln(finitecto,DataInit.RothC.ectoDPMpool);
     system.readln(finitecto,DataInit.RothC.ectoRPMpool);
     system.readln(finitecto,DataInit.RothC.ectoBIOpool);
     system.readln(finitecto,DataInit.RothC.ectoHUMpool);
     system.readln(finitecto,DataInit.RothC.ectoIOMpool);
     closefile(finitecto);
    end;
    for i:=1 to DataInit.NoLayers do
    begin
      {frac.in pool * frac.of mass * (C-mass/m2) * m2/ha     * tonnes/kg
                                     for dlz mm
       poolperc/100 * massperc/100 * kg/dm3 * mm *10000m2/ha * 1/1000tonnes/kg}
      DataInit.RothC.DPMpool[i]:=DataInit.RothCGEN.GENPoolPercs[1]*DataInit.OC[i]*DataInit.RHO[i]*DataInit.dlz/1000;
      DataInit.RothC.RPMpool[i]:=DataInit.RothCGEN.GENPoolPercs[2]*DataInit.OC[i]*DataInit.RHO[i]*DataInit.dlz/1000;
      DataInit.RothC.BIOpool[i]:=DataInit.RothCGEN.GENPoolPercs[3]*DataInit.OC[i]*DataInit.RHO[i]*DataInit.dlz/1000;
      DataInit.RothC.HUMpool[i]:=DataInit.RothCGEN.GENPoolPercs[4]*DataInit.OC[i]*DataInit.RHO[i]*DataInit.dlz/1000;
      DataInit.RothC.IOMpool[i]:=DataInit.RothCGEN.GENPoolPercs[5]*DataInit.OC[i]*DataInit.RHO[i]*DataInit.dlz/1000;
    end;
   end;
   ReadDataCOK:=true;
   {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving ReadDataC');
   except
    ReadDataCOK:=false;
    if q=maxq then dshowmessage('Year '+currentyear+'; Fatal error: This is not a (complete) RothC-file!')
    else
    begin
      str(q,Msg); Msg:=currentyear+'Fatal error: Error in inputfile '+FileName+' near line '+Msg;
      dshowmessage(Msg);
    end;
    dshowmessage('You will not be able to run SoilGen unless you correct this error and reload.');
    {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving ReadDataC with errors');
   end;
end;

procedure TfrmMAIN.ReadDataWE(FileName:string);
{reads the data file with mineral mass fractions, stoichiometry of Amorphite and Otherite,
 and calculates mass (per compartment)}
var maxq,i,j,q,idum:integer; fWE,fspecialminerals:textfile; txt,Msg:string; sum1,sum,totalmass:real;
    tmpMinPool:array[1..nrminerals]of profile;
begin
  {If GUItrace=1 then wwwriteln} singletracewriteln('Entering ReadDataWE');
  maxq:=mmWeatheringData.Lines.count;
  {initialisation}
  for i:=1 to DataInit.NoLayers do
  begin
    DataInit.WE.WE_Capool[i]:=0;
    DataInit.WE.WE_Mgpool[i]:=0;
    DataInit.WE.WE_Napool[i]:=0;
    DataInit.WE.WE_Kpool[i] :=0;
    DataInit.WE.WE_Sipool[i]:=0;
    DataInit.WE.WE_Fepool[i]:=0;
    DataInit.WE.WE_Alpool[i]:=0;
    DataInit.WE.Gibbsitepool[i]:=0;
    DataInit.WE.WE_SiPedogenic[i]:=0;
    DataInit.WE.WE_FePedogenic[i]:=0;
  end;
  {fill DataInit.WEGen with weathering parameters and mineral names already read from SoftConstants}
  for j:=1 to nrminerals do
  begin
      DataInit.WEGen.MinNames[j]:=untProcesses.MinNames[j];
      for i:=1 to DataInit.NoLayers do
      begin
        DataInit.WE.MinPools[j,i]:=0;
      end;
      for i:=1 to nrparams do
      DataInit.WEGen.MinDisPar[i,j]:=untProcesses.MinDisPar[i,j];
  end;
  {start reading}
  q:=0;
  try
  {First, read 2 files with the elemental composition of Amorphite and Otherite:
  Amorphite.txt and Otherite.txt}
  if not(fileexists('Amorphite.txt')) then raise EInOutError.Create('Amorphite.txt not present!');
  system.assign(fspecialminerals,'Amorphite.txt'); system.reset(fspecialminerals);
  system.readln(fspecialminerals);  {skip header line}
  system.readln(fspecialminerals,amorp_O);
  system.readln(fspecialminerals,amorp_Al);
  system.readln(fspecialminerals,amorp_Ca);
  system.readln(fspecialminerals,amorp_Mg);
  system.readln(fspecialminerals,amorp_K);
  system.readln(fspecialminerals,amorp_Na);
  system.readln(fspecialminerals,amorp_Fe);
  system.readln(fspecialminerals,amorp_Si);
  system.readln(fspecialminerals,amorp_H);
  system.readln(fspecialminerals,amorp_Cl);
  system.readln(fspecialminerals,amorp_C);
  system.readln(fspecialminerals,amorp_S);
  closefile(fspecialminerals);
  if not(fileexists('Otherite.txt')) then raise EInOutError.Create('Otherite.txt not present!');
  system.assign(fspecialminerals,'Otherite.txt'); system.reset(fspecialminerals);
  system.readln(fspecialminerals);  {skip header line}
  system.readln(fspecialminerals,other_O);
  system.readln(fspecialminerals,other_Al);
  system.readln(fspecialminerals,other_Ca);
  system.readln(fspecialminerals,other_Mg);
  system.readln(fspecialminerals,other_K);
  system.readln(fspecialminerals,other_Na);
  system.readln(fspecialminerals,other_Fe);
  system.readln(fspecialminerals,other_Si);
  system.readln(fspecialminerals,other_H);
  system.readln(fspecialminerals,other_Cl);
  system.readln(fspecialminerals,other_C);
  system.readln(fspecialminerals,other_S);
  closefile(fspecialminerals);
  {Then, calculate RFMamorphite and RFMotherite using the relative elemental contributions}
  {prevent division by zero if all 0 values were entered}
  sum1:=abs(amorp_O)+abs(amorp_Al)+abs(amorp_Ca)+abs(amorp_Mg)+abs(amorp_K)+
        abs(amorp_Na)+abs(amorp_Fe)+abs(amorp_Si)+
        abs(amorp_H)+abs(amorp_Cl)+abs(amorp_C)+abs(amorp_S);
  if sum1=0
  then RFMamorphite:=-999
  else RFMamorphite:=amorp_O*fw_O+amorp_Al*fw_Al+amorp_Ca*fw_Ca+amorp_Mg*fw_Mg+
                amorp_K*fw_K+amorp_Na*fw_Na+amorp_Fe*fw_Fe+amorp_Si*fw_Si+
                amorp_H*fw_H+amorp_Cl*fw_Cl+amorp_C*fw_C+amorp_S*fw_S;
  sum1:=abs(other_O)+abs(other_Al)+abs(other_Ca)+abs(other_Mg)+abs(other_K)+
        abs(other_Na)+abs(other_Fe)+abs(other_Si)+
        abs(other_H)+abs(other_Cl)+abs(other_C)+abs(other_S);
  if sum1=0
  then RFMotherite:=-999
  else RFMotherite:= other_O*fw_O+other_Al*fw_Al+other_Ca*fw_Ca+other_Mg*fw_Mg+
                 other_K*fw_K+other_Na*fw_Na+other_Fe*fw_Fe+other_Si*fw_Si+
                 other_H*fw_H+other_Cl*fw_Cl+other_C*fw_C+other_S*fw_S;
  {now read the weathering data file}
  system.assign(fWE,FileName);
  {initial mineral pools per layer}
  system.reset(fWE);
  repeat q:=q+1; system.readln(fWE,txt)
  until (pos('[Layer Albite K-Feldspar Muscovite Biotite Quartz Chlorite Anorthite',txt)>0)or(q=maxq);
  if q=maxq then raise EInOutError.Create('File incomplete');
  for i:=1 to DataInit.NoLayers do
  begin
   system.read(fWE,idum); {layer number}
   sum:=0;
   for j:=1 to 14 do
   begin
     system.read(fWE,DataInit.WE.MinPools[j][i]);
     {potentially newforming minerals should not start with value=0}
     if (j in [5,6,10,11,12]) and (DataInit.WE.MinPools[j][i]=0)
     then DataInit.WE.MinPools[j][i]:=seedmineral;
     sum:=sum+DataInit.WE.MinPools[j][i];
   end;
   system.read(fWE,DataInit.WE.Gibbsitepool[i]); sum:=sum+DataInit.WE.Gibbsitepool[i];
   {DataInit.WE.MinPools[17,i]:=DataInit.WE.Gibbsitepool[i];}
   for j:=15 to 16 do  {amorphite and otherite)}
   begin
     system.read(fWE,DataInit.WE.MinPools[j][i]); sum:=sum+DataInit.WE.MinPools[j][i];
   end;
   system.readln(fWE);
   if sum<>1 then
   for j:=1 to nrminerals do {rescale to have fractions sum to 1}
   begin
     DataInit.WE.MinPools[j][i]:=DataInit.WE.MinPools[j][i]/sum;
   end;
   DataInit.WE.Gibbsitepool[i]:=DataInit.WE.Gibbsitepool[i]/sum;
   q:=q+1;
   if q>maxq then raise EInOutError.Create('File incomplete');
  end;
  closefile(fWE);
  {now convert from mass_fraction_in_minerals to kg (1 sq.m per compartment of dlz mm thickness).
   to be used in calculations in untProcesses.DataInit}
  for i:=1 to DataInit.NoLayers do
  begin
    totalmass:=DataInit.RHO[i]*DataInit.DLZ*(100-DataInit.STONES[i]-DataInit.GRAVEL[i])/100; {kg dry matter over DLZ mm compartment thickness and 1 m2 area, excluding stones+gravel}
    totalmass:=totalmass-DataInit.calciteA[i]-DataInit.gypA[i]-1.7*DataInit.OCm[i];          {but we subtract mass of gypsum, calcite and OC, all assumed part of fine earth}
    totalmass:=totalmass+((DataInit.STONES[i]+DataInit.GRAVEL[i])/100)*DataInit.DLZ *2.65; {..raised by the mass in stones and gravel, with specific weight assumed to be 2.65 kg dm-3}
    {now totalmass is mass associated with the 14 minerals + gibbsite +otherite + amorphite}
    DataInit.WE.Gibbsitepool[i]:=DataInit.WE.Gibbsitepool[i]*totalmass;
    for j:=1 to nrminerals do
      DataInit.WE.MinPools[j][i]:=DataInit.WE.MinPools[j][i]*totalmass;
  end;
  {convert the mineral pools (kg) to elemental pools in minerals (mol)}
  {first convert kg mineral to mol mineral by using the
   Relative Formula Mass, RFM, per mineral from elemental weights and stoichiometric coeficients.
   These are calculated constants in untProcesses
   "1000" is used to convert kmol>mol}
  for i:=1 to DataInit.NoLayers do
  begin
    tmpMinPool[1][i]:=1000*DataInit.WE.MinPools[1][i]/untProcesses.RFMalbite;
    tmpMinPool[2][i]:=1000*DataInit.WE.MinPools[2][i]/untProcesses.RFMkfeldspar;
    tmpMinPool[3][i]:=1000*DataInit.WE.MinPools[3][i]/untProcesses.RFMmuscovite;
    tmpMinPool[4][i]:=1000*DataInit.WE.MinPools[4][i]/untProcesses.RFMbiotite;
    tmpMinPool[5][i]:=1000*DataInit.WE.MinPools[5][i]/untProcesses.RFMquartz;
    tmpMinPool[6][i]:=1000*DataInit.WE.MinPools[6][i]/untProcesses.RFMchlorite;
    tmpMinPool[7][i]:=1000*DataInit.WE.MinPools[7][i]/untProcesses.RFManorthite;
    tmpMinPool[8][i]:=1000*DataInit.WE.MinPools[8][i]/untProcesses.RFMForsterite;
    tmpMinPool[9][i]:=1000*DataInit.WE.MinPools[9][i]/untProcesses.RFMaugite;
    tmpMinPool[10][i]:=1000*DataInit.WE.MinPools[10][i]/untProcesses.RFMkaolinite;
    tmpMinPool[11][i]:=1000*DataInit.WE.MinPools[11][i]/untProcesses.RFMillite;
    tmpMinPool[12][i]:=1000*DataInit.WE.MinPools[12][i]/untProcesses.RFMmontmorillonite;
    tmpMinPool[13][i]:=1000*DataInit.WE.MinPools[13][i]/untProcesses.RFMhornblende;
    tmpMinPool[14][i]:=1000*DataInit.WE.MinPools[14][i]/untProcesses.RFMfayalite;
    {Line below this line converts initial Gibbsite Al(OH)3 (kmol) to mol Al in Gibbsite}
    DataInit.WE.Gibbsitepool[i]:=1000*DataInit.WE.Gibbsitepool[i]/untProcesses.RFMgibbsite;
    tmpMinPool[17][i]:=1000*DataInit.WE.MinPools[17][i]/untProcesses.RFMgibbsite;
    {special minerals: only if elemental composition is non-zero}
    if RFMamorphite>0
       then tmpMinPool[15][i]:=1000*DataInit.WE.MinPools[15][i]/RFMamorphite
       else tmpMinPool[15][i]:=0;
    if RFMotherite>0
       then tmpMinPool[16][i]:=1000*DataInit.WE.MinPools[16][i]/RFMotherite
       else tmpMinPool[16][i]:=0;
  end;
  {then use the stoichiometric composition of each mineral to obtain mmoles of element:
  {1 albite Na Al Si3 O8}
  {2 K-feldspar K Al Si3 O8}
  {3 muscovite K Al3 Si3 O10 (OH)2}
  {4 biotite K Mg1.5 Fe1.5 Al Si3 O10 (OH)2}
  {5 Quartz Si O2}
  {6 Clorite Clinochlore Mg5 Al2 Si3 O10 (OH)8 }
  {7 anorthite Ca Al2 Si2 O8 }
  {8 Forsterite Mg2 Si O4}
  {9 augite orthoferrosilite Fe Si O3 }
  {10 kaolinite Al2 Si2 O5 (OH)4}
  {11 Illite Si3.4 O10 (OH)2 Mg0.02 Al2.38 Ca0.01 K0.44, Godderis et al 2006}
  {12 Montmorillonite Si4 O10 (OH)2 Mg0.33 Al1.67 Ca0.04125 Mg0.04125 K0.0825 Na0.0825 }
  {13 hornblende Ca2 Mg4 Al Si6.7 Al O22.4 from Profile model }
  {14 fayalite Fe2 Si O4}
  {15 amorphite, input composition}
  {16 otherite, input composition}
  for i:=1 to DataInit.NoLayers do
  begin
   DataInit.WE.WE_Capool[i]:=0*tmpMinPool[1][i]+0*tmpMinPool[2][i]+0*tmpMinPool[3][i]+
                             0*tmpMinPool[4][i]+0*tmpMinPool[5][i]+0*tmpMinPool[6][i]+
                             1*tmpMinPool[7][i]+0*tmpMinPool[8][i]+0*tmpMinPool[9][i]+
                             0*tmpMinPool[10][i]+0.01*tmpMinPool[11][i]+0.04125*tmpMinPool[12][i]+
                             2*tmpMinPool[13][i]+0*tmpMinPool[14][i]+amorp_Ca*tmpMinPool[15][i]+
                             other_Ca*tmpMinPool[16][i];
   DataInit.WE.WE_Mgpool[i]:=0*tmpMinPool[1][i]+0*tmpMinPool[2][i]+0*tmpMinPool[3][i]+
                             1.5*tmpMinPool[4][i]+0*tmpMinPool[5][i]+5*tmpMinPool[6][i]+
                             0*tmpMinPool[7][i]+2*tmpMinPool[8][i]+0*tmpMinPool[9][i]+
                             0*tmpMinPool[10][i]+0.02*tmpMinPool[11][i]+(0.33+0.04125)*tmpMinPool[12][i]+
                             4*tmpMinPool[13][i]+0*tmpMinPool[14][i]+amorp_Mg*tmpMinPool[15][i]+
                             other_Mg*tmpMinPool[16][i];
   DataInit.WE.WE_Napool[i]:=1*tmpMinPool[1][i]+0*tmpMinPool[2][i]+0*tmpMinPool[3][i]+
                             0*tmpMinPool[4][i]+0*tmpMinPool[5][i]+0*tmpMinPool[6][i]+
                             0*tmpMinPool[7][i]+0*tmpMinPool[8][i]+0*tmpMinPool[9][i]+
                             0*tmpMinPool[10][i]+0*tmpMinPool[11][i]+0.0825*tmpMinPool[12][i]+
                             0*tmpMinPool[13][i]+0*tmpMinPool[14][i]+amorp_Na*tmpMinPool[15][i]+
                             other_Na*tmpMinPool[16][i];
   DataInit.WE.WE_Kpool[i] :=0*tmpMinPool[1][i]+1*tmpMinPool[2][i]+1*tmpMinPool[3][i]+
                             1*tmpMinPool[4][i]+0*tmpMinPool[5][i]+0*tmpMinPool[6][i]+
                             0*tmpMinPool[7][i]+0*tmpMinPool[8][i]+0*tmpMinPool[9][i]+
                             0*tmpMinPool[10][i]+0.44*tmpMinPool[11][i]+0.0825*tmpMinPool[12][i]+
                             0*tmpMinPool[13][i]+0*tmpMinPool[14][i]+amorp_K*tmpMinPool[15][i]+
                             other_K*tmpMinPool[16][i];
   DataInit.WE.WE_Sipool[i]:=3*tmpMinPool[1][i]+3*tmpMinPool[2][i]+3*tmpMinPool[3][i]+
                             3*tmpMinPool[4][i]+1*tmpMinPool[5][i]+3*tmpMinPool[6][i]+
                             2*tmpMinPool[7][i]+1*tmpMinPool[8][i]+1*tmpMinPool[9][i]+
                             2*tmpMinPool[10][i]+3.4*tmpMinPool[11][i]+4*tmpMinPool[12][i]+
                             6.7*tmpMinPool[13][i]+1*tmpMinPool[14][i]+amorp_Si*tmpMinPool[15][i]+
                             other_Si*tmpMinPool[16][i];
   DataInit.WE.WE_Fepool[i]:=0*tmpMinPool[1][i]+0*tmpMinPool[2][i]+0*tmpMinPool[3][i]+
                             1.5*tmpMinPool[4][i]+0*tmpMinPool[5][i]+0*tmpMinPool[6][i]+
                             0*tmpMinPool[7][i]+0*tmpMinPool[8][i]+1*tmpMinPool[9][i]+
                             0*tmpMinPool[10][i]+0*tmpMinPool[11][i]+0*tmpMinPool[12][i]+
                             0*tmpMinPool[13][i]+2*tmpMinPool[14][i]+amorp_Fe*tmpMinPool[15][i]+
                             other_Fe*tmpMinPool[16][i];
   DataInit.WE.WE_Alpool[i]:=1*tmpMinPool[1][i]+1*tmpMinPool[2][i]+3*tmpMinPool[3][i]+
                             1*tmpMinPool[4][i]+0*tmpMinPool[5][i]+2*tmpMinPool[6][i]+
                             2*tmpMinPool[7][i]+0*tmpMinPool[8][i]+0*tmpMinPool[9][i]+
                             2*tmpMinPool[10][i]+2.38*tmpMinPool[11][i]+1.67*tmpMinPool[12][i]+
                             1*tmpMinPool[13][i]+0*tmpMinPool[14][i]+amorp_Al*tmpMinPool[15][i]+
                             other_Al*tmpMinPool[16][i];
   DataInit.WE.WE_FePedogenic[i]:=0; {Fe released by weathering; not further processed besides addition from weathering}
   DataInit.WE.WE_SiPedogenic[i]:=0; {Si released by weathering; not further processed besides addition from weathering}
   {once we will add WE_Cpool, WE_Spool and WE_Clpool}
  end;
  readweatherOK:=true;
  {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving ReadDataWE');
  except
    readweatherOK:=false;
    if q=maxq then dshowmessage('Year '+currentyear+'; Fatal error: This is not a (complete) weathering data file!')
    else
    begin
      str(q,Msg); Msg:='Year '+currentyear+': Error in inputfile '+FileName+' near line '+Msg;
      dshowmessage(Msg);
    end;
    dshowmessage('You will not be able to run SoilGen unless you correct this error and reload.');
    {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving ReadDataWE with errors');
  end;
end;

procedure TfrmMAIN.WriteDataC(FileName:string);
{gets the annual input data for the weathering routines and writes the inputfile "WE*."
 to be called for the current year
 Uses:
 * general data read in ReadDataC:
    * distribution of OC over various pools by depth (including ectorganic layer)
    * distribution of inputs of plantresidues and manure over the months for
      all defined vegetation types
 * yearly data concerning vegetation type, plant residue input and manure input
  (read from the climate file)}
var fc:textfile; i,m,chem:integer;
    maxrootcomp:integer; cumrdf:real;
begin
  {If GUItrace=1 then wwwriteln} singletracewriteln('Entering WriteDataC');
  system.assign(fc,FileName); system.rewrite(fc);
  system.Writeln(fc,'[general data]  : vegetation, annual PlantResidue + Manure input (ton C/ha)');
  system.writeln(fc,DataYearIn.RothC.vegetatie);
  system.writeln(fc,DataYearIn.RothC.plantressum,' ',DataYearIn.RothC.manuresum,' (LitterInput cq manureInput this year)');
  system.writeln(fc,'[profile data]  : initial size of C-pools (ton C/ha) and accumulated moisture deficit (mm)]');
  system.writeln(fc,'[layer  DPMpool  RPMpool  BIOpool  HUMpool  IOMpool  CO2pool  accTSMD]');
  system.write(fc,'  0     ',DataYearIn.RothC.ectoDPMpool,' ',DataYearIn.RothC.ectoRPMpool,' ',
                  DataYearIn.RothC.ectoBIOpool,' ',DataYearIn.RothC.ectoHUMpool,' ',
                  DataYearIn.RothC.ectoIOMpool,' ',DataYearIn.RothC.ectoCO2pool,' ',' 0 ');
  for chem:=1 to untProcesses.nchem do  {nchem=value for nchem set in untProcesses.Readc routine}
    system.write(fc,DataYearIn.CHEMDPMpool[chem,0],' ',DataYearIn.CHEMRPMpool[chem,0],' ',
                    DataYearIn.CHEMBIOpool[chem,0],' ',DataYearIn.CHEMHUMpool[chem,0],' ',
                    DataYearIn.CHEMIOMpool[chem,0],' ',DataYearIn.CHEMCO2pool[chem,0],' ');
  system.writeln(fc);
  for i:=1 to DataYearIn.NoLayers do
  begin
    system.write(fc,i:3,'     ',DataYearIn.RothC.DPMpool[i],' ',DataYearIn.RothC.RPMpool[i],' ',
                    DataYearIn.RothC.BIOpool[i],' ',DataYearIn.RothC.HUMpool[i],' ',
                    DataYearIn.RothC.IOMpool[i],' ',DataYearIn.RothC.CO2pool[i],' ',
                    DataYearIn.RothC.accTSMD[i],' ');
    for chem:=1 to untProcesses.nchem do  {nchem=constant set in untProcesses.Readc routine}
        system.write(fc,DataYearIn.CHEMDPMpool[chem,i],' ',DataYearIn.CHEMRPMpool[chem,i],' ',
                        DataYearIn.CHEMBIOpool[chem,i],' ',DataYearIn.CHEMHUMpool[chem,i],' ',
                        DataYearIn.CHEMIOMpool[chem,i],' ',DataYearIn.CHEMCO2pool[chem,i],' ');
    system.writeln(fc);
  end;
  system.writeln(fc,'[monthly inputs]   : distribution of inputs (%) of PlantResidues and Manure for all vegetations per month');
  system.writeln(fc,'[month PlantResiduesIN ManureIN for '+DataYearIn.RothC.vegetatie+']');
  {select totalPlantResiduesIN and totalManureIN based on the value for vegetation}
  if pos('agriculture',DataYearIn.RothC.vegetatie)>0    then
  begin
    DataYearIn.RothC.totalPlantResiduesIn:=DataYearIn.RothCGen.GENagriPlant;
    DataYearIn.RothC.totalManureIn:=DataYearIn.RothCGen.GENagriMan; 
    for m:=1 to 12 do DataYearIn.RothC.RDepth[m]:=DataYearIn.RothCGen.GENagriRDEPTH;
  end;
  if pos('grassscrub',DataYearIn.RothC.vegetatie)>0     then   
  begin
    DataYearIn.RothC.totalPlantResiduesIn:=DataYearIn.RothCGen.GENgrasPlant;
    DataYearIn.RothC.totalManureIn:=DataYearIn.RothCGen.GENgrasMan;
    for m:=1 to 12 do DataYearIn.RothC.RDepth[m]:=DataYearIn.RothCGen.GENgrasRDEPTH;
  end;
  if pos('deciduouswood',DataYearIn.RothC.vegetatie)>0  then  
  begin
    DataYearIn.RothC.totalPlantResiduesIn:=DataYearIn.RothCGen.GENdeciPlant;
    DataYearIn.RothC.totalManureIn:=DataYearIn.RothCGen.GENdeciMan;  
    for m:=1 to 12 do DataYearIn.RothC.RDepth[m]:=DataYearIn.RothCGen.GENdeciRDEPTH;
  end;
  if pos('coniferouswood',DataYearIn.RothC.vegetatie)>0 then 
  begin
    DataYearIn.RothC.totalPlantResiduesIn:=DataYearIn.RothCGen.GENconiPlant;
    DataYearIn.RothC.totalManureIn:=DataYearIn.RothCGen.GENconiMan;
    for m:=1 to 12 do DataYearIn.RothC.RDepth[m]:=DataYearIn.RothCGen.GENconiRDEPTH;
  end;
  {and write it}
  for m:=1 to 12 do
  writeln(fc,m:3,'    ',DataYearIn.RothC.totalPlantResiduesIn[m],'        ',
                     DataYearIn.RothC.totalManureIn[m]);
  closefile(fc);
 {determine root distribution based on DataYearIn.RothC.RDepth[1],
  the rooting depth belonging to the vegetation type (month 1, but assumed constant
  over the year };
  maxrootcomp:=trunc(min(DataYearIn.RothC.RDepth[1],DataYearIn.unfrozen)/DataYearIn.dlz)+1;
  cumrdf:=0;
  for i:=1 to DataYearIn.NoLayers do
  begin
    if i<=maxrootcomp then
    begin
      {option 1: root density distribution triangular}
      {tmprdf:=min(DataYearIn.RothC.RDepth[1],DataYearIn.unfrozen);
      DataYearIn.RDF[i]:=(2/tmprdf)*(1-(i-1)/maxrootcomp);}
      {OPTION 2: root density distribution according to exponential distribution}
      DataYearIn.RDF[i]:=exp(-0.004*(i-0.5)*DataYearIn.DLZ);
      cumrdf:=cumrdf+DataYearIn.RDF[i];
    end
    else DataYearIn.RDF[i]:=0;
  end;
  for i:=1 to DataYearIn.NoLayers do DataYearIn.RDF[i]:=DataYearIn.RDF[i]/cumrdf;
  {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving WriteDataC');
end;

procedure TfrmMAIN.WriteDataWE(FileName:string);
{gets the annual input data for the weathering routines
 and writes the inputfile to be called from within  bttnRunClick.
 Uses:
 * general data read in ReadDataWE:
    * distribution of Ca, Mg, K, Na weatherable pools and and Al (Gibbsite) by depth.}
var fwe:textfile; i,j:integer;
begin
  {If GUItrace=1 then wwwriteln} singletracewriteln('Entering WriteDataWE');
  system.assign(fwe,FileName); system.rewrite(fwe);
  system.writeln(fwe,'[Mass of minerals (kg)]');
  system.writeln(fwe,
                 '[Layer Albite K-Feldspar Muscovite Biotite Quartz Chlorite Anorthite'+
                 'Forsterite Augite Kaolinite Illite Montmorillonite Hornblende Fayalite Gibbsite]');
  for i:=1 to DataYearIn.NoLayers do
  begin
    system.write(fwe,' ',i:2,' ');
    for j:=1 to nrminerals do system.write(fwe,DataYearIn.WE.MinPools[j][i],' ');
    system.writeln(fwe);
  end;
  system.writeln(fwe,'[Cations still locked in minerals] [or pedogenic accumulation of Fe + Si]');
  system.writeln(fwe,'[layer ',
                     'Gibbsite Al (mol)          ',
                     'Ca (mol)          ',
                     'Mg (mol)          ',
                     'K  (mol)          ',
                     'Na (mol)          ',
                     'Si (mol)          ',
                     'Fe (mol)          ',
                     'Al (mol)          ',
                     'Fe-pedogenic(mol) ',
                     'Si-pedogenic(mol)]');
  for i:=1 to DataYearIn.NoLayers do
  begin
    system.writeln(fwe,' ',i:2,' ',DataYearIn.WE.Gibbsitepool[i],' ',
      DataYearIn.WE.WE_Capool[i],' ',DataYearIn.WE.WE_Mgpool[i],' ',
      DataYearIn.WE.WE_Kpool[i],' ', DataYearIn.WE.WE_Napool[i],' ',
      DataYearIn.WE.WE_Sipool[i],' ', DataYearIn.WE.WE_Fepool[i],' ',
      DataYearIn.WE.WE_Alpool[i],' ', DataYearIn.WE.WE_FePedogenic[i],' ',
      DataYearIn.WE.WE_SiPedogenic[i]);
  end;
  closefile(fwe);
  {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving WriteDataWE');
end;

procedure TfrmMAIN.ReadOutC(FileNameOut:string);
{reads the annual outputfile from RothC and stores the pool-data (etc)
 in the record RothCRec within the dataYearOut record. }
 var i,dum,chem:integer; fcout:textfile;
begin
  {If GUItrace=1 then wwwriteln} singletracewriteln('Entering ReadOutC');
  system.assign(fcout,FileNameOut); system.reset(fcout);
  system.readln(fcout); system.readln(fcout);
  system.read(fcout,dum,dataYearOut.RothC.ectoDPMpool,dataYearOut.RothC.ectoRPMpool,
                        dataYearOut.RothC.ectoBIOpool,dataYearOut.RothC.ectoHUMpool,
                        dataYearOut.RothC.ectoIOMpool,dataYearOut.RothC.ectoCO2pool);
  for chem:=1 to untProcesses.nchem do  {nchem=value for nchem set in untProcesses.Readc routine}
    system.read(fcout,dataYearOut.CHEMDPMpool[chem,0],dataYearOut.CHEMRPMpool[chem,0],
                      dataYearOut.CHEMBIOpool[chem,0],dataYearOut.CHEMHUMpool[chem,0],
                      dataYearOut.CHEMIOMpool[chem,0],dataYearOut.CHEMCO2pool[chem,0]);
    system.readln(fcout);
  for i:=1 to dataYearOut.NoLayers do
  begin
      system.Read(fcout,dum,dataYearOut.RothC.DPMpool[i],dataYearOut.RothC.RPMpool[i],
                            dataYearOut.RothC.BIOpool[i],dataYearOut.RothC.HUMpool[i],
                            dataYearOut.RothC.IOMpool[i],dataYearOut.RothC.CO2pool[i],
                            dataYearOut.RothC.accTSMD[i],dataYearOut.apCO2[i]);
      for chem:=1 to untProcesses.nchem do  {nchem=constant set in untProcesses.Readc routine}
        system.read(fcout,dataYearOut.CHEMDPMpool[chem,i],dataYearOut.CHEMRPMpool[chem,i],
                          dataYearOut.CHEMBIOpool[chem,i],dataYearOut.CHEMHUMpool[chem,i],
                          dataYearOut.CHEMIOMpool[chem,i],dataYearOut.CHEMCO2pool[chem,i]);
      system.readln(fcout);
  end;
  closefile(fcout);
  {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving ReadOutC');
end;

procedure TfrmMAIN.ReadOutWE(FileNameOut:string);
{Reads annual output data from the weathering routines
 and and stores the pool-data in the record WE within the dataYearOut record.}
var fwe:textfile; i,j,idum:integer;
begin
  {If GUItrace=1 then wwwriteln} singletracewriteln('Entering ReadOutWE');
  system.assign(fwe,FileNameOut); system.reset(fwe);
  system.readln(fwe); system.readln(fwe);
  for i:=1 to dataYearOut.NoLayers do
  begin
   system.read(fwe,idum);
   for j:=1 to nrminerals do system.read(fwe,dataYearOut.WE.MinPools[j][i]);
   system.readln(fwe);
  end;
  system.readln(fwe); system.readln(fwe);
  for i:=1 to dataYearOut.NoLayers do
  begin
    system.readln(fwe,idum,dataYearOut.WE.Gibbsitepool[i],
      dataYearOut.WE.WE_Capool[i],dataYearOut.WE.WE_Mgpool[i],
      dataYearOut.WE.WE_Kpool[i], dataYearOut.WE.WE_Napool[i],
      dataYearOut.WE.WE_Sipool[i], dataYearOut.WE.WE_Fepool[i],
      DataYearOut.WE.WE_Alpool[i], dataYearOut.WE.WE_FePedogenic[i],
      dataYearOut.WE.WE_SiPedogenic[i]);
  end;
  closefile(fwe);
  {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving ReadOutWE');
end;



procedure TfrmMAIN.chckbxShowClick(Sender: TObject);
{determines if the input files can be shown as text in a Memo-box}
begin
  case chckbxShow.Checked of
   true:  begin
            mmDefaultLeachC.Visible:=false;  mmDefaultCModel.Visible:=false;
            mmWeatheringData.Visible:=false; mmEquilibrium.Visible:=false;
            rdgrpChooseDisplay.Enabled:=true;
            case rdgrpChooseDisplay.ItemIndex of
            0: mmDefaultLeachC.Visible:=true;
            1: mmEquilibrium.Visible:=true;
            2: mmDefaultCModel.Visible:=true;
            3: mmWeatheringData.Visible:=true;
            end;
          end;
   false: begin
            mmDefaultLeachC.Visible:=false;  mmDefaultCModel.Visible:=false;
            mmWeatheringData.Visible:=false; mmEquilibrium.Visible:=false;
            rdgrpChooseDisplay.Enabled:=false;
          end;
  end;
end;

Procedure TfrmMAIN.ReCalcOC;
{calculates updated %OC, clay, silt (sand), calcite, gypsum, RHO and
volumetric water content for the soil profile using decomposition routine output.
This is to take into account the C-mass added to the profile layers via
 the decomposition routine}
var oldCLAYm,oldSILTm,oldSANDm,oldWATERm,porosity,water:Profile;
    parden,totwater,totporosity: real;
    checkvolumeOK:boolean;
    i,j:integer;
begin
   {If GUItrace=1 then wwwriteln} singletracewriteln('Entering ReCalcOC');
   if WritePSD then system.write(fmasstemp,iyear);
   {oldRHO:=dataYearOut.RHO; oldAW:=dataYearOut.AW; }
   MVAR(GlobalTime*10000+Datayearin.nolayers*100,' MAIN, voor RECALCOC: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
   MVAR(GlobalTime*10000+Datayearin.nolayers*100,' MAIN, voor RECALCOC: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);
   for i:=1 to dataYearOut.NoLayers do
   begin
     {recalc CLAY, SILT en OC due to Physical Weathering}
     if PhysWeathAllowed then
     begin
       {old*m contains * as mass per compartment,m2}
       {calculated by grains * volume/grain * particle density; divide by 10000=to get from 100m2 to m2}
       for j:=-4 to 10 do mass[j,i]:=PW_K[j,i]*power((PW_X[j]/100000),3)*PW_PARDEN[j]/100;
       {NB: mass[11,i]=clay mass}
       {write to file containing mass distributions over 11 fractions}
       if (WritePSD) and (i=1) then
       begin
          {for j:=-3 to 11 do system.write(fmasstemp,mass[j,i],' '); }
          for j:=-4 to 11 do system.write(fmasstemp,PW_G[j,i],' ');
          for j:=-4 to 11 do system.write(fmasstemp,PW_K[j,i],' ');
       end;
     end;
     {in all cases: }
     oldSILTm[i]:=dataYearOut.SILTm[i];
     oldSANDm[i]:=dataYearOut.SANDm[i];
     oldCLAYm[i]:=dataYearOut.CLAYm[i];
     oldWATERm[i]:=dataYearOut.AW[i]*dataYearOut.DLZ;
     dataYearOut.OCm[i]:=(dataYearOut.RothC.DPMpool[i]+dataYearOut.RothC.RPMpool[i]+
                          dataYearOut.RothC.BIOpool[i]+dataYearOut.RothC.HUMpool[i]+
                          dataYearOut.RothC.IOMpool[i])/10; {1 ton C/ha =0.1 kg/m2)
     {convert  Clay, Silt, OC to weightpercentages.
      First estimate new bulk density assuming constant volume
      Than convert texture and OC to weight percentages}
     dataYearOut.RHO[i]:=(oldCLAYm[i]+oldSILTm[i]+oldSANDm[i]+1.7*dataYearOut.OCm[i])/(dataYearOut.DLZ*(100-datayearout.stones[i]-datayearout.gravel[i])/100);
     {undone: DataYearOut.RHO[i]:=HYPRESRHO(100*oldCLAYm[i]/(oldCLAYm[i]+oldSILTm[i]+oldSANDm[i]),1.7*100*OCm[i]/(oldCLAYm[i]+oldSILTm[i]+oldSANDm[i]+1.7*OCm[i]));  }
     {correct RHO for stone% because volume DLZ is only partly fine earth}
     {dataYearOut.CLAY[i]:=100*oldCLAYm[i]/(dataYearOut.RHO[i]*dataYearOut.DLZ);} 
     dataYearOut.CLAY[i]:=100*oldCLAYm[i]/(oldCLAYm[i]+oldSILTm[i]+oldSANDm[i]+1.7*OCm[i]);
     {dataYearOut.SILT[i]:=100*oldSILTm[i]/(dataYearOut.RHO[i]*dataYearOut.DLZ); } 
     dataYearOut.SILT[i]:=100*oldSILTm[i]/(oldCLAYm[i]+oldSILTm[i]+oldSANDm[i]+1.7*OCm[i]);
     {dataYearOut.OC[i]:=max(1E-5,100*dataYearOut.OCm[i]/(dataYearOut.RHO[i]*dataYearOut.DLZ));}  {markOC}
     datayearout.OC[i]:=1.7*100*OCm[i]/(oldCLAYm[i]+oldSILTm[i]+oldSANDm[i]+1.7*OCm[i]);
   end;

      MVAR(GlobalTime*10000+Datayearin.nolayers*100,' MAIN, na RECALCOC: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
      MVAR(GlobalTime*10000+Datayearin.nolayers*100,' MAIN, na RECALCOC: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);
   {PSD.txt: new line if physical weathering simulated}
   if WritePSD then system.writeln(fmasstemp);
   {convert  Water to volumefraction.
      Check if the maximal porosity corresponding to the new RHO
      is not exceeded}
   checkvolumeOK:=true;
   for i:=1 to dataYearOut.NoLayers do
   begin
      parden:=100/((dataYearOut.CLAY[i]/dataYearOut.RHOS2)+
                   ((100-dataYearOut.CLAY[i]-1.7*dataYearOut.OC[i])/dataYearOut.RHOS1)+
                   (1.7*dataYearOut.OC[i]/dataYearOut.RHOS3));
      {same calc particle density parden as internally in SoiGen}
      porosity[i]:=max(0.1,((100-DataYearOut.stoneperc[i])/100)*(1-(DataYearOut.RHO[i]/parden))); {do not allow negative porosities due to high OM...}
      water[i]:=oldWATERm[i]/dataYearOut.DLZ;
      {water[i] now contains volumetric moisture content; still to be checked with porosity}
      if water[i]>porosity[i] then checkvolumeOK:=false;
   end;
   if not(checkvolumeOK) then {porosity in minimally 1 compartment too low for water content}
   begin
     totwater:=0; totporosity:=0;
     for i:=1 to dataYearOut.NoLayers do
     begin totwater:=totwater+water[i]; totporosity:=totporosity+porosity[i]; end;
     if totwater<=totporosity then {there is sufficient porosity for all water}
     begin
       {fill in de mixed zone in each compartment the same fraction of the
        pore volume with water such that mass is conserved}
       for i:=1 to dataYearOut.NoLayers do
       begin
          dataYearOut.AW[i]:=porosity[i]*totwater/totporosity;
       end;
     end
     else {problem: not all water can be distributed}
     begin
        {write a message!}
        dshowmessage('Year '+currentyear+'; Not fatal (recalculation OC): Cannot redistribute water; porosity too low!');
        {solution: all pores completely filled with water, but surplus water is not
         redistributed over other profile parts; this to limit mass balance errors
         to water only and not chemical masses}
         for i:=1 to dataYearOut.NoLayers do
         begin
           dataYearOut.AW[i]:=porosity[i];
         end;
     end;
   end
   else {porosity check OK}
   begin
     {distribute the water definitively according to the calculated mass distribution}
     for i:=1 to dataYearOut.NoLayers do dataYearOut.AW[i]:=water[i];
   end;
   {normally, chemicals in solution and on exchange sites should be recalculated as well
    and chemical equilibria should again be established, but
    re-equilibration is done on a regular basis inside the SoilGen code and as well
    when the procedure Turbation is called.}
   {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving ReCalcOC');
end;

procedure TfrmMAIN.InitChem;
{initializes CEC, exchangeable cations, Gapon coefficients, pCO2, calcite and gypsum contents.
To be updated after reading inputfiles}
var M,I:integer;
begin
   {If GUItrace=1 then wwwriteln} singletracewriteln('Entering InitChem');
  for I:=1 to MaxLayers do
  begin
    {SC1=Mg/Ca; SC2=Ca/Na; SC3=Ca/K;  SC4=Mg/K;  SC5= Mg/Na; SC6= K/Na;
     SC7=Ca/Al; SC8=Ca/H;  SC9=Mg/Al; SC10=K/Al; SC11= Na/Al; SC12=Mg/H;
     SC13=K/H;  SC14=Na/H; SC15=Al/H }
    DataInit.ACEC[i]:=0;  DataInit.APCO2[i]:=pCO2atm;
    DataInit.CALCITEA[i]:=0; DataInit.GYPA[i]:=0;
    DataInit.SC1[i]:=0.5; DataInit.SC2[i]:=0.5; DataInit.SC3[i]:=0.5;
    DataInit.SC4[i]:=0.5; DataInit.SC5[i]:=0.5; DataInit.SC6[i]:=0.5;  
    DataInit.SC7[i]:=0.5; DataInit.SC8[i]:=0.5; DataInit.SC9[i]:=0.5;
    DataInit.SC10[i]:=0.5; DataInit.SC11[i]:=0.5; DataInit.SC12[i]:=0.5;
    DataInit.SC13[i]:=0.5; DataInit.SC14[i]:=0.5; DataInit.SC15[i]:=0.5;
    for M:=1 to MaxChem do
    begin
      DataInit.AXSE[m,i]:=0; DataInit.SEA[m,i]:=0;
    end;
  end;  
   {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving InitChem');
end;


procedure TfrmMAIN.ReadCheckFertil;
{reads the fertilization file "fertilization.txt" and produces "fertilINT.txt" with annual values,
 NO interpolation (no data=values of 0)}
var BPyear,year,yeara,yearb,startBPYearSim,dy:integer;
    ffert,ffert2:textfile;
    txta,txtb: string;
    Ca,Mg,Na,K,Cl,SO4,HCO3,CO3,Caa,Mga,Naa,Ka,Cla,SO4a,HCO3a,CO3a,
    Cab,Mgb,Nab,Kb,Clb,SO4b,HCO3b,CO3b:real;
begin
   {If GUItrace=1 then wwwriteln} singletracewriteln('Entering ReadCheckFertil');
   startBPYearSim:=spndtMultipleYears.Value;
   if (fileexists('fertilization.txt'))and(chckbxFertilization.Checked) then
   begin
     system.Assign(ffert2,'fertilINT.txt'); system.Rewrite(ffert2);
     system.Assign(ffert,'fertilization.txt');  system.Reset(ffert);
     system.Readln(ffert); {header weg}
     system.Readln(ffert,BPyear,Cab,Mgb,Nab,Kb,Clb,SO4b,HCO3b,CO3b,txtb);
     yearb:=startBPYearSim-BPyear;
     {if first input year is not starting year (1) of the simulation:
      fill the gap with 0}
     if yearb>1 then
     begin
       for year:=1 to min(startBPYearSim,yearb-1) do
       system.writeln(ffert2,year,'  0.00   0.00   0.00   0.00 ',
                             '  0.00   0.00   0.00   0.00 ',txtb);
     end;
     if not (eof(ffert)) then
     repeat
       {stel de *a variabelen gelijk aan de reeds ingelezen *b variabelen}
       yeara:=yearb;
       Caa:=Cab;   Mga:=Mgb;    Naa:=Nab;     Ka:=Kb;
       Cla:=Clb;   SO4a:=SO4b;  HCO3a:=HCO3b; CO3a:=CO3b;
       txta:=txtb;
       {write the *a variables to file}
       if (yeara<=startBPYearSim)and(yeara>0)
       then system.writeln(ffert2,yeara,' ',Caa:5:2,' ',Mga:5:2,' ',Naa:5:2,' ',Ka:5:2,' ',
                             Cla:5:2,' ',SO4a:5:2,' ',HCO3a:5:2,' ',CO3a:5:2,' ',txta);
       {read the new set of *.b variables}
       system.Readln(ffert,BPyear,Cab,Mgb,Nab,Kb,Clb,SO4b,HCO3b,CO3b,txtb);
       yearb:=startBPYearSim-BPyear;
       {if there is more than 1 year between *a and *b: set to 0 (not fertilization)}
       if yearb-yeara>1 then
       begin
         dy:=yearb-yeara;
         for year:=min(startBPYearSim,yeara)+1
             to    min(startBPYearSim,yearb-1)
             do
         begin
           Ca:=0;   Mg:=0;      Na:=0;
           K:=0;    Cl:=0;      SO4:=0;
           HCO3:=0; CO3:=0;
           if (year>0)
           then system.writeln(ffert2,year,' ',Ca:5:2,' ',Mg:5:2,' ',Na:5:2,' ',K:5:2,' ',
                             Cl:5:2,' ',SO4:5:2,' ',HCO3:5:2,' ',CO3:5:2,' ',txta);
         end;
       end;
     until (eof(ffert));
     {write last year}
     if (yearb<=startBPYearSim)and(yearb>0)
     then system.writeln(ffert2,yearb,' ',Cab:5:2,' ',Mgb:5:2,' ',Nab:5:2,' ',Kb:5:2,' ',
                             Clb:5:2,' ',SO4b:5:2,' ',HCO3b:5:2,' ',CO3b:5:2,' ',txtb);
     {if last input year is not final simulation year:
      fill the gap (with 0)}
     if yearb<startBPYearSim then
     begin
       for year:=yearb+1 to startBPYearSim do
       if (year>0)
       then system.writeln(ffert2,year,'  0.00   0.00   0.00   0.00 ',
                             '  0.00   0.00   0.00   0.00 ',txtb);
     end;
     closefile(ffert);
     closefile(ffert2);
   end
   else begin if fileexists('fertilINT.txt') then deletefile('fertilINT.txt'); end;
   {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving ReadCheckFertil');
end;

procedure TfrmMAIN.btbttnOpenWeatheringClick(Sender: TObject);
{opens the file with initial mineralogy input}
begin
  {de-activate menu item to display dump.txt. This is because it will fill
   the data fields, which may override the read input}
   showmemorydump1.Visible:=false;
   {If GUItrace=1 then wwwriteln} singletracewriteln('Entering btbttnOpenWeatheringClick');
   pndlgOPEN.Filter:= 'Initial mineralogy input files (*.txt)|*.txt';
   if pndlgOPEN.Execute then
   begin
      mmWeatheringData.Lines.LoadFromFile(pndlgOPEN.FileName);
      mmWeatheringData.Enabled:=true;
      ReadDataWE(pndlgOPEN.FileName);
      ALLOWED:=readweatherOK;
      if readweatherOK then
      begin
        btbttnEditWeathering.Enabled:=true;
      end;
      FirstWeatherFileName:=pndlgOPEN.FileName;
      {DataYearIn:=DataInit;}  CopyRecToFrom(DatayearIn,DataInit);
      if (not(bttnErrorInputs.Visible))and(fileexists('Messages.txt')and(warningsissued))
      then bttnErrorInputs.Visible:=true;
   end;  
   {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving btbttnOpenWeatheringClick');
end;

procedure TfrmMAIN.rdgrpChooseDisplayClick(Sender: TObject);
{determines which of the input files is shown as text in a Memo-box}
begin
  mmDefaultLeachC.Visible:=false;  mmDefaultCModel.Visible:=false;
  mmWeatheringData.Visible:=false; mmEquilibrium.Visible:=false;
  mmBedrock.Visible:=false;
  case rdgrpChooseDisplay.ItemIndex of
    0: mmDefaultLeachC.Visible:=true;
    1: mmEquilibrium.Visible:=true;
    2: mmDefaultCModel.Visible:=true;
    3: mmWeatheringData.Visible:=true;
    4: mmBedrock.Visible:=true;
  end;
end;

procedure TfrmMAIN.btbttnClimateChangeClick(Sender: TObject);
{opens the window to allow entering/editing the file with climate data
 and saving it to the correct format}
var i:integer;
begin
   for i:=1 to MaxNoYears do
   begin
     untClimate.climdata[i].BPyear:=-1;
     untClimate.climdata[i].P:=0;
     untClimate.climdata[i].PE:=0;
     untClimate.climdata[i].JanT:=0;
     untClimate.climdata[i].JulT:=0;
     untClimate.climdata[i].PlantResSum:=0;
     untClimate.climdata[i].ManureSum:=0;
     untClimate.climdata2[i].VEG:='';
     untClimate.climdata2[i].Period:='';
     untClimate.frmClimate.ResetClim;
   end;
   untClimate.frmClimate.ShowModal;
end;

procedure TfrmMAIN.chckbxClimateClick(Sender: TObject);
{identifies if a correctly formatted and named climate file is available}
begin
  if chckbxClimate.Checked
  then btbttnClimateChange.Enabled:=false
  else btbttnClimateChange.Enabled:=true;
end;

procedure TfrmMAIN.chckbxBioturbationClick(Sender: TObject);
{identifies if a correctly formatted and named bioturbation file is available}
begin
  if chckbxBioturbation.Checked
  then btbttnBioturbation.Enabled:=false
  else btbttnBioturbation.Enabled:=true;
end;

procedure TfrmMAIN.debuglog(txt:string);
{writes a (non-fatal) input error to 'Messages.txt'}
var fdebug:textfile;
begin
  if fileexists('Messages.txt')
  then begin system.Assign(fdebug,'Messages.txt'); system.append(fdebug); end
  else begin system.Assign(fdebug,'Messages.txt'); system.rewrite(fdebug); end;
  system.writeln(fdebug,txt); system.flush(fdebug);
  closefile(fdebug);
end;

procedure TfrmMAIN.btbttnOpenEquilibriumClick(Sender: TObject);
{opens the window to allow editing the file with chemical equilibrium constants data
 and saving it to the correct format}
begin
  {de-activate menu item to display dump.txt. This is because it will fill
   the data fields, which may override the read input}
   showmemorydump1.Visible:=false;
   {If GUItrace=1 then wwwriteln} singletracewriteln('Entering btbttnOpenEquilibriumClick');
   pndlgOPEN.Filter:= 'Equilibrium constant input files (*.txt)|*.txt';
   if pndlgOPEN.Execute then
   begin
      mmEquilibrium.Lines.LoadFromFile(pndlgOPEN.FileName);
      mmEquilibrium.Enabled:=true;
      EQFileName:=pndlgOPEN.FileName;
      ReadDataEQ(pndlgOPEN.FileName);
      if (not(bttnErrorInputs.Visible))and(fileexists('Messages.txt')and(warningsissued))
      then bttnErrorInputs.Visible:=true;
      if ReadDataEQOK
      then
      begin
        btbttnOpenDefaultCModel.Enabled:=true;
        btbttnEditEquilibria.Enabled:=true;
      end;
   end;  
   {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving btbttnOpenEquilibriumClick');
end;

procedure TfrmMAIN.ReadDataRock(FileName:string);
{reads the datafile "bedrock.txt" with density and mass fraction per mineral and also the rock density}
var i,q,maxq,item:integer; dum1,dum2:real; f:textfile; msg:string;
begin
   {If GUItrace=1 then wwwriteln} singletracewriteln('Entering ReadDataRock');
  maxq:=mmBedRock.Lines.Count;
  try
    system.assign(f,'Bedrock.txt'); system.reset(f);
    system.readln(f);   q:=1;
    system.readln(f,Rock.rockAlbite[1],Rock.rockAlbite[3]);   q:=q+1;
    system.readln(f,Rock.rockKFeldspar[1],Rock.rockKFeldspar[3]); q:=q+1;
    system.readln(f,Rock.rockMuscovite[1],Rock.rockMuscovite[3]); q:=q+1;
    system.readln(f,Rock.rockBiotite[1],Rock.rockBiotite[3]); q:=q+1;
    system.readln(f,Rock.rockAnorthite[1],Rock.rockAnorthite[3]); q:=q+1;
    system.readln(f,Rock.rockForsterite[1],Rock.rockForsterite[3]);  q:=q+1;
    system.readln(f,Rock.rockAugite[1],Rock.rockAugite[3]);  q:=q+1;
    system.readln(f,Rock.rockHornblende[1],Rock.rockHornblende[3]); q:=q+1;
    system.readln(f,Rock.rockFayalite[1],Rock.rockFayalite[3]); q:=q+1;
    system.readln(f,Rock.rockQuartz[1],Rock.rockQuartz[3]);  q:=q+1;
    system.readln(f,Rock.rockChlorite[1],Rock.rockChlorite[3]);  q:=q+1;
    system.readln(f,Rock.rockKaolinite[1],Rock.rockKaolinite[3]); q:=q+1;
    system.readln(f,Rock.rockIllite[1],Rock.rockIllite[3]); q:=q+1;
    system.readln(f,Rock.rockMontmorillonite[1],Rock.rockMontmorillonite[3]);q:=q+1;
    system.readln(f,Rock.rockAmorphite[1],Rock.rockAmorphite[3]); q:=q+1;
    system.readln(f,Rock.rockOtherite[1],Rock.rockOtherite[3]); q:=q+1;
    system.readln(f,Rock.rockGibbsite[1],Rock.rockGibbsite[3]); q:=q+1;
    system.readln(f,Rock.rockCalcite[1],Rock.rockCalcite[3]); q:=q+1;
    system.readln(f,Rock.rockGypsum[1],Rock.rockGypsum[3]); q:=q+1;
    system.readln(f,Rock.rockdensity); q:=q+1;
    closefile(f);
    ReadBedRockOK:=true;
    rdgrpChooseDisplay.Columns:=5;
    rdgrpChooseDisplay.ItemIndex:=0;   {was 4}
   {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving ReadDataRock');
  except
    ReadBedRockOK:=false; 
    rdgrpChooseDisplay.Columns:=5;  {was 4}
    rdgrpChooseDisplay.ItemIndex:=0; {was 3}
    if q<maxq then
    begin str(q,Msg); Msg:=' Error in inputfile '+'Bedrock.txt'+' near line '+Msg; end;
    dshowmessage(Msg);
    dshowmessage('You will not be able to run SoilGen unless you correct this error and reload.');
    {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving ReadDataRock with errors');
  end;
end;

procedure TfrmMAIN.ReadDataEQ(filename:string);
{reads the datafile with chemical equilibrium constants and dH-values
 for Arrhenius temperature correction}
var i,q,maxq,item:integer; dum1,dum2:real; fEQ{,fdum}:textfile; msg:string;
begin
   {If GUItrace=1 then wwwriteln} singletracewriteln('Entering ReadDataEQ');
  maxq:=mmEquilibrium.Lines.Count;
  try
  {read the datafile with chemical equilibrium constants and dH-values
   for Arrhenius temperature correction}
    system.Assign(fEQ,FileName);
    for i:=1 to maxK do
    begin  q:=0;
      system.reset(fEQ);
      repeat q:=q+1; system.readln(fEQ,dum1,dum2,txt);
        while pos(' ',txt)=1 do delete(txt,1,1);
      until (pos(DataInit.knames[i],txt)>0)or(q=maxq);
      if q=maxq then raise EInOutError.Create('File incomplete')
                else begin
                       DataInit.kvalues[i,1]:=dum1;
                       DataInit.kvalues[i,2]:=dum2;
                       for item:=-10 to 40 do
                       begin   {Arrhenius / Van'tHoff corrections for various temperatures}
                         DataInit.KValueTable[i,item]:=
                            dum1*exp(dum2*1000*(  (1/298.15)-(1/(273.15+item))  )
                                     /8.314);
                       end;
                     end;
    end;
    for item:=-10 to 40 do
    begin
     {Arrhenius / Van'tHoff corrections for various temperatures for Gibbsite, Gypsum, Calcite.
      Only used in saprolite!}
         DataInit.KValueTable[50,item]:= power(10,9.35)*exp(-95.395*1000*((1/298.15)-(1/(273.15+item)))/8.314); {Gibbsite}
         DataInit.KValueTable[51,item]:= power(10,-4.58)*exp(-0.456*1000*((1/298.15)-(1/(273.15+item)))/8.314); {Gypsum}
         DataInit.KValueTable[52,item]:= power(10,-8.356)*exp(-9.611*1000*((1/298.15)-(1/(273.15+item)))/8.314); {Calcite}
    end;
    closefile(fEQ);
    {for debugging purposes only: }
    {system.assign(fdum,'K-table.csv'); system.rewrite(fdum);
    system.write(fdum,'Name+Temp>;');
    for item:=-10 to 40 do system.write(fdum,item,';'); system.writeln(fdum);
    for i:=1 to MaxK do
    begin
      system.write(fdum,DataInit.knames[i],';');
      for item:=-10 to 40 do system.write(fdum,log10(DataInit.KValueTable[i,item]):6:3,';');
      system.writeln(fdum);
    end;
    closefile(fdum); }

    ReadDataEQOK:=true;
   {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving ReadDataEQ');
  except
    ReadDataEQOK:=false;
    if q<maxq then
    begin str(q,Msg); Msg:='Year '+currentyear+': Error in inputfile '+filename+' near line '+Msg end
    else Msg:='Year '+currentyear+'; Fatal error: Equilibrium constant not found in file '+filename;
    dshowmessage(Msg);
    dshowmessage('You will not be able to run SoilGen unless you correct this error and reload.');
    {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving ReadDataEQ with errors');
  end;
end;

procedure TfrmMAIN.FormClose(Sender: TObject; var Action: TCloseAction);
{closes the main window, on fatal error or on program termination}
var ii,j,cols,rows,rhocols,rhorows,rhosize:integer;  massum,mass,rhovalue:real;
    fout:textfile; namefin:array[1..22] of string; namefout:string ;
   regel1,regel2,regel3,txt:string;
begin
if not(chckbxAbortNOW.Checked or othererror)
then
begin
  if fileexists('Albite(kgNaAlSi3O8perm2).tdd') then
begin
  {open all mineral masses files}
  system.assign(f67,'Albite(kgNaAlSi3O8perm2).tdd'); reset(f67);          for ii:=1 to 3 do system.readln(f67);
  system.assign(f68,'K-feldspar(kgKAlSi3O8perm2).tdd'); reset(f68);       for ii:=1 to 3 do system.readln(f68);
  system.assign(f69,'Muscovite(kgKAl3Si3O10(OH)2perm2).tdd'); reset(f69); for ii:=1 to 3 do system.readln(f69);
  system.assign(f70,'Biotite(kgKMg1.5Fe1.5AlSi3O10(OH)2perm2).tdd'); reset(f70);for ii:=1 to 3 do system.readln(f70);
  system.assign(f71,'Quartz(kgSiO2perm2).tdd'); reset(f71);               for ii:=1 to 3 do system.readln(f71);
  system.assign(f72,'ChloriteAsClinochlore(kgMg5Al2Si3O10(OH)8perm2).tdd'); reset(f72);for ii:=1 to 3 do system.readln(f72);
  system.assign(f73,'Anorthite(kgCaAl2Si2O8perm2).tdd'); reset(f73);      for ii:=1 to 3 do system.readln(f73);
  system.assign(f74,'Fosterite(kgMg2SiO4perm2).tdd'); reset(f74);         for ii:=1 to 3 do system.readln(f74);
  system.assign(f75,'AugiteAsOrthoferrosilite(kgFeSiO3perm2).tdd'); reset(f75);for ii:=1 to 3 do system.readln(f75);
  system.assign(f76,'Kaolinite(kgAl2Si2O5(OH)4perm2).tdd'); reset(f76);   for ii:=1 to 3 do system.readln(f76);
  system.assign(f77,'Illite(kgSi3.4O10(OH)2Mg0.02Al2.38Ca0.01K0.44perm2).tdd'); reset(f77);for ii:=1 to 3 do system.readln(f77);
  system.assign(f78,'MontmorilloniteAverageComposition(kgSi4O10(OH)2Mg0.33Al1.67Ca0.04125Mg0.04125K0.0825Na0.0825perm2).tdd'); reset(f78); for ii:=1 to 3 do system.readln(f78);
  system.assign(f79,'Hornblende(kgCa2Mg4AlSi6.7AlO22.4perm2).tdd'); reset(f79); for ii:=1 to 3 do system.readln(f79);
  system.assign(f80,'Fayalite(kgFe2SiO4perm2).tdd'); reset(f80);          for ii:=1 to 3 do system.readln(f80);
  system.assign(f82,'Amorphite(kgperm2).tdd'); reset(f82);                for ii:=1 to 3 do system.readln(f82);
  system.assign(f83,'Otherite(kgperm2).tdd'); reset(f83);                 for ii:=1 to 3 do system.readln(f83);
  system.assign(f4a,'Calcite(kgCaCO3perm2).tdd'); reset(f4a);             for ii:=1 to 3 do system.readln(f4a);
  system.assign(f14a,'Gypsum(kgCaSO4perm2).tdd'); reset(f14a);            for ii:=1 to 3 do system.readln(f14a);
  system.assign(f44a,'Gibbsite(kgAlOH3perm2).tdd'); reset(f44a);          for ii:=1 to 3 do system.readln(f44a);
  system.assign(f87a,'Pedogenic-Fe(kgFeOOHperm2).tdd'); reset(f87a);      for ii:=1 to 3 do system.readln(f87a);
  system.assign(f88a,'Pedogenic-Si(kgH4SiO4perm2).tdd'); reset(f88a);     system.readln(f88a,rhocols,regel1); system.readln(f88a,rhorows,regel2); system.readln(f88a,rhosize,regel3);
  {open outputfile 'Masssum (kgperm2).tdd'}
  system.assign(fout,'Masssum (kgperm2).tdd'); system.rewrite(fout);
  system.WriteLn(fout,rhocols,' (ncols, no. compartments) UNITS=(kgperm2)');  system.WriteLn(fout,rhorows,regel2); system.WriteLn(fout,rhosize,regel3);
  {calculate total mass using all mineral output files}
  lblNOTE.Caption:='reading all mineral files and calculating total mineral mass';lblNOTE.refresh;
  for rows:=1 to rhorows do
  begin
   for cols:=1 to rhocols do
   begin
     massum:=0;
     system.read(f67,mass); if mass>-998 then massum:=massum+mass;    
     system.read(f68,mass); if mass>-998 then massum:=massum+mass;
     system.read(f69,mass); if mass>-998 then massum:=massum+mass;
     system.read(f70,mass); if mass>-998 then massum:=massum+mass;
     system.read(f71,mass); if mass>-998 then massum:=massum+mass;
     system.read(f72,mass); if mass>-998 then massum:=massum+mass;
     system.read(f73,mass); if mass>-998 then massum:=massum+mass;
     system.read(f74,mass); if mass>-998 then massum:=massum+mass;
     system.read(f75,mass); if mass>-998 then massum:=massum+mass;
     system.read(f76,mass); if mass>-998 then massum:=massum+mass;
     system.read(f77,mass); if mass>-998 then massum:=massum+mass;
     system.read(f78,mass); if mass>-998 then massum:=massum+mass;
     system.read(f79,mass); if mass>-998 then massum:=massum+mass;
     system.read(f80,mass); if mass>-998 then massum:=massum+mass;
     system.read(f82,mass); if mass>-998 then massum:=massum+mass;
     system.read(f83,mass); if mass>-998 then massum:=massum+mass;
     system.read(f4a,mass); if mass>-998 then massum:=massum+mass;
     system.read(f14a,mass); if mass>-998 then massum:=massum+mass;
     system.read(f44a,mass); if mass>-998 then massum:=massum+mass;
     system.read(f87a,mass); if mass>-998 then massum:=massum+mass;
     system.read(f88a,mass); if mass>-998 then massum:=massum+mass;
     {write to Masssum (kgperm2).tdd}
     if massum=0 then system.write(fout,' -999') else system.write(fout,massum,' ');
   end;
   system.readln(f67);    system.readln(f68);   system.readln(f69);
   system.readln(f70);    system.readln(f71);   system.readln(f72);
   system.readln(f73);    system.readln(f74);   system.readln(f75);
   system.readln(f76);    system.readln(f77);   system.readln(f78);
   system.readln(f79);    system.readln(f80);   system.readln(f82);
   system.readln(f83);    system.readln(f4a);   system.readln(f14a);
   system.readln(f44a);   system.readln(f87a);  system.readln(f88a);
   system.writeln(fout);
  end;
  lblNOTE.Caption:='Total mineral mass written to Masssum (kgperm2).tdd';lblNOTE.refresh;
  closefile(f67);    closefile(f68);   closefile(f69);
  closefile(f70);    closefile(f71);   closefile(f72);
  closefile(f73);    closefile(f74);   closefile(f75);
  closefile(f76);    closefile(f77);   closefile(f78);
  closefile(f79);    closefile(f80);   closefile(f82);
  closefile(f83);    closefile(f4a);   closefile(f14a);
  closefile(f44a);   closefile(f87a);  closefile(f88a);
  closefile(fout);
end;
  {finalize by closing the window after the message box is OK'd}
  if fileexists(edtMonitName.Text)
  then txt:=versionid+' has finished. For scenario settings read <'+edtMonitName.Text+
       '>'
  else txt:=versionid+' has finished.';
  if fileexists('Messages.txt')
  then txt:=txt+#13#10+#13#10+'Some problems were encountered during runtime;'+#13#10+
                   'Read the file <Messages.txt> for details';
  if fileexists('dump.txt')
  then txt:=txt+#13#10+#13#10+'A memory dump exists. Re-start SoilGen and use menu'+#13#10+
            'to display the variable values at the time of the crash.';
  showmessage(txt);
end;
end;

procedure TfrmMAIN.bttnErrorInputsClick(Sender: TObject);
{shows the contents of the file Messages.txt (containing non-fatal input errors}
begin
  untNonFatal.frmNonFatal.memo.Lines.LoadFromFile('Messages.txt');
  untNonFatal.frmNonFatal.ShowModal;
end;

procedure TfrmMAIN.btbttnEditSoilDataClick(Sender: TObject);
{opens the window to enter and edit initial soil data and mineralogical data for all profile layers}
begin
  {de-activate menu item to display dump.txt. This is because it will fill
   the data fields, which may override the read input}
   showmemorydump1.Visible:=false;
   untEditSoilData.frmEditSoilData.ShowModal;
end;

procedure TfrmMAIN.dshowmessage(txt:string);
{same as debuglog, used also for fatal errors}
begin
  debuglog(txt); {debuglog('***'); }
  crashtext:=txt;
end;

procedure TfrmMAIN.btbttnFertilizationChangeClick(Sender: TObject);
{opens the window to allow entering/editing the file with fertilization data
 and saving it to the correct format}
begin
  untImportFertilizerData.frmImportFertilizerData.ShowModal;
end;

procedure TfrmMAIN.chckbxFertilizationClick(Sender: TObject);
{identifies if a correctly formatted and named fertilization file is available}
begin
  if chckbxFertilization.Checked
  then btbttnFertilizationChange.Enabled:=false
  else btbttnFertilizationChange.Enabled:=true;
end;

procedure TfrmMAIN.chckbxClayMigrationClick(Sender: TObject);
{determines if clay migration is to be simulated,
 always TRUE and the checkbox is not visible at present}
begin
  if chckbxClayMigration.Checked
  then
  begin
     ClayMigrationAllowed:=true;
     ShowWindowHeader;
  end
  else
  begin
     ClayMigrationAllowed:=false;
     ShowWindowHeader;
  end;
end;

procedure TfrmMAIN.ShowWindowHeader;
{writes a context-specific line at the top of the GUI-screen
 ending with "Note: semi-permanent beta-version"}
var txt:string;
begin
  if PhysWeathAllowed
  then txt:='and Physical weathering ' else txt:='';
  if ClayMigrationAllowed
  then  txt:=txt+'and Clay migration '
  else  txt:=txt+'';
  MacroMicroMM:=untProcesses.clay_hthetamacro; {hydraulic head (mm) at which macropores empty}
  grpbxEditData.Visible:=EditAllowed;
  lblNote.Caption:='Note: semi-permanent beta-version';

end;

procedure TfrmMAIN.FormCreate(Sender: TObject);
{some initializations when the program starts and the main form is created}
var yearBP:integer; txt:string;
begin
  startdir:=getcurrentdir;
  chckbxSpecialClimate.Checked:=CheckBoxMonthlyData;
  chckbxSpecialClimate.Visible:=CheckBoxMonthlyData;
  chckbxSpecialClimate.enabled:=false;
  spndtTraceStart.MaxValue:=MaxNoYears;
  spndtMultipleYears.MaxValue:=MaxNoYears;
  frmMain.Caption:=versionid+': Soil genesis model for various (but not all) parent materials';
  str(MaxNoYears,txt);
  txt:='(ending at present, starting at max '+txt+' BP)';
  label8.Caption:=txt;
  chemstarted:=false;
  {tracing can be set ON directly}
  {rdgrpTrace.Visible:=true;}
  {but tracing can be modified only after time extent is set}
  {spndtTraceStart.Visible:=false;
  lblTraceStart.Visible:=false; }
  pCO2atm:=def_pCO2atm;
  CurrentPlowMixFraction:=0.5; {default mixing fraction by plowing; should be 0.5}
  for YearBP:=1 to MaxNoyears do SeriespCO2[YearBP]:=pCO2atm;
  {initialisation in case no d14C values are provided by file:
   d14C=0, so the transformed variable =(d14c/1000)+1 should get the value =1}
  for YearBP:=-MaxNoYears to MaxNoYears do d14C[yearBP]:=1;
  CheckSoftConstants;
end;

procedure TfrmMAIN.About1Click(Sender: TObject);
{shows the window with "About"  information}
begin
  untAbout.frmAbout.ShowModal;
end;

procedure TfrmMAIN.chckbxEventsClick(Sender: TObject);
{identifies if a correctly formatted and named events file is available}
begin
  if chckbxEvents.Checked
  then btbttnEventsChange.Enabled:=false
  else btbttnEventsChange.Enabled:=true;
end;

procedure TfrmMAIN.ShiftCompartments(nr:integer);
{shifts the data of all profile-variables NR compartments upward or downward
 NR is number of added compartments on top of profile; thus data are shifted
 downward when NR is positive. Ecto-organic layers are either buried + incorporated
 in the first added compartment or set to 0 in case of compartments removed by erosion}
var i,j,newcompno,oldmaxcomp:integer; Soilprops: Profile;
begin
 {If GUItrace=1 then wwwriteln} singletracewriteln('Entering ShiftCompartments');
{RothC Record as part of DataYearInit, DataYearIn and DataYearOut records}
 for i:=1 to 12 do  {monthly values}
 begin
   Shift1Profile(DataInit.RothC.Temperatures[i],nr);
   Shift1Profile(DataYearIn.RothC.Temperatures[i],nr);
   Shift1Profile(DataYearOut.RothC.Temperatures[i],nr);
   Shift1Profile(DataInit.RothC.PlantresiduesIN[i],nr);
   Shift1Profile(DataYearIn.RothC.PlantresiduesIN[i],nr);
   Shift1Profile(DataYearOut.RothC.PlantresiduesIN[i],nr);   
   Shift1Profile(DataInit.RothC.ManureIN[i],nr);
   Shift1Profile(DataYearIn.RothC.ManureIN[i],nr);
   Shift1Profile(DataYearOut.RothC.ManureIN[i],nr);
 end;
 Shift1Profile(DataYearIn.RothC.ClayPerc,nr);      Shift1Profile(DataYearOut.RothC.ClayPerc,nr);      Shift1Profile(DataInit.RothC.ClayPerc,nr);
 Shift1Profile(DataYearIn.RothC.fracManureIN,nr);  Shift1Profile(DataYearOut.RothC.fracManureIN,nr);  Shift1Profile(DataInit.RothC.fracManureIN,nr);
 Shift1Profile(DataYearIn.RothC.fracPlantIN,nr);   Shift1Profile(DataYearOut.RothC.fracPlantIN,nr);   Shift1Profile(DataInit.RothC.fracPlantIN,nr);
 Shift1Profile(DataYearIn.RothC.DPMpool,nr);       Shift1Profile(DataYearOut.RothC.DPMpool,nr);       Shift1Profile(DataInit.RothC.DPMpool,nr);
 Shift1Profile(DataYearIn.RothC.RPMpool,nr);       Shift1Profile(DataYearOut.RothC.RPMpool,nr);       Shift1Profile(DataInit.RothC.RPMpool,nr);
 Shift1Profile(DataYearIn.RothC.BIOpool,nr);       Shift1Profile(DataYearOut.RothC.BIOpool,nr);       Shift1Profile(DataInit.RothC.BIOpool,nr);
 Shift1Profile(DataYearIn.RothC.HUMpool,nr);       Shift1Profile(DataYearOut.RothC.HUMpool,nr);       Shift1Profile(DataInit.RothC.HUMpool,nr);
 Shift1Profile(DataYearIn.RothC.IOMpool,nr);       Shift1Profile(DataYearOut.RothC.IOMpool,nr);       Shift1Profile(DataInit.RothC.IOMpool,nr);
 Shift1Profile(DataYearIn.RothC.CO2pool,nr);       Shift1Profile(DataYearOut.RothC.CO2pool,nr);       Shift1Profile(DataInit.RothC.CO2pool,nr);
 Shift1Profile(DataYearIn.RothC.DPMXpool,nr);       Shift1Profile(DataYearOut.RothC.DPMXpool,nr);     Shift1Profile(DataInit.RothC.DPMXpool,nr);
 Shift1Profile(DataYearIn.RothC.RPMXpool,nr);       Shift1Profile(DataYearOut.RothC.RPMXpool,nr);     Shift1Profile(DataInit.RothC.RPMXpool,nr);
 Shift1Profile(DataYearIn.RothC.BIOXpool,nr);       Shift1Profile(DataYearOut.RothC.BIOXpool,nr);     Shift1Profile(DataInit.RothC.BIOXpool,nr);
 Shift1Profile(DataYearIn.RothC.HUMXpool,nr);       Shift1Profile(DataYearOut.RothC.HUMXpool,nr);     Shift1Profile(DataInit.RothC.HUMXpool,nr);
 Shift1Profile(DataYearIn.RothC.IOMXpool,nr);       Shift1Profile(DataYearOut.RothC.IOMXpool,nr);     Shift1Profile(DataInit.RothC.IOMXpool,nr);
 Shift1Profile(DataYearIn.RothC.CO2Xpool,nr);       Shift1Profile(DataYearOut.RothC.CO2Xpool,nr);     Shift1Profile(DataInit.RothC.CO2Xpool,nr);
 Shift1Profile(DataYearIn.RothC.accTSMD,nr);       Shift1Profile(DataYearOut.RothC.accTSMD,nr);       Shift1Profile(DataInit.RothC.accTSMD,nr);
 Shift1Profile(DataYearIn.RothC.pCO2,nr);          Shift1Profile(DataYearOut.RothC.pCO2,nr);          Shift1Profile(DataInit.RothC.pCO2,nr);
 Shift1Profile(DataYearIn.stoneperc,nr);          Shift1Profile(DataYearOut.stoneperc,nr);            Shift1Profile(DataInit.stoneperc,nr);
 for i:=1 to MaxChem do  {shift all ions associated with OC, except those of ectorganic layer (j=0)}
 begin
   for j:=1 to MaxLayers do SoilProps[j]:=DataYearIn.CHEMDPMpool[i][j];
   Shift1Profile(SoilProps,nr);
   for j:=1 to MaxLayers do DataYearIn.CHEMDPMpool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataYearIn.CHEMRPMpool[i][j];
   Shift1Profile(SoilProps,nr);
   for j:=1 to MaxLayers do DataYearIn.CHEMRPMpool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataYearIn.CHEMBIOpool[i][j];
   Shift1Profile(SoilProps,nr);
   for j:=1 to MaxLayers do DataYearIn.CHEMBIOpool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataYearIn.CHEMHUMpool[i][j];
   Shift1Profile(SoilProps,nr);
   for j:=1 to MaxLayers do DataYearIn.CHEMHUMpool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataYearIn.CHEMIOMpool[i][j];
   Shift1Profile(SoilProps,nr);
   for j:=1 to MaxLayers do DataYearIn.CHEMIOMpool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataYearIn.CHEMCO2pool[i][j];
   Shift1Profile(SoilProps,nr);
   for j:=1 to MaxLayers do DataYearOut.CHEMCO2pool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataYearOut.CHEMDPMpool[i][j];
   Shift1Profile(SoilProps,nr);
   for j:=1 to MaxLayers do DataYearOut.CHEMDPMpool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataYearOut.CHEMRPMpool[i][j];
   Shift1Profile(SoilProps,nr);
   for j:=1 to MaxLayers do DataYearOut.CHEMRPMpool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataYearOut.CHEMBIOpool[i][j];
   Shift1Profile(SoilProps,nr);
   for j:=1 to MaxLayers do DataYearOut.CHEMBIOpool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataYearOut.CHEMHUMpool[i][j];
   Shift1Profile(SoilProps,nr);
   for j:=1 to MaxLayers do DataYearOut.CHEMHUMpool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataYearOut.CHEMIOMpool[i][j];
   Shift1Profile(SoilProps,nr);
   for j:=1 to MaxLayers do DataYearOut.CHEMIOMpool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataYearOut.CHEMCO2pool[i][j];
   Shift1Profile(SoilProps,nr);
   for j:=1 to MaxLayers do DataYearOut.CHEMCO2pool[i][j]:=SoilProps[j];
    for j:=1 to MaxLayers do SoilProps[j]:=DataInit.CHEMDPMpool[i][j];
   Shift1Profile(SoilProps,nr);
   for j:=1 to MaxLayers do DataInit.CHEMDPMpool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataInit.CHEMRPMpool[i][j];
   Shift1Profile(SoilProps,nr);
   for j:=1 to MaxLayers do DataInit.CHEMRPMpool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataInit.CHEMBIOpool[i][j];
   Shift1Profile(SoilProps,nr);
   for j:=1 to MaxLayers do DataInit.CHEMBIOpool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataInit.CHEMHUMpool[i][j];
   Shift1Profile(SoilProps,nr);
   for j:=1 to MaxLayers do DataInit.CHEMHUMpool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataInit.CHEMIOMpool[i][j];
   Shift1Profile(SoilProps,nr);
   for j:=1 to MaxLayers do DataInit.CHEMIOMpool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataInit.CHEMCO2pool[i][j];
   Shift1Profile(SoilProps,nr);
   for j:=1 to MaxLayers do DataInit.CHEMCO2pool[i][j]:=SoilProps[j];
 end;
 {ectorganic layer}
 if nr>1 then {only if more than 1 compartment is added it is considered burial of ectorganic material}
 begin
   DataYearIn.RothC.DPMpool[nr]:=DataYearIn.RothC.DPMpool[nr]+DataYearIn.RothC.ectoDPMpool;
   DataYearOut.RothC.DPMpool[nr]:=DataYearOut.RothC.DPMpool[nr]+DataYearOut.RothC.ectoDPMpool;
   DataYearIn.RothC.RPMpool[nr]:=DataYearIn.RothC.RPMpool[nr]+DataYearIn.RothC.ectoRPMpool;
   DataYearOut.RothC.RPMpool[nr]:=DataYearOut.RothC.RPMpool[nr]+DataYearOut.RothC.ectoRPMpool;
   DataYearIn.RothC.BIOpool[nr]:=DataYearIn.RothC.BIOpool[nr]+DataYearIn.RothC.ectoBIOpool;
   DataYearOut.RothC.BIOpool[nr]:=DataYearOut.RothC.BIOpool[nr]+DataYearOut.RothC.ectoBIOpool;
   DataYearIn.RothC.HUMpool[nr]:=DataYearIn.RothC.HUMpool[nr]+DataYearIn.RothC.ectoHUMpool;
   DataYearOut.RothC.HUMpool[nr]:=DataYearOut.RothC.HUMpool[nr]+DataYearOut.RothC.ectoHUMpool;
   DataYearIn.RothC.IOMpool[nr]:=DataYearIn.RothC.IOMpool[nr]+DataYearIn.RothC.ectoIOMpool;
   DataYearOut.RothC.IOMpool[nr]:=DataYearOut.RothC.IOMpool[nr]+DataYearOut.RothC.ectoIOMpool;
   DataYearIn.RothC.CO2pool[nr]:=DataYearIn.RothC.CO2pool[nr]+DataYearIn.RothC.ectoCO2pool;
   DataYearOut.RothC.CO2pool[nr]:=DataYearOut.RothC.CO2pool[nr]+DataYearOut.RothC.ectoCO2pool;
   DataYearIn.RothC.DPMXpool[nr]:=DataYearIn.RothC.DPMXpool[nr]+DataYearIn.RothC.ectoDPMXpool;
   DataYearOut.RothC.DPMXpool[nr]:=DataYearOut.RothC.DPMXpool[nr]+DataYearOut.RothC.ectoDPMXpool;
   DataYearIn.RothC.RPMXpool[nr]:=DataYearIn.RothC.RPMXpool[nr]+DataYearIn.RothC.ectoRPMXpool;
   DataYearOut.RothC.RPMXpool[nr]:=DataYearOut.RothC.RPMXpool[nr]+DataYearOut.RothC.ectoRPMXpool;
   DataYearIn.RothC.BIOXpool[nr]:=DataYearIn.RothC.BIOXpool[nr]+DataYearIn.RothC.ectoBIOXpool;
   DataYearOut.RothC.BIOXpool[nr]:=DataYearOut.RothC.BIOXpool[nr]+DataYearOut.RothC.ectoBIOXpool;
   DataYearIn.RothC.HUMXpool[nr]:=DataYearIn.RothC.HUMXpool[nr]+DataYearIn.RothC.ectoHUMXpool;
   DataYearOut.RothC.HUMXpool[nr]:=DataYearOut.RothC.HUMXpool[nr]+DataYearOut.RothC.ectoHUMXpool;
   DataYearIn.RothC.IOMXpool[nr]:=DataYearIn.RothC.IOMXpool[nr]+DataYearIn.RothC.ectoIOMXpool;
   DataYearOut.RothC.IOMXpool[nr]:=DataYearOut.RothC.IOMXpool[nr]+DataYearOut.RothC.ectoIOMXpool;
   DataInit.RothC.DPMpool[nr]:=DataInit.RothC.DPMpool[nr]+DataInit.RothC.ectoDPMpool;
   DataInit.RothC.RPMpool[nr]:=DataInit.RothC.RPMpool[nr]+DataInit.RothC.ectoRPMpool;
   DataInit.RothC.BIOpool[nr]:=DataInit.RothC.BIOpool[nr]+DataInit.RothC.ectoBIOpool;
   DataInit.RothC.HUMpool[nr]:=DataInit.RothC.HUMpool[nr]+DataInit.RothC.ectoHUMpool;
   DataInit.RothC.IOMpool[nr]:=DataInit.RothC.IOMpool[nr]+DataInit.RothC.ectoIOMpool;
   DataInit.RothC.CO2pool[nr]:=DataInit.RothC.CO2pool[nr]+DataInit.RothC.ectoCO2pool;
   DataInit.RothC.DPMXpool[nr]:=DataInit.RothC.DPMXpool[nr]+DataInit.RothC.ectoDPMXpool;
   DataInit.RothC.RPMXpool[nr]:=DataInit.RothC.RPMXpool[nr]+DataInit.RothC.ectoRPMXpool;
   DataInit.RothC.BIOXpool[nr]:=DataInit.RothC.BIOXpool[nr]+DataInit.RothC.ectoBIOXpool;
   DataInit.RothC.HUMXpool[nr]:=DataInit.RothC.HUMXpool[nr]+DataInit.RothC.ectoHUMXpool;
   DataInit.RothC.IOMXpool[nr]:=DataInit.RothC.IOMXpool[nr]+DataInit.RothC.ectoIOMXpool;
   for i:=1 to MaxChem do
   begin
    DataInit.CHEMDPMpool[i][nr]:=DataInit.CHEMDPMpool[i][nr]+DataInit.CHEMDPMpool[i][0];
    DataInit.CHEMRPMpool[i][nr]:=DataInit.CHEMRPMpool[i][nr]+DataInit.CHEMRPMpool[i][0];
    DataInit.CHEMBIOpool[i][nr]:=DataInit.CHEMBIOpool[i][nr]+DataInit.CHEMBIOpool[i][0];
    DataInit.CHEMHUMpool[i][nr]:=DataInit.CHEMHUMpool[i][nr]+DataInit.CHEMHUMpool[i][0];
    DataInit.CHEMIOMpool[i][nr]:=DataInit.CHEMIOMpool[i][nr]+DataInit.CHEMIOMpool[i][0];
    DataInit.CHEMCO2pool[i][nr]:=DataInit.CHEMCO2pool[i][nr]+DataInit.CHEMCO2pool[i][0];
    DataYearIn.CHEMDPMpool[i][nr]:=DataYearIn.CHEMDPMpool[i][nr]+DataYearIn.CHEMDPMpool[i][0];
    DataYearIn.CHEMRPMpool[i][nr]:=DataYearIn.CHEMRPMpool[i][nr]+DataYearIn.CHEMRPMpool[i][0];
    DataYearIn.CHEMBIOpool[i][nr]:=DataYearIn.CHEMBIOpool[i][nr]+DataYearIn.CHEMBIOpool[i][0];
    DataYearIn.CHEMHUMpool[i][nr]:=DataYearIn.CHEMHUMpool[i][nr]+DataYearIn.CHEMHUMpool[i][0];
    DataYearIn.CHEMIOMpool[i][nr]:=DataYearIn.CHEMIOMpool[i][nr]+DataYearIn.CHEMIOMpool[i][0];
    DataYearIn.CHEMCO2pool[i][nr]:=DataYearIn.CHEMCO2pool[i][nr]+DataYearIn.CHEMCO2pool[i][0];
    DataYearOut.CHEMDPMpool[i][nr]:=DataYearOut.CHEMDPMpool[i][nr]+DataYearOut.CHEMDPMpool[i][0];
    DataYearOut.CHEMRPMpool[i][nr]:=DataYearOut.CHEMRPMpool[i][nr]+DataYearOut.CHEMRPMpool[i][0];
    DataYearOut.CHEMBIOpool[i][nr]:=DataYearOut.CHEMBIOpool[i][nr]+DataYearOut.CHEMBIOpool[i][0];
    DataYearOut.CHEMHUMpool[i][nr]:=DataYearOut.CHEMHUMpool[i][nr]+DataYearOut.CHEMHUMpool[i][0];
    DataYearOut.CHEMIOMpool[i][nr]:=DataYearOut.CHEMIOMpool[i][nr]+DataYearOut.CHEMIOMpool[i][0];
    DataYearOut.CHEMCO2pool[i][nr]:=DataYearOut.CHEMCO2pool[i][nr]+DataYearOut.CHEMCO2pool[i][0];
   end;
 end;
 {both after adding new compartments and after erosion, the ectorganic layer is gone > thus set to 0}
 if nr>1 then {only if more than 1 compartment is added it is considered burial of ectorganic material}
 begin
 DataInit.RothC.ectoDPMpool:=0; DataInit.RothC.ectoRPMpool:=0; DataInit.RothC.ectoBIOpool:=0;
 DataInit.RothC.ectoHUMpool:=0; DataInit.RothC.ectoIOMpool:=0; DataInit.RothC.ectoCO2pool:=0;
 DataInit.RothC.ectoDPMXpool:=0; DataInit.RothC.ectoRPMXpool:=0; DataInit.RothC.ectoBIOXpool:=0;
 DataInit.RothC.ectoHUMXpool:=0; DataInit.RothC.ectoIOMXpool:=0;
 DataYearIn.RothC.ectoDPMpool:=0; DataYearIn.RothC.ectoRPMpool:=0; DataYearIn.RothC.ectoBIOpool:=0;
 DataYearIn.RothC.ectoHUMpool:=0; DataYearIn.RothC.ectoIOMpool:=0; DataYearIn.RothC.ectoCO2pool:=0;
 DataYearOut.RothC.ectoDPMpool:=0; DataYearOut.RothC.ectoRPMpool:=0; DataYearOut.RothC.ectoBIOpool:=0;
 DataYearOut.RothC.ectoHUMpool:=0; DataYearOut.RothC.ectoIOMpool:=0; DataYearOut.RothC.ectoCO2pool:=0;
 DataYearIn.RothC.ectoDPMXpool:=0; DataYearIn.RothC.ectoRPMXpool:=0; DataYearIn.RothC.ectoBIOXpool:=0;
 DataYearIn.RothC.ectoHUMXpool:=0; DataYearIn.RothC.ectoIOMXpool:=0;
 DataYearOut.RothC.ectoDPMXpool:=0; DataYearOut.RothC.ectoRPMXpool:=0; DataYearOut.RothC.ectoBIOXpool:=0;
 DataYearOut.RothC.ectoHUMXpool:=0; DataYearOut.RothC.ectoIOMXpool:=0;
 for i:=1 to MaxChem do
 begin
   DataInit.CHEMDPMpool[i][0]:=0; DataInit.CHEMRPMpool[i][0]:=0;
   DataInit.CHEMBIOpool[i][0]:=0; DataInit.CHEMHUMpool[i][0]:=0;
   DataInit.CHEMIOMpool[i][0]:=0; DataInit.CHEMCO2pool[i][0]:=0;
   DataYearIn.CHEMDPMpool[i][0]:=0; DataYearIn.CHEMRPMpool[i][0]:=0;
   DataYearIn.CHEMBIOpool[i][0]:=0; DataYearIn.CHEMHUMpool[i][0]:=0;
   DataYearIn.CHEMIOMpool[i][0]:=0; DataYearIn.CHEMCO2pool[i][0]:=0;
   DataYearOut.CHEMDPMpool[i][0]:=0; DataYearOut.CHEMRPMpool[i][0]:=0;
   DataYearOut.CHEMBIOpool[i][0]:=0; DataYearOut.CHEMHUMpool[i][0]:=0;
   DataYearOut.CHEMIOMpool[i][0]:=0; DataYearOut.CHEMCO2pool[i][0]:=0;
 end;
 end;
 {weathering Record as part of datayearIn and dataYearOut records}
 Shift1Profile(DataInit.WE.Gibbsitepool,nr);
 Shift1Profile(DataInit.WE.WE_Capool,nr);
 Shift1Profile(DataInit.WE.WE_Mgpool,nr);
 Shift1Profile(DataInit.WE.WE_Kpool,nr);
 Shift1Profile(DataInit.WE.WE_Napool,nr);
 Shift1Profile(DataInit.WE.WE_Sipool,nr);
 Shift1Profile(DataInit.WE.WE_Fepool,nr);
 Shift1Profile(DataInit.WE.WE_Alpool,nr);
 Shift1Profile(DataInit.WE.WE_FePedogenic,nr);
 Shift1Profile(DataInit.WE.WE_SiPedogenic,nr);
 Shift1Profile(DataYearIn.WE.Gibbsitepool,nr);     Shift1Profile(DataYearOut.WE.Gibbsitepool,nr);
 Shift1Profile(DataYearIn.WE.WE_Capool,nr);        Shift1Profile(DataYearOut.WE.WE_Capool,nr);
 Shift1Profile(DataYearIn.WE.WE_Mgpool,nr);        Shift1Profile(DataYearOut.WE.WE_Mgpool,nr);
 Shift1Profile(DataYearIn.WE.WE_Kpool,nr);         Shift1Profile(DataYearOut.WE.WE_Kpool,nr);
 Shift1Profile(DataYearIn.WE.WE_Napool,nr);        Shift1Profile(DataYearOut.WE.WE_Napool,nr);
 Shift1Profile(DataYearIn.WE.WE_Sipool,nr);        Shift1Profile(DataYearOut.WE.WE_Sipool,nr);
 Shift1Profile(DataYearIn.WE.WE_Fepool,nr);        Shift1Profile(DataYearOut.WE.WE_Fepool,nr);
 Shift1Profile(DataYearIn.WE.WE_Alpool,nr);        Shift1Profile(DataYearOut.WE.WE_Alpool,nr);
 Shift1Profile(DataYearIn.WE.WE_FePedogenic,nr);   Shift1Profile(DataYearOut.WE.WE_FePedogenic,nr);
 Shift1Profile(DataYearIn.WE.WE_SiPedogenic,nr);   Shift1Profile(DataYearOut.WE.WE_SiPedogenic,nr);
 for j:=1 to nrminerals do
 begin
   Shift1Profile(DataInit.WE.MinPools[j],nr); Shift1Profile(DataYearIn.WE.MinPools[j],nr);    Shift1Profile(DataYearOut.WE.MinPools[j],nr);
 end;

 {data record}
  {DataInit}
 begin
    Shift1Profile(DataInit.STONES,nr); Shift1Profile(DataInit.GRAVEL,nr);      Shift1Profile(DataInit.Freezecounter,nr);
    Shift1Profile(DataInit.Rootcounter,nr); Shift1Profile(DataInit.stoneperc,nr);
    Shift1Profile(DataInit.CLAY,nr);   Shift1Profile(DataInit.SILT,nr);        Shift1Profile(DataInit.OC,nr);
    Shift1Profile(DataInit.SAND,nr);   Shift1Profile(DataInit.SANDm,nr);       Shift1Profile(DataInit.fineSANDm,nr);
    Shift1Profile(DataInit.CLAYm,nr);  Shift1Profile(DataInit.SILTm,nr);       Shift1Profile(DataInit.OCm,nr);
    Shift1Profile(DataInit.STONESm,nr); Shift1Profile(DataInit.GRAVELm,nr);
    Shift1Profile(DataInit.AW,nr);     Shift1Profile(DataInit.AH,nr);          Shift1Profile(DataInit.RDF,nr);
    Shift1Profile(DataInit.ATEM,nr);   Shift1Profile(DataInit.TEM,nr);
    Shift1Profile(DataInit.AAEV,nr);   Shift1Profile(DataInit.ABCAM,nr);
    Shift1Profile(DataInit.RHO,nr);    Shift1Profile(DataInit.CMATCH,nr);      Shift1Profile(DataInit.HMATCH,nr);
    Shift1Profile(DataInit.RP,nr);     Shift1Profile(DataInit.LAMBDA,nr);      Shift1Profile(DataInit.APH,nr);
    Shift1Profile(DataInit.W,nr);      Shift1Profile(DataInit.fc,nr);          Shift1Profile(DataInit.d,nr);
    Shift1Profile(DataInit.CHARGE,nr);      Shift1Profile(DataInit.SC1,nr);
    Shift1Profile(DataInit.SC2,nr);    Shift1Profile(DataInit.SC3,nr);         Shift1Profile(DataInit.SC4,nr);
    Shift1Profile(DataInit.SC5,nr);    Shift1Profile(DataInit.SC6,nr);         Shift1Profile(DataInit.SC7,nr);
    Shift1Profile(DataInit.SC8,nr);    Shift1Profile(DataInit.SC9,nr);         Shift1Profile(DataInit.SC10,nr);
    Shift1Profile(DataInit.SC11,nr);   Shift1Profile(DataInit.SC12,nr);        Shift1Profile(DataInit.SC13,nr);
    Shift1Profile(DataInit.SC14,nr);   Shift1Profile(DataInit.SC15,nr);
    Shift1Profile(DataInit.APCO2,nr);  Shift1Profile(DataInit.oldCALCITEA,nr); Shift1Profile(DataInit.CALCITEA,nr);
    Shift1Profile(DataInit.oldGYPA,nr);Shift1Profile(DataInit.GYPA,nr);
    Shift1Profile(DataInit.ACEC,nr);        Shift1Profile(DataInit.alkalinity,nr);
    Shift1Profile(DataInit.CEC_empirical,nr);
    Shift1Profile(DataInit.FacCEC,nr);
    Shift1Profile(DataInit.FacCEC_OC,nr);
    Shift1Profile(DataInit.ACEC_clay,nr);
    Shift1Profile(DataInit.ACEC_OC,nr);
    Shift1Profile(DataInit.ClayFrCEC,nr);
    Shift1Profile(DataInit.OCFrCEC,nr);
    Shift1Profile(DataInit.BS,nr);     Shift1Profile(DataInit.CO3,nr);         Shift1Profile(DataInit.HCO3,nr);
    Shift1Profile(DataInit.EC,nr);     Shift1Profile(DataInit.SAR,nr);         Shift1Profile(DataInit.ESP,nr);
    Shift1Profile(DataInit.ET,nr);     Shift1Profile(DataInit.PH,nr);          Shift1Profile(DataInit.FLUX,nr);
    Shift1Profile(DataInit.DISSOLVEDCATEQ,nr);  Shift1Profile(DataInit.DELGIBB,nr);  Shift1Profile(DataInit.DELGYP,nr);
    Shift1Profile(DataInit.DECALCITE,nr);  Shift1Profile(DataInit.OSPOT,nr);   Shift1Profile(DataInit.delco2,nr);
    Shift1Profile(DataInit.parden,nr); Shift1Profile(DataInit.porosity,nr);    Shift1Profile(DataInit.ClayDispInd,nr);
    Shift1Profile(DataInit.TEMPGEM,nr);     Shift1Profile(DataInit.klei2,nr);
    Shift1Profile(DataInit.silt2,nr);  Shift1Profile(DataInit.zand2,nr);       Shift1Profile(DataInit.volpercclay,nr);
    Shift1Profile(DataInit.DOC,nr);    Shift1Profile(DataInit.DOCperc,nr);
    Shift1Profile(DataInit.SUM21,nr);  Shift1Profile(DataInit.MM21,nr);
    for i:=1 to MaxChem do
    begin
      Shift1Profile(DataInit.SEA[i],nr); Shift1Profile(DataInit.AXSE[i],nr);
      Shift1Profile(DataInit.AXSE_oc[i],nr);Shift1Profile(DataInit.AXSE_clay[i],nr);
    end;
 end;
 {DataYearIn}
 begin
    Shift1Profile(DataYearIn.STONES,nr); Shift1Profile(DataYearIn.GRAVEL,nr);      Shift1Profile(DataYearIn.Freezecounter,nr);
    Shift1Profile(DataYearIn.Rootcounter,nr); Shift1Profile(DataYearIn.stoneperc,nr);
    Shift1Profile(DataYearIn.CLAY,nr);   Shift1Profile(DataYearIn.SILT,nr);        Shift1Profile(DataYearIn.OC,nr);
    Shift1Profile(DataYearIn.SAND,nr);   Shift1Profile(DataYearIn.SANDm,nr);       Shift1Profile(DataYearIn.fineSANDm,nr);
    Shift1Profile(DataYearIn.CLAYm,nr);  Shift1Profile(DataYearIn.SILTm,nr);       Shift1Profile(DataYearIn.OCm,nr);
    Shift1Profile(DataYearIn.STONESm,nr); Shift1Profile(DataYearIn.GRAVELm,nr);
    Shift1Profile(DataYearIn.AW,nr);     Shift1Profile(DataYearIn.AH,nr);          Shift1Profile(DataYearIn.RDF,nr);
    Shift1Profile(DataYearIn.ATEM,nr);   Shift1Profile(DataYearIn.TEM,nr);
    Shift1Profile(DataYearIn.AAEV,nr);   Shift1Profile(DataYearIn.ABCAM,nr);
    Shift1Profile(DataYearIn.RHO,nr);    Shift1Profile(DataYearIn.CMATCH,nr);      Shift1Profile(DataYearIn.HMATCH,nr);
    Shift1Profile(DataYearIn.RP,nr);     Shift1Profile(DataYearIn.LAMBDA,nr);      Shift1Profile(DataYearIn.APH,nr);
    Shift1Profile(DataYearIn.W,nr);      Shift1Profile(DataYearIn.fc,nr);          Shift1Profile(DataYearIn.d,nr);
    Shift1Profile(DataYearIn.CHARGE,nr);      Shift1Profile(DataYearIn.SC1,nr);
    Shift1Profile(DataYearIn.SC2,nr);    Shift1Profile(DataYearIn.SC3,nr);         Shift1Profile(DataYearIn.SC4,nr);
    Shift1Profile(DataYearIn.SC5,nr);    Shift1Profile(DataYearIn.SC6,nr);         Shift1Profile(DataYearIn.SC7,nr);
    Shift1Profile(DataYearIn.SC8,nr);    Shift1Profile(DataYearIn.SC9,nr);         Shift1Profile(DataYearIn.SC10,nr);
    Shift1Profile(DataYearIn.SC11,nr);   Shift1Profile(DataYearIn.SC12,nr);        Shift1Profile(DataYearIn.SC13,nr);
    Shift1Profile(DataYearIn.SC14,nr);   Shift1Profile(DataYearIn.SC15,nr);
    Shift1Profile(DataYearIn.APCO2,nr);  Shift1Profile(DataYearIn.oldCALCITEA,nr); Shift1Profile(DataYearIn.CALCITEA,nr);
    Shift1Profile(DataYearIn.oldGYPA,nr);Shift1Profile(DataYearIn.GYPA,nr);
    Shift1Profile(DataYearIn.ACEC,nr);        Shift1Profile(DataYearIn.alkalinity,nr);
    Shift1Profile(DataYearIn.CEC_empirical,nr);
    Shift1Profile(DataYearIn.FacCEC,nr);
    Shift1Profile(DataYearIn.FacCEC_OC,nr);
    Shift1Profile(DataYearIn.ACEC_clay,nr);
    Shift1Profile(DataYearIn.ACEC_OC,nr);
    Shift1Profile(DataYearIn.ClayFrCEC,nr);
    Shift1Profile(DataYearIn.OCFrCEC,nr);
    Shift1Profile(DataYearIn.BS,nr);     Shift1Profile(DataYearIn.CO3,nr);         Shift1Profile(DataYearIn.HCO3,nr);
    Shift1Profile(DataYearIn.EC,nr);     Shift1Profile(DataYearIn.SAR,nr);         Shift1Profile(DataYearIn.ESP,nr);
    Shift1Profile(DataYearIn.ET,nr);     Shift1Profile(DataYearIn.PH,nr);          Shift1Profile(DataYearIn.FLUX,nr);
    Shift1Profile(DataYearIn.DISSOLVEDCATEQ,nr);  Shift1Profile(DataYearIn.DELGIBB,nr);  Shift1Profile(DataYearIn.DELGYP,nr);
    Shift1Profile(DataYearIn.DECALCITE,nr);  Shift1Profile(DataYearIn.OSPOT,nr);   Shift1Profile(DataYearIn.delco2,nr);
    Shift1Profile(DataYearIn.parden,nr); Shift1Profile(DataYearIn.porosity,nr);    Shift1Profile(DataYearIn.ClayDispInd,nr);
    Shift1Profile(DataYearIn.TEMPGEM,nr);     Shift1Profile(DataYearIn.klei2,nr);
    Shift1Profile(DataYearIn.silt2,nr);  Shift1Profile(DataYearIn.zand2,nr);       Shift1Profile(DataYearIn.volpercclay,nr);
    Shift1Profile(DataYearIn.DOC,nr);    Shift1Profile(DataYearIn.DOCperc,nr);
    Shift1Profile(DataYearIn.SUM21,nr);  Shift1Profile(DataYearIn.MM21,nr);
    for i:=1 to MaxChem do
    begin
      Shift1Profile(DataYearIn.SEA[i],nr); Shift1Profile(DataYearIn.AXSE[i],nr);
      Shift1Profile(DataYearIn.AXSE_oc[i],nr);Shift1Profile(DataYearIn.AXSE_clay[i],nr);
    end;
 end;
 {DataYearOut}
 begin
    Shift1Profile(DataYearOut.STONES,nr); Shift1Profile(DataYearOut.GRAVEL,nr);      Shift1Profile(DataYearOut.Freezecounter,nr);
    Shift1Profile(DataYearIn.Rootcounter,nr); Shift1Profile(DataYearOut.stoneperc,nr);
    Shift1Profile(DataYearOut.CLAY,nr);   Shift1Profile(DataYearOut.SILT,nr);        Shift1Profile(DataYearOut.OC,nr);
    Shift1Profile(DataYearOut.SAND,nr);   Shift1Profile(DataYearOut.SANDm,nr);       Shift1Profile(DataYearOut.fineSANDm,nr);
    Shift1Profile(DataYearOut.CLAYm,nr);  Shift1Profile(DataYearOut.SILTm,nr);       Shift1Profile(DataYearOut.OCm,nr); 
    Shift1Profile(DataYearOut.STONESm,nr); Shift1Profile(DataYearOut.GRAVELm,nr);
    Shift1Profile(DataYearOut.AW,nr);     Shift1Profile(DataYearOut.AH,nr);          Shift1Profile(DataYearOut.RDF,nr);
    Shift1Profile(DataYearOut.ATEM,nr);   Shift1Profile(DataYearOut.TEM,nr);
    Shift1Profile(DataYearOut.AAEV,nr);   Shift1Profile(DataYearOut.ABCAM,nr);
    Shift1Profile(DataYearOut.RHO,nr);    Shift1Profile(DataYearOut.CMATCH,nr);      Shift1Profile(DataYearOut.HMATCH,nr);
    Shift1Profile(DataYearOut.RP,nr);     Shift1Profile(DataYearOut.LAMBDA,nr);      Shift1Profile(DataYearOut.APH,nr);
    Shift1Profile(DataYearOut.W,nr);      Shift1Profile(DataYearOut.fc,nr);          Shift1Profile(DataYearOut.d,nr);
    Shift1Profile(DataYearOut.CHARGE,nr);      Shift1Profile(DataYearOut.SC1,nr);
    Shift1Profile(DataYearOut.SC2,nr);    Shift1Profile(DataYearOut.SC3,nr);         Shift1Profile(DataYearOut.SC4,nr);
    Shift1Profile(DataYearOut.SC5,nr);    Shift1Profile(DataYearOut.SC6,nr);         Shift1Profile(DataYearOut.SC7,nr);
    Shift1Profile(DataYearOut.SC8,nr);    Shift1Profile(DataYearOut.SC9,nr);         Shift1Profile(DataYearOut.SC10,nr);
    Shift1Profile(DataYearOut.SC11,nr);   Shift1Profile(DataYearOut.SC12,nr);        Shift1Profile(DataYearOut.SC13,nr);
    Shift1Profile(DataYearOut.SC14,nr);   Shift1Profile(DataYearOut.SC15,nr);
    Shift1Profile(DataYearOut.APCO2,nr);  Shift1Profile(DataYearOut.oldCALCITEA,nr); Shift1Profile(DataYearOut.CALCITEA,nr);
    Shift1Profile(DataYearOut.oldGYPA,nr);Shift1Profile(DataYearOut.GYPA,nr);
    Shift1Profile(DataYearOut.ACEC,nr);        Shift1Profile(DataYearOut.alkalinity,nr);
    Shift1Profile(DataYearOut.CEC_empirical,nr);
    Shift1Profile(DataYearOut.FacCEC,nr);
    Shift1Profile(DataYearOut.FacCEC_OC,nr);
    Shift1Profile(DataYearOut.ACEC_clay,nr);
    Shift1Profile(DataYearOut.ACEC_OC,nr);
    Shift1Profile(DataYearOut.ClayFrCEC,nr);
    Shift1Profile(DataYearOut.OCFrCEC,nr);
    Shift1Profile(DataYearOut.BS,nr);     Shift1Profile(DataYearOut.CO3,nr);         Shift1Profile(DataYearOut.HCO3,nr);
    Shift1Profile(DataYearOut.EC,nr);     Shift1Profile(DataYearOut.SAR,nr);         Shift1Profile(DataYearOut.ESP,nr);
    Shift1Profile(DataYearOut.ET,nr);     Shift1Profile(DataYearOut.PH,nr);          Shift1Profile(DataYearOut.FLUX,nr);
    Shift1Profile(DataYearOut.DISSOLVEDCATEQ,nr);  Shift1Profile(DataYearOut.DELGIBB,nr);  Shift1Profile(DataYearOut.DELGYP,nr);
    Shift1Profile(DataYearOut.DECALCITE,nr);  Shift1Profile(DataYearOut.OSPOT,nr);   Shift1Profile(DataYearOut.delco2,nr);
    Shift1Profile(DataYearOut.parden,nr); Shift1Profile(DataYearOut.porosity,nr);    Shift1Profile(DataYearOut.ClayDispInd,nr);
    Shift1Profile(DataYearOut.TEM,nr);    Shift1Profile(DataYearOut.TEMPGEM,nr);     Shift1Profile(DataYearOut.klei2,nr);
    Shift1Profile(DataYearOut.silt2,nr);  Shift1Profile(DataYearOut.zand2,nr);       Shift1Profile(DataYearOut.volpercclay,nr);
    Shift1Profile(DataYearOut.DOC,nr);    Shift1Profile(DataYearOut.DOCperc,nr);
    Shift1Profile(DataYearOut.SUM21,nr);  Shift1Profile(DataYearOut.MM21,nr);
    for i:=1 to MaxChem do
    begin
      Shift1Profile(DataYearOut.SEA[i],nr); Shift1Profile(DataYearOut.AXSE[i],nr);
      Shift1Profile(DataYearOut.AXSE_oc[i],nr);Shift1Profile(DataYearOut.AXSE_clay[i],nr);
    end;
 end;
 {other profile variables: weathering indices}
 Shift1Profile(CIA,nr);  Shift1Profile(IndexB,nr);
 Shift1Profile(CIW,nr);  Shift1Profile(PIA,nr);
 Shift1Profile(WI,nr);  Shift1Profile(CPA,nr);  Shift1Profile(TRB,nr);
 Shift1Profile(al2o3init,nr);  Shift1Profile(caoinit,nr);
 Shift1Profile(na2oinit,nr);
 {other profile variables: 14C-activities}
 Shift1Profile(Fk_DPM,nr);  Shift1Profile(Fk_RPM,nr);
 Shift1Profile(Fk_BIO,nr);  Shift1Profile(Fk_HUM,nr);
 Shift1Profile(Fk_IOM,nr);
 {other profile variables: plant available water}
 Shift1Profile(AvailableWater,nr);

 if physweathallowed then
 Begin
 {Physical weathering: Shift1Profile cannot be used because not of type "profile"}
  oldmaxcomp:=DatayearIn.NoLayers; {Thus, NoLayers may not yet be adapted at this point!!!}
  if nr>0 then {layers added; shift all existing layers NR position downward in soil}
  begin
    for i:=-4 to 11 do
    begin
      for newcompno:=oldmaxcomp+nr downto nr+1 do PW_S[i][newcompno]:=PW_S[i][newcompno-nr];
      for newcompno:=oldmaxcomp+nr downto nr+1 do PW_K[i][newcompno]:=PW_K[i][newcompno-nr];
      for newcompno:=oldmaxcomp+nr downto nr+1 do PW_G[i][newcompno]:=PW_G[i][newcompno-nr];
      {new layer properties on top still to be added;
      first copy from former top layer then override with input data from Events.txt:
      1. Copy from former top layer}
      for newcompno:=1 to nr do PW_S[i][newcompno]:=PW_S[i][nr+1];
      for newcompno:=1 to nr do PW_K[i][newcompno]:=PW_K[i][nr+1];
      for newcompno:=1 to nr do PW_G[i][newcompno]:=PW_G[i][nr+1];
      {2. override from Events.txt: done in calling routine if applicable}
    end
  end
  else {layers removed; shift all existing layers NR positions upward in soil}
  begin
    for i:=-4 to 11 do
    begin
      for newcompno:=1 to oldmaxcomp+nr do PW_S[i][newcompno]:=PW_S[i][newcompno-nr];
      for newcompno:=1 to oldmaxcomp+nr do PW_K[i][newcompno]:=PW_K[i][newcompno-nr];
      for newcompno:=1 to oldmaxcomp+nr do PW_G[i][newcompno]:=PW_G[i][newcompno-nr];
    end;
  end;
 End;
 {finally adapt the NoLayers and the PDepth variables}   
 DataInit.NoLayers:=DataInit.NoLayers+nr;
 DataInit.PDEPTH:=DataInit.PDEPTH+nr*DataYearIn.DLZ;
 DataYearIn.NoLayers:=DataYearIn.NoLayers+nr;
 DataYearIn.PDEPTH:=DataYearIn.PDEPTH+nr*DataYearIn.DLZ;
 DataYearOut.NoLayers:=DataYearOut.NoLayers+nr;
 DataYearOut.PDEPTH:=DataYearOut.PDEPTH+nr*DataYearOut.DLZ;
 {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving ShiftCompartments');
end;

procedure TfrmMAIN.Shift1profile(var soilprop:Profile; nr:integer);
{shift 1 property of the profile upwards or downwards}
var newcompno,oldmaxcomp:integer;
begin
  oldmaxcomp:=DatayearIn.NoLayers; {Thus, NoLayers may not yet be adapted at this point!!!}
  if nr>0 then {layers added; shift all existing layers NR position downward in soil}
  begin
    for newcompno:=oldmaxcomp+nr downto nr+1 do Soilprop[newcompno]:=Soilprop[newcompno-nr];
    {new layer properties on top still to be added;
    first copy from former top layer then override with input data from Events.txt:
    1. Copy from former top layer}
    for newcompno:=1 to nr do Soilprop[newcompno]:=Soilprop[nr+1];
    {2. override from Events.txt: done in calling routine if applicable}
  end
  else {layers removed; shift all existing layers NR positions upward in soil}
  begin
    for newcompno:=1 to oldmaxcomp+nr do Soilprop[newcompno]:=Soilprop[newcompno-nr];
  end
end;

procedure TfrmMAIN.ReadCheckEvents;
{reads the events file "events.txt" and produces "EventsINT.txt" with annual values
 for various possible events, NO interpolation (no data=values of 0)}
var year,yeara,yearb,startBPYearSim:integer;
    ffert,ffert2:textfile;
    txta,txtb,txtevent: string;
    nocompa,nocompb,newcompartments:integer;
begin
   {If GUItrace=1 then wwwriteln} singletracewriteln('Entering ReadCheckEvents');
   MaxPlusCompartments:=DataInit.NoLayers; newcompartments:=DataInit.NoLayers;
   startBPYearSim:=spndtMultipleYears.Value;
   if (fileexists('Events.txt'))and(chckbxEvents.Checked) then
   begin
     system.Assign(ffert2,'EventsINT.txt'); system.Rewrite(ffert2);
     system.Assign(ffert,'Events.txt');  system.Reset(ffert);
     system.Readln(ffert); {skip header}
     if not(eof(ffert))
     then
     begin
      system.Read(ffert,yearb); leeswoord(ffert,txtevent); system.Readln(ffert,nocompb,txtb);
      if txtevent='Erosion' then newcompartments:=newcompartments-nocompb;
      if txtevent='Deposition' then newcompartments:=newcompartments+nocompb;
      if newcompartments>MaxPlusCompartments then MaxPlusCompartments:=newcompartments;
      {if first input year is not starting year (1) of the simulation:
       fill the gap with 0}
      if yearb>1 then
      begin
        for year:=1 to min(startBPYearSim,yearb-1) do
        system.writeln(ffert2,year,' none  0 NoData');
      end;
     end
     else system.writeln(ffert2,'1 none  0 NoData');
     if not (eof(ffert)) then
     repeat
       {set the *a variables equal to already read *b variables}
       yeara:=yearb;     nocompa:=nocompb;     txta:=txtb;
       {write the *a variables to file}
       if (yeara<=startBPYearSim)and(yeara>0)
       then system.writeln(ffert2,yeara,' ',txtevent,' ',nocompa,' ',txta);
       {read the new set of *.b variables}
       system.Read(ffert,yearb); leeswoord(ffert,txtevent); system.Readln(ffert,nocompb,txtb);
       if txtevent='Erosion' then newcompartments:=newcompartments-nocompb;
       if txtevent='Deposition' then newcompartments:=newcompartments+nocompb;
       if newcompartments>MaxPlusCompartments then MaxPlusCompartments:=newcompartments;
       {if there is more than 1 year between *a and *b: set to 0 (not fertilization)}
       if yearb-yeara>1 then
       begin
         for year:=min(startBPYearSim,yeara)+1
             to    min(startBPYearSim,yearb-1)
             do
         begin
           if (year>0)
           then system.writeln(ffert2,year,' none  0 NoData');
         end;
       end;
     until (eof(ffert));
     {write last year}
     if (yearb<=startBPYearSim)and(yearb>0)
     then system.writeln(ffert2,yearb,' ',txtevent,' ',nocompb,' ',txtb);
     {if last input year is not final simulation year:
      fill the gap (with 0)}
     if yearb<startBPYearSim then
     begin
       for year:=yearb+1 to startBPYearSim do
       if (year>0)
       then system.writeln(ffert2,year,' none  0 NoData');
     end;
     closefile(ffert);
     closefile(ffert2);
   end
   else begin if fileexists('EventsINT.txt') then deletefile('EventsINT.txt'); end;
   {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving ReadCheckEvents');
   {RockAddition}
   MaxPlusCompartments:=MaxPlusCompartments+Rock.RockBottomLayers;
end;

procedure TfrmMAIN.LeesWoord(var f:textfile; var woord:string);
{reads one word (string between spaces or at beginning/end of line) from a text file}
var a:char;
begin
  repeat read(f,a) until a<>' '; woord:=a;
  repeat read(f,a); if a<>' 'then woord:=woord+a until (a=' ')or(eoln(f));
end;

procedure TfrmMAIN.btbttnEventsChangeClick(Sender: TObject);
{opens the window to enter/edit the various possible "events"
 that can be invoked over the simulation time and saves to the correct format}
begin
  untEvents.frmEvents.ShowModal;
end;

procedure TfrmMAIN.CalcSlopeEffects(var slopewindfacP,slopewindfacPE:real);
{calculates the effect of slope, its exposition, wind direction and latitude
 on correction factors for P and PE}
var alfa,delta,gamma,beta,v1,v2,vwind,azimuth:real;
    tanalfa,tanbeta,coscombi,incrad0,incradalfa:real;
    feffe:textfile;
begin
  {If GUItrace=1 then wwwriteln} singletracewriteln('Entering CalcSlopeEffects');
  alfa:=DataInit.slope;   {input slope angle}
  delta:=DataInit.upSlopeBearing; {input bearing of upslope}
  azimuth:=delta+180; if azimuth>360 then azimuth:=azimuth-360;
  gamma:=DataInit.downWindBearing; {input bearing of downwind direction}
  v1:=5; {default raindrop velocity, m/s}
  vwind:=5; {default wind speed, m/s, Beaufort 3}
  v2:=vwind*cos((delta-gamma)*pi/180); {wind speed along slope, corrected for their bearings}
  beta:=arctan(v2/v1)*180/pi;     {compensation for impact angle raindrops}
  tanalfa:=tan(alfa*pi/180);
  tanbeta:=tan(abs(beta)*pi/180);
  coscombi:=cos((delta-gamma)*pi/180);
  slopewindfacP:=1+tanbeta*tanalfa*coscombi; {corrected Precipitation as a fraction}
  slopewindfacPE:=1/cos(alfa*pi/180);     {corrected EvapoTranspiration as a fraction because of area increase only}
  {calculate factor PE with algorithm of Swift, 1976 (WRR 12-1: 108-112), which calculates
   incoming radiation for any slope angle, aspect and latitude. The idea is to calc radiation
   for slope 0 and slope alfa with bearing delta, both for the same latitude, over 1 year,
   and take their ratio to obtain a correction factor for PE.
   This correction facor is however only applied to short vegetations (grasscrub and agriculture)
   because the effect of exposition on a forest floor is assumed to be minimal}
  incrad0:=IncRad(0,azimuth,DataInit.latitude);
  incradalfa:=IncRad(alfa,azimuth,DataInit.latitude);
  slopewindfacPE:=incradalfa/incrad0;
  {end routine from Swift, 1976}
  {if report of calc needed:}
  system.assign(feffe,'CalcSlopeEffects.txt'); system.rewrite(feffe);
  {system.writeln(feffe,'alfa=',alfa);
  system.writeln(feffe,'delta=',delta);
  system.writeln(feffe,'gamma=',gamma);
  system.writeln(feffe,'beta=', beta);
  system.writeln(feffe,'v1=',v1);
  system.writeln(feffe,'v2=',v2);
  system.writeln(feffe,'vwind=',vwind);
  system.writeln(feffe,'azimuth=',azimuth);
  system.writeln(feffe,'tanalfa=',tanalfa);
  system.writeln(feffe,'tanbeta=',tanbeta);
  system.writeln(feffe,'coscombi=',coscombi);
  system.writeln(feffe,'incrad0=',incrad0);
  system.writeln(feffe,'incradalfa=',incradalfa); }
  system.writeln(feffe,'slopewindfacP=',slopewindfacP);
  system.writeln(feffe,'slopewindfacPE=',slopewindfacPE);
  closefile(feffe);
  {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving CalcSlopeEffects');
end;

procedure TfrmMAIN.Showmemorydump1Click(Sender: TObject);
{(via pulldown menu): if a previous run stopped with a crash,
 this functionality depicts the status of model variables via a new window}
begin
  untMemDump.frmMemDump.ShowModal;
end;

procedure TfrmMAIN.rdgrpTraceClick(Sender: TObject);
{allows to trace which procedures/functions were visited during runtime,
 to roughly identify at what point a crash occurs}
begin
  lblTraceStart.Visible:=(rdgrpTrace.ItemIndex=1);
  spndtTraceStart.Visible:=(rdgrpTrace.ItemIndex=1);
  GUItrace:=rdgrpTrace.ItemIndex;
  if chemstarted then
  begin rdgrpTrace.Color:=clYellow; spndtTraceStart.Color:=clYellow end;
end;

procedure TfrmMAIN.chckbxPhysWeathClick(Sender: TObject);
{determines if physical weathering is to be simulated,
 always TRUE and the checkbox is not visible at present}
begin
  PhysWeathAllowed:=chckbxPhysWeath.Checked;
  ShowWindowHeader;
end;

procedure TfrmMAIN.singletracewriteln(txt:string);
{stores <txt> in a global variable <singletracetext>,
 where <txt> indicates the execution point of the program. For debugging purposes.}
begin
   singletracetext:=txt;
end;

procedure TfrmMAIN.SplitCEC;
{split up the CEC and exchangeable cations into clay and OC-contributions
 using the regression function in Foth and Ellis (1996)}
var i,j:integer;
    B0,B1,B2:real;
begin
 {The regression function in Foth and Ellis (1996) is used to split up the CEC:
 1: CEC_empirical = 32 + 36.7*OC + 1.96*CLAY (Foth+Ellis 1996) is calculated with OC and CLAY
 2: Compare with read ACEC (=CECsoil) and derive a correction factor FacCEC = ACEC/CEC_empirical
 3: Derive a correction factor FacCEC_OC = 36.7 * FacCEC, so that
 4: CECsoil = FacCEC_OC*OC + ClayCEC, so that ClayCEC = ACEC - FacCEC_OC*OC
 5: Calculate the fraction of ACEC in Clay: ClayFrCEC = ClayCEC/ACEC
 6: Calculate the fraction of ACEC in OC: OCFrCEC = FacCEC_OC*OC/ACEC
 7: next split up ACEC and AXSE[chem] profiles using these fractions and
    store result profiles in ACEC_clay+AXSE_clay[chem] and ACEC_OC+AXSE_OC[chem].
    Elsewhere in the program, the *_OC parts are adapted when OC changes,
    and *_clay parts are adapted as a side-effect of claymigration and physical weathering.
 }
 {If GUItrace=1 then wwwriteln} singletracewriteln('Entering SplitCEC');
 for i:=1 to DataInit.NoLayers do
 begin
   B0:=32; B1:=36.7; B2:=1.96; {Foth and Ellis 1996 values}
   B0:=37.83; B1:=14.25; B2:=3.82; {adapted values via local regression CECmeas with simOC and simClay}
   {1} DataInit.CEC_empirical[i]:=B0+B1*DataInit.OC[i]+B2*DataInit.clay[i];
   {2} DataInit.FacCEC[i]:=DataInit.ACEC[i]/DataInit.CEC_empirical[i];
   {3} DataInit.FacCEC_OC[i]:=B1*DataInit.FacCEC[i];  {FacCEC_OC contains adjusted regression coefficient}
   {4+7} DataInit.ACEC_clay[i]:=DataInit.ACEC[i]-DataInit.FacCEC_OC[i]*DataInit.OC[i];
         DataInit.ACEC_OC[i]:=DataInit.ACEC[i]-DataInit.ACEC_clay[i];
   {5} DataInit.ClayFrCEC[i]:=DataInit.ACEC_clay[i]/DataInit.ACEC[i];
   {6} DataInit.OCFrCEC[i]:=DataInit.ACEC_OC[i]/DataInit.ACEC[i];
   if DoSplitCec then
   begin
   {7} for j:=1 to MaxChem do
       begin
         DataInit.AXSE_oc[j,i]:=DataInit.AXSE[j,i]*DataInit.OCFrCEC[i];
         {_oc values will be adapted each year when the OC is adapted in RECALCOC;
          _clay values will be adapted in the CHEM.claymigration procedures and
                                       in the CHEM.physicalweathering procedure}
         DataInit.AXSE_clay[j,i]:=DataInit.AXSE[j,i]*DataInit.ClayFrCEC[i];
       end
   end
   {else
   begin
      for j:=1 to maxchem do
      begin
        DataInit.AXSE_oc[j,i]:=0; DataInit.AXSE_clay[j,i]:=DataInit.AXSE[j,i];
      end;
      DataInit.ACEC_clay[i]:=DataInit.ACEC[i];  DataInit.ACEC_oc[i]:=0;
   end;}
 end;
 {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving SplitCEC');
end;

procedure TfrmMAIN.AdjustCECAXSE2OC;
{adjust the OC-contributions of exchangeable cations from/to the soil solution.
 Note: for clay this is done via the clay migration routine}
{1: adjust AXSE_OC[chem] with a factor relating it to OC
    and remove or add the exchangeable cations
    from/to the soil to maintain mass balance.
    If AXSE increases, it is taken from the soil solution
      if there is a shortage, the AXSE-increase is limited to the available amount;
    if AXSE decreases, it is added to the soil solution.
 2: adjust ACEC_OC via the summation of AXSE_OC[chem]
 3. adjust AXSE[chem] and ACEC via the summation of _OC and _clay}
var i,ii,j,itemp:integer;
    tempaxse,newCEC_OC,deltaxse,neededmoles,availablemoles,newmoles,depletablemoles:real;
    VAL: array[1..maxchem] of integer;
begin
 if DoSplitCec then
 begin
  {If GUItrace=1 then wwwriteln} singletracewriteln('Entering AdjustCECAXSE2OC');
  {initialisation}
{   Valency of chemical species}
  for j:=1 to maxchem do val[j]:=1;
  VAL[1]:=2;   {Ca}
  VAL[2]:=2;   {Mg}
  VAL[6]:=2;   {SO4}
  VAL[8]:=2;   {CO3}
  VAL[12]:=3;  {Al}
  VAL[13]:=3;  {Org}
  VAL[14]:=2;  {HOrg}
  for i:=1 to dataYearOut.NoLayers do
  begin
     itemp:=min(40,max(-10,round(DataYearOut.tem[i])));
     for ii:=1 to maxK do dataYearOut.KValue[ii]:=DataInit.kvaluetable[ii,itemp];
     for j:=1 to maxchem do
     begin
  {1}  if (j in [1,2,3,4,10,12])
       then {chemical is part of CEC}
       begin
         newCEC_OC:=DataYearOut.FacCEC_OC[i]*dataYearOut.OC[i];
         {newCEC_OC=the new CEC_OC predicted by updated OC}
         tempaxse:=DataYearOut.AXSE_oc[j,i]*newCEC_OC/dataYearOut.ACEC_OC[i];
         {tempaxse=the new exchangeable chemical j assuming that it is linearly related to ACEC_OC }
         deltaxse:=tempaxse-DataYearOut.AXSE_oc[j,i];
         {change in exchangeable chemical j}
       end
       else {chemical is no part of CEC}
       begin
         tempaxse:=0; deltaxse:=0;
       end;
       if deltaxse<0      {change in exchangeable cations is negative: decrease exchange + add to soil solution}
       then
       begin
         DataYearOut.AXSE_oc[j,i]:=tempaxse;  {new exchange amount accepted}
         newmoles:=DataYearOut.SEA[j,i]*DataYearOut.AW[i]*DataYearOut.DLZ-
                  deltaxse*DataYearOut.RHO[i]*((100-DataYearOut.STONES[i]-DataYearOut.GRAVEL[i])/100)*DataYearOut.DLZ/VAL[j]; {total new moles in Volume 1 layer}
         DataYearOut.SEA[j,i]:=newmoles/(DataYearOut.AW[i]*DataYearOut.DLZ); {expressed as conc}
       end
       else
       if deltaxse>0
       then {change in exchangeable cations is positive: increase exchange + deplete  soil solution}
       begin
         neededmoles:=abs(deltaxse)*DataYearOut.RHO[i]*((100-DataYearOut.STONES[i]-DataYearOut.GRAVEL[i])/100)*DataYearOut.DLZ/VAL[j];
         availablemoles:=DataYearOut.SEA[j,i]*DataYearOut.AW[i]*DataYearOut.DLZ;
         {supply what's needed, but always leave minimally 5% of original moles in concentration
          to avoid numerical problems in chemical routines - they do not like 0-concentrations}
         if availablemoles<1.05*neededmoles
         then {soil solution cannot supply all because it contains less than 105% of what's needed;
               use what's depletable (=95%)}
         begin
           {if desired send an indicator to messages.txt:
           GlobalDepth:=i*round(DataYearIn.dlz); str(GlobalDepth,txt1); str(GlobalTime,txt2);
           debuglog('At Year '+txt2+' and depth '+txt1+' mm');
           debuglog('RecalcOC tells there was not enough in solution to satisfy needed extra exchange phase.');}
           depletablemoles:=min(0.95*availablemoles,neededmoles);
          {AXSE_oc=moles in "old"axse +0.95available moles; backconverted to mmolc/kg soil }
           DataYearOut.AXSE_oc[j,i]:=((DataYearOut.AXSE_oc[j,i]*DataYearOut.RHO[i]*((100-DataYearOut.STONES[i]-DataYearOut.GRAVEL[i])/100)*DataYearOut.DLZ/VAL[j])
                                      +depletablemoles)*
                                     (VAL[j]/(DataYearOut.RHO[i]*(1+(DataYearOut.STONES[i]+DataYearOut.GRAVEL[i])/100)*DataYearOut.DLZ));
           DataYearOut.SEA[j,i]:=(availablemoles-depletablemoles)/(DataYearOut.AW[i]*DataYearOut.DLZ); {expressed as conc}
         end
         else {soil solution can supply what's needed; use the needed amount}
         begin
          {AXSE_oc=moles in "old"axse +neededmoles; backconverted to mmolc/kg soil }
           DataYearOut.AXSE_oc[j,i]:=((DataYearOut.AXSE_oc[j,i]*DataYearOut.RHO[i]*((100-DataYearOut.STONES[i]-DataYearOut.GRAVEL[i])/100)*DataYearOut.DLZ/VAL[j])
                                      +neededmoles)*
                                     (VAL[j]/(DataYearOut.RHO[i]*((100-DataYearOut.STONES[i]-DataYearOut.GRAVEL[i])/100)*DataYearOut.DLZ));
           DataYearOut.SEA[j,i]:=(availablemoles-neededmoles)/(DataYearOut.AW[i]*DataYearOut.DLZ); {expressed as conc}
         end;
       end;
     end;
  {2}dataYearOut.ACEC_OC[i]:=max(1E-7,dataYearOut.AXSE_OC[1][i]+dataYearOut.AXSE_OC[2][i]+
                               dataYearOut.AXSE_OC[3][i]+dataYearOut.AXSE_OC[4][i]+
                               dataYearOut.AXSE_OC[10][i]+dataYearOut.AXSE_OC[12][i]);
  {3}for j:=1 to maxchem do
     begin
       dataYearOut.AXSE[j,i]:=dataYearOut.AXSE_OC[j,i]+dataYearOut.AXSE_clay[j,i];
     end;
     dataYearOut.ACEC[i]:=dataYearOut.AXSE[1][i]+dataYearOut.AXSE[2][i]+
                               dataYearOut.AXSE[3][i]+dataYearOut.AXSE[4][i]+
                               dataYearOut.AXSE[10][i]+dataYearOut.AXSE[12][i];
  end;
  {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving AdjustCECAXSE2OC');
 end;
end;

procedure TfrmMAIN.BatchMode;
{reading model inputs using the contents of "SoilGenBatch.txt" (only if this file exists)}
var fbatch:textfile;
    SoilFileName,EquilibriumFileName,CModelFileName,WeatheringFileName,
    location:string;
    NoSimulationYears: integer;
Begin
startscreen:=false; grpbxOnStart.visible:=false;

location:=AppendPathDelim(startdir);
system.assign(fbatch,location+'SoilGenBatch.txt');
system.reset(fbatch);

system.readln(fbatch,SoilFileName); SoilFileName:=location+SoilFileName;
{read soil file}
  {de-activate menu item to display dump.txt. This is because it will fill
   the data fields, which may override the read input}
   showmemorydump1.Visible:=false;
   {If GUItrace=1 then wwwriteln} singletracewriteln('Entering btbttnOpenDefaultLeachCClick');
   pndlgOPEN.Filter:= 'Initial Soil status input files (*.txt)|*.txt';
      mmDefaultLeachC.Lines.LoadFromFile(SoilFileName);
      mmDefaultLeachC.Enabled:=true;
      InitChem;
      ReadData(SoilFileName);
      FirstLeachCFileName:=SoilFileName;
      {DataYearIn:=DataInit;}  CopyRecToFrom(DataYearIn,DataInit);
      DataYearIn.oldCALCITEA:=DataYearIn.CALCITEA;
      DataYearIn.oldGYPA:=DataYearIn.GYPA;
      if fileexists('Messages.txt')and(warningsissued)
      then bttnErrorInputs.Visible:=true
      else bttnErrorInputs.Visible:=false;
      if ReadDataOK then
      begin
        btbttnOpenEquilibrium.Enabled:=true;
        btbttnEditSoilData.Enabled:=true;
      end;
  {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving btbttnOpenDefaultLeachCClick');

system.readln(fbatch,EquilibriumFileName);EquilibriumFileName:=location+EquilibriumFileName;
EQFileName:=EquilibriumFileName;
{read equilibrium file}
  {de-activate menu item to display dump.txt. This is because it will fill
   the data fields, which may override the read input}
   showmemorydump1.Visible:=false;
   {If GUItrace=1 then wwwriteln} singletracewriteln('Entering btbttnOpenEquilibriumClick');
   pndlgOPEN.Filter:= 'Equilibrium constant input files (*.txt)|*.txt';
      mmEquilibrium.Lines.LoadFromFile(EquilibriumFileName);
      mmEquilibrium.Enabled:=true;
      ReadDataEQ(EquilibriumFileName);
      if fileexists('Messages.txt')and(warningsissued)
      then bttnErrorInputs.Visible:=true
      else bttnErrorInputs.Visible:=false;
      if ReadDataEQOK
      then
      begin
        btbttnOpenDefaultCModel.Enabled:=true;
        btbttnEditEquilibria.Enabled:=true;
      end;
   {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving btbttnOpenEquilibriumClick');

system.readln(fbatch,CModelFileName);CModelFileName:=location+CModelFileName;
{read C-model inputfile}
  {de-activate menu item to display dump.txt. This is because it will fill
   the data fields, which may override the read input}
   showmemorydump1.Visible:=false;
   {If GUItrace=1 then wwwriteln} singletracewriteln('Entering btbttnOpenDefaultCModelClick');
   pndlgOPEN.Filter:= 'C-input files (*.txt)|*.txt';
      mmDefaultCModel.Lines.LoadFromFile(CModelFileName);
      mmDefaultCModel.Enabled:=true;
      ReadDataC(CModelFileName);
      FirstRothCFileName:=CModelFileName;
      {DataYearIn:=DataInit;}  CopyRecToFrom(DataYearIn,DataInit);
      if fileexists('Messages.txt')and(warningsissued)
      then bttnErrorInputs.Visible:=true
      else bttnErrorInputs.Visible:=false;
      if ReadDataCOK then
      begin
        btbttnOpenWeathering.Enabled:=true;
        btbttnEditCTurnover.Enabled:=true;
      end;
{If GUItrace=1 then wwwriteln} singletracewriteln('Leaving btbttnOpenDefaultCModelClick');

system.readln(fbatch,WeatheringFileName);WeatheringFileName:=location+WeatheringFileName;
{read weathering inputfile}
  {de-activate menu item to display dump.txt. This is because it will fill
   the data fields, which may override the read input}
   showmemorydump1.Visible:=false;
   {If GUItrace=1 then wwwriteln} singletracewriteln('Entering btbttnOpenWeatheringClick');
   pndlgOPEN.Filter:= 'Initial mineralogy input files (*.txt)|*.txt';
      mmWeatheringData.Lines.LoadFromFile(WeatheringFileName);
      mmWeatheringData.Enabled:=true;
      ReadDataWE(WeatheringFileName);
      ALLOWED:=readweatherOK;
      if readweatherOK then btbttnEditWeathering.Enabled:=true;
      FirstWeatherFileName:=WeatheringFileName;
      {DataYearIn:=DataInit; }CopyRecToFrom(DataYearIn,DataInit);
      if fileexists('Messages.txt')and(warningsissued)
      then bttnErrorInputs.Visible:=true
      else bttnErrorInputs.Visible:=false;
   {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving btbttnOpenWeatheringClick');

{soil production: hardrock data}
   {de-activate menu item to display dump.txt. This is because it will fill
    the data fields, which may override the read input}
    showmemorydump1.Visible:=false;    
    Rock.RockBottomLayers:=0;
    if fileexists('Bedrock.txt') then
    begin
       mmBedrock.Lines.LoadFromFile('Bedrock.txt');
       mmBedrock.Enabled:=true;
       ReadDataRock('Bedrock.txt');
       ALLOWED:=readbedrockOK;
       BedrockFileName:='Bedrock.txt'; 
       Rock.RockBottomLayers:=rbl;
       rdgrpChooseDisplay.columns:=5;
       {DataYearIn:=DataInit;}  CopyRecToFrom(DatayearIn,DataInit);
       if (not(bttnErrorInputs.Visible))and(fileexists('Messages.txt')and(warningsissued))
       then bttnErrorInputs.Visible:=true;
    end;

{in case of continuation run: store some variables that may override the
 values from continue.rec by the values from the soil and turnover input files}
KeepSelectedData;

tbcntrlSHELL.TabIndex:=1; tbcntrlSHELLChange(Self);{2nd tab sheet}
system.readln(fbatch,NoSimulationYears);
spndtMultipleYears.Value:=round(NoSimulationYears);

system.readln(fbatch,DataInit.slope);
spndtSlopeAngle.Value:=round(DataInit.slope);

system.readln(fbatch,DataInit.upslopebearing);
spndtSlopeBearing.Value:=round(DataInit.upslopebearing);

system.readln(fbatch,DataInit.downwindbearing);
spndtWindBearing.Value:=round(DataInit.downwindbearing);

system.readln(fbatch,DataInit.latitude);
spndtLatitude.Value:=round(DataInit.latitude);

lstbxScreenVariables.Clear;
lstbxScreenVariables.Items.Add(cmbbxScreenVariables.Items[28]);
lstbxScreenVariables.Items.Add(cmbbxScreenVariables.Items[1]);
lstbxScreenVariables.Items.Add(cmbbxScreenVariables.Items[3]);
lstbxScreenVariables.Items.Add(cmbbxScreenVariables.Items[31]);
lstbxScreenVariables.Items.Add(cmbbxScreenVariables.Items[27]);
lstbxScreenVariables.Items.Add(cmbbxScreenVariables.Items[21]);

{batchmodeYN:=false;}
closefile(fbatch);
{in case of a continuation}
 if fileexists('continue.rec')then
 showmessage('SoilGen found continuation files.'+#13#10+#13#10+
             'The new run will be started with final values of the preceding run.'+#13#10+
             'A summary of the status of variables will be reported in <ContinueReport.txt>.'+#13#10+#13#10+
             'You have to specify the following before pressing RUN:'+#13#10+
             '1. The number of simulation years'+#13#10+
             'Correct versions of the following files (not mandatory) '+#13#10+
             '2. <climate.txt>'+#13#10+
             '3. <BioturbINT.txt> (annual bioturbation values as mass fractions)'+#13#10+
             '4. <Fertilization.txt>'+#13#10+
             '5. <Events.txt>'+#13#10+
             'These files can also be imported via the user interface'+#13#10+
             '6. file with atmospheric pCO2-data.'+#13#10+
             'Please note that the starting year in the continuation run will again be 1 '+#13#10+
             'and that the above files must reflect this !');
End;

function TfrmMAIN.IncRad(inclination,aspect,latitude:real):real;
{algorithm from Swift, 1976 to calculate solar radiation on mountain slopes}
{This function is an implementation of Swift, 1976
 Swift, L. W., Jr., 1976. Algorithm for solar radiation on mountain slopes,
 Water Resour. Res., 12(1), 108-112, doi:10.1029/WR012i001p00108
 and was checked with his table 1 to provide the same results. It did.}
const arcconv=180/pi;
var J:integer;
    L1,L2,D1,Rsum,D,E,F,R0,R1,R2,R3,R4,R5,R6,T,T0,T1,T2,T3,T4,T5,T6,T7,T8,T9:real;
    R2365:array[1..365]of real;
function FUNC1(J:integer; W,X,Y:real):real;
begin
  FUNC1:=W-X*cos(((J+Y)*0.986)*pi/180)
end;
function FUNC2(D,Y:real):real;
var tmp:real;
begin
  tmp:=-tan(Y*pi/180)*tan(D*pi/180);
  if tmp<-1 then tmp:=-1; if tmp>1 then tmp:=1;
  FUNC2:=arcconv*arccos(tmp);
end;
function FUNC3(D,V,W,X,Y:real):real;
begin
  FUNC3:=R1*(sin(D*pi/180)*sin(W*pi/180)*(X-Y)/15+
             cos(D*pi/180)*cos(W*pi/180)*(sin((X+V)*pi/180)-sin((Y+V)*pi/180))*12/pi);
end;
function FUNC4(latitude:real;J:integer):real;
{This function was added by pf2011 and calculates the daily solar radiation in MJ/m2,day
 at an horizontal surface using latitude and Julian day number}
var solarrad,dr,omega,declination,latRAD,relsunshine:real;
begin
   dr:=1+0.033*COS(2*PI*J/365); {dr=inverse relative distance Earth-Sun}
   latRAD:=PI*latitude/180;    {latRAD=latitude in radians; pos=north}
   declination:=0.409*SIN(2*PI*J/365-1.39); {declination=solar declination in radians}
   omega:=ARCCOS(-TAN(latRAD)*TAN(declination)); {omega=solar time angle in radians}
   relsunshine:=0.5; {relsunshine=n/N = relative sunshine duration; considered a constant}
   solarrad:=(24*60/PI)*0.082*dr*
             (omega*SIN(latRAD)*SIN(declination)+COS(latRAD)*COS(declination)*SIN(omega))*
             (0.25+0.5*relsunshine);
   FUNC4:=solarrad;
end;
begin
  R0:=1.95; {value for solar constant}
  R2:=1; {dummy value for meaured solar radiation on horizontal surface cal/cm2/day
          using this value makes that the target value R6 is actually
          the potential solar radiation on a sloping surface based on map area, cal/cm2/day
          This R2 is no longer used and replaced by R2365[J], the solar radiation
          received at day J at an horizontal plane at given latitude}
  {start; fig.1 top left part Swift 1976}
  L1:=arcconv*arcsin(cos(inclination*pi/180)*sin(latitude*pi/180)+
                     sin(inclination*pi/180)*cos(latitude*pi/180)*cos(aspect*pi/180));
  {alternative calculation L2; fig.2 left part Swift 1976}
  D1:=cos(inclination*pi/180)*cos(latitude*pi/180)-
      sin(inclination*pi/180)*sin(latitude*pi/180)*cos(aspect*pi/180);
  if D1=0 then D1:=1e-10;
  L2:=arcconv*arctan(sin(inclination*pi/180)*sin(aspect*pi/180)/D1);
  if D1<0 then L2:=L2+180;  {L2 in degrees}
  {loop 1; fig.1 top right part Swift 1976}
  Rsum:=0;
  for J:=1 to 365 do
  begin
    R2365[J]:=FUNC4(latitude,J);
    D:=FUNC1(J,0.4,23.3,10);
    E:=FUNC1(J,1,0.0167,-3);
    R1:=60*R0/(E*E);
    T:=FUNC2(D,L1);
    T7:=T-L2;; T6:=-T-L2;
    T:=FUNC2(D,latitude);
    T1:=T; T0:=-T;
    if T7<T1 then T3:=T7 else T3:=T1;
    if T6>T0 then T2:=T6 else T2:=T0;
    {alternative T4 calc: Fig 2 top right part Swift 1976}
    if T3<T2 then begin T2:=0; T3:=0; end;
    T6:=T6+360;
    if T6<T1
    then begin T8:=T6; T9:=T1; R4:=FUNC3(D,L2,L1,T3,T2)+FUNC3(D,L2,L1,T9,T8); end
    else
    begin T7:=T7-360;
          if T7>T0
          then begin T8:=T0; T9:=T7; R4:=FUNC3(D,L2,L1,T3,T2)+FUNC3(D,L2,L1,T9,T8);end
          else R4:=FUNC3(D,L2,L1,T3,T2);
    end;
    T4:=T2/15; T5:=T3/15;
    R3:=FUNC3(D,0,latitude,T1,T0);
    F:=R4/R3;
    {R5:=F*R2 OLD;}
    R5:=F*R2365[J];
    R6:=R5/cos(inclination*pi/180);
    Rsum:=Rsum+R6;
  end;
  INCRAD:=Rsum;
end;

procedure TfrmMAIN.CheckSoftConstants;
{initialize all process constants and overwrite with values found in "SoftConstants.txt"
 (if this file is found)}
var fsc:textfile; txt,txtline:string[100]; a:array[1..nrminerals] of string[100]; b:array[1..nrparams] of string[100];
  found:boolean; sb1,dum:real; row,col,nrows,ncols,code:integer;
  imineral,iparam:integer;
begin
  {set default values first}
  untProcesses.k_DPM:=def_k_DPM;
  untProcesses.k_RPM:=def_k_RPM;
  untProcesses.k_BIO:=def_k_BIO;
  untProcesses.k_HUM:=def_k_HUM;
  untProcesses.DPM2RPM_agriculture:=def_DPM2RPM_agriculture;
  untProcesses.DPM2RPM_grassscrub:=def_DPM2RPM_grassscrub;
  untProcesses.DPM2RPM_deciduouswood:=def_DPM2RPM_deciduouswood;
  untProcesses.DPM2RPM_coniferouswood:=def_DPM2RPM_coniferouswood;
  untProcesses.frEctoLitter_grassscrub:=def_frEctoLitter_grassscrub;
  untProcesses.frEndoLitter_grassscrub:=def_frEndoLitter_grassscrub;
  untProcesses.frEctoLitter_deciduouswood:=def_frEctoLitter_deciduouswood;
  untProcesses.frEndoLitter_deciduouswood:=def_frEndoLitter_deciduouswood;
  untProcesses.frEctoLitter_coniferouswood:=def_frEctoLitter_coniferouswood;
  untProcesses.frEndoLitter_coniferouswood:=def_frEndoLitter_coniferouswood;
  untProcesses.frEctoLitter_agriculture:=def_frEctoLitter_agriculture;
  untProcesses.frEndoLitter_agriculture:=def_frEndoLitter_agriculture;
  untProcesses.facscaleCO2BIOHUM:=def_facscaleCO2BIOHUM;
  untProcesses.BIO2HUMratio:=def_BIO2HUMratio;
  untProcesses.frPINT_agriculture:=def_frPINT_agriculture;
  untProcesses.frPINT_grassscrub:=def_frPINT_grassscrub;
  untProcesses.frPINT_deciduouswood:=def_frPINT_deciduouswood;
  untProcesses.frPINT_coniferouswood:=def_frPINT_coniferouswood;
  untProcesses.clay_kd:=def_clay_kd;
  untProcesses.clay_kr:=def_clay_kr;
  untProcesses.clay_hthetamacro:=def_clay_hthetamacro;
  untProcesses.CM_n:=def_CM_n;
  untProcesses.CM_fref:=def_CM_fref;
  untProcesses.CM_vref:=def_CM_vref;
  untProcesses.clay_ectobd:=def_clay_ectobd;
  untProcesses.clay_ectothick:=def_clay_ectothick;
  untProcesses.frStoneSplit:=def_PW_frStoneSplit;
  PW_Pmax:=def_PW_Pmax;
  PW_dTdtPmax:=def_PW_dTdtPmax;
  untProcesses.relAl_Agriculture:=def_relAl_Agriculture; untProcesses.relK_Agriculture:=def_relK_Agriculture;
  untProcesses.relCa_Agriculture:=def_relCa_Agriculture;
  untProcesses.relMg_Agriculture:=def_relMg_Agriculture; untProcesses.relNa_Agriculture:=def_relNa_Agriculture;
  untProcesses.relAl_Coniferouswood:=def_relAl_Coniferouswood; untProcesses.relK_Coniferouswood:=def_relK_Coniferouswood;
  untProcesses.relCa_Coniferouswood:=def_relCa_Coniferouswood;
  untProcesses.relMg_Coniferouswood:=def_relMg_Coniferouswood; untProcesses.relNa_Coniferouswood:=def_relNa_Coniferouswood;
  untProcesses.relAl_Deciduouswood:=def_relAl_Deciduouswood; untProcesses.relK_Deciduouswood:=def_relK_Deciduouswood;
  untProcesses.relCa_Deciduouswood:=def_relCa_Deciduouswood;
  untProcesses.relMg_Deciduouswood:=def_relMg_Deciduouswood; untProcesses.relNa_Deciduouswood:=def_relNa_Deciduouswood;
  untProcesses.relAl_Grassscrub:=def_relAl_Grassscrub; untProcesses.relK_Grassscrub:=def_relK_Grassscrub;
  untProcesses.relCa_Grassscrub:=def_relCa_Grassscrub;
  untProcesses.relMg_Grassscrub:=def_relMg_Grassscrub; untProcesses.relNa_Grassscrub:=def_relNa_Grassscrub;
  untProcesses.MinDisPar[1,1]:=def_Albite_logkh25;     untProcesses.MinDisPar[2,1]:=def_Albite_kEah;
  untProcesses.MinDisPar[4,1]:=def_Albite_logkoh25;    untProcesses.MinDisPar[5,1]:=def_Albite_kEaoh;
  untProcesses.MinDisPar[3,1]:=def_Albite_n;           untProcesses.MinDisPar[6,1]:=def_Albite_m;
  untProcesses.MinDisPar[1,2]:=def_KFeldspar_logkh25;  untProcesses.MinDisPar[2,2]:=def_KFeldspar_kEah;
  untProcesses.MinDisPar[4,2]:=def_KFeldspar_logkoh25; untProcesses.MinDisPar[5,2]:=def_KFeldspar_kEaoh;
  untProcesses.MinDisPar[3,2]:=def_KFeldspar_n;        untProcesses.MinDisPar[6,2]:=def_KFeldspar_m;
  untProcesses.MinDisPar[1,3]:=def_Muscovite_logkh25;  untProcesses.MinDisPar[2,3]:=def_Muscovite_kEah;
  untProcesses.MinDisPar[4,3]:=def_Muscovite_logkoh25; untProcesses.MinDisPar[5,3]:=def_Muscovite_kEaoh;
  untProcesses.MinDisPar[3,3]:=def_Muscovite_n;        untProcesses.MinDisPar[6,3]:=def_Muscovite_m;
  untProcesses.MinDisPar[1,4]:=def_Biotite_logkh25;    untProcesses.MinDisPar[2,4]:=def_Biotite_kEah;
  untProcesses.MinDisPar[4,4]:=def_Biotite_logkoh25;   untProcesses.MinDisPar[5,4]:=def_Biotite_kEaoh;
  untProcesses.MinDisPar[3,4]:=def_Biotite_n;          untProcesses.MinDisPar[6,4]:=def_Biotite_m;
  untProcesses.MinDisPar[1,5]:=def_Quartz_logkh25;     untProcesses.MinDisPar[2,5]:=def_Quartz_kEah;
  untProcesses.MinDisPar[4,5]:=def_Quartz_logkoh25;    untProcesses.MinDisPar[5,5]:=def_Quartz_kEaoh;
  untProcesses.MinDisPar[3,5]:=def_Quartz_n;           untProcesses.MinDisPar[6,5]:=def_Quartz_m;
  untProcesses.MinDisPar[1,6]:=def_Chlorite_logkh25;   untProcesses.MinDisPar[2,6]:=def_Chlorite_kEah;
  untProcesses.MinDisPar[4,6]:=def_Chlorite_logkoh25;  untProcesses.MinDisPar[5,6]:=def_Chlorite_kEaoh;
  untProcesses.MinDisPar[3,6]:=def_Chlorite_n;         untProcesses.MinDisPar[6,6]:=def_Chlorite_m;
  untProcesses.MinDisPar[1,7]:=def_Anorthite_logkh25;  untProcesses.MinDisPar[2,7]:=def_Anorthite_kEah;
  untProcesses.MinDisPar[4,7]:=def_Anorthite_logkoh25; untProcesses.MinDisPar[5,7]:=def_Anorthite_kEaoh;
  untProcesses.MinDisPar[3,7]:=def_Anorthite_n;        untProcesses.MinDisPar[6,7]:=def_Anorthite_m;
  untProcesses.MinDisPar[1,8]:=def_Forsterite_logkh25;  untProcesses.MinDisPar[2,8]:=def_Forsterite_kEah;
  untProcesses.MinDisPar[4,8]:=def_Forsterite_logkoh25; untProcesses.MinDisPar[5,8]:=def_Forsterite_kEaoh;
  untProcesses.MinDisPar[3,8]:=def_Forsterite_n;        untProcesses.MinDisPar[6,8]:=def_Forsterite_m;
  untProcesses.MinDisPar[1,9]:=def_Augite_logkh25;     untProcesses.MinDisPar[2,9]:=def_Augite_kEah;
  untProcesses.MinDisPar[4,9]:=def_Augite_logkoh25;    untProcesses.MinDisPar[5,9]:=def_Augite_kEaoh;
  untProcesses.MinDisPar[3,9]:=def_Augite_n;           untProcesses.MinDisPar[6,9]:=def_Augite_m;
  untProcesses.MinDisPar[1,10]:=def_Kaolinite_logkh25; untProcesses.MinDisPar[2,10]:=def_Kaolinite_kEah;
  untProcesses.MinDisPar[4,10]:=def_Kaolinite_logkoh25;untProcesses.MinDisPar[5,10]:=def_Kaolinite_kEaoh;
  untProcesses.MinDisPar[3,10]:=def_Kaolinite_n;       untProcesses.MinDisPar[6,10]:=def_Kaolinite_m;
  untProcesses.MinDisPar[1,11]:=def_Illite_logkh25;    untProcesses.MinDisPar[2,11]:=def_Illite_kEah;
  untProcesses.MinDisPar[4,11]:=def_Illite_logkoh25;   untProcesses.MinDisPar[5,11]:=def_Illite_kEaoh;
  untProcesses.MinDisPar[3,11]:=def_Illite_n;          untProcesses.MinDisPar[6,11]:=def_Illite_m;
  untProcesses.MinDisPar[1,12]:=def_Montmorrillonite_logkh25; untProcesses.MinDisPar[2,12]:=def_Montmorrillonite_kEah;
  untProcesses.MinDisPar[4,12]:=def_Montmorrillonite_logkoh25; untProcesses.MinDisPar[5,12]:=def_Montmorrillonite_kEaoh;
  untProcesses.MinDisPar[3,12]:=def_Montmorrillonite_n;untProcesses.MinDisPar[6,12]:=def_Montmorrillonite_m;
  untProcesses.MinDisPar[1,13]:=def_Hornblende_logkh25;untProcesses.MinDisPar[2,13]:=def_Hornblende_kEah;
  untProcesses.MinDisPar[4,13]:=def_Hornblende_logkoh25;untProcesses.MinDisPar[5,13]:=def_Hornblende_kEaoh;
  untProcesses.MinDisPar[3,13]:=def_Hornblende_n;      untProcesses.MinDisPar[6,13]:=def_Hornblende_m;
  untProcesses.MinDisPar[1,14]:=def_Fayalite_logkh25;  untProcesses.MinDisPar[2,14]:=def_Fayalite_kEah;
  untProcesses.MinDisPar[4,14]:=def_Fayalite_logkoh25; untProcesses.MinDisPar[5,14]:=def_Fayalite_kEaoh;
  untProcesses.MinDisPar[3,14]:=def_Fayalite_n;        untProcesses.MinDisPar[6,14]:=def_Fayalite_m;
  untProcesses.MinDisPar[1,15]:=def_Amorphite_logkh25;  untProcesses.MinDisPar[2,15]:=def_Amorphite_kEah;
  untProcesses.MinDisPar[4,15]:=def_Amorphite_logkoh25; untProcesses.MinDisPar[5,15]:=def_Amorphite_kEaoh;
  untProcesses.MinDisPar[3,15]:=def_Amorphite_n;        untProcesses.MinDisPar[6,15]:=def_Amorphite_m;
  untProcesses.MinDisPar[1,16]:=def_Otherite_logkh25;  untProcesses.MinDisPar[2,16]:=def_Otherite_kEah;
  untProcesses.MinDisPar[4,16]:=def_Otherite_logkoh25; untProcesses.MinDisPar[5,16]:=def_Otherite_kEaoh;
  untProcesses.MinDisPar[3,16]:=def_Otherite_n;        untProcesses.MinDisPar[6,16]:=def_Otherite_m;
  {add things here if SoftConstants.txt is to be extended}
  if fileexists(AppendPathDelim(startdir)+'SoftConstants.txt')
  then {read, check and assign values to softconstants in untProcesses}
  begin
     system.assign(fsc,AppendPathDelim(startdir)+'SoftConstants.txt');
     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('grassscrub decomposable / resistant plant material ratio',txt)>0;
       if found then untProcesses.DPM2RPM_grassscrub:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('grassscrub fraction of litter that is ecto-organic',txt)>0;
       if found then untProcesses.frEctoLitter_grassscrub:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('grassscrub fraction of litter that is endo-organic',txt)>0;
       if found then untProcesses.frEndoLitter_grassscrub:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('grassscrub: fraction of Precipitation lost by Interception Evaporation',txt)>0;
       if found then untProcesses.frPINT_grassscrub:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('deciduouswood decomposable / resistant plant material ratio',txt)>0;
       if found then untProcesses.DPM2RPM_deciduouswood:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('deciduouswood fraction of litter that is ecto-organic',txt)>0;
       if found then untProcesses.frEctoLitter_deciduouswood:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('deciduouswood fraction of litter that is endo-organic',txt)>0;
       if found then untProcesses.frEndoLitter_deciduouswood:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);    
     repeat
       system.readln(fsc,txt);
       found:=pos('deciduouswood: fraction of Precipitation lost by Interception Evaporation',txt)>0;
       if found then untProcesses.frPINT_deciduouswood:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('coniferouswood decomposable / resistant plant material ratio',txt)>0;
       if found then untProcesses.DPM2RPM_coniferouswood:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('coniferouswood fraction of litter that is ecto-organic',txt)>0;
       if found then untProcesses.frEctoLitter_coniferouswood:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('coniferouswood fraction of litter that is endo-organic',txt)>0;
       if found then untProcesses.frEndoLitter_coniferouswood:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc); 
     repeat
       system.readln(fsc,txt);
       found:=pos('coniferouswood: fraction of Precipitation lost by Interception Evaporation',txt)>0;
       if found then untProcesses.frPINT_coniferouswood:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('agriculture decomposable / resistant plant material ratio',txt)>0;
       if found then untProcesses.DPM2RPM_agriculture:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('agriculture fraction of litter that is ecto-organic',txt)>0;
       if found then untProcesses.frEctoLitter_agriculture:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('agriculture fraction of litter that is endo-organic',txt)>0;
       if found then untProcesses.frEndoLitter_agriculture:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('agriculture: fraction of Precipitation lost by Interception Evaporation',txt)>0;
       if found then untProcesses.frPINT_agriculture:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('rate for decomposable plant material pool',txt)>0;
       if found then untProcesses.k_DPM:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('rate for resistant plant material pool',txt)>0;
       if found then untProcesses.k_RPM:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('rate for biomass pool',txt)>0;
       if found then untProcesses.k_BIO:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('rate for humus pool',txt)>0;
       if found then untProcesses.k_HUM:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('scaling factor for CO2/(BIO+HUM) ratio',txt)>0;
       if found then untProcesses.facscaleCO2BIOHUM:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('BIO/HUM ratio',txt)>0;
       if found then untProcesses.BIO2HUMratio:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('kd: soil detachability coefficient (g.J-1)',txt)>0;
       if found then untProcesses.clay_kd:=getal(txt);
     until found or(eof(fsc));
     
     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('kr: replenishment rate coefficient (g.m-2.h-1)',txt)>0;
       if found then untProcesses.clay_kr:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('h-theta(macro): pressure head (hPa) at which macropores empty',txt)>0;
       if found then untProcesses.clay_hthetamacro:=getal(txt)*10.19744289; {converted to mm for internal use}
     until found or(eof(fsc)); 

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('n: filtering coefficient (-)',txt)>0;
       if found then untProcesses.CM_n:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('fref: reference filter coefficient (m-1)',txt)>0;
       if found then untProcesses.CM_fref:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('vref: pore water velocity (m.h-1) at which fref is measured',txt)>0;
       if found then untProcesses.CM_vref:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('bulk density (kg.dm-3) of ectorganic layers',txt)>0;
       if found then untProcesses.clay_ectobd:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('thickness (mm) of ectorganic layer at which no more splash',txt)>0;
       if found then untProcesses.clay_ectothick:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('Max. split probability',txt)>0;
       if found then PW_Pmax:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('Temperature gradient (oC/h) with max. split probability',txt)>0;
       if found then PW_dTdtPmax:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos(' Fraction of coarse material entering the gravel fraction (frost spalling or root cracking events)',txt)>0;
       if found then begin PW_frStoneSplit:=getal(txt); untProcesses.frStoneSplit:=PW_frStoneSplit;  end;
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('Relative Ca contribution Grassscrub',txt)>0;
       if found then untProcesses.relCa_Grassscrub:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('Relative Mg contribution Grassscrub',txt)>0;
       if found then untProcesses.relMg_Grassscrub:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('Relative Na contribution Grassscrub',txt)>0;
       if found then untProcesses.relNa_Grassscrub:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('Relative K contribution Grassscrub',txt)>0;
       if found then untProcesses.relK_Grassscrub:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('Relative Al contribution Grassscrub',txt)>0;
       if found then untProcesses.relAl_Grassscrub:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('Relative Ca contribution Deciduouswood',txt)>0;
       if found then untProcesses.relCa_Deciduouswood:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('Relative Mg contribution Deciduouswood',txt)>0;
       if found then untProcesses.relMg_Deciduouswood:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('Relative Na contribution Deciduouswood',txt)>0;
       if found then untProcesses.relNa_Deciduouswood:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('Relative K contribution Deciduouswood',txt)>0;
       if found then untProcesses.relK_Deciduouswood:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('Relative Al contribution Deciduouswood',txt)>0;
       if found then untProcesses.relAl_Deciduouswood:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('Relative Ca contribution Coniferouswood',txt)>0;
       if found then untProcesses.relCa_Coniferouswood:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('Relative Mg contribution Coniferouswood',txt)>0;
       if found then untProcesses.relMg_Coniferouswood:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('Relative Na contribution Coniferouswood',txt)>0;
       if found then untProcesses.relNa_Coniferouswood:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('Relative K contribution Coniferouswood',txt)>0;
       if found then untProcesses.relK_Coniferouswood:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('Relative Al contribution Coniferouswood',txt)>0;
       if found then untProcesses.relAl_Coniferouswood:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('Relative Ca contribution Agriculture',txt)>0;
       if found then untProcesses.relCa_Agriculture:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('Relative Mg contribution Agriculture',txt)>0;
       if found then untProcesses.relMg_Agriculture:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('Relative Na contribution Agriculture',txt)>0;
       if found then untProcesses.relNa_Agriculture:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('Relative K contribution Agriculture',txt)>0;
       if found then untProcesses.relK_Agriculture:=getal(txt);
     until found or(eof(fsc));

     system.reset(fsc);
     repeat
       system.readln(fsc,txt);
       found:=pos('Relative Al contribution Agriculture',txt)>0;
       if found then untProcesses.relAl_Agriculture:=getal(txt);
     until found or(eof(fsc));

     {mineral dissolution parameters}
     nrows:=nrminerals;
     ncols:=nrparams;
     a[7]:='Anorthite';      a[1]:='Albite';      a[6]:='Chlorite';     a[2]:='K-Feldspar';
     a[3]:='Muscovite';      a[13]:='Hornblende'; a[4]:='Biotite';      a[5]:='Quartz';
     a[8]:='Forsterite';     a[9]:='Augite';     a[14]:='Fayalite';     a[10]:='Kaolinite';
     a[11]:='Illite';        a[12]:='Montmorillonite';
     a[15]:='Amorphite';     a[16]:='Otherite';
     for row:=1 to nrminerals do untProcesses.MinNames[row]:=a[row];
     b[1]:='log10 kH25 (mol/m2/s)' ;             b[2]:='kEaH (KJ/mol/K)';
     b[4]:='log10 kOH25 (mol/m2/s)';             b[5]:='kEaOH (KJ/mol/K)';
     b[3]:='reaction order n';                   b[6]:='reaction order m';

     for row:=1 to nrows do
     begin
       for col:=1 to ncols do
       begin
        system.reset(fsc);
        repeat
         system.readln(fsc,txtline);
         found:=pos((a[row]+': '+b[col]),txtline)>0;
         if found then
         begin
           txt:=copy(txtline,1,pos((a[row]+': '+b[col]),txtline)-1);
           system.val(trim(txt),untProcesses.MinDisPar[col,row],code);
           DataInit.WEGen.MinDisPar[col,row]:=untProcesses.MinDisPar[col,row];
         end;
        until found or (eof(fsc));
       end;
     end;

     {add things here if softconstants.txt is to be extended}
     closefile(fsc);
     sb1:=untProcesses.frEctoLitter_grassscrub+untProcesses.frEndoLitter_grassscrub;
     if sb1<>1 then
     begin
       untProcesses.frEctoLitter_grassscrub:=untProcesses.frEctoLitter_grassscrub/sb1;
       untProcesses.frEndoLitter_grassscrub:=untProcesses.frEndoLitter_grassscrub/sb1;
     end;
     sb1:=untProcesses.frEctoLitter_deciduouswood+untProcesses.frEndoLitter_deciduouswood;
     if sb1<>1 then
     begin
       untProcesses.frEctoLitter_deciduouswood:=untProcesses.frEctoLitter_deciduouswood/sb1;
       untProcesses.frEndoLitter_deciduouswood:=untProcesses.frEndoLitter_deciduouswood/sb1;
     end;
     sb1:=untProcesses.frEctoLitter_coniferouswood+untProcesses.frEndoLitter_coniferouswood;
     if sb1<>1 then
     begin
       untProcesses.frEctoLitter_coniferouswood:=untProcesses.frEctoLitter_coniferouswood/sb1;
       untProcesses.frEndoLitter_coniferouswood:=untProcesses.frEndoLitter_coniferouswood/sb1;
     end;
     sb1:=untProcesses.frEctoLitter_agriculture+untProcesses.frEndoLitter_agriculture;
     if sb1<>1 then
     begin
       untProcesses.frEctoLitter_agriculture:=untProcesses.frEctoLitter_agriculture/sb1;
       untProcesses.frEndoLitter_agriculture:=untProcesses.frEndoLitter_agriculture/sb1;
     end;
     ResultTxt:='Soft constants (values read from SoftConstants.txt)';
  end
  else {keep default values}
  begin
     nrows:=nrminerals;
     ncols:=nrparams;
     for row:=1 to nrows do for col:=1 to ncols do
        DataInit.WEGen.MinDisPar[col,row]:=untProcesses.MinDisPar[col,row];
     ResultTxt:='Soft constants (default values; no file SoftConstants.txt found)';
  end;

  {now transform log-KH25 and log-KOK25 to actual values: index 1 and 4. If the value=-999
   (missing data or no effect) then the transformed value becomes 0.
   This enables a generic formulation of the mineral dissolution rates in untProcesses}

  for row:=1 to nrows do
  begin
   if DataInit.WEGen.MinDisPar[1,row]<-998.99
   then
   begin
     dum:=DataInit.WEGen.MinDisPar[1,row];
     DataInit.WEGen.MinDisPar[1,row]:=0; untProcesses.MinDisPar[1,row]:=0;
   end
   else
   begin
     DataInit.WEGen.MinDisPar[1,row]:=power(10,DataInit.WEGen.MinDisPar[1,row]);
     untProcesses.MinDisPar[1,row]:=DataInit.WEGen.MinDisPar[1,row];
   end;
   if DataInit.WEGen.MinDisPar[4,row]<-998.99
   then
   begin
     DataInit.WEGen.MinDisPar[4,row]:=0; untProcesses.MinDisPar[4,row]:=0;
   end
   else
   begin
     DataInit.WEGen.MinDisPar[4,row]:=power(10,DataInit.WEGen.MinDisPar[4,row]);
     untProcesses.MinDisPar[4,row]:=DataInit.WEGen.MinDisPar[4,row];
   end;
  end;
  {now correct n and m values: index 3 and 6. If the value=-999
   (missing data or no effect) then the value becomes 0.
   This enables a generic formulation of the mineral dissolution rates in untProcesses}
   for row:=1 to nrows do
   begin
    if DataInit.WEGen.MinDisPar[3,row]<-998.99
    then
    begin
      DataInit.WEGen.MinDisPar[3,row]:=0; untProcesses.MinDisPar[3,row]:=0;
    end ;
    if DataInit.WEGen.MinDisPar[6,row]<-998.99
    then
    begin
      DataInit.WEGen.MinDisPar[6,row]:=0; untProcesses.MinDisPar[6,row]:=0;
    end
   end;
   {now correct activation energy values: index 2 and 5. If the value=-999
   (missing data or no effect) then the value becomes 0.
   This enables a generic formulation of the mineral dissolution rates in untProcesses}
   for row:=1 to nrows do
   begin
    if DataInit.WEGen.MinDisPar[2,row]<-998.99
    then
    begin
      DataInit.WEGen.MinDisPar[2,row]:=0; untProcesses.MinDisPar[2,row]:=0;
    end ;
    if DataInit.WEGen.MinDisPar[5,row]<-998.99
    then
    begin
      DataInit.WEGen.MinDisPar[5,row]:=0; untProcesses.MinDisPar[5,row]:=0;
    end
   end;
  settings1.Visible:=true;
end;

procedure TfrmMAIN.Settings1Click(Sender: TObject);
{opens a window to be used to adapt the SofConstants (and saves the result to SoftConstants.txt)}
var txt:string; col,row:integer; dum:real;
begin
  untSoftConstants.frmSoftConstants.strgrdMinDispar.Colcount:=nrparams+1;
  untSoftConstants.frmSoftConstants.strgrdMinDispar.Rowcount:=nrminerals+1;
  str(untProcesses.k_DPM:6:4,txt); untSoftConstants.frmSoftConstants.edtk_DPM.Text:=txt;
  str(untProcesses.k_RPM:6:4,txt); untSoftConstants.frmSoftConstants.edtk_RPM.text:=txt;
  str(untProcesses.k_BIO:6:4,txt); untSoftConstants.frmSoftConstants.edtk_BIO.text:=txt;
  str(untProcesses.k_HUM:6:4,txt); untSoftConstants.frmSoftConstants.edtk_HUM.text:=txt;
  str(untProcesses.DPM2RPM_agriculture:6:4,txt); untSoftConstants.frmSoftConstants.edtDPM2RPM_agriculture.text:=txt;
  str(untProcesses.DPM2RPM_grassscrub:6:4,txt); untSoftConstants.frmSoftConstants.edtDPM2RPM_grassscrub.text:=txt;
  str(untProcesses.DPM2RPM_deciduouswood:6:4,txt); untSoftConstants.frmSoftConstants.edtDPM2RPM_deciduouswood.text:=txt;
  str(untProcesses.DPM2RPM_coniferouswood:6:4,txt); untSoftConstants.frmSoftConstants.edtDPM2RPM_coniferouswood.text:=txt;
  str(untProcesses.frEctoLitter_grassscrub:6:4,txt); untSoftConstants.frmSoftConstants.edtfrEctoLitter_grassscrub.text:=txt;
  str(untProcesses.frEndoLitter_grassscrub:6:4,txt); untSoftConstants.frmSoftConstants.edtfrEndoLitter_grassscrub.text:=txt;
  str(untProcesses.frEctoLitter_deciduouswood:6:4,txt); untSoftConstants.frmSoftConstants.edtfrEctoLitter_deciduouswood.text:=txt;
  str(untProcesses.frEndoLitter_deciduouswood:6:4,txt); untSoftConstants.frmSoftConstants.edtfrEndoLitter_deciduouswood.text:=txt;
  str(untProcesses.frEctoLitter_coniferouswood:6:4,txt); untSoftConstants.frmSoftConstants.edtfrEctoLitter_coniferouswood.text:=txt;
  str(untProcesses.frEndoLitter_coniferouswood:6:4,txt); untSoftConstants.frmSoftConstants.edtfrEndoLitter_coniferouswood.text:=txt;
  str(untProcesses.frEctoLitter_agriculture:6:4,txt); untSoftConstants.frmSoftConstants.edtfrEctoLitter_agriculture.text:=txt;
  str(untProcesses.frEndoLitter_agriculture:6:4,txt); untSoftConstants.frmSoftConstants.edtfrEndoLitter_agriculture.text:=txt;
  str(untProcesses.frPINT_agriculture:6:4,txt); untSoftConstants.frmSoftConstants.edtfrPINT_agriculture.Text:=txt;
  str(untProcesses.frPINT_grassscrub:6:4,txt); untSoftConstants.frmSoftConstants.edtfrPINT_grassscrub.Text:=txt;
  str(untProcesses.frPINT_coniferouswood:6:4,txt); untSoftConstants.frmSoftConstants.edtfrPINT_coniferouswood.Text:=txt;
  str(untProcesses.frPINT_deciduouswood:6:4,txt); untSoftConstants.frmSoftConstants.edtfrPINT_deciduouswood.Text:=txt;
  str(untProcesses.facscaleCO2BIOHUM:6:4,txt); untSoftConstants.frmSoftConstants.edtfacscaleCO2BIOHUM.text:=txt;
  str(untProcesses.BIO2HUMratio:18:16,txt); untSoftConstants.frmSoftConstants.edtBIO2HUMratio.text:=txt;
  str(untProcesses.clay_kd:6:4,txt); untSoftConstants.frmSoftConstants.edtclay_Kd.Text:=txt;
  str(untProcesses.clay_kr:6:4,txt); untSoftConstants.frmSoftConstants.edtclay_Kd.Text:=txt;
  dum:=untProcesses.clay_hthetamacro/10.19744289;
  str(dum:6:2,txt); untSoftConstants.frmSoftConstants.edtclay_hthetamacro.Text:=txt;
  {hthetamacro backconverted from mm water to hPa}
  str(untProcesses.CM_n:7:5,txt); untSoftConstants.frmSoftConstants.edtclay_n.Text:=txt;
  str(untProcesses.CM_fref:6:4,txt); untSoftConstants.frmSoftConstants.edtclay_fref.Text:=txt;
  str(untProcesses.CM_vref:6:4,txt); untSoftConstants.frmSoftConstants.edtclay_vref.Text:=txt;
  str(untProcesses.clay_ectobd:6:4,txt); untSoftConstants.frmSoftConstants.edtclay_ectobd.Text:=txt;
  str(untProcesses.clay_ectothick:6:4,txt); untSoftConstants.frmSoftConstants.edtclay_ectothick.Text:=txt;
  str(PW_Pmax:10:7,txt); untSoftConstants.frmSoftConstants.edtPW_Psmax.Text:=txt;
  if chckbxPhysWeath.Enabled=false
  then str(PW_dTdtPmax/24:10:7,txt) {dTdtPmax has to be backtransformed to hourly gradient}
  else str(PW_dTdtPmax:10:7,txt);
  untSoftConstants.frmSoftConstants.edtPW_B.Text:=txt;
  str(PW_frStoneSplit:6:4,txt); untSoftConstants.frmSoftConstants.edtStoneSplit.Text:=txt;

  str(untProcesses.relAl_Agriculture:6:4,txt); untSoftConstants.frmSoftConstants.edtAgriculture_relAl.Text:=txt;
  str(untProcesses.relK_Agriculture:6:4,txt); untSoftConstants.frmSoftConstants.edtAgriculture_relK.Text:=txt;
  str(untProcesses.relCa_Agriculture:6:4,txt); untSoftConstants.frmSoftConstants.edtAgriculture_relCa.Text:=txt;
  str(untProcesses.relMg_Agriculture:6:4,txt); untSoftConstants.frmSoftConstants.edtAgriculture_relMg.Text:=txt;
  str(untProcesses.relNa_Agriculture:6:4,txt); untSoftConstants.frmSoftConstants.edtAgriculture_relNa.Text:=txt;
  str(untProcesses.relAl_Coniferouswood:6:4,txt); untSoftConstants.frmSoftConstants.edtConiferouswood_relAl.Text:=txt;
  str(untProcesses.relK_Coniferouswood:6:4,txt); untSoftConstants.frmSoftConstants.edtConiferouswood_relK.Text:=txt;
  str(untProcesses.relCa_Coniferouswood:6:4,txt); untSoftConstants.frmSoftConstants.edtConiferouswood_relCa.Text:=txt;
  str(untProcesses.relMg_Coniferouswood:6:4,txt); untSoftConstants.frmSoftConstants.edtConiferouswood_relMg.Text:=txt;
  str(untProcesses.relNa_Coniferouswood:6:4,txt); untSoftConstants.frmSoftConstants.edtConiferouswood_relNa.Text:=txt;
  str(untProcesses.relAl_Deciduouswood:6:4,txt); untSoftConstants.frmSoftConstants.edtDeciduouswood_relAl.Text:=txt;
  str(untProcesses.relK_Deciduouswood:6:4,txt); untSoftConstants.frmSoftConstants.edtDeciduouswood_relK.Text:=txt;
  str(untProcesses.relCa_Deciduouswood:6:4,txt); untSoftConstants.frmSoftConstants.edtDeciduouswood_relCa.Text:=txt;
  str(untProcesses.relMg_Deciduouswood:6:4,txt); untSoftConstants.frmSoftConstants.edtDeciduouswood_relMg.Text:=txt;
  str(untProcesses.relNa_Deciduouswood:6:4,txt); untSoftConstants.frmSoftConstants.edtDeciduouswood_relNa.Text:=txt;
  str(untProcesses.relAl_Grassscrub:6:4,txt); untSoftConstants.frmSoftConstants.edtGrassscrub_relAl.Text:=txt;
  str(untProcesses.relK_Grassscrub:6:4,txt); untSoftConstants.frmSoftConstants.edtGrassscrub_relK.Text:=txt;
  str(untProcesses.relCa_Grassscrub:6:4,txt); untSoftConstants.frmSoftConstants.edtGrassscrub_relCa.Text:=txt;
  str(untProcesses.relMg_Grassscrub:6:4,txt); untSoftConstants.frmSoftConstants.edtGrassscrub_relMg.Text:=txt;
  str(untProcesses.relNa_Grassscrub:6:4,txt); untSoftConstants.frmSoftConstants.edtGrassscrub_relNa.Text:=txt;
  for col:=1 to nrparams do for row:=1 to nrminerals do
  begin
    if ((col=1)or(col=4))and(untProcesses.MinDisPar[col,row]=0) then txt:='-999'
    else
    begin
      if ((col=1)or(col=4))and(untProcesses.MinDisPar[col,row]>0)
      then str(log10(untProcesses.MinDisPar[col,row]):7:3,txt)
      else str(untProcesses.MinDisPar[col,row]:7:3,txt);
    end;
    untSoftConstants.frmSoftConstants.strgrdMinDispar.Cells[col,row]:=txt;
  end;
  untSoftConstants.frmSoftConstants.strgrdMinDispar.refresh;
  {hiero}
  untSoftConstants.frmSoftConstants.Caption:=ResultTxt;

  untSoftConstants.frmSoftConstants.ShowModal;
  untSoftConstants.frmSoftConstants.Refresh;
end;

function TfrmMAIN.getal(txt:string):real;
{converts a string to a real number}
var code:integer; number:real;
begin
  txt:=trim(txt);
  system.Val(copy(txt,1,pos(' ',txt)-1),number,code);
  getal:=number;
end;

procedure TfrmMAIN.btbttnCO2Click(Sender: TObject);
{import pCO2 values for user-chosen years (will be interpolated)}
var YearBP,past,future,now,year:integer; getal:real; header:string; klaar:boolean;
begin
   klaar:=false;
   pndlgOPEN.Filter:= 'CO2 input files (*.txt)|*.txt';
   if pndlgOPEN.Execute then
   begin
      for YearBP:=1 to MaxNoyears do SeriespCO2[YearBP]:=-1;
      CO2FileName:=pndlgOpen.FileName;
      system.assign(CO2File,CO2FileName); system.reset(CO2File);
      system.readln(CO2File,header);
      while not(eof(CO2File)) do
      begin
        system.Readln(CO2File,YearBP,getal);
        if (YearBP<MaxNoyears)and(YearBP>0)
        then SeriespCO2[YearBP]:=getal;
      end;
      closefile(CO2File);
      {interpolate missing years}
      {1. First series of years until first input data: make constant}
      if SeriespCO2[MaxNoyears]<>-1
      then past:=MaxNoyears
      else
      begin
        now:=MaxNoYears;
        repeat
          now:=now-1;
        until (SeriespCO2[now]<>-1)or(now=1);
        past:=now;
        for year:=MaxNoYears downto past+1 do SeriespCO2[year]:=SeriespCO2[past];
      end;
      if (past=1) and (SeriespCO2[past]=-1)
      then begin SeriespCO2[past]:=pCO2atm; {default value} klaar:=true end;
      for year:=MaxNoYears downto past+1 do SeriespCO2[year]:=SeriespCO2[past];
      while not(klaar) do
      begin {2. read other data in blocks and interpolate intermediate years}
        while past>1 do
        begin {seek future}
           future:=past;
           repeat
             future:=future-1;
           until (SeriespCO2[future]<>-1)or(future=1);
           klaar:=(future=1)or(SeriespCO2[future]<>-1);
        {past=oldest year with data; future=youngest year with data. Now interpolate}
           if SeriespCO2[future]<>-1 then
           begin
             for year:=past-1 downto future+1 do
             begin
               SeriespCO2[year]:=SeriespCO2[past]-
                                (past-year)*(SeriespCO2[past]-SeriespCO2[future])/(past-future);
             end;
             past:=future;
           end
           else past:=1;
        end;
      end;
      {3. Last series of years from last input data to BPyear=1: make constant}
      past:=0;
      repeat past:=past+1 until SeriespCO2[past]<>-1;
      for year:=1 to past-1 do SeriespCO2[year]:=SeriespCO2[past];
   end;
   {tmp:}
   system.assign(CO2File,'CO2temp.txt'); system.rewrite(CO2File);
   system.writeln(CO2File,header);
   for yearBP:=MaxNoYears downto 1 do
   system.writeln(CO2File,YearBP,' ',SeriespCO2[yearBP]:10:8);
   closefile(CO2File);
end;

procedure TfrmMAIN.Prepare_Continuation;
{create a folder with the files needed to continue a finished run
 and puts all necessary files (incl. full soil variable status) in there}
var flocal:textfile; recfile:file; i,j:integer; tmp,fn1,fn2,fn3,fn4:string; ok:boolean;
begin
{1. create a folder with the files needed to continue this run}
  {if not} ok:=createdir(AppendPathDelim(startdir)+'continuation')
  {then showmessage('Continuation Dir already exists!')
  else showmessage('Continuation Dir created.')};
{2. copy to this subfolder : the soilgen executable
                             and possible other executables in the current folder,
                             e.g. postproc, tdgraph, tddcsvgraph}
  tmp:=GetExeExt;
  tmp:=AppendPathDelim(startdir)+'*'+tmp;
  if findfirst(tmp,faAnyFile,Frec) = 0
  then
  begin
    repeat
      bestandsnaam:=Frec.Name;
      while pos(PathDelim,bestandsnaam)>0 do delete(bestandsnaam,1,pos(PathDelim,bestandsnaam));
      copyfile(PChar(bestandsnaam),PChar(AppendPathDelim(startdir)+
                                         'continuation'+PathDelim+bestandsnaam),false);
    until FindNext(Frec) <> 0;
    FindClose(Frec);
  end;
{3. Prepare 2 (groups of) files that will contain the values of all relevant variables at the end of the current run}
{3a.The first group of files are plain textfiles that contain:
    3a1. The original soil input data file
    3a2. The original Chemical equilibrium settings file
    3a3. The original C data file
    3a4. The original Weathering data file
    3a5. The Batchmode file (will be recreated because it may not have existed
    These will be used only for initialization purposes (variable values will later be overridden)
    3a6. If it existed: the SoftConstants.txt file
    3a7. The Van Genuchten parameters per layer as far as these were input (not by PTF) in the current run.
    3a8. The Physical Weathering data per layer and texture class}
     fn1:=FirstLeachCFileName; while pos(PathDelim,fn1)>0 do delete(fn1,1,pos(PathDelim,fn1));
     fn1:='original '+fn1;
     fn2:=EQFileName; while pos(PathDelim,fn2)>0 do delete(fn2,1,pos(PathDelim,fn2));
     fn2:='original '+fn2;
     fn3:=FirstRothCFileName; while pos(PathDelim,fn3)>0 do delete(fn3,1,pos(PathDelim,fn3));
     fn3:='original '+fn3;
     fn4:=FirstWeatherFileName; while pos(PathDelim,fn4)>0 do delete(fn4,1,pos(PathDelim,fn4));
     fn4:='original '+fn4;
{3a1}copyfile(PChar(FirstLeachCFileName),PChar(AppendPathDelim(startdir)+'continuation'+PathDelim+fn1),false);
{3a2}copyfile(PChar(EQFileName),PChar(AppendPathDelim(startdir)+'continuation'+PathDelim+fn2),false);
{3a3}copyfile(PChar(FirstRothCFileName),PChar(AppendPathDelim(startdir)+'continuation'+PathDelim+fn3),false);
{3a4}copyfile(PChar(FirstWeatherFileName),PChar(AppendPathDelim(startdir)+'continuation'+PathDelim+fn4),false);
     copyfile(PChar('Amorphite.txt'),PChar(AppendPathDelim(startdir)+'continuation'+PathDelim+'Amorphite.txt'),false);
     copyfile(PChar('Otherite.txt'),PChar(AppendPathDelim(startdir)+'continuation'+PathDelim+'Otherite.txt'),false);
{3a5}system.Assign(flocal,AppendPathDelim(startdir)+'continuation'+PathDelim+'SoilGenBatch.txt');
     system.Rewrite(flocal);
     system.Writeln(flocal,fn1);
     system.Writeln(flocal,fn2);
     system.Writeln(flocal,fn3);
     system.Writeln(flocal,fn4);
     system.Writeln(flocal,1);   {so the new run is 1 year unless the user modifies this via GUI}
     system.Writeln(flocal,datayearin.slope); {slope}
     system.Writeln(flocal,datayearin.upSlopeBearing); {upslopebearing}
     system.Writeln(flocal,datayearin.downWindBearing); {downwindbearing}
     system.Writeln(flocal,datayearin.Latitude); {latitude}
     closefile(flocal);
{3a6}if fileexists(AppendPathDelim(startdir)+'SoftConstants.txt')
     then copyfile(PChar(AppendPathDelim(startdir)+'SoftConstants.txt'),
                   PChar(AppendPathDelim(startdir)+'continuation'+PathDelim+'SoftConstants.txt'),false);
{3a7}system.Assign(flocal,AppendPathDelim(startdir)+'continuation'+PathDelim+'continueVanGenuchtenMeasured.txt');
     system.Rewrite(flocal);
     for i:=1 to maxLayers do
       system.Writeln(flocal,VGNts[i],'  ',VGNtr[i],'  ',VGNal[i],'  ',VGNen[i],'  ',
                             VGNel[i],'  ',VGNks[i],'  ',VGNem[i]);
     closefile(flocal);
{3a8}system.Assign(flocal,AppendPathDelim(startdir)+'continuation'+PathDelim+'continuePhysicalWeathering.txt');
     system.Rewrite(flocal);
     system.Writeln(flocal,PW_E,'  ',PW_P);
     for i := -3 to 11 do system.Write(flocal,PW_X[i],'  ');
     system.Writeln(flocal);
     for i := -3 to 11 do system.Write(flocal,PW_PARDEN[i],'  ');
     system.Writeln(flocal);
     for j:= 1 to maxlayers do
     begin
       for i:=-4 to 11 do
          system.Write(flocal,PW_S[i,j]:20,'  ',PW_K[i,j]:20,'  ',PW_G[i,j],'  ',mass[i,j]);
       system.Writeln(flocal);
     end;
     closefile(flocal);
{3b.The second file is an unformatted file containing all relevant variable values at the and of the current run.
    These are stored in the record "DataYearIn" at the time Prepare_Continuation is called
    (DataYearIn = dataYearOut of last run year + effect of salt loss on particles)}
    DataRecGrootte:=Sizeof(DataInit);
    AssignFile(recfile,AppendPathDelim(startdir)+'continuation'+PathDelim+'continue.rec');
    system.Rewrite(recfile,DataRecGrootte); system.BlockWrite(recfile,DataInit,1);
    system.Close(recfile);
    DataRecGrootte:=Sizeof(DataYearIn);
    AssignFile(recfile,AppendPathDelim(startdir)+'continuation'+PathDelim+'continue2.rec');
    system.Rewrite(recfile,DataRecGrootte); system.BlockWrite(recfile,DataYearIn,1);
    system.Close(recfile);
    DataRecGrootte:=Sizeof(DataYearOut);
    AssignFile(recfile,AppendPathDelim(startdir)+'continuation'+PathDelim+'continue3.rec');
    system.Rewrite(recfile,DataRecGrootte); system.BlockWrite(recfile,DataYearOut,1);
    system.Close(recfile)
end;

procedure TfrmMain.ReadContinuation;
{read the files in the continuation folder and initializes the model variables}
var flocal:textfile; recfile:file; i,j:integer;
begin
  {At this stage, via the batchfile all soil variables are initialized the same way
  as in the preceding run (with the same inputfiles).
  Additionally, 3 files are present:
  a continue.rec+continue2.rec+continue3.rec (values of most soil variables at the end of the preceding run: dataInit, dataYearIn, dataYearOut)
  b continuePhysicalWeathering.txt (particle disaggregation data values at end preceding run)
  c continueVanGenuchtenMeasured.txt (values-if present- of measured vanGenuchten parameters.}
 {c} system.Assign(flocal,AppendPathDelim(startdir)+'continueVanGenuchtenMeasured.txt');
     system.Reset(flocal);
     for i:=1 to maxLayers do
       system.Readln(flocal,VGNts[i],VGNtr[i],VGNal[i],VGNen[i],VGNel[i],VGNks[i],VGNem[i]);
     closefile(flocal);
 {b} system.Assign(flocal,AppendPathDelim(startdir)+'continuePhysicalWeathering.txt');
     system.Reset(flocal);
     system.Readln(flocal,PW_E,PW_P);
     for i := -3 to 11 do system.read(flocal,PW_X[i]);
     system.readln(flocal);
     for i := -3 to 11 do system.read(flocal,PW_PARDEN[i]);
     system.readln(flocal);
     for j:= 1 to maxLayers do
     begin
       for i:=-4 to 11 do
          system.read(flocal,PW_S[i,j],PW_K[i,j],PW_G[i,j],mass[i,j]);
       system.readln(flocal);
     end;
     closefile(flocal);
{a. The unformatted file containing all relevant variable values at the end of the current run.
    These are stored in the record "DataYearIn" at the time Prepare_Continuation is called
    (DataYearIn = dataYearOut of last run year + effect of salt loss on particles)}
    DataRecGrootte:=Sizeof(DataInit);
    AssignFile(recfile,AppendPathDelim(startdir)+'continue.rec'); system.Reset(recfile,DataRecGrootte);
    system.BlockRead(recfile,DataInit,1);
    system.Close(recfile);
    DataRecGrootte:=Sizeof(DataYearIn);
    AssignFile(recfile,AppendPathDelim(startdir)+'continue2.rec'); system.Reset(recfile,DataRecGrootte);
    system.BlockRead(recfile,DataYearIn,1);
    system.Close(recfile);
    DataRecGrootte:=Sizeof(DataYearOut);
    AssignFile(recfile,AppendPathDelim(startdir)+'continue3.rec'); system.Reset(recfile,DataRecGrootte);
    system.BlockRead(recfile,DataYearOut,1);
    system.Close(recfile);
    {now copy the contents of DataYearIn to DataInit,
     because dataYearIn still contains the initial data for the preceding run}
    CopyRecToFrom(DataInit,DatayearIn);     {copy records basic inputfile to DataYearIn}
end;

procedure TfrmMain.CopyRecToFrom(var TargetRec,SourceRec:DataRec);
{copy a data record: SourceRec to TargetRec}
{var recfile:file; DataRecGrootte:integer;}
begin
    TargetRec:=SourceRec;
    {DataRecGrootte:=Sizeof(SourceRec);
    AssignFile(recfile,startdir+'\copy.rec'); system.Rewrite(recfile,DataRecGrootte);
    system.BlockWrite(recfile,SourceRec,1);
    system.Close(recfile);
    AssignFile(recfile,startdir+'\copy.rec'); system.Reset(recfile,DataRecGrootte);
    system.BlockRead(recfile,TargetRec,1);
    system.Close(recfile); }

end;

procedure TfrmMain.ContinueStartStatus;
{reads a data record "continue2.rec" from the continuation folder
 and reports its contents in "ContinueReport.txt"}
var recfile:file; i,j:integer; txt:string;
begin
  DataRecGrootte:=Sizeof(OnDisplay);
  AssignFile(recfile,AppendPathDelim(startdir)+'continue2.rec'); system.Reset(recfile,DataRecGrootte);
  system.BlockRead(recfile,OnDisplay,1); {that is DataYearIn of previous run,
                                          which is equal to DataYearOut but includes effect of salt change on texture,
                                          this is the initial data set for the continuation run}
  system.Close(recfile);
  {now override the records for the data that must be kept from the Read Default Inputfiles}
  OverrideSelectedData;
  system.Assign(fouttxt,'ContinueReport.txt'); system.Rewrite(fouttxt);
with OnDisplay do
begin
syswT('GENERAL SETTINGS DATA');
syswlI(idate,' idate');
syswlI(day1,'day1');
syswlI(CUMT1,' CUMT1');
syswlI(NYC,' NYC');
syswlI(NoLayers,' NoLayers (nr layers in soil profile)');
syswlI(BBC,' BBC (bottom boundary condition: 1:fixed depth water table; 2:free drainage, 3:zero flux, 4:lysimeter, 5: varying bottom head');
syswlI(IFLOW,' IFLOW: 1: Richards; 2: Addiscott tipping bucket; 3: steady-state');
syswlI(INITFLG,' INITFLG');
syswlR(NSUM,' NSUM: <Summary print interval (d) (999 for calendar month printouts)');
syswlI(nd1,' nd1: <Surface to [depth 1?] mm (summary file)');
syswlI(nd2,' nd2: <Surface to [depth 2?] mm (summary file)');
syswlI(nd3,' nd3: <Surface to [depth 3?] mm (summary file)');
syswlI(nd4,' nd4: <4th segment: Root zone (1); profile (2); Depth 3 to lower boundary (3); Surface to shallowest of lower');
syswlI(KLPLT,' KLPLT: <Plants present: 1 yes, 0 no.');
syswlI(NCROP,' NCROP: <No. of crops (>0)');
syswlI(NN1,' NN1: Selectivity coefficients: 0: Input=correct. 1: Calculate from Solution and exchangeable cations. 2: Selectivity coefficients and soluble cations are correct, calculate exchangeable cations.');
syswlI(ITERCH,' ITERCH: < Time steps between chemical equilibrations');
syswlI(IFR,' IFR: < Number of broadcast applications. (At least 1)');
syswlI(Icult ,' Icult: < Number of cultivations. (At least 1)');
syswlI(NIRRIG,' NIRRIG: < Number of water applications.');
syswlI(ch_irr_flag,' ch_irr_flag: < For a separate irrigation file, set to 1 otherwise 0');
syswlI(NWEEK,' NWEEK: number of weeks in 1 simulation year');
syswlI(PRIOR,' PRIOR: < Use water contents (1) or potentials (2) as starting values in input files (should be 2 after year 1)');
syswlR(DELM,' DELM: <Largest time interval within a day (0.1 day or less)');
syswlR(PDEPTH,' PDEPTH: profile depth (mm)');
syswlR(DLZ,' DLZ (compartment thickness (mm)');
syswlR(ABBPOT,' ABBPOT:<Water table depth (mm), if the lower boundary is 1 (water table)');
syswlR(THETA1,' THETA1:< Steady-state flow water content (theta); 999: saturated column');
syswlR(FWBTC,' FWBTC: (for breakthrough file: Incremental depth of drainage water per output (mm)');
syswlR(BCAM1,' BCAM1: <Exponent (BCAM) in Campbells water retention equation (if uniform profile)');
syswlR(AEV1,' AEV1: <Air-entry value (AEV) (kPa)(if uniform profile)');
syswlR(CMAT1,' CMAT1: <Conductivity (mm/day) (if uniform profile)');
syswlR(HMAT1,' HMAT1: <corresponding matric potential (kPa)(if uniform profile)');
syswlR(RP1,' RP1: <Pore interaction parameter (P) in Campbells conductivity equation (should be 1)(if uniform profile)');
syswlR(ALAMB1,' ALAMB1: <Dispersivity (mm)(if uniform profile)');
syswlR(RHOS2,' RHOS2: Particle density: Clay (kg/dm3)');
syswlR(RHOS1,' RHOS1: Particle density: Silt and sand (kg/dm3)');
syswlR(RHOS3,' RHOS3: Particle density: Organic matter (kg/dm3)');
syswlR(RHO1,' RHO1: <Soil bulk density (kg/dm3)(if uniform profile)');
syswlR(parden1,' parden1: <particle density (kg/dm3)(if uniform profile)');
syswlR(CELLW,' CELLW: < Depth (mm) of water in mixing cell (BBC 1 and 5 only). 0 for no mixing cell');
syswlR(DIFO,' DIFO: <Molecular diffusion coefficient in water (mm2/day)');
syswlR(rocn2,' rocn2: <Curve number (CN2)');
syswlR(roslope,' roslope: <Slope, %. Used to adjust CN2 according to equation of Williams (1991)');
syswlR(HDRY,' HDRY: <Wilting point (soil) kPa');
syswlR(HLOW,' HLOW: <Min.root water potential(kPa)');
syswlR(AK2,' AK2: <Maximum ratio of actual to potential transpiration (dry surface)');
syswlR(RRES,' RRES: <Root resistance (weights water uptake by depth). (>1, No weighting: 1.0).');
syswlR(unfrozen,' unfrozen: unfrozen part of the profile above the water table');
syswlR(WaterTableDepth,' WaterTableDepth: water table depth read from events.txt (mm) or very deep (999)');
system.writeln(fouttxt,' First Year = ',firstyear);
syswT('');
syswT('PROFILE DATA');
syswT('layer Clay  Silt  Sand  OC    RHO   parden porosity gravel  stones');
for i:=1 to NoLayers do system.writeln(fouttxt,i:5,' ',CLAY[i]:5:2,' ',SILT[i]:5:2,' ',SAND[i]:5:2,' ',OC[i]:5:2,' ',RHO[i]:5:3,' ',parden[i]:5:3,' ',porosity[i]:5:3,
                                                       GRAVEL[i]:5:2,' ',STONES[i]:5:2);
syswT('');
syswT('layer ClayDispInd klei2 silt2  zand2 volpercclay CLAYm    SILTm    SANDm    OCm      Gravelm  Stonesm (mass 1 sq m)');
for i:=1 to NoLayers do system.writeln(fouttxt,i:5,' ',ClayDispInd[i]:11:8,' ',klei2[i]:5:2,' ',silt2[i]:5:2,' ',zand2[i]:5:2,' ',volpercclay[i]:11:7,' ',
                                               CLAYm[i]:8:1,' ',SILTm[i]:8:1,' ',SANDm[i]:8:1,' ',OCm[i]:8:1,'  ',gravelm[i]:8:1,'  ',stonesm[i]:8:1);
syswT('');
syswT('layer ModRet AW   AH    ATEM  AAEV  ABCAM CMATCH HMATCH (initial soil physical values)');
for i:=1 to NoLayers do system.writeln(fouttxt,i:5,' ',MODRET[i]:5,' ',AW[i]:5:2,' ',AH[i]:5:2,' ',ATEM[i]:5:2,' ',AAEV[i]:5:2,' ',ABCAM[i]:5:3,' ',CMATCH[i]:5:3,' ',HMATCH[i]:5:3);
syswT('');
syswT('layer RDF   RP   LAMBDA W     TEMP TEMPGEM DOC   DOCperc(current soil physical values)   ET    FLUX');
for i:=1 to NoLayers do system.writeln(fouttxt,i:5,' ',RDF[i]:5:3,' ',RP[i]:5:1,' ',LAMBDA[i]:5:1,' ',W[i]:5:3,' ',TEM[i]:5:1,' ',TEMPGEM[i]:5:1,' ',
                                                       DOC[i]:5:3,' ',DOCperc[i]:5:2,'                                  ',ET[i]:5:2,' ',FLUX[i]:5:2);
syswT('');
syswT('layer measured vanGenuchten parameters (applicable + Values VGNal,VGNtr,VGNts,VGNks,VGNen,VGNel,VGNem)');
for i:=1 to NoLayers do system.writeln(fouttxt,i:5,' ',VGNmeasured[i],' ',VGNal[i]:15,' ',VGNtr[i]:15,' ',VGNts[i]:15,' ',VGNks[i]:15,' ',VGNen[i]:15,' ',VGNel[i]:15,' ',VGNem[i]:15);


syswT('layer SC1   SC2   SC3   SC4   SC5   SC6   SC7   SC8   SC9   SC10   SC11   SC12   SC13   SC14   SC15 (selectivity coefficients)');
for i:=1 to NoLayers do system.writeln(fouttxt,i:5,' ',SC1[i]:5:3,' ',SC2[i]:5:3,' ',SC3[i]:5:3,' ',SC4[i]:5:3,' ',SC5[i]:5:3,' ',
                                                       SC6[i]:5:3,' ',SC7[i]:5:3,' ',SC8[i]:5:3,' ',SC9[i]:5:3,' ',SC10[i]:5:3,' ',
                                                       SC11[i]:5:3,' ',SC12[i]:5:3,' ',SC13[i]:5:3,' ',SC14[i]:5:3,' ',SC15[i]:5:3);
syswT('');
syswT('layer oldpH pH  oldCALC CALC oldGYP GYP   ALKAL BS    SAR   ESP    EC');
for i:=1 to NoLayers do system.writeln(fouttxt,i:5,' ',APH[i]:5:3,' ',PH[i]:5:3,' ',oldCALCITEA[i]:5:3,' ',CALCITEA[i]:5:3,' ',
                                                       oldGYPA[i]:5:3,' ',GYPA[i]:5:3,' ',alkalinity[i]:5:2,' ',BS[i]:5:2,' ',
                                                       SAR[i]:5:2,' ',ESP[i]:5:2,' ',EC[i]:5:2);
syswT('');
syswT('layer ACEC      ACEC_oc   ACEC_clay ClayFrCEC OCFrCEC   FacCEC    FacCEC_OC CEC_empirical');
for i:=1 to NoLayers do system.writeln(fouttxt,i:5,' ',ACEC[i]:9:5,' ',ACEC_oc[i]:9:5,' ',ACEC_clay[i]:9:5,' ',ClayFrCEC[i]:9:5,' ',OCFrCEC[i]:9:5,' ',
                                                       FacCEC[i]:9:5,' ',FacCEC_OC[i]:9:5,' ',CEC_empirical[i]:9:5);
syswT('');
syswT('layer CHARGE    APCO2     CO3       HCO3 DISSOLVEDCATEQ DELGIBB   DELGYP    DECALCITE OSPOT     delco2');
for i:=1 to NoLayers do system.writeln(fouttxt,i:5,' ',CHARGE[i]:9:5,' ',APCO2[i]:9:5,' ',CO3[i]:9:5,' ',HCO3[i]:9:5,' ',DISSOLVEDCATEQ[i]:9:5,' ',
                                                       DELGIBB[i]:9:5,' ',DELGYP[i]:9:5,' ',DECALCITE[i]:9:5,' ',OSPOT[i]:9:5,' ',delco2[i]:9:5);
syswT('');
syswT('layer SUM21     MM21');
for i:=1 to NoLayers do system.writeln(fouttxt,i:5,' ',SUM21[i]:5:2,' ',MM21[i]:5:2);
syswT('');
syswT('CHEMICAL STUFF');
syswT('nr Description');
for i:=1 to maxK do syswlI(i,KNAMES[i]);
syswT('Chemical equilibrium coefficients table for different temperatures:');
txt:='nr -10    -9      -8      -7      -6      -5      -4      -3      -2      -1'+
     '      0       1       2       3       4       5       6       7       8     '+
     '  9       10      11      12      13      14      15      16      17      18'+
     '      19      20      21      22      23      24      25      26      27    '+
     '  28      29      30      31      32      33      34      35      36      37'+
     '      38      39      40 ';
syswT(txt);
for i:=1 to maxK do
begin
   system.write(fouttxt,i:4);
   for j:=-10 to 40 do system.write(fouttxt,KValueTable[i,j]:7,' ');
   system.writeln(fouttxt);
end;
syswT('');
syswT('Cumulative mineralization from plant residues');
syswT('Species list:');
syswT('1=Ca');
syswT('2=Mg');
syswT('3=Na');
syswT('4=K');
syswT('5=Cl');
syswT('6=SO4');
syswT('7=HCO3');
syswT('8=CO3');
syswT('9=Balion');
syswT('10=H');
syswT('11=OH');
syswT('12=Al');
syswT('13=FeIII');
syswT('14=FeII');
syswT('15=H4SiO4');
syswT('16=DOC, not used in this programme');
syswT('17=dispersible clay assumed to have no charge');
syswT('18=Ca on exchange complex of dispersed clay');
syswT('19=Mg on exchange complex of dispersed clay');
syswT('20=Na on exchange complex of dispersed clay');
syswT('21=K on exchange complex of dispersed clay');
syswT('22=H on exchange complex of dispersed clay');
syswT('23=Al on exchange complex of dispersed clay');
syswT('Cumulative mineralization from plant residues:');
for i:=1 to maxChem do system.write(fouttxt,i:10);
system.writeln(fouttxt);
for i:=1 to maxChem do system.write(fouttxt,CMINER[i]:9:5,' ');
system.writeln(fouttxt);
syswT('Amount of species per layer in DPMpool:');
system.write(fouttxt,'Layer');
for i:=1 to maxChem do system.write(fouttxt,i:10);
system.writeln(fouttxt);
for i:=1 to NoLayers do
begin
  system.write(fouttxt,i:4,' ');
  for j:=1 to MaxChem do system.write(fouttxt,CHEMDPMpool[j,i]:9:4,' ');
  system.writeln(fouttxt);
end;
syswT('Amount of species per layer in RPMpool:');
system.write(fouttxt,'Layer');
for i:=1 to maxChem do system.write(fouttxt,i:10);
system.writeln(fouttxt);
for i:=1 to NoLayers do
begin
  system.write(fouttxt,i:4,' ');
  for j:=1 to MaxChem do system.write(fouttxt,CHEMRPMpool[j,i]:9:4,' ');
  system.writeln(fouttxt);
end;
syswT('Amount of species per layer in BIOpool:');
system.write(fouttxt,'Layer');
for i:=1 to maxChem do system.write(fouttxt,i:10);
system.writeln(fouttxt);
for i:=1 to NoLayers do
begin
  system.write(fouttxt,i:4,' ');
  for j:=1 to MaxChem do system.write(fouttxt,CHEMBIOpool[j,i]:9:4,' ');
  system.writeln(fouttxt);
end;
syswT('Amount of species per layer in HUMpool:');
system.write(fouttxt,'Layer');
for i:=1 to maxChem do system.write(fouttxt,i:10);
system.writeln(fouttxt);
for i:=1 to NoLayers do
begin
  system.write(fouttxt,i:4,' ');
  for j:=1 to MaxChem do system.write(fouttxt,CHEMHUMpool[j,i]:9:4,' ');
  system.writeln(fouttxt);
end;
syswT('Amount of species per layer in IOMpool:');
system.write(fouttxt,'Layer');
for i:=1 to maxChem do system.write(fouttxt,i:10);
system.writeln(fouttxt);
for i:=1 to NoLayers do
begin
  system.write(fouttxt,i:4,' ');
  for j:=1 to MaxChem do system.write(fouttxt,CHEMIOMpool[j,i]:9:4,' ');
  system.writeln(fouttxt);
end;
syswT('Amount of species per layer in CO2pool:');
system.write(fouttxt,'Layer');
for i:=1 to maxChem do system.write(fouttxt,i:10);
system.writeln(fouttxt);
for i:=1 to NoLayers do
begin
  system.write(fouttxt,i:4,' ');
  for j:=1 to MaxChem do system.write(fouttxt,CHEMCO2pool[j,i]:9:4,' ');
  system.writeln(fouttxt);
end;

syswT('Amount of species per layer in solution :');
system.write(fouttxt,'Layer');
for i:=1 to maxChem do system.write(fouttxt,i:10);
system.writeln(fouttxt);
for i:=1 to NoLayers do
begin
  system.write(fouttxt,i:4,' ');
  for j:=1 to MaxChem do system.write(fouttxt,SEA[j][i]:9:4,' ');
  system.writeln(fouttxt);
end;
syswT('Amount of species per layer at exchange complex :');
system.write(fouttxt,'Layer');
for i:=1 to maxChem do system.write(fouttxt,i:10);
system.writeln(fouttxt);
for i:=1 to NoLayers do
begin
  system.write(fouttxt,i:4,' ');
  for j:=1 to MaxChem do system.write(fouttxt,AXSE[j][i]:9:4,' ');
  system.writeln(fouttxt);
end;
syswT('Amount of species per layer at exchange complex (at OC) :');
system.write(fouttxt,'Layer');
for i:=1 to maxChem do system.write(fouttxt,i:10);
system.writeln(fouttxt);
for i:=1 to NoLayers do
begin
  system.write(fouttxt,i:4,' ');
  for j:=1 to MaxChem do system.write(fouttxt,AXSE_oc[j][i]:9:4,' ');
  system.writeln(fouttxt);
end;
syswT('Amount of species per layer at exchange complex (at clay) :');
system.write(fouttxt,'Layer');
for i:=1 to maxChem do system.write(fouttxt,i:10);
system.writeln(fouttxt);
for i:=1 to NoLayers do
begin
  system.write(fouttxt,i:4,' ');
  for j:=1 to MaxChem do system.write(fouttxt,AXSE_clay[j][i]:9:4,' ');
  system.writeln(fouttxt);
end;
syswT('Cropping: igrowth icut    rtype   PDAY    ESTART  RDFDAY  ESTOP   DAYHAR');
for i:=1 to NCROP do
begin
system.writeln(fouttxt,'          ',igrowth[i]:7,' ',icut[i]:7,' ',rtype[i]:7,' ',PDAY[i]:7,' ',ESTART[i]:7,' ',RDFDAY[i]:7,' ',ESTOP[i]:7,' ',DAYHAR[i]:7);
end;
syswT('Cropping: RDEP    AK1     PANFAC');
for i:=1 to NCROP do
begin
system.writeln(fouttxt,'          ',RDEP[i]:7:2,' ',AK1[i]:7:2,' ',PANFAC[i]:7:2);
end;
syswT('C-RECORDS');
txt:='Current vegetation:='+RothC.vegetatie+'.';
syswT(txt);
syswT('GENPOOLPERCS for DPM-RPM-BIO-HUM-IOM');
for i:=1 to 5 do system.writeln(fouttxt,RothCGen.GENPOOLPERCS[i],' ');
syswT('Rooting depths (mm) for gras-deciduouswood-coniferouswood-agriculture');
system.writeln(fouttxt,RothCGen.GENgrasRDEPTH:5:1,' ',RothCGen.GENdeciRDEPTH:5:1,' ',
                       RothCGen.GENconiRDEPTH:5:1,' ',RothCGen.GENagriRDEPTH:5:1);
syswT('Monthly distribution C-inputs');
syswT('Month GENgrasPlant GENgrasMan   GENdeciPlant GENdeciMan   GENconiPlant GENconiMan   GENagriPlant GENagriMan');
for i:=1 to 12 do system.writeln(fouttxt,i:5,' ',RothCGen.GENgrasPlant[i]:12:7,' ',
                                                 RothCGen.GENgrasMan[i]:12:7,' ',
                                                 RothCGen.GENdeciPlant[i]:12:7,' ',
                                                 RothCGen.GENdeciMan[i]:12:7,' ',
                                                 RothCGen.GENconiPlant[i]:12:7,' ',
                                                 RothCGen.GENconiMan[i]:12:7,' ',
                                                 RothCGen.GENagriPlant[i]:12:7,' ',
                                                 RothCGen.GENagriMan[i]:12:7);
syswT('Profiles of C-pools');
syswT('Layer     DPMpool     RPMpool     BIOpool     HUMpool     IOMpool     CO2pool     accTSMD     pCO2');
system.writeln(fouttxt,'ECTO  ',RothC.ectoDPMpool:12:7,' ',
                                RothC.ectoRPMpool:12:7,' ',
                                RothC.ectoBIOpool:12:7,' ',
                               RothC.ectoHUMpool:12:7,' ',
                               RothC.ectoIOMpool:12:7,' ',
                               RothC.ectoCO2pool:12:7);
for i:=1 to NoLayers do system.writeln(fouttxt,i:5,' ',RothC.DPMpool[i]:12:7,' ',
                                                       RothC.RPMpool[i]:12:7,' ',
                                                       RothC.BIOpool[i]:12:7,' ',
                                                       RothC.HUMpool[i]:12:7,' ',
                                                       RothC.IOMpool[i]:12:7,' ',
                                                       RothC.CO2pool[i]:12:7,' ',
                                                       RothC.accTSMD[i]:12:7,' ',
                                                       RothC.pCO2[i]:12:7);

syswT('WEATHERING-RECORDS');
syswT('Profiles of "Gibbsite" and Ca-Mg-K-Na-Si-Fe-Al in primary minerals');
syswT('Layer  Gibbsitepool WE_Capool WE_Mgpool WE_Kpool  WE_Napool WE_Sipool WE_Fepool WE_Alpool WE_FePedogenic WE_SiPedogenic');
for i:=1 to NoLayers do
    system.writeln(fouttxt,i:5,' ',WE.Gibbsitepool[i]:10:2,' ',
                                   WE.WE_Capool[i]:10:2,' ',
                                   WE.WE_Mgpool[i]:10:2,' ',
                                   WE.WE_Kpool[i]:10:2,' ',
                                   WE.WE_Napool[i]:10:2,' ',
                                   WE.WE_Sipool[i]:10:2,' ',
                                   WE.WE_Fepool[i]:10:2,' ',
                                   WE.WE_Alpool[i]:10:2,
                                   WE.WE_FePedogenic[i]:10:2,' ',
                                   WE.WE_SiPedogenic[i]:10:2,' ');
syswT('Profiles of primary minerals');
syswT('Layer  Albite K-Feldspar Muscovite Biotite Quartz Chlorite Anorthite Forsterite '+
      'Augite Kaolinite Illite Montmorillonite Hornblende Fayalite Gibbsite Amorphite Otherite');
for i:=1 to NoLayers do
begin
  system.write(fouttxt,i:5,'  ');
  for j:=1 to 14 do
  system.write(fouttxt,WE.MinPools[j][i],' ');
  system.write(fouttxt,WE.Gibbsitepool[i],' ');
  for j:=15 to 16 do
  system.write(fouttxt,WE.MinPools[j][i],' ');
  system.writeln(fouttxt);
end;
end;
closefile(fouttxt);

end;

procedure TfrmMain.syswT(s:string);
{writes a line of input text to outputfile <fouttxt>}
begin
  system.Writeln(fouttxt,s);
end;

procedure TfrmMain.syswlI(i:integer;s:string);
{writes an integer and explanatory text to outputfile <fouttxt>}
begin
  system.Writeln(fouttxt,i:5,' ',s);
end;

procedure TfrmMain.syswlR(r:real;s:string);
{writes a real and explanatory text to outputfile <fouttxt>}
begin
    system.Writeln(fouttxt,r:15,' ',s);
end;

procedure TfrmMain.Decay;
{radioactive decay over 1 year of all 14C pools}
var i:integer; decfac:real;
begin
  decfac:=exp(-ln(2)/5730);
  for i:=1 to DataYearIn.Nolayers do
  begin
     {actual decay}
     DataYearIn.RothC.DPMXpool[i]:=decfac*DataYearIn.RothC.DPMXpool[i];
     DataYearIn.RothC.RPMXpool[i]:=decfac*DataYearIn.RothC.RPMXpool[i];
     DataYearIn.RothC.BIOXpool[i]:=decfac*DataYearIn.RothC.BIOXpool[i];
     DataYearIn.RothC.HUMXpool[i]:=decfac*DataYearIn.RothC.HUMXpool[i];
     DataYearIn.RothC.IOMXpool[i]:=decfac*DataYearIn.RothC.IOMXpool[i];
     {update output values: actual ratios 14C/C per pool}
     if (DataYearIn.RothC.DPMpool[i]=0)
     then Fk_DPM[i]:=-999 {d14C[yearBP]}
     else Fk_DPM[i]:=DataYearIn.RothC.DPMXpool[i]/DataYearIn.RothC.DPMpool[i];

     if (DataYearIn.RothC.RPMpool[i]=0)
     then Fk_RPM[i]:=-999 {d14C[yearBP]}
     else Fk_RPM[i]:=DataYearIn.RothC.RPMXpool[i]/DataYearIn.RothC.RPMpool[i];

     if (DataYearIn.RothC.BIOpool[i]=0)
     then Fk_BIO[i]:=-999 {d14C[yearBP]}
     else Fk_BIO[i]:=DataYearIn.RothC.BIOXpool[i]/DataYearIn.RothC.BIOpool[i];

     if (DataYearIn.RothC.HUMpool[i]=0)
     then Fk_HUM[i]:=-999 {d14C[yearBP]}
     else Fk_HUM[i]:=DataYearIn.RothC.HUMXpool[i]/DataYearIn.RothC.HUMpool[i];

     if (DataYearIn.RothC.IOMpool[i]=0)
     then Fk_IOM[i]:=-999 {d14C[yearBP]}
     else Fk_IOM[i]:=DataYearIn.RothC.IOMXpool[i]/DataYearIn.RothC.IOMpool[i];
  end;
  DataYearIn.RothC.ectoDPMXpool:=decfac*DataYearIn.RothC.ectoDPMXpool;
  DataYearIn.RothC.ectoRPMXpool:=decfac*DataYearIn.RothC.ectoRPMXpool;
  DataYearIn.RothC.ectoBIOXpool:=decfac*DataYearIn.RothC.ectoBIOXpool;
  DataYearIn.RothC.ectoHUMXpool:=decfac*DataYearIn.RothC.ectoHUMXpool;
  DataYearIn.RothC.ectoIOMXpool:=decfac*DataYearIn.RothC.ectoIOMXpool;
  {now copy the calculated values for DataYearIn to dataYearOut record,
   because these will be used in writetimedepth to write to the tdd-files}
   for i:=1 to DataYearIn.Nolayers do
   begin
     DataYearOut.RothC.DPMXpool[i]:=DataYearIn.RothC.DPMXpool[i];
     DataYearOut.RothC.RPMXpool[i]:=DataYearIn.RothC.RPMXpool[i];
     DataYearOut.RothC.BIOXpool[i]:=DataYearIn.RothC.BIOXpool[i];
     DataYearOut.RothC.HUMXpool[i]:=DataYearIn.RothC.HUMXpool[i];
     DataYearOut.RothC.IOMXpool[i]:=DataYearIn.RothC.IOMXpool[i];
   end;
   DataYearOut.RothC.ectoDPMXpool:=DataYearIn.RothC.ectoDPMXpool;
   DataYearOut.RothC.ectoRPMXpool:=DataYearIn.RothC.ectoRPMXpool;
   DataYearOut.RothC.ectoBIOXpool:=DataYearIn.RothC.ectoBIOXpool;
   DataYearOut.RothC.ectoHUMXpool:=DataYearIn.RothC.ectoHUMXpool;
   DataYearOut.RothC.ectoIOMXpool:=DataYearIn.RothC.ectoIOMXpool;
end;

procedure TfrmMain.WeatheringIndices(var CIA,IndexB,CIW,PIA,WI,CPA:Profile);
{calculate weathering indices at the end of the current year}
var al2o3,na2o,cao,caosilicate,k2o,mgo,dum,trbca,trbmg,trbna,trbk,valence:real;
    i:integer;
    begin
     for i:=1 to datayearin.NoLayers do
     begin
       {al2o3 revised:}
       al2o3init[i]:=(DataInit.We.WE_Alpool[i]+dataYearIn.WE.Gibbsitepool[i])*(2*26.98+3*16); {initial g Al2O3 in primary minerals PLUS initial Gibbsite}
       dum:=(datayearin.WE.Gibbsitepool[i]+dataYearIn.We.WE_Alpool[i]);   {mol Al in precipitated phase PLUS initial Al in primary minerals}
       dum:=dum*(2*26.98+3*16);                                         {convert mol to g Al2O3}
       al2o3:=dum;
       if al2o3init[i]=0 then al2o3init[i]:=1e-12;
       if al2o3=0 then al2o3:=1e-12;

       {na2o revised:}
       na2oinit[i]:=DataInit.We.WE_Napool[i]*(2*22.99+16);              {initial g Na in primary minerals [1..14]}
       dum:=(datayearin.WE.WE_Napool[i]);                               {mol Na in primary minerals [1..14]}
       dum:=dum*(2*22.99+16);                                           {convert mol to g Na2O}
       na2o:=dum;
       if na2oinit[i]=0 then na2oinit[i]:=1e-12;
       if na2o=0 then na2o:=1e-12;
       valence:=1;
       trbna:=100*(datayearin.WE.WE_Napool[i]/DataInit.DLZ)*
                  (1/DataYearIn.RHO[i])*(valence);                      {TRB from minerals, converting from mol/compartment to cmol+/kg soil}
       trbna:=trbna+0.1*DataYearIn.AXSE[3,i];                           {added: exchange phase, converting from mmol+/comp to cmol+/kg soil}
       trbna:=trbna+0.1*DatayearIn.SEA[3,i]*DatayearIn.AW[i]*valence;   {added: solute phase, converting from mmol/liter to cmol+/kg soil}
       trbna:=trbna+0.1*valence*
                    (DataYearIn.CHEMDPMpool[3,i]+DataYearIn.CHEMRPMpool[3,i]+
                     DataYearIn.CHEMBIOpool[3,i]+DataYearIn.CHEMHUMpool[3,i]+
                     DataYearIn.CHEMIOMpool[3,i])/DataInit.DLZ;         {added: in organic pools, converting from mmol/comp to cmol+/kg soil}

       {cao and caosilicate revised:}
       {caoinit[i]:=(40.08+16)*(dataYearIn.We.WE_Capool[i]+                mol Ca in primary minerals [1..14]
                                dataYearIn.calcitea[i]/(1000*(40.08+12.01+3*16))+      + in calcite...
                                dataYearIn.gypa[i]/(1000*(40.08+32.07+4*16)));          + in gypsum}
       {revised:}
       caoinit[i]:=(40.08+16)*(DataInit.We.WE_Capool[i]+                {mol Ca in primary minerals [1..14]}
                                DataInit.calcitea[i]*1000/(40.08+12.01+3*16)+      {+ in calcite...}
                                DataInit.gypa[i]*1000/(40.08+32.07+4*16));          {+ in gypsum}
       dum:=(DataYearIn.WE.WE_Capool[i]);                               {mol Ca in primary minerals [1..14]}
       dum:=dum*(40.08+16);                                             {convert mol to g CaO}
       caosilicate:=dum;
       {dum:=(40.08+16)*datayearin.calcitea[i]/(1000*(40.08+12.01+3*16));g CaO from calcite}
       {revised:}
       dum:=(40.08+16)*datayearin.calcitea[i]*1000/(40.08+12.01+3*16);  {g CaO from calcite}
       cao:=caosilicate+dum;
       {dum:=(40.08+16)*datayearin.gypa[i]/(1000*(40.08+32.07+4*16));    g CaO from gypsum}
       {revised:}
       dum:=(40.08+16)*datayearin.gypa[i]*1000/(40.08+32.07+4*16);      {g CaO from gypsum}
       cao:=cao+dum;                                                    {g CaO from all sources}
       if caoinit[i]=0 then caoinit[i]:=1e-12;
       if caosilicate=0 then caosilicate:=1e-12;
       if cao=0 then cao:=1e-12;
       valence:=2;
       trbca:=100*((cao/(40.08+16))/DataInit.DLZ)*
                  (1/DataYearIn.RHO[i])*(valence);                      {TRB from minerals, converting from mol/compartment to cmol+/kg soil}
       trbca:=trbca+0.1*DataYearIn.AXSE[1,i];                           {added: exchange phase, converting from mmol+/comp to cmol+/kg soil}
       trbca:=trbca+0.1*DatayearIn.SEA[1,i]*DatayearIn.AW[i]*valence;   {added: solute phase, converting from mmol/liter to cmol+/kg soil}
       trbca:=trbca+0.1*valence*
                    (DataYearIn.CHEMDPMpool[1,i]+DataYearIn.CHEMRPMpool[1,i]+
                     DataYearIn.CHEMBIOpool[1,i]+DataYearIn.CHEMHUMpool[1,i]+
                     DataYearIn.CHEMIOMpool[1,i])/DataInit.DLZ;         {added: in organic pools, converting from mmol/comp to cmol+/kg soil}

       {k2o revised:}
       dum:=(datayearin.WE.WE_Kpool[i]);                                {mol K in primary minerals [1..14]}
       dum:=dum*(2*39.10+16);                                           {convert mol to g K2O}
       k2o:=dum;
       if k2o=0 then k2o:=1e-12;
       valence:=1;
       trbk:=100*(datayearin.WE.WE_Kpool[i]/DataInit.DLZ)*
                  (1/DataYearIn.RHO[i])*(valence);                      {TRB from minerals, converting from mol/compartment to cmol+/kg soil}
       trbk:=trbk+0.1*DataYearIn.AXSE[4,i];                             {added: exchange phase, converting from mmol+/comp to cmol+/kg soil}
       trbk:=trbk+0.1*DatayearIn.SEA[4,i]*DatayearIn.AW[i]*valence;     {added: solute phase, converting from mmol/liter to cmol+/kg soil}
       trbk:=trbk+0.1*valence*
                    (DataYearIn.CHEMDPMpool[4,i]+DataYearIn.CHEMRPMpool[4,i]+
                     DataYearIn.CHEMBIOpool[4,i]+DataYearIn.CHEMHUMpool[4,i]+
                     DataYearIn.CHEMIOMpool[4,i])/DataInit.DLZ;         {added: in organic pools, converting from mmol/comp to cmol+/kg soil}

       {mgo revised:}
       dum:=(datayearin.WE.WE_Mgpool[i]);                               {mol Mg in primary minerals [1..14]} ;
       dum:=dum*(24.31+16);                                             {convert mol to g MgO}
       mgo:=dum;
       if mgo=0 then mgo:=1e-12;
       valence:=2;
       trbmg:=100*(datayearin.WE.WE_Mgpool[i]/DataInit.DLZ)*
                  (1/DataYearIn.RHO[i])*(valence);                      {TRB from minerals, converting from mol/compartment to cmol+/kg soil}
       trbmg:=trbmg+0.1*DataYearIn.AXSE[2,i];                           {added: exchange phase, converting from mmol+/comp to cmol+/kg soil}
       trbmg:=trbmg+0.1*DatayearIn.SEA[2,i]*DatayearIn.AW[i]*valence;   {added: solute phase, converting from mmol/liter to cmol+/kg soil}
       trbmg:=trbmg+0.1*valence*
                    (DataYearIn.CHEMDPMpool[2,i]+DataYearIn.CHEMRPMpool[2,i]+
                     DataYearIn.CHEMBIOpool[2,i]+DataYearIn.CHEMHUMpool[2,i]+
                     DataYearIn.CHEMIOMpool[2,i])/DataInit.DLZ;         {added: in organic pools, converting from mmol/comp to cmol+/kg soil}

       {now calculate the weathering indices}
       CIA[i]:=100*al2o3/(al2o3+na2o+cao+k2o);
       IndexB[i]:=(caosilicate+na2o+k2o)/(al2o3+caosilicate+na2o+k2o);
       CIW[i]:=100*al2o3/(al2o3+na2o+cao);
       PIA[i]:=100*(al2o3-k2o)/(al2o3+cao+na2o-k2o);
       WI[i]:=(al2o3/(cao+na2o))/(al2o3init[i]/(caoinit[i]+na2oinit[i]));
       CPA[i]:=100*(al2o3)/(al2o3+na2o);
       {also calculate Total Reserve of Bases}
       {equal to Ca, K, Na and Mg in minerals, exchange phase, solution, in OM}
       {expressed in cmolc kg-1 soil}
       TRB[i]:=trbca+trbna+trbk+trbmg;

     end;
    end;

Procedure TfrmMain.KeepSelectedData;
{for continuation run: store some variables
 that may override values from continue.rec by values from soil and turnover input files}
var i:integer;
begin
    for i:=1 to 5 do keep.GENpoolpercs[i]:=DataInit.RothCGen.GENpoolpercs[i];
    keep.GENgrasRDEPTH:=DataInit.RothCGen.GENgrasRDEPTH;
    keep.GENdeciRDEPTH:=DataInit.RothCGen.GENdeciRDEPTH;
    keep.GENconiRDEPTH:=DataInit.RothCGen.GENconiRDEPTH;
    keep.GENagriRDEPTH:=DataInit.RothCGen.GENagriRDEPTH;
    keep.NCROP:=DataInit.NCROP;
    keep.Icult:=DataInit.Icult;
    for i:=1 to 12 do
    begin
        keep.GENgrasPlant[i]:=DataInit.RothCGen.GENgrasPlant[i];
        keep.GENgrasMan[i]:=DataInit.RothCGen.GENgrasMan[i];
        keep.GENdeciPlant[i]:=DataInit.RothCGen.GENdeciPlant[i];
        keep.GENdeciMan[i]:=DataInit.RothCGen.GENdeciMan[i];
        keep.GENconiPlant[i]:=DataInit.RothCGen.GENconiPlant[i];
        keep.GENconiMan[i]:=DataInit.RothCGen.GENconiMan[i];
        keep.GENresiPlant[i]:=DataInit.RothCGen.GENresiPlant[i];
        keep.GENresiMan[i]:=DataInit.RothCGen.GENresiMan[i];
        keep.GENagriPlant[i]:=DataInit.RothCGen.GENagriPlant[i];
        keep.GENagriMan[i]:=DataInit.RothCGen.GENagriMan[i];
    end;
    for i:=1 to keep.NCROP do
    begin
        keep.igrowth[i]:=DataInit.igrowth[i];
        keep.icut[i]:=DataInit.icut[i];
        keep.rtype[i]:=DataInit.rtype[i];
        keep.PDAY[i]:=DataInit.PDAY[i];
        keep.ESTART[i]:=DataInit.ESTART[i];
        keep.RDFDAY[i]:=DataInit.RDFDAY[i];
        keep.ESTOP[i]:=DataInit.ESTOP[i];
        keep.DAYHAR[i]:=DataInit.DAYHAR[i];
        keep.RDEP[i]:=DataInit.RDEP[i];
        keep.AK1[i]:=DataInit.AK1[i];
        keep.PANFAC[i]:=DataInit.PANFAC[i];
    end;
    for i:=1 to keep.Icult do
    begin
       keep.Dcult[i]:=DataInit.Dcult[i];
       keep.cultdep[i]:=DataInit.Cultdep[i];
    end;
end;

Procedure TfrmMain.OverrideSelectedData;
{for continuation run: override the records for the data
 that must be kept from the Read Default Inputfiles}
var i:integer;
begin
  {DataYearIn}
  for i:=1 to 5 do DataYearIn.RothCGen.GENpoolpercs[i]:=keep.GENpoolpercs[i];
  DataYearIn.RothCGen.GENgrasRDEPTH:=keep.GENgrasRDEPTH;
  DataYearIn.RothCGen.GENdeciRDEPTH:=keep.GENdeciRDEPTH;
  DataYearIn.RothCGen.GENconiRDEPTH:=keep.GENconiRDEPTH;
  DataYearIn.RothCGen.GENagriRDEPTH:=keep.GENagriRDEPTH;
  for i:=1 to 12 do
  begin
    DataYearIn.RothCGen.GENresiPlant[i]:=keep.GENresiPlant[i];
    DataYearIn.RothCGen.GENresiMan[i]:=keep.GENresiMan[i];
    DataYearIn.RothCGen.GENagriPlant[i]:=keep.GENagriPlant[i];
    DataYearIn.RothCGen.GENagriMan[i]:=keep.GENagriMan[i];
  end;
  DataYearIn.NCROP:=keep.NCROP;  DataYearIn.Icult:=keep.Icult;
  for i:=1 to keep.NCROP {MaxNoCrops} do
  begin
    DataYearIn.igrowth[i]:=keep.igrowth[i];
    DataYearIn.icut[i]:=keep.icut[i];
    DataYearIn.rtype[i]:=keep.rtype[i];
    DataYearIn.PDAY[i]:=keep.PDAY[i];
    DataYearIn.ESTART[i]:=keep.ESTART[i];
    DataYearIn.RDFDAY[i]:=keep.RDFDAY[i];
    DataYearIn.ESTOP[i]:=keep.ESTOP[i];
    DataYearIn.DAYHAR[i]:=keep.DAYHAR[i];
    DataYearIn.RDEP[i]:=keep.RDEP[i];
    DataYearIn.AK1[i]:=keep.AK1[i];
    DataYearIn.PANFAC[i]:=keep.PANFAC[i];
  end;
  for i:=1 to keep.Icult do
  begin
     DataYearIn.Dcult[i]:=keep.Dcult[i];
     DataYearIn.Cultdep[i]:=keep.cultdep[i];
  end;

  {DataYearOut}
  for i:=1 to 5 do DataYearOut.RothCGen.GENpoolpercs[i]:=keep.GENpoolpercs[i];
  DataYearOut.RothCGen.GENgrasRDEPTH:=keep.GENgrasRDEPTH;
  DataYearOut.RothCGen.GENdeciRDEPTH:=keep.GENdeciRDEPTH;
  DataYearOut.RothCGen.GENconiRDEPTH:=keep.GENconiRDEPTH;
  DataYearOut.RothCGen.GENagriRDEPTH:=keep.GENagriRDEPTH;
  for i:=1 to 12 do
  begin
    DataYearOut.RothCGen.GENresiPlant[i]:=keep.GENresiPlant[i];
    DataYearOut.RothCGen.GENresiMan[i]:=keep.GENresiMan[i];
    DataYearOut.RothCGen.GENagriPlant[i]:=keep.GENagriPlant[i];
    DataYearOut.RothCGen.GENagriMan[i]:=keep.GENagriMan[i];
  end;
  DataYearOut.NCROP:=keep.NCROP; DataYearOut.Icult:=keep.Icult;
  for i:=1 to keep.NCROP {MaxNoCrops} do
  begin
    DataYearout.igrowth[i]:=keep.igrowth[i];
    DataYearOut.icut[i]:=keep.icut[i];
    DataYearOut.rtype[i]:=keep.rtype[i];
    DataYearOut.PDAY[i]:=keep.PDAY[i];
    DataYearOut.ESTART[i]:=keep.ESTART[i];
    DataYearOut.RDFDAY[i]:=keep.RDFDAY[i];
    DataYearOut.ESTOP[i]:=keep.ESTOP[i];
    DataYearOut.DAYHAR[i]:=keep.DAYHAR[i];
    DataYearOut.RDEP[i]:=keep.RDEP[i];
    DataYearOut.AK1[i]:=keep.AK1[i];
    DataYearOut.PANFAC[i]:=keep.PANFAC[i];
  end;
  for i:=1 to keep.Icult do
  begin
     DataYearOut.Dcult[i]:=keep.Dcult[i];
     DataYearOut.Cultdep[i]:=keep.cultdep[i];
  end;

  {DataInit}
  for i:=1 to 5 do DataInit.RothCGen.GENpoolpercs[i]:=keep.GENpoolpercs[i];
  DataInit.RothCGen.GENgrasRDEPTH:=keep.GENgrasRDEPTH;
  DataInit.RothCGen.GENdeciRDEPTH:=keep.GENdeciRDEPTH;
  DataInit.RothCGen.GENconiRDEPTH:=keep.GENconiRDEPTH;
  DataInit.RothCGen.GENagriRDEPTH:=keep.GENagriRDEPTH;
  for i:=1 to 12 do
  begin
    DataInit.RothCGen.GENresiPlant[i]:=keep.GENresiPlant[i];
    DataInit.RothCGen.GENresiMan[i]:=keep.GENresiMan[i];
    DataInit.RothCGen.GENagriPlant[i]:=keep.GENagriPlant[i];
    DataInit.RothCGen.GENagriMan[i]:=keep.GENagriMan[i];
  end;
  DataInit.NCROP:=keep.NCROP; DataInit.Icult:=keep.Icult;
  for i:=1 to keep.NCROP {MaxNoCrops} do
  begin
    DataInit.igrowth[i]:=keep.igrowth[i];
    DataInit.icut[i]:=keep.icut[i];
    DataInit.rtype[i]:=keep.rtype[i];
    DataInit.PDAY[i]:=keep.PDAY[i];
    DataInit.ESTART[i]:=keep.ESTART[i];
    DataInit.RDFDAY[i]:=keep.RDFDAY[i];
    DataInit.ESTOP[i]:=keep.ESTOP[i];
    DataInit.DAYHAR[i]:=keep.DAYHAR[i];
    DataInit.RDEP[i]:=keep.RDEP[i];
    DataInit.AK1[i]:=keep.AK1[i];
    DataInit.PANFAC[i]:=keep.PANFAC[i];
  end;
  for i:=1 to keep.Icult do
  begin
     DataInit.Dcult[i]:=keep.Dcult[i];
     DataInit.Cultdep[i]:=keep.cultdep[i];
  end;

  {OnDisplay}
  for i:=1 to 5 do OnDisplay.RothCGen.GENpoolpercs[i]:=keep.GENpoolpercs[i];
  OnDisplay.RothCGen.GENgrasRDEPTH:=keep.GENgrasRDEPTH;
  OnDisplay.RothCGen.GENdeciRDEPTH:=keep.GENdeciRDEPTH;
  OnDisplay.RothCGen.GENconiRDEPTH:=keep.GENconiRDEPTH;
  OnDisplay.RothCGen.GENagriRDEPTH:=keep.GENagriRDEPTH;
  for i:=1 to 12 do
  begin
    OnDisplay.RothCGen.GENresiPlant[i]:=keep.GENresiPlant[i];
    OnDisplay.RothCGen.GENresiMan[i]:=keep.GENresiMan[i];
    OnDisplay.RothCGen.GENagriPlant[i]:=keep.GENagriPlant[i];
    OnDisplay.RothCGen.GENagriMan[i]:=keep.GENagriMan[i];
  end;
  OnDisplay.NCROP:=keep.NCROP; OnDisplay.Icult:=keep.Icult;
  for i:=1 to keep.NCROP {MaxNoCrops} do
  begin
    OnDisplay.igrowth[i]:=keep.igrowth[i];
    OnDisplay.icut[i]:=keep.icut[i];
    OnDisplay.rtype[i]:=keep.rtype[i];
    OnDisplay.PDAY[i]:=keep.PDAY[i];
    OnDisplay.ESTART[i]:=keep.ESTART[i];
    OnDisplay.RDFDAY[i]:=keep.RDFDAY[i];
    OnDisplay.ESTOP[i]:=keep.ESTOP[i];
    OnDisplay.DAYHAR[i]:=keep.DAYHAR[i];
    OnDisplay.RDEP[i]:=keep.RDEP[i];
    OnDisplay.AK1[i]:=keep.AK1[i];
    OnDisplay.PANFAC[i]:=keep.PANFAC[i];
  end;
  for i:=1 to keep.Icult do
  begin
     OnDisplay.Dcult[i]:=keep.Dcult[i];
     OnDisplay.Cultdep[i]:=keep.cultdep[i];
  end;
end;

procedure TfrmMain.EmulateTurningPlow(plowedcomp:integer);
{mimick a turning plow by inverting the layers, repeated for all properties, over the plowed depth}
var i,j,newcompno,oldmaxcomp:integer; Soilprops: Profile;
  dumPW_S,dumPW_K:array[-4..11,1..MaxLayers] of INT64; {physical weathering: parameters per size class+depth class}
  dumPW_G:array[-4..11,1..MaxLayers] of real;          {physical weathering: parameters per size class}
begin
 {If GUItrace=1 then wwwriteln} singletracewriteln('Entering EmulateTurningPlow');
{RothC Record as part of DataYearIn and DataYearOut records}
 for i:=1 to 12 do  {monthly values}
 begin
   Turnplow1profile(DataYearIn.RothC.Temperatures[i],plowedcomp);
   Turnplow1profile(DataYearOut.RothC.Temperatures[i],plowedcomp);
   Turnplow1profile(DataYearIn.RothC.PlantresiduesIN[i],plowedcomp);
   Turnplow1profile(DataYearOut.RothC.PlantresiduesIN[i],plowedcomp);
   Turnplow1profile(DataYearIn.RothC.ManureIN[i],plowedcomp);
   Turnplow1profile(DataYearOut.RothC.ManureIN[i],plowedcomp);
 end;
 Turnplow1profile(DataYearIn.RothC.ClayPerc,plowedcomp);      Turnplow1profile(DataYearOut.RothC.ClayPerc,plowedcomp);
 Turnplow1profile(DataYearIn.RothC.fracManureIN,plowedcomp);  Turnplow1profile(DataYearOut.RothC.fracManureIN,plowedcomp);
 Turnplow1profile(DataYearIn.RothC.fracPlantIN,plowedcomp);   Turnplow1profile(DataYearOut.RothC.fracPlantIN,plowedcomp);
 Turnplow1profile(DataYearIn.RothC.DPMpool,plowedcomp);       Turnplow1profile(DataYearOut.RothC.DPMpool,plowedcomp);
 Turnplow1profile(DataYearIn.RothC.RPMpool,plowedcomp);       Turnplow1profile(DataYearOut.RothC.RPMpool,plowedcomp);
 Turnplow1profile(DataYearIn.RothC.BIOpool,plowedcomp);       Turnplow1profile(DataYearOut.RothC.BIOpool,plowedcomp);
 Turnplow1profile(DataYearIn.RothC.HUMpool,plowedcomp);       Turnplow1profile(DataYearOut.RothC.HUMpool,plowedcomp);
 Turnplow1profile(DataYearIn.RothC.IOMpool,plowedcomp);       Turnplow1profile(DataYearOut.RothC.IOMpool,plowedcomp);
 Turnplow1profile(DataYearIn.RothC.CO2pool,plowedcomp);       Turnplow1profile(DataYearOut.RothC.CO2pool,plowedcomp);
 Turnplow1profile(DataYearIn.RothC.DPMXpool,plowedcomp);      Turnplow1profile(DataYearOut.RothC.DPMXpool,plowedcomp);
 Turnplow1profile(DataYearIn.RothC.RPMXpool,plowedcomp);      Turnplow1profile(DataYearOut.RothC.RPMXpool,plowedcomp);
 Turnplow1profile(DataYearIn.RothC.BIOXpool,plowedcomp);      Turnplow1profile(DataYearOut.RothC.BIOXpool,plowedcomp);
 Turnplow1profile(DataYearIn.RothC.HUMXpool,plowedcomp);      Turnplow1profile(DataYearOut.RothC.HUMXpool,plowedcomp);
 Turnplow1profile(DataYearIn.RothC.IOMXpool,plowedcomp);      Turnplow1profile(DataYearOut.RothC.IOMXpool,plowedcomp);
 Turnplow1profile(DataYearIn.RothC.CO2Xpool,plowedcomp);      Turnplow1profile(DataYearOut.RothC.CO2Xpool,plowedcomp);
 Turnplow1profile(DataYearIn.RothC.accTSMD,plowedcomp);       Turnplow1profile(DataYearOut.RothC.accTSMD,plowedcomp);
 Turnplow1profile(DataYearIn.RothC.pCO2,plowedcomp);          Turnplow1profile(DataYearOut.RothC.pCO2,plowedcomp);
 for i:=1 to MaxChem do  {shift all ions associated with OC, except those of ectorganic layer (j=0)}
 begin
   for j:=1 to MaxLayers do SoilProps[j]:=DataYearIn.CHEMDPMpool[i][j];
   Turnplow1profile(SoilProps,plowedcomp);
   for j:=1 to MaxLayers do DataYearIn.CHEMDPMpool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataYearIn.CHEMRPMpool[i][j];
   Turnplow1profile(SoilProps,plowedcomp);
   for j:=1 to MaxLayers do DataYearIn.CHEMRPMpool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataYearIn.CHEMBIOpool[i][j];
   Turnplow1profile(SoilProps,plowedcomp);
   for j:=1 to MaxLayers do DataYearIn.CHEMBIOpool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataYearIn.CHEMHUMpool[i][j];
   Turnplow1profile(SoilProps,plowedcomp);
   for j:=1 to MaxLayers do DataYearIn.CHEMHUMpool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataYearIn.CHEMIOMpool[i][j];
   Turnplow1profile(SoilProps,plowedcomp);
   for j:=1 to MaxLayers do DataYearIn.CHEMIOMpool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataYearIn.CHEMCO2pool[i][j];
   Turnplow1profile(SoilProps,plowedcomp);
   for j:=1 to MaxLayers do DataYearOut.CHEMCO2pool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataYearOut.CHEMDPMpool[i][j];
   Turnplow1profile(SoilProps,plowedcomp);
   for j:=1 to MaxLayers do DataYearOut.CHEMDPMpool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataYearOut.CHEMRPMpool[i][j];
   Turnplow1profile(SoilProps,plowedcomp);
   for j:=1 to MaxLayers do DataYearOut.CHEMRPMpool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataYearOut.CHEMBIOpool[i][j];
   Turnplow1profile(SoilProps,plowedcomp);
   for j:=1 to MaxLayers do DataYearOut.CHEMBIOpool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataYearOut.CHEMHUMpool[i][j];
   Turnplow1profile(SoilProps,plowedcomp);
   for j:=1 to MaxLayers do DataYearOut.CHEMHUMpool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataYearOut.CHEMIOMpool[i][j];
   Turnplow1profile(SoilProps,plowedcomp);
   for j:=1 to MaxLayers do DataYearOut.CHEMIOMpool[i][j]:=SoilProps[j];
   for j:=1 to MaxLayers do SoilProps[j]:=DataYearOut.CHEMCO2pool[i][j];
   Turnplow1profile(SoilProps,plowedcomp);
   for j:=1 to MaxLayers do DataYearOut.CHEMCO2pool[i][j]:=SoilProps[j];
 end;

 {weathering Record as part of datayearIn and dataYearOut records}
 Turnplow1profile(DataYearIn.WE.Gibbsitepool,plowedcomp);     Turnplow1profile(DataYearOut.WE.Gibbsitepool,plowedcomp);
 Turnplow1profile(DataYearIn.WE.WE_Capool,plowedcomp);        Turnplow1profile(DataYearOut.WE.WE_Capool,plowedcomp);
 Turnplow1profile(DataYearIn.WE.WE_Mgpool,plowedcomp);        Turnplow1profile(DataYearOut.WE.WE_Mgpool,plowedcomp);
 Turnplow1profile(DataYearIn.WE.WE_Kpool,plowedcomp);         Turnplow1profile(DataYearOut.WE.WE_Kpool,plowedcomp);
 Turnplow1profile(DataYearIn.WE.WE_Napool,plowedcomp);        Turnplow1profile(DataYearOut.WE.WE_Napool,plowedcomp);
 Turnplow1profile(DataYearIn.WE.WE_Sipool,plowedcomp);        Turnplow1profile(DataYearOut.WE.WE_Sipool,plowedcomp);
 Turnplow1profile(DataYearIn.WE.WE_Fepool,plowedcomp);        Turnplow1profile(DataYearOut.WE.WE_Fepool,plowedcomp);
 Turnplow1profile(DataYearIn.WE.WE_Alpool,plowedcomp);        Turnplow1profile(DataYearOut.WE.WE_Alpool,plowedcomp);
 Turnplow1profile(DataYearIn.WE.WE_FePedogenic,plowedcomp);   Turnplow1profile(DataYearOut.WE.WE_FePedogenic,plowedcomp);
 Turnplow1profile(DataYearIn.WE.WE_SiPedogenic,plowedcomp);   Turnplow1profile(DataYearOut.WE.WE_SiPedogenic,plowedcomp);
 for j:=1 to nrminerals do
 begin
    Turnplow1profile(DataYearIn.WE.MinPools[j],plowedcomp);   Turnplow1profile(DataYearOut.WE.MinPools[j],plowedcomp);
 end;

 {data record}
 {DataYearIn}
 begin
    Turnplow1profile(DataYearIn.CLAY,plowedcomp);   Turnplow1profile(DataYearIn.SILT,plowedcomp);        Turnplow1profile(DataYearIn.OC,plowedcomp);
    Turnplow1profile(DataYearIn.SAND,plowedcomp);   Turnplow1profile(DataYearIn.GRAVEL,plowedcomp);      Turnplow1profile(DataYearIn.STONES,plowedcomp);
    Turnplow1profile(DataYearIn.SANDm,plowedcomp);  Turnplow1profile(DataYearIn.fineSANDm,plowedcomp);
    Turnplow1profile(DataYearIn.CLAYm,plowedcomp);  Turnplow1profile(DataYearIn.SILTm,plowedcomp);       Turnplow1profile(DataYearIn.OCm,plowedcomp);
    Turnplow1profile(DataYearIn.AW,plowedcomp);     Turnplow1profile(DataYearIn.AH,plowedcomp);          Turnplow1profile(DataYearIn.RDF,plowedcomp);
    Turnplow1profile(DataYearIn.ATEM,plowedcomp);   Turnplow1profile(DataYearIn.TEM,plowedcomp);
    Turnplow1profile(DataYearIn.AAEV,plowedcomp);   Turnplow1profile(DataYearIn.ABCAM,plowedcomp);
    Turnplow1profile(DataYearIn.RHO,plowedcomp);    Turnplow1profile(DataYearIn.CMATCH,plowedcomp);      Turnplow1profile(DataYearIn.HMATCH,plowedcomp);
    Turnplow1profile(DataYearIn.RP,plowedcomp);     Turnplow1profile(DataYearIn.LAMBDA,plowedcomp);      Turnplow1profile(DataYearIn.APH,plowedcomp);
    Turnplow1profile(DataYearIn.W,plowedcomp);      Turnplow1profile(DataYearIn.fc,plowedcomp);          Turnplow1profile(DataYearIn.d,plowedcomp);
    Turnplow1profile(DataYearIn.CHARGE,plowedcomp); Turnplow1profile(DataYearIn.SC1,plowedcomp);
    Turnplow1profile(DataYearIn.SC2,plowedcomp);    Turnplow1profile(DataYearIn.SC3,plowedcomp);         Turnplow1profile(DataYearIn.SC4,plowedcomp);
    Turnplow1profile(DataYearIn.SC5,plowedcomp);    Turnplow1profile(DataYearIn.SC6,plowedcomp);         Turnplow1profile(DataYearIn.SC7,plowedcomp);
    Turnplow1profile(DataYearIn.SC8,plowedcomp);    Turnplow1profile(DataYearIn.SC9,plowedcomp);         Turnplow1profile(DataYearIn.SC10,plowedcomp);
    Turnplow1profile(DataYearIn.SC11,plowedcomp);   Turnplow1profile(DataYearIn.SC12,plowedcomp);        Turnplow1profile(DataYearIn.SC13,plowedcomp);
    Turnplow1profile(DataYearIn.SC14,plowedcomp);   Turnplow1profile(DataYearIn.SC15,plowedcomp);
    Turnplow1profile(DataYearIn.APCO2,plowedcomp);  Turnplow1profile(DataYearIn.oldCALCITEA,plowedcomp); Turnplow1profile(DataYearIn.CALCITEA,plowedcomp);
    Turnplow1profile(DataYearIn.oldGYPA,plowedcomp);Turnplow1profile(DataYearIn.GYPA,plowedcomp);
    Turnplow1profile(DataYearIn.ACEC,plowedcomp);   Turnplow1profile(DataYearIn.alkalinity,plowedcomp);
    Turnplow1profile(DataYearIn.CEC_empirical,plowedcomp);
    Turnplow1profile(DataYearIn.FacCEC,plowedcomp);
    Turnplow1profile(DataYearIn.FacCEC_OC,plowedcomp);
    Turnplow1profile(DataYearIn.ACEC_clay,plowedcomp);
    Turnplow1profile(DataYearIn.ACEC_OC,plowedcomp);
    Turnplow1profile(DataYearIn.ClayFrCEC,plowedcomp);
    Turnplow1profile(DataYearIn.OCFrCEC,plowedcomp);
    Turnplow1profile(DataYearIn.BS,plowedcomp);     Turnplow1profile(DataYearIn.CO3,plowedcomp);         Turnplow1profile(DataYearIn.HCO3,plowedcomp);
    Turnplow1profile(DataYearIn.EC,plowedcomp);     Turnplow1profile(DataYearIn.SAR,plowedcomp);         Turnplow1profile(DataYearIn.ESP,plowedcomp);
    Turnplow1profile(DataYearIn.ET,plowedcomp);     Turnplow1profile(DataYearIn.PH,plowedcomp);          Turnplow1profile(DataYearIn.FLUX,plowedcomp);
    Turnplow1profile(DataYearIn.DISSOLVEDCATEQ,plowedcomp);  Turnplow1profile(DataYearIn.DELGIBB,plowedcomp);  Turnplow1profile(DataYearIn.DELGYP,plowedcomp);
    Turnplow1profile(DataYearIn.DECALCITE,plowedcomp);  Turnplow1profile(DataYearIn.OSPOT,plowedcomp);   Turnplow1profile(DataYearIn.delco2,plowedcomp);
    Turnplow1profile(DataYearIn.parden,plowedcomp); Turnplow1profile(DataYearIn.porosity,plowedcomp);    Turnplow1profile(DataYearIn.ClayDispInd,plowedcomp);
    Turnplow1profile(DataYearIn.TEMPGEM,plowedcomp);     Turnplow1profile(DataYearIn.klei2,plowedcomp);
    Turnplow1profile(DataYearIn.silt2,plowedcomp);  Turnplow1profile(DataYearIn.zand2,plowedcomp);       Turnplow1profile(DataYearIn.volpercclay,plowedcomp);
    Turnplow1profile(DataYearIn.DOC,plowedcomp);    Turnplow1profile(DataYearIn.DOCperc,plowedcomp);
    Turnplow1profile(DataYearIn.SUM21,plowedcomp);  Turnplow1profile(DataYearIn.MM21,plowedcomp);
    for i:=1 to MaxChem do
    begin
      Turnplow1profile(DataYearIn.SEA[i],plowedcomp); Turnplow1profile(DataYearIn.AXSE[i],plowedcomp);
      Turnplow1profile(DataYearIn.AXSE_oc[i],plowedcomp);Turnplow1profile(DataYearIn.AXSE_clay[i],plowedcomp);
    end;
 end;
 {DataYearOut}
 begin
    Turnplow1profile(DataYearOut.CLAY,plowedcomp);   Turnplow1profile(DataYearOut.SILT,plowedcomp);        Turnplow1profile(DataYearOut.OC,plowedcomp);
    Turnplow1profile(DataYearOut.SAND,plowedcomp);   Turnplow1profile(DataYearOut.GRAVEL,plowedcomp);      Turnplow1profile(DataYearOut.STONES,plowedcomp);
    Turnplow1profile(DataYearOut.SANDm,plowedcomp);  Turnplow1profile(DataYearIn.fineSANDm,plowedcomp);
    Turnplow1profile(DataYearOut.CLAYm,plowedcomp);  Turnplow1profile(DataYearOut.SILTm,plowedcomp);       Turnplow1profile(DataYearOut.OCm,plowedcomp);
    Turnplow1profile(DataYearOut.AW,plowedcomp);     Turnplow1profile(DataYearOut.AH,plowedcomp);          Turnplow1profile(DataYearOut.RDF,plowedcomp);
    Turnplow1profile(DataYearOut.ATEM,plowedcomp);   Turnplow1profile(DataYearOut.TEM,plowedcomp);
    Turnplow1profile(DataYearOut.AAEV,plowedcomp);   Turnplow1profile(DataYearOut.ABCAM,plowedcomp);
    Turnplow1profile(DataYearOut.RHO,plowedcomp);    Turnplow1profile(DataYearOut.CMATCH,plowedcomp);      Turnplow1profile(DataYearOut.HMATCH,plowedcomp);
    Turnplow1profile(DataYearOut.RP,plowedcomp);     Turnplow1profile(DataYearOut.LAMBDA,plowedcomp);      Turnplow1profile(DataYearOut.APH,plowedcomp);
    Turnplow1profile(DataYearOut.W,plowedcomp);      Turnplow1profile(DataYearOut.fc,plowedcomp);          Turnplow1profile(DataYearOut.d,plowedcomp);
    Turnplow1profile(DataYearOut.CHARGE,plowedcomp); Turnplow1profile(DataYearOut.SC1,plowedcomp);
    Turnplow1profile(DataYearOut.SC2,plowedcomp);    Turnplow1profile(DataYearOut.SC3,plowedcomp);         Turnplow1profile(DataYearOut.SC4,plowedcomp);
    Turnplow1profile(DataYearOut.SC5,plowedcomp);    Turnplow1profile(DataYearOut.SC6,plowedcomp);         Turnplow1profile(DataYearOut.SC7,plowedcomp);
    Turnplow1profile(DataYearOut.SC8,plowedcomp);    Turnplow1profile(DataYearOut.SC9,plowedcomp);         Turnplow1profile(DataYearOut.SC10,plowedcomp);
    Turnplow1profile(DataYearOut.SC11,plowedcomp);   Turnplow1profile(DataYearOut.SC12,plowedcomp);        Turnplow1profile(DataYearOut.SC13,plowedcomp);
    Turnplow1profile(DataYearOut.SC14,plowedcomp);   Turnplow1profile(DataYearOut.SC15,plowedcomp);
    Turnplow1profile(DataYearOut.APCO2,plowedcomp);  Turnplow1profile(DataYearOut.oldCALCITEA,plowedcomp); Turnplow1profile(DataYearOut.CALCITEA,plowedcomp);
    Turnplow1profile(DataYearOut.oldGYPA,plowedcomp);Turnplow1profile(DataYearOut.GYPA,plowedcomp);
    Turnplow1profile(DataYearOut.ACEC,plowedcomp);   Turnplow1profile(DataYearOut.alkalinity,plowedcomp);
    Turnplow1profile(DataYearOut.CEC_empirical,plowedcomp);
    Turnplow1profile(DataYearOut.FacCEC,plowedcomp);
    Turnplow1profile(DataYearOut.FacCEC_OC,plowedcomp);
    Turnplow1profile(DataYearOut.ACEC_clay,plowedcomp);
    Turnplow1profile(DataYearOut.ACEC_OC,plowedcomp);
    Turnplow1profile(DataYearOut.ClayFrCEC,plowedcomp);
    Turnplow1profile(DataYearOut.OCFrCEC,plowedcomp);
    Turnplow1profile(DataYearOut.BS,plowedcomp);     Turnplow1profile(DataYearOut.CO3,plowedcomp);         Turnplow1profile(DataYearOut.HCO3,plowedcomp);
    Turnplow1profile(DataYearOut.EC,plowedcomp);     Turnplow1profile(DataYearOut.SAR,plowedcomp);         Turnplow1profile(DataYearOut.ESP,plowedcomp);
    Turnplow1profile(DataYearOut.ET,plowedcomp);     Turnplow1profile(DataYearOut.PH,plowedcomp);          Turnplow1profile(DataYearOut.FLUX,plowedcomp);
    Turnplow1profile(DataYearOut.DISSOLVEDCATEQ,plowedcomp);  Turnplow1profile(DataYearOut.DELGIBB,plowedcomp);  Turnplow1profile(DataYearOut.DELGYP,plowedcomp);
    Turnplow1profile(DataYearOut.DECALCITE,plowedcomp);  Turnplow1profile(DataYearOut.OSPOT,plowedcomp);   Turnplow1profile(DataYearOut.delco2,plowedcomp);
    Turnplow1profile(DataYearOut.parden,plowedcomp); Turnplow1profile(DataYearOut.porosity,plowedcomp);    Turnplow1profile(DataYearOut.ClayDispInd,plowedcomp);
    Turnplow1profile(DataYearOut.TEMPGEM,plowedcomp);     Turnplow1profile(DataYearOut.klei2,plowedcomp);
    Turnplow1profile(DataYearOut.silt2,plowedcomp);  Turnplow1profile(DataYearOut.zand2,plowedcomp);       Turnplow1profile(DataYearOut.volpercclay,plowedcomp);
    Turnplow1profile(DataYearOut.DOC,plowedcomp);    Turnplow1profile(DataYearOut.DOCperc,plowedcomp);
    Turnplow1profile(DataYearOut.SUM21,plowedcomp);  Turnplow1profile(DataYearOut.MM21,plowedcomp);
    for i:=1 to MaxChem do
    begin
      Turnplow1profile(DataYearOut.SEA[i],plowedcomp); Turnplow1profile(DataYearOut.AXSE[i],plowedcomp);
      Turnplow1profile(DataYearOut.AXSE_oc[i],plowedcomp);Turnplow1profile(DataYearOut.AXSE_clay[i],plowedcomp);
    end;
 end;
 if physweathallowed then
 Begin
 {Physical weathering: Turnplow1profile cannot be used because not of type "profile"}
 {first copy the original data}
  for i:=-4 to 11 do for j:=1 to MaxLayers do
  begin
    dumPW_S[i][j]:=PW_S[i][j];
    dumPW_K[i][j]:=PW_K[i][j];
    dumPW_G[i][j]:=PW_G[i][j];
  end;
  for i:=-4 to 11 do
  begin
    for j:=1 to plowedcomp do PW_S[i][j]:=dumPW_S[i][plowedcomp-j+1];
    for j:=1 to plowedcomp do PW_K[i][j]:=dumPW_K[i][plowedcomp-j+1];
    for j:=1 to plowedcomp do PW_G[i][j]:=dumPW_G[i][plowedcomp-j+1];
  end
 End;
 {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving EmulateTurningPlow');
end;


procedure TfrmMain.Turnplow1profile(var soilprop:Profile; plowedcomp:integer);
{mimick a turning plow by inverting the layers for 1 soil property}
{resequences the soil compartment in the plow layer.
 the deepest compartment involved becomes the top compartment,
 the second deepest the second from the top, etc}
Var dummyprof:Profile; var i:integer;
begin
  dummyprof:=soilprop; {put the input profile in a dummy variable}
  for i:=1 to plowedcomp do soilprop[i]:=dummyprof[plowedcomp-i+1];
end;

procedure TfrmMAIN.WriteFileHeaderDaily(var ff:textfile; txt:string);
{write headers of yearly time-depth diagrams with info on time-depth gridcell sizes}
begin
    system.Writeln(ff,MaxPlusCompartments:6,' (ncols, no. compartments) UNITS='+txt);
    system.Writeln(ff,'   365 (nrows, no. days)');
    system.Writeln(ff,DataInit.DLZ:6:0,' (colsize [mm per compartments])');
    system.Flush(ff);
end;

procedure TfrmMAIN.AssignDailyTDD(CurrentYear:integer);
{open (max 6 selected) daily time-depth diagrams}
var img,i:integer;
    txt,tddname,filename,eenheden:string;
begin
  {If GUItrace=1 then wwwriteln} singletracewriteln('Entering AssignDailyTDD (header of file with daily diagrams)');
  str(CurrentYear,txt);
  for img:=1 to lstbxScreenVariables.Items.Count do
  begin
    for i:=1 to maxTDD do
    if lstbxScreenVariables.Items[img-1]=cmbbxScreenVariables.Items[i-1]
    then
    begin
      filename:=cmbbxScreenVariables.Items[i-1];
      tddname:=txt+'_'+filename+'.tdd';
      while(pos(' ',tddname)>0) do delete(tddname,pos(' ',tddname),1);
      eenheden:=copy(filename,pos('(',filename),length(filename)+1-pos('(',filename));
      if img=1 then begin system.assign(fdaily1,tddname); system.rewrite(fdaily1); WriteFileHeaderDaily(fdaily1,eenheden); {closefile(fdaily1);} end;
      if img=2 then begin system.assign(fdaily2,tddname); system.rewrite(fdaily2); WriteFileHeaderDaily(fdaily2,eenheden); {closefile(fdaily2);} end;
      if img=3 then begin system.assign(fdaily3,tddname); system.rewrite(fdaily3); WriteFileHeaderDaily(fdaily3,eenheden); {closefile(fdaily3);} end;
      if img=4 then begin system.assign(fdaily4,tddname); system.rewrite(fdaily4); WriteFileHeaderDaily(fdaily4,eenheden); {closefile(fdaily4);} end;
      if img=5 then begin system.assign(fdaily5,tddname); system.rewrite(fdaily5); WriteFileHeaderDaily(fdaily5,eenheden); {closefile(fdaily5);} end;
      if img=6 then begin system.assign(fdaily6,tddname); system.rewrite(fdaily6); WriteFileHeaderDaily(fdaily6,eenheden); {closefile(fdaily6);} end;
    end;
  end;
  {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving AssignDailyTDD (header of file with daily diagrams)');
end;

procedure TfrmMAIN.CloseDailyTDD;
{close (max 6 selected) daily time-depth diagrams}
var img,i:integer;
begin
  {If GUItrace=1 then wwwriteln} singletracewriteln('Entering CloseDailyTDD (close file with daily diagrams)');
  for img:=1 to lstbxScreenVariables.Items.Count do
  begin
    for i:=1 to maxTDD do
    if lstbxScreenVariables.Items[img-1]=cmbbxScreenVariables.Items[i-1]
    then
    begin
      if img=1 then closefile(fdaily1);
      if img=2 then closefile(fdaily2);
      if img=3 then closefile(fdaily3);
      if img=4 then closefile(fdaily4);
      if img=5 then closefile(fdaily5);
      if img=6 then closefile(fdaily6);
    end;
  end;
  {If GUItrace=1 then wwwriteln} singletracewriteln('Leaving CloseDailyTDD (close file with daily diagrams)');
end;

procedure TfrmMAIN.CheckNonsense;
{check on nonsense inputs in the soil data file.
 Following checks are done:
 1. Do clay+silt+sand+1.7*OC sum up to 100 and are all these positive?
 2. Do exchangeable cations sum up to the CEC?
 3. Is the charge balance of the solution neutral?
 4. Is the charge balance of the rain water composition neutral?
 5. Are the values for calcite and gypsum in range [0;1>}
var i,j:integer; checktol,checkbal,bestvalue:real; Msg,strbest:string;
begin
 warningsissued:=false;

 {1} checktol:=1E-3;
 for i:=1 to DataInit.NoLayers do
 begin
   checkbal:=DataInit.CLAY[i]+DataInit.SILT[i]+DataInit.SAND[i]+1.7*DataInit.OC[i];
   if (abs(checkbal-100)>checktol)or(DataInit.CLAY[i]<0)or
      (DataInit.SILT[i]<0)or(DataInit.SAND[i]<0)or(DataInit.OC[i]<0)
   then
   begin
    str(i,Msg); Msg:='Compartment '+Msg+'; Warning: Texture fractions contain error(s) in inputfile.'; dshowmessage(Msg);
    warningsissued:=true;
   end;
 end;
 {2}  checktol:=1E-3;
 for i:=1 to DataInit.NoLayers do
 begin
   checkbal:=DataInit.AXSE[1][i]+DataInit.AXSE[2][i]+DataInit.AXSE[3][i]+DataInit.AXSE[4][i]
             -DataInit.ACEC[i]+DataInit.AXSE[10][i]+DataInit.AXSE[12][i];
   bestvalue:=checkbal+DataInit.ACEC[i]; str(bestvalue:10:6,strbest);
   if (abs(checkbal)>checktol)
   then
   begin
    str(i,Msg);
    Msg:='Compartment '+Msg+'; Warning: CEC not equal to sum exchangeable cations in inputfile (perhaps change CEC to '+strbest+').';
    dshowmessage(Msg);
    warningsissued:=true;
   end;
 end;
 {3}  checktol:=1E-3;
 for i:=1 to DataInit.NoLayers do
 begin
   checkbal:=2*DataInit.SEA[1][i]+2*DataInit.SEA[2][i]+DataInit.SEA[3][i]+DataInit.SEA[4][i]
             -DataInit.SEA[5][i]-2*DataInit.SEA[6][i]-DataInit.SEA[7][i]
             +3*(DataInit.SEA[12][i]+DataInit.SEA[13][i])+2*DataInit.SEA[14][i];
   bestvalue:=checkbal+DataInit.SEA[7][i]; str(bestvalue:10:6,strbest);
   if (abs(checkbal)>checktol)
   then
   begin
    str(i,Msg);
    Msg:='Compartment '+Msg+'; Warning: Charge of solution not 0 in inputfile (assuming 6<pH<8: perhaps change Alkalinity to '+strbest+').';
    dshowmessage(Msg);
    warningsissued:=true;
   end;
 end;
 {4} checktol:=1E-3;
 for i:=1 to DataInit.NIRRIG do
 begin
   checkbal:=2*DataInit.ARSE[1][i]+2*DataInit.ARSE[2][i]+DataInit.ARSE[3][i]+DataInit.ARSE[4][i]
             -DataInit.ARSE[5][i]-2*DataInit.ARSE[6][i]-DataInit.ARSE[7][i]
             +3*DataInit.ARSE[12][i];
   bestvalue:=checkbal+DataInit.ARSE[7][i]; str(bestvalue:10:6,strbest);
   if (abs(checkbal)>checktol)
   then
   begin
    str(i,Msg);
    Msg:='Day '+Msg+'; Warning: Charge of rainwater not 0 in inputfile (assuming 6<pH<8: perhaps change Alkalinity to '+strbest+').';
    dshowmessage(Msg);
    warningsissued:=true;
   end;
 end;
 {5}
 for i:=1 to DataInit.NoLayers do     {recalc to mass fractions calcite/gypsum and then check; mfr=mass/(RHO*DLZ*(1-stonesfr-gravelfr))}
 begin
   if ((DataInit.CALCITEA[I]/(DataInit.RHO[i]*DataInit.DLZ*(100-(DataInit.stones[i]-DataInit.gravel[i])/100)))>1)or
      ((DataInit.CALCITEA[I]/(DataInit.RHO[i]*DataInit.DLZ*(100-(DataInit.stones[i]-DataInit.gravel[i])/100)))<0)
   then
   begin
    str(i,Msg); Msg:='compartment '+Msg+'; Warning: Calcite mass fraction outside range [0;1> in inputfile.'; dshowmessage(Msg);
   end;
   if ((DataInit.GYPA[I]/(DataInit.RHO[i]*DataInit.DLZ*(100-(DataInit.stones[i]-DataInit.gravel[i])/100)))>1)or
      ((DataInit.GYPA[I]/(DataInit.RHO[i]*DataInit.DLZ*(100-(DataInit.stones[i]-DataInit.gravel[i])/100)))<0)
   then
   begin
    str(i,Msg); Msg:='compartment '+Msg+'; Warning: Gypsum mass fraction outside range [0;1> in inputfile.'; dshowmessage(Msg);
    warningsissued:=true;
   end;
 end;
 if warningsissued
    then begin Msg:='*** Soil data file checked for possibly erroneous inputs. Results are above. ***'; dshowmessage(Msg); end;
end;

procedure TfrmMAIN.CheckStartFromScratch;
{In case of a start of SoilGen without input files: check for presence of amorphite.txt and otherite.txt}
begin
  if startscreen then grpbxOnStart.visible:=true else grpbxOnStart.visible:=false;
  if fileexists('Amorphite.txt')
  then lblAmorphite.Caption:='Amorphite.txt available.'
  else lblAmorphite.Caption:='Amorphite.txt will be created.';
  lblAmorphite.visible:=true;
  if fileexists('Otherite.txt')
  then lblOtherite.Caption:='Otherite.txt available.'
  else lblOtherite.Caption:='Otherite.txt will be created.';
  lblOtherite.Visible:=true;
  StartScreen:=false;
end;

procedure TfrmMAIN.MVAR(year:integer; txt:string; getal:real);
{not active now; allows monitoring for RHO at start+end of various functions and procedures}
var fMVAR:textfile; txtyear:string; yearlocal,code:integer;
begin
 {str(year,txtyear); txtyear:=leftstr(txtyear,4); system.val(txtyear,yearlocal,code);}
{ if  (yearlocal>1810)and(yearlocal<1814) then
 begin
  if fileexists('MVAR.txt')
  then begin system.assign(fMVAR,'MVAR.txt'); system.append(fMVAR) end
  else begin system.assign(fMVAR,'MVAR.txt'); system.rewrite(fMVAR)end;
  system.writeln(fMVAR,year:8,' ',txt,' ',getal:12:5);
  closefile(fMVAR);
 end;}
end;

Function TfrmMAIN.HYPRESRHO(clayperc,omperc:real):Real;
{pedotransfer function to estimate RHO, the bulk density, from clay and OM.
 Only called when inputs (soil+events) are changed via the GUI}
begin
  HYPRESRHO:=1/(0.6117+0.003601*clayperc+0.002172*sqr(max(omperc,0.001))+0.01715*LN(max(OMperc,0.001)));
end;

initialization

startscreen:=true;
warningsissued:=false;
claymigrationallowed:=true;
PhysWeathAllowed:=true;
rootcounterthreshold:=def_rootcounterthreshold;
GUIcount10:=0;
crashtext:='';

end.
