unit untProcesses;
{**************************************************************************************}
{**************************** This file is part of SoilGen ****************************}
{****************************SoilGen is free software:*********************************}
{you can redistribute it and/or modify it under the terms******************************}
{of the GNU General Public License as published by the Free Software Foundation,*******}
{either version 3 of the License, or (at your option) any later version.***************}
{  It would be polite to refer to the SoilGen model by mentioning the documentation:  }
{  Peter Finke. 2024. Modelling soil development under global change.                 }
{  SpringerBriefs in Earth System Sciences 10032. ISBN: 978-3-031-55582-4.            }
{  https://doi.org/10.1007/978-3-031-55583-1                                          }
{**************************************************************************************}
{SoilGen is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;**}
{without even the implied warranty of MERCHANTABILITY or ******************************}
{FITNESS FOR A PARTICULAR PURPOSE.*****************************************************}
{See the GNU General Public License for more details.**********************************}
{**************************************************************************************}
{You should have received a copy of the GNU General Public License along with SoilGen.*}
{If not, see <https://www.gnu.org/licenses/>.******************************************}
{$MODE Delphi}

{$DEBUGINFO ON}
{mag niet gebruikt in unit: $M 16384,1000000000}
{This unit contains the soil processes that act at a temporal resolution of less than 1 year}
{the unit consists of
 A: the complete source from the LEACHC model by Hutson (release 2003);
 B: an OM-decomposition submodel based on RothC-26.3 (Coleman, 2005);
 C: a weathering and clay new formation routine, inclusion of Al+Fe+Si chemistry,
    based Transition State Theory (c.f. PhD-thesis Opolot);
 D: a clay migration submodel (modified after the model proposed by Jarvis et al, 1999);
 E: a submodel for physical weathering;
 F: chemistry additions:
    (i) reaction constants for Arrhenius temperature correction (read from table made in shell)
    (ii)Al-chemistry (equilibrium pH-Gibbsite) and its effect on obtaining the pH
        in the Chem2, Equilchm2 routines which are extensions of chem and equilchm.

 Ad A:
 The LEACHC model was translated from FTN95 to Pascal by PF2006.
 Changes are commented in the text by <PF2006>. These changes imply
 (i)   removal of some apparent errors, i.e. missing initialisations;
       Most of these errors were detected when compiling the original FTN95 code
       with the Salford FTN95 compiler.
 (ii)  program structure changes because of syntax incompatibilities FTN>PAS;
 (iii) additional functionality.
 Source files:
  Declarations and constants:Parms.inc; Blocks.inc, Blockc.inc,
  Coordination, general:     LEACHC.for > MAINC.for > mainc2.for (eqv to mainc.inc)
                             LEACHC procedure is now renamed to StartProcesses
  Time calculations:         DAYCALC.for; CADATE.for
  Input reading:             READC.for (called READCNEW in this program), READIRRC.for
  Physics subroutines:
  * Prediction pF-theta-H:   RETPRED.for, RETFUN.for (incl.functions THETA, POTL, HC, DWC)
                             WATDAT.for
  * Richards' equation:      WATFLO.for; TSTEP.for
  * boundary situations:     POTET.for; ETRANS.for; RUNOFF.for; autoirrg.for; CELLC.for
  Chemical subroutines:
  * Chemical equilibria:     EQUICHLM.for; CHEM.for (including the subroutines
                              ACT ECII XCHANG xsolve change PRECIP SINK)
  * CDE-equation:            SOLC.for
  Output (partly in Mainc2!):OUTC.for
  alternative water flow:    CALFC; (incl.subroutine wateq)
  Heat flow (made active by PF2006): HEAT.for
  Plant growth and soil management: CULTC, GROWTH, WUPTAK;
  Added by pf2006:           SINKC
  pf08:                      READCNEW (memory transfer of data from calling program
                                       instead of reading from file)
                             estimate_KatH (for initialisation purposes)

  ad B:
  The submodel on OM-decomposition was developed in accordance to RothC26.3
  to work with daily timesteps. The following routines were implemented:
  * Input reading and initialization:   RothC_Init;
                                              calling the function ClassifyMonth;
  * Daily decomposition             :   RothC_Daily;
  * Output writing                  :   RothC_Exit;
  * conversion                      :   function tonha_to_pco2;
                                        function pco2_to_tonha;
  The main subroutine <RothC_Daily> is called from within the SoilGen module MAINC2
  once per day. Some of the inputs are actually per month.
  The outcomes are used to update (in SoilGen)
  + the OC-content of the soil layers and associated cations
  + the pCO2.
  Decay constants are corrected for 4 protection mechanisms, defined / activated by the user.

  ad C:
  Weathering: The following routines were implemented:
  * Input reading and initialization:   WE_Init;
  * Daily weathering                :   WE_Daily (and supporting procedures);
  * Output writing                  :   WE_Exit;
  Aluminium chemistry: changes in former LEACHC-procedures
  EQUICHLM.for; CHEM.for (including the subroutines ACT ECII XCHANG xsolve change PRECIP SINK)

  ad D:
  * Input reading and initialization:   Init_Clay_Migration;
  * Calculation dispersible clay mass
    (chemical in all compartments and
    physical in upper 1 mm by rainsplash)
    for each timestep before entering
    the CDE                         :   Clay_Migration1_PreCDE;
  * Redistribution of Clay and
    exchange phase after the CDE    :   Clay_Migration2_PostCDE;
  * Conversion to output variables  :   Exit_Clay_Migration;

  ad E: for every Heat-timestep     :   Physical_Weathering;

  ad F: At start of every chemical
    equilibrization                 :   AdaptConst (Arrhenius correction)
  *
  }
interface

uses SysUtils, Math, untSoilGenConst;

{source: PARMS.inc}
{---------------------------------------------------------------------}
{  Most arrays are dimensioned using parameters.  This file specifies the }
{  MAXIMUM array sizes that can be used in a simulation.  These will usually }
{  exceed those actually used.}
{  These are:}

{  LL: Number of nodes.  Number of nodes = Number of segments + 2 boundary nodes.}
{  MM: Number of chemical species.}
{  NFR: Number of broadcast chemical applications.}
{  IPPT: Number of water applications (rain + irrigation).  If the trial}
{  automated irrigation is used, then this array must be large enough to }
{  accommodate the specified rain and irrigation as well as the irrigation}
{  events triggered by LEACHM.}
{  NC: Number of crops. }
{  IWEEK: Number of weeks. }
{  IPNT: Number of specified print times for print option 2 for the .OUT file.}
{  NCULT:  Number of cultivations}
const {for debugging purposes}
  testbedtrue=false; {set to true if the new geochemistry is to be tested}
  bflog = True;     {set to true if <LEACHC.log> must be output with runtime messages
                       from the LEACHC-core program}
  bfdebug = True;   {set to true if error messages from LEACHC-core must be added to
                       <Messages.txt>}
      {VGNactive=true; set to true if theta, K, DWC and POTL are to be calculated using the
                       Van Genuchten functions}
  {for LEACHC}
  DETT = 1E-10;
  {smallest allowable timestep, originally DETT:=1E-10; tried 1E-5 this is 38% faster}
  Kdoc = 0.043; {coefficient to convert DOC into CORG, after Bi, 1995; Analyst 120)}
  LL = MaxLayers;
  MM = MaxChem;
  NFR = MaxNoChemApp;
  IPPT = MaxNoIrrig;
  NC = MaxNoCrops;
  IWEEK = MaxNoWeeks;
  IPNT = 5;
  NCULT = MaxNoCultiv;
      {Solubility products of gypsum (SP1) and calcite (SP2) as well as other
       chemical equilibrium constants were read in calling program}
      {On SP2:
       A calibration was done comparing decalcification speed
       (how many years to decalcify a 12% CaCO3 soil with 472 mm net annual percolation
        to a depth of 1100 mm)
       with a "metamodel" by Egli & Fitze.}
  nchem = 16;
  nchem1 = 23;      {major ion/dispersible matter species:}
      {1=Ca++; 2=Mg++; 3=Na+; 4=K+; 5=Cl-; 6:=SO4--; 7=HCO3-; 8=CO3--;
       9=balion+/-; 10=H+; 11=OH-; 12=Al+++; 13=Fe+++; 14=Fe++; 15=H4SiO4s;
       16=DOC-0;
       17-23 is dispersible matter (part of clay mass and exchange-species):
       17=Clay; XCa-=18; XMg=19; XNa:=20; XK=21; XH:=22; XAl:=23}

      {for RothC-26.3 the constants are now set in main program, possibly read from SoftConstants.txt}

      {maximum pCO2 (in bar) allowed; this means that immediate convective transport
       towards atmosphere is assumed if this value is exceeded any day.
       Higher values only occur in soils that are practically water-saturated,
       this may include partially frozen soils}
  MaxpCO2 = 0.2;
  {for Newton-Raphson}
  NR_ntrial = 50;     {max nr iterations}
  NR_tolx = 1.0e-7;   {tolerance criterion for summed up absolute function increments}
  NR_n = 7;           {nr of major species =H, Al, Ca, Mg, Na, K, SO4}
  NR_np = NR_n;       {nr of equations to solved simultaneously, = Nr_n}
  NR_tolf = 1.0e-7;   {tolerance criterion for summed up absolute equation values}
  T_const = 298.15;    {temp in K at 25oC}
  R = 0.008414;{universal gas constant in KJ/mol/K}
  {molar weights}
  fw_O = 16.000;
  fw_Al = 26.982;
  fw_Ca = 40.080;
  fw_Mg = 24.312;
  fw_K = 39.102;
  fw_Na = 22.990;
  fw_Fe = 55.847;
  fw_Si = 28.084;
  fw_H = 1;
  fw_Cl = 35.453;
  fw_C = 12.0107;
  fw_S = 32.065;
  {Relative Formula Mass, RFM, per mineral from elemental weights and stoichiometric coeficients}
  RFMalbite = fw_Na + fw_Al + 3 * fw_Si + 8 * fw_O;        {Albite NA Al Si3 O8}
  RFMkfeldspar = fw_K + fw_Al + 3 * fw_Si + 8 * fw_O;       {K-feldspar K Al Si3 O8}
  RFMmuscovite = fw_K + 3 * fw_Al + 3 * fw_Si + 10 * fw_O + 2 * (fw_O + fw_H);
  {Muscovite K Al3 Si3 O10 (OH)2}
  RFMquartz = fw_Si + 2 * fw_O;                      {Quartz Si O2}
  RFMbiotite = fw_K + 1.5 * fw_Mg + 1.5 * fw_Fe + fw_Al + 3 * fw_Si + 10 * fw_O + 2 * (fw_O + fw_H);
  {Biotite K Mg1.5 Fe1.5 Al Si3 O10 (OH)2}
  RFManorthite = fw_Ca + 2 * fw_Al + 2 * fw_Si + 8 * fw_O; {Anorthite Ca Al2 Si2 O8 }
  RFMchlorite = 5 * fw_Mg + 2 * fw_Al + 3 * fw_Si + 10 * fw_O + 8 * (fw_O + fw_H);
  {Chlorite Clinochlore Mg5 Al2 Si3 O10 (OH)8 }
  RFMhornblende = 2 * fw_Ca + 4 * fw_Mg + fw_Al + 6.7 * fw_Si + fw_Al + 22.4 * fw_O;
  {Hornblende Ca2 Mg4 Al Si6.7 Al O22.4 from Profile model }
  RFMForsterite = 2 * fw_Mg + fw_Si + 4 * fw_O;   {Forsterite Mg2 Si O4}
  RFMaugite = fw_Fe + fw_Si + 3 * fw_O;       {AugiteAsOrthoferrosilite Fe Si O3 }
  RFMfayalite = 2 * fw_Fe + fw_Si + 4 * fw_O;  {Fayalite Fe2 Si O4}
  RFMkaolinite = 2 * fw_Al + 2 * fw_Si + 5 * fw_O + 4 * (fw_O + fw_H);   {Kaolinite Al2 Si2 O5 (OH)4}
  RFMillite = 3.4 * fw_Si + 10 * fw_O + 2 * (fw_O + fw_H) + 0.02 * fw_Mg + 2.38 * fw_Al +
    0.01 * fw_Ca + 0.44 * fw_K;
  {Illite Si3.4 O10 (OH)2 Mg0.02 Al2.38 Ca0.01 K0.44, Godderis et al 2006}
  RFMmontmorillonite = 4 * fw_Si + 10 * fw_O + 2 * (fw_O + fw_H) + 0.33 * fw_Mg + 1.67 * fw_Al + 0.04125 *
    (fw_Ca + fw_Mg) + 0.0825 * (fw_K + fw_Na);
                 {Montmorillonite Si4 O10 (OH)2 Mg0.33 Al1.67 Ca0.04125 Mg0.04125 K0.0825 Na0.0825
                  this is the average for Ca/Na/Mg/K montmorillonite, Godderis et al 2006}
  RFMgibbsite = fw_Al + 3 * (fw_O + fw_H);   {Gibbsite Al(OH)3}
  RFMgypsum = fw_Ca+fw_S+4*fw_O;             {Gypsum CaSO4}
  RFMcalcite = fw_Ca+fw_C+3*fw_O;            {Calcite CaCO3}
  {RFMamorphite and RFMotherite are based on input elemental amounts,
   so these are not constants but variables}
  maxomega=1.1; {upper allowed limit of omega (for newforming minerals)}
type
  {real = double;  }
  {for Newton-Raphson}
  glnarray = array [1..NR_n] of real;
  glnbyn = array [1..NR_n, 1..NR_n] of real;
  glindx = array [1..NR_n] of integer;
  glnpbynp = glnbyn;
  {for LEACHC}
  rLL = array[1..LL] of real;
  rMM = array[1..MM] of real;
  {for RothC-26.3}
  Monthly = array[1..12] of real;
  Profile = array[1..LL] of real;
  iProfile = array[1..LL] of INT64;
  Mindat = array[1..nrminerals] of real;

  RothCRec = record
    Rainfall, EvapoPan, SoilCover, RDepth,
    totalaccTSMD, totalPlantResiduesIN, totalManureIN: Monthly;
    Temperatures, PlantresiduesIN, ManureIN: array[1..12] of profile;
    ClayPerc, fracManureIN, fracPlantIN, x2,x3: profile;
    DPM2RPM, ThicknessLayer, frEctoLitter, frEndoLitter, PlantResSUM, ManureSUM,
    totalmaxTSMD: real;
    DPMpool, RPMpool, BIOpool, HUMpool, IOMpool, CO2pool, accTSMD, pCO2_r,
    DPMXpool, RPMXpool, BIOXpool, HUMXpool, IOMXpool, CO2Xpool: profile;
    ectoDPMpool, ectoRPMpool, ectoBIOpool, ectoHUMpool, ectoIOMpool, ectoCO2pool: real;
    vegetatie: string;
  end;
{source: BlockC.inc}
var  {for LEACHC}
  {  Integers}
  {chem1}
  {NCHEM changed to constant} IFR, ITERCH, NN1, icounttim, MPORE: integer;
  {  Reals}
  {chem2}
  CDCO2, CELLW, DIFA, DIFB, DIFO, PPTS1, PPTS6, PPTS8, PPTS12, SOLS1, SOLS6,
  SOLS8, SOLS12, counttim,SEFeII,SEFeIII: real;
  {  LL arrays}
  {chem3..chem7}
  ACEC, ACEC_oc, ACEC_clay, APCO2, APH, CEC, CEC_oc, CEC_clay, DELCO2,
  DELGIBB, DELGYP, DECALCITE, EC, ESP, GYP, GYPA, CALCITE, CALCITEA,
  PCO2, PH, SAR, SC1, SC2, SC3, SC4, SC5, SC6, SC7, SC8, SC9, SC10, SC11, SC12,
  SC13, SC14, SC15, TOTANS, TOTCAT, SUM21, MM21, phlab,stoneperc: rLL;
  {  MM arrays}
  {chem8+chem9}
  APDSE, BALSE, BBSE, CFER, CLSE, CMASS, SFER, WSFER, CRSE, CISE, CUSE,
  DELSE, LSE, PDSE, VAL: rMM;
  {  MM,LL arrays}
  {chem10}
  AXSE, SEA, XSE, AXSE_oc, AXSE_clay, XSE_oc, XSE_clay: array[1..mm, 1..LL] of real;
  SE, SS: array[1..mm, 1..LL] of real;
  {  Miscellaneous}
  {chem11}
  FSE, TSE: array[1..MM, 1..4] of real;
  {chem12}
  DFER, INCOR: array[1..NFR] of integer;
  {chem13}
  FER: array[1..mm, 1..NFR] of real;
  {chem32}
  RUNOFF_LOAD, WRUNOFF_LOAD, CRUNOFF_LOAD: rMM;
  {source: BLOCKS.inc}
  { Integers}
  {sh1}
  ADAY, AMONTH, AYEAR, BBC, CROPNO, CUMT, DAY, DAY1, DAYM, INITFLG: integer;
  {sh2}
  IDATE, IFGTEM, IFILE, IFLG2, IFLG3, IFLOW, IHEAD, IRRNO, cultno: integer;
  {sh3}
  JK, KILPLT, KK, KLPLT, MONTH, NCROP, NP1, UNITS, iy, im, cycno: integer;
  {sh4}
  NIRRIG, NN, NPRINT, NPRIN, NYC, PRIOR, n1prt: integer;
  {sh4a}
  nirriga: integer;
  irrflg: array[1..ippt] of integer;
  {sh5}
  PRNTAB, PRNTOP, R30, R54, R9, R24, TRACE, WEEK, WEEKO, YEAR, NWEEK, DATE: integer;
  igrowth, RTYPE, ICUT: array [1..nc] of integer;
  { Reals}
  {sh6}
  ABBPOT, ADDFC, ADDTH, AK2, AWATER, BBPOT, CEVAPA, CONQ, CRAIN, CIRRIG: real;
  {sh7}
  CTRANA, WTRANA, DELM, DELT, DELWAT,{DETT,}DLYEP, DLYPAN: real;
  {sh8}
  DLYTP, DLZ, ER, ERMAX, EXCESS, FMAX, HDRY, HEAD, HHIGH, WINFIL, RHOS1,
  RHOS2, RHOS3, ROCN2, ROSLOPE, RNOFF, RDZ, dlz0: real;
  {sh9}
  HLOW, HROOT, IRATE, IRRBEG, IRREND, IRRIG, NSUM, RSUME, POROSITY: real;
  {sh10}
  PRINTOUT, PRINTSUM, PWATER, R20, RRES, THETA1, TIMDAY, TIME: real;
  {sh101}
  TRANA, TRANP, PEVAP, TRL, WATBAL, WFDD, PRINT, PDEPTH, FWBTC, IFGBTC,
  FWBTCI, WBBPOT, BBINC: real;
  { LL arrays }
  {sh12}
  A, AAEV, ABCAM, AEV, RP, AH, AKS, ATEM, CLAY, SILT, SAND, VGM, TR, volpercclay, PARDEN: rLL;
  {sh13}
  AW, B, BCAM, CMATCH, CTRAN, FLUX: rLL;
  {sh14}
  FLXDEN, G, H, HM, HMATCH, KS, L: rLL;
  {sh15}
  RDF, RG, RHO, RL, SALT, TEM, TG: rLL;
  {sh16}
  TM, TRAN, TS, W, WATER, microporosity: rLL;
  {SH17}
  WTRAN, Y, FC, WP, D, OC, LAMBDA: rLL;
  {sh18}
  modret, dd: rLL;
  { IWEEK arrays}
  {sh19}
  AMP, AVETEM, PAN: array[1..IWEEK] of real;
  ETIME: array[1..IWEEK] of integer;
  {sh20}
  WTDEP, CULTDEP: array[1..IWEEK] of real;
  { IPNT arrays}
  {sh21}
  PRTIME: array[1..IPNT] of real;
  { IPPT arrays}
  {sh22}
  AIRRBG, IRATEA, IRRIGA: array[1..IPPT] of real;
  { NC arrays}
  {sh23}
  AK1, CRPCOV: array[1..NC] of real;
  DAYHAR, ESTART: array[1..NC] of integer;
  DCULT: array[1..NCULT] of integer;
  {sh24}
  ESTOP, PDAY: array[1..NC] of integer;
  PANFAC, PD, RDEP: array[1..NC] of real;
  {sh25}
  RDFDAY: array[1..NC] of integer;
  FGO, FGS: real;
  { Miscellaneous}
  {sh26}
  GEOMOM, GEOMOS, GEOMQ: array[1..3] of real;
  {sh27}
  ND: array[0..4] of integer;
  ifourth_layer: real;
  {sh28}
  PART: array[1..4, 1..LL] of real;
  {sh29}
  ALAMO: array[1..5] of real;
  {sh30}FW0: real;
  FW, CFW, TH: array[1..4] of real;
  {sh31}
  FILE1, FILE2, FILE3, FILE4, FILE5, FILE6, FILE15: string;
  CH_IRR_FLAG: integer;
  {sh32}
  FILE8, file0, fileWE: string;
  {irrig1}
  CHECKDEP, CRITIRRIG, REPLENISH_DEP: real;
  {irrig2}
  RSE, seirr, ISE: rMM;
  ARSE: array[1..MM, 1..IPPT] of real;
  {double}
  clw: double;
  wfw: array [1..LL] of double;
  {added in conversion to Pascal}
  f1, f9, f10, f11, f12, f13, f14, f15, f16, f17, f40, f45, flog, fdebug: textfile;
  bf1, bf9, bf10, bf11, bf12, bf13, bf14, bf15, bf16, bf17, bf40, bf45: boolean;
  dimok: boolean;
  GDIF, BAROM, RDFTOT: real; {nodig?}
  IDAYM, icult: integer;
  TIMDAYOLD: real;
  {added pf2006 to obtain average temperatures for the simulation period + for month per node}
  TEMPGEM: rLL;
  TEMPCOUNT, TEMPCOUNTMONTH, oldmonth, oldday: integer;
{added pf2006 to obtain continuous [0;1] scale indicator variable for
  clay instability for the simulation period per node}
  ClayDispInd: rLL;
  ClayCount: real;
{added pf08: reaction constants now read in calling programme
      and assigned to global variables below in LEACHC-routine}
  SP1, SP2, KH, KW, KA1, KA2, KD1, KD2, KD3, KD4, KD5, KD6, KD7, KD8, KD9, KD10, KD11,
  KSP, KAl1, KAl2, KAl4, Kf1, Kf2, Kf3, Kf4, Kf5, Ko1, Ko2, Kg1, Kg2, Kg3, KHF, KCaF,
  KAlbit,KKFeld,KMusco,KBioti,KQuart,KChlor,KAnort,KForst,KAugit,KKaoli,KIllit,
  KMontm,KHornb,KFayal,KAmorp,KOther,KGibbs,KGypsu,KCalci: real;
  {from MAINC}
  FERNO, SEG, iyrtot, iautoirrg, IFGSUM: integer;
  file16, file17: string;
  tcalcite, tgypsum: array[1..4] of real;
  {from MAINc2 and all other subroutines: the non-local fortran variables}
  FLAGC, NODEWT, IFGRAT, iinitialize, ITIME_TO_PRINT, IET, IFLG,
  iflgrunoff, IFLAG, IFIRST, IFOURTH, IEND_OF_DAY, IWRITE_SUM: integer;
  txt1, txt2, txt3, txt4: string;
  RR1, RR1i, RR2, RR3, RR4, RR5, RR6, RR7, RR8, RR9, RR10, RR11, RR12, RR13, RR23, RR24,
  volume, DEP1, DEP2, DEP3, DIFF, Wnew, Wchange, Solute_added,
  R1, R2, R3, R3a1, R4, R5, R6, R7, R8, R10, R11, R13, R14, R17, R18,
  R111, R113, R114, R115, R116, R112, R117, R118, R119,
  R120, R121, R122, R123, R124, R125, R126, R127, EFLDEN, CTRANP, WTRANP, CEVAPP, WEVAPP,
  WEVAPA, CFW0, rots, rowp, rofc, row, s1, ros, TTRANP, TER, FRACw, DELTHEAT, Eflux,
  ospot, YIELDW, rindex, runoff_mm, PVOL, PV, wetdep, DELTSUM, WETLOW, ratio,
  relCa, relMg, relK, relNa, relAl, relCl, relSO4, relHCO3, relCO3, rdum: real;
  DUSE, USE, csink, CHEMDPMpool, CHEMRPMpool, CHEMBIOpool, CHEMHUMpool,
  CHEMIOMpool, CHEMCO2pool: array[1..mm, 1..LL] of real;
  CMINER: rMM;
  {pf2006 for RothC-26.3}
  RothC: RothCRec;
  MaxRootComp: integer;
  RCName: string;
  kHenry, checksum: real;
  DailyCO2PoolIncrease: Profile;
  {pf08 for Weathering routines}
  GEN_kCa, GEN_kMg, GEN_kK, GEN_kNa, GEN_alfaCa, GEN_alfaMg, GEN_alfaK, GEN_alfaNa,
  GEN_Anorthite, GEN_Chlorite, GEN_Microcline, GEN_Albite, GEN_ElovichRate,
  GEN_ElovichExp, GEN_logKAl: real;
  {pf08 for DOC}
  DOC, DOCperc: Profile;
  {pf08 to choose between "old" and new chemistry from calling programme}
  freezecounter,rootcounter: Profile;
  typechem: integer;
  debugpoint, externalcall: boolean;
  {pf08 for clay migration}
  claym, siltm, sandm, finesandm, gravelm, stonesm, ocm, totalm, clayfrac, oldRHO, newRHO: rLL;
  fracyear, CM_Mmax, CM_Ms, CM_As, CM_R, CM_E, CM_D, CM_P, CM_Cs,
  MixPool, PoolT, PoolB, SC_redfac: real;
  {tmp} ftemp: textfile;
  last100: array[1..100] of string;
  count10: integer;
  {physical weathering}
  tem_old, dTdt: rLL;
  {optimization trials}
  freqdelt: array[1..10] of real;
  ftrial: textfile;
  {addition of Van Genuchten functions and -parameters}
  VGNal, VGNtr, VGNts, VGNks, VGNen, VGNel, VGNem: rLL;
  VGNactiveLOCAL: boolean;
  {to allow water tables due to sea level change}
  WaterTableDepth: real;
  {former constants RothC, now set in main program possibly read from SoftConstants.txt}
  k_DPM, k_RPM, k_BIO, k_HUM, DPM2RPM_agriculture, DPM2RPM_grassscrub,
  DPM2RPM_deciduouswood, DPM2RPM_coniferouswood, frEctoLitter_grassscrub,
  frEndoLitter_grassscrub, frEctoLitter_deciduouswood, frEndoLitter_deciduouswood,
  frEctoLitter_coniferouswood, frEndoLitter_coniferouswood,
  frEctoLitter_agriculture, frEndoLitter_agriculture,
  frPINT_grassscrub, frPINT_deciduouswood, frPINT_coniferouswood, frPINT_agriculture,
  facscaleCO2BIOHUM, BIO2HUMratio, clay_kd, clay_kr, clay_hthetamacro,
  CM_fref, CM_vref, CM_n, clay_ectobd, clay_ectothick, frStoneSplit,
  relAl_Agriculture, relK_Agriculture, relCa_Agriculture, relMg_Agriculture,
  relNa_Agriculture, relAl_Coniferouswood, relK_Coniferouswood,
  relCa_Coniferouswood, relMg_Coniferouswood, relNa_Coniferouswood,
  relAl_Deciduouswood, relK_Deciduouswood, relCa_Deciduouswood,
  relMg_Deciduouswood, relNa_Deciduouswood, relAl_Grassscrub,
  relK_Grassscrub, relCa_Grassscrub, relMg_Grassscrub, relNa_Grassscrub: real;
  {to get more realistic water uptake when water table is inside rooting zone}
  WaterTableInRootingZone: boolean;
  sumirrig: real;
  {for d14C correction and age calculation}
  yearBP: integer;
  d14CValue: real;
  OK14C: boolean;
  {new weathering routine}
  MinDisPar: array[1..nrparams, 1..nrminerals] of real;
  IAP,Omega:Mindat;
  MinNames: array[1..nrminerals] of string;
  MinPools: array[1..nrminerals] of Profile;   {content (kg/m2) per mineral remaining}
  Gibbsitepool, WE_Capool, WE_Mgpool, WE_Kpool,
  WE_Napool, WE_Sipool, WE_Fepool, WE_Alpool, WE_FePedogenic, WE_SiPedogenic, finh
  : Profile; {content (mol/m2) per ion species remaining
                                                       in mineral pool. NB: WE_Al=primary minerals;
                                                       Gibbsitepool=secondary mineral!}
  RFMamorphite,RFMotherite : real;
  {pf2023 physical weathering and effect stoniness on hydraulic charateristics}
  GRAVEL,STONES:rLL;
  {protection of OM against decay}
  Fp1,Fp2,Fp3,Fp4,Fprot: Profile;
  {temp}
  fcc: textfile;
  {for daily tdd}
  dailytdd1,dailytdd2,dailytdd3,dailytdd4,dailytdd5,dailytdd6:Profile;
  pmic,pmes,pmac: rLL;

{routines translated from LEACHC}
procedure StartProcesses;
{Process driver1: first initialization, modified after LEACHC}

procedure MAINC;
{Process driver2: continued initialization, modified after LEACHC}

procedure MAINC2(var IFGSUM: integer);
{Process driver3: start of program and the master of the process flow. Temporal resolution of days-hours-smaller}

procedure watdat;
{create an output table (part of *.out) of h-theta-K data}

procedure POTET(var pPAN, pDLYPAN, pDLYEP, pDLYTP, pCRPCOV, pERMAX, pPANFAC: real);
{calculate potential evapotranspiration for the day}

procedure RUNOFF(POROSITY, WP, FC, SW, SLOPE, CN2, IRRIG: real; var RO, s, s1: real; tem: real);
{calculate runoff via the curve number approach (called from MAINC2)}

procedure wuptak;
{calculate water sink term for WATFLO ie. root uptake}

procedure tstep;
{calculate the length of the new time increment}

procedure WATFLO(var ros, rots, rowp, rofc, row, s1, DIFF: real);
{calculate water flux between nodes and new potentials by Richards'equation}

procedure solc;
{calculate transport of solute (incl. dispersed) species}

procedure outc;
{write Cumulative and mass balance to file *.out}

procedure READCNEW;
{Obtain soil data for the newly started year, copying these from the datarecord DataYearIn filled at start of the year (with variable status at end of previous year)
 and in the order of the standard soil input file.}

procedure READIRRC;
{read irrigation when given in separate files (not so in SoilGen; called from READCNEW)}

procedure RETPRED(compnr: integer;
  var CL, SI, RHO, CARBON, CMATCH, HMATCH, A, B, AKS, RP, F, MODRET, PARDEN: real);
{determine the soil hydraulic parameters using the Hypres PTF(by calling predictVGN)
 towards Van Genuchten parameters, includes conversion to Campbell parameters}

procedure DAYCALC(var AMONTH, ADAY, AYEAR, DAYNO, CUMT, IDATE: integer);
{calculate Julian day number from calender notation (called from READCNEW and AUTOIRRG)}

procedure CHEM2(var CACO, CASO, ALOX, PH, BD, TAL, TCA, TMG, TNA, TK, TCL, TSO4,
  THCO3, TCO3, BALION, TH, TOH, THETA, EC, SAR, XCA, XMG, XNA, XK, XH, XAL, CEC,
  ESP, K1, K2, K3, K4, K5, K6, K7, K8, K9, K10, K11, K12, K13, K14, K15, DelGibb,
  DELGYP, DECALCITE, PCO2: real; var NN1: integer; SP1, SP2: real;
  var OSPOT: real; var trace: integer; var delco2: real; var n1prt: integer;
  temperature: real);
{equilibrate the chemical composition of soil solution, precipitated phase
 and exchange phase while finding the soil pH}

procedure EQUILCHM2(var K1, K2, K3, K4, K5, K6, K7, K8, K9, K10, K11, K12, K13, K14, K15,
  AlOH_2p, ALOH2_p, ALOH4_m, NASO4,
  NA, NACO3, MG, MGCO3, MGHCO3, MGSO4, K, KSO4,
  MGOH, OH, KHC, KA1C, KA2C, KWC, KD1C, KD2C, KD3C,
  KD4C, KD5C, KD6C, KD7C, KD8C, KD9C, KD10C, KD11C,
  BALION, SP1, SP2, K1C, K2C, K3C, K4C, K5C, K6C, K7C, K8C,
  K9C, K10C, K11C, K12C, K13C, K14C, K15C, PH, PCO2, CACO3,
  CA, CO3, CAHCO3, HCO3, CAOH, CASO4, SO4, Al, SP1C, SP2C,
  KSPc, H, ACT1, ACT2, TCA, TMG, TNA, TK, TAL, XCA, XMG, XNA, XK, XAL, XH,
  calcite_sol, gypsum_sol, Gibbsite_sol, TCL, TOH, TH, TCO3, THCO3, TSO4,
  ISS, CEC: real; ITER: integer; var THETA, BD: real;
  var NN1, trace: integer);
{calculate chemical equilibriums for a set pH (in iteration called from CHEM2)}

function ACT(Z, ISS: real): real;
{calculates ionic activity coefficients using calculated ionic  strength (called from EQUILCHM2)}

procedure ECII(var TCA, TMG, TNA, TK, TCL, TSO4, THCO3, TCO3, BALION, EC, ISS,
  NASO4, NA, NACO3, MG, MGHCO3, K, KSO4, MGOH, OH, CA, CO3, CAHCO3,
  HCO3, CAOH, SO4, H, ACT1, ACT2, CACO, SPCON: real);
{estimates electrical conductivity (called from CHEM2 from iterated chemical composition)}

procedure XCHANG2(var CA, MG, NA, K, Al, H, XCA, XMG, XNA, XK, XAl, XH, BD, THETA, CEC,
  K1C, K2C, K3C, K4C, K5C, K6C, K7C, K8C, K9C, K10C, K11C, K12C, K13C, K14C, K15C: real;
  var NN1: integer);
{calculate exchangeable cations using known selectivity coefficients and solution concentrations  (called from EQUILCHM2)}

procedure xsolve(cconst: real; var s, x: real; nval: integer);
{calculate exchangeable cations using known selectivity coefficients and solution concentrations for 1 cation pair (called from EQUILCHM2)}

function fx(x, a, b, c, d: real): real;
{calculate cubic function value for x using input coefficients (called from RTBIS)}

function rtbis(x1, x2, xacc, a, b, c, d: real): real;
{calculate root of cubic function by bisection (called from XSOLVE)}

procedure change2(var ca, mg, na, k, h, al, xca, xmg, xna, xk, xh, xal, soln,
  cecadj, sca, smg, sna, sk, sh, sal: real);
{corrects exchangeable cations calculated with XSOLVE so that the total amount in solution and on CEC
 does match with the input values (mass balance correction; called from EQUILCHM2)}

procedure PRECIP(var SPS, PPTS, CATS, ANS, TCATS, TANS: real; var ITYPE: integer);
{calculates precipitation of calcite or gypsum based on concentrations and equilibrium constants (called from EQUILCHM2)}

procedure SINK(var CAT, AN, SP, X2, X3: double);
{uptake of cations Ca, Mg, Na, K and Al by plants via transpiration stream but forced by target plant element composition}

{FUNCTION THETA(thetares,thetasat,alfa,n,m,h,hmm:real):REAL; old version}
function THETA(compnr: integer; P, G, TS, A, B, RL, TM, HM: real): real;
{calculate volumetric watercontent for current matric potential}

function POTL(thetaact, thetares, thetasat, alfa, n, m: real): real;
{calculate matric potential for current volumetric water content}
{FUNCTION HC(compnr:integer; Ksat,alfa,n,m,gamma,h,temperature:real):real; old version}
function HC(compnr: integer; P, G, RKS, A, B, TS, RL, RP, TM, HM, T: real): real;
{calculate hydraulic conductivity, if needed corrected for stoniness and frozen soil}

{FUNCTION DWC(thetares,thetasat,alfa,n,m,h,hmm:real): double; old version}
function DWC(compnr: integer; PS, GS, TSS, BS, RLS, ASS, TMS, HMS: real): double;
{calculates differential water capacity (called from WATFLO)}

procedure autoirrg(nchem: integer);
{void: calculates automatic irrigation (not used inSoilGen; called from MAINC2)}

procedure CADATE;
{calculate calender date from next  Julian day (called from MAINC2)}

procedure CALFC(var EFLUX: real);
{transport of water and chemicals according to a modified Addiscott (1977) tipping-bucket concept (called from MAINC2)}

procedure WATEQ(i: integer; var we1, we2: real);
{calculate equilibrium water contents (called from CALFC)}

procedure CELLC;
{set new lower boundary concentrations for solute phase}

procedure cultc;
{void: cultivation (tillage). Not used in SoilGen but replaced by a bioturbation routine.}

procedure ETRANS;
{Evapotranspiration during the length of the time increment}

procedure GROWTH(var KK: integer; var RDFDAY: integer; var PD: real;
  DAY: integer; var RDZ, DLZ: real; var RDF: rLL;    var CRPCOV: real; var ESTART, ESTOP, DAYHAR: integer;
  var AK1: real; var KILPLT: integer; var TRL: real; var PDAY, igrowth: integer; var RDEP: real);
{calculates root growth for corn, by growing day, scaled to max rooting depth (called from MAINC2)}

procedure HEAT(var deltheat: real);
{calculate heat flow during 10% of a day}

{routine developed by pf2006 to provide runtime I/O}
procedure wwriteln(txt: string);          {writes a message to the logfile}
procedure singletracewriteln(txt: string);{indicates the execution point of the program, for debugging purposes}
procedure debugout(txt: string);          {writes a (non-fatal) input error to 'Messages.txt'}
procedure closerun_errors;                {Conditional: if fatal error(s) occurred then mention this in the logfile}
procedure stopc;                          {stop the run for this year and close the associated outputfiles}

{routine developed by pf2006 to allow for plant uptake of Cations Ca Mg K Na Al}
procedure SinkC;
{uptake of cations Ca, Mg, Na, K and Al by plants via transpiration stream but forced by target plant element composition}

{routines developed by pf2008 to allow for migration of dispersible clay}
procedure Init_Clay_Migration;
{initialization for clay migration submodel using mass of clay silt + sand at end previous year}

procedure Clay_Migration1_PreCDE;
{calculate amount of clay in dispersion}

procedure Clay_Migration2_PostCDE;
{calculate consequences of dispersible clay transport}

procedure Exit_Clay_Migration;
{backconverts the clay, silt, oc and sand masses to  mass percentages of clay, silt, sand and OC}

{routines developed by pf2006 based on the concepts of RothC-26.3}
procedure RothC_Init(RCName: string);
{initialization and read input file for C-submodel, status end of previous year}

function ClassifyMonth(realday: real): integer;
{assign month to Julian day (called from RothC_daily)}

procedure RothC_Exit;
{write C-and associated element pools to outputfile RC*.out}

function tonha_to_pco2(ton, kh, pores: real): real;
{void: conversion ton C per ha to pCO2. Not used.}

function pco2_to_tonha(p, kh, pores: real): real;
{backconversion pCO2 to ton C per ha (called from unGUI.bttnRunClick)}

procedure RothC_Daily(month: integer);
{C-submodel on a daily basis}

{pf2023: protection mechanisms for decay of OM}
procedure CalcProtection(comp:integer);
{calculate effect of selected protection mechanisms on decay of OM (called from RothC_daily)}

{routines developed by pf2006 to account for diffusive CO2-transport during 1 day}
procedure CO2_diffusion;
{redistribute produced and present CO2 over the profile by diffuson}

procedure CO2_diff_Eq(var Ds, P, calcpCO2: profile; dt: real; var stable: boolean);
{solve CO2-diffusion equation (called from CO2-diffusion)}

{routines developed by pf2006 to pass data directly to calling program}
procedure PassDataOUT;
{pass data from Processes unit to calling main program untGUI.Run at end of year via the record DataYearOut}

{routines developed by pf08 related to weathering}
procedure WE_init;
{initialization and read input file for Weathering submodel, status end previous year.
reads remaining cation pools in primary minerals and gets general data from calling program}

procedure AdaptRjkRedfac(var redfacQtz,redfacChlor,redfacKaol,redfacIllit,redfacMontm:real;
                         var rjkqtz,rjkchlor,rjkkaol,rjkillit,rjkmontm:real;
                         rjkalb,rjkkfeld,rjkbio,rjkmusco,rjkanort,rjkForster,rjkaugit,rjkhornb,rjkfayal,rjkamorp,rjkother,
                         Aalb,Akfeld,Abio,Amusco,Aanort,AForster,Aaugit,Ahornb,Afayal,Aamorp,Aother,
                         Aqtz,Achlor,Akaol,Aillit,Amontm,
                         Moalb,Mokfeld,Momusco,Mobio,Moanort,MoForster,Moaugit,Mohornb,Mofayal,MoAmorp,MoOther,
                         Moqtz,Mochlor,Mokaol,Moillit,Momontm:real; ilayer:integer);
{reduce weathering rates for newforming minerals if the soil solution cannot deliver the cations (called from WE_daily)}

Procedure IAPOMEGA(var IAP,Omega: Mindat; var finh: Profile);
{calculates IonActivityProducts per mineral and the saturation degree Omega (called from WE_daily)}

procedure WE_Daily;
{weathering of soil minerals on a daily basis; calculates daily cation release by weathering and adjusts pools}

procedure WE_Yearly;
{void: calculates yearly cation release by weathering and adjusts pools; not used}

procedure WE_Exit;
{write minerals and cations locked in minerals to WE*.out,
i.e. weathering data (size remaining cation pools in primary minerals)}


procedure estimate_KatH(compnr: integer; var clay, silt, rho, oc: real;
  var ksat, kath, ath: real);
{estimate the hydraulic conductivity at input hydraulic head using the fitted hydraulic parameters;
 currently used to fill cmatch-hmatch per soil compartment
 when PTF for soil hydraulic characteristics are estimated}

procedure AdaptConst(temperature: real);
{change reaction constants using current temperatures (called from StartProcesses + CHEM2)}

procedure Physical_Weathering(var deltheat: real);
{calculate effect of physical weathering on grain size distribution during 10% of day}

procedure predictVGN(compnr: integer; cl, si, carbon, rho, PARDEN: real);
{predict Van Genuchten parameters using Hypres continuous functions (called from RETPRED + estimateKatH)}

procedure tmpdmp(x: real; txt: string; y: real);
{void; currently inactive temporary write of 2 params to outputfile (called from clay_migration)}

procedure calcphlab(var RHO, PH, AW, PHLAB: rLL; DLZ: real);
{transform the calculated pH as if it were measured in a 1:5 water extract; "pH-H2O 1#5"}

{write tdd for current year with daily values}
procedure WriteDailyTDD;
{write a time-depth data table to file for (max 6) selected soil properties for each day dring the current year}

procedure SelectDailyTimeDepth(name:string; var DailyTDD:Profile);
{choose selected soil variable for daily output (called from WriteDailyTimeDepth)}

{weathering routines for the bedrock layer}
procedure WE_init_sapro;
{bedrock layer: initialization weathering routines}

Procedure IAPOMEGA_sapro(var saprOmega: Mindat; var saprfinh: Profile; var OmegaGibb,OmegaGyps,OmegaCalc:real);
{bedrock layer: calculates IonActivityProducts per mineral and the saturation degree Omega (called from WE_daily_sapro).
At present:  weathering under low ionic concentrations assumed,
no newformation calcite, gypsum, gibbsite and clay minerals, no Al inhibition}

procedure WE_Daily_sapro;
{bedrock layer: weathering of minerals on a daily basis}

implementation

uses untGUI, untDecayProtection;

{ *******************************************************************}
{ *******************************************************************}
{ **                                                               **}
{ **                       L E A C H C                             **}
{ **                                                               **}
{ **                       Version 3.0                             ** }
{ **                          1992                                 **}
{ **                                                               **}
{ **     A process- based model of water and inorganic chemical    **}
{ **             movement and reactions in the                     **}
{ **                    unsaturated zone.                          **}
{ **                                                               **}
{ **                           by                                  **}
{ **                                                               ** }
{ **               J.L. Hutson and R.J. Wagenet                    **}
{ **                                                               **}
{ **        Department of Soil, Crop and Atmospheric Sciences,     **}
{ **                   Cornell University                          **}
{ **                   Ithaca, N.Y. 14853                          **}
{ **                                                               **}
{ **            Copyright 1996, Cornell University                 **}
{ **                                                               **}
{ *******************************************************************}
{ *******************************************************************}
{ **                                                               ** }
{ **  This software is distributed without warranty. Distribution  **}
{ **  or resale without the permission of the authors or the       **}
{ **  Department of Soil, Crop and Atmospheric Sciences,           **}
{ **  Cornell University, is not permitted.                        **             }
{ **                                                               **}
{ ******************************************************************* }
{ *******************************************************************}
procedure StartProcesses;
{Process driver1: first initialization, modified after LEACHC}
{  All arrays used in the common blocks are initialized here.}
{  They may be reinitialized in MAINC, which was originally the main}
{  program.  Some compilers require the arrays to be initialized, so this}
{  initialization was added.}
{---------------------------------------------------------------------}
{  Common variables and parameters are defined in the following:}
{PM:      INCLUDE 'PARMS.INC'
      INCLUDE 'BLOCKS.INC'
      INCLUDE 'BLOCKC.INC'}
{----------------------------}
{  Variables in BLOCKS.INC}
var
  I, J, M, ISS: integer;
begin
{ StartProcesses 1 } {process driver | Various initializations}
  untGUI.frmMAIN.MVAR(GlobalTime*10000+Datayearin.nolayers*100,' start untProcesses.StartProcesses: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
  untGUI.frmMAIN.MVAR(GlobalTime*10000+Datayearin.nolayers*100,' start untProcesses.StartProcesses: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);
  {recognize 2 RFM-variables from calling program}
  RFMamorphite:=untGUI.RFMamorphite;
  RFMotherite:=untGUI.RFMotherite;
  system.Assign(ftmp, 'tmpdmp.txt');
  if fileexists('tmpdmp.txt') then
    system.Append(ftmp)
  else
    system.Rewrite(ftmp);
  externalcall := False;
  VGNactiveLOCAL := False;
  {system.assign(ftrial,'trials.txt');
  if not fileexists('trials.txt') then system.Rewrite(ftrial) else system.append(ftrial); }
  for I := 1 to 10 do
  begin
    freqdelt[I] := 0;
  end;
  typechem := untGUI.TypeChem;
  untGUI.Chemstarted := True;
  count10 := 1;
  bf1 := False;
  bf9 := False;
  bf10 := False;
  bf11 := False;
  bf12 := False;
  bf13 := False;
  bf14 := False;
  bf15 := False;
  bf16 := False;
  bf17 := False;
  bf40 := False;
  bf45 := False;
  debugpoint := False;
  {if fileexists('LEACHC.log')
  then
  begin assign(flog,'LEACHC.log'); append(flog); end
  else
  begin}
  if bflog then
  begin
    system.Assign(flog, 'LEACHC.log');
    system.rewrite(flog);
  end;
  {end;}
  {next 4 lines added by pf2006 to initialise temperaturecounter+claystabilitycounter}
  tempcount := 0;
  tempcountmonth := 0;
  claycount := 0;
  oldmonth := 1;
  oldday := 1;
  {next lines added pf08:
   assign the old LEACHC solubility constants and other equilibrium constants
   their value from the values read by the calling program (at 25oC at this point)}
  AdaptConst(25);

  ADAY := 0;
  AMONTH := 0;
  AYEAR := 0;
  BBC := 0;
  CROPNO := 0;
  CUMT := 0;
  DAY := 0;
  DAY1 := 0;
  DAYM := 0;
  IDATE := 0;
  IFGTEM := 1;
  IFILE := 0;
  IFLG2 := 0;
  IFLG3 := 0;
  IHEAD := 0;
  IRRNO := 0;
  JK := 0;
  KILPLT := 0;
  KK := 0;
  KLPLT := 0;
  MONTH := 0;
  MPORE := 0;
  NCROP := 0;
  NIRRIG := 0;
  NN := 0;
  NPRINT := 0;
  NYC := 0;
  PRIOR := 0;
  IFLOW := 0;
  PRNTAB := 0;
  PRNTOP := 0;
  R30 := 0;
  R54 := 0;
  R9 := 0;
  {TRACE:=0;}
  WEEK := 0;
  YEAR := 0;

  ABBPOT := 0;
  ADDFC := 0;
  ADDTH := 0;
  AK2 := 0;
  AWATER := 0;
  BBPOT := 0;
  CEVAPA := 0;
  CLW := 0;
  CONQ := 0;
  CRAIN := 0;
  CTRANA := 0;
  DELM := 0;
  WTRANA := 0;
  DELT := 0;
  DELWAT := 0;
  {DETT:=0;}
  DLYEP := 0;
  DLYPAN := 0;
  DLYTP := 0;
  DLZ := 0;
  ER := 0;
  ERMAX := 0;
  EXCESS := 0;
  FMAX := 0;
  HDRY := 0;
  HEAD := 0;
  HHIGH := 0;
  HLOW := 0;
  HROOT := 0;
  IRATE := 0;
  IRRBEG := 0;
  IRREND := 0;
  IRRIG := 0;
  NSUM := 0;
  PWATER := 0;
  R20 := 0;
  RRES := 0;
  THETA1 := 0;
  TIMDAY := 0;
  TIME := 0;
  TRANA := 0;
  TRANP := 0;
  TRL := 0;
  WATBAL := 0;
  WFDD := 0;
  PRINT := 0;
  PDEPTH := 0;
  FWBTC := 0;

  for I := 1 to LL do
  begin
    A[i] := 0;
    AAEV[i] := 0;
    ABCAM[i] := 0;
    AEV[i] := 0;
    AH[i] := 0;
    AKS[i] := 0;
    RP[i] := 0;
    ATEM[i] := 0;

    {added pf2006: next 2 lines}
    TEMPGEM[i] := 0;
    ClayDispInd[i] := 0;

    AW[i] := 0;
    B[i] := 0;
    BCAM[i] := 0;
    CMATCH[i] := 0;
    CTRAN[i] := 0;
    FLUX[i] := 0;
    FLXDEN[i] := 0;
    G[i] := 0;
    H[i] := 0;
    HM[i] := 0;
    HMATCH[i] := 0;
    KS[i] := 0;
    L[i] := 0;
    RDF[i] := 0;
    RG[i] := 0;
    RHO[i] := 0;
    RL[i] := 0;
    SALT[i] := 0;
    TEM[i] := 0;
    TG[i] := 0;
    TM[i] := 0;
    TRAN[i] := 0;
    TS[i] := 0;
    W[i] := 0;
    WATER[i] := 0;
    WFW[i] := 0;
    WTRAN[i] := 0;
    Y[i] := 0;
    FC[i] := 0;
    WP[i] := 0;
    D[i] := 0;
  end;

  for I := 1 to IWEEK do
  begin
    AMP[i] := 0;
    AVETEM[i] := 0;
    ETIME[i] := 0;
    PAN[i] := 0;
    WTDEP[i] := 0;
  end;

  for I := 1 to IPNT do
    PRTIME[i] := 0;

  for I := 1 to IPPT do
  begin
    AIRRBG[i] := 0;
    IRATEA[i] := 0;
    IRRIGA[i] := 0;
  end;

  for I := 1 to NC do
  begin
    AK1[i] := 0;
    CRPCOV[i] := 0;
    DAYHAR[i] := 0;
    ESTART[i] := 0;
    ESTOP[i] := 0;
    PANFAC[i] := 0;
    PD[i] := 0;
    PDAY[i] := 0;
    RDEP[i] := 0;
    RDFDAY[i] := 0;
    rtype[i] := 0;
  end;

  for I := 1 to 3 do
  begin
    GEOMOM[i] := 0;
    GEOMOS[i] := 0;
    GEOMQ[i] := 0;
  end;

  for ISS := 1 to 4 do
  begin
    ND[ISS] := 0;
    for I := 1 to LL do
      PART[ISS, I] := 0;
  end;

  for I := 1 to 5 do
    ALAMO[i] := 0;

  {  Variables in BLOCKC.INC }
  { Integers}
  {NCHEM:=0; }
  IFR := 0;
  ITERCH := 0;
  NN1 := 0;
  { Reals}
  CDCO2 := 0;
  CELLW := 0;
  DIFA := 0;
  DIFB := 0;
  DIFO := 0;
  PPTS1 := 0;
  PPTS6 := 0;
  PPTS8 := 0;
  PPTS12 := 0;
  SOLS1 := 0;
  SOLS6 := 0;
  SOLS8 := 0;
  SOLS12 := 0;
  { LL arrays}
  for I := 1 to LL do
  begin
    ACEC[i] := 0;
    APCO2[i] := 0;
    APH[i] := 7;
    CEC[i] := 0;
    DELCO2[i] := 0;
    DelGibb[i] := 0;   {pf08}
    DELGYP[i] := 0;
    DECALCITE[i] := 0;
    EC[i] := 0;
    ESP[i] := 0;
    GYP[i] := 0;
    GYPA[i] := 0;
    LAMBDA[i] := 0;
    CALCITE[i] := 0;
    CALCITEA[i] := 0;
    PCO2[i] := 0;
    PH[i] := 0;
    SAR[i] := 0;
      {SC1=Mg/Ca; SC2=Ca/Na; SC3=Ca/K;  SC4=Mg/K;  SC5= Mg/Na; SC6= K/Na;
       SC7=Ca/Al; SC8=Ca/H;  SC9=Mg/Al; SC10=K/Al; SC11= Na/Al; SC12=Mg/H;
       SC13=K/H;  SC14=Na/H; SC15=Al/H }
    SC1[i] := 0;
    SC2[i] := 0;
    SC3[i] := 0;
    SC4[i] := 0;
    SC5[i] := 0;
    SC6[i] := 0;
    SC7[i] := 0;
    SC8[i] := 0;
    SC9[i] := 0;
    SC10[i] := 0;
    SC11[i] := 0;
    SC12[i] := 0;
    SC13[i] := 0;
    SC14[i] := 0;
    SC15[i] := 0;
    TOTANS[i] := 0;
    TOTCAT[i] := 0;
  end;
  {  MM arrays}
  for M := 1 to MM do
  begin
    APDSE[m] := 0;
    BALSE[m] := 0;
    BBSE[m] := 0;
    CFER[m] := 0;
    CLSE[m] := 0;
    CRSE[m] := 0;
    CUSE[m] := 0;
    {pf2006: initialize cumulative mineralization of OM into chemicals}
    CMINER[m] := 0;

    DELSE[m] := 0;
    LSE[m] := 0;
    PDSE[m] := 0;
    VAL[m] := 0;
    CMASS[m] := 0;
  end;

  for M := 1 to MM do
  begin
    for I := 1 to LL do
    begin
      AXSE[m, i] := 0;
      SE[m, i] := 0;
      SEA[m, i] := 0;
      SS[m, i] := 0;
      XSE[m, i] := 0;
    end;
  end;
  for M := 1 to MM do
    for J := 1 to 4 do
    begin
      FSE[m, j] := 0;
      TSE[m, j] := 0;
    end;

  for J := 1 to NFR do
  begin
    DFER[J] := 0;
    INCOR[J] := 0;
    for M := 1 to MM do
      FER[m, j] := 0;
  end;

  for M := 1 to MM do
    for J := 1 to IPPT do
      ARSE[m, j] := 0;

  { StartProcesses 2 } {process driver | call MAINC}
  MAINC;

  { StartProcesses 3 } {process driver | call STOPC}
  {      STOP}
  wwriteln(' Simulation complete.');
  wwriteln(' Open ' + file1 + ' in a text editor.');
  wwriteln('  Files ' + file1 + ' and ' + file2 + ' can be imported ');
  wwriteln(' into a spreadsheet or graphing package.');
  {      counttim:=counttim/icounttim}
  {      write(11,*)' Average time between 2 chem-calls (d):',}
  {     #counttim}
  { above 3 lines added by PF2006 to optimize equilibration frequency}
  STOPC;
  untGUI.Chemstarted := False;
  Close(ftmp);
end;

procedure MAINC;
{Process driver2: continued initialization, modified after LEACHC}
{ *********************************************************************}
{ *********************************************************************}
{ **                                                                 **}
{ **                       L E A C H C                               **}
{ **                                                                 **}
{ **                                                                 **}
{ **     A process-based model of water and inorganic chemical       **}
{ **                 transport and reactions                         **}
{ **                 in the unsaturated zone                         **}
{ **                                                                 **}
{ **                           by                                    **}
{ **                                                                 **}
{ **                      John L. Hutson                             **}
{ **                                                                 **}
{ **                                                                 **}
{ **        The Flinders University of South Australia               **}
{ **       GPO Box 2100, Adelaide, South Australia 5001              **}
{ **                                                                 **}
{ ** Copyright 1985-97, John Hutson, R.J.Wagenet                     **}
{ ** Copyright 1998- 2002 John Hutson                                **}
{ *********************************************************************}
{ *********************************************************************}
{ **                                                               ** }
{ **  This software is distributed without warranty. Distribution  **}
{ **  or resale without the permission of the authors or the       **}
{ **  Department of Soil, Crop and Atmospheric Sciences,           **}
{ **  Cornell University, is not permitted.                        **             }
{ **                                                               **}
{ ******************************************************************* }
{ *******************************************************************}
{** Code translated from Fortram to Pascal by Peter Finke, with many additions and adaptations **}

{  I is the node number, ranging from 1 to KK. Nodes 1 and KK are}
{  outside of the soil profile, therefore nodes 2 to K are used for}
{  mass balancing.}

{  SEG is the layer number used in the summary output file (*.SUM).}
{  These depths are defined in the input or default to thirds of the }
{  profile depth. Depths are approximated to the nearest node.}
{    SEG=1 : Surface to 1/3 of profile depth (or defined depth ND(1)),}
{    SEG=2 : 1/3 to 2/3 of profile depth (or depths ND(1) to ND(2)).}
{    SEG=3 : 2/3 to bottom of profile (or ND(2) to ND(3)),}
{    SEG=4 : Surface to bottom of root zone (node 2 to ND(4)).}
{ *******************************************************************}

{  Parameter, variable, and array definitions and declarations follow.}
{  Variable names and definitions are as uniform as possible throughout}
{  all versions and subroutines.}
{---------------------------------------------------------------------}
{  Common variables and parameters are defined in the following:}
{pm:      INCLUDE 'PARMS.INC'
      INCLUDE 'BLOCKS.INC'
      INCLUDE 'BLOCKC.INC'}
{--------------------------------------------------------------------}
{  In .SUM file: Water flux, cumulative flux (mm), water content (mm)}
{       Real          FW(4),    CFW(4),                 TH(4)}
{       Sources:  rain/irrig    Dry chemical (depth of incorporation)}
{********************************************************************}
var
  i: integer;
begin
  { MAINC 1 } {process driver | Assign and open intra-annual input- and output files}
  untGUI.frmMAIN.MVAR(GlobalTime*10000+Datayearin.nolayers*100,' start untProcesses.MAINC: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
  untGUI.frmMAIN.MVAR(GlobalTime*10000+Datayearin.nolayers*100,' start untProcesses.MAINc: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);

  OK14C := untGUI.OK14C;
  iyrtot := 1;
  N1PRT := 0;
  iautoirrg := 0;
  {  Open input data file}
  wwriteln('-----------------------------------------------------');
  wwriteln('   If you have not prepared a data file:');
  wwriteln('   An input data file for LEACHC is created by copying and');
  wwriteln('   renaming the sample data file CHEMTEST.  The data in the');
  wwriteln('   file is edited to reflect the scenario to be simulated.');
  wwriteln('   Proceed with the simulation if a data file is ready.');
  wwriteln('   Otherwise, stop this run and prepare a data file.');
  wwriteln('-----------------------------------------------------');
  wwriteln('  Enter input data file name > ');
  wwriteln(FILE0);
  system.Assign(f10, FILE0);
  system.reset(f10);
  bf10 := True;
  {  Remove or replace this section to suit desired file specification.}
  { All commented lines below are no longer used code fragments from the original Fortran code}
  {   Data - unit 10; Output - unit 11; Daily summary - unit 12;}
  {   Breakthrough curve - unit 13}
  {   Additional output  - units  14.....)}
  {   Temporary file, containing names of output files, opened here.}
{      system.assign(f9,'TITLE'); rewrite(f9);
      system.WRITELN(f9,FILE0+'.OUT');
     system.writeLN(f9,FILE0+'.SUM');
     system.writeLN(f9,FILE0+'.BTC');
     system.writeLN(f9,FILE0+'.IRR');
     system.writeLN(f9,FILE0+'.SCH');
     system.writeLN(f9,FILE0+'.sal');
     system.writeLN(f9,FILE0+'.wat');
     system.writeLN(f9,FILE0+'.mat');
      closefile(f9);  }
  {----------------------------}
{     system.reset(f9);
      system.readLN(f9,FILE1);
      system.readLN(f9,FILE2);
      system.readLN(f9,FILE3);
      system.readLN(f9,FILE5);
      system.readLN(f9,FILE6);
      system.readLN(f9,FILE15);
      system.readLN(f9,FILE16);
      system.readLN(f9,FILE17);
      closefile(F9); }
  FILE1 := FILE0 + '.OUT';
  FILE2 := FILE0 + '.SUM';
  FILE3 := FILE0 + '.BTC';
  FILE5 := FILE0 + '.IRR';
  FILE6 := FILE0 + '.SCH';
  FILE15 := FILE0 + '.sal';
  FILE16 := FILE0 + '.wat';
  FILE17 := FILE0 + '.mat';
  {----------------------------}
  {  Open *.OUT file}
  system.Assign(f11, FILE1);
  system.rewrite(f11);
  bf11 := True;
  system.writeLN(f11, FILE1); system.Flush(f11);
  {------------------------------}
  wwriteln(' Starting to read data');

  { MAINC 2 } {process driver | call READCNEW (read soil data from end of previous year)}
  {  Call data read}
  READCNEW; {reads data from memory; was: READCOLD (read data from LEACHC inputfile)}

  { MAINC 3 } {process driver | call RothC_Init (initialisation and read input file for C-submodel, status end of previous year)}
  {pf2006: initialisation and read input file for C-submodel (next 2 lines)}
  RCName := 'RC' + copy(FILE0, 3, length(FILE0) - 2);
  RothC_Init(RCName);

  { MAINC 4 } {process driver | call WE_Init (initialisation and read input file for Weathering submodel, status end previous year)}
  {pf08: initialisation and read input file for Weathering submodel}
  WE_init;

  { MAINC 5 } {process driver | Conditional: if bedrock then call WE_Init_sapro (initialisation and read input file for Weathering submodel for bedrock, status end previous year)}
  if untGUI.rock.RockBottomLayers>0 then WE_init_sapro;

  { MAINC 6 } {process driver | call Init_Clay_Migration (initialisation for clay migration submodel via mass of clay silt + sand at end previous year)}
  {pf08: initialisation for clay migration submodel}
  Init_Clay_Migration;
      {in case there is weathering checked, but clay migration is not simulated,
       the clay mass pool needs initialisation as well:
       No longer applies: masses of clay/silt/sand/oc are passed between chem and shell units
      if not(claymigrationallowed) then
      for i:=2 to kk-1 do CLAYm[i]:=CLAY[i]*DLZ*RHO[i]/100;  these are kg/m2}

  {----------------------------}
  {  Open *.SUM file}
  IFGSUM := 0;
  if (IFILE >= 2) then
  begin
    system.Assign(f12, FILE2);
    system.rewrite(f12);
    bf12 := True;
    IFGSUM := 1;
  end;
  {----------------------------}
  {  Open *.BTC (breakthrough curve) file}
  IFGBTC := 0;
  if (IFILE >= 3) then
  begin
    system.Assign(f13, FILE3);
    system.rewrite(f13);
    bf13 := True;
    IFGBTC := 1;
  end;
  {----------------------------}
  {  Open *.irr (automated irrigation applications) file}
  if (ch_irr_flag = 1) then
  begin
    system.Assign(f45, FILE5);
    system.rewrite(f45);
    bf45 := True;
  end;
  {----------------------------}
  { Files for custom writes}
  if (IFILE >= 4) then
  begin
    system.Assign(f15, FILE15);
    system.rewrite(f15);
    bf15 := True;
    system.Assign(f16, FILE16);
    system.rewrite(f16);
    bf16 := True;
    system.Assign(f17, FILE17);
    system.rewrite(f17);
    bf17 := True;
  end;
  {--------------------------------------------------------------------  }
  {  Bulk of the subroutine is identical for LEACHC and LEACHCG.}
  {   INCLUDE file called MAIN.INC.}
  {      INCLUDE 'MAINC.INC'}
  { MAINC 7 } {process driver | call MAINC2}
  mainc2(IFGSUM);
end;

procedure MAINC2(var IFGSUM: integer);
{Process driver3: start of program and the master of the process flow. Temporal resolution of days-hours-smaller}
{----------------------------      }
{  INITIALIZE VARIABLES}
{----------------------------}
{      counttim:=0.}
{      icounttim:=0}
{ above 2 lines added by PF2006 to estimate real time between equilibria reset, not used at present}
{------------------}
{ These RR variables are used to accumulate annual totals.  They are }
{ initialized to zero each year (on the start day and month) if that}
{ option was required (set through INITFLG)}
var
  i, j, m, seg, IT, mCYCNO, mDAY: integer;
begin
  { MAINC2 1 } {process driver | Continued initialization and dimensioning of model}
  untGUI.frmMAIN.MVAR(GlobalTime*10000+Datayearin.nolayers*100,' start untProcesses.MAINC2: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
  untGUI.frmMAIN.MVAR(GlobalTime*10000+Datayearin.nolayers*100,' start untProcesses.MAINc2: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);

  RR1 := 0;
  RR1i := 0;
  RR2 := 0;
  RR3 := 0;
  RR4 := 0;
  RR5 := 0;
  RR6 := 0;
  RR7 := 0;
  RR8 := 0;
  RR9 := 0;
  RR10 := 0;
  RR11 := 0;
  RR12 := 0;
  RR13 := 0;
  RR23 := 0;
  RR24 := 0;
  FLAGC := 0;
  {  Convert kPa to mm water hydraulic head}
  HHIGH := HHIGH * 102.2;
  HLOW := HLOW * 102.2;
  HDRY := HDRY * 102.2;

  {  Volume (dm3) associated with 1 m2 of each depth segment (for equal}
  {   depth increments VOLUME is constant)}
  VOLUME := 1 * DLZ;

  {  Reset starting date}
  DAYM := ADAY;
  MONTH := AMONTH;
  YEAR := AYEAR;
  IY := YEAR;
  IM := MONTH;
  IDAYM := DAYM;

  {  Reset root distribution (for growing roots)}
  if (igrowth[MAX(1, CROPNO)] = 2) then
  begin
    TRL := 0;
    RDZ := 0;
    for I := 1 to KK do
      RDF[i] := 0;
  end
  {  Adjust R values to sum to 1 for constant root distribution}
  else
  if (igrowth[MAX(1, CROPNO)] = 1) then
  begin
    RDFTOT := 0;
    for J := 2 to KK - 1 do
      RDFTOT := RDFTOT + RDF[j];
    for J := 2 to KK - 1 do
    begin
      if (RDFTOT <= 0) then
        RDF[j] := 0;
      if (RDFTOT > 0) then
        RDF[j] := RDF[j] / RDFTOT;
      if (RDF[j] > 0) then
        RDZ := (J - 1) * DLZ;
    end;
  end; {if}
  {----------------------------}
  {  Set node numbers at which daily water and chemical fluxes are to be }
  {   recorded in the .SUM file (unit 12). There are three of these, ND1}
  {   to ND3. ND4 is the bottom of the root zone which may vary with time.}
  {   The lowest node is KK-1 as flux from the bottom of the profile }
  {   (between nodes KK-1 and KK) is designated KK-1. ND4 corresponds to }
  {   the bottom of the root zone and is calculated later.}
  for SEG := 1 to 3 do
    ND[SEG] := round(ND[SEG] / DLZ) + 1;
  if (ND[1] > KK) or (ND[1] < 2) then
    ND[1] := round(1 + (PDEPTH / (3 * DLZ)));
  if (ND[2] > KK) or (ND[2] <= 2) then
    ND[2] := round(1 + (2 * PDEPTH / (3 * DLZ)));
  if (ND[3] > KK) or (ND[3] <= 2) then
    ND[3] := KK - 1;
  ND[4] := KK - 1;
  DEP1 := (ND[1] - 1) * DLZ;
  DEP2 := (ND[2] - 1) * DLZ;
  DEP3 := (ND[3] - 1) * DLZ;
  str(dep1: 7: 1, txt1);
  str(dep2: 7: 1, txt2);
  str(dep3: 7: 1, txt3);
  wwriteln(' Depth boundaries in summary file set at ' + txt1 + '  ' + txt2 +
    ' and ' + txt3 + ' mm.');
  system.writeln(f11, ' Depth boundaries in summary file set at ' + txt1 + '  ' + txt2 +
    ' and ' + txt3 + ' mm.');  system.Flush(f11);
  {-------------------------        }

  for I := 1 to KK do
  begin
    W[i] := AW[i];
    Y[i] := W[i];
    L[i] := 0;
    RG[i] := 0;
    B[i] := 0;
    TEM[i] := ATEM[i];
    {----------------------------}
    {  Convert kPa to mm water hydraulic head}
    H[i] := 102.2 * AH[i];
    G[i] := H[i];
    {----------------------------}
    {  Hydraulic conductivity and retention parameters}
    AEV[i] := AAEV[i];
    BCAM[i] := ABCAM[i];
    KS[i] := AKS[i];
  end;

  { MAINC2 2 } {process driver | call WATDAT to create an output table (part of *.out) of h-theta-K data}
  {  Examine hydrological properties }
  WATDAT;
  wwriteln(' K-Theta-h relations calculated and written to .OUT file. ');
  wwriteln(' ');
  {----------------------------     }

  { MAINC2 3 } {process driver | call THETA to estimate microporosity and air-filled porosity using H}
  {pf08: estimate microporosity per layer by calculating the water content at <input> mm hydraulic head}
  for i := 2 to kk - 1 do
    microporosity[i] :=
      THETA(i, untGUI.MacroMicroMM, RG[i], TS[i], AEV[i], BCAM[i], L[i], TM[i], HM[i]);
  {  Use profile potential or theta values from data file according   }
  {   TO VALUE OF PRIOR    }
  if (PRIOR = 2) then {H used to calc theta; not the case in default input file}
  begin
    for I := 1 to KK do
    begin
      W[i] := THETA(I, H[i], RG[i], TS[i], AEV[i], BCAM[i], L[i], TM[i], HM[i]);
      {   Air-filled pore fraction, minimized to 0.01 in case of low porosity}
      TG[i] := max(0.01,TS[i] - W[i]);
      Y[i] := W[i];
    end;
  end
  else {theta used to calc h; this is the case in default input file}
  begin
 { MAINC2 4 } {process driver | call POTL (using initial theta and depending on bottom boundary condition)}
    for I := 1 to KK do
    begin
      H[i] := POTL(W[i], VGNtr[i], VGNts[i], VGNal[i], VGNen[i], VGNem[i]);
      {   Air-filled pore fraction, minimized to 0.01}
      TG[i] := max(0.01,TS[i] - W[i]);
      G[i] := H[i];
    end;
  end;
  {----------------------------}
  {  Adjust pressure potentials in the saturated zone if a water}
  {  table is present.}
  BBPOT := ABBPOT;
  if (BBC = 1) or (BBC = 5) then
  begin
    NODEWT := KK - trunc(BBPOT / DLZ);
    DIFF := BBPOT - (KK - NODEWT) * DLZ;
    if (NODEWT < KK) then
      for I := NODEWT to KK - 1 do
        H[i] := DIFF + DLZ * (I - NODEWT);
    {  First few nodes above the phreatic surface set to avoid large initial}
    {  potential gradients.}
    for I := NODEWT - 1 downto MAX(1, NODEWT - 4) do
      if (I <= KK) then
        H[i] := MAX(H[i], DIFF - DLZ * (NODEWT - I));
    for I := 1 to KK do
    begin
      Wnew := THETA(i, H[i], RG[i], TS[i], AEV[i], BCAM[i], L[i], TM[i], HM[i]);
      {  Change in water content}
      Wchange := Wnew - W[i];
      {  Extra solute from water table       }
      for m := 1 to nchem do
      begin
        Solute_added := Wchange * bbse[m];
        sea[m, i] := (sea[m, i] * w[i] + solute_added * wchange) / (wchange + w[i]);
      end;

      w[i] := Wnew;
      Y[i] := W[i];
      G[i] := H[i];
    end;
  end; {if}
  WEEKO := 0;

  { MAINC2 5 } {process driver | calculate fieldcapacity and wilting point}
  for i := 1 to kk do
  begin
    R13 := 102.2 * fc[i];
    fc[i] := THETA(i, R13, RG[i], TS[i], AEV[i], BCAM[i], L[i], TM[i], HM[i]);
    r13 := -1500. * 102.2;
    wp[i] := THETA(i, r13, RG[i], TS[i], AEV[i], BCAM[i], L[i], TM[i], HM[i]);
    R13 := 102.2 * D[i];
    d[i] := THETA(i, r13, RG[i], TS[i], AEV[i], BCAM[i], L[i], TM[i], HM[i]);
  end;

 { MAINC2 6 } {process driver | Initialize chemical parameters (i.e. valence of species) }
  {********************************************************************}
  {********************************************************************}
  {  Initialize chemical parameters and data}
  {********************************************************************}
  {   Solubility products of gypsum (SP1) and calcite (SP2)}
{  these are already set (PF2006) as constants
{ SP1:=.24E-4
  SP2:=.113E-7  }
  {********************************************************************}
  IFGRAT := 0;
  {   Valency of chemical species}
  VAL[1] := 2;   {Ca}
  VAL[2] := 2;   {Mg}
  VAL[3] := 1;   {Na}
  VAL[4] := 1;   {K}
  VAL[5] := 1;   {Cl}
  VAL[6] := 2;   {SO4}
  VAL[7] := 1;   {HCO3}
  VAL[8] := 2;   {CO3}
  VAL[9] := 1;   {Balion}
  VAL[10] := 1;  {H}
  VAL[11] := 1;  {OH}
  VAL[12] := 3;  {Al}
  VAL[13] := 3;  {was Org now FeIII}
  VAL[14] := 2;  {was HOrg now FeII}
  VAL[15] := 0;  {was H2Org now H4SiO4. PF2023: changed from 1 to 0}
  VAL[16] := 0;  {DOC, used to calculate CORG in this programme}
  VAL[17] := 0;  {dispersible clay assumed to have no charge}
  {  Adjust input soil chemical concentrations to reflect equilibrium.}
  {   Calculates equilibrium H, OH, CO3 and HCO3}
  for I := 2 to KK - 1 do
  begin
    str(I - 1, txt1);
    wwriteln(' Bringing segment' + txt1 + ' into chemical equilibrium.');
    {  calcite and gypsum are converted from decimal fraction on a weight  }
{   basis to mmoles/dm**3 soil.  (kg/kgx1000:=(mg/kg)/(mg/mmol)
   CHANGED pf10: from kg/compartment (obtained from readc routines to mmoles/dm**3 soil }
    GYPA[i] := GYPA[i] * 1000000 / (172 * DLZ);
    CALCITEA[i] := CALCITEA[i] * 1000000 / (100 * DLZ);
    {gibbsite does not need conversion, is in mmol/sq.m as expected in chem2}
    R1 := 0;
    R2 := 0;
    R3 := 0;
    R4 := 0;
    R5 := 0;
    R6 := 0;
    R7 := 0;
    R8 := 0;
    r17 := 0;
    r18 := 0;
    R3a1 := 0;
    {!!!}
    debugpoint := True;

    { MAINC2 7 } {process driver | call CHEM2 (to bring compartments in chemical equilibrium and calc pH)}
    SEFeII:=0; SEFeIII:=0; {FeII and FeIII conc for call to Chem2}
    case typechem of
      1:  begin end;
      2:
        CHEM2(CALCITEA[i], GYPA[i], Gibbsitepool[i], APH[i], RHO[i], SEA[12, i],
          SEA[1, I], SEA[2, I], SEA[3, I], SEA[4, I], SEA[5, I], SEA[6, I], SEA[7, I], SEA[8, I],
          SEA[9, I], SEA[10, I], SEA[11, I], W[i], R1, R2,
          AXSE[1, I], AXSE[2, I], AXSE[3, I], AXSE[4, I], AXSE[10, i], AXSE[12, i], ACEC[i],
          R3, SC1[i], SC2[i], SC3[i], SC4[i], SC5[i], SC6[i],
          SC7[i], SC8[i], SC9[i], SC10[i], SC11[i], SC12[i], SC13[i], SC14[i], SC15[i],
          R3a1, R4, R5, APCO2[i], NN1,
          SP1, SP2, R17, trace, r18, n1prt, atem[i]);
      3:begin end;
    end;
  end; { 9080 CONTINUE}

  { MAINC2 8 } {process driver | call calcPHLAB (calculate pH-H2O at virtual 1:5 dilution)}
  calcphlab(RHO, APH, AW, PHLAB, DLZ);{calculate pH-H2O in 1:5 dilution as in lab protocol}

  { MAINC2 9 } {process driver | void: set exchangeable and soluble ions using the Gapon coefficients (in SoilGen Gapon=input)}
  if (nn1 = 1) then  {nn1 is 0, thus not used in SoilGen}
  begin
    wwriteln(' Calculated selectivity coefficients are:');
    system.writeln(f11, '             Calculated selectivity coefficients ');
    system.writeln(f11, 'Segment           Selectivity coefficients');
    {SC1=Mg/Ca; SC2=Ca/Na; SC3=Ca/K;  SC4=Mg/K;  SC5= Mg/Na; SC6= K/Na;
     SC7=Ca/Al; SC8=Ca/H;  SC9=Mg/Al; SC10=K/Al; SC11= Na/Al; SC12=Mg/H;
     SC13=K/H;  SC14=Na/H; SC15=Al/H }
    for i := 2 to kk - 1 do
    begin
      str(i - 1, txt1);
      str(SC1[i]: 15: 8, txt2);
      str(SC2[i]: 15: 8, txt3);
      str(SC3[i]: 15: 8, txt4);
      txt1 := txt1 + ' ' + txt2 + ' ' + txt3 + ' ' + txt4;
      str(SC4[i]: 15: 8, txt2);
      str(SC5[i]: 15: 8, txt3);
      str(SC6[i]: 15: 8, txt4);
      txt1 := txt1 + ' ' + txt2 + ' ' + txt3 + ' ' + txt4;
      str(SC7[i]: 15: 8, txt2);
      str(SC8[i]: 15: 8, txt3);
      str(SC9[i]: 15: 8, txt4);
      txt1 := txt1 + ' ' + txt2 + ' ' + txt3 + ' ' + txt4;
      str(SC10[i]: 15: 8, txt2);
      str(SC11[i]: 15: 8, txt3);
      str(SC12[i]: 15: 8, txt4);
      txt1 := txt1 + ' ' + txt2 + ' ' + txt3 + ' ' + txt4;
      str(SC13[i]: 15: 8, txt2);
      str(SC14[i]: 15: 8, txt3);
      str(SC15[i]: 15: 8, txt4);
      txt1 := txt1 + ' ' + txt2 + ' ' + txt3 + ' ' + txt4;
      wwriteln(txt1);
      system.writeln(f11, txt1);
    end;
    system.Flush(f11);
  end; {if }
  NN1 := 0;
  for M := 1 to MM do
  begin
    SEA[M, 1] := SEA[M, 2];
    SEA[M, KK] := SEA[M, KK - 1];
  end;

  for I := 1 to kk do
  begin
    for M := 1 to MM do
    begin
      XSE[M, I] := AXSE[M, I];  {was: m:=1 TO 8 BUT NEW SPECIES ADDED}
      XSE_clay[M, I] := AXSE_clay[M, I];
      XSE_oc[M, I] := AXSE_oc[M, I];
    end;
    for M := 1 to MM do
      SE[M, I] := SEA[M, I];    {was: m:=1 TO 11 BUT NEW SPECIES ADDED}
    PH[i] := APH[i];
    PCO2[i] := APCO2[i];
    CEC[i] := ACEC[i];
    CALCITE[i] := CALCITEA[i];
    GYP[i] := GYPA[i];
  end;

  { MAINC2 10 } {process driver | Repeat for all cycles=repetitions of 1 year (but SoilGen has only 1 cycle)}
  {*******************************************************************}
  {  If more than one year or cycle is to be simulated then all}
  {   variables and counters are initialized here. Concentrations at the}
  {   end of a cycle are used as starting values for the next. All rain,}
  {   irrigation, chemical applications are repeated each cycle if NYC}
  {   is greater than 1. If CUMT (last day no) is >365 and NYC := 1 then}
  {   the specified CUMT value is used. }
  {      if (NYC>1)CUMT:=MIN0(CUMT,365)}
  for mCYCNO := 1 to NYC do {Do 1111 loop}
  begin
    { MAINC2 11 } {process driver | set or reset counters and accumulators for water and solute}
    CYCNO := mCYCno;
    iinitialize := 1;
    DAYM := ADAY;
    MONTH := AMONTH;
    YEAR := AYEAR;
    IY := YEAR;
    IM := MONTH;
    IDAYM := DAYM;
    {----------------------------}
    {   Water application counters}
    IRRNO := 1;
    {   Crop counter  }
    CROPNO := 0;
    {  Set first irrigation time, rate and amount}
    IRRBEG := AIRRBG[irrno];
    IRATE := IRATEA[irrno];
    IRRIG := IRRIGA[irrno];
    {   Print counters:}
    {     Print number (option 3)}
    NPRIN := 1;
    {     Flag for print after 1st time step (thereafter 1)}
    N1PRT := 0;
    {     Time elapsed between .SUM file prints}
    RSUME := 0;
    {----------------------------}
    {   Max water flux in profile set high to ensure a small 1st time step}
    FMAX := power(10, 10);
    {   Time elapsed in print period}
    R24 := 0;
    {   Time step}
    DELT := 0;
    {   End of irrigation (set in WATFLO)}
    IRREND := 0;
    {   Bottom boundary condition (appropriate value set in WATFLO)   }
    R9 := 0;
    {  Calls to heat routine}
    JK := 1;
    {   Flags }
    ITIME_TO_PRINT := 0;
    IFLG2 := 0;
    IFLG3 := 0;
    {   Number of iterations within WATFLO (set in TSTEP)}
    R30 := 0;
    {   Crop cover fraction (set in GROWTH)}
    CRPCOV[1] := 0;
    {   Water remaining to infiltrate (mm)}
    HEAD := 0;
    {   Potential daily transpiration and evaporation (mm)  }
    DLYTP := 0;
    DLYEP := 0;
    {   Actual transpiration during a time step (profile total) mm}
    TRANA := 0;
    {   Potential transpiration for a time step mm}
    TRANP := 0;
    {   Actual evaporation flux density during step}
    EFLDEN := 0;
    {   Surface water flux density}
    FLXDEN[1] := power(10, 10);
    {   Potential evaporative flux density at .55 day}
    ERMAX := 0;
    {   Maximum possible evaporative flux density from soil surface}
    WFDD := 0;
    {   Crown potential (mm water)   }
    HROOT := G[1];
    cultno := 1;
    FGS := 0;
    {**********************************************************}
    {********************************************************************}
    {**********************************************************}
    {  For each chemical species:}
    {  Broadcast chemical application number}
    FERNO := 1;
    {   Cumulative Ca, CO3 and SO4 precipitated and dissolved}
    PPTS1 := 0;
    PPTS6 := 0;
    PPTS8 := 0;
    PPTS12 := 0;
    SOLS1 := 0;
    SOLS6 := 0;
    SOLS8 := 0;
    SOLS12 := 0;
    {   Cumulative CO2 exchange with atmosphere}
    CDCO2 := 0;
    {----------------------------}
    {  For each chemical species:}
    for M := 1 to mm do
      { bovenstaande regel was (PF2006): DO 402 M:=1,9. }
    begin
      SFER[m] := 0;
      wrunoff_load[m] := 0;
      crunoff_load[m] := 0;
      { bovenstaande 3 regels (PF2006) toegevoegd vanwege Undefined value.}
      {   Chemical concentrations at top and bottom nodes}
      SE[M, 1] := 0;
      SS[M, 1] := 0;
      SS[M, KK] := BBSE[m];
      SE[M, KK] := BBSE[m];
      if (irrflg[irrno] = 1) then
        RSE[m] := ARSE[M, IRRNO];
      if (irrflg[irrno] = 0) then
        ISE[m] := ARSE[M, IRRNO];
      if (irrflg[irrno] = 3) then
        ISE[m] := ARSE[M, IRRNO];
      {   Cumulative chemical spp added in water}
      CRSE[m] := 0;
      CISE[m] := 0;
      {   Cumulative chemical spp added by dry application}
      CFER[m] := 0;
      CLSE[m] := 0;
      {   Cumulative uptake by plants}
      CUSE[m] := 0;
      {pf2006: Cumulative mineralization from plant residues}
      CMINER[m] := 0;
      {   Total of each species initially in profile (mmol/m2)}
      APDSE[m] := 0;
    end;
    {----------------------------}
    {-------------------------        }
    for I := 2 to KK - 1 do
    begin
      {    Electrical conductivity}
      EC[i] := 0;
      {    Change in Gibbsite, gypsum, calcite, and atmospheric exchange of CO3 and HCO3}
      DelGibb[i] := 0;  {pf08}
      DELGYP[i] := 0;
      DECALCITE[i] := 0;
      DELCO2[i] := 0;
      {----------------------------}
      {    Sorbed anions set to 0}
      for M := 5 to 8 do
        XSE[m, i] := 0;
      {    PF2023: Sorbed FeII, FeIII, H4SiO4, DOC set to 0}
      for M := 13 to 16 do
        XSE[m, i] := 0;
      {----------------------------}
      {  Concs at start and end of time step set to initial values}
      for M := 1 to MM do
        SS[m, i] := SE[m, i];  {was: m:=1 TO 9 BUT NEW SPECIES ADDED}
      {----------------------------}
    end;

    { MAINC2 12 } {process driver | Repeat for every day in the year}
    {**************************************************************}
    {**************************************************************}
    {**************************************************************}
    {---------------DAILY CYCLE STARTS HERE------------------------     }
    {**************************************************************}

    untGUI.frmMAIN.ggProgressCurrentYear.Max := CUMT;
    {comment the previous line out if no graphical interface is required}
    for mDAY := DAY1 to CUMT do {do 1000 loop       }
    begin
      { MAINC2 13 } {process driver | Initialize counters and accumulators for this day}
      DAY := mDAY;
      {for debugging purposes}
      {if (Day>=210)and(day<=212) then trace:=1 else trace:=0; }
      {--------------------------------------------------------------     }
      if (iinitialize = 1) then
      begin
        {  INITIALIZE VARIABLES }

        rr1 := 0;
        rr1i := 0;
        rr2 := 0;
        rr3 := 0;
        rr4 := 0;
        rr5 := 0;
        rr6 := 0;
        rr7 := 0;
        rr8 := 0;
        rr9 := 0;
        rr10 := 0;
        rr11 := 0;
        rr12 := 0;
        rr13 := 0;
        RR23 := 0;
        RR24 := 0;
        {   Initial profile water content (mm)}
        AWATER := 0;
        POROSITY := 0;
        {   Cumulative rain and irrigation (mm)}
        CRAIN := 0;
        CIRRIG := 0;
        {   Cumulative potential transpiration (mm)}
        CTRANP := 0;
        WTRANP := 0;
        {   Cumulative actual transpiration mm}
        CTRANA := 0;
        WTRANA := 0;
        {   Cumulative potential evaporation (mm)}
        CEVAPP := 0;
        WEVAPP := 0;
        {   Cumulative actual evaporation (mm)}
        CEVAPA := 0;
        WEVAPA := 0;
        {   Cumulative drainage (mm)}
        CLW := 0;
        FWBTCI := 0;
        {   Cumulative excess water }
        EXCESS := 0;
        {  For each segment}
        for SEG := 1 to 4 do
        begin
          {   Cumulative water flux across bottom boundary (mm)}
          FW[SEG] := 0;
          CFW[SEG] := 0;
        end;
        FW0 := 0;
        CFW0 := 0;
        {  For each node}
        wfw[1] := 0;
        for I := 2 to KK - 1 do
        begin
          {   Water extraction by roots (m3/m3 during time step)}
          A[i] := 0;
          {   Water extraction by roots (mm during print interval)}
          WTRAN[i] := 0;
          {    (mm cumulative from start)}
          CTRAN[i] := 0;
          {   Cumulative water flux per print interval mm}
          WFW[i] := 0;
          {   Initial depth of water in profile.   }
          AWATER := AWATER + W[i] * DLZ;
          {   Overall porosity of profile (mm)}
          POROSITY := POROSITY + TS[i] * DLZ;
        end;

        {*********************************************************************}
        {*********************************************************************}
        {  For each chemical species:}
        for m := 1 to nchem1 do
        begin
          {   Total chemical initially present in profile}
          {   This line commented out by PF2006 because SFER uninitialized and }
          {   APDSE[m] set to zero anyway (label 402)}
          {   original line:  APDSE[m]:=SFER[m]   }
          {   Cumulative chemical spp added in water}
          CRSE[m] := 0;
          CISE[m] := 0;
          {   Cumulative chemical spp added by dry application}
          CFER[m] := 0;
          {      WFER[m]:=0;     }
          {   Cumulative uptake by plants}
          {      CPUSE[m]:=0;   }
          {   Drainage losses in time step}
          LSE[m] := 0;
          {   Cumulative drainage losses}
          CLSE[m] := 0;
          {   Cumulative volatilization losses}
          {     from soil surface}
          {      CVSE[m]:=0;    }
          {     from ponded water during irrigation }
          {      VSEIRR[m]:=0.}
          crunoff_load[m] := 0;
          { the line above has been un-commented due to Undef errors (PF2006)}
        end;
        {**********************************************************}
        {   For each chemical species in node I}
        for I := 1 to KK - 1 do
        begin
          for M := 1 to NCHEM do   {was: m:=1 TO 9 BUT NEW SPECIES ADDED}
          begin
            {c   Chemical losses (per time step)}
            {c  Initialize variables  }
            {      CUSE[m,i]:=0.   }
            {      WUSE[m,i]:=0.  }
            {      USE[m,i]:=0.}
            {C-------------------------        }
            {c  Total chemical spp initially in system (mg)  }
            if (i >= 2) then
              if (m = 15) or (m = 16) then  {PF2023: H4SiO4 and DOC have valence 0}
                APDSE[m] := VOLUME * (SE[m, i] * W[i]) + APDSE[m]
              else
                APDSE[m] := VOLUME * (SE[m, i] * W[i] + XSE[m, i] * RHO[i] / VAL[m]) + APDSE[m];
          end;
        end;
        sols1 := 0;
        sols6 := 0;
        sols8 := 0;
        sols12 := 0;
        ppts1 := 0;
        ppts6 := 0;
        ppts8 := 0;
        ppts12 := 0;
        cdco2 := 0;
      end; {if iinitialize=1}
      iinitialize := 0;
      {  Initialize time accumulators   }
      TIMDAY := 0;
      R20 := 0;
      TIME := DAY - 1;
      WEEK := round((DAY - 1) / 7) + 1;
      {------------------------------    }
      {------------------------------}
      {  At start of a new week record the current bottom boundary potential}
      {  and calculate the next week's increment (BBINC).}
      if (WEEK > WEEKO) and (BBC = 5) then
      begin
        FMAX := power(10, 10);
        WBBPOT := BBPOT;
        WEEKO := WEEK;
        BBINC := PDEPTH - WTDEP[WEEK] + DLZ / 2 - WBBPOT;
      end; {if}
      {  Screen print for start of day.}
      str(DAY, txt1);
      str(round(crain + cirrig), txt2);
      str(round(cevapa + ctrana), txt3);
      str(round(clw), txt4);
      if (abs(nsum - 999) >= 0.01) then
        wwriteln(' DAY' + txt1 + '  Infiltration:' + txt2 + ' mm   ET:' + txt3 +
          ' mm  Drainage:' + txt4 + ' mm');
      {----------------------------}

      { MAINC2 14 } {process driver | Conditional: if plants present then for crop development stage determine cropcover}
      {  If plants are present then set}
      {   KILPLT:=0 (plants absent) for periods before planting date and }
      {   after harvest.}
      KILPLT := KLPLT;
      if (DAY >= PDAY[1]) then
      begin
        if (KLPLT = 1) then
        begin
          if (DAY = PDAY[MIN(CROPNO + 1, NCROP)]) and (CROPNO < NCROP) then
            CROPNO := MIN(CROPNO + 1, NCROP);
          if (DAY > DAYHAR[CROPNO]) and (icut[CROPNO] = 2) then
          begin
            KILPLT := 0;
            CRPCOV[CROPNO] := 0;
            RDZ := 0;
          end;
        end;
      end
      else if (DAY < PDAY[1]) and (icut[1] = 2) then
        KILPLT := 0;
      {  Accumulate temperature period counter if necessary}
      if (DAY > ETIME[JK]) then
        JK := JK + 1;

      { MAINC2 15 } {process driver | Conditional: if plants have emerged then call GROWH to set plant props for current day}
      {**********************************************************************}
      {**********************************************************************}
      {  During the crop period call plant growth and automated irrigation}
      {  Calculate cumulative root and cover growth }
      if (KILPLT = 1) then
        GROWTH(KK, RDFDAY[MAX(1, CROPNO)], PD[MAX(1, CROPNO)],
          DAY, RDZ, DLZ, RDF, CRPCOV[MAX(1, CROPNO)], ESTART[MAX(1, CROPNO)],
          ESTOP[MAX(1, CROPNO)], dayhar[MAX(1, CROPNO)], AK1[MAX(1, CROPNO)],
          KILPLT, TRL, PDAY[MAX(1, CROPNO)], IGROWTH[MAX(1, CROPNO)],
          RDEP[MAX(1, CROPNO)]);
      {**********************************************************************}
      {  Automated irrigation (if set)}

      if (CH_IRR_FLAG = 1) then
        AUTOIRRG(NCHEM);
      {----------------------------}
      {  End of routines called during growth period}
      {**********************************************************************}
      {**********************************************************************}

      { MAINC2 16 } {process driver | call RothC_Daily (C-submodel on a daily basis)}
{pf2006: routines called daily for C-decomposition cycle, CO2-difusion.
 also: initialisation daily uptake of chemicals by transpiration}
      {call C-submodel on a daily basis}
      RothC_Daily(month); {month is needed because PlantLitter input is input by month}

      { MAINC2 17 } {process driver | call CO2_diffusion (redistribute produced and present CO2 over the profile by diffuson)}
    {each day: adjust pCO2-distribution over the soil profile
              using the gas diffusion equation on inserting
              * the daily production of CO2 by decomposition
              * the air-filled pore volume
              * the initial pCO2
              * the upper boundary condition pCO2-atmosphere=0.0003 Bar
              * the lower boundary condition pCO2=0.0003 Bar}
      CO2_diffusion;
      for i := 1 to kk do
        pCO2[i] := RothC.pCO2_r[i];

      { MAINC2 18 } {process driver | call WE_daily (weathering of soil minerals on a daily basis)}
      {call weathering routine}
      WE_daily;

      { MAINC2 19 } {process driver | conditional: if bedrock the call WE_daily_sapro (weathering of rock minerals on a daily basis)}
      if untGUI.rock.RockBottomLayers>0 then WE_daily_sapro;

      { MAINC2 20 } {process driver | Initialize daily uptake of chemicals via transpiration stream}
   {reinitialize daily uptake of chemicals via transpiration stream.
    the DUSE of the previous day has already been added to the litter pools
    and has been transferred partly to the CHEMCO2-pool in RothC_daily.
    This part in the CHEMCO2-pool is mineralised and thus available again.
    The Endorganic part is released via the sink term in routine SINKC over the profile layers,
    and the ectorganic part is added as fictitious fertilizer}
      for i := 2 to kk - 1 do
        for m := 1 to nchem do
        begin
          DUSE[m, i] := 0;
        end;
      {  End pf2006}
      {  For root zone}
      if (ifourth_layer = 1) then
        ND[4] := MIN(KK - 1, round(1 + RDZ / DLZ));
      {------------------------   }

      { MAINC2 21 } {process driver | call POTET (potential evapotranspiration for the day)}
      {  Potential evaporation and transpiration for day from pan data}
      {   If IFLOW := 3 this routine is not called and ET is 0.}
      DLYEP := 0;
      DLYTP := 0;
      if (IFLOW <> 3) then
      begin
        POTET(PAN[WEEK], DLYPAN, DLYEP, DLYTP, CRPCOV[MAX(1, CROPNO)],
          ERMAX, PANFAC[MAX(1, CROPNO)]);
      end;

      { MAINC2 22 } {process driver | Add chemical amendments to water phase, possibly partly incorporated in the soil}
      {*********************************************************************}
      {  Add chemical amendments to water phase (partitioned in CHEM)}
      if (DAY = DFER[FERNO]) then
      begin
        for M := 1 to NCHEM do   {was: m:=1 TO 8 BUT NEW SPECIES ADDED}
        begin
          if (INCOR[FERNO] > 0) then
          begin
            for I := 2 to (MIN(KK - 1, INCOR[FERNO] + 1)) do
            begin
              SE[m, i] := SE[m, i] + 1000 * FER[M, FERNO] / (VOLUME * W[2] * INCOR[FERNO]);
              SS[m, i] := SE[m, i];
            end;
          end; {if}
        end;
        FLAGC := ITERCH - 1;
      end; {if}
      {*********************************************************************}
   {pf09: the effect of cultivation on mixing of the plow layer
   is taken into account in the calling program}
      {if (DAY=Dcult[cultno])and(DataYearIn.RothC.vegetatie='agriculture') THEN cultc;}
      {*********************************************************************}

      { MAINC2 23 } {process driver | Conditional: if capacity flow then set rain, potential evapotranspiration for the day}
      {  Do the following for the capacity flow option:}
      {  Set rain or irrigation }
      if (IFLOW = 2) then
      begin
        rnoff := 0;
        if (TIME + 1 >= IRRBEG) then
        begin
          if (roslope > 0.01) then
          begin
            rots := (ts[2] + ts[3]) / 2;
            rowp := (wp[2] + wp[3]) / 2;
            rofc := (fc[2] + fc[3]) / 2;
            row := (w[2] + w[3]) / 2;
            s1 := 0;
            ros := 0;
            RUNOFF(rots, roWP, roFC, roW, ROSLOPE, ROCN2, IRRIG, Rnoff, ros, s1, tem[3]);
          end
          else
          begin
            rnoff := 0;
          end;
          HEAD := IRRIG - Rnoff;
          IHEAD := 1;
        end;
        {------------------------------}
        {  Potential evaporation and transpiration for day }

        B[1] := HC(1, H[1], RG[1], KS[1], AEV[1], BCAM[1], TS[1], L[1], RP[1], TM[1], hm[1], TEM[1]);
        for I := 2 to KK - 1 do
        begin
          B[I] := HC(i, H[I], RG[I], KS[I], AEV[I], BCAM[I], TS[I], L[I], RP[I], TM[i], hm[i], TEM[i]);
          B[I - 1] := (B[I] + B[I - 1]) / 2;
        end;
        {  The factor 0.5 is an arbitrary adjustment}
        WFDD := -0.5 * B[1] * (-300000 - H[2]) / DLZ;
        TTRANP := 0;
        TER := 0;
        {  For 5 0.1 day time steps, accumulate potential evaporation}
        {  and transpiration}
        for IT := 4 to 8 do
        begin
          TIMDAY := (IT) / 10;
          DELT := 0.1;
          TRANP := 0;
          IET := 1;
          if (IHEAD = 1) then
          begin
            IET := 0;
            if (TIME + TIMDAY - 0.1 >= IRRBEG + HEAD / IRATE) or (TIME + TIMDAY <= IRRBEG) then
              IET := 1;
          end;
          if (IET = 1) then
            ETRANS;
          TER := TER + ER * 0.1;
          TTRANP := TTRANP + TRANP;
        end;
        TRANP := TTRANP;
        ER := TER / 0.5;
      end; {IF}
      {*********************************************************************}
      {*********************************************************************}

      { MAINC2 24 } {process driver | Repeat for time intervals within day}
      {------------------START OF TIME INTERVALS WITHIN DAY-----------------   }
      repeat {until iend_of_day}
        {130 CONTINUE     }
        {********************************************************************}
        {  ---------------------SUBROUTINES----------------------------     }

        {  Calculate the length of the new time increment}

        {   toegevoegd PF2006}
        TIMDAYOLD := TIMDAY;

        { MAINC2 25 } {process driver | calc TSTEP (length of new time increment)}
        if (IFLOW <> 2) then
          TSTEP;
        {---------------}
        if (IFLOW = 2) then
        begin
          TIMDAY := 0;
          DELT := 1.0;
          R20 := 1;
          TIMDAY := TIMDAY + DELT;
        end;
        {----------------------------}
        {  Summary file time increment}
        RSUME := RSUME + DELT;
        {----------------------------}

        { MAINC2 26 } {process driver | adjust bottom boundary if water table adjustment (not in SoilGen)}
        {  Adjust bottom boundary if water table adjustment is required.}
        if (BBC = 5) then
        begin
          {  FRACw is the fraction of the week elapsed}
          FRACw := (-7 * (WEEK - 1) + DAY + TIMDAY - 1) / 7;
          {  Set bottom boundary potential (if option 5).}
          BBPOT := WBBPOT + BBINC * FRACw;
          H[KK] := BBPOT;
          {  Change potential of all saturated nodes }
          {   by the change in the lower boundary.}
          IFLG := 0;
          I := KK;
          while (IFLG = 0) and (I > 2) do
          begin
            {845}
            I := I - 1;
            if (G[i] >= 0) then
              G[i] := MAX(0, g[i] - (g[kk] - BBPOT));
            if (G[i] < 0) then
              IFLG := 1;
            {if (IFLG=0) and (I>2)GO TO 845}
          end;
        end; {if}
        {----------------------------}

        { MAINC2 27 } {process driver | calc ETRANS (Evapotranspiration during the length of the time increment)}
        {  Potential evaporation and transpiration for time period }
        {  (calculated daily above for capacity model)}
        if (IFLOW = 1) then
        begin
          TRANP := 0;
          ETRANS;
        end; {if}
        {----------------------------}

        { MAINC2 28 } {process driver | calc WUPTAK (Water sink term for WATFLO ie. root uptake)}
        {  Water sink term for WATFLO ie. root uptake }
        if (IHEAD = 0) and (TRANP > 0) and (IFLOW = 1) then
        begin
          WUPTAK;
        end;
        if (IFLOW = 2) then
        begin
          DELT := 0.5;
          WUPTAK;
          DELT := 1.0;
        end;
        {----------------------------}

        { MAINC2 29 } {process driver | Conditional: if Richards' equation then call WATFLO (water fluxes and potential updates) for current time interval within the day}
        {  Water flux between nodes and new potentials       }
        if (IFLOW = 1) then
          WATFLO(ros, rots, rowp, rofc, row, s1, DIFF);
        {   For steady-state, water fluxes are set here}
        if (IFLOW = 3) then
        begin
          if (HEAD <= 0) and (TIME >= IRRBEG) then
          begin
            HEAD := IRRIG;
            IHEAD := 1;
            if (IRATE > 0) then
              IRREND := IRRIG / IRATE;
            if (IRATE = 0) then
              IRREND := 99999;
          end;
          for I := 1 to kk do
          begin
            if (IHEAD = 1) then
            begin
              FLXDEN[i] := IRATE;
              FMAX := MAX(ABS(FLXDEN[i]), ABS(CONQ * DLZ / DELM));
            end
            else
            if (IHEAD = 0) then
            begin
              FLXDEN[i] := 0;
              FMAX := ABS(CONQ * DLZ / DELM);
            end;
            FLUX[i] := FLXDEN[i] * DELT;
          end;
        end; {if}
        {------------------------------}

        { MAINC2 30 } {process driver | Conditional: if time=tenth of day then call HEAT + call Physical_Weathering}
        {next line modified by pf2006; from: if (IFGTEM=1) and (R20>=0.099999) THEN}
        if (IFGTEM = 1) and (trunc(TIMDAYOLD * 10) <> trunc(TIMDAY * 10)) then
          {do once for every new 10th of the day}
        begin
          {  Temperature prediction and heat flow}
          if (IFLOW <> 2) then
            DELTHEAT := 0.1;
          if (IFLOW = 2) then
            DELTHEAT := 1;
          HEAT(DELTHEAT);
          if untGUI.PhysWeathAllowed then
            Physical_Weathering(deltheat);
      {next lines added by pf2006 to allow calculation
       of yearly&monthly average temperatures}
          for i := 1 to kk do
          begin
            TEMPGEM[i] := TEMPGEM[i] + TEM[i];
            {YYY1}
            RothC.Temperatures[oldmonth][i] := RothC.Temperatures[oldmonth][i] + TEM[i];
          end;
          TEMPCOUNT := TEMPCOUNT + 1;
          tempcountmonth := tempcountmonth + 1;
          {end added lines pf2006}
        end; {if}

    { MAINC2 31 } {process driver | call SINKC (uptake of cations Ca, Mg, Na, K and Al by plants)}
    {next lines added by pf2006 to allow for the uptake of cations Ca, Mg, Na, K and Al
     by plants}
        SinkC;

    { MAINC2 32 } {process driver | call Clay_Migration1_PreCDE (calculate amount of clay in dispersion)}
        {next line added by pf08 to allow for dispersible clay transport}
        Clay_Migration1_PreCDE;
        {end added lines pf200*}

    { MAINC2 33 } {process driver | prep + call SOLC ( transport of solute (incl. dispersed) species)}
        {*********************************************************************}
        {  Transport of solute species                }

        {   Surface conc. set to that in irrigation water plus solute }
        {    on the soil surface (SFERn mg/m2) that dissolves.}
        if (IHEAD = 1) then
        begin
          if (IFLOW <> 2) then
            WINFIL := FLUX[1];
          if (IFLOW = 2) then
            WINFIL := HEAD;
          for M := 1 to NCHEM1 do     {was: NCHEM but this is wrong for splash clay}
          begin
            SS[M, 1] := 0;
            if (WINFIL > 0.000001) then
            begin
              if (irrflg[irrno] = 1) then
                SS[M, 1] := RSE[m];
              if (irrflg[irrno] = 0) or (irrflg[irrno] = 3) then
                SS[M, 1] := ISE[m] + SFER[m] / WINFIL;
            end;
          end;
        end
        {  For no water flow or prior to the first irrigation specification}
        else
          for M := 1 to NCHEM1 do
            SS[M, 1] := 0;
        {---------------}
        if (IFLOW <> 2) then
          SOLC;

     { MAINC2 34 } {process driver | call Clay_Migration2_PostCDE (calculate consequences of dispersible clay transport)}
        {calculate consequences of dispersible clay transport}
        Clay_Migration2_PostCDE;

     { MAINC2 35 } {process driver | Conditional: if fixed or fluctuating water table then call CELLC (set new lower boundary concentrations)}
        {--------------------------------------------------------------------}
        {  New lower boundary concentration is that in a mixing cell below the}
        {    simulated profile which will only influence the value of SE(m,kk)}
        {    when water moves upwards into the profile.  The initial conc-}
        {    entration in the cell is the initial value for the bottom segment}
        {    concentration.}
        if (CELLW > 0.0001) and ((BBC = 1) or (BBC = 5)) then
          CELLC;

    { MAINC2 36 } {process driver | Conditional: every x timesteps: call CHEM2 (chemical equilibrium) + call calcphlab (1:5 dilution pH as from lab analysis)}
        {********************************************************************}
        {   CHEMICAL EQUILIBRIUM:  this subroutine is called after the initial}
        {    time step and thereafter after every ITERCH time steps (counter}
        {    is FLAGC)}
        FLAGC := FLAGC + 1;
        {      counttim:=counttim+DELT}
        { above line added by PF2006 to estimate average time between 2 equilibria resets}
        if (FLAGC = ITERCH) or (TIME = 0) then
        begin
          {      icounttim:=icounttim+1}
          { above line added by PF2006 to estimate average time between 2 equilibria resets      }
          for I := 2 to KK - 1 do
          begin
            SEFeII:=SE[14,i]; SEFeIII:=SE[13,i]; {FeII and FeIII conc for call to Chem2}
            case typechem of
              1:begin end;
              2:
                CHEM2(CALCITE[i], GYP[i], Gibbsitepool[i], PH[i], RHO[i], SE[12, i],
                  SE[1, I], SE[2, I], SE[3, I], SE[4, I], SE[5, I], SE[6, I], SE[7, I], SE[8, I],
                  SE[9, I], SE[10, I], SE[11, I], W[i], EC[i], SAR[i],
                  XSE[1, I], XSE[2, I], XSE[3, I], XSE[4, I], XSE[10, i], XSE[12, i], CEC[i],
                  ESP[i], SC1[i], SC2[i], SC3[i], SC4[i], SC5[i], SC6[i],
                  SC7[i], SC8[i], SC9[i], SC10[i], SC11[i], SC12[i], SC13[i], SC14[i], SC15[i],
                  DELGIBB[i], DELGYP[i], DECALCITE[i], PCO2[i], NN1,
                  SP1, SP2, SALT[i], trace, DELCO2[i], n1prt, tem[i]);
              3: begin end;
            end;
          end;
          calcphlab(RHO, PH, AW, PHLAB, DLZ);
          {calculate pH-H2O in 1:5 dilution as in lab protocol}
          FLAGC := 0;
          {  NN1 was set to 1 in the input if chemical equilibrium was assumed.}
          {   It is now set to 0 for all subsequent time steps.}
          NN1 := 0;
        end; {if}

        { MAINC2 37 } {process driver | Conditional: if tipping bucket/Addiscott model then call CALFC}
        if (IFLOW = 2) then
        begin
          CALFC(EFLUX);
          FLAGC := ITERCH - 1;
        end;

        { MAINC2 38 } {process driver | Cumulative totals of chemicals and mass balancing, profile totals}
{********************************************************************
{----------- CUMULATIVE TOTALS AND MASS BALANCING--------------------           }
        {--------------------------------------------------------------------}
        {  Initialise profile totals:}
        {  Chemical }
        for M := 1 to NCHEM do
          PDSE[m] := 0;
        {------------------------------               }
        {--------------------------------------------------------------------}
        for M := 1 to NCHEM do   {was: m:=1 TO 8 BUT NEW SPECIES ADDED}
        begin
          {  Added as fertilizer mmol/sq.m }
          if (DFER[FERNO] = DAY) then
            CFER[m] := CFER[m] + FER[M, FERNO] * 1000;
          {  Added in water (increment if there was a downward flux of water }
          {   the surface in this time period)   }
          if (IHEAD = 1) then
          begin
            if (irrflg[irrno] = 1) then
              CRSE[m] := CRSE[m] + RSE[m] * winfil;
            if (irrflg[irrno] = 0) or (irrflg[irrno] = 3) then
              CISE[m] := CISE[m] + ISE[m] * winfil;
          end;
        end;
        {  Increment FERNO to next day of fertilizer application                 }
        if (DFER[FERNO] = DAY) then
          FERNO := FERNO + 1;
        {********************************************************************}
        { CHANGE TO DESIRED UNITS AND/OR CALC TOTAL TRANSPIRATION WATER, ANIONS }
        {  CATIONS (MEQ/L) IN EACH DEPTH SEGMENT}
        {  DEPTH INCREMENT  }
        for I := 2 to KK - 1 do
        begin
          TOTANS[i] := SE[5, I] + 2 * SE[6, I] + SE[7, I] + 2 * SE[8, I] + MAX(SE[9, I], 0);
          TOTCAT[i] := 2 * SE[1, I] + 2 * SE[2, I] + SE[3, I] + SE[4, I] - MIN(0, SE[9, I]);
          {-------------------------  }
          { SUMMING OF DEPTH INCREMENT VALUES TO PRODUCE PROFILE TOTALS   }

          for M := 1 to NCHEM do  {was: m:=1 TO 9 BUT NEW SPECIES ADDED}
          begin
            if (m = 15) or (m = 16) then {H4SiO4 and DOC have no charge}
              PDSE[m] := VOLUME * (SE[m, i] * W[i]) + PDSE[m]
            else
              PDSE[m] := VOLUME * (SE[m, i] * W[i] + XSE[m, i] * RHO[i] / VAL[m]) + PDSE[m];
          end;
          {-------------------------  }
          { Cumulative from start (never reinitialized)   }
          {   i)Precipitation and dissolution of CALCITE and gypsum  }
          if (DELGYP[i] > 0) then
            PPTS6 := PPTS6 + DELGYP[i] * VOLUME
          else
            SOLS6 := SOLS6 - DELGYP[i] * VOLUME;
          if (DECALCITE[i] > 0) then
            PPTS8 := PPTS8 + DECALCITE[i] * VOLUME
          else
            SOLS8 := SOLS8 - DECALCITE[i] * VOLUME;
          if (DELGYP[i] + DECALCITE[i] > 0) then
            PPTS1 := PPTS1 + (DELGYP[i] + DECALCITE[i]) * VOLUME
          else
            SOLS1 := SOLS1 - (DELGYP[i] + DECALCITE[i]) * VOLUME;
          { ii) Precipitation and dissolution fromGibbsite pf08}
          if (DELGIBB[i] > 0) then
            PPTS12 := PPTS12 + DELGIBB[i] * VOLUME
          else
            SOLS12 := SOLS12 - DELGIBB[i] * VOLUME;
          {  Gains from atm +}
          CDCO2 := CDCO2 + DELCO2[i] * VOLUME;
          DELGYP[i] := 0;
          DECALCITE[i] := 0;
          DELGIBB[i] := 0;
          DELCO2[i] := 0;
        end;
        {-------------------------  }

        { MAINC2 39 } {process driver | Set solute concentrations for next time period}
        {  SET SOLUTE CONCENTRATIONS FOR NEXT TIME PERIOD   }
        for I := 1 to KK - 1 do
        begin
          for M := 1 to NCHEM do
            SS[m, i] := SE[m, i];        {was: m:=1 TO 9 BUT NEW SPECIES ADDED}
          SALT[i] := 2 * (se[1, i] + se[2, i]) + se[3, i] + se[4, i] + se[5, i] + 2 * se[6, i] +
            se[7, i] + 2 * se[8, i] + se[9, i] + se[10, i] + se[11, i] + 3 * se[12, i];
        end;
        {-------------------------  }

        { MAINC2 40 } {process driver | calculate leachate losses}
        {  Total chemical lost in leachate. (Individual species losses}
        {  are calculated in SOLC as LSE[m]) (mg/m2). }
        for m := 1 to nchem do
          CLSE[m] := CLSE[m] + LSE[m];
        {   CHANGE FROM START: }
        {  Chemical species}
        for M := 1 to NCHEM do
          DELSE[m] := PDSE[m] - APDSE[m];  {was: m:=1 TO 9 BUT NEW SPECIES ADDED}
        {-------------------------  }
        {  MASS BALANCE OF CATIONS AND ANIONS}
   {originally:
   for M:=2 to 5 do BALSE[m]:=CFER[m]+CRSE[m]-CLSE[m]-CUSE[m]-DELSE[m]+CISE[m];
   BALSE[9]:=CRSE[9]-CLSE[9]-CUSE[9]-DELSE[9]+CISE[9];
   BALSE[1]:=CFER[1]+CRSE[1]-CLSE[1]-CUSE[1]-DELSE[1]-PPTS1+SOLS1+CISE[1];
   BALSE[6]:=CFER[6]+CRSE[6]-CLSE[6]-CUSE[6]-DELSE[6]-PPTS6+SOLS6+CISE[6];
   BALSE[7]:=CFER[7]+CRSE[7]-CLSE[7]-CUSE[7]-DELSE[7]+CISE[7]+CFER[8]+
             CRSE[8]-CLSE[8]-CUSE[8]-DELSE[8]-PPTS8+SOLS8+cdco2;  }
        {checkthis}
        for M := 2 to 5 do
          BALSE[m] := CFER[m] + CMINER[m] + CRSE[m] - CLSE[m] - CUSE[m] - DELSE[m] + CISE[m];
        BALSE[9] := CRSE[9] - CLSE[9] - CUSE[9] - DELSE[9] + CISE[9] + CMINER[9];
        BALSE[1] := CFER[1] + CMINER[1] + CRSE[1] - CLSE[1] - CUSE[1] - DELSE[1] - PPTS1 + SOLS1 + CISE[1];
        BALSE[6] := CFER[6] + CMINER[6] + CRSE[6] - CLSE[6] - CUSE[6] - DELSE[6] - PPTS6 + SOLS6 + CISE[6];
        BALSE[7] := CFER[7] + CMINER[7] + CRSE[7] - CLSE[7] - CUSE[7] - DELSE[7] +
          CISE[7] + CFER[8] + CMINER[8] + CRSE[8] - CLSE[8] - CUSE[8] - DELSE[8] -
          PPTS8 + SOLS8 + cdco2;
        {pf08: Gibbsite and Al}
        BALSE[12] := CFER[12] + CMINER[12] + CRSE[12] - CLSE[12] - CUSE[12] - DELSE[12] -
          PPTS12 + SOLS12 + CISE[12];
        {pf2023: FeII and FeIII. still check needed if ll these are needed (or 0)}
        {********************************************************************}
        {*********************************************************************}
        {  If osmotic potential is required as component of total soil water}
        {  potential, reflect total salt conc in me/l as SALT[m,i].}
        ospot := 0;
        {*********************************************************************}
        {********************************************************************}

        { MAINC2 41 } {process driver | Cumulate potential and actal transpiration, evaporation, drainage}
        {  Initialise profile water content}
        PWATER := 0;
        {------------------------------               }
        { Cumulative potential transpiration     }
        if (IHEAD = 0) then
          CTRANP := CTRANP + TRANP;
        cevapp := cevapp + er * delt;
        {-------------------------                    }
        { Cumulative actual transpiration   }
        CTRANA := CTRANA + TRANA;
        {  Cumulative evaporation (mm)     }
        if (IFLOW <> 2) and (FLXDEN[1] < 0) then
          CEVAPA := CEVAPA - FLUX[1];
        if (IFLOW = 2) then
          CEVAPA := CEVAPA + EFLUX;
        {-------------------------                    }
        { Cumulative drainage (mm)                    }
        CLW := CLW + FLUX[KK - 1];
        {  for breakthrough curve}
        FWBTCI := FWBTCI + FLUX[KK - 1];
        {-------------------------}
        { Leaching flux and cum. flux at selected depths (for}
        {  summary file)}
        FW0 := FW0 + FLUX[1];
        CFW0 := CFW0 + FLUX[1];
        for SEG := 1 to 4 do
        begin
          FW[SEG] := FW[SEG] + FLUX[ND[SEG]];
          CFW[SEG] := CFW[SEG] + FLUX[ND[SEG]];
        end;
        {-------------------------                    }
        { Relative yield based on transpiration }
        if (CTRANP > 0.001) then
          YIELDW := 100 * CTRANA / CTRANP;
        {----------------------------                    }

        { MAINC2 42 } {process driver | Update air-filled porosity}
        for I := 2 to KK - 1 do
        begin
          {  Air-filled porosity, minimized to 0.01       }
          TG[i] := max(0.01,TS[i] - W[i]);
          {  Cumulative flux of water since previous print (mm).}
          if (I > 1) then
            WFW[I - 1] := WFW[I - 1] + FLUX[I - 1];
          {  If osmotic potential is required as component of total soil water}
          {  potential:}
          {     Osmotic potential (passed through to WUPTAK as SALT (in unots of}
          {     hydraulic head equivalent (mm)). As an approximation we use}
          {        OP (kPa) := 0.4 x EC (mS/m) (from Agric. Salinity Assessment and Management}
          {       p 48).  }
          {      SALT[i]:=-102.2*EC[i]*0.4}
          {----------------------------}
          {  Depth of water (mm) from theta            }
          WATER[i] := W[i] * DLZ;
          {  Total water in profile (mm)}
          PWATER := PWATER + WATER[i];
          {  Transpiration : per segment during time step (mm)        }
          TRAN[i] := A[i] * DLZ * DELT;
          {  Transpiration:per segment per print period (mm)        }
          WTRAN[i] := WTRAN[i] + TRAN[i];
          {  Transpiration: cumulative total per print period (mm)      }
          WTRANA := WTRANA + TRAN[i];
          {  Transpiration : cumulative per segment from start (mm)       }
          CTRAN[i] := CTRAN[i] + TRAN[i];
          {  Reinitialize transpiration sink term}
          A[i] := 0;
        end;
        {  Drainage flux across lowest two nodes}
        WFW[KK - 1] := WFW[KK - 1] + FLUX[KK - 1];
        {  Change from start:   }
        {     Water             }
        DELWAT := PWATER - AWATER;
        {---------------------------------------------------------------------}

        { MAINC2 43 } {process driver | Conditional: if infiltration in progress: reduce amount that still needs to infiltrate}
        {  If infiltration is in progress: }
        if (IHEAD = 1) then
        begin
          {  Reduce water remaining to infiltrate and accumulate rain/irrig }
          if (IFLOW <> 2) then
          begin
            if (irrflg[irrno] = 1) then
              CRAIN := CRAIN + FLUX[1];
            if (irrflg[irrno] = 0) or (irrflg[irrno] = 3) then
              CIRRIG := CIRRIG + FLUX[1];
            HEAD := MAX(0, HEAD - FLUX[1]);
          end;
          if (IFLOW = 2) then
          begin
            if (irrflg[irrno] = 1) then
              CRAIN := CRAIN + HEAD;
            if (irrflg[irrno] = 0) or (irrflg[irrno] = 3) then
            begin
              CIRRIG := CIRRIG + HEAD;
              HEAD := 0;
            end;
          end;

          { MAINC2 44 } {process driver | Conditional: if end of infiltation reached then make evaluation of runoff risk and act accordingly}
          {  If the end of the infiltration is reached,}
          if (TIME + DELT - AIRRBG[IRRNO + 1] > -0.00001) or
            (TIME + DELT - AIRRBG[irrno] >= IRREND) or (HEAD <= 0) then
          begin
            {*********************************************************************}
            {  At this point an evaluation of surface runoff risk is made.}
            {  We generate a potential runoff hazard.}
            {  Change this function as necessary.  Currently use an index based}
            {  on runoff fraction, amount, rain intensity, slope}
            if (irrig > 0) then
              rindex := ((head + rnoff) / irrig) * (min(5, head + rnoff) / 5) * sqrt(
                (min(roslope, 20) / 20)) * (min(100, irate) / 100) * 0.1
            else
              rindex := 0.25;
            { conditional calculation of rindex to prevent division by zero in case of 0 irrigation}
            {      write(*,*)' Runoff index: ',rindex}
            for m := 1 to nchem do
            begin
              runoff_load[m] := sfer[m] * min(0.25, rindex);
              wrunoff_load[m] := wrunoff_load[m] + runoff_load[m];
              crunoff_load[m] := crunoff_load[m] + runoff_load[m];
              sfer[m] := max(0, sfer[m] - runoff_load[m]);
            end;
            if (rnoff + head > 0) then
              iflgrunoff := 1;
            runoff_mm := rnoff + head;
            {  add water that has not infiltrated to 'excess',}
            EXCESS := EXCESS + HEAD + rnoff;
            {  Set the maximum flux density high to ensure a short time step at }
            {  the start of redistribution}
            FMAX := 1 * power(10, 10);
            {  set head to 0,}
            HEAD := 0;
            {  Make sure there are no positive potentials above the water table}
            if (bbc = 1) then
            begin
              for i := 1 to nodewt do
              begin
                h[i] := min(0, h[i]);
                g[i] := h[i];
              end;
            end; {if}
            rnoff := 0;
            IHEAD := 0;
            {  next irrigation/rainfall number and set time, amount and rate.}
            IRRNO := IRRNO + 1;
            IRRIG := IRRIGA[irrno];
            IRATE := IRATEA[irrno];
            IRRBEG := AIRRBG[irrno];
            { Concentrations in next infiltration}
            for M := 1 to NCHEM do
            begin
              if (irrflg[irrno] = 1) then
                RSE[m] := ARSE[M, IRRNO];
              if (irrflg[irrno] = 0) then
                ISE[m] := ARSE[M, IRRNO];
              if (irrflg[irrno] = 3) then
                ISE[m] := ARSE[M, IRRNO];
            end;
          end;
        end;
        {------------------------------}

        { MAINC2 45 } {process driver | calculate water balance}
        {  Mass balance of water    }
        WATBAL := CRAIN + CIRRIG - CEVAPA - CTRANA - CLW - DELWAT;
{******************************************************************** }

{ MAINC2 46 } {process driver | Increase time counter to end of current time step}
{  Increment TIME to end of current time step}
        TIME := (DAY - 1) + TIMDAY;
        {****************************************************************}
        {----------------------------------------------------------------               }
        {------PRINT ACCORDING TO PRINT OPTIONS--------------------------               }
        {----------------------------------------------------------------               }
        {  Set ITIME_TO_PRINT to 1 if time to print                 }
        if (PRNTOP = 1) then
          ITIME_TO_PRINT := 1;
        if (PRNTOP = 2) then
        begin
          if (ABS(PRINT - 999) > 0.01) and (TIME + 0.0001 - R24 >= PRINT) then
          begin
            ITIME_TO_PRINT := 1;
            R24 := round(TIME * 10 / 10);
          end;
        end;

        if (TIME + 0.0001 >= PRTIME[NPRIN]) and (PRNTOP = 3) then
        begin
          ITIME_TO_PRINT := 1;
          NPRIN := NPRIN + 1;
          if (NPRIN > NPRINT) then
            PRTIME[NPRIN] := CUMT + 10;
        end;
        {----------------------------}
        {  Print after 1st time step (N1PRT initialized to 0, set to 1 here)}
        if (N1PRT = 0) then
        begin
          ITIME_TO_PRINT := 1;
          system.writeln(f11);
          if (iflow = 1) then
            system.writeln(f11, CYCNO, ' LEACHM used the Richards ' +
              'equation and CDE option.');
          if (iflow = 2) then
            system.writeln(f11, CYCNO, ' LEACHM used the capacity model.');
          str(CYCNO, txt1);
          if (iflow = 1) then
            wwriteln(txt1 + ' LEACHM using the Richards equation and CDE option.');
          if (iflow = 2) then
            wwriteln(txt1 + ' LEACHM using the capacity option.');
          if (iflow = 3) then
            wwriteln(txt1 + ' LEACHM using steady-state or interrupted steady-state water flow.');
          if (iflow = 3) then
            system.writeln(f11, CYCNO,
              ' LEACHM using steady-state or interrupted steady-state water flow.');
        end;
        system.Flush(f11);
        {----------------------  }

        { MAINC2 47 } {process driver | Conditional: if time to print to file: call CHEM2, call calcphlab, call OUTC}
        if (ITIME_TO_PRINT = 1) then
        begin
          for I := 2 to KK - 1 do
          begin
            SEFeII:=SE[14,i]; SEFeIII:=SE[13,i]; {FeII and FeIII conc for call to Chem2}
            case typechem of
              1:begin end;
              2:
                CHEM2(CALCITE[i], GYP[i], Gibbsitepool[i], PH[i], RHO[i], SE[12, i],
                  SE[1, I], SE[2, I], SE[3, I], SE[4, I], SE[5, I], SE[6, I], SE[7, I], SE[8, I],
                  SE[9, I], SE[10, I], SE[11, I], W[i], EC[i], SAR[i],
                  XSE[1, I], XSE[2, I], XSE[3, I], XSE[4, I], XSE[10, i], XSE[12, i], CEC[i],
                  ESP[i], SC1[i], SC2[i], SC3[i], SC4[i], SC5[i], SC6[i],
                  SC7[i], SC8[i], SC9[i], SC10[i], SC11[i], SC12[i], SC13[i], SC14[i], SC15[i],
                  DELGIBB[i], DELGYP[i], DECALCITE[i], PCO2[i], NN1,
                  SP1, SP2, SALT[i], trace, DELCO2[i], n1prt, tem[i]);
              3: begin end;
            end;
          end;
          calcphlab(RHO, PH, AW, PHLAB, DLZ);
          {calculate pH-H2O in 1:5 dilution as in lab protocol}
          FLAGC := 0;
          OUTC;
          IINITIALIZE := INITFLG;
          {  Reinitialize print interval totals         }
          for I := 1 to kk do
          begin
            WTRAN[i] := 0;
            WFW[i] := 0;
          end;
          WTRANA := 0;
          itime_to_print := 0;
        end; {if}
        {----------------------------}
        {  If no print the program continues here}
        IFLAG := 0;
        for I := 1 to kk do
        begin
          Y[i] := W[i];
          G[i] := H[i];
        end;
        {*********************************************************************}

        { MAINC2 48 } {process driver | Write to various output files (not used by SoilGen): breakthrough file, segment totals, etc.}
        {  Write to breakthrough data file}
        if (IFGBTC = 1) and (IFILE >= 3) then
        begin
          if (ABS(NSUM - 999) < 0.01) then
          begin
            system.Write(f13, '      Years         mm        PV         Ca         cCa' +
              '          Mg         cMg          Na         cNa ' +
              '           K          cK          Cl         cCl ' +
              '          SO4        cSO4        HCO3       cHCO3' +
              '          CO3        cCO3       Balion     cBalion        EC');
            IFGBTC := 0;
          end
          else
          if (ABS(NSUM - 999) >= 0.01) then
          begin
            system.Write(f13, '       Days         mm        PV         Ca         cCa' +
              '          Mg         cMg          Na         cNa ' +
              '           K          cK          Cl         cCl ' +
              '          SO4        cSO4        HCO3       cHCO3' +
              '          CO3        cCO3       Balion     cBalion        EC');
            IFGBTC := 0;
          end;
        end; {if}

        if (FWBTCI >= FWBTC) and (IFILE >= 3) then
        begin
          PVOL := 0;
          for I := 2 to KK - 1 do
            PVOL := PVOL + W[i] * DLZ;
          PV := CLW / PVOL;
          {  Flux averaged concentration}
          if (ABS(NSUM - 999) < 0.01) then
          begin
            str((((cycno - 1) * cumt) + time) / 365.25: 11: 5, txt1);
            str(CLW: 12: 2, txt2);
            str(PV: 12: 4, txt3);
            txt1 := txt1 + ' ' + txt2 + ' ' + txt3;
            for I := 1 to 9 do
            begin
              str(LSE[I] / FLUX[KK - 1]: 12: 3, txt2);
              str(clse[I]: 12: 3, txt3);
              txt1 := txt1 + ' ' + txt2 + ' ' + txt3;
            end;
            str(ec[kk - 1]: 12: 1, txt2);
            txt1 := txt1 + txt2;
            system.writeln(f13, txt1);
            FWBTCI := 0;
          end
          else
          begin
            if (ABS(NSUM - 999) >= 0.01) then
            begin
              str((((cycno - 1) * cumt) + time): 11: 3, txt1);
              str(CLW: 12: 2, txt2);
              str(PV: 12: 4, txt3);
              txt1 := txt1 + ' ' + txt2 + ' ' + txt3;
              for I := 1 to 9 do
              begin
                str(LSE[I] / FLUX[KK - 1]: 12: 3, txt2);
                str(clse[I]: 12: 3, txt3);
                txt1 := txt1 + ' ' + txt2 + ' ' + txt3;
              end;
              str(ec[kk - 1]: 12: 1, txt2);
              txt1 := txt1 + txt2;
              system.writeln(f13, txt1);
              FWBTCI := 0;
            end;
          end {if};
        end; {if}
        {********************************************************************}
        {  For water table or lower boundary (option 4 for 4th macro segment}
        if (ifourth_layer = 4) then
          ND[4] := MIN(KK - 1, round(1 + wetdep / DLZ));
        {********************************************************************}
        {  Accumulate segment totals for SUM file.}
        {  Use time-weighted monthly totals if monthly prints selected}
        if (ABS(NSUM - 999) >= 0.01) then
        begin
          DELTSUM := 1;
          for SEG := 1 to 4 do
          begin
            TH[SEG] := 0;
            TCALCITE[SEG] := 0;
            TGYPSUM[SEG] := 0;
            for M := 1 to NCHEM do
              TSE[M, SEG] := 0;
          end;
        end
        else
          DELTSUM := DELT;
        for SEG := 1 to 3 do
        begin
          if (SEG > 1) then
            IFIRST := ND[SEG - 1] + 1;
          if (SEG = 1) then
            IFIRST := 2;
          for J := IFIRST to ND[SEG] do
          begin
            TH[SEG] := TH[SEG] + WATER[j] * DELTSUM;
            {  Calcite and gypsum in macrosegments as g/m2}
            TCALCITE[SEG] := TCALCITE[SEG] + deltsum * DLZ * CALCITE[j] * 100 / 1000;
            TGYPSUM[SEG] := TGYPSUM[SEG] + deltsum * DLZ * GYP[j] * 172 / 1000;
            for M := 1 to NCHEM do  {was: m:=1 TO 9 BUT NEW SPECIES ADDED.
                              Took care of case that DOC (species 16) with valency 0
                              and no adsorption occurs}
              if VAL[M] <> 0 then
                TSE[M, SEG] := TSE[M, SEG] + (XSE[m, j] * RHO[j] /
                  val[m] + SE[m, j] * W[j]) * DELTSUM * DLZ
              else
                TSE[M, SEG] := TSE[M, SEG] + (SE[m, j] * W[j]) * DELTSUM * DLZ;
          end;
        end;
        {------------------}
        {  When ND[4] is whole profile or root zone}
        if (IFOURTH_LAYER <> 3) then
          IFOURTH := 2;
        if (IFOURTH_LAYER = 3) then
          IFOURTH := ND[3] + 1;
        for J := IFOURTH to ND[4] do
        begin
          TH[4] := TH[4] + WATER[j] * DELTSUM;
          {  Calcite and gypsum in macrosegments as g/m2}
          TCALCITE[4] := TCALCITE[4] + deltsum * DLZ * CALCITE[j] * 100 / 1000;
          TGYPSUM[4] := TGYPSUM[4] + deltsum * DLZ * GYP[j] * 172 / 1000;
          for M := 1 to NCHEM do   {was: m:=1 TO 9 BUT NEW SPECIES ADDED.
                              Took care of case that DOC (species 16) with valency 0
                              and no adsorption occurs}
            if VAL[M] <> 0 then
              TSE[M, 4] := TSE[M, 4] + (XSE[m, j] * RHO[j] / val[m] +
                SE[m, j] * W[j]) * DELTSUM * DLZ
            else
              TSE[M, 4] := TSE[M, 4] + (SE[m, j] * W[j]) * DELTSUM * DLZ;
        end;
        {------------------}
        { Surface chemical}
        {      do m:=1,nchem}
        {      wsfer[m]:=wsfer[m]+sfer[m]*delt}
        {      end do}

        {********************************************************************}

        { MAINC2 49 } {process driver | Conditional: if end of day then call CADATE (calender date of next day)}
        {  Is this the last time step on this day?}
        {   Default: no}
        IEND_OF_DAY := 0;
        {  If yes:}
        if (ABS(TIMDAY - 1) < 0.0001) then
        begin
          IEND_OF_DAY := 1;

          {  Calendar date of next day .}
          CADATE;
          {look for the depth where in the middle of the summer the soil is still frozen}
          DataYearOut.unfrozen := DataYearIn.PDEPTH;
          if (daym = 15) and (month = 7) then
          begin
            i := kk - 1;
            repeat
              if tem[i] < 0 then
                DataYearOut.unfrozen := DataYearIn.DLZ * (i - 1);
              i := i - 1;
            until (tem[i] > 0) or (i = 1);
            DataYearIn.unfrozen := DataYearOut.unfrozen;
          end;
          {this depth will limit the maximal rooting depth the next year}

          {  To reset cumulative values each year of execution, use the next line}
          {   otherwise comment it out}
          {      if (DAYM=ADAY) and (MONTH=AMONTH)INITIALIZE:=INITFLG}
          {   except on the last day.}
          {      if (DAY+1=CUMT)INITIALIZE:=0}
        end;
        {------------------------------------------------------------------}

        { MAINC2 50 } {process driver | Conditional: if time_to_print then write to *.SUM}
        if (iend_of_day = 1) and (prntop = 2) and (abs(print - 999) <= 0.01) and
          ((month > im) or (year <> iy)) then
          ITIME_TO_PRINT := 1;
        {*******************************************************************}
        {  Write to summary files}
        {   If a .SUM file was requested:}
        if (IFILE >= 2) then
        begin
          { Time to write a sum file?}
          IWRITE_SUM := 0;
          if (ABS(NSUM - 999) > 0.01) then
          begin
            if (RSUME - NSUM > -1E-4) or (N1PRT = 0) then
              IWRITE_SUM := 1;
          end
          else
          begin
            if (MONTH > IM) or (YEAR <> IY) then
              IWRITE_SUM := 1;
          end;

          {  If .SUM write flag set to 1 then write:}
          if (IWRITE_SUM = 1) then

          begin
            {  Screen print }
            if (ABS(NSUM - 999) < 0.01) then
            begin
              str(time: 10: 3, txt1);
              str(iy: 3, txt2);
              str(im: 3, txt3);
              str(round(crain): 6, txt4);
              txt1 := txt1 + ' ' + txt2 + ' ' + txt3 + ' ' + txt4;
              str(round(cevapa + ctrana): 6, txt2);
              str(round(clw): 6, txt3);
              str(round(excess): 6, txt4);
              txt1 := txt1 + ' ' + txt2 + ' ' + txt3 + ' ' + txt4;
              wwriteln(txt1);
            end;
            if (IFGSUM = 1) then
            begin
              if (ABS(NSUM - 999) < 0.01) then
                system.writeln(f12, '     Years      Year     Month',
                  '      Rain Irrigation  Runoff   ActEvap   PotEvap   ActTran',
                  '   PotTran  Drainage  WaterTbl',
                  '  WatFlux0  WatFlux1  WatFlux2  WatFlux3  WatFlux4',
                  '  mmWater1  mmWater2  mmWater3  mmWater4',
                  '    Theta1    Theta2    Theta3    Theta4',
                  '   Matric1   Matric2   Matric3   Matric4',
                  '       SAR1       SAR2       SAR3       SAR4        EC1',
                  '        EC2        EC3        EC4        pH1        pH2',
                  '        pH3        pH4     Sol1Ca     Sol1Mg     Sol1Na',
                  '      Sol1K     Sol1Cl    Sol1SO4   Sol1HCO3    Sol1CO3',
                  '     Sol2Ca     Sol2Mg     Sol2Na      Sol2K     Sol2Cl',
                  '    Sol2SO4   Sol2HCO3    Sol2CO3     Sol3Ca     Sol3Mg',
                  '     Sol3Na      Sol3K     Sol3Cl    Sol3SO4   Sol3HCO3',
                  '    Sol3CO3     Sol4Ca     Sol4Mg     Sol4Na      Sol4K',
                  '     Sol4Cl    Sol4SO4   Sol4HCO3    Sol4CO3       XCa1',
                  '       XMg1       XNa1        XK1       XCa2       XMg2',
                  '       XNa2        XK2       XCa3       XMg3       XNa3',
                  '        XK3       XCa4       XMg4       XNa4        XK4',
                  '     Tot1Ca     Tot1Mg     Tot1Na      Tot1K     Tot1Cl',
                  '    Tot1SO4   Tot1HCO3    Tot1CO3    Tot1Bal     Tot2Ca',
                  '     Tot2Mg',
                  '     Tot2Na      Tot2K     Tot2Cl    Tot2SO4   Tot2HCO3',
                  '    Tot2CO3     Tot2Bal    Tot3Ca     Tot3Mg     Tot3Na',
                  '      Tot3K',
                  '     Tot3Cl    Tot3SO4   Tot3HCO3    Tot3CO3    Tot3Bal',
                  '     Tot4Ca',
                  '     Tot4Mg     Tot4Na      Tot4K     Tot4Cl    Tot4SO4',
                  '   Tot4HCO3    Tot4CO3    Tot4Bal  T1Calcite   T1Gypsum',
                  '  T2Calcite',
                  '   T2Gypsum  T3Calcite   T3Gypsum  T4Calcite   T4Gypsum')
              {------------------}
              else
              if (ABS(NSUM - 999) >= 0.01) then
                system.writeln(f12, '      Days      Year     Month',
                  '      Rain  Irrigation  Runoff   ActEvap   PotEvap   ActTran',
                  '   PotTran  Drainage  WaterTbl',
                  '  WatFlux0  WatFlux1  WatFlux2  WatFlux3  WatFlux4',
                  '  mmWater1  mmWater2  mmWater3  mmWater4',
                  '    Theta1    Theta2    Theta3    Theta4',
                  '   Matric1   Matric2   Matric3   Matric4',
                  '       SAR1       SAR2       SAR3       SAR4        EC1',
                  '        EC2        EC3        EC4        pH1        pH2',
                  '        pH3        pH4     Sol1Ca     Sol1Mg     Sol1Na',
                  '      Sol1K     Sol1Cl    Sol1SO4   Sol1HCO3    Sol1CO3',
                  '     Sol2Ca     Sol2Mg     Sol2Na      Sol2K     Sol2Cl',
                  '    Sol2SO4   Sol2HCO3    Sol2CO3     Sol3Ca     Sol3Mg',
                  '     Sol3Na      Sol3K     Sol3Cl    Sol3SO4   Sol3HCO3',
                  '    Sol3CO3     Sol4Ca     Sol4Mg     Sol4Na      Sol4K',
                  '     Sol4Cl    Sol4SO4   Sol4HCO3    Sol4CO3       XCa1',
                  '       XMg1       XNa1        XK1       XCa2       XMg2',
                  '       XNa2        XK2       XCa3       XMg3       XNa3',
                  '        XK3       XCa4       XMg4       XNa4        XK4',
                  '     Tot1Ca     Tot1Mg     Tot1Na      Tot1K     Tot1Cl',
                  '    Tot1SO4   Tot1HCO3    Tot1CO3    Tot1Bal     Tot2Ca',
                  '     Tot2Mg',
                  '     Tot2Na      Tot2K     Tot2Cl    Tot2SO4   Tot2HCO3',
                  '    Tot2CO3     Tot2Bal    Tot3Ca     Tot3Mg     Tot3Na',
                  '      Tot3K',
                  '     Tot3Cl    Tot3SO4   Tot3HCO3    Tot3CO3    Tot3Bal',
                  '     Tot4Ca',
                  '     Tot4Mg     Tot4Na      Tot4K     Tot4Cl    Tot4SO4',
                  '   Tot4HCO3    Tot4CO3    Tot4Bal  T1Calcite   T1Gypsum',
                  '  T2Calcite',
                  '   T2Gypsum  T3Calcite   T3Gypsum  T4Calcite   T4Gypsum');
              {------------------}
              IFGSUM := 0;
            end; {if (IFGSUM=1)}
            {  Identify depth to water table }
            WETDEP := 9999;
            WETLOW := 9999;
            {------------------}
            I := KK;
            while I > 3 do
            begin { 4555}
              I := I - 1;
              {-------------}
              if (H[i] > 0) then
              begin
                if (wetdep = 9999) then
                  wetlow := (I - 1.5) * dlz + h[i];
                WETDEP := (I - 1.5) * DLZ - H[i];
              end;
              {-------------}
              {if (I>3)GO TO 4555}
            end;
            {------------------}
            if (ABS(NSUM - 999) <= 0.01) then
            begin
              {  Time-weighted contents for profile water content,}
              {  soluble + exch cations, calcite and gypsum}
              for SEG := 1 to 4 do
              begin
                TH[SEG] := TH[SEG] / RSUME;
                tCALCITE[seg] := tCALCITE[seg] / rsume;
                tgypsum[seg] := tgypsum[seg] / rsume;
                for M := 1 to NCHEM do
                  TSE[M, SEG] := TSE[M, SEG] / RSUME;
              end;
              {--------------    }
              {  Monthly print}
              system.writeln(f12, (((cycno - 1) * cumt) + time) / 365.25: 10: 4, iy: 10, im: 10,
                CRAIN - rr1: 10: 1, CIRRIG - rr1i: 10: 1, Excess - rr2: 10: 1,
                CEVAPA - rr3: 10: 1, CEVAPP - rr23: 10: 1,
                CTRANA - rr4: 10: 1, ctranp - rr24: 10: 1, clw - rr5: 10: 1, -wetdep: 10: 1,
                FW0: 10: 1, FW[1]: 10: 1, FW[2]: 10: 1, FW[3]: 10: 1, FW[4]: 10: 1,
                TH[1]: 10: 1, TH[2]: 10: 1, TH[3]: 10: 1, TH[4]: 10: 1,
                W[nd[1]]: 10: 3, W[nd[2]]: 10: 3, W[nd[3]]: 10: 3, W[nd[4]]: 10: 3,
                H[nd[1]] / 102.2: 10: 1, H[nd[2]] / 102.2: 10: 1, H[nd[3]] /
                102.2: 10: 1, H[nd[4]] / 102.2: 10: 1,
                SAR[ND[1]]: 11: 2, SAR[ND[2]]: 11: 2, SAR[ND[3]]: 11: 2, SAR[ND[4]]: 11: 2,
                EC[ND[1]]: 11: 2, EC[ND[2]]: 11: 2, EC[ND[3]]: 11: 2, EC[ND[4]]: 11: 2,
                PH[nd[1]]: 11: 2, PH[nd[2]]: 11: 2, PH[nd[3]]: 11: 2, PH[nd[4]]: 11: 2,
                SE[1, ND[1]]: 11: 3, SE[2, ND[1]]: 11: 3, SE[3, ND[1]]: 11: 3, SE[4, ND[1]]: 11: 3,
                SE[5, ND[1]]: 11: 3, SE[6, ND[1]]: 11: 3, SE[7, ND[1]]: 11: 3, SE[8, ND[1]]: 11: 3,
                SE[1, ND[2]]: 11: 3, SE[2, ND[2]]: 11: 3, SE[3, ND[2]]: 11: 3, SE[4, ND[2]]: 11: 3,
                SE[5, ND[2]]: 11: 3, SE[6, ND[2]]: 11: 3, SE[7, ND[2]]: 11: 3, SE[8, ND[2]]: 11: 3,
                SE[1, ND[3]]: 11: 3, SE[2, ND[3]]: 11: 3, SE[3, ND[3]]: 11: 3, SE[4, ND[3]]: 11: 3,
                SE[5, ND[3]]: 11: 3, SE[6, ND[3]]: 11: 3, SE[7, ND[3]]: 11: 3, SE[8, ND[3]]: 11: 3,
                SE[1, ND[4]]: 11: 3, SE[2, ND[4]]: 11: 3, SE[3, ND[4]]: 11: 3, SE[4, ND[4]]: 11: 3,
                SE[5, ND[4]]: 11: 3, SE[6, ND[4]]: 11: 3, SE[7, ND[4]]: 11: 3, SE[8, ND[4]]: 11: 3,
                xse[1, ND[1]]: 11: 3, xse[2, ND[1]]: 11: 3, xse[3, ND[1]]: 11: 3, xse[4, ND[1]]: 11: 3,
                xse[1, ND[2]]: 11: 3, xse[2, ND[2]]: 11: 3, xse[3, ND[2]]: 11: 3, xse[4, ND[2]]: 11: 3,
                xse[1, ND[3]]: 11: 3, xse[2, ND[3]]: 11: 3, xse[3, ND[3]]: 11: 3, xse[4, ND[3]]: 11: 3,
                xse[1, ND[4]]: 11: 3, xse[2, ND[4]]: 11: 3, xse[3, ND[4]]: 11: 3, xse[4, ND[4]]: 11: 3,
                Tse[1, 1]: 11: 3, Tse[2, 1]: 11: 3, Tse[3, 1]: 11: 3,
                Tse[4, 1]: 11: 3, Tse[5, 1]: 11: 3, Tse[6, 1]: 11: 3,
                Tse[7, 1]: 11: 3, Tse[8, 1]: 11: 3, Tse[9, 1]: 11: 3,
                Tse[1, 2]: 11: 3, Tse[2, 2]: 11: 3, Tse[3, 2]: 11: 3,
                Tse[4, 2]: 11: 3, Tse[5, 2]: 11: 3, Tse[6, 2]: 11: 3,
                Tse[7, 2]: 11: 3, Tse[8, 2]: 11: 3, Tse[9, 2]: 11: 3,
                Tse[1, 3]: 11: 3, Tse[2, 3]: 11: 3, Tse[3, 3]: 11: 3,
                Tse[4, 3]: 11: 3, Tse[5, 3]: 11: 3, Tse[6, 3]: 11: 3,
                Tse[7, 3]: 11: 3, Tse[8, 3]: 11: 3, Tse[9, 3]: 11: 3,
                Tse[1, 4]: 11: 3, Tse[2, 4]: 11: 3, Tse[3, 4]: 11: 3,
                Tse[4, 4]: 11: 3, Tse[5, 4]: 11: 3, Tse[6, 4]: 11: 3,
                Tse[7, 4]: 11: 3, Tse[8, 4]: 11: 3, Tse[9, 4]: 11: 3,
                TCALCITE[1]: 11: 3, tgypsum[1]: 11: 3,
                TCALCITE[2]: 11: 3, tgypsum[2]: 11: 3,
                TCALCITE[3]: 11: 3, tgypsum[3]: 11: 3,
                TCALCITE[4]: 11: 3, tgypsum[4]: 11: 3);
            end
            else
            if (abs(nsum - 999) > 0.01) then
            begin
              {   Periodic print}
              system.writeln(f12, (((cycno - 1) * cumt) + time): 10: 4, iy: 10, im: 10,
                CRAIN: 10: 1, CIRRIG: 10: 1, Excess: 10: 1, CEVAPA: 10: 1, CEVAPP: 10: 1,
                CTRANA: 10: 1, ctranp: 10: 1, clw: 10: 1, -wetdep: 10: 1,
                FW0: 10: 1, FW[1]: 10: 1, FW[2]: 10: 1, FW[3]: 10: 1, FW[4]: 10: 1,
                TH[1]: 10: 1, TH[2]: 10: 1, TH[3]: 10: 1, TH[4]: 10: 1,
                W[nd[1]]: 10: 3, W[nd[2]]: 10: 3, W[nd[3]]: 10: 3, W[nd[4]]: 10: 3,
                H[nd[1]] / 102.2: 10: 1, H[nd[2]] / 102.2: 10: 1, H[nd[3]] /
                102.2: 10: 1, H[nd[4]] / 102.2: 10: 1,
                SAR[ND[1]]: 11: 2, SAR[ND[2]]: 11: 2, SAR[ND[3]]: 11: 2, SAR[ND[4]]: 11: 2,
                EC[ND[1]]: 11: 2, EC[ND[2]]: 11: 2, EC[ND[3]]: 11: 2, EC[ND[4]]: 11: 2,
                PH[nd[1]]: 11: 2, PH[nd[2]]: 11: 2, PH[nd[3]]: 11: 2, PH[nd[4]]: 11: 2,
                SE[1, ND[1]]: 11: 3, SE[2, ND[1]]: 11: 3, SE[3, ND[1]]: 11: 3, SE[4, ND[1]]: 11: 3,
                SE[5, ND[1]]: 11: 3, SE[6, ND[1]]: 11: 3, SE[7, ND[1]]: 11: 3, SE[8, ND[1]]: 11: 3,
                SE[1, ND[2]]: 11: 3, SE[2, ND[2]]: 11: 3, SE[3, ND[2]]: 11: 3, SE[4, ND[2]]: 11: 3,
                SE[5, ND[2]]: 11: 3, SE[6, ND[2]]: 11: 3, SE[7, ND[2]]: 11: 3, SE[8, ND[2]]: 11: 3,
                SE[1, ND[3]]: 11: 3, SE[2, ND[3]]: 11: 3, SE[3, ND[3]]: 11: 3, SE[4, ND[3]]: 11: 3,
                SE[5, ND[3]]: 11: 3, SE[6, ND[3]]: 11: 3, SE[7, ND[3]]: 11: 3, SE[8, ND[3]]: 11: 3,
                SE[1, ND[4]]: 11: 3, SE[2, ND[4]]: 11: 3, SE[3, ND[4]]: 11: 3, SE[4, ND[4]]: 11: 3,
                SE[5, ND[4]]: 11: 3, SE[6, ND[4]]: 11: 3, SE[7, ND[4]]: 11: 3, SE[8, ND[4]]: 11: 3,
                xse[1, ND[1]]: 11: 3, xse[2, ND[1]]: 11: 3, xse[3, ND[1]]: 11: 3, xse[4, ND[1]]: 11: 3,
                xse[1, ND[2]]: 11: 3, xse[2, ND[2]]: 11: 3, xse[3, ND[2]]: 11: 3, xse[4, ND[2]]: 11: 3,
                xse[1, ND[3]]: 11: 3, xse[2, ND[3]]: 11: 3, xse[3, ND[3]]: 11: 3, xse[4, ND[3]]: 11: 3,
                xse[1, ND[4]]: 11: 3, xse[2, ND[4]]: 11: 3, xse[3, ND[4]]: 11: 3, xse[4, ND[4]]: 11: 3,
                Tse[1, 1]: 11: 3, Tse[2, 1]: 11: 3, Tse[3, 1]: 11: 3,
                Tse[4, 1]: 11: 3, Tse[5, 1]: 11: 3, Tse[6, 1]: 11: 3,
                Tse[7, 1]: 11: 3, Tse[8, 1]: 11: 3, Tse[9, 1]: 11: 3,
                Tse[1, 2]: 11: 3, Tse[2, 2]: 11: 3, Tse[3, 2]: 11: 3,
                Tse[4, 2]: 11: 3, Tse[5, 2]: 11: 3, Tse[6, 2]: 11: 3,
                Tse[7, 2]: 11: 3, Tse[8, 2]: 11: 3, Tse[9, 2]: 11: 3,
                Tse[1, 3]: 11: 3, Tse[2, 3]: 11: 3, Tse[3, 3]: 11: 3,
                Tse[4, 3]: 11: 3, Tse[5, 3]: 11: 3, Tse[6, 3]: 11: 3,
                Tse[7, 3]: 11: 3, Tse[8, 3]: 11: 3, Tse[9, 3]: 11: 3,
                Tse[1, 4]: 11: 3, Tse[2, 4]: 11: 3, Tse[3, 4]: 11: 3,
                Tse[4, 4]: 11: 3, Tse[5, 4]: 11: 3, Tse[6, 4]: 11: 3,
                Tse[7, 4]: 11: 3, Tse[8, 4]: 11: 3, Tse[9, 4]: 11: 3,
                TCALCITE[1]: 11: 3, tgypsum[1]: 11: 3,
                TCALCITE[2]: 11: 3, tgypsum[2]: 11: 3,
                TCALCITE[3]: 11: 3, tgypsum[3]: 11: 3,
                TCALCITE[4]: 11: 3, tgypsum[4]: 11: 3);
            end;
            RR1 := CRAIN;
            RR1i := CIRRIG;
            RR2 := EXCESS;
            RR3 := CEVAPA;
            RR4 := CTRANA;
            RR5 := CLW;
            RR23 := cevapp;
            rr24 := ctranp;
            {  Reset summary file accumulators}
            RSUME := 0;
            FW0 := 0;
            for seg := 1 to 4 do
            begin
              for m := 1 to nchem do
                FSE[M, SEG] := 0;
              FW[SEG] := 0;
            end;
            {------------------}
            for SEG := 1 to 4 do
            begin
              TH[SEG] := 0;
              tCALCITE[seg] := 0;
              tgypsum[seg] := 0;
              for M := 1 to NCHEM do
              begin
                wsfer[m] := 0;
                wrunoff_load[m] := 0;
                TSE[M, SEG] := 0;
              end;
            end;
            {-------------}
          end; {if (IWRITE_SUM=1) }
        end; {if (IFILE>=2)}
        {-----------------------------}
        n1prt := 1;

        { MAINC2 51 } {process driver | Next time interval within day, until iend_of_day}
      until (iend_of_day <> 0);
      {if (iend_of_day=0)go to 130}
      {********************************************************************}
      {  The following are special writes that have been used to record}
      {  data on a daily, monthly or annual basis, usually as a grid of }
      {  values on depth/time axes.  They are one-off's, and may not mean much.}
      {  Triggered when the output file number is set to some value >3.  These}
      {  outputs are not part of the standard LEACHM package.}
      {--------------------------------------------------------------------}
      {  To record profile data for a desired chemical species at the end}
      {  of each calendar month, use the following.  Change DSE(1,i) to }
      {  reflect the chemical species number.  Written here for species '1'.}
      {  Also, change the do statement to reflect the desired nodes.}
{  if (ifile>=4) and (nyc=cycno) and (iy>98) then
  begin
      write(f15,'(f9.4,3i4,100f12.2)')time/365.25:9:4,iy:4,im:4,idaym:4,
     #(ec[i],i:=kk-1,2,-1)
      write(f16,'(f9.4,3i4,100f7.3)')time/365.25,iy,im,idaym,
     #(w[i]/rho[i],i:=kk-1,2,-1)
      write(f17,'(f9.4,3i4,100f9.1)')time/365.25,iy,im,idaym,
     #(h[i]/102.2,i:=kk-1,2,-1)
  end;}

  { MAINC2 52 } {process driver | Refresh the form of the GUI after updating the day number}
      {  Increment month and year printed in .SUM file}
      im := month;
      iy := year;
      untGUI.frmMAIN.ggProgressCurrentYear.Position := DAY;
      if DAY mod 30 = 0 then
        untGUI.frmMAIN.GeneralRefresh;
      {pf2006 : remove the above 2 lines if no graphical progress output desired}
      {--------------------------------------------------------------------}
 {lines below added by pf2006 to calculate indicator for clay stability.
  No longer used because since 2008 clay migration process is simulated.
 for i:=2 to kk-1 do
 begin
   if (TypeChem=1)or(TypeChem=2) then {use an indicator based simply on pH, vanBreemen and Buurman 2002
   begin
     rdum:=1;
     if pH[i]<4 then rdum:=0;
     if (pH[i]>=4)and(pH[i]<=5) then rdum:=pH[i]-4;
     if pH[i]>7.5 then rdum:=0;
     if (pH[i]>=7)and(pH[i]<=7.5) then rdum:=1-(pH[i]-7)/0.5;
     ClayDispInd[i]:=ClayDispInd[i]+rdum;
   end
   else {use an indicator based on pH and ESP, vanBreemen and Buurman 2002
   begin
     if pH[i]<7
     then rdum:=0
     else rdum:=16.5-16.5/power((PH[i]-6),2.7);  {precalc right instability margin
     if   (ESP[i]>max(150-30*PH[i],0))  {left instability margin
       and(ESP[i]>rdum)                 {right instability margin
       and(GYPA[i]<0.001)               {but no gypsum present
     then ClayDispInd[i]:=ClayDispInd[i]+1;
   end;
 end;
 ClayCount:=ClayCount+1;
 }

 { MAINC2 53 } {process driver | Conditional: if set in GUI then call WriteDailyTDD (selected soil variables)}
 {  Return for start of new day}
    if untGUI.writeDaily then WriteDailyTDD;

    { MAINC2 54 } {process driver | Next day}
    end; {do 1000 CONTINUE  }

    {  Start next cycle of boundary conditions}
    {      write(*,*)nyc,ch_irr_flag}
    if (nyc > 1) and (CH_IRR_FLAG = 1) then
    begin
      {  Reset rainfall records, eliminating automatic irrigations      }
      IRRNO := 0;
      for J := 1 to NIRRIG do
      begin
        if (IRRFLG[j] <> 3) then
        begin
          IRRNO := IRRNO + 1;
          AIRRBG[irrno] := AIRRBG[j];
          IRATEA[irrno] := IRATEA[j];
          IRRIGA[irrno] := IRRIGA[j];
          irrflg[irrno] := irrflg[j];
          for M := 1 to NCHEM do
            ARSE[m, IRRNO] := ARSE[m, j];
        end;
      end;
      NIRRIG := IRRNO;
    end; {if}

        { MAINC2 55 } {process driver | Next cycle (not used in SoilGen)}
  end; {do ...label 1111 CONTINUE}
  {  End of cycle or year}

  { MAINC2 56 } {process driver | call RothC_exit (at end of year; write C-and associated element pools to outputfile RC*.out)}
  {PF2006: exit conversions C-, Weathering and Clay migration subprogrammes}
  RothC_exit;

  { MAINC2 57 } {process driver | call WE_Exit (at end of year; write minerals and cations locked in minerals to WE*.out)}
  WE_Exit;

  { MAINC2 58 } {process driver | call PassdataOut (at end of year; passes data from Processes unit to calling main program untGUI.Run at end of year via the record DataYearOut)}
  {PF2006: pass data directly to calling program to avoid loss of precision in write>read}
  PassDataOUT;
  {includes Exit_Clay_Migration;}
  {trial: what were the timestep lengths in this 1-year run?}
 {system.assign(ftrial,'trials.txt'); system.Rewrite(ftrial);
 system.writeln(ftrial,'timestep frequency');
 for i:=1 to 10 do writeln(ftrial,min(power(10,-i),delm),': ',freqdelt[i]:15:0);}
  {closefile(ftrial); }
  {--------------------------------------------------------------------}
  untGUI.frmMAIN.MVAR(GlobalTime*10000+Datayearin.nolayers*100,' eind untProcesses.MAINC: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
  untGUI.frmMAIN.MVAR(GlobalTime*10000+Datayearin.nolayers*100,' eind untProcesses.MAINc: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);

end;

procedure WATDAT;
{create an output table (part of *.out) of h-theta-K data}
{---------------------------------------------------------------------;}
{ Profile variables dimensioned for LL nodes (LL: No of nodes);}
var
  K, E: array[1..LL, 1..6] of real;
  P: array[1..6] of real;
  Z, J: integer;
  R13, DEPTH: real;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering WATDAT');
  {----------------------------------------------------------;}
  {  Point of coincidence of exponential and parabolic functions; Hutson&Cass modification to Campbell}
  for J := 1 to KK do
  begin
    TS[J] := max(TS[J],0.05);
    L[j] := 2 * BCAM[j] * TS[j] / (1 + 2 * BCAM[j]);
    RG[j] := AEV[j] * power((L[j] / TS[j]), (-BCAM[j]));
    {----------------------------;}
    {  Various options for setting the maximum matric potential to prevent;}
    {  numerical errors arising from the differential water capacity;}
    {  approaching zero;}
    {   i) to correspond to a defined differential water capacity (SPCMIN);}
    {      SPCMIN:=1E-04;}
    {      HM[j]:=SPCMIN*AEV[j]*AEV[j]*(L[j]/TS[j])**(-2.*BCAM[j]);}
    {      HM[j]:=-HM[j]/(2.*TS[j]*(1.-L[j]/TS[j]));}
    {      HM[j]:=AMAX1(-2.,HM[j],RG[j]);}
    {  ii) maximum matric potential set to -2 mm hydraulic head;}
    HM[j] := -2;
    {----------------------------;}
    {  Water content corresponding to maximum matric potential;}
    if (hm[j] > rg[j]) then
      TM[j] := TS[j] - (TS[j] * HM[j] * HM[j] * (1 - L[j] / TS[j])) /
        (AEV[j] * AEV[j] * power((L[j] / TS[j]), (-2 * BCAM[j])));
    if (hm[j] <= rg[j]) then
      tm[j] := TS[j] * power((hm[j] / aev[j]), (-1 / bcam[j]));
    {----------------------------;}
  end; {500 CONTINUE;}
  {----------------------------------------------------------;}
  system.writeln(f11, '                   ',
    'SOIL HYDROLOGICAL CHARACTERISTICS');
  system.writeln(f11);
  system.writeln(f11);
  system.writeln(f11, '          ',
    '   PREDICTED RETENTIVITY AND CONDUCTIVITY DATA');
  system.writeln(f11);
  system.writeln(f11);
  system.writeln(f11, ' Depth', '                     ',
    'Water content, theta', '                      ',
    'Retention parameters');
  system.writeln(f11, '   mm ', '                    ', '                     ',
    '                      ', ' used in simulation');
  system.writeln(f11);
  system.writeln(f11, ' Depth_mm   0_kPa    -3_kPa   -10_kPa   -30_kPa  ',
    '-100_kPa -1500_kPa   a_kPa      b    p   BD_kg/dm3');
  system.writeln(f11);
  system.writeln(f11);
  {  THETA AND CONDUCTIVITY AT SPECIFIED POTENTIALS (P[i] VALUES)
   these are in mm water;}
  P[1] := 0;
  P[2] := -306.6;
  P[3] := -1022;
  P[4] := -3066;
  P[5] := -10220;
  P[6] := -153300;
  for Z := 1 to 6 do
  begin
    for J := 1 to KK do
    begin
      R13 := P[Z];
      E[J, Z] := THETA(J, R13, RG[j], TS[j], AEV[j], BCAM[j], L[j], TM[j], HM[j]);
      K[J, Z] := HC(j, R13, RG[j], KS[j], AEV[j], BCAM[j], TS[j], L[j], RP[j], TM[j], HM[j], TEM[j]);
      { Arguments TM,HM added by PF2006;}
    end;
  end;
  for J := 2 to KK - 1 do
  begin
    DEPTH := -(j - 1.5) * DLZ;
    system.writeln(f11, DEPTH: 6: 0, E[J, 1]: 10: 3, E[J, 2]: 10: 3, E[J, 3]: 10: 3,
      E[J, 4]: 10: 3, E[J, 5]: 10: 3, E[J, 6]: 10: 3, '  ',
      AEV[j] / 102.2: 7: 3, BCAM[j]: 8: 3, RP[j]: 6: 2, rho[j]: 7: 3);
  end;
  system.writeln(f11);
  system.writeln(f11);
  system.writeln(f11, ' Depth                     Gravimetric water content');
  system.writeln(f11);
  system.writeln(f11, ' Depth_mm   0_kPa    -3_kPa   -10_kPa   -30_kPa  -100_kPa -1500_kPa  ');
  system.writeln(f11);
  system.writeln(f11);
  for J := 2 to KK - 1 do
  begin
    DEPTH := -(j - 1.5) * DLZ;
    system.writeln(f11, DEPTH: 6: 0, E[J, 1] / rho[j]: 10: 3, E[J, 2] / rho[j]: 10: 3,
      E[J, 3] / rho[j]: 10: 3,
      E[J, 4] / rho[j]: 10: 3, E[J, 5] / rho[j]: 10: 3, E[J, 6] / rho[j]: 10: 3);
  end;
  system.writeln(f11);
  system.writeln(f11);
  system.writeln(f11, ' Depth                      Conductivity mm/day ');
  system.writeln(f11);
  system.writeln(f11, ' Depth_mm  0_kPa    -3_kPa   -10_kPa   -30_kPa  -100_kPa -1500_kPa  ');
  system.writeln(f11);
  system.writeln(f11);
  for J := 2 to KK - 1 do
  begin
    DEPTH := -(j - 1.5) * DLZ;
    system.writeln(f11, depth: 6: 0, K[J, 1], ' ', K[J, 2], ' ', K[J, 3], ' ',
      K[J, 4], ' ', K[J, 5], ' ', K[J, 6], ' ');
  end;
  system.Flush(f11);
  {if (TRACE = 1) then }
    singletracewriteln(' Leaving Watdat');
end; {watdat}

procedure POTET(var pPAN, pDLYPAN, pDLYEP, pDLYTP, pCRPCOV, pERMAX, pPANFAC: real);
{calculate potential evapotranspiration for the day}
{ THIS ROUTINE CALCULATES POTENTIAL EVAPORATION AND TRANSPIRATION }
{ FOR THE DAY. POTENTIAL TRANSPIRATION IS CALCULATED BY MULTIPLYING }
{ PAN EVAPORATION BY CROP COVER. POTENTIAL EVAPORATION IS THE }
{ DIFFERENCE BETWEEN PAN EVAPORATION AND POTENTIAL TRANSPIRATION. }
{------------------------ }
{ PAN:=PAN EVAPORATION mm/week }
{ DLYPAN:=PAN EVAPORATION mm/day }
{ DLYTP:=POTENTIAL TRANSPIRATION mm/day }
{ CRPCOV:=CROP COVER (FRACTION) }
{ DLYEP:=POTENTIAL EVAPORATION mm/day }
begin
  {if (TRACE = 1) then }
    singletracewriteln(' Entering Potet');
  pDLYPAN := pPAN / 7;
  pDLYTP := pDLYPAN * pCRPCOV * pPANFAC;
  pDLYEP := pDLYPAN * (1 - pCRPCOV) * pPANFAC;
  { Maximum rate of evaporation (at 0.55 day) }
  pERMAX := pDLYEP * PI;
  {if (TRACE = 1) then }
    singletracewriteln(' Leaving Potet');
end;

procedure RUNOFF(POROSITY, WP, FC, SW, SLOPE, CN2, IRRIG: real; var RO, s, s1: real; tem: real);
{calculate runoff via the curve number approach (called from MAINC2)}
{  From SWRRB and/or Williams (Modeling plant and soil systems, p440)}
{  Equations from these sources are similar, but were derived using}
{  different ratios to express water content.}
{  We have used the Williams equations here, because there is an equation}
{   which adjusts CN2 according to slope.}
{  As long as water and porosity are expressed in the same volumetric}
{   units, the absolute values do not matter, as ratios are used.}
{---------------------------------------------------------------------}
var
  cn2o, UL, FFC, CN1, CN3, CN2S, S3, C1, C2, W1, W2: real;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering Runoff');
  cn2o := cn2;
  {   UL is the upper limit of water storage in the root zone, mm}
  UL := (POROSITY - WP) / (FC - WP);
  {  FFW is the fraction of field capacity, SW is soil water content in}
  {   the root zone.}
  FFC := (SW - WP) / (FC - WP);
  {  Input CV2 (the mean curve number)}
  {  CV1 and CV3 are calculated as (Williams, eq 4, 5)}
  CN1 := CN2 - 20 * (100 - CN2) / (100 - CN2 + EXP(2.533 - 0.0636 * (100 - CN2)));
  CN3 := CN2 * EXP(0.00673 * (100 - CN2));
  {  The curve number CN2 is adjusted for slope, assuming that handbook}
  {    CN2 value is for a 5% slope.  }
  CN2S := (CN3 - CN2) * (1 - 2 * EXP(-13.86 * SLOPE / 100)) / 3 + CN2;
  CN2 := CN2S;
  {  (Should this be used to calculate CN1 and CN3?  We do that here.)}
  {  Recalculate CN1 and CN3 using slope-adjusted CN2}
  CN1 := CN2 - 20 * (100 - CN2) / (100 - CN2 + EXP(2.533 - 0.0636 * (100 - CN2)));
  CN3 := CN2 * EXP(0.00673 * (100 - CN2));
  {  S3 is retention at field capacity    }
  S3 := 254 * (100 / CN3 - 1);
  {  and s1 is retention at wilting point.}
  S1 := 254 * (100 / CN1 - 1);
  {   Constants for eqs 7 and 8}
  C1 := 1 / (1 - S3 / S1);
  {   S1 must be greater than 2.54?  Where does 2.54 come from?}
  C2 := UL / (1 - 2.54 / S1) - UL;
  c2 := max(1e-7, c2);
  W2 := (LN(C1 - 1) - LN(C2)) / (UL - 1);
  W1 := LN(C1 - 1) + W2;
  {  Runoff is calculated as a function of rainfall and the retention}
  {  parameter, S, which is a function of water content, and the above}
  {  constants.}
  S := S1 * (1 - FFC / (FFC + EXP(W1 - W2 * FFC)));
  {  If temperature at 200 mm is less than 0C then reduce S.}
  if (tem < 0) then
    s := s1 * (1 - exp(-0.00292 * S));
  {  If rainfall is less than 0.2S then runoff is zero.}
  if (IRRIG >= 0.2 * S) then
    RO := power((IRRIG - 0.2 * S), 2) / (IRRIG + 0.8 * S)
  else
    RO := 0.;
  ro := min(ro, irrig * 0.95);
  cn2 := cn2o;
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Runoff');
end;

procedure WUPTAK;
{calculate water sink term for WATFLO ie. root uptake}
var
  RTGRD: rLL;
  wSUM, HTOP, HBOT, root_to_soil: real;
  i, j: integer;
  {----------------------------}
  {  THIS SUBROUTINE CALCULATES ROOT UPTAKE OF WATER.  A VALUE OF HROOT IS}
  {  CHOOSEN THAT ALLOWS THE ROOT EXTRACTION OVER THE ROOT PROFILE TO }
  {  EQUAL THE POTENTIAL TRANSPIRATION.  THIS IS DONE USING THE BISECTION }
  {  ITERATIVE PROCEDURE. }
  {----------------------------   }
  {  A:=ROOT EXTRACTION TERM (M**3/M**3-DAY)   }
  {  TRANP:=POTENTIAL TRANSPIRATION DURING DELT (MM)   }
  {   THEREFORE FLUX DENSITY DURING DELT:=TRANP/DELT MM/DAY}
  {  HBOT:=LOWER LIMIT OF INTERVAL }
  {  HHIGH:=MAXIMUM ROOT WATER POTENTIAL (MM WATERKPA) }
  {  HLOW:=MINIMUM ROOT WATER POTENTIAL (MM WATER)  }
  {  HROOT:=ROOT WATER POTENTIAL AT THE SOIL SURFACE (NOW MM WATER) }
  {  HTOP:=UPPER LIMIT OF INTERVAL }
  {  RRES:=ROOT FLOW RESISTANCE TERM   }
  {  RTGRD:=ROOT GRADIENT (MM) }
  {  SALT:=TOTAL SALT CONCENTRATION (ME/L) }
  {  SE'S ARE THE CONCENTRATIONS OF DIFFERENT SALT SPECIES (ME/L) }
  {-----------------------------  }
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering Wuptak');
  wSUM := 0;
  {----------------------------}
  for I := 1 to KK do
  begin
    {  Set root extraction term A[i] to zero and calculate osmotic}
    {   potential.}
    A[i] := 0;
    {  To ensure no osmotic effects, set SALT to 0.}
    {       salt[i]:=0}
    {  Effective water potential := matric + osmotic - root resistance term.}
    {  (Positive pressure potentials assumed zero to prevent water}
    {  spontaneously moving from soil to atmosphere)}
    RTGRD[i] := MIN(0, G[i]) + SALT[i] - DLZ * RRES * i;
  end;
  {----------------------------}
  if (KILPLT = 1) and (TRANP > 0) then
  begin
    J := 0;
    HTOP := HHIGH;
    HBOT := HLOW;
    HROOT := (HLOW + HHIGH) / 2;
    while (J < 40) do
    begin
      {--------------------------------------------------------------------}
      if (ABS(wSUM - TRANP) > 0.001) then;
      {----------------------------}
      {  Total the sink terms (theta change rate * depth increment }
      {   * time step)}

      wSUM := 0;
      for I := 2 to KK - 1 do
      begin
        {----------------------------}
        {  Don't allow water to flow from root to soil and stop uptake if soil}
        {   below wilting point.}

        if (HROOT >= RTGRD[i]) or (RTGRD[i] <= HDRY) then
          A[i] := 0
        else
        begin
          {----------------------------}
          {  If root potential less than effective soil water potential:}

          {  Add the next two lines if 'root density' is to decrease as root}
          {  fraction decreases. }
          root_to_soil := 1e+10;
          {      if(rdf[i]>0.0001)root_to_soil:=10./rdf[i]}
          if (rdf[i] > 0.0001) then
            root_to_soil := 10;
          {  Options for K and potential weighting}
          {      A[i]:=amin1(2.,B[i]**.33)}
          {     #*RDF[i]*(RTGRD[i]-HROOT)/(DLZ*root_to_soil)   }
          {xxx the next line was conditionally de-activated pf7-11-2011}
          if not (WaterTableInRootingZone) then
            A[i] := B[i] * RDF[i] * (RTGRD[i] - HROOT) / (DLZ * root_to_soil)
          {  Use this for potential weighting only}
          {xxx the next line was conditionally activated pf7-11-2011}
          else
            A[i] := RDF[i] * (RTGRD[i] - HROOT) / (DLZ * root_to_soil);
          wSUM := wSUM + A[i] * DLZ * DELT;
        end;
      end;
      {------------------------------------   }
      {  If profile too dry to satisfy transpiration, stop iteration}

      if (wSUM < TRANP) and (HROOT = HLOW) then
        J := 40
      {----------------------------}
      {  If transpiration too high then HROOT must increase   }

      else
      begin
        if (wSUM > TRANP) then
        begin
          HBOT := HROOT;
          HROOT := (HBOT + HTOP) / 2;
        end
        {----------------------------}
        {  If transpiration too low then HROOT must decrease}

        else if (wSUM < TRANP) and (HROOT > HLOW) then
        begin
          HTOP := HROOT;
          HROOT := (HBOT + HTOP) / 2;
        end;
        {----------------------------}
        {  If transpiration correct}

        if (ABS(wSUM - TRANP) <= 0.001) then
          J := 40;
        {--------------------------------------------------------------------}
      end;
      J := J + 1;
    end; {while} {was: IF(J<40)GO TO 6;}
  end;
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Wuptak');
end;

procedure TSTEP;
{calculate the length of the new time increment}
{ TIMES ELAPSED: TIMDAY:=PER DAY     }
{                R20:=PER .1 DAY     }
{  DELT:=TIME STEP days   }
{  DELM:=MAXIMUM TIME STEP day       }
{----------------------------}
{  Set starting lower boundary for lysimeter if first time step}
var
  ir9old, E1, E2, E3, R9old, ENDIRR, DELTE, TIMIRR: real;
  i: integer;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering Tstep');
  if (N1PRT = 0) then     {only at starting time of annual run}
  begin
    if (BBC = 4) then     {only when BBC=lysimeter}
    begin
      if ((H[KK - 1] + DLZ / 2) - BBPOT >= 0.1) then
        R9 := 1           {set watflo BBC to fixed potential}
      else
        R9 := 3;          {set watflo BBC to 0 flux}
      R9OLD := R9;
    end;
  end;

  {DETT:=1E-10;  now set via constant}
  ENDIRR := 1;
  DELTE := 1;
  if (R20 - delm >= -0.0000001) then
  begin
    R20 := 0;
    TIMDAY := round(TIMDAY / delm) * delm;
    {9999 FORMAT('      Time elapsed : ',F6.1,' day');}
  end;
  if (HEAD > 0) and (FLXDEN[1] > 0) then
    ENDIRR := HEAD / FLXDEN[1] * 1.001;
  R30 := 2;
  {----------------------- }
  {  Calculate new time period as minimum of     }
  {    i) DELM, ii)time to print, iii)time to irrigate      }
  {   iv) end of irrigation, v) time to end of .1 day period}
  {  vi) time for theta change of CONQ}
  {  Estimate time for CONQ*DLZ mm water to evaporate       }
  if (ERMAX > 0) and (HEAD <= 0) then
  begin
    E1 := CONQ * DLZ / max(ERMAX * (SIN(2 * PI * (TIME + DELM / 2 - 0.3))), 0.001);
    E2 := CONQ * DLZ / max(WFDD, 0.001);
    E3 := max(E1, E2);
  end
  else
    E3 := 1;
  {--------------------------         }
  {  For lysimeter when BBC changes}
  if (BBC = 4) then
  begin
    ir9old := r9;
    if ((H[KK - 1] + dlz / 2) - BBPOT > 0) then
    begin
      R9 := 1;
      IFLG3 := 1;
      delte := dett;
      for i := 1 to kk do
      begin
        h[i] := min(0, h[i]);
        g[i] := min(0, g[i]);
      end;
    end
    else
      R9 := 3;
    if (ir9old = 3) and (r9 = 1) then
      delt := dett;
  end;
  if (BBC <> 4) then
    R9 := BBC;
  if (TIME >= IRRBEG) and (IHEAD = 0) then
  begin
    IHEAD := 2;
    IFLG3 := 1;
    IFLG2 := 0;
    delte := dett;
  end;
  if (IFLG3 = 1) and (delt < delm) then
  begin
    DELTE := min(delm, DETT * power(1.2, IFLG2));
    IFLG2 := IFLG2 + 1;
  end;
  if (IFLG2 = 900) or ((IFLOW = 3) and (IFLG2 = 2)) then
  begin
    DELTE := 1;
    IFLG3 := 0;
  end;
  {--------------------------}
  {  Time to next irrigation or rain}
  TIMIRR := (IRRBEG - TIME) * 1.0001;
  if (TIMIRR < DETT) then
    TIMIRR := 1.0;
  {--------}
  DELT := min(E3, CONQ * DLZ / FMAX);
  DELT := min(DELT, DELM);
  DELT := min(DELT, ENDIRR);
  DELT := min(DELT, delm - R20);
  DELT := min(DELT, DELTE);
  DELT := min(DELT, TIMIRR);
  {oorspr. ftn: DELT:=min(E3,CONQ*DLZ/FMAX,DELM,ENDIRR,delm-R20,DELTE,TIMIRR)}
  {------------------------}
  DELT := max(DELT, DETT);
  DELT := round(DELT / DETT) * DETT;
  if (DELM = DELT) then
    R30 := 1;
  R20 := R20 + DELT;
  FMAX := power(10, (-10));
  TIMDAY := TIMDAY + DELT;
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Tstep');
  {optimization trials: what log-timesteps do occur}
  {freqdelt[round(-log10(delt))]:=freqdelt[round(-log10(delt))]+1; }
end;

procedure WATFLO(var ros, rots, rowp, rofc, row, s1, DIFF: real);
{calculate water flux between nodes and new potentials by Richard's equation}
{  This program solves Richard's equation for water flow.       }
{  It is based on Hutson's (1983) thesis and uses the              }
{  functions described there for characterizing retentivity        }
{  and conductivity}

{   LL: No of nodes}
var
  F1, G1, SPC: array[1..LL] of double;
  R32, R33, R34, R35, dh: double;
  HO: rLL;
  PKSMIN, watcap, p1, PSIMAX, wHI, HL, SPOT, r13, SFLUX, EV, rwnew, R37: real;
  i, MINKS, NF, niter: integer;
  goto4300, goto2768: boolean;
  {------------------------                }
  {  Lower boundary conditions now set in TSTEP}
  {------------------------}
  {      write(*,'(e10.3,5f10.2,i4)')time,h[kk],h[kk-1],g[kk],g[kk-1]}
  {     #,bbpot,r9}
  {      ENDIF}
  {  Minimum allowable water application rate is 1 mm/day}
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering Watflo');
  IRATE := MAX(1, IRATE);
  {  Initialize variables     }
  TRANA := 0;
  PKSMIN := KS[1];
  r35 := 0;
  r32 := 0;
  r33 := 0;
  r34 := 0;
  for i := 1 to kk do
  begin
    pksmin := min(pksmin, ks[i]);
    HO[i] := G[i];
    g1[i] := 0;
    f1[i] := 0;
  end;
  {*******************************************************************}
  { Select top boundary condition according to presence of           }
  {   ponded water, time to start irrigation or evaporative flux     }
  {*******************************************************************}
  {   S T A R T   O F   I N F I L T R A T I O N    E V E N T   }
  {*******************************************************************}
  {*******************************************************************}
  if (IHEAD = 2) then
  begin
    IHEAD := 1;
    rnoff := 0;
    rots := (ts[2] + ts[3]) / 2;
    rowp := (wp[2] + wp[3]) / 2;
    rofc := (fc[2] + fc[3]) / 2;
    row := (w[2] + w[3]) / 2;
    s1 := 0;
    ros := 0;
    HEAD := IRRIG - Rnoff;
    if (ABS(IRATE - 999.9) < 0.001) then
      IRREND := 1 * power(10, 5)
    else
      IRREND := IRRIG / IRATE;
  end;
  { ---------------------------}
  if (ihead = 1) then
  begin
    if (ABS(IRATE - 999.9) > 0.001) then
    begin
      {  Irrigation or rain application rate is set to ponded code (999.9)}
      {   if air-filled pore space above a possible flux -limiting soil node}
      {   (identified by a Ks lower than the application flux or a water}
      {   content greater than 0.99 saturation) is less than the amount of }
      {    water to infiltrate.}
      WATCAP := 0;
      MINKS := 2;
      {----------------------------}
      i := 1;
      while i < KK do
      begin {2730}
        i := i + 1;
        if (KS[i] < IRATE) or (W[i] >= 0.9999 * tm[i]) then
        begin
          MINKS := i;
          i := KK;
        end;
        {IF(i<KK)GO TO 2730;}
      end;
      {----------------------------}
      for i := 2 to MINKS do
      begin
        p1 := HM[i];
        WATCAP := watcap + (ts[i] - W[i]) * DLZ;
      end;
      if (PKSMIN < IRATE) and (IRATE * DELT >= WATCAP) then
        IRATE := MAX(PKSMIN, WATCAP / DELT + PKSMIN);
      if (R9 = 3) and ((IRATE * DELT - WATCAP) > -0.0001) then
        IRATE := WATCAP / DELT;
    end; {if}
  end; {IF}
  {*******************************************************************}
  {*******************************************************************}
  {     I N F I L T R A T I O N   S U R F A C E   B O U N D A R Y}
  {*******************************************************************}
  {  Ponded infiltration ie. surface node saturated but no           }
  {   positive potential      }
  if (IHEAD = 1) then
  begin
    if (abs(IRATE - 999.9) < 0.001) then
    begin
      G[1] := 0;
      H[1] := 0;
      F1[1] := 0;
      G1[1] := 0;
    end
    else
    begin
      {---------------------------             }
      {  Constant flux infiltration (surface potential giving rise to    }
      {   desired infiltration rate is approximated by iteration         }
      PSIMAX := HM[1];
      wHI := 5 * dlz;
      HL := -300000;
      SPOT := MIN(G[2], hm[2]);
      r13 := g[2];
      B[2] := HC(2, r13, RG[2], KS[2], AEV[2], BCAM[2], TS[2], L[2], RP[2], TM[2], HM[2], TEM[2]);
      { Arguments TM,HM added by PF2006     }
      goto2768 := False;
      for NF := 1 to 60 do
      begin
        if not (goto2768) then
        begin
          r13 := min(spot, hm[1]);
          B[1] := HC(2, SPOT, RG[2], KS[2], AEV[2], BCAM[2], TS[2], L[2], RP[2],
            TM[2], HM[2], TEM[2]);
          { Arguments TM,HM added by PF2006     }
          B[1] := (B[1] + B[2]) / 2;
          SFLUX := B[1] - B[1] * (G[2] - SPOT) / DLZ;
          goto2768 := (ABS(1 - SFLUX / IRATE) < 0.001);
          if not (goto2768) then
          begin
            if (SFLUX - IRATE < 0) then
            begin
              HL := SPOT;
              SPOT := (SPOT + whi) / 2;
            end
            else
            begin
              whi := SPOT;
              SPOT := (SPOT + HL) / 2;
            end;
          end;
        end;
      end;
      G[1] := SPOT;
      H[1] := SPOT;
      {----------------------------}
      F1[1] := 1;
      G1[1] := (irate - B[1]) * 2 * DLZ / B[1] - G[1] + G[2];
    end;
    {----------------------------}
  end;
  {*******************************************************************}
  {*******************************************************************}
  {        E V A P O R A T I O N   S U R F A C E   B O U N D A R Y}
  {*******************************************************************}
  {  Evaporative flux density set to -ER mm/day (calculated in ETRANS)            }

  if (IHEAD = 0) then
  begin
    EV := -ER;
    g[1] := g[2];
    B[1] := HC(1, G[1], RG[1], KS[1], AEV[1], BCAM[1], TS[1], L[1], RP[1], TM[1], HM[1], TEM[1]);
    { Arguments TM,HM added by PF2006     }
    B[2] := HC(2, G[2], RG[2], KS[2], AEV[2], BCAM[2], TS[2], L[2], RP[2], TM[2], HM[2], TEM[2]);
    { Arguments TM,HM added by PF2006     }
    B[1] := (b[1] + b[2]) / 2;
    if (abs(ev) > 0.0001) then
    begin
      {  Evaporation flux density compared with an estimate of maximum}
      {   possible from the surface segment to choose}
      if (EV - B[1] * (-3000 * 102.2 - G[2]) / DLZ <= 0)
      {  i) a falling rate evaporation (soil conductivity limiting)} then
      begin
        G1[1] := -3000 * 102.2 - G[2];
        F1[1] := 1;
      end
      {  ii) constant rate evaporation (evaporativity limiting)}
      else
      begin
        F1[1] := 1;
        G1[1] := (EV - B[1]) * 2 * DLZ / B[1] - G[1] + G[2];
      end;
    end
    else
    begin
      F1[1] := 1.0;
      if (B[1] > 0.000001) then
        G1[1] := MIN(-DLZ, -B[1] * 2 * DLZ / B[1] - G[1] + G[2])
      else
        G1[1] := -DLZ;
    end;
  end;
  {*******************************************************************}
  {*******************************************************************}
  {  Estimated conductivity (B[i]) and differential water capacity}
  {   (SPC[i]) of segments (two iterations start here; three if DELT<DELM)}
  {   Hydraulic conductivity mm/day between nodes calculated as      }
  {    mean of two nodal values at start and end of time step           }
  goto4300 := False;
  for NITER := 1 to 4 do {do 3500}
  begin
    if not (goto4300) then
    begin
      B[1] := HC(1, G[1], RG[1], KS[1], AEV[1], BCAM[1], TS[1], L[1], RP[1], TM[1], HM[2], TEM[2]);
      { Arguments TM,HM added by PF2006     }
      for i := 2 to KK do
      begin
        B[i] := HC(i, G[i], RG[i], KS[i], AEV[i], BCAM[i], TS[i], L[i], RP[i], TM[i], HM[i], TEM[i]);
        { Arguments TM,HM added by PF2006}
        B[i - 1] := (B[i] + B[i - 1]) / 2;
      end;
      {----------------------------            }
      for i := 2 to KK - 1 do
      begin
        rwnew := THETA(i, H[i], RG[i], TS[i], AEV[i], BCAM[i], L[i], TM[i], HM[i]);
        {if (abs(y[i]-rwnew)>0.00001)
        THEN spc[i]:=(rwnew-y[i])/(h[i]-g[i])
        ELSE}
        begin
          r13 := min((h[i] + g[i]) / 2, hm[i]);
          SPC[i] := DWC(i, r13, RG[i], TS[i], BCAM[i], L[i], AEV[i], TM[i], HM[i]);
        end;
      end;
      {------------------------------          }
      {  Coefficients for tridiagonal matrix algorithm      }
      for i := 2 to KK - 1 do
      begin
        if (KILPLT = 0) then
          A[i] := 0;
        p1 := 0;
        if (KILPLT = 1) then
          A[i] := MIN(A[i], MAX(W[i] - THETA(i, HDRY, RG[i], TS[i], AEV[i], BCAM[i], L[i],
            TM[i], HM[i]), p1) / DELT);
        R32 := B[i];
        R34 := B[i - 1];
        R33 := 2 * DLZ * DLZ * SPC[i] / DELT + R32 + R34;
        R35 := R32 * G[i + 1] + R34 * G[i - 1] + (R33 - 2 * R32 - 2 * R34) * G[i];
        R35 := R35 + 2 * DLZ * (B[i - 1] - B[i]) - A[i] * 2 * DLZ * DLZ;
        {-----         }
        {   Forward solution        }
        F1[i] := R32 / (R33 - R34 * F1[i - 1]);
        G1[i] := (R35 + R34 * G1[i - 1]) / (R33 - R34 * F1[i - 1]);
      end;
      {--------------------------------        }
      {  Set bottom boundary conditions        }
      {  (Lysimeter is either water table, or zero flux if no drainage)}
      {   a) Water table          }
      if (R9 = 1) or (R9 = 5) then
      begin
        H[kk] := BBPOT;
        g[kk] := bbpot;
        W[kk] := THETA(kk, H[kk], RG[kk], TS[kk], AEV[kk], BCAM[kk], L[kk], TM[kk], HM[kk]);
      end;
      {----          }
      {  b) Zero flux             }
      if (R9 = 3) then
      begin
        H[kk] := (2 * DLZ - G[kk] + G[kk - 1] + G1[kk - 1]) / (1 - F1[kk - 1]);
        W[kk] := THETA(kk, H[kk], RG[kk], TS[kk], AEV[kk], BCAM[kk], L[kk], TM[kk], HM[kk]);
      end;
      {----          }
      {  c) Unit gradient         }
      if (R9 = 2) then
      begin
        H[kk] := (G1[kk - 1] - G[kk] + G[kk - 1]) / (1 - F1[kk - 1]);
        W[kk] := THETA(kk, H[kk], RG[kk], TS[kk], AEV[kk], BCAM[kk], L[kk], TM[kk], HM[kk]);
      end;
      {---------------------      }
      {  Tridiagonal matrix algorithm - reverse sweep       }
      DIFF := 0;
      for i := KK - 1 downto 1 do
      begin
        {dh:=0; }dh := h[i + 1];
        H[i] := F1[i] * dh + G1[i];
        if (i > 1) and (i < kk) then
          DIFF := DIFF + ABS(H[i] - HO[i]);
        HO[i] := H[i];
      end;
      {----------------------     }
      goto4300 := (DIFF < 2 * DELT);
    end; {IF (DIFF<2*DELT) GO TO 4300;}
  end; {3500 CONTINUE}
  {-----------------------    }
  {  Calculate theta, flux and flux density             }
  {former ftn label 4300}
  for i := 1 to KK - 1 do
  begin
    W[i] := THETA(i, H[i], RG[i], TS[i], AEV[i], BCAM[i], L[i], TM[i], HM[i]);
    FLXDEN[i] := (B[i]) * (1 - ((H[i + 1] + G[i + 1] - H[i] - G[i])) / (2 * DLZ));
    FLUX[i] := FLXDEN[i] * DELT;
    {     INCREMENT CUMULATIVE TRANSPIRATION              }
    TRANA := TRANA + A[i] * DLZ * DELT;
    {------------------------                }
    {  Record maximum absolute flux for calc of next DELT              }
    R37 := ABS(FLXDEN[i]);
    FMAX := MAX(FMAX, R37);
    {  New air-filled pore fraction, minimized to 0.01}
    TG[i] := max(0.01,TS[i] - W[i]);
  end;
  {------------------------                }
  W[1] := THETA(1, H[1], RG[1], TS[1], AEV[1], BCAM[1], L[1], TM[1], HM[1]);
  {-------------------------               }
  {  Maximum evaporation rate for new theta values      }
  {   (used for calculating max evaporation in next time step)       }
  B[2] := HC(2, H[2], RG[2], KS[2], AEV[2], BCAM[2], TS[2], L[2], RP[2], TM[2], HM[2], TEM[2]);
  { Arguments TM,HM added by PF2006}
  B[1] := B[2];
  WFDD := -B[1] * (-3000 * 102.2 - H[2]) / DLZ;
  if (wfdd <= 0.0001) then
    wfdd := 0;
  {--------------------------              }
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Watflo');
end;

procedure SOLC;
{calculate transport of solute (incl. dispersed) species}
{--------------------------------------------------------------------}
var
  TW, A1, B1, C1, D1, CB, DB: rLL;
  AB, kd, BB, DIFFSE, scfrac: array[1..mm, 1..ll] of real;
  kh: rMM;
  F1, G1: array[1..LL] of double;
  doublese: double;
  SEG, I, M, J: integer;
  FLXDN1, W1, DIFFUS, BETA1, BETA2, BETA3, BETA4: real;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering Solc');
  {----------------------------     }
  {----------------------------     }
  FLXDN1 := 1;
  {----------------------------}
  {  Calculate top and bottom water contents              }
  if (FLXDEN[1] < 0) then
  begin
    FLXDN1 := FLXDEN[1];
    FLXDEN[1] := 0;
  end
  else
  begin
    W1 := W[1];
    FLXDN1 := FLXDEN[1];
    W[1] := Y[1] - FLXDEN[1] * DELT / DLZ;
  end;
  if (R9 = 3) then
    FLXDEN[kk - 1] := 0;
  {-----------------------        }
  {  Mean theta; diffusion coefficient; pore water velocity between   }
  {   nodes; pore water velocity across nodes             }
  for I := 2 to KK - 1 do
  begin
    TW[i] := (Y[i + 1] + W[i + 1] + Y[i] + W[i]) / 4;
    { Tortuosity and water content adjustments done by Millington and Quirk}
    {   equation}
    {      DIFG[i]:=GDIF*((TS[i]-(Y[i]+W[i])/2)**3.33)/(TS[i]**2.0)+BAROM           }

    DIFFUS := DIFO * power(TW[i], 3.33) / power(TS[i], 2.0) +
      +TW[i] * lambda[i] * ABS(FLXDEN[i]) / TW[i];
    {  Old Bresler tortuosity adjustment, and old numerical disp. correction}
    {      DIFFUS:=DIFO*DIFA*EXP(DIFB*TW[i])+}
    {     #AMAX1(0,TW[i]*(-0.84+1.264*lambda[i]-0.6458*dlz))}
    {     #*ABS(FLXDEN[i])/TW[i]     }

    for M := 1 to NCHEM1 do
    begin
      DIFFSE[m, i] := DIFFUS;
      {      IF (I=1) and (FLXDEN[1]=0) DIFFSE[m,i]:=0.  }
      {     the above line replaced because i<>1 in all cases inside this loop}
      {     and DIFFSE(M,1) is needed below. Therefore replaced by (PF2006)}
      DIFFSE[M, 1] := 0;
    end;
  end;
  {-----------------------}
  for I := 2 to KK - 1 do
  begin
    DB[i] := FLXDEN[i] / (4. * DLZ);
    CB[i] := FLXDEN[i - 1] / (4. * DLZ);
    for M := 1 to NCHEM1 do
    begin
      {  For Freundlich isotherm assume linear for duration of SOLP}
      AB[m, i] := (DIFFSE[M, I - 1]) / (2 * DLZ * DLZ);
      BB[m, i] := (DIFFSE[m, i]) / (2 * DLZ * DLZ);
    end;
  end;
  {------------------------       }
  for M := 1 to nchem1 do
  begin
    AB[M, 2] := 0;
    BB[m, kk - 1] := 0;
    DIFFSE[M, 1] := 0;
    DIFFSE[m, kk - 1] := 0;
  end;
  {------------------------       }
  for M := 1 to NCHEM1 do
  begin
    kh[m] := 0;
    for I := 2 to KK - 1 do
    begin
      scfrac[m, i] := 1;
      kd[m, i] := 0;
      {csink[m,i]:=0; csink is now set in sinkc for the current timestep
                      it contains chemical losses via the transpiration stream}
      {--------------------------     }
      {  Coefficients for tridiagonal matrix     }
      if (i = 2) then
      begin
        if (FLXDEN[i - 1] >= 0) then
        begin
          BETA1 := 2;
          BETA4 := 0;
        end
        else if (FLXDEN[i - 1] > 0) then
        begin
          BETA1 := 0;
          BETA4 := 2;
        end;
        A1[i] := -AB[m, i] - BETA1 * CB[i];
        B1[i] := W[i] / DELT + AB[m, i] + BB[m, i] + DB[i] - BETA4 * CB[i] +
          (RHO[i] * scfrac[m, i] * KD[m, i] + (TS[i] - W[i]) * KH[m]) / DELT;
        C1[i] := -BB[m, i] + DB[i];
        D1[i] := SS[M, I - 1] * (AB[m, i] + BETA1 * CB[i]) + SS[m, i] *
          (Y[i] / DELT + (RHO[i] * scfrac[m, i] * KD[m, i] + (TS[i] - Y[i]) * KH[m]) /
          DELT - AB[m, i] - DB[i] + BETA4 * CB[i] - BB[m, i]) + CSINK[m, i] + SS[M, I + 1] *
          (BB[m, i] - DB[i]);
      end
      else if (i > 2) and (i < kk - 1) then
      begin
        A1[i] := -AB[m, i] - CB[i];
        B1[i] := W[i] / DELT + AB[m, i] + BB[m, i] + DB[i] - CB[i] +
          (RHO[i] * scfrac[m, i] * KD[m, i] + (TS[i] - W[i]) * KH[m]) / DELT;
        C1[i] := -BB[m, i] + DB[i];
        D1[i] := SS[M, I - 1] * (AB[m, i] + CB[i]) + SS[m, i] *
          (Y[i] / DELT + (RHO[i] * scfrac[m, i] * KD[m, i] + (TS[i] - Y[i]) * KH[m]) /
          DELT - AB[m, i] - DB[i] + CB[i] - BB[m, i]) + CSINK[m, i] + SS[M, I + 1] *
          (BB[m, i] - DB[i]);
      end
      else if (i = kk - 1) then
      begin
        if (FLXDEN[i] > 0) then
        begin
          BETA2 := 2;
          BETA3 := 0;
        end
        else if (flxden[i] < 0) then
        begin
          BETA2 := 0;
          BETA3 := 2;
        end
        else if (flxden[i] = 0) then
        begin
          BETA2 := 0;
          BETA3 := 0;
        end;
        A1[i] := -AB[m, i] - CB[i];
        B1[i] := W[i] / DELT + AB[m, i] + BB[m, i] + BETA2 * DB[i] - CB[i] +
          (RHO[i] * scfrac[m, i] * KD[m, i] + (TS[i] - W[i]) * KH[m]) / DELT;
        C1[i] := -BB[m, i] + BETA3 * DB[i];
        D1[i] := SS[M, I - 1] * (AB[m, i] + CB[i]) + SS[m, i] *
          (Y[i] / DELT + (RHO[i] * scfrac[m, i] * KD[m, i] + (TS[i] - Y[i]) * KH[m]) /
          DELT - AB[m, i] - BETA2 * DB[i] + CB[i] - BB[m, i]) + CSINK[m, i] + SS[M, I + 1] *
          (BB[m, i] - BETA3 * DB[i]);
      end;
    end;
    {    go to 8888; useless statements commented}
    {      DO 30 I:=2,KK-1               }
    {  Set beta values to 0 or 1 according to direction of water flow   }
{    IF (FLXDEN[i]>0) THEN
    BETA2:=1;
    BETA3:=0;
    ELSE;
    BETA2:=0;
    BETA3:=1;
    ENDIF;
    IF (FLXDEN[i-1]>0) THEN
    BETA1:=1;
    BETA4:=0;
    ELSE;
    BETA1:=0;
    BETA4:=1;
    ENDIF;}
    {--------------------------     }
    {  Coefficients for tridiagonal matrix      }
{    A1[i]:=-AB[m,i]-BETA1*CB[i];
    B1[i]:=W[i]/DELT+AB[m,i]+BB[m,i]+BETA2*DB[i]-BETA4*CB[i];
       #+(RHO[i]*scfrac[m,i]*KD[m,i]+(TS[i]-W[i])*KH[m])/DELT;
    C1[i]:=-BB[m,i]+BETA3*DB[i];
    D1[i]:=SS(M,I-1)*(AB[m,i]+BETA1*CB[i]);
       #+SS[m,i]*(Y[i]/DELT+(RHO[i]*scfrac[m,i]*KD[m,i]+(TS[i];
       #-Y[i])*KH[m])/DELT-AB[m,i]-BETA2*DB[i]+BETA4*CB[i]-BB[m,i]);
       #+SINK[m,i]+SS(M,I+1)*(BB[m,i]-BETA3*DB[i]);}
    {   30 CONTINUE      }
    { 8888  continue; end of useless statements}
    {--------------------------     }
    {  Solving tridiagonal matrix               }
    {  i) Calc. f & g coefficients from nodes 2 to K        }
    F1[2] := C1[2] / B1[2];
    G1[2] := (D1[2] - A1[2] * SS[M, 1]) / B1[2];
    for J := 3 to KK - 1 do
    begin
      F1[j] := C1[j] / (B1[j] - F1[j - 1] * A1[j]);
      G1[j] := (D1[j] - A1[j] * G1[j - 1]) / (B1[j] - A1[j] * F1[j - 1]);
      if (ABS(F1[j]) <= 1e-32) then
        F1[j] := 0;
      if (ABS(G1[j]) <= 1e-32) then
        G1[j] := 0;
    end;
    {----------         }
    {  Set bottom boundary condition            }
    { i)Water table of constant concentration   }
    if (R9 = 1) then
      SE[m, kk] := SS[m, kk];
    { ii) Unit hydraulic gradient               }
    if (R9 = 2) then
      SE[m, kk] := 0;
    { iii) Zero flux (depends on whether salt is added or lost)         }
    if (R9 = 3) then
      SE[m, kk] := 0;
    {----               }
    {   New solute concentrations : SE[m,i] values           }
    J := KK;
    doublese := se[m, kk];
    while J > 2 do
    begin
      J := J - 1;
      {      doublese:=(dmax1(0.d0,G1[j]-F1[j]*doublese))}
      {      IF (doublese<1.d-60)doubleSE:=0.d0}
      {      se[m,j]:=sngl(doublese)}
      SE[m, j] := max(0, G1[j] - F1[j] * (SE[M, J + 1]));
      if (SE[m, j] < 1E-16) then
        SE[m, j] := 1E-16;{was: SE[m,j]:=0; }
    end;
  end;
  {-------------------------      }
  {  Adjust boundary values back to original water contents and fluxes            }
  if (FLXDN1 <= 0) then
    FLXDEN[1] := FLXDN1;
  {------------------------       }
  {  Adsorbed NH4 and NO3 (if KD(2,i) gt 0).            }
  for M := 1 to nchem1 do
  begin
    for I := 2 to KK - 1 do
      {  Concentrations in gas phase              }
    begin
      {SE to pCO2 conversion; unused} if m = 0 then
        pCO2[i] := SE[m, i] / DataInit.kvalue[1];
    end;
    {  Calculate cumulative drainage as flux * conc. from segment K    }
    if (FLXDEN[kk - 1] >= 0) then
      LSE[m] := FLXDEN[kk - 1] * DELT * (SS[m, kk - 1] + SE[m, kk - 1]) / 2;
    if (FLXDEN[kk - 1] < 0) then
      LSE[m] := FLXDEN[kk - 1] * DELT * (SS[m, kk] + SE[m, kk]) / 2;
  end;
  {-----------------------        }
  {  Solute flux across summary segment boundaries}

  for SEG := 1 to 4 do
  begin
    I := ND[seg];
    for M := 1 to NCHEM1 do
    begin
      if (I < KK - 1) then
        FSE[m, seg] := FSE[m, seg] + (DIFFSE[m, i] * (SE[m, i] + SS[m, i] - SE[M, I + 1] - SS[M, I + 1]) / DLZ +
          FLXDEN[i] * (SS[m, i] + SE[m, i] + SS[M, I + 1] + SE[M, I + 1])) * DELT / 4;
      if (I = KK - 1) then
      begin
        if (FLXDEN[kk - 1] >= 0) then
          FSE[m, seg] := FSE[m, seg] + FLXDEN[kk - 1] * DELT * (SS[m, kk - 1] + SE[m, kk - 1]) / 2;
        if (FLXDEN[kk - 1] < 0) then
          FSE[m, seg] := FSE[m, seg] + FLXDEN[kk - 1] * DELT * (SS[m, kk] + SE[m, kk]) / 2;
      end;
    end;
  end;
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Solc');
end;

procedure OUTC;
{write Cumulative and mass balance to file *.out}
{--------------------------------------------------------------------}
var
  HKPA: rLL;
  CALCITE1, GYP1, DEPTH, HROOTK, TRL1: real;
  i, m: integer;
  strM, strD, strY: string;
begin
  ;
  {if (TRACE = 1) then}
    singletracewriteln(' Entering Outc');
  {********************************************************************}
  if (PRNTAB >= 1) then
  begin
    str(MONTH + 100: 3, strM);
    strM := copy(strM, 2, 2);
    str(DAYM + 100: 3, strD);
    strD := copy(strD, 2, 2);
    str(YEAR + 100: 3, strY);
    strY := copy(strY, 2, 2);
    for i := 1 to 2 do
      system.writeln(f11);
    system.writeln(f11, '***************************************************',
      '***************************************************');
    system.writeln(f11);
    system.writeln(f11);
    system.writeln(f11, '  TIME', TIME: 11: 4, ' DAYS     CUMULATIVE TOTALS AND MASS',
      ' BALANCE (cycle', CYCNO, ')');
    if (IDATE = 1) then
      system.writeln(f11, '  DATE  ', strM, '/', strD, '/', strY);
    if (IDATE = 2) then
      system.writeln(f11, '  DATE  ', strD, '/', strM, '/', strY);
    if (IDATE = 3) then
      system.writeln(f11, '  DATE  ', strY, '/', strM, '/', strD);
    system.writeln(f11, '                                WATER           Ca          Mg',
      '          Na           K          Cl         SO4        HCO3',
      '         CO3      BALION          Al         Org        HOrg',
      '       H2Org         DOC');
    system.writeln(f11, '                                  mm                ',
      '-------------mmol/sq.m-------------------------------mmol/sq.m-',
      '------------------------------mmol/sq.m------------------------');
    system.writeLN(f11);
    system.Write(f11, ' Initially in profile        :', AWATER: 8: 1, ' ');
    for m := 1 to 9 do
      system.Write(f11, APDSE[m]: 12: 1, ' ');
    for m := 12 to 16 do
      system.Write(f11, APDSE[m]: 12: 1, ' ');
    system.writeln(f11);
    system.Write(f11, ' Currently in profile        :', PWATER: 8: 1, ' ');
    for m := 1 to 9 do
      system.Write(f11, PDSE[m]: 12: 1, ' ');
    for m := 12 to 16 do
      system.Write(f11, PDSE[m]: 12: 1, ' ');
    system.writeln(f11);
    system.writeln(f11, ' Could not infiltrate        :', EXCESS: 8: 1);
    system.Write(f11, ' Change                      :', DELWAT: 8: 1, ' ');
    for m := 1 to 9 do
      system.Write(f11, DELSE[m]: 12: 1, ' ');
    for m := 12 to 16 do
      system.Write(f11, DELSE[m]: 12: 1, ' ');
    system.writeln(f11);
    system.Write(f11, ' Added: i) Rain              :', CRAIN: 8: 1, ' ');
    for m := 1 to 9 do
      system.Write(f11, CRSE[m]: 12: 1, ' ');
    for m := 12 to 16 do
      system.Write(f11, CRSE[m]: 12: 1, ' ');
    system.writeln(f11);
    system.Write(f11, '       ii) Irrigation        :', CIRRIG: 8: 1, ' ');
    for m := 1 to 9 do
      system.Write(f11, CISE[m]: 12: 1, ' ');
    for m := 12 to 16 do
      system.Write(f11, CISE[m]: 12: 1, ' ');
    system.writeln(f11);
    system.Write(f11, '      iii) As amendm/ecto-min.', '        ', ' ');
    for m := 1 to 9 do
      system.Write(f11, (CFER[m] + CMINER[m]): 12: 1, ' ');
    for m := 12 to 16 do
      system.Write(f11, (CFER[m] + CMINER[m]): 12: 1, ' ');
    system.writeln(f11);
    system.writeln(f11, '       iv) Frm Calc/Gyp/Gibbs:', '        ', ' ', SOLS1: 12: 1, ' ',
      '                                                ', '    ',
      SOLS6: 12: 1, '              ', SOLS8: 12: 1, '              ', SOLS12: 12: 1);
    system.writeln(f11, '        v) From CO2          :', '                    ',
      '                                              ',
      '                     ', max(0, CDCO2): 12: 1);
    system.Write(f11, ' Lost:  i) In drainage       :', CLW: 8: 1, ' ');
    for m := 1 to 9 do
      system.Write(f11, CLSE[m]: 12: 1, ' ');
    for m := 12 to 16 do
      system.Write(f11, CLSE[m]: 12: 1, ' ');
    system.writeln(f11);
    system.writeln(f11, '       iv) Evaporation/to CO2:', CEVAPA: 8: 1, '            ',
      '                                              ',
      '                     ', min(0, CDCO2): 12: 1);
    system.writeln(f11, '      iii) To Calc/Gyps/Gibbs:', '        ', PPTS1: 12: 1,
      '                                                      ', PPTS6: 12: 1,
      '              ', PPTS8: 12: 1, '              ', PPTS12: 12: 1);
    system.Write(f11, '       ii) Tran/plant uptake :', CTRANA: 8: 1, ' ');
    for m := 1 to 9 do
      system.Write(f11, CUSE[m]: 12: 1, ' ');
    for m := 12 to 16 do
      system.Write(f11, CUSE[m]: 12: 1, ' ');
    system.writeln(f11);
    system.Write(f11, ' Mass error                  :', WATBAL: 8: 1, ' ');
    for m := 1 to 7 do
      system.Write(f11, BALSE[m]: 12: 1, ' ');
    system.Write(f11, '             ', balse[9]: 12: 1, ' ');
    for m := 12 to 16 do
      system.Write(f11, balse[m]: 12: 1, ' ');
    system.writeln(f11);
  end; {IF}
  {-------------------------  }
  { PRINT DEPTH DATA IN PROFILE   }
  if (PRNTAB >= 2) then
  begin
    system.writeln(f11);
    system.writeln(f11);
    system.writeln(f11);
    system.writeln(f11, '   Node  Theta   Potnl   Flux     ET    Cl    SO4   ',
      'HCO3    CO3 Balion   Total Calcite Gypsum   SAR    ESP     EC',
      '     pH');
    if (units = 1) then
      system.writeln(f11, '    mm', '            ', 'kPa     mm      mm',
        '   ', '--------------',
        '---mmol/l----------------   mass fraction', '               ', 'mS/m');
    if (units = 2) then
      system.writeln(f11, '    mm', '            ', 'kPa     mm      mm',
        '   ', '--------------',
        '---mmol/m2---------------   mass fraction', '               ', 'mS/m');
    system.writeln(f11);
    for I := 2 to KK - 1 do
    begin
      {   FLUX BETWEEN NODES I-1 AND I PRINTED NEXT TO DEPTH I.   }
      {   N.B THROUGHOUT MAIN, WATFLO AND SOLUTE , I REFERS TO I+.5 FOR   }
      {   VALUES CALCULATED AT MIDPOINTS BETWEEN NODES}
      HKPA[i] := H[i] / 102.2;
      CALCITE1 := CALCITE[i] * 100 / (1000000 * RHO[i]);
      GYP1 := GYP[i] * 172 / (1000000 * RHO[i]);
      {no gibbsite conversion needed as it is not printed here}
      DEPTH := -(i - 1.5) * DLZ;
      system.Write(f11, DEPTH: 8: 0, ' ', W[i]: 8: 6, ' ', HKPA[i]: 9: 3, ' ',
        WFW[i - 1]: 7: 2, ' ', WTRAN[i]: 7: 2, ' ');
      for m := 5 to 9 do
        system.Write(f11, SE[M, I]: 15: 10, ' ');
      system.writeln(f11, TOTANS[i]: 15: 10, ' ', CALCITE1: 15: 10, ' ', GYP1: 15: 10, ' ',
        SAR[i]: 7: 1, ' ', ESP[i]: 7: 1, ' ', EC[i]: 7: 1, ' ', PH[i]: 7: 2);
    end;
    system.writeln(f11, '        Drainage flux :', WFW[KK - 1]: 6: 2);
    system.writeln(f11);
    system.writeln(f11, '  (Water fluxes are cumulative since the previous',
      ' printout and, except for');
    system.writeln(f11, ' the drainage flux, refer to ',
      'the upper boundary of each depth segment.');
  end;
  {----------------------------   }
  {  PRINT PLANT GROWTH AND NUTRIENT ABSORPTION DATA  }
  if (PRNTAB >= 2) then
  begin
    for i := 1 to 4 do
      system.writeln(f11);
    system.writeln(f11, '          CROP AND ROOT DATA, EXCHANGEABLE',
      ' AND DISSOLVED CATIONS');
    for i := 1 to 2 do
      system.writeln(f11);
    HROOTK := HROOT / 102.2;
    TRL1 := TRL;
    system.writeln(f11, '  Time:', TIME: 11: 3, ' Days     Crop cover:',
      CRPCOV[max(1, CROPNO)]: 6: 3,
      '   Root', '  Potential:', HROOTK: 10: 4, 'kPa');
    system.writeln(f11);
    system.writeln(f11, '   Node  Roots         Ca            Mg',
      '            Na            K             H             Al            Dissolved');
    system.writeln(f11, '  Depth', '            ', 'Exch   Soln   ', 'Exch   Soln   ',
      'Exch   Soln   ', 'Exch   Soln   ', 'Exch   Soln   ',
      'Exch   Soln   ', '  cations');
    system.writeln(f11, '    mm  fraction  ', 'me/kg mmol/l  ', 'me/kg mmol/l  ',
      'me/kg mmol/l  ', 'me/kg mmol/l  ', 'me/kg mmol/l  ',
      'me/kg mmol/l  ', '     me/l   ');
    system.writeln(f11);
    for I := 2 to KK - 1 do
    begin
      DEPTH := -(i - 1.5) * DLZ;
      system.Write(f11, DEPTH: 7: 0, RDF[i]: 9: 3, ' ');
      for m := 1 to 4 do
        system.Write(f11, XSE[m, i]: 15: 10, ' ', SE[m, i]: 15: 10, ' ');
      system.Write(f11, XSE[10, i]: 15: 10, ' ', SE[10, i]: 15: 10, ' ');  {H}
      system.Write(f11, XSE[12, i]: 15: 10, ' ', SE[12, i]: 15: 10, ' ');  {Al}
      system.writeln(f11, TOTCAT[i]: 15: 10);
    end;
  end;
  {-------------------------- }
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Outc');
end;

procedure READCNEW;
{Obtain soil data for the newly started year, copying these from the datarecord DataYearIn filled at start of the year
(with variable status at end of previous year) and in the order of the standard soil input file.}
var
  i, j, m, PRNTOP2, CUMT1, NDUM, n1: integer;
  txt, txt2, txt3, txt4: string;
  frac1, RHO1, parden1, AEV1, BCAM1, CMAT1, HMAT1, RP1, ALAMB1, DUM,
  R3A, R9A, R12, R15, R16, RA, RB, RC, RD, RE, pHw, pCO2: real;
  CHARGE: rLL;
begin
  untGUI.frmMAIN.MVAR(GlobalTime*10000+Datayearin.nolayers*100,' start untProcesses.ReadCNew: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
  untGUI.frmMAIN.MVAR(GlobalTime*10000+Datayearin.nolayers*100,' start untProcesses.ReadCNew: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);

  {if (TRACE = 1) then}
    singletracewriteln(' Entering readCNew');
  closefile(f10); {f10 is not read but was opened thus has to be closed}
  n1prt := 0;
  {TRACE:=0;}
  idate := untGUI.DataYearIn.idate;
  day1 := untGUI.DataYearIn.day1;
  CUMT1 := untGUI.DataYearIn.CUMT1;
  {Set start date and final day number}
  if idate = 1 then
  begin
    ayear := day1 mod 100;
    amonth := trunc(day1 / 10000);
    aday := trunc(day1 / 100) mod 100;
  end
  else if idate = 2 then
  begin
    ayear := day1 mod 100;
    aday := trunc(day1 / 10000);
    amonth := trunc(day1 / 100) mod 100;
  end;
  daym := aday;
  month := amonth;
  year := ayear;
  np1 := 100000;
  if cumt1 >= 10100 then
    daycalc(amonth, aday, ayear, cumt1, np1, idate);
  cumt := cumt1;
  day1 := 1;
  {------------------------------}
  DELM := untGUI.DataYearIn.DELM;
  conq := 0.01;
  NYC := untGUI.DataYearIn.NYC;
  PDEPTH := untGUI.DataYearIn.PDEPTH;
  DLZ := untGUI.DataYearIn.DLZ;
  BBC := untGUI.DataYearIn.BBC;
  ABBPOT := untGUI.DataYearIn.ABBPOT;
 {The option below was tried out with BBC=1 and BBC=4
  but was deactivated again because of numerical problems
  or very slow performance}
 {if DataYearIn.WaterTableDepth<=DataYearIn.PDEPTH
 then
 begin
   BBC:=4; ABBPOT:=DataYearIn.WaterTableDepth;
 end; }
  {-----}
  if (bbc = 1) or (bbc = 5) or (bbc = 4) then
    abbpot := pdepth - abbpot + dlz / 2;
  KK := round(PDEPTH / DLZ) + 2;
  str(kk - 2, txt);
  wwriteln('SoilGen is using ' + txt + ' segments');
  IFLOW := untGUI.DataYearIn.IFLOW;
  THETA1 := untGUI.DataYearIn.THETA1;
  IFILE := untGUI.DataYearIn.IFILE;
  UNITS := untGUI.DataYearIn.UNITS;
  R54 := untGUI.DataYearIn.R54;
  PRNTOP2 := untGUI.DataYearIn.PRNTOP2;
  prntop := prntop2 + 1;
  PRINT := untGUI.DataYearIn.PRINT2;
  NPRINT := untGUI.DataYearIn.NPRINT;
  PRNTAB := untGUI.DataYearIn.PRNTAB;
  INITFLG := untGUI.DataYearIn.INITFLG;
  NSUM := untGUI.DataYearIn.NSUM;
  nd[1] := untGUI.DataYearIn.nd1;
  nd[2] := untGUI.DataYearIn.nd2;
  nd[3] := untGUI.DataYearIn.nd3;
  IFOURTH_LAYER := untGUI.DataYearIn.nd4;
  FWBTC := untGUI.DataYearIn.FWBTC;
  TRL := 0;
  dimok := True;
  if kk > LL then
  begin
    wwriteln('Depth node array too small - increase LL.');
    dimok := False;
  end;
  NWEEK := CUMT div 7 + 1;
  if NWEEK > IWEEK then
  begin
    wwriteln('Increase parameter IWEEK for longer time period.');
    dimok := False;
  end;
  if NPRINT > IPNT then
  begin
    wwriteln('Too many prints - increase parameter IPNT.');
    dimok := False;
  end;
  wwriteln('First data table read.');
  if dimok then
  begin
    for i := 1 to NPRINT do
    begin
      DATE := untGUI.DataYearIn.PRTIME[i];
      FRAC1 := untGUI.DataYearIn.FRAC1print[i];
      if (date >= 10100) then
        daycalc(amonth, aday, ayear, date, np1, idate);
      PRTIME[i] := date - 1 + frac1;
    end;
    wwriteln('Print times read for option 2.');
    for i := 2 to KK - 1 do
    begin
      NDUM := untGUI.DataYearIn.NDUM;
      CLAY[i] := untGUI.DataYearIn.CLAY[i - 1];
      SILT[i] := untGUI.DataYearIn.SILT[i - 1];
      SAND[i] := untGUI.DataYearIn.SAND[i - 1];
      OC[i] := untGUI.DataYearIn.OC[i - 1];
      STONES[i]:=untGUI.DataYearIn.STONES[i - 1];
      GRAVEL[i]:=untGUI.DataYearIn.GRAVEL[i - 1];
      {addition: mass passed as well between chem and shell units}
      CLAYm[i] := untGUI.DataYearIn.CLAYm[i - 1];
      SILTm[i] := untGUI.DataYearIn.SILTm[i - 1];
      SANDm[i] := untGUI.DataYearIn.SANDm[i - 1];
      fineSANDm[i] := untGUI.DataYearIn.fineSANDm[i - 1];
      OCm[i] := untGUI.DataYearIn.OCm[i - 1];
      STONESm[i]:=untGUI.DataYearIn.STONESm[i - 1];
      GRAVELm[i]:=untGUI.DataYearIn.GRAVELm[i - 1];
      MODRET[i] := untGUI.DataYearIn.MODRET[i - 1];
      AW[i] := untGUI.DataYearIn.AW[i - 1];
      AH[i] := untGUI.DataYearIn.AH[i - 1];
      RDF[i] := untGUI.DataYearIn.RDF[i - 1];
      ATEM[i] := untGUI.DataYearIn.ATEM[i - 1];
      {montmorillonite and total2:1 clay mineral percentages}
      {originally, inputs for the whole profile, from SoftConstants.txt.
      These are only for initialization here, updated elsewhere}
      MM21[i] := untGUI.DataYearIn.MM21[i - 1];
      SUM21[i] := untGUI.DataYearIn.SUM21[i - 1];
      stoneperc[i]:= untGUI.DataYearIn.stoneperc[i-1];
      SAR[i] := untGUI.DataYearIn.SAR[i - 1];
      pH[i] := untGUI.DataYearIn.pH[i - 1];
    end;
    ATEM[kk]:=ATEM[KK-1]; {this line will use the input temperatures for the heat reservoir as well}
    PRIOR := untGUI.DataYearIn.PRIOR;
    RHOS2 := untGUI.DataYearIn.RHOS2;
    RHOS1 := untGUI.DataYearIn.RHOS1;
    RHOS3 := untGUI.DataYearIn.RHOS3;
    wwriteln('Particle size, starting values, root data read.');
    RHO1 := untGUI.DataYearIn.RHO1;
    parden1 := untGUI.DataYearIn.parden1;
    AEV1 := untGUI.DataYearIn.AEV1;
    BCAM1 := untGUI.DataYearIn.BCAM1;
    CMAT1 := untGUI.DataYearIn.CMAT1;
    HMAT1 := untGUI.DataYearIn.HMAT1;
    RP1 := untGUI.DataYearIn.RP1;
    ALAMB1 := untGUI.DataYearIn.ALAMB1;
    ADDFC := untGUI.DataYearIn.ADDFC;
    ADDTH := untGUI.DataYearIn.ADDTH;
    for i := 2 to KK - 1 do
    begin
      AAEV[i] := untGUI.DataYearIn.AAEV[i - 1];
      ABCAM[i] := untGUI.DataYearIn.ABCAM[i - 1];
      RHO[i] := untGUI.DataYearIn.RHO[i - 1];
      CMATCH[i] := untGUI.DataYearIn.CMATCH[i - 1];
      HMATCH[i] := untGUI.DataYearIn.HMATCH[i - 1];
      RP[i] := untGUI.DataYearIn.RP[i - 1];
      LAMBDA[i] := untGUI.DataYearIn.LAMBDA[i - 1];
      fc[i] := untGUI.DataYearIn.fc[i - 1];
      d[i] := untGUI.DataYearIn.d[i - 1];
      if MODRET[i] <> 0 then
        estimate_KatH(i, clay[i], silt[i], rho[i], oc[i], dum, CMATCH[i], HMATCH[i]);
    end; 
    WaterTableDepth := untGUI.DataYearIn.WaterTableDepth;
    {if (TRACE = 1) then}
      singletracewriteln(' Just left nocompartments times estimate_KatH');
    for I := 2 to KK - 1 do
    begin
      {  Mean particle density}
      PARDEN[i] := 100 / (CLAY[I] / RHOS2 + (100 - CLAY[I] - 1.7 * OC[I]) / RHOS1 + 1.7 * OC[I] / RHOS3);
      {  Calculate porosity}
      TS[I] := max(((100-DataYearIn.stoneperc[i])/100)*(1 - RHO[I] / PARDEN[i]),0.05); {calculate using the bulkdensity but make it minimally 5%}
      {in case of input VGN-parameter reset to that value if smaller; see PredictVGN}
    end;
{--------------------------------------------------------------------
C  For a uniform profile replace the above values with defaults }
    for I := 1 to KK do
    begin
      if AEV1 < 0 then
        AAEV[I] := AEV1;
      if BCAM1 > 0 then
        ABCAM[I] := BCAM1;
      if CMAT1 > 0 then
        CMATCH[I] := CMAT1;
      if HMAT1 > 0 then
        HMATCH[I] := HMAT1;
      if RP1 > 0 then
        RP[I] := RP1;
      if RHO1 > 0 then
      begin
        RHO[I] := RHO1;
        TS[I] := max(1 - RHO[I] / PARDEN[i],0.05); {calculate using the bulkdensity but make it minimally 5%}
      end;
      if PARDEN1 > 0 then
        TS[I] := max(1 - RHO[I] / PARDEN[i],0.05); {calculate using the bulkdensity but make it minimally 5%}
      if ALAMB1 > 0 then
        LAMBDA[I] := ALAMB1;
      if ADDFC < -0.0001 then
        FC[I] := ADDFC;
      if ADDTH < -0.0001 then
        D[I] := ADDTH;
    end;
    {pf08: set read RHO equal to oldRHO}
    for i := 1 to kk do
      oldRHO[i] := RHO[i];
{--------------------------------------------------------------------
C  Call RETPRED (which predicts retentivity and conductivity if MODRET >0.}
    for I := 2 to KK - 1 do
    begin
      str(I - 1, txt);
      wwriteln(' Predicting K-theta-h for depth segment ' + txt);
      RETPRED(i, CLAY[I], SILT[I], RHO[I], OC[I], CMATCH[I], HMATCH[I]
        , AAEV[I], ABCAM[I], AKS[I], RP[I], TS[I], MODRET[I], PARDEN[i]);
      {calculate the pore volumes in macropore, mesopore and micropore domains}
      pmac[i]:=TS[i]-THETA(i,-200, RG[i], TS[i], AEV[i], BCAM[i], L[i], TM[i], HM[i]);
      pmic[i]:=THETA(i,-6000, RG[i], TS[i], AEV[i], BCAM[i], L[i], TM[i], HM[i]);
      pmes[i]:=TS[i]-pmac[i]-pmic[i];
      if (MODRET[I] <> 0) then
      begin
        str(AAEV[I]: 9: 3, txt);
        str(ABCAM[I]: 5: 2, txt2);
        str(TS[I]: 6: 3, txt3);
        str(AKS[i], txt4);
        wwriteln(' a = ' + txt + ' kPa   b = ' + txt2 + '  Theta sat = ' + txt3 +
          '   Ksat = ' + txt4 + ' mm/d');
      end;
    end;
    {  Set boundary nodes}
    CLAY[1] := CLAY[2];
    CLAY[KK] := CLAY[KK - 1];
    SILT[1] := SILT[2];
    SILT[KK] := SILT[KK - 1];
    RHO[1] := RHO[2];
    RHO[KK] := RHO[KK - 1];
    OC[1] := OC[2];
    OC[KK] := OC[KK - 1];
    CMATCH[1] := CMATCH[2];
    CMATCH[KK] := CMATCH[KK - 1];
    HMATCH[1] := HMATCH[2];
    HMATCH[KK] := HMATCH[KK - 1];
    AH[1] := AH[2];
    AH[KK] := AH[KK - 1];
    if (BBC = 1) or (BBC = 5) then
      AH[KK] := ABBPOT / 102.2;
    AW[1] := AW[2];
    AW[KK] := AW[KK - 1];
    AAEV[1] := AAEV[2];
    AAEV[KK] := AAEV[KK - 1];
    ABCAM[1] := ABCAM[2];
    ABCAM[KK] := ABCAM[KK - 1];
    AKS[1] := AKS[2];
    AKS[KK] := AKS[KK - 1];
    TS[1] := TS[2];
    TS[KK] := TS[KK - 1];
    RP[1] := RP[2];
    RP[KK] := RP[KK - 1];
    VGNal[1] := VGNal[2];
    VGNal[KK] := VGNal[KK - 1];
    VGNtr[1] := VGNtr[2];
    VGNtr[KK] := VGNtr[KK - 1];
    VGNts[1] := VGNts[2];
    VGNts[KK] := VGNts[KK - 1];
    VGNks[1] := VGNks[2];
    VGNks[KK] := VGNks[KK - 1];
    VGNen[1] := VGNen[2];
    VGNen[KK] := VGNen[KK - 1];
    VGNem[1] := VGNem[2];
    VGNem[KK] := VGNem[KK - 1];
    VGNel[1] := VGNel[2];
    VGNel[KK] := VGNel[KK - 1];
    {---------------------------- }
    {  Convert 'air-entry value' from mm H2O to kPa NB PF: from kPa to mm H2O!}
    for I := 1 to KK do
      AAEV[I] := AAEV[I] * 102.2;
    rocn2 := untGUI.DataYearIn.rocn2;
    roslope := untGUI.DataYearIn.roslope;
    wwriteln('PHYSICAL PROPERTIES COMPLETE.');
    KLPLT := untGUI.DataYearIn.KLPLT;
    NCROP := untGUI.DataYearIn.NCROP;
    if NCROP > NC then
    begin
      wwriteln('Increase parameter NC for more crops.');
      dimok := False;
    end;
    if dimok then
    begin
      HHIGH := 0;
      HDRY := untGUI.DataYearIn.HDRY;
      HLOW := untGUI.DataYearIn.HLOW;
      AK2 := untGUI.DataYearIn.AK2;
      RRES := untGUI.DataYearIn.RRES;
      for i := 1 to NCROP do
      begin
        igrowth[i] := untGUI.DataYearIn.igrowth[i];
        icut[i] := untGUI.DataYearIn.icut[i];
        rtype[i] := untGUI.DataYearIn.rtype[i];
        PDAY[i] := untGUI.DataYearIn.PDAY[i];
        ESTART[i] := untGUI.DataYearIn.ESTART[i];
        RDFDAY[i] := untGUI.DataYearIn.RDFDAY[i];
        ESTOP[i] := untGUI.DataYearIn.ESTOP[i];
        DAYHAR[i] := untGUI.DataYearIn.DAYHAR[i];
        RDEP[i] := untGUI.DataYearIn.RDEP[i];
        AK1[i] := untGUI.DataYearIn.AK1[i];
        PANFAC[i] := untGUI.DataYearIn.PANFAC[i];
        PD[I] := 1;
        if PDAY[I] >= 10100 then
          daycalc(amonth, aday, ayear, PDAY[I], np1, idate);
        if ESTART[I] >= 10100 then
          daycalc(amonth, aday, ayear, ESTART[I], np1, idate);
        if RDFDAY[I] >= 10100 then
          daycalc(amonth, aday, ayear, RDFDAY[I], np1, idate);
        if ESTOP[I] >= 10100 then
          daycalc(amonth, aday, ayear, ESTOP[I], np1, idate);
        if DAYHAR[I] >= 10100 then
          daycalc(amonth, aday, ayear, DAYHAR[I], np1, idate);
      end;
      WWRITELN('Crop data read.');

      for i := 2 to KK - 1 do
      begin
        CHARGE[i] := 0;
        SEA[1, I] := untGUI.DataYearIn.SEA[1, I - 1];
        SEA[2, I] := untGUI.DataYearIn.SEA[2, I - 1];
        SEA[3, I] := untGUI.DataYearIn.SEA[3, I - 1];
        SEA[4, I] := untGUI.DataYearIn.SEA[4, I - 1];
        SEA[5, I] := untGUI.DataYearIn.SEA[5, I - 1];
        SEA[6, I] := untGUI.DataYearIn.SEA[6, I - 1];
        SEA[7, I] := untGUI.DataYearIn.SEA[7, I - 1];
        {PF2023: waarom geen carbonaat SEA[8,I] hier?}
        SEA[12, I] := untGUI.DataYearIn.SEA[12, I - 1];
        SEA[13, I] := untGUI.DataYearIn.SEA[13, I - 1];
        SEA[14, I] := untGUI.DataYearIn.SEA[14, I - 1];
        SEA[15, I] := untGUI.DataYearIn.SEA[15, I - 1];
        DOCperc[i] := untGUI.DataYearIn.DOCperc[i - 1];

        {pf08}
        {convert percentage DOC <DOCPERC> to mmol/m2 <DOC>: }
   {since molecular mass DOC is unknown it is guessed to be 1000 g/mol,
   (e.g. Ludwig, U., T. Grlschek, P. Neitzel and W. Nestler. 1997.
    Ultrafiltration: a technique for determining the molecular-mass
    distribution of group parameters of organic pollutants in natural waters.
    Journal of Chromatography A, 763 (1997) 315-321: 51% of mass in molecules<1000 g/mol.
    thus 1000 g/mol = 1 mmol/g > MOLMASSDOC:=1 mmolC/gC:
   DOC:=DOCperc[i]*RHO[i]*MOLMASSDOC*DLZ/10, where MOLMASSDOC=mmolC/gC;}
        DOC[i] := DOCperc[i] * RHO[i] * 1 * DLZ / 10;
        SEA[8][I] := 0; {PF2023: waarom carbonaat SEA[8,I] hier op 0 gezet?}
        for m := 1 to 4 do
          CHARGE[I] := CHARGE[I] + SEA[M, I] * VAL[M];
        for m := 5 to 7 do     {PF2023: waarom geen m:=5 to 8 hier?}
          CHARGE[I] := CHARGE[I] - SEA[M, I] * VAL[M];
        CHARGE[I] := CHARGE[I] + SEA[12, I] * VAL[12]; {Al3+, pf08}
        for m := 13 to 15 do
          CHARGE[I] := CHARGE[I] - SEA[M, I] * VAL[M]; {PF2023: Changed to FeIII, FeII, H4SiO4}
      end;
      for i := 1 to 7 do
        BBSE[i] := 0; {untGUI.DataYearIn.CHARGE[i];}
      for i := 12 to 16 do
        BBSE[i] := 0; {untGUI.DataYearIn.CHARGE[i];}
      BBSE[8] := 0;
      CELLW := untGUI.DataYearIn.CELLW;
      wwriteln('Soluble cations and anions system.read.');
      for i := 2 to KK - 1 do
      begin
        CHARGE[i] := 0;
        AXSE[1][I] := untGUI.DataYearIn.AXSE[1][I - 1];
        AXSE[2][I] := untGUI.DataYearIn.AXSE[2][I - 1];
        AXSE[3][I] := untGUI.DataYearIn.AXSE[3][I - 1];
        AXSE[4][I] := untGUI.DataYearIn.AXSE[4][I - 1];
        ACEC[I] := untGUI.DataYearIn.ACEC[I - 1];
        AXSE[10][I] := untGUI.DataYearIn.AXSE[10][I - 1];
        AXSE[12][i] := untGUI.DataYearIn.AXSE[12][I - 1];
        AXSE_oc[1][I] := untGUI.DataYearIn.AXSE_oc[1][I - 1];
        AXSE_oc[2][I] := untGUI.DataYearIn.AXSE_oc[2][I - 1];
        AXSE_oc[3][I] := untGUI.DataYearIn.AXSE_oc[3][I - 1];
        AXSE_oc[4][I] := untGUI.DataYearIn.AXSE_oc[4][I - 1];
        ACEC_oc[I] := untGUI.DataYearIn.ACEC_oc[I - 1];
        AXSE_oc[10][I] := untGUI.DataYearIn.AXSE_oc[10][I - 1];
        AXSE_oc[12][i] := untGUI.DataYearIn.AXSE_oc[12][I - 1];
        AXSE_clay[1][I] := untGUI.DataYearIn.AXSE_clay[1][I - 1];
        AXSE_clay[2][I] := untGUI.DataYearIn.AXSE_clay[2][I - 1];
        AXSE_clay[3][I] := untGUI.DataYearIn.AXSE_clay[3][I - 1];
        AXSE_clay[4][I] := untGUI.DataYearIn.AXSE_clay[4][I - 1];
        ACEC_clay[I] := untGUI.DataYearIn.ACEC_clay[I - 1];
        AXSE_clay[10][I] := untGUI.DataYearIn.AXSE_clay[10][I - 1];
        AXSE_clay[12][i] := untGUI.DataYearIn.AXSE_clay[12][I - 1];
        for j := 1 to 12 do
        begin
          xse[j, i] := axse[j, i];
          xse_clay[j, i] := axse_clay[j, i];
          xse_oc[j, i] := axse_oc[j, i];
        end;
        CEC[i] := acec[i];
        CEC_oc[i] := acec_oc[i];
        CEC_clay[i] := acec_clay[i];
      end;
      wwriteln('Exchangeable cations system.read.');
      for i := 2 to KK - 1 do
      begin
  {SC1=Mg/Ca; SC2=Ca/Na; SC3=Ca/K;  SC4=Mg/K;  SC5= Mg/Na; SC6= K/Na;
   SC7=Ca/Al; SC8=Ca/H;  SC9=Mg/Al; SC10=K/Al; SC11= Na/Al; SC12=Mg/H;
   SC13=K/H;  SC14=Na/H; SC15=Al/H }
        SC1[I] := untGUI.DataYearIn.SC1[I - 1];
        SC2[I] := untGUI.DataYearIn.SC2[I - 1];
        SC3[I] := untGUI.DataYearIn.SC3[I - 1];
        APCO2[I] := untGUI.DataYearIn.APCO2[I - 1];
        CALCITEA[I] := untGUI.DataYearIn.CALCITEA[I - 1];
        GYPA[I] := untGUI.DataYearIn.GYPA[I - 1];
        SC7[I] := untGUI.DataYearIn.SC7[I - 1];
        SC8[I] := untGUI.DataYearIn.SC8[I - 1];
        SC4[i] := SC1[i] * SC3[i];
        SC5[i] := SC1[i] * SC2[i];
        SC6[i] := SC2[i] / SC3[i];
        SC9[i] := SC1[i] * SC7[i];
        SC10[i] := SC7[i] / SC3[i];
        SC11[i] := SC7[i] / SC2[i];
        SC12[i] := SC8[i] * SC1[i];
        SC13[i] := SC8[i] / SC3[i];
        SC14[i] := SC8[i] / SC2[i];
        SC15[i] := SC8[i] / SC7[i];
      end;
      wwriteln('Selectivity coefficients system.read.');
      NN1 := untGUI.DataYearIn.NN1;
      ITERCH := untGUI.DataYearIn.ITERCH;
      {checking exchange phase charge balance}
      for i := 2 to KK - 1 do
      begin
        if typechem = 1 then
          begin end
        else
          CHARGE[i] := AXSE[1, i] + AXSE[2, i] + AXSE[3, i] + AXSE[4, i] + AXSE[10, i] + AXSE[12, i] - ACEC[i];
        if (abs(CHARGE[i]) > 0.001) and (nn1 = 1) then
        begin
          str(i, txt);
          str(charge[i]: 5: 1, txt2);
          wwriteln('  WARNING! : Total exch. cations - CEC at segment ' + txt +
            ' is ' + txt2 + ' mmol+/kg.   CEC will be adjusted!');
          if typechem = 1 then
            begin end
          else
            ACEC[i] := AXSE[1, i] + AXSE[2, i] + AXSE[3, i] + AXSE[4, i] + AXSE[10, i] + AXSE[12, i];
        end;
      end;
      DIFO := untGUI.DataYearIn.DIFO;
      difa := 0;
      difb := 0;
      GDIF := 0;
      BAROM := 0;
      {thermal conductivity and heat capacity data}
      for i := 1 to KK do
      begin
        PART[2, i] := CLAY[i] * RHO[i] / (100 * RHOS2);
        PART[1, i] := (100 - CLAY[i] - 1.7 * OC[i]) * RHO[i] / (100 * RHOS1);
        PART[3, i] := 1.7 * OC[i] * RHO[i] / (100 * RHOS3);
        PART[4, i] := 0;
      end;
      ALAMO[1] := 8.8;
      ALAMO[2] := 2.9;
      ALAMO[3] := 0.25;
      ALAMO[4] := 0.6;
      ALAMO[5] := 0.025;
      {conversion from W/m/K to cal/m/s/C}
      for i := 1 to 5 do
        ALAMO[i] := ALAMO[i] / 418;
      {shape factors in thermal conductivity model}
      GEOMQ[1] := 0.125;
      GEOMOS[1] := 0.125;
      GEOMOM[1] := 0.5;
      GEOMQ[2] := 0.125;
      GEOMOS[2] := 0.125;
      GEOMOM[2] := 0.5;
      GEOMQ[3] := 0.75;
      GEOMOS[3] := 0.75;
      GEOMOM[3] := 0;
      {chemical applications}
      IFR := untGUI.DataYearIn.IFR;
      if ifr > nfr then
      begin
        wwriteln('Increase parameter NFR for more chemical applications.');
        dimok := False;
      end;
      if dimok then
      begin
        for i := 1 to IFR do
        begin
          DFER[i] := untGUI.DataYearIn.DFER[i];
          INCOR[i] := untGUI.DataYearIn.INCOR[i];
          FER[1, i] := untGUI.DataYearIn.FER[1, i];
          FER[2, i] := untGUI.DataYearIn.FER[2, i];
          FER[3, i] := untGUI.DataYearIn.FER[3, i];
          FER[4, i] := untGUI.DataYearIn.FER[4, i];
          FER[5, i] := untGUI.DataYearIn.FER[5, i];
          FER[6, i] := untGUI.DataYearIn.FER[6, i];
          FER[7, i] := untGUI.DataYearIn.FER[7, i];
          FER[8, i] := untGUI.DataYearIn.FER[8, i];
          FER[12, i] := untGUI.DataYearIn.FER[12, i];
          if DFER[i] >= 10100 then
            daycalc(amonth, aday, ayear, DFER[i], np1, idate);
        end;
        wwriteln('Chemical applications read.');
        Icult := untGUI.DataYearIn.Icult;
        if icult > ncult then
        begin
          wwriteln('Increase parameter NCULT for more cultivations.');
          dimok := False;
        end;
        if dimok then
        begin
          for i := 1 to Icult do
          begin
            Dcult[i] := untGUI.DataYearIn.Dcult[i];
            cultdep[i] := untGUI.DataYearIn.cultdep[i];
            if DCULT[i] >= 10100 then
              daycalc(amonth, aday, ayear, DCULT[i], np1, idate);
          end;
          wwriteln('Cultivation dates read.');
          NIRRIG := untGUI.DataYearIn.NIRRIG;
          if NIRRIG > IPPT then
          begin
            wwriteln('Increase parameter IPPT for more water applications.');
            dimok := False;
          end;
          if dimok then
          begin
            ch_irr_flag := untGUI.DataYearIn.ch_irr_flag;
            if (ch_irr_flag = 1) then
            begin
              system.Assign(f40, FILE6);
              system.reset(f40);
              bf40 := True;
              for i := 1 to 3 do
                system.readln(f40); {read(40,'(//)') }
              system.readln(f40, checkdep);
              system.readln(f40, critirrig);
              critirrig := -abs(critirrig * 102.2);
              for i := 1 to 5 do
                system.readln(f40); {read(40,'(////)') }
              system.readln(f40, replenish_dep);
              for i := 1 to 5 do
                system.readln(f40); {read(40,'(////)') }
              system.readln(f40, seirr[1], seirr[2], seirr[3], seirr[4], seirr[5],
                seirr[6], seirr[7], seirr[12]);
              wwriteln(' Automated irrigation criteria read.');
              seirr[8] := 0;
              ;
              seirr[9] := 0;
              seirr[10] := 0;
              seirr[11] := 0;
              {   Bring water composition into equilibrium with atmospheric CO2.}
              {   Returned pH value represents equilibrium with PCO2.  PCO2 assumed}
              {   0.0003 for rain and irrigation water.}
              pco2 := untGUI.pCO2atm;
              PHw := 7.0;
              R1 := 0;
              R2 := 0;
              R3 := 0;
              R3A := 1.0;
              R4 := 0;
              R5 := 0;
              R6 := 0;
              R7 := 0;
              R8 := 0;
              R9A := 0;
              R10 := 0;
              R11 := 0;
              R12 := 0;
              R13 := 0;
              R14 := 0;
              R15 := 0;
              R16 := 0;
              R17 := 0;
              R18 := 0;
              RA := 0;
              RB := 0;
              RC := 0;
              RD := 0;
              RE := 0;
              N1 := 0;
              R111 := 0;
              R113 := 0;
              R114 := 0;
              R115 := 0;
              R116 := 0;
              R112 := 0;
              R117 := 0;
              R118 := 0;
              R119 := 0;
              R120 := 0;
              R121 := 0;
              R122 := 0;
              R123 := 0;
              R124 := 0;
              R125 := 0;
              R126 := 0;
              R127 := 0;
              SEFeII:=0; SEFeIII:=0; {FeII and FeIII conc for call to Chem2}
              case typechem of
                1:begin end;
                2:
                  chem2(R1, R2, R111, PHw, R3, seirr[12], seirr[1], seirr[2],
                    seirr[3], seirr[4], seirr[5], seirr[6], seirr[7], seirr[8],
                    seirr[9], seirr[10], seirr[11], R3A, R4, R5, R6, R7,
                    R8, R9A, R117, R112, R10, R11, R12, R13, R14, R15, RA, RB,
                    R118, R119, R120, R121, R122, R123, R124, R125, R126, R127, RC, RD, pco2, N1,
                    SP1, SP2, R17, trace, R18, n1prt, ATEM[2]);
                3: begin end;
              end;
              str(phw: 4: 1, txt);
              str(seirr[9]: 5: 1, txt2);
              wwriteln(' pH: ' + txt + ' ' + txt2);
              wwriteln(' Automated irrigation criteria read.');
            end; { if   }
            {---------------------------------  }
            {  If steady state water flow is desired ensure no plants are present }
            {   and that specified water content will be used, rather than a potential}
            if (IFLOW = 3) then
            begin
              PRIOR := 1;
              KLPLT := 0;
              BBC := 2;
              for I := 1 to KK do
                if (abs(theta1 - 999) < 0.01) then
                  AW[I] := ts[i]
                else
                  AW[I] := theta1;
            end;
            { Ensure that water content does not exceed porosity }
            if (PRIOR = 1) then
              for I := 2 to KK - 1 do
              begin
                if (AW[I] > TS[I]) then
                begin
                  str(i, txt);
                  str(aw[i]: 5: 2, txt2);
                  str(ts[i]: 5: 2, txt3);
                  debugout('  Specified water content is greater than porosity');
                  debugout(' At segment ' + txt + ' theta is ' + txt2 + ', porosity ' + txt3);
                  debugout(' theta set equal to porosity.');
                  AW[i] := TS[i];
     {wwriteln('  ****** SoilGen terminated *******');
     closerun_errors;}
                end;
              end;
            {--------------------------------------------------------------------}
            {  To read irrigation and rain from separate files  }
            for i := 1 to IPPT do
              for m := 1 to mm do
                ARSE[m, i] := 0;
            {  above j and m loops added to fully initialize ARSE(m,j)  (PF2006)}
            if (ch_irr_flag = 1) then
              readirrc;
            if (ch_irr_flag = 0) then
            begin
              irrflg[nirrig + 1] := 1;
              {   above line added because of initialization problem in Mainc2.for (PF2006)}
              wwriteln(' Reading rainfall data.........');
              sumirrig := 0;
              for i := 1 to NIRRIG do
              begin
                IRRIGA[i] := untGUI.DataYearIn.IRRIGA[i];
                sumirrig := sumirrig + IRRIGA[i];
                IRATEA[i] := untGUI.DataYearIn.IRATEA[i];
                ARSE[1, i] := untGUI.DataYearIn.ARSE[1, i];
                ARSE[2, i] := untGUI.DataYearIn.ARSE[2, i];
                ARSE[3, i] := untGUI.DataYearIn.ARSE[3, i];
                ARSE[4, i] := untGUI.DataYearIn.ARSE[4, i];
                ARSE[5, i] := untGUI.DataYearIn.ARSE[5, i];
                ARSE[6, i] := untGUI.DataYearIn.ARSE[6, i];
                ARSE[7, i] := untGUI.DataYearIn.ARSE[7, i];
                ARSE[12, i] := untGUI.DataYearIn.ARSE[12, i];
                DATE := untGUI.DataYearIn.AIRRBG[i];
                FRAC1 := untGUI.DataYearIn.FRAC1[i];
                irrflg[i] := 1;
                if (DATE >= 10100) then
                  DAYCALC(AMONTH, ADAY, AYEAR, DATE, NP1, IDATE);
                if ((i - trunc(i / 1000) * 1000) < 0.00001) then
                begin
                  str(i, txt);
                  wwriteln(' Read ' + txt + ' events');
                end;
                AIRRBG[i] := DATE - 1 + FRAC1;
              end;
              {   Bring each water composition into equilibrium with atmospheric CO2. }
              {   Returned pH value represents equilibrium with PCO2.  PCO2 assumed   }
              {   0.0003 for rain and irrigation water}
            end;
            for i := 1 to nirrig do
            begin
              {  Concentrations of balancing ion (9), H (10) and OH (11) set to 0}
              {   Returned as values in equilibrium with PCO2, Ca, HCO3, CO3}
              pco2 := untGUI.pCO2atm;
              PHw := 7.0;
              ARSE[8, i] := 0;
              ARSE[9, i] := 0;
              ARSE[10, i] := 0;
              ARSE[11, i] := 0;
              str(i, txt);
              wwriteln(' Bringing water' + txt + ' into chemical equilibrium.');
              R1 := 0;
              R2 := 0;
              R3 := 0;
              R3A := 1.0;
              R4 := 0;
              R5 := 0;
              R6 := 0;
              R7 := 0;
              R8 := 0;
              R9A := 0;
              R10 := 0;
              R11 := 0;
              R12 := 0;
              R13 := 0;
              R14 := 0;
              R15 := 0;
              R16 := 0;
              R17 := 0;
              R18 := 0;
              RA := 0;
              RB := 0;
              RC := 0;
              RD := 0;
              RE := 0;
              N1 := 0;
              R111 := 0;
              R113 := 0;
              R114 := 0;
              R115 := 0;
              R116 := 0;
              R112 := 0;
              R117 := 0;
              R118 := 0;
              R119 := 0;
              R120 := 0;
              R121 := 0;
              R122 := 0;
              R123 := 0;
              R124 := 0;
              R125 := 0;
              R126 := 0;
              R127 := 0;
              SEFeII:=0; SEFeIII:=0; {FeII and FeIII conc for call to Chem2}
              case typechem of
                1:begin end;
                2:
                  chem2(R1, R2, R111, PHw, R3, ARSE[12, i], ARSE[1, i], ARSE[2, i],
                    ARSE[3, i], ARSE[4, i], ARSE[5, i], ARSE[6, i], ARSE[7, i], ARSE[8, i],
                    ARSE[9, i], ARSE[10, i], ARSE[11, i], R3A, R4, R5, R6, R7,
                    R8, R9A, R117, R112, R10, R11, R12, R13, R14, R15, RA, RB,
                    R118, R119, R120, R121, R122, R123, R124, R125, R126, R127, RC, RD, pco2, N1,
                    SP1, SP2, R17, trace, R18, n1prt, ATEM[2]);
                3:begin end;
              end;
              str(phw: 4: 1, txt);
              str(ARSE[11, i]: 8: 3, txt2);
              wwriteln(' pH: ' + txt + ' ' + txt2);
            end;
            {----------------------------}
            {  Extra fictitious irrigation to satisfy program between final}
            {   irrigation and last simulation day.  (Start past last day)}
            AIRRBG[NIRRIG + 1] := (CUMT + 10);
            IRRIGA[NIRRIG + 1] := 10;
            IRATEA[NIRRIG + 1] := 10;

            wwriteln(' Rain/irrig and dissolved chemicals read.');
            {********************************************************************}
            {         POTENTIAL ET AND WATER TABLE DEPTH}
            for i := 1 to NWEEK do
            begin
              PAN[i] := untGUI.DataYearIn.PAN[i];
              WTDEP[i] := untGUI.DataYearIn.WTDEP[i];
              AVETEM[i] := untGUI.DataYearIn.AVETEM[i];
              AMP[i] := untGUI.DataYearIn.AMP[i];
              {  ETIME is set to about the middle of the week to try to represent}
              {  weekly average temperatures (subject to change)  }
              ETIME[i] := i * 7 - 3;
            end;
            {--------------------------------------------------------------------}
            wwriteln(' ET and water table data read.');
            {--------------------------------------------------------------------}
          end; {if dimok}
        end; {if dimok}
      end; {if dimok}
    end; {if dimok}
  end; {if dimok}
  {ET and water table data read.}
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving ReadCNEW');
    untGUI.frmMAIN.MVAR(GlobalTime*10000+Datayearin.nolayers*100,' einde untProcesses.ReadCNew: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
    untGUI.frmMAIN.MVAR(GlobalTime*10000+Datayearin.nolayers*100,' einde untProcesses.ReadCNew: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);

end;

procedure READIRRC;
{read irrigation when given in separate files (not so in SoilGen; called from READCNEW)}
var
  arse_rain, arse_irrig: array[1..7] of real;
  i, j, jr, ji, irf, iif, m, no_irrigs, icrit, iDATE_rain, iDATE_irrig: integer;
  FRAC1_rain, rRRIGA_rain, rRATEA_rain, FRAC1_irrig, rRRIGA_irrig, rRATEA_irrig, rtime: real;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering Readirrc');
  np1 := 100000;
  wwriteln(' Reading rainfall and irrigation data.........');
  for i := 1 to 3 do
    system.readln(f40); {read(40,'(//)')}
  system.readln(f40, no_irrigs);
  for i := 1 to 4 do
    system.readln(f40); {read(40,'(///)') }
  j := 0;
  jr := 1;
  ji := 1;
  iif := 1;
  irf := 1;
  icrit := nirrig + no_irrigs + 2;

  while (jr <= nirrig) or (ji <= no_irrigs) do
  begin
    if (irf = 1) and (jr <= nirrig) then
    begin
      system.readln(f40, iDATE_rain, FRAC1_rain, rRRIGA_rain, rRATEA_rain,
        ARSE_rain[1], ARSE_rain[2], ARSE_rain[3], ARSE_rain[4], ARSE_rain[5],
        ARSE_rain[6], ARSE_rain[7]);
      if (iDATE_rain >= 10100) then
        DAYCALC(AMONTH, ADAY, AYEAR, iDATE_rain, NP1, IDATE);
      irf := 0;
    end;
    {********************************* }
    { Read irrigation event            }
    if (iif = 1) and (ji <= no_irrigs) then
    begin
      system.readln(f40, iDATE_irrig, FRAC1_irrig, rRRIGA_irrig, rRATEA_irrig,
        ARSE_irrig[1], ARSE_irrig[2], ARSE_irrig[3], ARSE_irrig[4],
        ARSE_irrig[5], ARSE_irrig[6], ARSE_irrig[7]);
      if (iDATE_irrig >= 10100) then
        DAYCALC(AMONTH, ADAY, AYEAR, iDATE_irrig, NP1, IDATE);
      iif := 0;
    end;
    {********************************************************************* }
    {  If rain date is past the current irrigation date, insert irrigation.}
    if (idate_rain > idate_irrig) then
    begin
      j := j + 1;
      AIRRBG[j] := (iDATE_irrig) - 1 + FRAC1_irrig;
      IRRIGA[j] := rrriga_irrig;
      IRATEA[j] := rratea_irrig;
      for m := 1 to 7 do
        arse[m, j] := arse_irrig[m];
      irrflg[j] := 0;
      iif := 1;
      ji := ji + 1;
    end;
    {  If rain date is less than current irrigation date, insert rain}
    if (idate_rain < idate_irrig) then
    begin
      j := j + 1;
      irrflg[j] := 1;
      AIRRBG[j] := (iDATE_rain) - 1 + FRAC1_rain;
      IRRIGA[j] := rrriga_rain;
      IRATEA[j] := rratea_rain;
      for m := 1 to 7 do
        arse[m, j] := arse_rain[m];
      irf := 1;
      jr := jr + 1;
    end;
    {  If rain date equals irrigation date, combine the two.}
    if (idate_rain = idate_irrig) then
    begin
      if (frac1_irrig <= frac1_rain) then
      begin
        j := j + 1;
        AIRRBG[j] := (iDATE_irrig) - 1. + FRAC1_irrig;
        IRRIGA[j] := rrriga_irrig;
        IRATEA[j] := rratea_irrig;
        rtime := irriga[j] / iratea[j];
        for m := 1 to 7 do
          arse[m, j] := arse_irrig[m];
        irrflg[j] := 0;
        iif := 1;
        ji := ji + 1;
        j := j + 1;
        AIRRBG[j] := (iDATE_rain) - 1 + frac1_rain + rtime + 0.1;
        IRRIGA[j] := rrriga_rain;
        IRATEA[j] := rratea_rain;
        for m := 1 to 7 do
          arse[m, j] := arse_rain[m];
        irrflg[j] := 1;
        irf := 1;
        jr := jr + 1;
      end
      else
      if (frac1_irrig > frac1_rain) then
      begin
        j := j + 1;
        AIRRBG[j] := (iDATE_rain) - 1. + frac1_rain;
        IRRIGA[j] := rrriga_rain;
        IRATEA[j] := rratea_rain;
        rtime := irriga[j] / iratea[j];
        for m := 1 to 7 do
          arse[m, j] := arse_rain[m];
        irrflg[j] := 1;
        irf := 1;
        jr := jr + 1;
        j := j + 1;
        AIRRBG[j] := (iDATE_irrig) - 1 + frac1_irrig + rtime + 0.1;
        IRRIGA[j] := rrriga_irrig;
        IRATEA[j] := rratea_irrig;
        for m := 1 to 7 do
          arse[m, j] := arse_irrig[m];
        irrflg[j] := 0;
        iif := 1;
        ji := ji + 1;
      end; {if}
    end; {if}

    if (jr > nirrig) and (irf = 1) then
    begin
      idate_rain := 9999999;
      irf := 0;
    end;
    if (ji > no_irrigs) and (iif = 1) then
    begin
      idate_irrig := 9999999;
      iif := 0;
    end;
  end; {while...do}
  nirrig := j;
  AIRRBG[NIRRIG + 1] := (CUMT + 10);
  IRRIGA[NIRRIG + 1] := 10;
  IRATEA[NIRRIG + 1] := 10;
  {--------------------------------------------------------------------}
  wwriteln(' Rain/irrig and dissolved chemicals read.');
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving ReadIrrC');
end;

procedure RETPRED(compnr: integer;
  var CL, SI, RHO, CARBON, CMATCH, HMATCH, A, B, AKS, RP, F, MODRET, PARDEN: real);
{determine the soil hydraulic parameters using the Hypres PTF(by calling predictVGN)
 towards Van Genuchten parameters, includes conversion to Campbell parameters}
{--------------------------------------------------------------------}
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering Retpred (using Van Genuchten functions)');
  if MODRET <> 0 then
  begin
    predictVGN(compnr, cl, si, carbon, rho, PARDEN);
    {VGN were transformed to Campbell params in predictVN as well}
    A := AAEV[compnr];
    B := ABCAM[compnr];
    AKS := VGNks[compnr];
  end;
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Retpred');
end;

procedure DAYCALC(var AMONTH, ADAY, AYEAR, DAYNO, CUMT, IDATE: integer);
{calculate Julian day number from calender notation (called from READCNEW and AUTOIRRG)}
{ CALCULATE DAY NUMBER FOR THE CURRENT CALENDAR DATE.}
{ DATA NEEDED ARE STARTING MONTH DAY AND YEAR CORRESPONDING TO DAY 1.}
var
  MON, MONL: array[1..12] of integer;
  DAYM, MONTH, YEAR, PMONTH, PDAY, PYEAR: integer;
  ready: boolean;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering DayCalc');
  ready := False;
  MON[1] := 31;
  MON[2] := 28;
  MON[3] := 31;
  MON[4] := 30;
  MON[5] := 31;
  MON[6] := 30;
  MON[7] := 31;
  MON[8] := 31;
  MON[9] := 30;
  MON[10] := 31;
  MON[11] := 30;
  MON[12] := 31;
  MONL[1] := 31;
  MONL[2] := 29;
  MONL[3] := 31;
  MONL[4] := 30;
  MONL[5] := 31;
  MONL[6] := 30;
  MONL[7] := 31;
  MONL[8] := 31;
  MONL[9] := 30;
  MONL[10] := 31;
  MONL[11] := 30;
  MONL[12] := 31;
  {----------------------------}
  if (IDATE = 1) then
  begin
    PYEAR := DAYNO mod 100;
    PMONTH := trunc(DAYNO / 10000);
    PDAY := trunc(DAYNO / 100) mod 100;
  end
  else if (IDATE = 2) then
  begin
    PYEAR := DAYNO mod 100;
    PMONTH := trunc(DAYNO / 100) mod 100;
    PDAY := trunc(DAYNO / 10000);
  end
  else if (IDATE = 3) then
  begin
    ADAY := DAYNO mod 100;
    AMONTH := trunc(DAYNO / 100) mod 100;
    AYEAR := trunc(DAYNO / 10000);
  end;
  {----------------------------}
  DAYM := ADAY;
  MONTH := AMONTH;
  YEAR := AYEAR;
  DAYNO := 1;
  {----------------------------}
  while (dayno <= cumt) and (not ready) do
  begin
    ready := (MONTH = PMONTH) and (DAYM = PDAY) and (YEAR = PYEAR);
    if not (ready) then
    begin
      DAYNO := DAYNO + 1;
      DAYM := DAYM + 1;
      if (YEAR mod 4 = 0) and (DAYM > MONL[MONTH]) then
      begin
        DAYM := 1;
        MONTH := 1 + MONTH;
      end;
      if (YEAR mod 4 > 0) and (DAYM > MON[MONTH]) then
      begin
        DAYM := 1;
        MONTH := MONTH + 1;
      end;
      if (MONTH > 12) then
      begin
        MONTH := 1;
        YEAR := (YEAR + 1) mod 100;
      end;
    end;
  end;
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Daycalc');
end;

procedure CHEM2(var CACO, CASO, ALOX, PH, BD, TAL, TCA, TMG, TNA, TK, TCL, TSO4,
  THCO3, TCO3, BALION, TH, TOH, THETA, EC, SAR, XCA, XMG, XNA, XK, XH, XAL, CEC,
  ESP, K1, K2, K3, K4, K5, K6, K7, K8, K9, K10, K11, K12, K13, K14, K15, DelGibb,
  DELGYP, DECALCITE, PCO2: real; var NN1: integer; SP1, SP2: real;
  var OSPOT: real; var trace: integer; var delco2: real; var n1prt: integer;
  temperature: real);
{equilibrate the chemical composition of soil solution, precipitated phase
 and exchange phase while finding the soil pH}
{  *****************************************************************
C  *****************************************************************
C  *                                                               *
C  *                    CHEMical EQuilibrium                       *
C  *                                                               *
C  *     A routine for calculating equilibrium soil solution       *
C  *      composition in calcareous and/or gypsiferous soils       *
C  *      taking into account gibbsite - Al equilibria as well     *
C  *      (this version based on concentrations and uses           *
C  *       constants adjusted by activities during each iteration) *
C  *                                                               *
C  *             J.L. Hutson, Dept of Agronomy, Cornell            *
C  *                  University,Ithaca, N.Y. 14853                *
C  * Al-chemistry+other ions added by Peter Finke, UGent, 2010+223 *
C  *                                                               *
C  *****************************************************************
C  *****************************************************************
C
C
C  Input data required are:
C  CASO:=Gypsum   CACO:=calcite (decimal fraction on a mass basis kg/kg)
C  PH: pH   BD: Bulk density (kg/dm**3)
C  TCA,TMG,TNA,TK,TCL,TSO4: mmol/l of these ions in solution (after
C   input these concentrations are converted to mol/l) (total
C   analytical which includes single ions and ion pairs)
C  CA, MG, NA etc. are concentrations of a particular species (mol/l).
C  THETA: Fractional water content (m3/m3)
C  NN1: Flag which determines whether selectivity coefficients are
C   provided (NN1:=0) or require to be calculated from the solution
C   concentrations and exchangeable cations (NN1:=1).
C  XCA,XMG,XNA,XK: Exchangeable cations (meq/kg).
C  Values calculated for THCO3,TCO3 (mmol/l) and pH.
C  EC: Electrical conductivity (mS/cm).
C  SAR: Sodium adsorption ratio
C  The values of pCO2, BD, THETA and NN1 do not change during
C   execution of CHEMEQ.
C
C  Other constants:
C     KH  Henry's law constant for CO2
C     KW  Stability constant of water
C     KA1 First dissociation constant of H2CO3
C     KA2 Second dissociation constant of H2CO3
C     KD1 Stability constant of CaCO3
C     KD2 Stability constant of CaHCO3+
C     KD3 Stability constant of CaOH+
C     KD4 Stability constant of CaSO4
C     KD5 Stability constant of MgCO3
C     KD6 Stability constant of MgHCO3+
C     KD7 Stability constant of MgOH+
C     KD8 Stability constant of MgSO4
C     KD9 Stability constant of NaSO4-
C     KD10 Stability constant of NaCO3-
C     KD11 Stability constant of KSO4-
C     SP1 Solubility product of gypsum
C     SP2 Solubility product of calcite
C  A subscript ..C (e.g. KD5C) indicates concentration-based constant.
C  H: Hydrogen ion concentration (moles/l).
C  ADJGYP and ADJLIM convert gypsum and calcite between decimal fractions
C   and moles/l in soil solution.
C  ACT1 and ACT2 are the activity coefficients for mono- and divalent
C   ions. Chemical symbols preceded by A represent ion activities.
C  PCO2: partial pressure of carbon dioxide (atmos).
  {           Gapon coefficients;
              K1=Mg/Ca; K2=Ca/Na; K3=Ca/K;  K4=Mg/K;  K5= Mg/Na; K6= K/Na;
              K7=Ca/Al; K8=Ca/H;  K9=Mg/Al; K10=K/Al; K11= Na/Al; K12=Mg/H;
              K13=K/H;  K14=Na/H; K15=Al/H
C********************************************************************}
var
  KI, MGI, NAI, ISS, NASO4, NA, NACO3, MG, MGCO3, MGHCO3, MGSO4, K, KSO4, MGOH, OH, KHC,
  KA1C, KA2C, KWC, KD1C, KD2C, KD3C, KD4C, KD5C, KD6C, KD7C, KD8C, KD9C, KD10C, KD11C,
  K1C, K2C, K3C, K4C, K5C, K6C, r1, r2, RATIO, spcon, TALK, COI, ADJGYP,
  ADJLIM, ACALCITE, AGYP, SO4I, CLI, CAI, XSUM, SUM1, BALI, PPT, SO4, TCAI, TNAI,
  TKI, TCLI, TSO4I, THCO3I, TCO3I, XCAI, XMGI, XNAI, XKI,
  CACOI, CASOI, PHHIGH, PHLOW, TMGI, CACO3, CA, CASO4, CO3, HCO3, SP1C, CAHCO3, CAOH, SP2C,
  KSPc, H, ACT1, ACT2, CHGNEW, ADJGIB, HYDI, THYDI, XHYDI, ALI, TALI, XALI, AL, ALOXI, AALOX,
  K7C, K8C, K9C, K10C, K11C, K12C, K13C, K14C, K15C, AlOH_2p, ALOH2_p, ALOH4_m, memvalue: real;
  i1, i2, ITER, IFLG: integer;
  txt1, txt2: string;
  ph1, ph2, chg1, chg2, AAAAA, BBBBB, lastcharge, phdelt: real;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering Chem2');
  {change the default reaction constants using the current temperatures}
  AdaptConst(temperature);
  {C  Constants which are used in CALL statements in place of numeric values}
  r1 := 1;
  r2 := 2;
  i1 := 1;
  i2 := 2;
  spcon := 0;     {added to prevent very small negative values}
  CACO:=max(1e-50, CACO);
  CASO:=max(1e-50, CASO);
  TCA:=max(1e-50, TCA);
  TMG:=max(1e-50, TMG);
  TNA:=max(1e-50, TNA);
  TK:=max(1e-50, TK);
  TAL:=max(1e-50,TAL);
  TCL:=max(1e-50, TCL);
  TSO4:=max(1e-50, TSO4);
  THCO3:=max(1e-50, THCO3);
  TCO3:=max(1e-50, TCO3);
  TH:=max(1e-50,TH);
  TOH:=max(1e-50,TOH);
  XCA:=max(1e-50, XCA);
  XMG:=max(1e-50,XMG);
  XNA:=max(1e-50,XNA);
  XK:=max(1e-50,XK);
  XAL:=max(1e-50,XAL);
  XH:=max(1e-50,XH);
{--------------------------------------------------------------------
C  Concentrations are converted from mmol/l to moles/l.  }
  TALK := 2 * TCO3 + THCO3 + TOH - TH;
  COI := TCO3 + THCO3;
  TCO3 := 0;
  THCO3 := 0;
  TH := TH / 1000;
  {+}TAL := TAL / 1000;
  TOH := toh / 1000;
  TCA := TCA / 1000;
  TMG := TMG / 1000;
  TNA := TNA / 1000;
  TK := TK / 1000;
  TCL := TCL / 1000;
  TSO4 := TSO4 / 1000;
  TALK := TALK / 1000;
  BALION := BALION / 1000;
  COI := COI / 1000;
{----------------------------
C  calcite and gypsum are converted from mmol/dm3 soil to moles/l soil solution. }
  ADJGYP := 0.001 / THETA;
  CASO := CASO * ADJGYP;
  ADJLIM := 0.001 / THETA;
  CACO := CACO * ADJLIM;
  ACALCITE := CACO;
  AGYP := CASO;
  {+}ADJGIB := 0.001 / (THETA * DLZ); {includes conversion from 1 sq meter x DLZ mm to 1 dm3}
  {+}ALOX := ALOX * ADJGIB;
  {+}AALOX := ALOX;
{-----------------------------
C  Convert from meq/kg to eq/kg}
  XCA := XCA / 1000;
  XMG := XMG / 1000;
  XNA := XNA / 1000;
  XK := XK / 1000;
  {+}XH := XH / 1000;
  {+}XAL := XAL / 1000;
  CEC := CEC / 1000;
  {----------------------------}
  if (NN1 = 2) or (ABS(CEC) < 0.000001) then
  begin
    XCA := 0;
    XMG := 0;
    XNA := 0;
    XK := 0;
    {+}XAL := 0;
    XH := 0;
  end;
{----------------------------
C  Starting values of chemical species recorded so that precipitation
C   and dissolution can be calculated (mol/dm3 soil).  These totals
C   exclude calcite and gypsum, which may numerically mask small changes.}
  SO4I := TSO4 * THETA;
  CLI := TCL * THETA;
  CAI := BD * XCA / 2. + TCA * THETA;
  MGI := BD * XMG / 2. + THETA * TMG;
  NAI := BD * XNA + THETA * TNA;
  KI := BD * XK + THETA * TK;
  {+}HYDI := BD * XH + TH * THETA;
  {+}ALI := BD * XAL / 3 + TAL * THETA;
{---------------------------------
C  Check whether CEC greater than total cations}
  XSUM := (XCA + XMG + XNA + XK + XAL + XH);
  SUM1 := 3 * ALI + 2 * (CAI + MGI) + KI + NAI + HYDI;
  RATIO := 1; {DUMMY INITIALISATIE}
  if (CEC > 0) then      {was: CEC>0)}
  begin
    RATIO := (CEC * BD) / SUM1;
    if (RATIO > 1) and (NN1 = 2) then
    begin
      {wwriteln(' CEC exceeds total cations - program should be terminated');}
      {write(ratio); STOP}
    end;
  end;
{---------------------------------
C  If solution cations represent exchangeable + solution, then
C   partition cations according to the ratio of CEC/total cations.}
  if (CEC > 0) and (NN1 = 0) and (XSUM = 0) then   {was: CEC>0}
  begin
    XCA := (RATIO * 2 * CAI) / BD;
    TCA := (CAI - RATIO * CAI) / (THETA);
    XMG := (RATIO * 2 * MGI) / BD;
    TMG := (MGI - RATIO * MGI) / (THETA);
    XNA := (RATIO * NAI) / BD;
    TNA := (NAI - RATIO * NAI) / THETA;
    XK := (RATIO * KI) / BD;
    TK := (KI - RATIO * KI) / THETA;
    {+}XAL := (RATIO * 3 * ALI) / BD;
    {+}TAL := (ALI - RATIO * ALI) / THETA;
    {+}XH := (RATIO * HYDI) / BD;
    {+}TH := (HYDI - RATIO * HYDI) / THETA;
  end;
{----------------------------
C  Overall charge balance}
  if (n1prt = 0) then
  begin
    BALI := (3 * TAL + 2 * (TCA + TMG - TCO3 - TSO4) + TNA + TK - TCL - TALK) * THETA +
      (XCA + XMG + XNA + XK + XAL + XH - CEC) * BD;
{      system.write(*,9000)BALI
       9000 FORMAT(//,' The overall charge imbalance (cations - anions - CEC)'
       #,/,' is ',e10.4,' mol/dm3',/)
       WRITE(*,9070)
       9070 FORMAT(' Any imbalance between cations and anions is corrected by'
       #,/,' introducing a balancing univalent anion, which may have a'
       #,/,' negative concentration if anions exceed cations.',/)
       pause ' If you wish to continue, press return, else break.'}
    BALION := BALI / theta;
  end;
{---------------------------------
C  If concentrations of Ca and SO4 and/or CO3 are high
C   calcite and gypsum are increased and solution concentrations of
C   Ca, SO4 and CO3 are decreased, otherwise activity coefficient
C   calculations will be in error.}
  PPT := MIN(TCA, TCO3);
  CACO := PPT + CACO;
  TCA := TCA - PPT;
  TCO3 := TCO3 - PPT;
  PPT := MIN(TCA, TSO4);
  CASO := CASO + PPT;
  TCA := TCA - PPT;
  TSO4 := TSO4 - PPT;
  {+}PPT := MIN(TAL, TOH / 3);
  {+}ALOX := PPT + ALOX;
  {+}TAL := TAL - PPT;
  {useless for TH and TOH}
{---------------------------------
C  First approximations of free ion concentrations are made from
C  the total concentrations that can be considered at this point. }
  SO4 := TSO4;
  K := TK;
  NA := TNA;
  MG := TMG;
  {+}AL := TAL;
  {  First estimate of ionic strength from concentrations of cations and anions}
  ISS := 0.5 * (9 * TAL + 4. * (TCA + TMG + TSO4 + TCO3) + (TNA + TK + TCL + THCO3 + ABS(BALION)));
  TCAI := TCA;
  TMGI := TMG;
  TNAI := TNA;
  TKI := TK;
  TCLI := TCL;
  TSO4I := TSO4;
  THCO3I := THCO3;
  TCO3I := TCO3;
  XCAI := XCA;
  XMGI := XMG;
  XNAI := XNA;
  XKI := XK;
  {+}TALI := TAL;
  {+}XALI := XAL;
  {+}HYDI := TH;
  {+}XHYDI := XH;
  {+}ALOXI := ALOX;
  CACOI := CACO;
  CASOI := CASO;
{********************************************************************
C********************************************************************
C  CHEMICAL EQUILIBRIUM LOOP
C--------------------------------------------------------------------
C--------------------------------------------------------------------
C  Iterate to find pH at which charge balance corresponds to the
C  reference value above
C----------------------------
C  Iterate pH values between 3 and 10 using a bisection procedure
C  assuming that there is only one charge balance minimum
   THIS IS SOMETIMES (seldomly) NOT TRUE WITH AL-CHEMISTRY INCLUDED}
  PHHIGH := 10;
  PHLOW := 3;
  PH := (PHHIGH + PHLOW) / 2;
  IFLG := 0;
{ pH step size associated with iteration numbers:
iter  dPH
start  3.0000   =(10-4)/2  or PHHIGH-PH
1  1.5000   / 2  0.7500 / 3  0.3750 / 4  0.1875
5  0.0938   / 6  0.0469 / 7  0.0234 / 8  0.0117
9  0.0059   /10  0.0029 /11  0.0015 /12  0.0007
13  0.0004 /14  0.0002 /15  0.0001
Ergo iter=8 gets a pH-approximation more accurate than 0.01
    iter=10 gets a pH-approximation more accurate than 0.003 }

  for ITER := 1 to 10 do
  begin
    CASO := CASOI;
    CACO := CACOI;
    {+}ALOX := ALOXI;
    {+}TAL := TALI;
    {+}XAL := XALI;
    {+}XH := XHYDI;
    TCL := TCLI;
    TSO4 := TSO4I;
    THCO3 := THCO3I;
    TCO3 := TCO3I;
    TCA := TCAI;
    TMG := TMGI;
    TNA := TNAI;
    TK := TKI;
    XCA := XCAI;
    XMG := XMGI;
    XNA := XNAI;
    XK := XKI;
    {------------- }
    str(iter, txt1);
    txt1 := '  Entering EQUILCHM2' + txt1;
    {if trace = 1 then}
      singletracewriteln(txt1);
    EQUILCHM2(K1, K2, K3, K4, K5, K6, K7, K8, K9, K10, K11, K12, K13, K14, K15, AlOH_2p, ALOH2_p, ALOH4_m,
      NASO4, NA, NACO3, MG, MGCO3, MGHCO3, MGSO4, K, KSO4,
      MGOH, OH, KHC, KA1C, KA2C, KWC, KD1C, KD2C, KD3C,
      KD4C, KD5C, KD6C, KD7C, KD8C, KD9C, KD10C, KD11C,
      BALION, SP1, SP2, K1C, K2C, K3C, K4C, K5C, K6C, K7C, K8C,
      K9C, K10C, K11C, K12C, K13C,
      K14C, K15C, PH, PCO2, CACO3, CA, CO3, CAHCO3, HCO3, CAOH, CASO4, SO4, AL,
      SP1C, SP2C, KSPc, H, ACT1, ACT2, TCA, TMG, TNA, TK, TAL, XCA, XMG, XNA, XK, XAL, XH,
      CACO, CASO, ALOX, TCL, TOH, TH, TCO3, THCO3, TSO4,
      ISS, CEC, ITER, THETA, BD, NN1, trace);
{--------------------------------------------------------------------
C  Charge imbalance in solution}
   CHGNEW := 3 * TAL + 2 * (TCA + TMG - TCO3 - TSO4) + TNA + TK + TH - TCL - THCO3 - TOH - BALION+
              2*SEFeII/1000+3*SEFeIII/1000;
    {  Set new pH bounds}
    if (CHGNEW <= 0) then
      PHHIGH := PH;
    if (CHGNEW >= 0) then
      PHLOW := PH;
{----------------------------
C  New pH for next iteration }
    PH := (PHHIGH + PHLOW) / 2;
  end;
{********************************************************************
C  END OF CHEMICAL EQUILIBRIUM LOOP.
C********************************************************************
C********************************************************************
C  Call ECII to calculate final electrical conductivity}
  ECII(TCA, TMG, TNA, TK, TCL, TSO4, THCO3, TCO3, BALION, EC, ISS,
    NASO4, NA, NACO3, MG, MGHCO3, K, KSO4, MGOH, OH, CA, CO3, CAHCO3, HCO3, CAOH,
    SO4, H, ACT1, ACT2, CACO, SPCON);
  {  Electrical conductivity converted from mS/cm to mS/m}
  EC := EC * 100;
{-----------------------------
C  Set new exchange and solubility constants}
  if (NN1 = 1) then
  begin
    K1 := K1C;
    K2 := K2C * ACT1 / SQRT(ACT2);
    K3 := K3C * ACT1 / SQRT(ACT2);
    K4 := K4C * ACT1 / SQRT(ACT2);
    K5 := K5C * ACT1 / SQRT(ACT2);
    K6 := K6C;
  end;
{----------------------------
C Precipitation or dissolution of calcite and gypsum during equilibration (mmol/dm3).}
  DELGYP := (CASO - AGYP) * 1000 * theta;
  DECALCITE := (CACO - ACALCITE) * 1000 * theta;
  DelGibb := (ALOX - AALOX) * 1000 * theta;
  {  DELCO2, sink/source of Atm. CO2.  Gains from atm }
  delco2 := ((tco3 + thco3) - coi + (caco - aCALCITE)) * 1000 * theta;
{---------------------------------
C  Moles/l in solution are converted to mmol/l.   }
  TCA := TCA * 1000;
  TMG := TMG * 1000;
  TNA := TNA * 1000;
  TK := TK * 1000;
  {+}TAL := TAL * 1000;
  {+}XAL := XAL * 1000;
  {+}XH := XH * 1000;
  XCA := XCA * 1000;
  XMG := XMG * 1000;
  XNA := XNA * 1000;
  XK := XK * 1000;
  CEC := CEC * 1000;
  TSO4 := TSO4 * 1000;
  THCO3 := THCO3 * 1000;
  TCO3 := TCO3 * 1000;
  TCL := TCL * 1000;
  TH := TH * 1000;
  TOH := TOH * 1000;
  BALION := BALION * 1000;
{---------------------------------
C  calcite and gypsum converted back to mmol/dm3 soil.}
  CASO := CASO / ADJGYP;
  CACO := CACO / ADJLIM;
  {+}ALOX := ALOX / ADJGIB;
{----------------------------
C  SAR and ESP calculated.}
  if (TCA + TMG > 0) then
    SAR := TNA / SQRT((TCA + TMG))
  else
    SAR := 0;
  if (CEC > 0) then
    ESP := XNA * 100 / CEC
  else
    ESP := 0;
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Chem2');
end;  {CHEM2}


{********************************************************************
C*  C H E M I C A L   E Q U I L I B R I U M   S U B R O U T I N E S *
C                 (used by CHEM2.FOR and CHEMEQ.FOR)
C********************************************************************   }

procedure EQUILCHM2(var K1, K2, K3, K4, K5, K6, K7, K8, K9, K10, K11, K12, K13, K14, K15,
  AlOH_2p, ALOH2_p, ALOH4_m, NASO4,
  NA, NACO3, MG, MGCO3, MGHCO3, MGSO4, K, KSO4,
  MGOH, OH, KHC, KA1C, KA2C, KWC, KD1C, KD2C, KD3C,
  KD4C, KD5C, KD6C, KD7C, KD8C, KD9C, KD10C, KD11C,
  BALION, SP1, SP2, K1C, K2C, K3C, K4C, K5C, K6C, K7C, K8C,
  K9C, K10C, K11C, K12C, K13C, K14C, K15C, PH, PCO2, CACO3,
  CA, CO3, CAHCO3, HCO3, CAOH, CASO4, SO4, Al, SP1C, SP2C,
  KSPc, H, ACT1, ACT2, TCA, TMG, TNA, TK, TAL, XCA, XMG, XNA, XK, XAL, XH,
  calcite_sol, gypsum_sol, Gibbsite_sol, TCL, TOH, TH, TCO3, THCO3, TSO4,
  ISS, CEC: real; ITER: integer; var THETA, BD: real;
  var NN1, trace: integer);
{calculate chemical equilibriums for a fixed/set pH (in iteration called from CHEM2)}
{C  *      Al-chemistry added by Peter Finke, UGent, 2010           *}
var
  CAI, soi, dso, dca, r1, r2, CCONST, H2C03, calcite_sol_old, BCA, BMG, BNA, BK,
  dgypsum_sol, CL, CAf, r3, dgi, ACT3, KAl1c, KAL2c, KAL4c, gibbsite_soil_old, BAL, BH: real;
  i1, i2, II: integer;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering Equilchm2');
  CAI := BD * XCA / (theta * 2) + TCA + calcite_sol + gypsum_sol; {not used}
  soi := tso4 + gypsum_sol;                              {not used}
  dso := 0;
  dca := 0;
  {+}dgi := 0;
  {  Constants which are used in CALL statements in place of numeric values}
  r1 := 1;
  r2 := 2;
  r3 := 3;
  i1 := 1;
  i2 := 2;
  {----------------------------}
  act1 := 1;
  act2 := 1;
  act3 := 1;
  for II := 1 to 20 do
  begin
{  Solution Ca and SO4 put into variables representing potential
C   supply.  Calcite and gypsum will also act as sources and sinks.}
    dca := dca + tca;
    dso := dso + tso4;
    {+}dgi := dgi + tal;
    {  The following species are determined by pCO2, pH and activities.}
    TCA := 0;
    THCO3 := 0;
    TCO3 := 0;
    TOH := 0;
    TSO4 := 0;
    TH := 0;
    {+}TAL := 0;
{----------------------------
C  Thermodynamic constants changed to conditional constants}
    ACT1 := ACT(R1, ISS);
    ACT2 := ACT(R2, ISS);
    {+}ACT3 := ACT(R3, ISS);
    KHC := KH;
    KA1C := KA1 / (ACT1 * ACT1);
    KA2C := KA2 / ACT2;
    KWC := KW / (ACT1 * ACT1);
    KD1C := KD1 / (ACT2 * ACT2);
    KD2C := KD2 / ACT2;
    KD3C := KD3 / ACT2;
    KD4C := KD4 / (ACT2 * ACT2);
    KD5C := KD5 / (ACT2 * ACT2);
    KD6C := KD6 / ACT2;
    KD7C := KD7 / ACT2;
    KD8C := KD8 / (ACT2 * ACT2);
    KD9C := KD9 / ACT2;
    KD10C := KD10 / ACT2;
    KD11C := KD11 / ACT2;
    SP1C := SP1 / (ACT2 * ACT2);
    SP2C := SP2 / (ACT2 * ACT2);
    {Al-hydroxide species}
    {+}KSPc := KSP * ACT1 * ACT1 * ACT1 / ACT3;
    {+}KAl1c := KAl1 * ACT3 / (ACT2 * ACT1);
    {+}KAl2c := KAl2 * ACT3 / (ACT1 * ACT1 * ACT1);
    {+}KAl4c := KAl4 * ACT3 / (ACT1 * ACT1 * ACT1 * ACT1 * ACT1);
    K1C := K1;
    K2C := K2 * SQRT(ACT2) / ACT1;
    K3C := K3 * SQRT(ACT2) / ACT1;
    K4C := K4 * SQRT(ACT2) / ACT1;
    K5C := K5 * SQRT(ACT2) / ACT1;
    K6C := K6;
    {added activity-modified Gapon}
    {+}K7C := K7 * SQRT(ACT2) * POWER(ACT3, -1 / 3);
    {+}K8C := K8 * SQRT(ACT2) / ACT1;
    {+}K9C := K9 * SQRT(ACT2) * POWER(ACT3, -1 / 3);
    {+}K10C := K10 * ACT1 * POWER(ACT3, -1 / 3);
    {+}K11C := K11 * ACT1 * POWER(ACT3, -1 / 3);
    {+}K12C := K12 * SQRT(ACT2) / ACT1;
    {+}K13C := K13;
    {+}K14C := K14;
    {+}K15C := K15 * POWER(ACT3, 1 / 3) / ACT1;
{----------------------------
C  Hydrogen ion concentration calculated from pH}
    H := power(10, -PH) / act1;
    TH := H;
    {  Hence OH concentration}
    OH := KWC / H;
{----------------------------
C  HCO3 from 1st dissociation constant of H2CO3}
    CCONST := KA1C * PCO2 * KHC;
    H2C03 := PCO2 * KHC;
    HCO3 := CCONST / H;
{----------------------------
C  CO3 from 2nd dissociation constant of H2CO3}
    CO3 := HCO3 * KA2C / H;
{----------------------------
C  Ca from solubility product of CaCO3, if necessary
c      write(SP2C,CO3)}
    if (CO3 > 0) then
      CA := MAX(1e-13, SP2C / CO3);
{----------------------------
C  Dissolve calcite if it is present and the pCO2 is zero (ie. CO3 is 0)}
    if (pco2 = 0) and (calcite_sol > 0) then
    begin
      tca := dca;
      CA := TCA / (1 + CO3 / KD1C + HCO3 / KD2C + OH / KD3C + SO4 / KD4C);
      {  The soil solution is equilibrated with calcite.}
      wwriteln('The soil solution is equilibrated with calcite');
      PRECIP(SP2C, calcite_sol, CA, CO3, TCA, TCO3, i2);
    end;
{----------------------------
C  This corresponds to a total Ca concentration of:}
    TCA := CA * (1 + CO3 / KD1C + HCO3 / KD2C + OH / KD3C + SO4 / KD4C);
{  If there is insufficient Ca in the system to supply this then
C  use all that is available:}
    TCA := MIN(calcite_sol + DCA + gypsum_sol, TCA);
    {  which corresponds to a free Ca concentration of:}
    CA := TCA / (1 + CO3 / KD1C + HCO3 / KD2C + OH / KD3C + SO4 / KD4C);
    {  Adjust soluble Ca reserve}
    DCA := DCA - TCA;

    if (DCA < 0) then
    begin
      calcite_sol_old := calcite_sol;
      calcite_sol := calcite_sol_old + DCA;
{c      tco3:=tco3+calcite_sol_old-calcite_sol
c      CO3:=TCO3/(1.+CA/KD1C+MG/KD5C+NA/KD10C) }
      DCA := 0;
    end;
    if (calcite_sol < 0) then
    begin
      DSO := DSO - calcite_sol;
      ;
      gypsum_sol := gypsum_sol + calcite_sol;
      calcite_sol := 0;
    end;
{----------------------------
C  SO4 from solubility product of CaSO4, if necessary}
    if (CA > 0) then
      SO4 := SP1C / CA;
    {--------------}
    if (ca = 0) and (gypsum_sol > 0) then
    begin
      tso4 := dso;
      SO4 := TSO4 / (1 + CA / KD4C + MG / KD8C + NA / KD9C + K / KD11C);
      {  The soil solution is equilibrated with gypsum.}
      wwriteln('The soil solution is equilibrated with gypsum');
      PRECIP(SP1C, gypsum_sol, CA, SO4, TCA, TSO4, i2);
    end;
{--------------
C  This corresponds to a total SO4 concentration of:}
    TSO4 := SO4 * (1 + CA / KD4C + MG / KD8C + NA / KD9C + K / KD11C);
{  If there is insufficient SO4 in the system to supply this then
C  use all that is available:}
    TSO4 := MIN(gypsum_sol + DSO, TSO4);
    {  which corresponds to a free SO4 concentration of:}
    SO4 := TSO4 / (1 + CA / KD4C + MG / KD8C + NA / KD9C + K / KD11C);
    {  Adjust soluble SO4 reserve}
    DSO := DSO - TSO4;
    if (DSO < 0) then
    begin
      gypsum_sol := gypsum_sol + DSO;
      DCA := DCA - DSO;
      DSO := 0;
    end;
    gypsum_sol := MAX(0, gypsum_sol);
    calcite_sol := MAX(0, calcite_sol);

    TCO3 := CO3 * (1 + CA / KD1C + MG / KD5C + NA / KD10C);
    THCO3 := HCO3 * (1 + CA / KD2C + MG / KD6C);
    TOH := OH * (1 + CA / KD3C + MG / KD7C);
    CA := TCA / (1 + CO3 / KD1C + HCO3 / KD2C + OH / KD3C + SO4 / KD4C);
    MG := TMG / (1 + CO3 / KD5C + HCO3 / KD6C + OH / KD7C + SO4 / KD8C);
    NA := TNA / (1 + SO4 / KD9C + CO3 / KD10C);
    K := TK / (1 + SO4 / KD11C);
    {next: Al and Gibbsite}
    {1: How much Al in solution?}
    Al := KSPc * H * H * H; {solubility Gibbsite}
    {2. Which corresponds to a total Al-concentration of:}
    TAL := Al * (1 + (KAl1c / H) + (KAl2c / (H * H)) + (KAl4c / (H * H * H * H)));
    {3. In case there is insufficient supply, take what's available:}
    TAL := min(Gibbsite_sol, TAL);
    {which corresponds to free Al concentration of:}
    Al := TAL / (1 + (KAl1c / H) + (KAl2c / (H * H)) + (KAl4c / (H * H * H * H)));
    {4. Adjust soluble Al-reserve}
    dgi := dgi - TAL;
    {5. In case this reserve is negative, take it from Gibbsite and set reserve to 0}
    if dgi < 0 then
    begin
      gibbsite_sol := gibbsite_sol + dgi;
      dgi := 0;
    end;
    {6. finally check if gibbsite pool is still positive, else correct TAL and Al}
    if gibbsite_sol < 0 then
    begin
      TAl := TAl + gibbsite_sol;
      Al := TAl / (1 + (KAl1c / H) + (KAl2c / (H * H)) + (KAl4c / (H * H * H * H)));
      gibbsite_sol := 0;
    end;
    {+update TOH}
    TOH := OH * (1 + CA / KD3C + MG / KD7C + Al * KAl1c / KWc + OH * Al * KAl2c / (KWc * KWc) + OH * OH * OH * Al * KAl4c /
      (SQR(SQR(KWc))));
{********************************************************************
C  If CEC IS NONZERO THE eXCHANGe subroutine brings the solution
C   into equilibrium with the exchange phase.
C
c      if(iter>4)then}
    if (CEC > 0) then      {was: if (CEC>0) THEN; aangepast vw crash bij CEC van bijna 0}
    begin
      BCA := CA;
      BMG := MG;
      BNA := NA;
      BK := K;
      {+}BAL := Al;
      {+}BH := H;
      XCHANG2(CA, MG, NA, K, Al, H, XCA, XMG, XNA, XK, XAl, XH, BD, THETA, CEC,
        K1C, K2C, K3C, K4C, K5C, K6C, K7C, K8C, K9C, K10C, K11C, K12C, K13C, K14C, K15C,
        NN1);
      {C-------------}
      TCA := TCA + CA - BCA;
      TMG := TMG + MG - BMG;
      TNA := TNA + NA - BNA;
      TK := TK + K - BK;
      {+}TH := TH + H - BH;
      {+}TAL := TAL + AL - BAL;
      CA := TCA / (1 + CO3 / KD1C + HCO3 / KD2C + OH / KD3C + SO4 / KD4C);
      MG := TMG / (1 + CO3 / KD5C + HCO3 / KD6C + OH / KD7C + SO4 / KD8C);
      NA := TNA / (1 + SO4 / KD9C + CO3 / KD10C);
      K := TK / (1 + SO4 / KD11C);
      Al := TAl / (1 + (KAl1c / H) + (KAl2c / (H * H)) + (KAl4c / (H * H * H * H)));
      TCO3 := CO3 * (1 + CA / KD1C + MG / KD5C + NA / KD10C);
      THCO3 := HCO3 * (1 + CA / KD2C + MG / KD6C);
      TOH := OH * (1 + CA / KD3C + MG / KD7C);
      {revised TOH}
      TOH := OH * (1 + CA / KD3C + MG / KD7C + Al * KAl1c / KWc + OH * Al * KAl2c /
        (KWc * KWc) + OH * OH * OH * Al * KAl4c / (SQR(SQR(KWc))));
      {c      TSO4:=SO4*(1+CA/KD4C+MG/KD8C+NA/KD9C+K/KD11C)}
      SO4 := TSO4 / (1 + CA / KD4C + MG / KD8C + NA / KD9C + K / KD11C);
    end;
{********************************************************************
C  Change in calcite and gypsum
C   Any remaining Ca and SO4 reserve is gypsum}
    dgypsum_sol := MIN(MAX(0, dso), MAX(0, dca));
    dso := dso - dgypsum_sol;
    dca := dca - dgypsum_sol;
    gypsum_sol := gypsum_sol + dgypsum_sol;
    dgypsum_sol := MIN(MAX(0, dso), MAX(0, calcite_sol));
    dso := dso - dgypsum_sol;
    calcite_sol := calcite_sol - dgypsum_sol;
    gypsum_sol := gypsum_sol + dgypsum_sol;
    if (PCO2 > 0) then
      calcite_sol := calcite_sol + dca;
    dca := 0;
    { ... and any remaining Al-reserve is gibbsite}
    {+}gibbsite_sol := gibbsite_sol + max(0, dgi);
    {+}dgi := 0;
{-------------
C   i) Concentrations (mol/l) calculated for each species}
    CL := TCL;
    CACO3 := CA * CO3 / KD1C;
    CAHCO3 := CA * HCO3 / KD2C;
    CAOH := CA * OH / KD3C;
    CASO4 := CA * SO4 / KD4C;
    MGCO3 := MG * CO3 / KD5C;
    MGHCO3 := MG * HCO3 / KD6C;
    MGOH := MG * OH / KD7C;
    MGSO4 := MG * SO4 / KD8C;
    NASO4 := NA * SO4 / KD9C;
    NACO3 := NA * CO3 / KD10C;
    KSO4 := K * SO4 / KD11C;
    AlOH_2p := KAl1c * Al / H;
    ALOH2_p := KAl2c * Al / (H * H);
    ALOH4_m := KAl4c * Al / (H * H * H * H);
{----------------------------
C   Ionic strength is calculated from ionic concentrations and used
C    to calculate mono- and divalent ion mean activity coefficients   }
    ISS := 0.5 * (9 * Al + 4. * (CA + MG + SO4 + CO3 + AlOH_2p) +
      (NA + K + H + CL + HCO3 + CAHCO3 + CAOH + MGHCO3 + MGOH +
      NASO4 + NACO3 + KSO4 + OH + ALOH2_p + ALOH4_m + ABS(BALION)));
    {----------------------------}
  end; {for II:=1 to 20 DO}
  CAf := BD * XCA / (theta * 2) + TCA + calcite_sol + gypsum_sol + dca;
  {      write(cai,caf)}
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Equilchm2');
end; {EQUILCHM2}

function ACT(Z, ISS: real): real;
{calculates ionic activity coefficients using calculated ionic  strength (called from EQUILCHM2)}
{********************************************************************
C  The function ACT calculates ionic activity coefficients using either
C   the approximation of Griffin and Jurinak (1973) or calculated ionic
C   strength.}
var
  I: real;
begin
  iss := MIN(0.5, MAX(0.000001, iss));
  I := SQRT(ISS);
  ACT := MIN(1.0, power(10.0, -0.509 * Z * Z * (I / (1.0 + I) - 0.3 * ISS)));
end; {ACT}

procedure ECII(var TCA, TMG, TNA, TK, TCL, TSO4, THCO3, TCO3, BALION, EC, ISS,
  NASO4, NA, NACO3, MG, MGHCO3, K, KSO4, MGOH, OH, CA, CO3, CAHCO3,
  HCO3, CAOH, SO4, H, ACT1, ACT2, CACO, SPCON: real);
{estimates electrical conductivity (called from CHEM2 from iterated chemical composition)}
{********************************************************************
c********************************************************************
C  The subroutine ECII calculates electrical conductivity}
var
  MGSO, MGADJ, CAADJ, SO4ADJ, CASOAD, EC1, EC2, EC3, EC4, EC5, EC6, EC7, EC8, EC9, EC10: real;
  A, C, LO, L, Z, W, U: array[1..18] of real;
  R: array[1..18, 0..5] of real;
  T: array[1..18, 1..18] of real;
  CC: array[0..5] of real;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering ECII');
{----------------------------
C  This part uses the exponential method of McNeal et al.
C  1970 (Soil Sci. 110:405-414) to calculate electrical conductivity
C  (EC) from Ca, Mg, Na,K,Cl,SO4,HCO3 AND CO3 concentrations in mol/l. }
  CAADJ := TCA;
  MGADJ := TMG;
  SO4ADJ := TSO4;
  CASOAD := 0;
{   The maximum Ca and Mg sulphate concentration is separated out before
C   calculating EC, as (Ca + Mg)SO4 has a unique EC/conc relationship.}
  CASOAD := MIN(CAADJ, SO4ADJ);
  CAADJ := MAX(0, CAADJ - CASOAD);
  SO4ADJ := MAX(0, SO4ADJ - CASOAD);
  MGSO := MIN(MGADJ, SO4ADJ);
  MGADJ := MAX(0, MGADJ - MGSO);
  SO4ADJ := MAX(0, SO4ADJ - MGSO);
  CASOAD := MAX(0, CASOAD + MGSO);
  {  EC is calculated using the coefficients obtained by McNeal et al.}
  EC1 := 0.0;
  if (CA > 0.0) then
    EC1 := 0.05641 * power(TCA * 2000, 0.9202);
  EC2 := 0.0;
  if (MG > 0.0) then
    EC2 := 0.05099 * power(TMG * 2000, 0.9102);
{  Option using individual ions
c      EC1:=0.0
c      IF(CA>0.0)EC1:=.05641*(power(2*CA+CAHCO3+CAOH)*1000,0.9202)
c      EC2:=0.0
c      IF(MG>0.0)EC2:=.05099*(power(2*MG+mghco3+mgoh)*1000,0.9102)  }
  EC3 := 0.0;
  if (TNA > 0.0) then
    EC3 := 0.04748 * power(TNA * 1000, 0.9495);
  EC4 := 0.0;
  if (TK > 0.0) then
    EC4 := 0.07263 * power(TK * 1000, 0.9706);
  EC5 := 0.0;
  if (SO4 > 0.0) then
    EC5 := 0.06900 * power(SO4 * 2000, 0.8973);
  EC6 := 0.0;
  if (TCO3 > 0.0) then
    EC6 := 0.07330 * power(TCO3 * 2000, 0.8719);
  EC7 := 0.0;
  if (THCO3 > 0.0) then
    EC7 := 0.04143 * power(THCO3 * 1000, 0.9501);
  EC8 := 0.0;
  if (TCL > 0.0) then
    EC8 := 0.07206 * power(TCL * 1000, 0.9671);
  EC9 := 0.0;
  if (CASOad > 0.0) then
    EC9 := 0.11330 * power(CASOad * 1000, 0.8463);
  EC10 := 0.0;
  if (ABS(BALION) > 0.0) then
    EC10 := 0.07206 * power(ABS(BALION) * 1000, 0.9671);
  EC := EC1 + EC2 + EC3 + EC4 + EC5 + EC6 + EC7 + EC8 + EC9 + EC10;
{----------------------------
C  Ionic strength calculated from concentrations of cations and anions
c      IS:=0.5*(4.*(TCA+TMG+TSO4+TCO3)+(TNA+TK+TCL+THCO3+ABS(BALION)))}
  {if (TRACE = 1) then}
    singletracewriteln(' leaving ECII');
end; {ECII}

procedure XCHANG2(var CA, MG, NA, K, Al, H, XCA, XMG, XNA, XK, XAl, XH, BD, THETA, CEC,
  K1C, K2C, K3C, K4C, K5C, K6C, K7C, K8C, K9C, K10C, K11C,
  K12C, K13C, K14C, K15C: real;
  var NN1: integer);
{calculate exchangeable cations using known selectivity coefficients and solution concentrations  (called from EQUILCHM2)}
{********************************************************************
C  The subroutine XCHANG2 is divided into two segments. If NN1 equals 1
C   selectivity coefficients are calculated from the cation activities
C   and exchangeable cations supplied, assuming them in equilibrium.
C   If NN1 := 0, new equilibrium is calculated for solution and
C   exchangeable cations using exchangeable cation and solution cation
C   concentrations, cation activities, bulk density (BD), volumetric
C   water content (THETA), and CEC values supplied from the calling
C   program.
C   If selectivity coefficients and solution concentrations
C   are known, calculate exchangeable cations. (NN1:=2)
C
C   TCA,TMG,...etc are moles/l of solution cations.
C   ACA,AMG,...etc are cation activities.
C   XCA,XMG,...etc are exch cations (mol/l).
C   K1...K6 are selectivity coefficients calculated using activities.
C   KA1...KA6 are selectivity coefficients for cation concentrations and
C    are calculated within this subroutine using Ki values and apparent
C    activity coefficients.
C----------------------------   }
var
  ZCA, ZMG, ZAL, ADJ3, ADJ2, ADJ1, cecadj, SCA, SMG, SNA, SK, SH, SAL, soln, xcaold, xmgold, xkold,
  xnaold, xhold, xalold, c1, c2, c6, c4, c11, c14, diff: real;
  escape: boolean;
  I, i1, i2, i3: integer;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering XChang2');
  i1 := 1;
  i2 := 2;
  i3 := 3;
{--------------------------------------------------------------------
C--------------------------------------------------------------------
C  OPTION 1 : Calculate selectivity coefficients from cation activities
C   and exchangeable cation levels (done when NN1 := 1).
C----------------------------}
  if (NN1 = 1) then
  begin
    ZCA := SQRT(CA);
    ZMG := SQRT(MG);
    ZAL := POWER(AL, (1 / 3));
    K1C := (ZCA * XMG) / (ZMG * XCA);
    K2C := (NA * XCA) / (ZCA * XNA);
    K3C := (K * XCA) / (ZCA * XK);
    K4C := (K * XMG) / (ZMG * XK);
    K5C := (NA * XMG) / (ZMG * XNA);
    K6C := (NA * XK) / (K * XNA);
    K7C := (ZAl * XCA) / (ZCA * XAL);
    K8C := (H * XCA) / (ZCA * XH);
    K9C := (ZAL * XMG) / (ZMG * XAL);
    K10C := (ZAL * XK) / (K * XAL);
    K11C := (ZAL * XNA) / (NA * XAL);
    K12C := (H * XMG) / (ZMG * XH);
    K13C := (H * XK) / (K * XH);
    K14C := (H * XNA) / (NA * XH);
    K15C := (H * XAL) / (ZAL * XH);
{1 = Mg/Ca;   2 = Ca/Na;  3 = Ca/K;  4  = Mg/K;  5  = Mg/Na; 6  = K/Na;   7 = Ca/Al; 8 = Ca/H;
 9  = Mg/Al; 10  = K/Al; 11  = Na/Al; 12  = Mg/H; 13  = K/H;  14  = Na/H; 15  = Al/H }
  end; {IF ... THEN}
{  OPTION 3:  If selectivity coefficients and solution concentrations
C   are known, calculate exchangeable cations. (NN1:=2)}
  if (NN1 = 2) then
  begin
    ZCA := SQRT(CA);
    ZMG := SQRT(MG);
    ZAL := POWER(AL, (1 / 3));
{--------------------------
C  New exchangeable cation levels are calculated using CEC, selectivity
C   coefficients and solution concentrations.
   CEC=XCA+XMG+XNA+XK+XH+XAL, whereby each exchanged species XWW is calculated by
   rewriting the equation so that XWW is written for ZWW+ZCA; ZWW+ZMG, etc}
    XCA := CEC / (1 + K1C * ZMG / ZCA + NA / (ZCA * K2C) + K / (ZCA * K3C) + H / (ZCA * K8C) + ZAL / (ZCA * K7C));
    {checked}
    XMG := CEC / (ZCA / (ZMG * K1C) + 1 + NA / (ZMG * K5C) + K / (ZMG * K4C) + H / (ZMG * K12C) + ZAL / (ZMG * K9C));
    {checked}
    XNA := CEC / (ZCA * K2C / NA + ZMG * K5C / NA + 1 + K * K6C / NA + H / (NA * K14C) + ZAL / (NA * K11C)); {checked}
    XK := CEC / (ZCA * K3C / K + ZMG * K4C / K + NA / (K * K6C) + 1 + H / (K * K13C) + ZAL / (K * K10C)); {checked}
    XH := CEC / (ZCA * K8C / H + ZMG * K12C / H + K * K13C / H + NA * K14C / H + 1 + ZAL * K15C / H); {checked}
    XAL := CEC / (ZCA * K7C / ZAL + ZMG * K9C / ZAL + K * K10C / ZAL + NA * K11C / ZAL + H / (ZAL * K15C) + 1); {checked}
  end; {IF ... THEN}
{--------------------------------------------------------------------
C  OPTION 2 : Bring solution and exchangeable cations into equilibrium
C   with specified selectivity coefficients.
C----------------------------}
  if (NN1 = 0) then
  begin
{  Adjustment factors are calculated to convert exchangeable cation
C  units between eq/kg soil and moles/l in solution.   }
    ADJ3 := BD / (THETA * 3);
    ADJ2 := 0.5 * BD / THETA;
    ADJ1 := BD / THETA;
    XCA := XCA * ADJ2;
    XMG := XMG * ADJ2;
    XNA := XNA * ADJ1;
    XK := XK * ADJ1;
    XH := XH * ADJ1;
    XAL := XAL * ADJ3;
    cecadj := cec * adj1;
{  The sum of each solution plus exchangeable cation is calculated
C   (mol/l). This must remain unchanged during XCHANG to preserve
C   mass balance. }
    SCA := CA + XCA;
    SMG := MG + XMG;
    SNA := NA + XNA;
    SK := K + XK;
    SH := H + XH;
    SAL := AL + XAL;
    soln := 3 * Al + 2 * (ca + mg) + na + k + h;
{--------------------------------------------------------------------
C  This loop brings the new exchangeable and solution cations into
C  equilibrium with each other, assuming that the CEC is constant and
C  equal to the sum of the exchangeable cations and that
C  exchangeable plus solution concentration remains constant for each
C  cation. }
    I := 0;
    escape := False;
    repeat
      I := I + 1;
{--------------------------------------------------------------------
C   Cycling through this loop leads to convergence to constant values
C   of exchangeable and solution cations.}
      xcaold := xca;
      xmgold := xmg;
      xnaold := xna;
      xkold := xk;
      xhold := xh;
      xalold := xal;
      {Calcium}
      c1 := k1c * sqrt(mg) / xmg;
      xsolve(c1, ca, xca, i2);
      change2(ca, mg, na, k, H, Al, xca, xmg, xna, xk, XH, XAl, soln, cecadj, sca, smg, sna, sk, SH, SAL);
      {Na}
      c2 := k2c * sqrt(ca) / (xca * 2);
      xsolve(c2, na, xna, i1);
      change2(ca, mg, na, k, H, Al, xca, xmg, xna, xk, XH, XAl, soln, cecadj, sca, smg, sna, sk, SH, SAL);
      {K}
      c6 := na / (xna * k6c);
      xsolve(c6, k, xk, i1);
      change2(ca, mg, na, k, H, Al, xca, xmg, xna, xk, XH, XAl, soln, cecadj, sca, smg, sna, sk, SH, SAL);
      {Mg}
      c4 := 2 * k / (xk * k4c);
      xsolve(c4, mg, xmg, i2);
      change2(ca, mg, na, k, H, Al, xca, xmg, xna, xk, XH, XAl, soln, cecadj, sca, smg, sna, sk, SH, SAL);
      {H}
      c14 := na / (xna / k14c);
      xsolve(c14, h, xh, i1);
      change2(ca, mg, na, k, H, Al, xca, xmg, xna, xk, XH, XAl, soln, cecadj, sca, smg, sna, sk, SH, SAL);
      {Al}
      c11 := na * K11c / xna;
      xsolve(c11, Al, xAl, i3);
      change2(ca, mg, na, k, H, Al, xca, xmg, xna, xk, XH, XAl, soln, cecadj, sca, smg, sna, sk, SH, SAL);

      diff := abs(xca - xcaold) + abs(xmg - xmgold) + abs(xk - xkold) + abs(xna - xnaold) +
        abs(xh - xhold) + abs(xal - xalold);
      if (diff < 0.01 * cecadj) and (i > 1) then
        escape := True;
    until (I = 5) or (escape);
    {  Exchangeable cations are converted from mol/l to eq/kg}
    XCA := XCA / ADJ2;
    XMG := XMG / ADJ2;
    XNA := XNA / ADJ1;
    XK := XK / ADJ1;
    XH := XH / ADJ1;
    XAL := XAL / ADJ3;
  end; {if ... then}
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Xchang2');
end;  {XCHANG2}

procedure xsolve(cconst: real; var s, x: real; nval: integer);
{calculate exchangeable cations using known selectivity coefficients
and solution concentrations for 1 cation pair (called from EQUILCHM2)}
var
  a, b, c, d, xacc, del, tot: real;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering Xsolve');
  del := (cconst * x - s) / (cconst + 1); {dummy initialisatie}
{  Let X:=exchangeable cation, S:=solution cation
   For monovalent cations, e.g. Na:
   1. XCa/XNa=KG.SQRT(Ca)/Na; so Na/XNa=KG*SQRT(Ca)/XCa=c2=cconst
   2. calc del in (Na+del)/(XNa-del)=cconst
   3. This gives del=(cconst*XNa-Na)/(cconst-+1)}
  if (nval = 1) then
    del := (cconst * x - s) / (cconst + 1)
  else if (nval = 2) then
  begin
    {  For divalent cations, e.g. Ca:
     1. XMg/XCa=KG*sqrt(MG)/SQRT(CA); so SQRT(Ca)/XCa=KG*SQRT(Mg)/XMg)=c1=cconst
     2. Calc del in SQRT(Ca+del)/(XCA-del)=cconst
     3. This gives a.del^2 + b*del + c = 0, solved below for del via discriminant formula}
    a := cconst * cconst;
    b := -2 * cconst * cconst * x - 1;
    c := cconst * cconst * x * x - s;
    del := (-b - sqrt(b * b - 4 * a * c)) / (2 * a);
  end
  else if (nval = 3) then
  begin
    {  For trivalent cations, thus Al:
    1. XNa/XAl=KG*Na/Al^0.33; so Al^0.33/XAl=KG*Na/XNa=c11=cconst
    2. (Al+del)^0.33/(XAl-del)=cconst
    3. This gives (Al=s; del; XAl=x; c11=cconst=c:
       c^3.x^3-s + del.(-3.c^3.x^2-1) + del^2(3.c^3.x) + del^3.(-c^3) = 0
       solved by application of a bisection method
       where del is allowed to vary in the range [-s;x] so that snew is in range [0;(s+x)]}
    a := cconst * cconst * cconst * x * x * x - s;
    b := -3 * cconst * cconst * cconst * x * x - 1;
    c := 3 * cconst * cconst * cconst * x;
    d := -cconst * cconst * cconst;
    xacc := (s + x) / 1000;
    del := rtbis(-s, x, xacc, a, b, c, d);
    if del = -999 then
      del := 0; {no change made because no bisection success}
  end; {if .. then}
  tot := s + x;
  s := max(0.0001 * tot, min(0.9999 * tot, s + del));
  x := tot - s;
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Xsolve');
end; {XSOLVE}

function fx(x, a, b, c, d: real): real;
{calculate cubic function value for x using input coefficients (called from RTBIS)}
begin
  fx := a + b * x + c * x * x + d * x * x * x;
end;

function rtbis(x1, x2, xacc, a, b, c, d: real): real;
{calculate root of cubic function by bisection (called from XSOLVE)}
(* Programs using routine RTBIS must externally define a function
fx(x:real):real which is to be analyzed for roots. *)
label
  99;
const
  jmax = 40;
var
  dx, f, fmid, xmid, rtb: real;
  j: integer;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering Rtbis');
  fmid := fx(x2, a, b, c, d);
  f := fx(x1, a, b, c, d);
  if ((f * fmid) >= 0.0) then
  begin
    {root not inside interval; assign result value -999 to be captured in calling routine}
    wwriteln('root not inside bisectioning interval; Initial value was kept.');
    rtb := -999;
    goto 99;
  end;
  if (f < 0.0) then
  begin
    rtb := x1;
    dx := x2 - x1;
  end
  else
  begin
    rtb := x2;
    dx := x1 - x2;
  end;
  for j := 1 to jmax do
  begin
    dx := dx * 0.5;
    xmid := rtb + dx;
    fmid := fx(xmid, a, b, c, d);
    if (fmid <= 0.0) then
      rtb := xmid;
    if ((abs(dx) < xacc) or (fmid = 0.0)) then
      goto 99;
  end;
  wwriteln('No convergence in bisectioning (more iterations needed); Last value was taken.');
  rtb := xmid;
  goto 99;
  99:
    rtbis := rtb;
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Rtbis');
end;

procedure change2(var ca, mg, na, k, h, al, xca, xmg, xna, xk, xh, xal, soln,
  cecadj, sca, smg, sna, sk, sh, sal: real);
{corrects exchangeable cations calculated with XSOLVE so that the total amount in solution and on CEC
 does match with the input values (mass balance correction; called from EQUILCHM2)}
var
  delsol, ratio, totsol, totex: real;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering Change2');
  delsol := 3 * al + 2 * (ca + mg) + na + k + h - soln;
  if (delsol >= 0) then
  begin
    totsol := 3 * al + 2 * (ca + mg) + na + k + h;
    ratio := 1 - (totsol - soln) / totsol;
    ca := ca * ratio;
    xca := sca - ca;
    mg := mg * ratio;
    xmg := smg - mg;
    na := na * ratio;
    xna := sna - na;
    k := k * ratio;
    xk := sk - k;
    h := h * ratio;
    xh := sh - h;
    al := al * ratio;
    xal := sal - al;
  end; {if ... then}
  if (delsol < 0) then
  begin
    totex := 3 * xal + 2 * (xca + xmg) + xna + xk + xh;
    ratio := 1 - (totex - cecadj) / totex;
    xca := xca * ratio;
    ca := sca - xca;
    xmg := xmg * ratio;
    mg := smg - xmg;
    xna := xna * ratio;
    na := sna - xna;
    xk := xk * ratio;
    k := sk - xk;
    xh := xh * ratio;
    h := sh - xh;
    xal := xal * ratio;
    al := sal - xal;
  end; {if ... then}
  {testpf2006
  if (ratio<0)or(ratio>1) then
  begin
    wwriteln('ratio buiten domein');
    ratio:=1;
  end; }
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Change2');
end; {CHANGE2}

procedure PRECIP(var SPS, PPTS, CATS, ANS, TCATS, TANS: real; var ITYPE: integer);
{calculates precipitation of calcite or gypsum based on concentrations and equilibrium constants (called from EQUILCHM2)}
{  The subroutine PRECIP uses cation and anion activities (ACAT & AAN),
C   concentrations (CAT & AN) and the solubility product (SP) to
C   determine if solid phase material (PPT) must dissolve or
C   precipitate to bring the system into chemical equilibrium.
C   It then calls the SINK subroutine to determine the quantity of ppt
C   to be dissolved or precipitated. XX is the amount by which PPT,
C   CAT, and AN must change to come into equilibrium.
    Errors removed by pf08
}
var
  SP, PPT, CAT, AN, TCAT, TAN, XX, x2, x3: double;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering Precip');
  SP := 0;
  PPT := 0;
  CAT := 0;
  AN := 0;
  TCAT := 0;
  TAN := 0;
  XX := 0;
  SP := SPS;
  PPT := PPTS;
  CAT := CATS;
  AN := ANS;
  TCAT := TCATS;
  TAN := TANS;
  XX := 0;
  X2 := 0;
  X3 := 0;
  SINK(CAT, AN, SP, X2, X3);
  {---------------------------------
  C  If the solution is under-saturated, i.e. dissolution:
  }
  if ((CAT * AN - SP) < 0) and (PPT > 0.00000001) then
  begin
    xx := MAX(MIN(0, x2), MIN(0, x3));
    if (PPT < XX) and (ITYPE = 2) then
      XX := PPT;
  {-------------------
  C  Add to cation and anion and subtract from the precipitate
  C   XX moles/l.
  }
    CAT := CAT + XX;
    TCAT := TCAT + XX;
    AN := AN + XX;
    TAN := TAN + XX;
    PPT := PPT - XX;
  end
{---------------------------------
C  If the solution is over-saturated, i.e. precipitation:
}
  else
  if (CAT * AN > SP) then
  begin
    xx := MIN(MAX(0, x2), MAX(0, x3));
    CAT := CAT - XX;
    TCAT := TCAT - XX;
    AN := AN - XX;
    TAN := TAN - XX;
    PPT := PPT + XX;
  end; {if ...then}
  SPS := SP;
  PPTS := PPT;
  CATS := CAT;
  ANS := AN;
  TCATS := TCAT;
  TANS := TAN;
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Precip');
end; {PRECIP}

procedure SINK(var CAT, AN, SP, X2, X3: double);
{uptake of cations Ca, Mg, Na, K and Al by plants via transpiration stream but forced by target plant element composition}
{
C********************************************************************
C  The subroutine SINK uses cation (CAT) and anion (AN) concentrations,
C   and the solubility product (SP) to calculate the cation and anion
C   concentration change (XX) due to solution or precipitation of solid
C   phase to bring the system into chemical equilibrium
C   for a given species. Solves: (CAT-XX)(AN-XX) = SP.
}
var
  X1: double;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering Sink');
  X1 := (AN + CAT) * (AN + CAT) - 4 * (CAT * AN - SP);  {discriminant}
  X2 := (CAT + AN - SQRT(X1)) / 2;              {solution 1}
  X3 := (CAT + AN + SQRT(X1)) / 2;              {solution 2}
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Sink');
end; {SINK}

function THETA(compnr: integer; P, G, TS, A, B, RL, TM, HM: real): real;
{calculate volumetric watercontent for current matric potential}
  {  RETENTIVITY AND HYDRAULIC CONDUCTIVITY FUNCTIONS CALCULATED}
  {  using either the Van Genuchten (1980) equations or}
  {   BY CAMPBELL'S (1974) EQUATION AND INCLUDING THE WET END }
  {   MODIFICATION DEVELOPED BY HUTSON (1984).}
var
  m, ttemp: real;
begin
  if VGNactiveLOCAL then
  begin {Van Genuchten}
    {if (TRACE = 1) then}
      singletracewriteln(' Entering Theta Van Genuchten');
    {Convert to the right units. (MM --> CM)}
    P := min(P, HM);
    P := -0.1 * P;
    m := 1 - (1 / VGNen[compnr]);
    THETA := min(VGNts[compnr],max(1.5*VGNtr[compnr],VGNtr[compnr] + (VGNts[compnr] - VGNtr[compnr]) /
      (power((1 + power((VGNal[compnr] * P), VGNen[compnr])), m))));
  end
  else
  begin {Campbell, minimize THETA to 1.5*ThetaRes, which is in most cases=0.01 unless it is input.}
        {This is to prevent THETA=0 at very low porosities such as in calcite accumulation horizons}
        {THETA=0 would give division by 0 in the chemistry routines}
        {Campbell based on h in mm}
    {if (TRACE = 1) then}
      singletracewriteln(' Entering Theta Campbell');
    if (MIN(HM, P) - G) <= 0 then
      THETA := min(TS,max(1.5*VGNtr[compnr], TS * power((MIN(HM, P) / A), (-1 / B))))
    else if (MIN(HM, P) - G) > 0 then
    begin
      ttemp := TS * (1 - RL / TS) * MIN(HM, P) * MIN(HM, P);
      THETA := min(TS,max(1.5*VGNtr[compnr], TS - ttemp / (A * A * power((RL / TS), (-2 * B)))));
    end;
  end;
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Theta');
end;

function POTL(thetaact, thetares, thetasat, alfa, n, m: real): real;
{calculate matric potential for current volumetric water content}
var
  S, h: real;
begin
  {future version should be based on Campbell equation}
  {Van Genuchten}
  {if (TRACE = 1) then}
    singletracewriteln(' Entering potl Van Genuchten');
  S := (thetaact - thetares) / (thetasat - thetares);
  if (S > 1) or (S < 0) then {test: should never be the case}
  begin
    if S > 1 then
      S := 1;
    if S < 0 then
      S := 0;
    raise EInOutError.Create('S<0 or >1 in POTL()');
  end;
  h := power(power(S, -1 / m) - 1, 1 / n) / alfa;
  if h < 0 then
    h := 0;
  {Convert to the right units. (cm --> mm)}
  POTL := h * -10;
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving potl');
end;

function HC(compnr: integer; P, G, RKS, A, B, TS, RL, RP, TM, HM, T: real): real;
{calculate hydraulic conductivity, if needed corrected for stoniness and frozen soil}
  { Arguments TM,HM added by PF2006 }
  { Pore interaction parameter (RP)}
var
  omega, choke, m, scalfacHC: real;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering HC');
  choke := 1;
  {correct HC by a scaling factor to accomodate stony soils.
   The scaling factor is scalfacHC=2*(1-f)/(2+f) as in Maxwell, 1873
   where f=volumetricstonepercentage/100
   This factor had best performance in a comparative study by Naseri et al., 2022 https://doi.org/10.5194/soil-8-99-2022}
   scalfacHC:=2*(1-stoneperc[compnr]/100)/(2+stoneperc[compnr]/100);
 {decrease conductivity when water table due to sea level is inside profile;
  this will cause a perched water table above this sea level depth}
  if (compnr - 2) * dlz = WaterTableDepth then
    choke := 0.01 {lo HC for comp of watertable}
  else if ((compnr - 1) * dlz >= pdepth) and (WaterTableDepth <= pdepth) then
    choke := 0.001; {lo HC for bottom comp as well to reduce drainage below impeding layer}
  if T < 0 then
    omega := 0.0001 * choke {hardly water flow in a frozen soil; omega accounts for this
                           effect; see Lundin, 1990}
  else
    omega := choke;
  if VGNactiveLOCAL then
  begin {Van Genuchten}
    {Convert to the right units. (MM --> CM)}
    P := min(P, 0);
    P := -0.1 * P;
    m := 1 - (1 / VGNen[compnr]);
    HC := scalfacHC*omega * VGNks[compnr] * sqr(power((1 + power((VGNal[compnr] * P), VGNen[compnr])), m) -
      power((VGNal[compnr] * P), (VGNen[compnr] - 1))) /
      power((1 + power((VGNal[compnr] * P), VGNen[compnr])), (m * (VGNel[compnr] + 2)));
  end
  else
  begin {Campbell}
    if (P <= G) then
      HC := scalfacHC*RKS * omega * power((A / P), (2 + (2 + RP) / B));
    if (P > G) then
      HC := scalfacHC*RKS * omega * power(power((THETA(compnr, P, G, TS, A, B, RL, TM, HM) / TS), B), (2 + (2 + RP) / B));
  end;
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving HC');
end;

function DWC(compnr: integer; PS, GS, TSS, BS, RLS, ASS, TMS, HMS: real): double;
{calculates differential water capacity (called from WATFLO)}
var
  P, G, TS, B, RL, A, HM, BIGTHETA, m, smalltheta: double;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering DWC');
  if VGNactiveLOCAL then
  begin {Van Genuchten}
    {Convert to the right units. (MM --> CM)}
    P := min(PS, HMS);
    P := -0.1 * P;
    smalltheta := THETA(compnr, PS, GS, TSS, ASS, BS, RLS, TMS, HMS);
    if ((smalltheta - VGNtr[compnr]) <= 0.001) or
      ((smalltheta - VGNts[compnr]) >= -0.001) then
    begin
      DWC := 0;
    end
    else
    begin
      m := 1 - 1 / VGNen[compnr];
      BIGTHETA := (smalltheta - VGNtr[compnr]) / (VGNts[compnr] - VGNtr[compnr]);
      DWC := (-m * (VGNts[compnr] - VGNtr[compnr]) / (1 - m)) * (BIGTHETA) * (1 - power(BIGTHETA, 1 / m)) / P;
    end;
  end
  else
  begin {Campbell}
    P := PS;
    G := GS;
    TS := TSS;
    B := BS;
    RL := RLS;
    A := ASS;
    HM := HMS;
    if (MIN(HM, P) - G) <= 0 then
      DWC := (-TS / (B * MIN(HM, P))) * power((MIN(HM, P) / A), (-1 / B))
    else
      DWC := -2 * TS * (1 - RL / TS) * MIN(HM, P) / (A * A * power((RL / TS), (-2 * B)));
  end;
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving DWC');
end;

procedure autoirrg(nchem: integer);
{void: calculates automatic irrigation (not used inSoilGen; called from MAINC2)}
var
  NODECHECK, IREPLENISH, I, M, IRR: integer;
  REPLENISH, FIELD_CAPACITY: real;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering Autoirrg');
  {  Automated irrigation (if set)}
  if (CH_IRR_FLAG = 1) then
  begin
    if (iautoirrg = 0) then
    begin
      if (ABS(NSUM - 999) <= 0.01) then
        system.writeln(f45, '  Years   Year  Month   Daym Irrigation');
      if (ABS(NSUM - 999) > 0.01) then
        system.writeln(f45, '   Days   Year  Month   Daym Irrigation');
      iautoirrg := 1;
    end;
    {  Nearest node to sensor placement depth}
    NODECHECK := MIN(KK, trunc(1 + CHECKDEP / DLZ));
    {  If the water potential at node NODECHECK has dropped below CRITIRRIG}
    if (H[NODECHECK] < CRITIRRIG) then
    begin
      {  Reset water deficit accumulator:}
      REPLENISH := 0;
      FIELD_CAPACITY := 0;
      {  Whole profile (change kk-1 (last node in profile) to another node}
      { if a shallower depth should be considered)}
      {  nearestnode to replenishment depth}
      if (ABS(REPLENISH_DEP - 999) > 1) then
        IREPLENISH := MIN(KK, trunc(1 + REPLENISH_DEP / DLZ))
      else
      if (ABS(REPLENISH_DEP - 999) <= 1) then
        IREPLENISH := MIN(KK, trunc(1 + RDZ / DLZ));
      {  Accumulate current profile water content and field capacity (mm)}
      for I := 2 to IREPLENISH do
      begin
        REPLENISH := REPLENISH + W[I] * DLZ;
        FIELD_CAPACITY := FIELD_CAPACITY + FC[I] * DLZ;
      end; {for}
      {  Ammount to irrigate (field capacity - current water content)}
      REPLENISH := MAX(0, FIELD_CAPACITY - REPLENISH);
      {  Increment the total number of infiltration events by 1}
      NIRRIG := NIRRIG + 1;
      {  Make the next irrigation the current replenishment, set to start}
      {  on the current day at time 0.05 day (change time if necessary)}
      IRRBEG := (DAY - 1) + 0.05;
      IRRIG := REPLENISH;
      {  Assume irrigation takes 0.2 day}
      IRATE := 5 * IRRIG;
      {  Include this for chemicals}
      for M := 1 to NCHEM do
        ISE[M] := SEIRR[M];
      {  Shift all infiltrations along to make room for this one }
      {  (there is a fictitious irrigation beyond the ending date)}
      for IRR := NIRRIG downto IRRNO do
      begin
        AIRRBG[IRR + 1] := MAX(DAY - 1 + 0.3, AIRRBG[IRR]);
        IRRIGA[IRR + 1] := IRRIGA[IRR];
        IRATEA[IRR + 1] := IRATEA[IRR];
        irrflg[irr + 1] := irrflg[irr];
        {  Include this for chemicals.  Shift rain-added chemical by one event}
        for M := 1 to NCHEM do
          ARSE[M, IRR + 1] := ARSE[M, IRR];
      end; {for irr}
      {       irrflg(irrno):=0}
      irrflg[irrno] := 3;
      {  Write to the *.IRR file to indicate when irrigation applied}
      if ABS(NSUM - 999) <= 0.01 then
        system.writeln(f45, (((cycno - 1) * cumt) + time) / 365.25: 8: 3, iy: 7,
          im: 7, daym: 7, replenish: 8: 1);
      if (ABS(NSUM - 999) > 0.01) then
        system.writeln(f45, ((cycno - 1) * cumt + time): 8: 1, iy: 7, im: 7, daym: 7, replenish: 8: 1);
      {      WRITE(45,*)DAY,DAYM,MONTH,YEAR,REPLENISH}
      system.writeln(f45, ' Irrigation of ', REPLENISH: 5: 1, ' mm on day', DAY: 5);
    end;
  end;
  {  End of automated irrigation}
  {--------------------------------}
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving AutoIrrg');
end;

procedure CADATE;
{calculate calender date from next  Julian day (called from MAINC2)}
{CALCULATE CALENDAR DAY AND MONTH FOR THE CURRENT DAY
{ DATA NEEDED ARE STARTING MONTH AND STARTING DAY. THESE ARE
  INPUT BY THE USER IN THE MAIN PROGRAM}
var
  MON, MONL: array[1..12] of integer;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering Cadate');
  MON[1] := 31;
  MON[2] := 28;
  MON[3] := 31;
  MON[4] := 30;
  MON[5] := 31;
  MON[6] := 30;
  MON[7] := 31;
  MON[8] := 31;
  MON[9] := 30;
  MON[10] := 31;
  MON[11] := 30;
  MON[12] := 31;
  MONL[1] := 31;
  MONL[2] := 29;
  MONL[3] := 31;
  MONL[4] := 30;
  MONL[5] := 31;
  MONL[6] := 30;
  MONL[7] := 31;
  MONL[8] := 31;
  MONL[9] := 30;
  MONL[10] := 31;
  MONL[11] := 30;
  MONL[12] := 31;
  DAYM := DAYM + 1;
  if (YEAR mod 4 = 0) and (DAYM > MONL[MONTH]) then
  begin
    DAYM := 1;
    MONTH := 1 + MONTH;
  end;

  if (YEAR mod 4 > 0) and (DAYM > MON[MONTH]) then
  begin
    DAYM := 1;
    MONTH := (MONTH + 1) mod 100;
  end;
  if (MONTH > 12) then
  begin
    MONTH := 1;
    YEAR := (YEAR + 1) mod 100;
  end;
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Cadate');
end;

procedure CALFC(var EFLUX: real);
{transport of water and chemicals according to a modified Addiscott (1977) tipping-bucket concept (called from MAINC2)}
{ LEACHING PROGRAM FOR PESTICIDES}
{ Based on}
{ ************************************************************}
{ program CALF by PETER NICHOLLS (IACR) and ALLAN WALKER (IHR)}

{ program for IBM compatible micro OCTOBER 1989}
{ Updated Version  January 1990}
{ ************************************************************}
{PM:      include 'parms.inc'
      include 'blocks.inc'
      include 'blockc.inc'}
{  MSE(mm): Flux of chemical between layers i-1 and i (mass/m2)}
{  MW[i]: Mobile water in layer i.  The positive difference between }
{   the water content and the 2-bar water content.}
{  SM(m,i): Amount of dissolved solute in the mobile phase (mass/litre)}
var
  mSE, dse, sm, kd: array[1..mm, 1..ll] of real;
  mw: rLL;
  TOTALW, EVAMAX, evamax1, fract, we1, we2, R13: real;
  seg, I, m: integer;
{**************************************************************
{ **************************************************************}
begin
  {  For mass balance check within this subroutine:}
  TOTALW := 0;
  for I := 2 to KK - 1 do
  begin
    TOTALW := TOTALW + W[i] * dlz;
    for m := 1 to nchem do
    begin
      dse[m, i] := se[m, i] * w[i] * dlz;
      kd[m, i] := 0;
    end;
  end;
  {  Calculate amount by which ET is reduced owing to by rain.}
  {  ET time is reduced by this amount.}
  {      if(ihead=1)then}
  {      TIMEA:=AMAX1(0.,mod(irrbeg,1.)-.3)  }
  {      TIMED:=AMin1(.5,mod(irrbeg,1.)+head/irate-.3)  }
  {      FRACT:=(COS(2.*PI*TIMEA)-COS(2.*PI*TIMED))/2.  }
  {      else}
  fract := 0;
  {      endif}
  {----------------------------}
  {  EVAPOTRANSPIRATION}

  {  TRANSPIRATION}
  TRANA := 0;
  for I := 2 to KK - 1 do
  begin
    a[i] := a[I] * (1 - fract);
    {  A[i] is daily transpiration rate (theta/day).  Transpiration}
    {   occurs during 0.5 of a day only}
    if (W[I] > WP[I]) then
      {  Reduce water content by transpiration, but not to below wilting }
      {   point}
    begin
      WNEW := W[I] - MIN(A[I] * 0.5, W[I] - WP[I]);
      A[I] := MIN(A[I], (W[I] - WP[I]) / 0.5);
      W[I] := WNEW;
    end
    else
      A[I] := 0;
    {  Accumulate profile transpiration}
    TRANA := TRANA + A[i] * 0.5 * DLZ;
  end;
  {----------------------------}

  {  EVAPORATION}
  {  Minimum water content is HLOW, ER is potential evaporation rate for}
  {   day}
  {  Evaporation allowed only from top segment.  EVAMAX is maximum.}
  EVAMAX := (W[2] - THETA(2, HLOW, RG[2], TS[2], AEV[2], BCAM[2], L[2], TM[2], HM[2])) * DLZ;
  evamax1 := -B[1] * (-3000 * 102.2 - g[2]) / DLZ;
  EVAMAX := MAX(0, min(EVAMAX, evamax1));
  {      er:=er*(1.-fract)}
  WNEW := W[2] - MIN(EVAMAX, ER * 0.5) / DLZ;
  {  Loss of water to evaporation is surface flux}
  EFLUX := (W[2] - WNEW) * dlz;
  {  New water content in upper layer}
  W[2] := WNEW;
  mw[2] := max(0, w[2] - d[2]);
  {--------------------------------------------------------------------}
  {  In this version local chemical equilibrium is attained each day.}
  {  For each segment, for each chemical:}
  for I := 2 to KK do
  begin
    for M := 1 to NCHEM do
    begin
      SE[m, i] := DSE[m, i] / (DLZ * (KD[m, i] * RHO[i] + W[i]));

      {----------------------------}
      {  Mobile phase water and chemical:}
      {   The concentration in the mobile phase SM[m,i] (above -200 kPa)}
      {   equals the equilibrium concentration SE[m,i].}
      if (w[i] > d[i]) then
        SM[m, i] := SE[m, i];
      if (w[i] <= d[i]) then
        SM[m, i] := 0;
    end;
    {  Mobile water content (theta) is that water retained above -200 kPa}
    mw[i] := max(0, w[i] - d[i]);
  end;
  {---------------------------------}
  {  RAIN or IRRIGATION}
  {  Add rain to top layer if necessary and mix added herbicide}
  {  with that already in the mobile fraction}
  if (IHEAD = 1) then
  begin
    {  Increase water content of upper segment (node 2) by an amount}
    {   equivalent to rain}
    WNEW := W[2] + HEAD / DLZ;

    for M := 1 to NCHEM do
    begin
      {  Add chemical dissolved in the rain or irrigation water to total}
      {   segment chemical content (mg/m2)}
      DSE[m, 2] := DSE[m, 2] + SS[m, 1] * HEAD;

      {  If original water content above -200 kPa threshold then added}
      {   chemical initially goes into mobile water only (with no sorption)}
      if (W[2] >= D[2]) then
        SM[m, 2] := (SS[m, 1] * (WNEW - W[2]) + SE[m, 2] * (W[2] - D[2])) / (WNEW - D[2])
      else
      begin
        {  If original water content below threshold then adjust both immobile}
        {   (SE) and, if necessary, mobile (SM) concentrations.}
        SE[m, 2] := (SS[m, 1] * (MIN(D[2], WNEW) - W[2]) + SE[m, 2] * W[2]) / MIN(WNEW, D[2]);
        if (WNEW > D[2]) then
          SM[m, 2] := SS[m, 1] * MAX(WNEW - D[2], 0) / (WNEW - D[2]);
        if (WNEW <= D[2]) then
          SM[m, 2] := 0;
      end;
    end; {for M:=1 ...}

    {  Adjust water content of top layer }
    W[2] := WNEW;
    MW[2] := W[2] - D[2];
  end; {if IHEAD=1}
  {  Net surface flux of water (positive downwards)}
  FLUX[1] := HEAD - EFLUX;
  {----------------------------}
  {  Mass balance check}
  {      CURRENTW:=0.}
  {      DO 20 I:=2,KK-1}
  {      CURRENTW:=CURRENTW+W[i]*dlz}
  {  20  CONTINUE  }
  {      WMASSBAL:=FLUX[1]+TOTALW-CURRENTW-TRANA}
  {  WATER AND SOLUTE FLUX THROUGH THE PROFILE}

  {  For water set lowest layer to be same as kk-1}
  W[kk] := W[kk - 1];
  FC[kk] := FC[kk - 1];
  D[kk] := D[kk - 1];
  RHO[kk] := RHO[kk - 1];
  {  Set lower chemical boundary to zero concentration.}
  for M := 1 to NCHEM do
  begin
    SE[M, KK] := 0;
    SM[M, KK] := 0;
  end;
  {--------------------------------------------------------------------}
  {--------------------------------------------------------------------}
  {  WATER FLOW BETWEEN CURRENT AND UPPER LAYERS}
  {   (At this stage fluxes are in theta units (except for the surface)}
  for I := 3 to KK do {DO 140 ftncode}
  begin
    {-----------------------------}
    { i) If field capacity is exceeded:}

    {  In this case water flux (FLUX[i-1]) is water held above field}
    {   capacity.  Flux i refers to flux between i and i+1. }
    if (W[i - 1] > FC[i - 1]) then
      FLUX[i - 1] := W[i - 1] - FC[i - 1];
    {-----------------------------}
    { ii) If field capacity is not exceeded:}

    if (W[i - 1] <= FC[i - 1]) then
    begin
      {  Estimate equilibrium water content between upper and current layer.}
      {   Current matric potentials are}
      H[i - 1] := POTL(W[i - 1], VGNtr[i - 1], VGNts[i - 1], VGNal[i - 1], VGNen[i - 1], VGNem[i - 1]);
      H[i] := POTL(W[i], VGNtr[i], VGNts[i], VGNal[i], VGNen[i], VGNem[i]);

      {  If these two layers were in equilibrium then their water}
      {   contents would be WE1 and WE2 (the equilibrium water contents)}
      WATEQ(I, WE1, WE2);

      {  Water content change in upper [i-1] layer is half of the difference}
      {   between current and equilibrium water content.  Flux is positive}
      {   downwards.}
      FLUX[i - 1] := (W[i - 1] - WE1) / 2;
      if (flux[i - 1] > 0) then
        FLUX[i - 1] := W[i - 1] - WE1;

      {   For a free-draining profile, upward movement is prohibited at the }
      {    lowest node.}
      if (I = KK) then
        FLUX[i - 1] := MAX(0, FLUX[i - 1]);
      {  The lower boundary flux can be approximated as unit gradient, using current K value}
      {    IF (I=KK) then FLUX[i-1]:=MAX(0,b[i-1])}
    end; {if}
    {--------------------------------------------------------------------}
    {  CHEMICAL TRANSPORT BETWEEN UPPER AND CURRENT SEGMENTS}
    {   (MSE is mass/dm3 of chemical moved between layers in one day.)}
    {-----------------------------}
    { i) If water is going to move downwards. (FLUX[i-1] is positive.}
    if (FLUX[i - 1] > 0) then
    begin
      {  Amount of chemical transported into i from i-1 is concentration}
      {   in i-1 times the water flux }
      for M := 1 to NCHEM do
      begin
        {  Move mobile water from layer above (mass/dm3)}
        MSE[m, i] := MIN(FLUX[i - 1], MW[i - 1]) * SM[M, I - 1];
        {  If insufficient mobile available, move balance as immobile}
        if (se[m, i] < 1e-20) then
          se[m, i] := 0; {was:=0 but this gives numerical problems}
        MSE[m, i] := MSE[m, i] + MAX(0, FLUX[i - 1] - MW[i - 1]) * SE[M, I - 1];
      end;
    end; {if}
    {-----------------------------}
    {  ii) If upper layer is smaller water content then flow will be upwards}
    {   (at this stage SM[i] will always equal SE[i]).}
    if (FLUX[i - 1] <= 0) then
      for M := 1 to NCHEM do
        MSE[m, i] := FLUX[i - 1] * SM[m, i];
    {--------------------------------------------------------------------}
    {  ADJUST WATER AND CHEMICAL IN UPPER LAYER [i-1]}

    {  New water content and mobile water content after losing or gaining }
    {   flux.}
    W[i - 1] := W[i - 1] - FLUX[i - 1];
    MW[i - 1] := MAX(0, W[i - 1] - D[i - 1]);
    {  Total amount of chemical in upper layer (mg/m2) after losing or}
    {   gaining MSE mg/dm3 chemical.}
    for M := 1 to NCHEM do
    begin
      DSE[M, I - 1] := DSE[M, I - 1] - MSE[m, i] * DLZ;
      {----------------------------}
      {  Partition chemical:}

      {   i) Solution concentration}
      SE[M, I - 1] := DSE[M, I - 1] / (DLZ * (KD[M, I - 1] * RHO[i - 1] + W[i - 1]));
      {----------------------------}
    end;
    {---------------------------------}
    {  ADJUST WATER AND CHEMICAL IN CURRENT LAYER [i]}

    {  New water content after losing or gaining flux[i-1] water}
    WNEW := W[i] + FLUX[i - 1];
    {  Total amount of chemical in i after losing or gaining}
    {  MSE[i]}

    for  M := 1 to NCHEM do
    begin
      DSE[m, i] := DSE[m, i] + MSE[m, i] * DLZ;

      {  Upward flow does not change solution concentrations, but may }
      {   eliminate mobile concentrations}
      if (FLUX[i - 1] <= 0) then
      begin
        if (WNEW <= D[i]) then
          SM[m, i] := 0;
        if (WNEW > D[i]) then
          SM[m, i] := SE[m, i];
      end;

      {  When flow is down then added chemical goes into immobile until water}
      {   content reaches the threshold, and then into the mobile phase. }
      {   The two phases do not equilibrate at this stage.  }

      {  If initial water content above threshold: }
      if (FLUX[i - 1] > 0) then
      begin
        if (W[i] >= D[i]) then
          SM[m, i] := (MSE[m, i] + SE[m, i] * (W[i] - D[i])) / (WNEW - D[i])
        else
          {  If water content below threshold then:}
        begin
          SE[m, i] := (MSE[m, i] * (MIN(D[i], WNEW) - W[i]) / FLUX[i - 1] + SE[m, i] * W[i]) / MIN(WNEW, D[i]);
          if (se[m, i] < 1.e-20) then
            se[m, i] := 0;
          if (WNEW > D[i]) then
            SM[m, i] := (MSE[m, i] * MAX(WNEW - D[i], 0) / FLUX[i - 1]) / (WNEW - D[i]);
          if (WNEW <= D[i]) then
            SM[m, i] := 0;
        end; {if then else}
      end; {IF}

    end; {554 continue}
    {-----------------------------}
    { New water content for layer i}
    W[i] := WNEW;
    MW[i] := MAX(0, W[i] - D[i]);
  end; {DO 140  CONTINUE}
  {--------------------------------------------------------------------}
  {  Calculate new potentials, etc.}
  flxden[1] := flux[1];
  for I := 2 to KK - 1 do
  begin
    R13 := MIN(TS[i], W[i]);
    H[i] := POTL(R13, VGNtr[i], VGNts[i], VGNal[i], VGNen[i], VGNem[i]);
    {  Convert water fluxes to mm from theta units.}
    FLUX[i] := FLUX[i] * DLZ;
    flxden[i] := flux[i];
    {      if(i=3)write(*,*)'flux3',flux(3)}
  end;
  {-----------------------------}
  {  Chemical leaching}
  {  Solute flux across summary segment boundaries}
  for SEG := 1 to 4 do
  begin
    I := ND[SEG];
    for M := 1 to NCHEM do
      FSE[M, SEG] := FSE[M, SEG] + mse[m, i + 1] * dlz;
  end;
  for M := 1 to NCHEM do
    LSE[M] := MSE[M, KK] * DLZ;
  {-----------------------------}
end;

procedure wateq(i: integer; var we1, we2: real);
{calculate equilibrium water contents (called from CALFC)}
{PM      include 'parms.inc'
      include 'blocks.inc'}
var
  iequil, t1, t2, phigh, plow, pav, p1new, p2new: real;
  niter: integer;
begin
  IEQUIL := 0;
  T1 := W[i - 1];
  T2 := W[i];
  PHIGH := MAX(H[i - 1], H[i]);
  PLOW := MIN(H[i - 1], H[i]);
  NITER := 0;
  {---------------------------}
  while (niter < 20) and (iequil = 0) do
  begin
    {100 continue}
    NITER := NITER + 1;
    PAV := (PHIGH + PLOW) / 2;
    P1NEW := PAV - DLZ;
    P2NEW := PAV;
    WE1 := THETA(i - 1, P1NEW, RG[i - 1], TS[i - 1], AEV[i - 1], BCAM[i - 1], L[i - 1], TM[i - 1], HM[i - 1]);
    WE2 := THETA(i, P2NEW, RG[i], TS[i], AEV[i], BCAM[i], L[i], TM[i], HM[i]);
    if (WE1 + WE2 > T1 + T2) then
      PHIGH := PAV;
    if (WE1 + WE2 < T1 + T2) then
      PLOW := PAV;
    {-----------------------------}
    if (ABS(WE1 + WE2 - T1 - T2) < 1E-4) then
    begin
      IEQUIL := 1;
      WE2 := T1 + T2 - WE1;
    end;
    {--------------}
  end; {if(iequil=0) AND (niter<20)go to 100  }
  {----------------------------}
end;

procedure CELLC;
{set new lower boundary concentrations for solute phase}
{PM      INCLUDE 'PARMS.inc'
      INCLUDE 'BLOCKS.inc'
      INCLUDE 'BLOCKC.inc'}
{----------------------------}
{  This subroutine mixes leached pesticide with solution in a hypo-}
{   thetical mixing cell below the profile, to calculate the con-}
{   centration of water moving into the profile from below.  This}
{   can only happen if a water table is present.}
var
  m: integer;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering Cellc');
  for M := 1 to NCHEM do    {DO 10 M:=1,NCHEM}
  begin

    {  First time step: calculate mass of chemical in the mixing cell}
    if (N1PRT = 0) then
      CMASS[M] := bbse[M] * CELLW;

    {   For downward water flux (positive FLUX(KK-1), add leached chemical}
    {   to mixing cell, and subtract that flowing from the lower boundary}
    {   of the mixing cell.  ((SS(M,KK-1)+SE(M,KK-1))*.5 is mean solute concentration}
    {   leaching from profile, which, multiplied by water flux, gives solute mass.)}

    if (FLUX[KK - 1] >= 0) then
      CMASS[m] := CMASS[m] + (SS[M, KK - 1] + SE[M, KK - 1]) * 0.5 * FLUX[KK - 1] -
        (cmass[m] / cellw) * flux[kk - 1]

    {  For upward water flux, reduce mass of chemical in mixing cell by}
    {  upward flowing solute, and add that flowing into the lower mixing cell }
    {  boundary (assumed to be at the initial mixing cell concenytration BBSE[m]}
    {  (remember that upward fluxes are negative)}
    else
    if (FLUX[KK - 1] < 0) then
      CMASS[m] := max(CMASS[m] + CMASS[m] / CELLW * FLUX[KK - 1], 0) - flux[kk - 1] * bbse[m];
    {  Set new mixing cell concentration}
    SS[M, KK] := max(0, CMASS[m] / CELLW);

  end;{do 10  CONTINUE}
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Cellc');
end;

procedure cultc;
{void: cultivation (tillage). Not used in SoilGen but replaced by a bioturbation routine.}
{this procedure is no longer used; cultivation is interpreted as complete mixing
 over the cultivated depth; thus the bioturbation routine can be used with
 special settings: untGUI.frmmain.Turbation(0,cultdept,cultdept,1,1,1)
 Here the values 1 may be less if defined so in the untGUI routine for plowing events}
var
  cultSE: rMM;
  cultwat, cultdept: real;
  cultint, i, m: integer;
  {*********************************************************************}
begin
    {if (TRACE = 1) then}
    singletracewriteln(' Entering CultC');
  {  Cultivation if on this day}
  {  For now, merely mix chemical species through specified depth}
  {   Calculate total chemical in cultivation depth}

  {   Cultivation depth nodes}
  cultint := trunc(cultdep[cultno] / dlz) + 1;
  cultwat := 0;
  cultdept := 0;

  {  Total cultivation depth to be used in model}
  for i := 2 to cultint do
    if (i > 1) then
      cultdept := cultdept + dlz;

  for m := 1 to nchem do
  begin
    cultse[m] := sfer[m];

    for i := 2 to cultint do
      {      VOLUME*(SE[M,i]*W[i]+XSE[M,i]*RHO[i]/VAL[m])+PDSE[m] }
      if (m = 16) or (m = 17) then
        cultse[m] := cultse[m] + DLZ * (SE[M, i] * W[i]) + PDSE[m]
      else
        cultse[m] := cultse[m] + DLZ * (SE[M, i] * W[i] + XSE[M, i] * RHO[i] / VAL[m]) + PDSE[m];
  end;
  for m := 1 to nchem do
    for i := 2 to cultint do
    begin
      SEFeII:=SE[14,i]; SEFeIII:=SE[13,i]; {FeII and FeIII conc for call to Chem2}
      case typechem of
        1:begin end;
        2:
          chem2(CALCITE[i], GYP[i], Gibbsitepool[i], PH[i], RHO[i], SE[12, i],
            SE[1, I], SE[2, I], SE[3, I], SE[4, I], SE[5, I], SE[6, I], SE[7, I], SE[8, I],
            SE[9, I], SE[10, I], SE[11, I], W[i], EC[i], SAR[i],
            XSE[1, I], XSE[2, I], XSE[3, I], XSE[4, I], XSE[10, i], XSE[12, i], CEC[i],
            ESP[i], SC1[i], SC2[i], SC3[i], SC4[i], SC5[i], SC6[i],
            SC7[i], SC8[i], SC9[i], SC10[i], SC11[i], SC12[i], SC13[i], SC14[i], SC15[i],
            DELGIBB[i], DELGYP[i], DECALCITE[i], PCO2[i], NN1,
            SP1, SP2, SALT[i], trace, DELCO2[i], n1prt, tem[i]);
        3: begin end;
      end;
      {   Chemical concentrations at top nodes}
      SE[M, 1] := 0;
      SS[M, 1] := 0;
    end;
  calcphlab(RHO, PH, AW, PHLAB, DLZ);{calculate pH-H2O in 1:5 dilution as in lab protocol}
  {*********************************************************************}
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Cultc');
end;

procedure ETRANS;
{Evapotranspiration during the length of the time increment}
{PM:      INCLUDE 'PARMS.inc'
      INCLUDE 'BLOCKS.inc'}
{----------------------------}

{  THIS ROUTINE CALCULATES i) A SURFACE EVAPORATION RATE (mm/d)     }
{                         ii) TRANSPIRATION }
{                         FOR THE TIME PERIOD DELT (mm water) }
{  RATE OF WATER LOSS IS ASSUMED TO VARY SINUSOIDALLY BETWEEN }
{   08H00 AND 20H00 WITH A MAXIMUM AT 14H00 }
{------------------------ }
{  FRACT:=FRACTION OF WATER LOST DURING DELT ASSUMING THAT     }
{     RATE OF LOSS:=(MAX RATE)*(2*PI*TIME) WHERE 0<TIME<0.5    }
{  TIMDAY:=TIME ELAPSED DURING PRESENT DAY as fraction of a day}
{  DELT:=TIME PERIOD days  }
{  DLYPAN:=PAN EVAPORATION mm/day   }
{  WFDD:=MAXIMUM EVAPORATION RATE FOR PRESENT SURFACE WATER    }
{     CONTENT mm/day (PASSED FROM PREVIOUS EXECUTION OF WATFLO)     }
{  ER:=ACTUAL EVAPORATION RATE RETURNED TO NEXT EXECUTION OF   }
{     WATFLO  }
{  TRANP:=TRANSPIRATION DURING DELT mm water }
{  PEVAP:=EVAPORATION DURING TIME PERIOD mm water  }
{  DLYEP:=POTENTIAL EVAPORATION FOR DAY mm (PASSED FROM SINK)    }
{  DLYTP:=POTENTIAL TRANSPIRATION FOR DAY mm (PASSED FROM SINK) }
{------------------------ }
var
  timea, timed, fract, pet, deficit: real;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering Etrans');
  {------------------------ }
  {  Time is adjusted so that 0.3 day is 0 and 0.8 day is 0.5. Outside the  }
  {   times time is 0.}
  if (TIMDAY <= 0.3) or (TIMDAY > 0.8) then
  begin
    TIMEA := 0;
    TIMED := 0;
  end
  else
  begin
    TIMEA := MAX(0, TIMDAY - DELT - 0.3);
    TIMED := MIN(0.5, TIMDAY - 0.3);
  end;
  FRACT := (COS(2 * PI * TIMEA) - COS(2 * PI * TIMED)) / 2;
  {------------------------      }
  {  Potential evaporation, transpiration and evapotranspiration}
  {   for DELT (mm)     }
  PEVAP := DLYEP * FRACT;
  TRANP := DLYTP * FRACT;
  PET := DLYPAN * FRACT;
  {------------------------      }
  {  Desired actual evaporation rate is adjusted to be the minimum of potential}
  {   rate and the maximum possible negative surface flux density    }
  ER := max(0, MIN(PEVAP / DELT, WFDD));
  {------------------------      }
  {  If evaporation is less than potential then potential transpiration is }
  {    increased by the evaporation deficit to a maximum of  }
  {    (potential transpiration * AK2)  }
  if (WFDD * DELT < PEVAP) then
  begin
    DEFICIT := max(0, PEVAP - WFDD * DELT);
    TRANP := min(TRANP * AK2, TRANP + deficit);
  end;
  {--------------------------    }
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Etrans');
end;

procedure GROWTH(var KK: integer; var RDFDAY: integer; var PD: real;
  DAY: integer; var RDZ, DLZ: real; var RDF: rLL;
  var CRPCOV: real; var ESTART, ESTOP, DAYHAR: integer;
  var AK1: real; var KILPLT: integer; var TRL: real;
  var PDAY, igrowth: integer; var RDEP: real);
{calculates root growth for corn, by growing day, scaled to max rooting depth (called from MAINC2)}
{PM:  INCLUDE 'PARMS.INC'}
{---------------------------------------------------------------------}
var
  rz, rl: rLL;
  roots_mature, skip: boolean;

  {----------------------------}

  {  THIS ROUTINE CALCULATES I) CORN ROOT GROWTH.  METHOD IS TAKEN FROM TH}
  {  EPA PUBLICATION, 600/3-78-029, SIMULATION OF NITROGEN MOVEMENT,  }
  {  TRANSFORMATION, AND UPTAKE IN PLANT ROOT ZONE, DAVIDSON ET AL. (1978)}
  {  PAGE 31.  RDF IS DETERMINED BY DIVIDING THE ROOT LENGTH PER DEPTH}
  {  INCREMENT BY THE TOTAL ROOT LENGTH.  }
  {                          II) CROP COVER. METHOD FOLLOWS THAT OF   }
  {  TILLOTSON ET AL. EQ.115  }
  {------------------------   }
  {  KK:=NUMBER OF DEPTH BOUNDARIES}
  {  RDZ:=DEPTH TO THE BOTTOM OF ROOT ZONE (MM)}
  {  PD:=PLANTING DENSITY (PLANTS/M**2 SOIL SURFACE)   }
  {  R:=ROOT LENGTH DENSITY (M ROOT/PLANT/MM DEPTH)}
  {  RZ:=ADJUSTED ROOT DENSITY (M ROOT/DM**3 BULK SOIL)}
  {  RDF:=PROPORTION OF TOTAL ACTIVE ROOTS }
  {  RDFDAY:=NUMBER OF DAYS TO REACH ROOT PROFILE MATURITY }
  {  RMAX:=MAXIMUM ROOT LENGTH DENSITY AT SOIL SURFACE (M ROOT/PLANT/  }
  {    MM DEPTH)  }
  {  TIM:=NUMBER OF DAYS SINCE PLANTING}
  {  TIME:=TIME IN DAYS}
  {  TRL:=TOTAL ROOT LENGTH (M)}
  {  ZHALF:=SOIL DEPTH AT WHICH ROOT LENGTH DENSITY IS ONE-HALF THE VALUE  }
  {    OF RMAX (MM)   }
  {   PLANTS PLANTED AT TIME:=PDAY }
  {---------------------  }
var
  i: integer;
  TIM, RMAX, BBREAK, ZHALF, BETA, DEPTH, A, B, R: real;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering Growth');
  skip := False;
  if (KILPLT = 0) then
    skip := True; {GO TO 50   }
  if (AK1 = 0) then
    skip := True; {GO TO 50}
  {---------------------  }
  {  IF CONSTANT ROOT PROFILE:}
  if (igrowth = 1) then
  begin
    for I := 1 to KK - 1 do
    begin
      RL[i] := TRL * RDF[i] * PD;
      RZ[i] := TRL * RDF[i] * PD / DLZ;
    end;
    CRPCOV := AK1;
  end;
  if (igrowth = 1) then
    skip := True; {GO TO 50}
  {-------------------------  }
  {  IF CORN GROWTH SIMULATED:}
  if (DAY <= PDAY) then
    skip := True; {GO TO 50}
  if not (skip) then
  begin
    roots_mature := (DAY > RDFDAY);
    if not (roots_mature) then
    begin
      TRL := 0;
      for I := 1 to KK do
      begin
        RDF[i] := 0;
        RZ[i] := 0;
        RL[i] := 0;
      end;
      {  ROOT GROWTH AND ROOT DENSITY SIMULATION  }
      TIM := (MIN(RDFDAY, DAY) - PDAY) * 113 / (RDFDAY - PDAY + 1);
      RMAX := 0.001 * (-0.05253 * power(TIM, 2) + 24.6667 * TIM - 120.0);
      BBREAK := (24 / 113) * (RDFDAY - PDAY + 1);
      if (TIM <= BBREAK) then
        RDZ := 10 * (0.06 * power(TIM, 2) - 0.1 * TIM);
      if (TIM > BBREAK) then
        RDZ := (10 * (-0.0112 * power(TIM, 2) + 2.90 * TIM - 15.0));
      {  Assume planting depth 50 mm.}
      RDZ := RDEP * RDZ + 50;
      ZHALF := RDZ * (-0.0001854 * power(TIM, 2) + 0.022 * TIM - 0.102);
      if (ZHALF < 0.4 * RDZ) then
        ZHALF := 0.4 * RDZ;
      BETA := (LN(2 * COS((PI * ZHALF) / (2 * RDZ)))) / power(ZHALF, 2);
      for I := 2 to KK - 1 do
      begin
        DEPTH := (i - 1.5) * DLZ;
        if (DEPTH < RDZ) then
        begin
          A := EXP(-BETA * power(DEPTH, 2));
          B := COS((PI * DEPTH) / (2 * RDZ));
          R := RMAX * A * B;
          RZ[i] := R * PD;
          RL[i] := RZ[i] * DLZ;
          TRL := RL[i] + TRL;
        end;
      end;
      if (TRL > 0) then
        for I := 2 to KK - 1 do
          RDF[i] := RL[i] / TRL;
      {-----------------------}
      {  CROP COVER CALCULATED AS A FRACTION BETWEEN 0 AND THE}
      {   MAXIMUM ALLOWED (AK1)   }
    end; {if roots_mature: hier verder}
    IF ((DAY<=ESTART))or((DAY>=DAYHAR)) then  {or((DAY>=DAYHAR) added 19 - 6 - 2017}
      CRPCOV := 0
    else
    begin
      TIM := (DAY - ESTART);

      CRPCOV := AK1 / (1 + EXP(6 - 12 * TIM / (ESTOP - ESTART)));
      {---------------------------------}
      {  Add these lines for a Tp decrease between maturity and harvest}

      {      if(day>=estop.and.day<=dayhar)then}
      {      growing_period:=estop-estart}
      {      crpcov:=ak1-(ak1-0.0)}
      {     #*((tim-growing_season)/(dayhar-estart-growing_season))**1.5}
      {      endif}
    end;
    {-----------------------}
    {  For vineyard}
    {      fgs:=0.}
    {      ELSE  }
    {      TIM:=FLOAT(DAY-ESTART) }
    {      fgs:=tim/float(dayhar-estart)}
    {      if(fgs<.4)crpcov:=(0.015+2.5*fgs)*ak1}
    {      if(fgs<=.7.and.fgs>=4)crpcov:=ak1}
    {      if(fgs>.7.and.fgs<=1.)crpcov:=(3.1775-3.15*fgs)*ak1}
    {      if(fgs>1.)crpcov:=0.}
    {      ENDIF }
    {-------------------------  }
    {-------------------------  }
  end; {ftn label 50}
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Growth');
end;

procedure HEAT(var deltheat: real);
{calculate heat flow during 10% of a day}
{ This version is designed to be called at DELTHEAT day intervals.}
{PM      INCLUDE 'PARMS.INC'
      INCLUDE 'BLOCKS.INC'}
{--------------------------------------------------------------------}
{     THIS SUBROUTINE SOLVES THE HEAT FLOW EQUATION.}

{     TAKA=K VALUES USED IN CALCULATING ALLAM}
{     ALAMA=THERMAL CONDUCTIVITY OF CONTINUOUS MEDIUM (CAL/CM SEC C)}
{     ALLAM=THERMAL CONDUCTIVITY AT HALFDEPTH, AND HALF TIME STEP (CAL/}
{       CM HR C)}
{     ALAMO=THERMAL CONDUCTIVITY OF SOIL PARTICLES (CAL/CM SEC C)}
{     ALAMV=THERMAL CONDUCTIVITY OF VAPOR PHASE (CAL/CM SEC C)}
{     GEOMOM=GEOMETRY OF ORGANIC MATTER PARTICLES}
{     GEOMA=GEOMETRY OF AIR PARTICLES}
{     GEOMOS=GEOMETRY OF OTHER SOLID PARTICLES}
{     GEOMQ=GEOMETRY OF QUARTZ PARTICLES}
{     TEM=TEMPERATURE, CURRENT TIME STEP (C)}

{  Unlike rest of SoilGen, this routine uses cgs units (as in LEACHM).}
{  Convert node spacing to cm.}
var
  beta, allam, tw, a1, b1, c1, d1, aa, bb: array[1..LL] of real;
  taka: array[1..4] of real;
  geoma: array[1..3] of real;
  f1, g1: array[1..LL] of double;
  DLZCM, ALAM4, ALAMA, ALAMV, TAK1, TAK2, TAK3, TAK4, A12, A13, FX,
  prevt, preva, preve, tmean, amean, range, tlow, thigh: real;
  I, J, JJ, IL, M, M1: integer;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering Heat');
  {store old temperatures}
  for i := 2 to kk do
    tem_old[i] := tem[i];

  DLZCM := DLZ / 10;
  ALAM4 := ALAMO[4];
  for I := 2 to kk do
  begin      {DO 10 I:=2,KK}
    ALAMO[4] := ALAM4;
    TW[i] := (W[i] + Y[i] + W[I - 1] + Y[I - 1]) / 4;
    if (I = 2) then
      TW[i] := (W[i] + Y[i]) / 2;
    if (I = KK) then
      TW[i] := TW[KK - 1];
    BETA[i] := (0.2 * RHO[i]) + TW[i];

    {     DETERMINING CONTINUOUS MEDIUM FOR DEPTH INCREMENT}

    if (W[i] >= 0.20) then
      ALAMA := ALAMO[5];
    if (W[i] < 0.20) then
      ALAMA := ALAMO[4];

    {     DETERMINING K VALUES USED IN CALCULATING THE THERMAL CONDUCTIVITY}
    {     OF THE SOIL.  (De Vries model)}

    TAK1 := 0;
    for J := 1 to 3 do
      TAK1 := (1 / (1 + ((ALAMO[1] / ALAMA) - 1) * GEOMQ[J])) + TAK1;
    TAKA[1] := TAK1 / 3;
    TAK2 := 0;
    for JJ := 1 to 3 do
      TAK2 := (1 / (1 + ((ALAMO[2] / ALAMA) - 1) * GEOMOM[JJ])) + TAK2;
    TAKA[2] := TAK2 / 3;
    TAK3 := 0;
    for IL := 1 to 3 do
      TAK3 := (1 / (1 + ((ALAMO[3] / ALAMA) - 1) * GEOMOS[IL])) + TAK3;
    TAKA[3] := TAK3 / 3;
    if (W[i] >= 0.20) then
      GEOMA[1] := min(0.333, 0.105 + (W[i] - 0.2) * 0.228 / (TS[i] - 0.2));
    if (W[i] < 0.20) then
      GEOMA[1] := 0.015 + 0.09 * W[i] / 0.2;
      {above 2 lines were recoded, pf2010, because dry soils yielded negative geoma[1]-values
       and Wierenga et al 1969 state:
       max value 0.333 (at saturation TS) and min value 0.015 (at W=0)
       and value 0.105 at W=0.2}
    GEOMA[2] := GEOMA[1];
    GEOMA[3] := 1 - GEOMA[1] - GEOMA[2];
    if (W[i] >= 0.20) then
      ALAMV := 0.000176;
    if (W[i] < 0.20) then
      ALAMV := W[i] / 1136;
    ALAMO[4] := ALAMO[4] + ALAMV;
    TAK4 := 0;
    for M := 1 to 3 do
      TAK4 := (1 / (1 + ((ALAMO[4] / ALAMA) - 1) * GEOMA[M])) + TAK4;
    TAKA[4] := TAK4 / 3;
    A12 := 0;
    A13 := 0;
    for M1 := 1 to 4 do
    begin
      if (M1 = 4) then
        PART[4, I] := 1 - W[i] - PART[1, I] - PART[2, I] - PART[3, I];
      A12 := TAKA[M1] * PART[M1, I] * ALAMO[M1] + A12;
      A13 := TAKA[M1] * PART[M1, I] + A13;
    end;
    if (W[i] >= 0.20) then
      FX := TW[i];
    if (W[i] < 0.20) then
      FX := 1 - TW[i] - PART[1, I] - PART[2, I] - PART[3, I];
    A12 := A12 + (ALAMA * FX);
    A13 := A13 + FX;

    {     CALCULATING THERMAL CONDUCTIVITIES AT HALF DEPTH, AND HALF TIME }
    {     STEPS}

    ALLAM[i] := (A12 / A13) * 3600 * 24;
    if (W[i] <= 0.20) then
      ALLAM[i] := (A12 / A13) * 1.65 * 3600 * 24;
    {10 CONTINUE}
  end;
  ALAMO[4] := ALAM4;
  {     ADJUSTING SURFACE TEMPERATURE CONDITION.  }
  {  This was the old sin wave which gave a maximum at 0.5 day and a}
  {  minimum at midnight}
  {      TEM[1]:=AVETEM(JK)+AMP(JK)*SIN(2.*PI*(TIMDAY-.25))}
  { Interpolate weekly values for current mean temperature and amplitude}
  if (jk = 1) then
  begin
    prevt := atem[1];
    preva := amp[1];
    preve := 0;
  end
  else if (jk > 1) then
  begin
    prevt := avetem[jk - 1];
    preva := amp[jk - 1];
    preve := etime[jk - 1];
  end; {if}
  tmean := prevt + (avetem[jk] - prevt) * (time + delt - preve) / (etime[jk] - preve);
  amean := preva + (amp[jk] - preva) * (time + delt - preve) / (etime[jk] - preve);
  {      TEM[1]:=tmean+amean*SIN(2.*PI*(TIMDAY-.25))}
  {  New diurnal temperature pattern.  Linear change from minimum at 6 am}
  {  to maximum at 0.6 day (14h24).  Linear drop to 0.875 day (9 pm)}
  {  when temperature is 0.3 x range above minimum.  Linear drop to }
  {  minimum at 6 am next day.  Maximum, minimum and amplitude updated}
  {  every time step, so actual temperature trace may not be linear. }
  RANGE := 2 * AMEAN;
  TLOW := TMEAN - AMEAN;
  THIGH := TMEAN + AMEAN;
  if (TIMDAY < 0.25) then
    TEM[1] := TLOW + 0.2 * RANGE - ((TIMDAY) / 0.25) * 0.2 * RANGE;
  if (TIMDAY >= 0.25) and (TIMDAY < 0.6) then
    TEM[1] := ((TIMDAY - 0.25) / 0.35) * RANGE + TLOW;
  if (TIMDAY >= 0.6) and (TIMDAY < 0.875) then
    TEM[1] := THIGH - 0.7 * RANGE * ((TIMDAY - 0.6) / 0.275);
  if (TIMDAY >= 0.875) then
    TEM[1] := THIGH - 0.7 * RANGE - 0.1 * RANGE * ((TIMDAY - 0.875) / 0.125);

  {     CALCULATING COEFFICIENTS FOR THE TRIDIAGONAL MATRIX.}

  for I := 2 to KK - 1 do
  begin
    {  Lowest depth increment set to 2 m to simulate a large heat sink for}
    {  the lower boundary condition.  Change this to change lower boundary}
    {  temperature lag and amplitude.}
    if (I = KK - 1) then
      DLZCM := 200;
    AA[i] := (DELTHEAT * ALLAM[i]) / (BETA[i] * 2 * DLZCM * DLZCM);
    BB[i] := (DELTHEAT * ALLAM[i + 1]) / (BETA[i + 1] * 2 * DLZCM * DLZCM);
    A1[i] := -AA[i];
    B1[i] := AA[i] + BB[i] + 1;
    C1[i] := -BB[i];
    D1[i] := AA[i] * TEM[i - 1] + (1 - AA[i] - BB[i]) * TEM[i] + BB[i] * TEM[I + 1];
  end;

  {     SOLVING THE TRIDIAGONAL MATRIX.}

  F1[2] := C1[2] / B1[2];
  G1[2] := (D1[2] - A1[2] * TEM[1]) / B1[2];
  for J := 3 to KK - 1 do
  begin
    F1[j] := C1[j] / (B1[j] - F1[j - 1] * A1[j]);
    G1[j] := (D1[j] - A1[j] * G1[j - 1]) / (B1[j] - A1[j] * F1[j - 1]);
  end;
  {  Set bottom boundary condition}
  {   Constant temperature}
  {     TEM[kk]:=TEM[kk]}
  {   Zero flux}
  TEM[kk] := (-TEM[kk] + TEM[kk - 1] + G1[kk - 1]) / (1. + F1[kk - 1]);
  J := KK;
  while (j > 2) do
  begin
    J := J - 1;
    TEM[j] := G1[j] - F1[j] * TEM[J + 1];
  end;
  {line below added by PF2006}
  for j := 2 to kk - 1 do
    untGUI.DataYearIn.TEM[j - 1] := TEM[j];

  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Heat');
end;

procedure wwriteln(txt: string);
{writes a message to the logfile}
begin
  if (bflog) and (not (externalcall)) then
  begin
    system.writeln(flog, txt);
    system.Flush(flog);
  end;
  {comment the line below out if no windows output required}
  {untGUI.frmMAIN.mmDOS.Lines.Add(txt); }
end;

procedure singletracewriteln(txt: string);
{indicates the execution point of the program, for debugging purposes}
begin
   untGUI.singletracetext:=txt;
end;

procedure debugout(txt: string);
{writes a (non-fatal) input error to 'Messages.txt'}
begin
  if bfdebug then
    untGUI.frmMAIN.debuglog(FILE0 + ': ' + txt);
end;

procedure closerun_errors;
{Conditional: if fatal error(s) occurred then mention this in the logfile}
begin
  wwriteln('SoilGen-run terminated with errors.');
  debugout('SoilGen-run terminated with errors.');
  stopc;
end;

procedure stopc;
{stop the run for this year and close the associated outputfiles}
begin
  {afsluiten en wegwezen}
  if bflog then
    closefile(flog);
  if bf1 then
    closefile(f1);
  if bf9 then
    closefile(f9);
  if bf11 then
    closefile(f11);
  if bf12 then
    closefile(f12);
  if bf13 then
    closefile(f13);
  if bf14 then
    closefile(f14);
  if bf15 then
    closefile(f15);
  if bf16 then
    closefile(f16);
  if bf17 then
    closefile(f17);
  if bf40 then
    closefile(f45);
  if bf45 then
    closefile(f45);
end;

procedure SinkC;
{uptake of cations Ca, Mg, Na, K and Al by plants via transpiration stream but forced by target plant element composition}
var
  i, m: integer;
  usemax, faccations, facanions, cbal, cbal1, cbal2: real;
  factors: rMM;
  tsttxt, tsttxt2: string;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering Sinkc');
{pf2006----- based on SinkP but strongly modified ---
C  THIS SUBROUTINE CALCULATES Chemical EXTRACTION BY A PLANT.
C  USE(M):  SPECIES M REMOVED FROM DEPTH INCREMENT BY PLANT(mmol/dm3/DELT)
C   (CONVERTED TO mmol/M2 BEFORE RETURN)
C  USEMAX:  MASS OF SPECIES M IN SOLUTION IN THE DEPTH INCREMENT (mmol)
C  SINK(M,I): SOURCE/SINK OF SPECIES M IN SEGMENT I (mmol/L-SOIL/DAY)
C********************************************************************
C  Initialise variables
}
  for I := 1 to KK do
    for M := 1 to NCHEM do {initialise plant uptake=0}
      USE[M, I] := 0;

{********************************************************************
C------------------------ UPTAKE BY PLANTS --------------------------
C--------------------------------------------------------------------
C  Plant uptake of chemicals is assumed to be in the transpiration stream.
   Concentrations in the transpiration stream are reduced so that the
   relative concentrations between Ca-Mg-Na-K in the vegetation biomass are preserved.
   These ratios are vegetation dependent and input.
   The reduction factor for cations is the minimum of the 4 ratios
   available/relative_concentration of the 4 species Ca-Mg-Na-K-Al
   The reduction factor for anions Cl-SO4-HCO3-CO3 is 1 (non-preferential uptake)
   but this can be reduced if there is a charge unbalance}
  for I := 2 to KK - 1 do   {do 10}
  begin
    faccations := min(ss[1, i] / relCa, min(ss[2, i] / relMg, min(
        ss[3, i] / relNa, min(ss[12, i] / relAl, ss[4, i] / relK))));
    if faccations = 0 then {one cation has 0-concentration}
    begin
      {preferential uptake according to relative concentrations in biomass is not possible;
       assume non-preferential uptake in transpiration stream instead}
        factors[1] := 1;
        factors[2] := 1;
        factors[3] := 1;
        factors[4] := 1;
        factors[12] := 1;
    end
    else {preferential uptake is possible}
    begin
      factors[1] := faccations * relCa;
      factors[2] := faccations * relMg;
      factors[3] := faccations * relNa;
      factors[4] := faccations * relK;
      factors[12] := faccations * relAl;
    end;
    {uptake anions non-preferential}
    factors[5] := 1;
    factors[6] := 1;
    factors[7] := 1;
    factors[8] := 1;
   {factors[1..4] now guarantuee that the relative concentrations Ca-Mg-Na-K-Al are preserved,
    (if this is possible) but charge balance must be preserved as well.
    If insufficient anions Cl-SO4-HCO3-CO3 are present
    then faccations is further reduced until charge balance is preserved.
    The same approach is taken if insufficient cations are present in transpiration stream}
    cbal1 := (factors[1] * ss[1, i] + factors[2] * ss[2, i]) * 2 + factors[3] * ss[3, i] +
      factors[4] * ss[4, i] + factors[12] * ss[12, i] * 3;
    cbal2 := (factors[6] * ss[6, i] + factors[8] * ss[8, i]) * 2 + factors[5] * ss[5, i] +
      factors[7] * ss[7, i];
    if cbal2 = 0 then
      cbal := 1
    else
      cbal := cbal1 / cbal2;
    if cbal < 1 then {uptake anions factors[4..8]*ss[5..8,i] must be further reduced
                   because there is a negative charge surplus}
    begin
      factors[6] := factors[6] * cbal / 2;
      factors[8] := factors[8] * cbal / 2;
      factors[5] := factors[5] * cbal;
      factors[7] := factors[7] * cbal;
    end
    else {uptake cations factors[1..4]*ss[1..4,i] must be further reduced
                   because there is a positive charge surplus}
    begin
      factors[1] := factors[1] / (2 * cbal);
      factors[2] := factors[2] / (2 * cbal);
      factors[3] := factors[3] / cbal;
      factors[4] := factors[4] / cbal;
      factors[12] := factors[12] / (3 * cbal);
    end;
    {cbal is now equal to 1}
    { All uptake in transpiration stream (mmol/dm3)}
    for M := 1 to NCHEM do  {do 420}   {was: m:=1 to actchem 1..8}
    begin
      if ((M > 8) and (M < 12)) or (M > 12) then
        USE[M, I] := 0
      else
      begin
        USE[M, I] := factors[m] * A[I] * SS[M, I] * delt;
       {  Flux density of species into roots is limited if uptake exceeds
          the dissolved chemical in a segment }
        USEMAX := MAX(SS[M, I], 0) * Y[I];
        if USE[M, I] > USEMAX then
          USE[M, I] := USEMAX;
      end;
      DUSE[m, i] := DUSE[m, i] + USE[m, i];
      CUSE[m] := CUSE[m] + USE[m, i];
    end; {420 CONTINUE}
  end; {  10 CONTINUE}
{  Calculation of sink term during a time step). This term is the combination of:
   1. loss via transpiration stream (USE)
   2. gain via mineralization  (delt*size of CHEMCO2pool, calculated daily in RothC_daily)
      for each chemical 1..nchem}
  for i := 2 to kk - 1 do
    for m := 1 to nchem do
    begin
      cminer[m] := cminer[m] + delt * CHEMCO2pool[m, i];
      {was: csink[m,i]:=(delt*CHEMCO2pool[m,i])-USE[m,i]/DELT; }
      csink[m, i] := (delt * CHEMCO2pool[m, i]) - USE[m, i];
    end;
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Sinkc');
end;

procedure Init_Clay_Migration;
{initialization for clay migration submodel using mass of clay silt + sand at end previous year}
{This routine initializes the clay migration routine at the beginning of the year
 by calculating the clay, silt and sand mass per compartment. The clay mass
 will in the routine Clay_Migration1_PreCDE be made partially dispersibe,
 transported in the CDE solver and be immobilised again in the Clay_Migration2_PostCDE.
 At the end of the year, the masses will be backconverted to mass percentages
 of clay, silt, sand and oc.
}
var
  i: integer;
  zand1, ry1, ry2, ry3, ry4, rsy: real;
begin
  untGUI.frmMAIN.MVAR(GlobalTime*100000+Datayearin.nolayers*1000+cumt,' start untProcesses.Init_Clay_Migration: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
  untGUI.frmMAIN.MVAR(GlobalTime*100000+Datayearin.nolayers*1000+cumt,' start untProcesses.Init_Clay_Migration: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);
  {if (TRACE = 1) then}
    singletracewriteln(' Entering Init_Clay_Migration');
  ClayCount := 0;
  for i := 2 to kk - 1 do
  begin
    {set clay dispersion indicator to 0}
    ClayDispInd[i] := 0;
   {determine mass per compartment of clay, silt, OC and sand over 1 sq.m
    No longer necessary here: mass of CSS+OC are transferred between chem and shell units }
     {CLAYm[i]:=CLAY[i]*DLZ*RHO[i]/100;  these are kg/m2
     SILTm[i]:=SILT[i]*DLZ*RHO[i]/100;
     OCm[i]:=OC[i]*DLZ*RHO[i]/100;
     SANDm[i]:=(100-CLAY[i]-SILT[i]-1.7*OC[i])*DLZ*RHO[i]/100;}
    totalm[i] := CLAYm[i] + SILTm[i] + SANDm[i] + 1.7 * OCm[i];  {markOC}
    {calc volumetric clay percentage for later usage in estimation of dispersible clay}
    zand1 := 100 - 1.7 * OC[i] - silt[i] - clay[i];    {markOC}
    {Mean particle density}
    ry1 := 1.7 * OC[i] / RHOS3;
    ry2 := silt[i] / RHOS1;
    ry3 := clay[i] / RHOS2;
    ry4 := zand1 / RHOS1;
    rsy := ry1 + ry2 + ry3 + ry4;
    {calculate volumetric clay percentage}
    volpercclay[i] := (100 - ts[i]) * ry3 / rsy;
  end;
  {initial values for clay migration from top layer by raindrop impact}
  if (ACEC[2] / 10) - 3 * OC[2] <= 0.40 * clay[2]   {apply either model 3.1 or model 3.2 from Brubaker et al., 1992
                                           to estimate the maximal water-dispersible clay content} then
    CM_Mmax := 0.635 * clay[2] / 100
  {3.1 converted to grammes dispersible per gram soil; negative intercept ignored}
  else
    CM_Mmax := 0.340 * clay[2] / 100;
  {3.2 converted to grammes dispersible per gram soil; negative intercept ignored}
  CM_Ms := CM_Mmax; {starting value for dispersible clay content = maximal value}
  CM_As := CM_Ms * RHO[2] * 1 * 1000; {starting value for the pool of detachable clay
                                for topsoil layer 1 mm, in g/m2}
  PoolT := CLAYm[2] * 1000 * (1) / dlz;       {Clay mass pool in upper 1 mm of top compartment}
  PoolB := CLAYm[2] * 1000 * (dlz - 1) / dlz;
  {Clay mass pool in upper (dlz-1) mm of top compartment}
  {values for filtering of transported clay, from Jarvis et al, 1999}
  {a combination of fref=2; vref=0.1 and n=0.7 corresponds to the macropore model settings of Jarvis}
  {a combination of fref=50; vref=irrelevant and n=0 corresponds to a micropores-only model}
  {CM_fref:=2;}{reference filter coefficient, m-1, 2 for macropores & 50 for micropores}
  {CM_vref:=0.1;}{porewater velocity at which fref is measured, m/h, 100 mm/h}
  {CM_n:=untGUI.calJarvisCM_n;} {filtering exponent: if 0=traditional clean bed filtering; 0.7=Jarvis'99;
                                 if 1=1st order function of concentration}
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Init_Clay_Migration');
   untGUI.frmMAIN.MVAR(GlobalTime*100000+Datayearin.nolayers*1000+cumt,' einde untProcesses.Init_Clay_Migration: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
   untGUI.frmMAIN.MVAR(GlobalTime*100000+Datayearin.nolayers*1000+cumt,' einde untProcesses.Init_Clay_Migration: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);

end;

procedure Clay_Migration1_PreCDE;
{calculate amount of clay in dispersion}
{This routine prepares the calculation of clay transport,
 by calculating concentration of clay that is dispersible.
 Along with the determination of the dispersible clay mass itself,
 the "dispersible" exchange phase (XCa, XMg, XNa, XK, XNa and XAl) is also determined.
 These variables are stored in the SS[m,i] variable:
 dispersible claymass=chemical17; XCa=18; XMg=19; XNa:=20; XK=21; XH:=21; XAl:=22)
 This allows later processing by the CDE-solver (solc).}
 {step a: determine what part of the clay mass is dispersible
          and put the result in a SS[m,i] variable as a -temporary- solute}
 {step b: reduce clay content by the dispersible clay
 {step c: repeat this for exchangeable Ca, Mg, K, Na, H and Al
  step d: determine dispersion from surface compartment due to rainsplash
          and put this amount in surface boundary condition
  step e: determine sink in all compartments due to filtering
          and put this amount in CDE-sink term}
var
  i, m: integer;
  porewatervelocity, frac, fracdisp, netfrac, netfrac2, CM_Pmax, bioturbfrac2,
  totecto, parden, redsoilcover, CSC, ElectrolyteConc, splashred_P, splashred_frac,locclay,lococ: real;
  rainsplashdispersion, filteringallowed: boolean;
  typeoffracdisp: integer;
  tstr,tmpstr1,tmpstr2: string;
  TW: rLL;
begin
  untGUI.frmMAIN.MVAR(GlobalTime*100000+Datayearin.nolayers*1000+cumt,' start untProcesses.Clay_Migration1_preCDE: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
  untGUI.frmMAIN.MVAR(GlobalTime*100000+Datayearin.nolayers*1000+cumt,' start untProcesses.Clay_Migration1_preCDE: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);

  typeoffracdisp := 3; {set to 1, 2 or 3}
  if (untGUI.ClayMigrationAllowed) then
 {approach in 4 steps:
  a: Estimate per compartment the fraction of the clay that is dispersible because of pH-range;
  b: reduce claymass with dispersible fraction
  c: express exchange species als mmol+/kg clay and convert dispersible part as concentration
  d: Just for the topsoil compartment: add the dispersion caused by rainsplash
     modified after model Jarvis et al., 1999: 1. replenishment goes via bioturbation
                                               2. detachment reduced if ectorganic material exists
                                               3. detachment also reduces when agricultural vegetation cover
  e: apply filtering}
  begin
    {if (TRACE = 1) then}
    singletracewriteln(' Entering Clay_Migration_PreCDE');
    for i := 2 to kk - 1 do  {all soil compartments: chemical dispersion}
    begin
   {initialisation: split up exchangeable cations in _oc and _clay part
    using the CEC_oc and CEC_clay fractions of CEC.
    _clay parts are likely to change.}
      XSE_oc[1, i] := XSE[1, i] * CEC_oc[i] / CEC[i];
      XSE_oc[2, i] := XSE[2, i] * CEC_oc[i] / CEC[i];
      XSE_oc[3, i] := XSE[3, i] * CEC_oc[i] / CEC[i];
      XSE_oc[4, i] := XSE[4, i] * CEC_oc[i] / CEC[i];
      XSE_clay[1, i] := XSE[1, i] * CEC_clay[i] / CEC[i];
      XSE_clay[2, i] := XSE[2, i] * CEC_clay[i] / CEC[i];
      XSE_clay[3, i] := XSE[3, i] * CEC_clay[i] / CEC[i];
      XSE_clay[4, i] := XSE[4, i] * CEC_clay[i] / CEC[i];
      XSE_oc[10, i] := XSE[10, i] * CEC_oc[i] / CEC[i];
      XSE_oc[12, i] := XSE[12, i] * CEC_oc[i] / CEC[i];
      XSE_clay[10, i] := XSE[10, i] * CEC_clay[i] / CEC[i];
      XSE_clay[12, i] := XSE[12, i] * CEC_clay[i] / CEC[i];
      {a: what fraction of the clay is dispersible?}
      if typeoffracdisp = 1 then {a1: approach based on pH (cf Van Breemen and Buurman 2002)}
      begin
        fracdisp := 1;
        if pHlab[i] < 4.5 then
          fracdisp := 0;
        if (pHlab[i] >= 4.5) and (pHlab[i] <= 5) then
          fracdisp := 2 * (pHlab[i] - 4.5);
        if pHlab[i] > 7.5 then
          fracdisp := 0;
        if (pHlab[i] >= 7) and (pHlab[i] <= 7.5) then
          fracdisp := 1 - (pHlab[i] - 7) / 0.5;
        {and possibly overruled by the presence of gypsum or calcite by activating next code line}
        {if (calcite[i]>0.001)or(gyp[i]>0.001) then fracdisp:=0;}
      end;
      if typeoffracdisp = 2 then
        {a2: alternative approach based on Brubaker, Holzhey and Brasher, 1992 in SSSA-J 56:1227-1232}
      begin
        if cec[i] / 10 - 3 * oc[i] <= 0.4 * clay[i] then
          fracdisp := 0.635
        else
          fracdisp := 0.34;
      end;
      if typeoffracdisp = 3 then
        {a3: alternative approach based on the data from Goldberg and Forster 1990}
      begin
     {based on Goldberg and Forster 1990 SSSA-J 54: 714-718
      after determining a relation to estimate minimal mmolc/liter
      at which soil clay mixtures still are flocculated:
      CSC=(b1*SUM21+b2*SAR+b3)*EXP(b4*(b5*MM+b6-pH)^2)
      where CSC=Critical Salt Concentration (mmolc/dm3)
            SUM21=percent 2:1 clay minerals (Montmorillonite+Illite)
            SAR=Sodium Adsorption Ratio
            MM21=percent Montmorillonite
            b1..b6 are coefficients estimated by nonlinear regression}
        CSC := (0.039492 * SUM21[i] + 0.740444 * SAR[i] + 1.66942) * EXP(
          -0.103479 * power(-0.002799 * MM21[i] + 7.94461 - pHlab[i], 2));
        if typechem = 1 then
          begin end
        else
          ElectrolyteConc := (3 * (SE[12, i]+SE[13, i])) +    {Al+FeIII: trivalent}
            (2 * (SE[1, i] + SE[2, i] + SE[14, i] {+ SE[6, i] + SE[8, i]}) +
            {Ca, Mg, FeII, SO4, CO3: bivalent}
            (SE[3, i] + SE[4, i] {+ SE[5, i] + SE[7, i]} + ABS(
            SE[9, i]) + SE[10, i] {+ SE[11, i]}))
        {Na, K, Cl, HCO3, balion, H, OK: monovalent};
        if (calcite[i] > 0.001) or (gyp[i] > 0.001) or (ElectrolyteConc >= CSC) then
          fracdisp := 0
        else   {so these brackets remove the boolean effect of presence of calcite and gypsum}
          fracdisp := min(1,max(0,(1 - (ElectrolyteConc / CSC))));
        {override for special edition}{fracdisp:=0;  comment when finished testing!!!}
      end;
      {store cumulative clay dispersion indicator and associated counter}
      ClayDispInd[i] := ClayDispInd[i] + fracdisp * delt;
      ClayCount := ClayCount + delt;
   {b: reduce claymass with current dispersible fraction; store dispersible in SS[]
    b1: calc netfrac, the fraction of the clay volume fraction exposed to macropores
                      = (ts-microporosity) * volpercclay
                        multiplied by the clay dispersibility (fracdisp)}
      netfrac := fracdisp * (ts[i] - microporosity[i]) * volpercclay[i] / 100;
      {set netfrac2 to 0 for debugging purposes and to netfrac = default}
      netfrac2 :=netfrac;
      {b2: now express dispersible clay as concentration}
      SS[17, i] := netfrac * CLAYm[i] / w[i];
      clayfrac[i] := CLAYm[i] / (CLAYm[i] + SILTm[i] + SANDm[i]);
      {c: express exchange species als mmol+/kg clay and convert dispersible part as concentration}
      {was: netfrac*XSE[x,i]*0.5*RHO[i]/w[i]; now clayfrac is used}
      if clayfrac[i] > 0 then
      begin
        {c1: exchangeable Ca}
        SS[18, i] := netfrac2 * XSE_clay[1, i] * 0.5 * Claym[i] / (w[i] * clayfrac[i]);
        {c2: exchangeable Mg}
        SS[19, i] := netfrac2 * XSE_clay[2, i] * 0.5 * Claym[i] / (w[i] * clayfrac[i]);
        {c3: exchangeable Na}
        SS[20, i] := netfrac2 * XSE_clay[3, i] * Claym[i] / (w[i] * clayfrac[i]);
        {c4: exchangeable K}
        SS[21, i] := netfrac2 * XSE_clay[4, i] * Claym[i] / (w[i] * clayfrac[i]);
        {c5: exchangeable H}
        SS[22, i] := netfrac2 * XSE_clay[10, i] * Claym[i] / (w[i] * clayfrac[i]);
        {c6: exchangeable Al}
        SS[23, i] := netfrac2 * XSE_clay[12, i] * Claym[i] / (3 * w[i] * clayfrac[i]);
      end
      else
      begin
        {c1: exchangeable Ca}
        SS[18, i] := 0;
        {c2: exchangeable Mg}
        SS[19, i] := 0;
        {c3: exchangeable Na}
        SS[20, i] := 0;
        {c4: exchangeable K}
        SS[21, i] := 0;
        {c5: exchangeable H}
        SS[22, i] := 0;
        {c6: exchangeable Al}
        SS[23, i] := 0;
      end;
      {c7: reduce undispersed clay mass and exchange species}
      CLAYm[i] := (1 - netfrac) * CLAYm[i];
      if untGUI.physweathallowed then
        untGUI.PW_K[11, i - 1] := round((1 - netfrac) * untGUI.PW_K[11, i - 1]);
      {also reduce number of grains}
      XSE_clay[1, i] := (1 - netfrac2) * XSE_clay[1, i];
      XSE_clay[2, i] := (1 - netfrac2) * XSE_clay[2, i];
      XSE_clay[3, i] := (1 - netfrac2) * XSE_clay[3, i];
      XSE_clay[4, i] := (1 - netfrac2) * XSE_clay[4, i];
      XSE_clay[10, i] := (1 - netfrac2) * XSE_clay[10, i];
      XSE_clay[12, i] := (1 - netfrac2) * XSE_clay[12, i];
      {c8: update of newRHO}
      locCLAY:=100*Claym[i]/(claym[i]+siltm[i]+sandm[i]);
      locOC:=100*1.7*OCm[i]/(claym[i]+siltm[i]+sandm[i]+1/7*OCm[i]);
      {undone: newRHO[i]:=untGUI.frmMAIN.HYPRESRHO(locCLAY,locOC); }
      newRHO[i] := (CLAYm[i] + SILTm[i] + 1.7 * OCm[i] + SANDm[i]) /(((100-stoneperc[i])/100)*dlz);
      {rho for fine earth only, thus correct for stoniness}
      {and Theta-sat}
   {PARDEN:=100/((CLAYm[i]*100/(DLZ*RHO[i]))/RHOS2+(100-(CLAYm[i]*100/(DLZ*RHO[i]))
           -1.7*(OCm[i]*100/(DLZ*RHO[i])))/RHOS1+1.7*(OCm[i]*100/(DLZ*RHO[i]))/RHOS3);
   TS[I]:=1-RHO[I]/PARDEN; }
      {adapt W and TG if necessary, this will cause small mass balance errors!}
   {W[i]:=min(W[i],TS[i]);
   TG[i]:=TS[i]-W[i]; }
    end; 
    rainsplashdispersion := True;
    if rainsplashdispersion then
    begin
 {step d:
   Just for the topsoil compartment: add the dispersion caused by rainsplash}
   {fill in (initial values set in Init_Clay_Migration):
    conform to model Jarvis et al., 1999
    da1: calculate R (rainfall intensity, mm/h, valid during delt)
    da2: calculate E (kinetic energy rainfall, J/m2,h), with 29*(1-0.72*exp(-0.05*R))
    da3: calculate D (Detachment, g/m2,h, during delt) with 15*E*R*(1-SoilCov)*Ms*delt*SC_Redfac
    da4: calculate P (rePlenishment, g/m2,h, during delt) with 0.1*(1-Ms/Mmax)*delt
    da5: calculate As (detachable pool, g/m2, after delt) with As=As(i-delt)-D+P
    da6: calculate Ms (readily dispersible clay fraction, g/g soil) with Ms=As/(RHO*1*1000)
    da7: calculate Cs (infiltration concentration, g/dm3, for delt) with D/Infiltration}
      {da1}
      if (ihead = 1) or (ihead = 2) {rainfall event in progress or starting} then
      begin
        {take the rainfall intensity of the current event, in mm/h}
        CM_R := iratea[irrno];
        {calculate precipitation (mm) during deltt}
        splashred_P := CM_R * (delt * 24);
     {calculate the fraction of the area actually hit by raindrops during delt
      1. Volume 1 drop (cm3) = V1drop = 4/3 * PI * (d/2)^3 where d = drop diameter (cm), set to 0.3
      2. Volume over 1 m2 = Vprec = splashred_P*1000    (in cm3)
      3. Nr of raindrops is then Nodrops = Vprec/V1drop
      4. Equivalent area covered by Nodrops is Nodrops*PI*(d/2)^2 (in cm2)
         This reduces to EqvArea= 1.5*Vprec/d
      5. So over-all, the fraction of 1 m2 hit by raindrops of 0.3 cm during the precipitation in deltt
         is: (1.5*splashred_P/0.3)/10000 where 10000 is cm2/m2; in short: 0.0005*splashred_P}
        splashred_frac := min(0.0005 * splashred_P, 1);
        {da2}
        CM_E := 29 * (1 - 0.72 * exp(-0.05 * CM_R));
        {kinetic energy as in USLE; see Brown&Foster 1987}
        {da3}
        totecto := RothC.DPMpool[1] + RothC.RPMpool[1] + RothC.BIOpool[1] +
          RothC.HUMpool[1] + RothC.IOMpool[1];
     {assuming that soil cover is 100% when the ectorganic layer is untGUI.calThickEcto mm thick on average,
      and that the bulk density of ectorganic layer is calRHO_ecto kg/dm3, converting 1 ton/ha to 0.1 kg/m2:}
        SC_RedFac := min((totecto * 0.1 / clay_EctoBD) / clay_Ectothick, 1);
     {as also the crop cover of an agricultural crop reduces splash detachment, we take the
     maximal value of SC_RedFac and CropCover}
        if RothC.vegetatie = 'agriculture' then
          redsoilcover := max(SC_RedFac, CRPCOV[MAX(1, CROPNO)])
        else
          redsoilcover := SC_RedFac;
        {reduce splash detachment using the fraction of the area actually hit by rainsdrops during delt}
        redsoilcover := max(redsoilcover, 1 - splashred_frac);
        {further reduction of splash-affected area under agriculture}
     {if RothC.vegetatie='agriculture'
     then redsoilcover:=max(redsoilcover,untGUI.MinSplashableSoilAreareduction);}
        {tmpdmp(delt*24,' hours with redsoilcover= ',redsoilcover); }
        CM_D := clay_kd * CM_E * CM_R * (1 - redsoilcover) * CM_Ms * (delt * 24);
        {=detachment during delt*24 hours}
        {reduce the detachment so that only effect of raindrops hitting clay particles is counted.}
        CM_D := CM_D * volpercclay[2] / 100;
      end
      else
      begin
        CM_R := 0;
        CM_E := 0;
        CM_D := 0;
      end;
      {da4}
      if CM_Mmax = 0 then
        CM_P := 0
      else
        CM_P := clay_kr * (1 - (CM_Ms / CM_Mmax)) * (delt * 24); {replenishment during delt*24 hours}
   {calculate maximal replenishment via bioturbation:
    bioturbfrac2:=untGUI.fractop*delt/365;
    mixpool:=(PoolT+PoolB)*bioturbfrac2;
    PoolT:=(1-bioturbfrac2)*PoolT+MixPool*dikT/dlz; PoolT refers to all-clay pool in upper dikT mm layer
    PoolB:=(1-bioturbfrac2)*PoolB+MixPool*dikB/dlz; PoolB refers to all-clay pool in lower dikB mm layer}
      if RothC.vegetatie = 'agriculture' then
        bioturbfrac2 := 0 {no "natural" bioturbation}
      else
        bioturbfrac2 := untGUI.fractop * delt / 365;  {bioturbation as mass fraction over delt}
      MixPool := (PoolT + PoolB) * bioturbfrac2;
      PoolT := (1 - bioturbfrac2) * PoolT + MixPool * 1 / dlz;
      {all-clay pool in upper 1 mm of top compartment}
      PoolB := (1 - bioturbfrac2) * PoolB + MixPool * (dlz - 1) / dlz;
      {all-clay pool in remaining (dlz-1) mm of top compartment}
      CM_Pmax := PoolT;
      {maximal replenishment = all-clay still present in upper 1 mm of top compartment}
      {CM_Pmax:=CLAYm[2]*1000;}{maximal replenishment = undispersed clay still present in top compartment}
      if CM_P > CM_Pmax then
        CM_P := CM_Pmax;  {no more replenishment than what is present}
      CM_D := min(CM_D, CM_As + CM_P);
      {Detachment cannot exceed previous pool size As plus current replenishment}
      {da5}
      if CM_Pmax > 0   {safety valve for claym[2] near 0} then
        netfrac := max(0, min(CM_D / (PoolT + PoolB), 1))
      else
        netfrac := 0;         {netfrac is fraction of clay mass in the whole upper compartment
                            (dlz mm thickness) that is lost from the upper 1 mm by rain detachment
                            during the current timestep}
      {for special edition: set to 0}{netfrac:=0; }
      PoolT := PoolT - CM_D;       {update all-clay pool in upper 1 mm}
      CM_As := max(CM_As - CM_D + CM_P, 0); {update detachable pool}
      {da6}
      CM_Ms := min(CM_Mmax, CM_As / (1000 * newRHO[2])); {update readily dispersible clay fraction}
      {da7}
      SFER[17] := netfrac * CLAYm[2];
      {SFER[17]:=CM_D/1000;}  {to be added to SS[m,1], just before calling solc,
                          as concentration in infiltrating water;
                          converted to kg/m2 to be combinable with "chemical dispersible clay",
                          assuming (as everywhere else in this programme) that 1 gram clay = 1 mole}
      {set netfrac2 to 0 for debugging purposes and to netfrac = default}
      netfrac2 := netfrac;
   {consequences for exchange phase: move the fraction of the exchangeable cation
    that moves with the dispersed clay to the "surface chemical" to be applied
    during the current timestep, which is being transformed to a concentration during solc,
    using the actual infiltration during the timestep}
      if clayfrac[2] > 0 then
      begin
        {consequences for exchangeable Ca, with conversion to mmol/m2}
        SFER[18] := netfrac2 * XSE_clay[1, 2] * 0.5 * (CLAYm[2] / clayfrac[2]) * DLZ;
        {consequences for exchangeable Mg}
        SFER[19] := netfrac2 * XSE_clay[2, 2] * 0.5 * (CLAYm[2] / clayfrac[2]) * DLZ;
        {consequences for exchangeable Na}
        SFER[20] := netfrac2 * XSE_clay[3, 2] * (CLAYm[2] / clayfrac[2]) * DLZ;
        {consequences for exchangeable K}
        SFER[21] := netfrac2 * XSE_clay[4, 2] * (CLAYm[2] / clayfrac[2]) * DLZ;
        {consequences for exchangeable H}
        SFER[22] := netfrac2 * XSE_clay[10, 2] * (CLAYm[2] / clayfrac[2]) * DLZ;
        {consequences for exchangeable Al}
        SFER[23] := netfrac2 * XSE_clay[12, 2] * (CLAYm[2] / clayfrac[2]) * DLZ / 3;
      end
      else
      begin   {no more clay, thus no more clay & exchangeable cation movement}
        {consequences for exchangeable Ca, with conversion to mmol/m2}
        SFER[18] := 0;
        {consequences for exchangeable Mg}
        SFER[19] := 0;
        {consequences for exchangeable Na}
        SFER[20] := 0;
        {consequences for exchangeable K}
        SFER[21] := 0;
        {consequences for exchangeable H}
        SFER[22] := 0;
        {consequences for exchangeable Al}
        SFER[23] := 0;
      end;
      {update remaining stuff}
      CLAYm[2] := (1 - netfrac) * CLAYm[2]; {update clay mass in compartment 2=top compartment}
      if untGUI.physweathallowed then
        untGUI.PW_K[11, 1] := round((1 - netfrac) * untGUI.PW_K[11, 1]);
      {also reduce number of grains}
      {consequences for exchangeable Ca, with conversion to mmol/m2}
      XSE_clay[1, 2] := (1 - netfrac2) * XSE_clay[1, 2];
      {consequences for exchangeable Mg}
      XSE_clay[2, 2] := (1 - netfrac2) * XSE_clay[2, 2];
      {consequences for exchangeable Na}
      XSE_clay[3, 2] := (1 - netfrac2) * XSE_clay[3, 2];
      {consequences for exchangeable K}
      XSE_clay[4, 2] := (1 - netfrac2) * XSE_clay[4, 2];
      {consequences for exchangeable H}
      XSE_clay[10, 2] := (1 - netfrac2) * XSE_clay[10, 2];
      {consequences for exchangeable Al}
      XSE_clay[12, 2] := (1 - netfrac2) * XSE_clay[12, 2];
      {update of newRHO }
      locCLAY:=100*Claym[2]/(claym[2]+siltm[2]+sandm[2]);
      locOC:=100*1.7*OCm[2]/(claym[2]+siltm[2]+sandm[2]+1.7*OCm[2]);
      {undone: newRHO[2]:=untGUI.frmMAIN.HYPRESRHO(locCLAY,locOC);}

      newRHO[2] := (CLAYm[2] + SILTm[2] + 1.7 * OCm[2] + SANDm[2]) / (((100-stoneperc[2])/100)*dlz);  {markOC}
      {RHO only for fine earth > correct for stoniness volume}
      {and theta-sat}
   {PARDEN:=100/((CLAYm[2]*100/(DLZ*RHO[2]))/RHOS2+(100-(CLAYm[2]*100/(DLZ*RHO[2]))
           -1.7*(OCm[2]*100/(DLZ*RHO[2])))/RHOS1+1.7*(OCm[2]*100/(DLZ*RHO[2]))/RHOS3);
   TS[2]:=1-RHO[2]/PARDEN; }
      {adapt W and TG if necessary, this will cause small mass balance errors!}
   {W[2]:=min(W[2],TS[2]);
   TG[2]:=TS[2]-W[2]; }
    end;
    filteringallowed := true;{for special edition allowing no clay migration: set to FALSE}
    if filteringallowed then
    begin
 {e: filtering
  A filtering term is calculated to be included as a sink term in the CDE}
      for i := 2 to kk - 1 do
        for m := 17 to 23 do  {clay + exchange species sink terms according to Jarvis et al, 1999}
        begin
          {general form: F = fref . vref^n . v^(1-n). c . theta}
          TW[i] := (Y[i + 1] + W[i + 1] + Y[i] + W[i]) / 4;
          porewatervelocity := 0.001 * (ABS(FLXDEN[i]) / TW[i]) / 24; {mm/d to m/h conversion included}
          if porewatervelocity = 0 then
            porewatervelocity := 1e-12;  {to prevent numerical misery}
          csink[m, i] := -CM_fref * power(CM_vref, CM_n) * power(porewatervelocity, 1 - CM_n) *
            ss[m, i] * w[i];
        end;
    end;
    {if (TRACE = 1) then}
      singletracewriteln(' Leaving Clay_Migration1_PreCDE');
  end;
  untGUI.frmMAIN.MVAR(GlobalTime*100000+Datayearin.nolayers*1000+cumt,' einde untProcesses.Clay_Migration1_preCDE: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
  untGUI.frmMAIN.MVAR(GlobalTime*100000+Datayearin.nolayers*1000+cumt,' einde untProcesses.Clay_Migration1_preCDE: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);
end;

procedure Clay_Migration2_PostCDE;
{calculate consequences of dispersible clay transport}
{This routine is called immediately after solving the CDE for dissolved and dispersed matter.
 Dispersed clay mass is added to the immobile clay per compartment and clay, silt, oc and Rho are
 recalculated as a consequence.
 Adsorbed Ca,Mg,Na,K,H and Al are calculated again per compartment because some of it
 was dispersed with the clay.
 Finally, the CEC per compartment is calculated again.}
var
  i: integer;
  parden,locclay,locOC: real;
  oldclaymass: real;
begin
  untGUI.frmMAIN.MVAR(GlobalTime*100000+Datayearin.nolayers*1000+cumt,' start untProcesses.Clay_Migration2_postCDE: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
  untGUI.frmMAIN.MVAR(GlobalTime*100000+Datayearin.nolayers*1000+cumt,' start untProcesses.Clay_Migration2_postCDE: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);

  if (untGUI.claymigrationallowed) then
  begin
    {if (TRACE = 1) then}
      singletracewriteln(' Entering Clay_Migration1_PostCDE');  
    for i := 2 to kk - 1 do
    begin
     {add dispersively transported claymass to fixed claymass, recalc rho
      and recalc fractions of clay, silt and OC}
      oldclaymass := claym[i];
      CLAYm[i] := CLAYm[i] + SE[17, i] * w[i];
      SE[17, i] := 0;
      SS[17, i] := 0; {reset}
      if untGUI.physweathallowed then
        untGUI.PW_K[11, i - 1] := round(CLAYm[i] * untGUI.PW_K[11, 2] / oldclaymass);
      {also change number of grains}
      {adapt newRHO and clayfrac}
      locCLAY:=100*Claym[i]/(claym[i]+siltm[i]+sandm[i]);
      locOC:=100*1.7*OCm[i]/(claym[i]+siltm[i]+sandm[i]+1.7*OCm[i]);
      {undone: newRHO[i]:=untGUI.frmMAIN.HYPRESRHO(locCLAY,locOC); }
      if (GlobalTime*100000+Datayearin.nolayers*1000+cumt=181216365)and(i=kk-1)
         then SE[17, i] := 0;
      newRHO[i] := (CLAYm[i] + SILTm[i] + 1.7 * OCm[i] + SANDm[i]) / (((100-stoneperc[i])/100)*dlz);      {markOC}
      {RHO for fine earth, thus correct DLZ as volume for stoniness}
      {Theta-sat }
     {PARDEN:=100/((CLAYm[i]*100/(DLZ*RHO[i]))/RHOS2+(100-(CLAYm[i]*100/(DLZ*RHO[i]))
             -1.7*(OCm[i]*100/(DLZ*RHO[i])))/RHOS1+1.7*(OCm[i]*100/(DLZ*RHO[i]))/RHOS3);
     TS[I]:=1-RHO[I]/PARDEN; }
      {adapt W and TG if necessary, this will cause small mass balance errors!}
     {W[i]:=min(W[i],TS[i]);
     TG[i]:=TS[i]-W[i]; }
      clayfrac[i] := CLAYm[i] / (CLAYm[i] + SILTm[i] + SANDm[i]);
      if CLAYm[i] > 0 then
      begin
        {recalc exchangeable Ca}
        {was: XSE[1,i]:=XSE[1,i]+SE[18,i]/(0.5*oldRHO[i]/w[i]);}
        XSE_clay[1, i] := XSE_clay[1, i] + SE[18, i] * 2 * w[i] * clayfrac[i] / CLAYm[i];
        SE[18, i] := 0;
        SS[18, i] := 0; {reset}
        {recalc exchangeable Mg}
        XSE_clay[2, i] := XSE_clay[2, i] + SE[19, i] * 2 * w[i] * clayfrac[i] / CLAYm[i];
        SE[19, i] := 0;
        SS[19, i] := 0; {reset}
        {recalc exchangeable Na}
        XSE_clay[3, i] := XSE_clay[3, i] + SE[20, i] * w[i] * clayfrac[i] / CLAYm[i];
        SE[20, i] := 0;
        SS[20, i] := 0; {reset}
        {recalc exchangeable K}
        XSE_clay[4, i] := XSE_clay[4, i] + SE[21, i] * w[i] * clayfrac[i] / CLAYm[i];
        SE[21, i] := 0;
        SS[21, i] := 0; {reset}
        {recalc exchangeable H}
        XSE_clay[10, i] := XSE_clay[10, i] + SE[22, i] * w[i] * clayfrac[i] / CLAYm[i];
        SE[22, i] := 0;
        SS[22, i] := 0; {reset}
        {recalc exchangeable Al}
        XSE_clay[12, i] := XSE_clay[12, i] + SE[23, i] * 3 * w[i] * clayfrac[i] / CLAYm[i];
        SE[23, i] := 0;
        SS[23, i] := 0; {reset}
      end
      else
      begin
        {XSE[x,i] is not adapted because clay content = 0}
        SE[18, i] := 0;
        SS[18, i] := 0; {reset}
        SE[19, i] := 0;
        SS[19, i] := 0; {reset}
        SE[20, i] := 0;
        SS[20, i] := 0; {reset}
        SE[21, i] := 0;
        SS[21, i] := 0; {reset}
        SE[22, i] := 0;
        SS[22, i] := 0; {reset}
        SE[23, i] := 0;
        SS[23, i] := 0; {reset}
      end;
      {recalc CEC and exhangeable cations by recombing _oc and (changed) _clay}
        CEC_clay[i] := XSE_clay[1, i] + XSE_clay[2, i] + XSE_clay[3, i] + XSE_clay[4, i] +
          XSE_clay[10, i] + XSE_clay[12, i];
      CEC[i] := CEC_clay[i] + CEC_oc[i];
      XSE[1, i] := XSE_clay[1, i] + XSE_oc[1, i];
      XSE[2, i] := XSE_clay[2, i] + XSE_oc[2, i];
      XSE[3, i] := XSE_clay[3, i] + XSE_oc[3, i];
      XSE[4, i] := XSE_clay[4, i] + XSE_oc[4, i];
      XSE[10, i] := XSE_clay[10, i] + XSE_oc[10, i];
      XSE[12, i] := XSE_clay[12, i] + XSE_oc[12, i];
    end;
    {if (TRACE = 1) then}
      singletracewriteln(' Leaving Clay_Migration2_PostCDE');
  end;
  untGUI.frmMAIN.MVAR(GlobalTime*100000+Datayearin.nolayers*1000+cumt,' einde untProcesses.Clay_Migration2_postCDE: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
  untGUI.frmMAIN.MVAR(GlobalTime*100000+Datayearin.nolayers*1000+cumt,' einde untProcesses.Clay_Migration2_postCDE: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);

end;

procedure Exit_Clay_Migration;
{backconverts the clay, silt, oc and sand masses to  mass percentages of clay, silt, sand and OC}
var
  i: integer;
begin
  untGUI.frmMAIN.MVAR(GlobalTime*100000+Datayearin.nolayers*1000+cumt,' start untProcesses.Exit_Clay_Migration: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
  untGUI.frmMAIN.MVAR(GlobalTime*100000+Datayearin.nolayers*1000+cumt,' start untProcesses.Exit_Clay_Migration: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);

  {if (TRACE = 1) then}
    singletracewriteln(' Entering Exit_Clay_Migration');
  for i := 2 to kk - 1 do
  begin
    {determine mass% per compartment of clay, silt and OC by the remaining mass}
    {RHO[i]:=(CLAYm[i]+SILTm[i]+1.7*OCm[i]+SANDm[i])/dlz; {markOC}
    CLAY[i]:=100*Claym[i]/(claym[i]+siltm[i]+sandm[i]+1.7*OCm[i]);
    SILT[i]:=100*Siltm[i]/(claym[i]+siltm[i]+sandm[i]+1.7*OCm[i]);
    SAND[i]:=100*Sandm[i]/(claym[i]+siltm[i]+sandm[i]+1.7*OCm[i]);
    OC[i]:=max(1e-5,100 * OCm[i] /(claym[i]+siltm[i]+sandm[i]+1.7*OCm[i]));
  end;
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Exit_Clay_Migration');
    untGUI.frmMAIN.MVAR(GlobalTime*100000+Datayearin.nolayers*1000+cumt,' einde untProcesses.Exit_Clay_Migration: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
    untGUI.frmMAIN.MVAR(GlobalTime*100000+Datayearin.nolayers*1000+cumt,' einde untProcesses.Exit_Clay_Migration: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);

end;

procedure RothC_Init(RCName: string);
{initialization and read input file for C-submodel, status end of previous year}
var
  i, dum, m, chem, w: integer;
  txt: string;
  fRothC: textfile;
  kHenry: real;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering RothC_Init');
  checksum := 0;
  {initialisation, some of these values will be reset later in this routine}
  with RothC do
  begin
    DPM2RPM := 0;
    ThicknessLayer := dlz; {layer thickness obtained from SoilGen}
    frEctoLitter := 0;
    frEndoLitter := 0;
    PlantResSUM := 0;
    ManureSUM := 0;
    totalmaxTSMD := 0;
    vegetatie := 'grassscrub';
    for m := 1 to 12 do
    begin
      totalaccTSMD[m] := 0; {accumulated moisture deficit}
      totalPlantResiduesIN[m] := 0;
      totalManureIN[m] := 0;
    end;
    for i := 1 to kk do
    begin
      {C-pools}
      DPMpool[i] := 0;
      RPMpool[i] := 0;
      BIOpool[i] := 0;
      HUMpool[i] := 0;
      IOMpool[i] := 0;
      CO2pool[i] := 0;
      {delta14C pools}
      DPMXpool[i] := 0;
      RPMXpool[i] := 0;
      BIOXpool[i] := 0;
      HUMXpool[i] := 0;
      IOMXpool[i] := 0;
      CO2Xpool[i] := 0;
      {init with values from calling main program}
      if i = 1 then
      begin
        DPMpool[i] := untGUI.DatayearIn.RothC.ectoDPMpool;
        RPMpool[i] := untGUI.DatayearIn.RothC.ectoRPMpool;
        BIOpool[i] := untGUI.DatayearIn.RothC.ectoBIOpool;
        HUMpool[i] := untGUI.DatayearIn.RothC.ectoHUMpool;
        IOMpool[i] := untGUI.DatayearIn.RothC.ectoIOMpool;
        CO2pool[i] := untGUI.DatayearIn.RothC.ectoCO2pool;
        if OK14c then
        begin
          DPMXpool[i] := untGUI.DatayearIn.RothC.ectoDPMXpool;
          RPMXpool[i] := untGUI.DatayearIn.RothC.ectoRPMXpool;
          BIOXpool[i] := untGUI.DatayearIn.RothC.ectoBIOXpool;
          HUMXpool[i] := untGUI.DatayearIn.RothC.ectoHUMXpool;
          IOMXpool[i] := untGUI.DatayearIn.RothC.ectoIOMXpool;
        end;
      end
      else
      begin
        DPMpool[i] := untGUI.DatayearIn.RothC.DPMpool[i - 1];
        RPMpool[i] := untGUI.DatayearIn.RothC.RPMpool[i - 1];
        BIOpool[i] := untGUI.DatayearIn.RothC.BIOpool[i - 1];
        HUMpool[i] := untGUI.DatayearIn.RothC.HUMpool[i - 1];
        IOMpool[i] := untGUI.DatayearIn.RothC.IOMpool[i - 1];
        CO2pool[i] := untGUI.DatayearIn.RothC.CO2pool[i - 1];
        if OK14c then
        begin
          DPMXpool[i] := untGUI.DatayearIn.RothC.DPMXpool[i - 1];
          RPMXpool[i] := untGUI.DatayearIn.RothC.RPMXpool[i - 1];
          BIOXpool[i] := untGUI.DatayearIn.RothC.BIOXpool[i - 1];
          HUMXpool[i] := untGUI.DatayearIn.RothC.HUMXpool[i - 1];
          IOMXpool[i] := untGUI.DatayearIn.RothC.IOMXpool[i - 1];
          CO2Xpool[i] := untGUI.DatayearIn.RothC.CO2Xpool[i - 1];
        end;
      end;
      {attached pools: ca,mg,na,k,cl,so4,hco3,co3 and balion}
      for chem := 1 to nchem do
      begin
        CHEMDPMpool[chem, i] := 0;
        CHEMRPMpool[chem, i] := 0;
        CHEMBIOpool[chem, i] := 0;
        CHEMHUMpool[chem, i] := 0;
        CHEMIOMpool[chem, i] := 0;
        CHEMCO2pool[chem, i] := 0;
      end;
      ClayPerc[i] := 0;
      pCO2_r[i] := max(untGUI.pCO2atm, pCO2[i]);  {pCO2 from SoilGen}
      fracManureIN[i] := 0;
      fracPlantIN[i] := 0;
      totalaccTSMD[1] := totalaccTSMD[1] + accTSMD[i];
      {add moisture deficit profile of previous year}
      for m := 1 to 12 do
      begin
        SoilCover[m] := 0;
        RDepth[m] := 0;
        Rainfall[m] := 0;
        EvapoPan[m] := 0;
        PlantresiduesIN[m][i] := 0;
        ManureIN[m][i] := 0;
        Temperatures[m][i] := 0;
      end;
    end;
  end;
  system.Assign(fRothC, RCName);
  system.reset(fRothC);
  {read from file with name like <RC*.>, where * might be <000001>}
  {general data: vegetation, LitterInput, manureInput
                 from vegetation derive:
                 * RothC.DPM2RPM ,
                 * RothC.frEctoLitter: fraction of litter added aboveground (ecto)
                 * RothC.frEndoLitter: fraction of litter added belowground (endo)}
  repeat
    system.readln(fRothC, txt)
  until pos('[general data]', txt) > 0;
  with RothC do
  begin
    system.readln(fRothC, vegetatie);
    system.Readln(fRothC, PlantResSUM, ManureSUM);
    if pos('agriculture', vegetatie) > 0 then
    begin
      DPM2RPM := DPM2RPM_agriculture;
      frEctoLitter := frEctoLitter_agriculture;
      frEndoLitter := frEndoLitter_agriculture;
      relCa := relCa_Agriculture;
      relMg := relMg_Agriculture;
      relNa := relNa_Agriculture;
      relK := relK_Agriculture;
      relAl := relAl_Agriculture;
    end;
    if pos('grassscrub', vegetatie) > 0 then
    begin
      DPM2RPM := DPM2RPM_grassscrub;
      frEctoLitter := frEctoLitter_grassscrub;
      frEndoLitter := frEndoLitter_grassscrub;
      relCa := relCa_Grassscrub;
      relMg := relMg_Grassscrub;
      relNa := relNa_Grassscrub;
      relK := relK_Grassscrub;
      relAl := relAl_Grassscrub;
    end;
    if pos('deciduouswood', vegetatie) > 0 then
    begin
      DPM2RPM := DPM2RPM_deciduouswood;
      frEctoLitter := frEctoLitter_deciduouswood;
      frEndoLitter := frEndoLitter_deciduouswood;
      relCa := relCa_Deciduouswood;
      relMg := relMg_Deciduouswood;
      relNa := relNa_Deciduouswood;
      relK := relK_Deciduouswood;
      relAl := relAl_Deciduouswood;
    end;
    if pos('coniferouswood', vegetatie) > 0 then
    begin
      DPM2RPM := DPM2RPM_coniferouswood;
      frEctoLitter := frEctoLitter_coniferouswood;
      frEndoLitter := frEndoLitter_coniferouswood;
      relCa := relCa_Coniferouswood;
      relMg := relMg_Coniferouswood;
      relNa := relNa_Coniferouswood;
      relK := relK_Coniferouswood;
      relAl := relAl_Coniferouswood;
    end;
  end;
  {profile data: RothC.DPMpool, *RPMpool, *BIOpool, *HUMpool, *IOMpool, *accTSMD}
  {accTSMD being the accumulated topsoil moisture deficit from previous year.
   It should be a negative number (=a deficit) or the value <0> !}
  system.reset(fRothC);
  repeat
    system.readln(fRothC, txt)
  until pos('[profile data]', txt) > 0;
  system.Readln(fRothC);
  for i := 1 to kk - 1 do
    with RothC do
    begin
      system.Read(fRothC, dum, DPMpool[i], RPMpool[i], BIOpool[i],
        HUMpool[i], IOMpool[i], CO2pool[i],
        accTSMD[i]);
      for chem := 1 to nchem do
        system.Read(fRothC, CHEMDPMpool[chem, i], CHEMRPMpool[chem, i],
          CHEMBIOpool[chem, i], CHEMHUMpool[chem, i],
          CHEMIOMpool[chem, i], CHEMCO2pool[chem, i]);
      system.readln(fRothC);
      {initial RothC.pCO2_r[i] set equal to value set in SoilGen}
      pCO2_r[i] := max(untGUI.pCO2atm, pCO2[i]);
    end;
  {monthly inputs: (profiles of) RothC.PlantresiduesIN, *ManureIN}
  {assumptions: 1. Manure all input in toplayer (ectorganic);
                2. Plantresidue input divided over ecto/endo organic
                   according to the RothC.frEctoLitter and RothC.frEndoLitter;
                   Ecto-organic input in layer 1;
                3. Endo-organic Litterinput distribution over the profile mirrors
                   the root density fraction distribution}
  system.reset(fRothC);
  repeat
    system.readln(fRothC, txt)
  until pos('[monthly inputs]', txt) > 0;
  system.Readln(fRothC);
  for m := 1 to 12 do
    with RothC do
    begin
      system.readln(fRothC, dum, totalPlantResiduesIN[m], totalManureIN[m]);
      {general: PlantResSum: Per month:
      1: split in ecto and endo-organic,
      2: distribute over depth according to rootdensityprofile}
      RothC.PlantresiduesIN[m][1] :=
        RothC.frEctoLitter * RothC.PlantResSum * RothC.totalPlantResiduesIN[m] / 100;
      ManureIN[m][1] := ManureSum * totalManureIN[m] / 100;
      for i := 2 to kk - 1 do
      begin
        fracPlantIN[i] := rdf[i];
        PlantresiduesIN[m][i] :=
          frEndoLitter * PlantResSum * fracPlantIN[i] * totalPlantResiduesIN[m] / 100;
        ManureIN[m][i] := 0;
      end;
    end;
  closefile(fRothC);
  {in PlantResiduesIN[m][i] we have now the litter input (ton C/ha)
   * per month m and
   * per compartment i, where compartment i=1 is ectoorganic
     and the other compartments endorganic}
  {Now obtain from SoilGen-input file <LC*.> (already read in READC)}
  {max. root depth (=max. depth where decomposition can taken place
   since it is max depth of root litter input).
   NB: it is assumed that roots do not grow vertically within the year,
   so rdf[i] gives root density function for whole year}
  for i := 2 to kk - 1 do
    if rdf[i] > 0 then
      maxrootcomp := i - 1;
  {now set the scaling factor for the max rooting depth that is connected to the crop}
  for i := 1 to NCROP do
    RDEP[i] := (maxrootcomp - 1) * RothC.ThicknessLayer / 1300;
  {the max rooting depth in the corn root system growth routine will now be changed from
   the default 1300 mm at root maturity to actual rooting depth}
  for i := 2 to kk - 1 do
  begin
    {profile data: RothC.ClayPerc. Attention: rescale so that Clay+Silt+Sand=100}
    RothC.ClayPerc[i] := 100 * Clay[i] / (100 - 1.7 * OC[i]); {markOC}
    if (i - 1) <= maxrootcomp then
      RothC.totalmaxTSMD := RothC.totalmaxTSMD -
        (0.1 * RothC.ThicknessLayer) *
        (20 + 1.3 * RothC.ClayPerc[i] - 0.01 * RothC.ClayPerc[i] * RothC.ClayPerc[i])
        / 23;
  end;
  {monthly inputs: RothC.Rainfall, RothC.EvapoPan
   These are to be derived from the read input for SoilGen, i.e.
   the variables
   * AIRRBG[1..NIRRIG] for the day on which the rain falls, and
   * IRRIGA[1..NIRRIG] for the amount of rain;
   * PAN[1..NWEEK] for the amount of pan evaporation in the week}
  for i := 1 to NIRRIG do
  begin
    m := ClassifyMonth(AIRRBG[i]);
    RothC.Rainfall[m] := RothC.Rainfall[m] + IRRIGA[i];
  end;
  for i := 1 to 365 do
  begin
    w := trunc(i / 7) + 1;
    m := ClassifyMonth(i);
    RothC.EvapoPan[m] := RothC.EvapoPan[m] + PAN[w] / 7;
  end;
  {calculate accumulated moisture deficit}
  for m := 1 to 12 do
  begin
    RothC.totalaccTSMD[m] := max(RothC.totalmaxTSMD,
      RothC.totalaccTSMD[m] + RothC.Rainfall[m] -
      RothC.EvapoPan[m]);
    if RothC.totalaccTSMD[m] > 0 then
      RothC.totalaccTSMD[m] := 0; {only true deficits}
    {these deficits are to be distributed over the soilprofile in the RothC_Monthly routine}
  end;
  wwriteln('C-data read');
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving RothC_Init');
end;

function ClassifyMonth(realday: real): integer;
{assign month to Julian day (called from RothC_daily)}
var
  MON2, MON, MONL: array[1..12] of integer;
  i, m: integer;
begin
  m := 1;
  {if (TRACE = 1) then}
    singletracewriteln(' Entering ClassifyMonth');
  MON[1] := 31;
  MON[2] := 59;
  MON[3] := 90;
  MON[4] := 120;
  MON[5] := 151;
  MON[6] := 181;
  MON[7] := 212;
  MON[8] := 243;
  MON[9] := 273;
  MON[10] := 304;
  MON[11] := 334;
  MON[12] := 365;
  MONL[1] := 31;
  MONL[2] := 60;
  MONL[3] := 91;
  MONL[4] := 121;
  MONL[5] := 152;
  MONL[6] := 182;
  MONL[7] := 213;
  MONL[8] := 244;
  MONL[9] := 274;
  MONL[10] := 305;
  MONL[11] := 335;
  MONL[12] := 366;
  if (YEAR mod 4 = 0) then
    MON2 := MONL
  else
    MON2 := MON;
  for i := 12 downto 1 do
    if realday <= MON2[i] then
      m := i;
  ClassifyMonth := m;
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving ClassifyMonth');
end;

procedure RothC_Exit;
{write C-and associated element pools to outputfile RC*.out}
var
  frctemp: textfile;
  i, m, chem: integer;
  tmp: real;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering RothC_Exit');
  {produce the file <RC*.out>}
  {This is an incomplete input file for the next year-run.
   Other data will be generated by the untGUI-shell. These other data are:
   general data: vegetation type (to obtain a value for dpm/rpm ratio)
   montly inputs: per month per layer the PlantresiduesIN and ManureIN}
  RothC.pCO2_r[1] := untGUI.pCO2atm; {RothC.CO2pool[1]:=0; }
  system.Assign(frctemp, RCName + '.out');
  system.Rewrite(frctemp);
  system.Writeln(frctemp, '[profile data]');
  system.Writeln(frctemp, '[layer       DPMpool         RPMpool         BIOpool' +
    '         HUMpool         IOMpool         CO2pool         accTSMD]        ',
    'Chemical pools DPM-RPM-BIO-HUM-IOM-CO2 repeated for ca	mg	na	k	cl	so4	hco3	co3	balion H OH Al Org HOrg H2Org DOC');
  for i := 1 to kk - 1 do
    with RothC do
    begin
      system.Write(frctemp, i - 1: 3, '   ', DPMpool[i]: 15: 12, ' ', RPMpool[i]: 15: 12, ' ',
        BIOpool[i]: 15: 12, ' ', HUMpool[i]: 15: 12, ' ',
        IOMpool[i]: 15: 12, ' ', CO2pool[i]: 15: 12, ' ',
        accTSMD[i]: 7: 2, ' ', pCO2[i]: 15: 12);
      for chem := 1 to nchem do
        system.Write(frctemp, ' ', CHEMDPMpool[chem, i]: 15: 5, ' ',
          CHEMRPMpool[chem, i]: 15: 5, ' ',
          CHEMBIOpool[chem, i]: 15: 5, ' ',
          CHEMHUMpool[chem, i]: 15: 5, ' ',
          CHEMIOMpool[chem, i]: 15: 5, ' ',
          CHEMCO2pool[chem, i]: 15: 5, ' ');
      system.writeln(frctemp);
    end;
  closefile(frctemp);
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving RothC_Exit');
end;

function tonha_to_pco2(ton, kh, pores: real): real;
{void: conversion ton C per ha to pCO2. Not used.}
{ton=ton C/ha;
 kh=Henry's law constant for CO2 and current temperature;
 pores is the mm thickness of the current soil layer equivalent to porosity}
var
  temp1, temp2, temp3: real;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering tonha_to_pco2');
  {convert from ton C/ha to partial pressure pCO2 in the pores of the current layer}
  temp1 := ton * 1000 / (10000 * pores); {temp1 is kg C/dm3 soilpores}
  temp2 := temp1 * 1000 / 12;          {temp2 is mol C (and CO2) /dm3 soilpores}
  temp3 := temp2 * kh;               {temp3 is pCO2 by using Henry's law}
  tonha_to_pco2 := temp3;
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving tonha_to_pco2');
end;

function pco2_to_tonha(p, kh, pores: real): real;
{backconversion pCO2 to ton C per ha (called from unGUI.bttnRunClick)}
{p=PCO2;
 kh=Henry's law constant for CO2 and current temperature;
 pores is the mm thickness of the current soil layer equivalent to porosity}
var
  temp1, temp2, temp3: real;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering pco2_to_tonha');
  {convert from partial pressure pCO2 in the pores of the current layer to ton C/ha}
  temp1 := p / kh;                   {temp1 is mol C (and CO2) /dm3 soilpores}
  temp2 := temp1 * 12 / 1000;          {temp2 is kg C /dm3 soilpores}
  temp3 := 10000 * pores * temp2 / 1000; {temp3 is ton C/ha}
  pco2_to_tonha := temp3;
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving pco2_to_tonha');
end;

procedure RothC_Daily(month: integer);
{C-submodel on a daily basis}
var
  i, chem, biolactlayer: integer;
  ma, mb, mc, mk, accTSMD, maxTSMD: profile;
  mt, sumTG, frDPM, frRPM, x, x2, lossDPM, lossRPM, lossBIO, lossHUM, plusBIOa,
  plusHUMa, plusCO2a, plusBIOb, plusHUMb, plusCO2b, plusBIOc, plusHUMc, plusCO2c, plusBIObX,
  frdaymonth, Fk_DPM, Fk_RPM, Fk_BIO, Fk_HUM, lossDPMX, lossRPMX, lossBIOX, plusBIOX, plusBIOaX,
  plusBIOcX, lossHUMX, plusHUMaX, plusHUMbX, plusHUMcX, plusCO2aX, plusCO2bX, plusCO2cX,
  x3RPM, x3DPM, x3BIO, x3HUM, porosityperc: real;
  clossDPM, clossRPM, clossBIO, clossHUM, cplusBIOa, cplusHUMa, cplusCO2a,
  cplusBIOb, cplusHUMb, cplusCO2b, cplusBIOc, cplusHUMc, cplusCO2c: rMM;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering RothC_Daily');
{1..5: obtain data to determine and modify rate coefficients;
 6..8: add montly litter and manure over the soil layers of interest;
 9..11: apply decomposition routine from RothC26.3 model}
  {1. Temperatures
      Montly average profiles for RothC.Temperatures are already available from SoilGen.
      These values were obtained in subroutine Mainc2. Search text for <YYY1 and YYY2>.}
  {2. Moisture deficits
      Obtained from SoilGen cumulative (end-of-month) moisture deficit (mm) per layer
      this implies a distribution of RothC.totalaccTSMD[month]
      over the total depth of the biologically active layer.
      It is assumed that this total depth equals the rooted depth.
      Below this depth, the moisture deficit is assumed 0 (thus no effect on decomposition rates};
  {2a. First obtain from SoilGen the RothC.Rdepth (MaxRootComp*ThicknessLayer),
       where MaxRootComp was determined in RothC_Init, and then
       calculate the thickness of the biologically active layer}
  RothC.Rdepth[month] := MaxRootComp * RothC.ThicknessLayer;
  biolactlayer := MaxRootComp + 1;
  {biolactlayer:=kk; }{if decomposition is to take place in all layers below rooting zone as well}
  {NB: 1 added because layer 1=index 2}
  for i := 1 to kk do
    RothC.accTSMD[i] := 0;
  {2b. Now distribute the moisture deficit over the biologically active layer.
       It is assumed that the vertical distribution of the moisture deficit
       mirrors the air-filled porefraction distribution at the moment
       that the RothC_Monthly subroutine is called from Mainc2}
  if RothC.totalaccTSMD[month] < 0 then
  begin
    sumTG := 0;
    for i := 2 to biolactlayer do
      sumTG := sumTG + TG[i];
    for i := 2 to biolactlayer do
      RothC.accTSMD[i] := RothC.totalaccTSMD[month] * TG[i] / sumTG;
  end;
  {3. SoilCover
      obtain from SoilGen the current RothC.SoilCover (CRPCOV[1])}
  RothC.SoilCover[month] := CRPCOV[1];
  {4. Calculate rate modifying factors}
  for i := 1 to kk - 1 do
  begin
    {beware: if i=1 then this is the litter layer}
    {4a. rate modifier: ma=for temperature}
    ma[i] := 47.9 / (1 + exp(106 / (max(-10, TEM[i]) + 18.3)));
    {4b. rate modifier: mb for moisture}
    maxTSMD[i] := (-0.1 * RothC.ThicknessLayer) *
      (20 + 1.3 * RothC.ClayPerc[i] - 0.01 * RothC.ClayPerc[i] * RothC.ClayPerc[i]) / 23;
    if abs(RothC.accTSMD[i]) < abs(0.444 * maxTSMD[i]) then
      mb[i] := 1
    else
      mb[i] := 0.2 + 0.8 * (maxTSMD[i] - RothC.accTSMD[i]) / (0.556 * maxTSMD[i]);
    mb[i] := min(max(0.2, mb[i]), 1);
    {should remain within bounds even under very dry conditions}
    {4c. rate modifier: mc for soil cover}
    if RothC.SoilCover[month] > 0 then
      mc[i] := 0.6
    else
      mc[i] := 1;
  end;
  {5. rate modifier for time (decomposition rates are for 1 year)}
  mt := 1 / 365;
  {6. Distribute the incoming litter over the soil layers}
  {6a. manure all in ectoorganic layer: set in RothC_INIT}
  {6b. litter for specified fraction ecto-organic: set in RothC_INIT}
  {6c. endo-organic litter fraction distributed over the rooted zone
       (mirrors the actual rooting distribution pattern RDF[2..kk-1])
       : set in RothC_INIT}
  {7. distribute incoming plant material over the pools
      DPM (decomposable plant material) and
      RPM (resistant plant material) using the DPM/RPM ratio typical for the vegetation.}
  frDPM := RothC.DPM2RPM / (RothC.DPM2RPM + 1);
  frRPM := 1 - frDPM;
  frDayMonth := 1 / (365 / 12);
  for i := 1 to kk - 1 do
  begin
    {calculate porosity percentage of the total soil, including stones.
    Only if the value is below 15% in the topsoil (upper 5 layers) then add new RPM and DPM to extorganic layer}
    if (i>1)and(i<6) then porosityperc:=(1-rho[i]/parden[i])*(100-stoneperc[i]) else porosityperc:=16 {dummy value};
    {7a additions from PlantResidues to DPM and RPM, but only if the porosity >=15% else all goes to ectorganic layer ("peat")}
    if porosityperc>=15 then RothC.DPMpool[i] := RothC.DPMpool[i] + frDayMonth * frDPM * RothC.PlantresiduesIN[month][i]
                        else RothC.DPMpool[1] := RothC.DPMpool[1] + frDayMonth * frDPM * RothC.PlantresiduesIN[month][i] ;
    if porosityperc>=15 then RothC.RPMpool[i] := RothC.RPMpool[i] + frDayMonth * frRPM * RothC.PlantresiduesIN[month][i]
                        else RothC.RPMpool[1] := RothC.RPMpool[1] + frDayMonth * frRPM * RothC.PlantresiduesIN[month][i];
    {7aX: effect of 14C pools}
    {d14Cvalue is taken from a table for all yearBP (relative to 1950) read in the main program}
    if OK14c then
    begin
      if porosityperc>=15 then RothC.DPMXpool[i]:=RothC.DPMXpool[i]+frDayMonth*frDPM*RothC.PlantresiduesIN[month][i]*d14Cvalue
                          else RothC.DPMXpool[1]:=RothC.DPMXpool[1]+frDayMonth*frDPM*RothC.PlantresiduesIN[month][i]*d14Cvalue;
      if porosityperc>=15 then RothC.RPMXpool[i]:=RothC.RPMXpool[i]+frDayMonth*frRPM*RothC.PlantresiduesIN[month][i]*d14Cvalue
                          else RothC.RPMXpool[1]:=RothC.RPMXpool[1]+frDayMonth*frRPM*RothC.PlantresiduesIN[month][i]*d14Cvalue;
    {dummy for the moment: input to pools from external sources (but not manure for HUM!
    RothC.HUMXpool[i]:=RothC.HUMXpool[i]+frDayMonth*EXTRAINPUT[month][i]*d14Cvalue;
    RothC.BIOXpool[i]:=RothC.BIOXpool[i]+frDayMonth*EXTRAINPUT[month][i]*d14Cvalue;
    }
    end;
  end;
  {7b additions from PlantResidues to DPM and RPM
      the attached ions ca,mg,na,k,cl,so4,hco3,co3 and balion.
      This is done by adding the daily uptake of each chemical via the
      transpiration stream to either the soil compartments (endorganic)
      or the litter layer (ectorganic, node 1)}
  for i := 2 to kk - 1 do
  begin
    for chem := 1 to nchem do
    begin
      CHEMDPMpool[chem, i] := CHEMDPMpool[chem, i] + frDPM * RothC.frEndoLitter * DUSE[chem, i];
      CHEMDPMpool[chem, 1] := CHEMDPMpool[chem, 1] + frDPM * RothC.frEctoLitter * DUSE[chem, i];
      CHEMRPMpool[chem, i] := CHEMRPMpool[chem, i] + frRPM * RothC.frEndoLitter * DUSE[chem, i];
      CHEMRPMpool[chem, 1] := CHEMRPMpool[chem, 1] + frRPM * RothC.frEctoLitter * DUSE[chem, i];
    end;
  end;
  {8. additions from manure to DPM, RPM and HUM. Fixed fractions.
      Attention: Mineralisation of ca,mg,na,k,cl,so4,hco3,co3 and balion from
      the manure is NOT implemented; is assumed to be given as anorganic fertilizer input}
  RothC.DPMpool[1] := RothC.DPMpool[1] + 0.49 * frDayMonth * RothC.ManureIN[month][1];
  RothC.RPMpool[1] := RothC.RPMpool[1] + 0.49 * frDayMonth * RothC.ManureIN[month][1];
  RothC.HUMpool[1] := RothC.HUMpool[1] + 0.02 * frDayMonth * RothC.ManureIN[month][1];
  {8X: effect of 14C pools}
  {d14Cvalue is taken from a table for all yearBP (relative to 1950) read in the main program}
  if OK14c then
  begin
    RothC.DPMXpool[1] := RothC.DPMXpool[1] + 0.49 * frDayMonth * RothC.ManureIN[month]
      [1] * d14Cvalue;
    RothC.RPMXpool[1] := RothC.RPMXpool[1] + 0.49 * frDayMonth * RothC.ManureIN[month]
      [1] * d14Cvalue;
    RothC.HUMXpool[1] := RothC.HUMXpool[1] + 0.02 * frDayMonth * RothC.ManureIN[month]
      [1] * d14Cvalue;
  end;
  {9. Decompose the pools DPM and RPM to CO2, BIO and HUM. Don't update yet!
   10.Decompose the BIO and HUM pools to CO2, BIO and HUM.
   11.Mass balance.}
  for i := 1 to kk - 1 do
  begin
    x := facscaleCO2BIOHUM * (1.85 + 1.60 * exp(-0.0786 * RothC.ClayPerc[i]));
    {x3RPM/x3DPM/x3BIO/x3HUM are experimental extra modifiers.
     They could for instance take into account
     that soils with a high total reserve of bases have faster decomposition.
     See DOI: 10.1038/NGEO2516 Nature Geoscience by Doettrl et al., 2015:
     "Soil carbon storage controlled by interactions between geochemistry and climate".
     By default x3*=1}
    if i=1 then
    begin x3DPM:=1; x3RPM:=1; x3BIO:=1; x3HUM:=1 end
    else
    begin
      {now recalc the protection factors x3 based on the settings and
       the current status of relevant soil variables}
      if ((untDecayProtection.frmDecayProtection.chckbxF1.checked) or
          (untDecayProtection.frmDecayProtection.chckbxF2.checked) or
          (untDecayProtection.frmDecayProtection.chckbxF3.checked) or
          (untDecayProtection.frmDecayProtection.chckbxF4.checked))
      then CalcProtection(i)
      else Fprot[i]:=1;
      x3DPM:=Fprot[i]; x3RPM:=Fprot[i]; x3BIO:=Fprot[i]; x3HUM:=Fprot[i];
    end;

    if i <= biolactlayer then
      x2 := ma[i] * mb[i] * mc[i] * mt  {pool-independent rate modifiers}
    else
      x2 := 0;                    {outside biologically active layer = no decomposition}
    RothC.x2[i]:=x2/mt;           {exclude year>day conversion of rates}
    if (RothC.DPMpool[i]+RothC.RPMpool[i]+RothC.BIOpool[i]+RothC.HUMpool[i])>0
    then RothC.x3[i]:=(x3DPM*RothC.DPMpool[i]+x3RPM*RothC.RPMpool[i]+x3BIO*RothC.BIOpool[i]+x3HUM*RothC.HUMpool[i])/
                     (RothC.DPMpool[i]+RothC.RPMpool[i]+RothC.BIOpool[i]+RothC.HUMpool[i])
                     {weighted average of the x3 for all 4 pools}
    else RothC.x3[i]:=1;
    {9.}
    lossDPM := -RothC.DPMpool[i] * (1 - exp(-x2 * x3DPM* k_DPM));
    lossRPM := -RothC.RPMpool[i] * (1 - exp(-x2 * x3RPM* k_RPM));
    plusBIOa := -(1 / (BIO2HUMratio + 1)) * (1 / (x + 1)) * (lossDPM + lossRPM);
    plusHUMa := -(BIO2HUMratio / (BIO2HUMratio + 1)) * (1 / (x + 1)) * (lossDPM + lossRPM);
    plusCO2a := -(x / (x + 1)) * (lossDPM + lossRPM);
    {9X: balance terms for the 14C pools}
    {extra rate corrections for delta14C: FK_RPM and FK_DPM}
    if OK14c then
    begin
      if yearBP < untGUI.frmMain.spndtMultipleYears.Value then
      begin
        if RothC.DPMpool[i] = 0 then
          Fk_DPM := d14Cvalue
        else
          Fk_DPM := RothC.DPMXpool[i] / RothC.DPMpool[i];
        if RothC.RPMpool[i] = 0 then
          Fk_RPM := d14Cvalue
        else
          Fk_RPM := RothC.RPMXpool[i] / RothC.RPMpool[i];
      end
      else
      begin
        Fk_DPM := d14Cvalue;
        Fk_RPM := d14Cvalue;
      end;
      lossDPMX := lossDPM * Fk_DPM;
      lossRPMX := lossRPM * Fk_RPM;
      plusBIOaX := -(1 / (BIO2HUMratio + 1)) * (1 / (x + 1)) * (lossDPMX + lossRPMX);
      plusHUMaX := -(BIO2HUMratio / (BIO2HUMratio + 1)) * (1 / (x + 1)) * (lossDPMX + lossRPMX);
      plusCO2aX := -(x / (x + 1)) * (lossDPMX + lossRPMX);
    end;
    {10.}
    lossBIO := -RothC.BIOpool[i] * (1 - exp(-x2 * x3BIO* k_BIO));
    plusBIOb := -(1 / (BIO2HUMratio + 1)) * (1 / (x + 1)) * (lossBIO);
    plusHUMb := -(BIO2HUMratio / (BIO2HUMratio + 1)) * (1 / (x + 1)) * (lossBIO);
    plusCO2b := -(x / (x + 1)) * (lossBIO);
    lossHUM := -RothC.HUMpool[i] * (1 - exp(-x2 * x3HUM * k_HUM));
    plusBIOc := -(1 / (BIO2HUMratio + 1)) * (1 / (x + 1)) * (lossHUM);
    plusHUMc := -(BIO2HUMratio / (BIO2HUMratio + 1)) * (1 / (x + 1)) * (lossHUM);
    plusCO2c := -(x / (x + 1)) * (lossHUM);
    checksum := checksum + lossDPM + lossRPM + lossHUM + lossBIO + plusHUMa + plusHUMb + plusHUMc +
      plusBIOa + plusBIOb + plusBIOc + plusCO2a + plusCO2b + plusCO2c;  {should be 0}
    {10X: additional balance terms for the 14C pools}
    {extra rate corrections for delta14C: FK_BIO and FK_HUM}
    if OK14c then
    begin
      if yearBP < untGUI.frmMain.spndtMultipleYears.Value then
      begin
        if RothC.BIOpool[i] = 0 then
          Fk_BIO := d14Cvalue
        else
          Fk_BIO := RothC.BIOXpool[i] / RothC.BIOpool[i];
        if RothC.HUMpool[i] = 0 then
          Fk_HUM := d14Cvalue
        else
          Fk_HUM := RothC.HUMXpool[i] / RothC.HUMpool[i];
      end
      else
      begin
        Fk_BIO := d14Cvalue;
        Fk_HUM := d14Cvalue;
      end;
      lossBIOX := lossBIO * Fk_BIO;
      plusBIObX := plusBIOb * Fk_BIO;
      plusHUMbX := plusHUMb * Fk_HUM;
      plusCO2bX := plusCO2b * Fk_BIO;
      lossHUMX := lossHUM * Fk_HUM;
      plusBIOcX := plusBIOc * Fk_HUM;
      plusHUMcX := plusHUMc * Fk_HUM;
      plusCO2cX := plusCO2c * Fk_HUM;
    end;
    for chem := 1 to nchem do
    begin
      {9a}
      clossDPM[chem] := -CHEMDPMpool[chem, i] * (1 - exp(-x2 * x3DPM* k_DPM));
      clossRPM[chem] := -CHEMRPMpool[chem, i] * (1 - exp(-x2 * x3RPM* k_RPM));
      cplusBIOa[chem] := -(1 / (BIO2HUMratio + 1)) * (1 / (x + 1)) * (clossDPM[chem] + clossRPM[chem]);
      cplusHUMa[chem] := -(BIO2HUMratio / (BIO2HUMratio + 1)) * (1 / (x + 1)) *
        (clossDPM[chem] + clossRPM[chem]);
      cplusCO2a[chem] := -(x / (x + 1)) * (clossDPM[chem] + clossRPM[chem]);
      {10a}
      clossBIO[chem] := -CHEMBIOpool[chem, i] * (1 - exp(-x2 * x3BIO * k_BIO));
      ;
      cplusBIOb[chem] := -(1 / (BIO2HUMratio + 1)) * (1 / (x + 1)) * clossBIO[chem];
      cplusHUMb[chem] := -(BIO2HUMratio / (BIO2HUMratio + 1)) * (1 / (x + 1)) * clossBIO[chem];
      cplusCO2b[chem] := -(x / (x + 1)) * clossBIO[chem];
      ;
      clossHUM[chem] := -CHEMHUMpool[chem, i] * (1 - exp(-x2 * x3HUM* k_HUM));
      cplusBIOc[chem] := -(1 / (BIO2HUMratio + 1)) * (1 / (x + 1)) * clossHUM[chem];
      cplusHUMc[chem] := -(BIO2HUMratio / (BIO2HUMratio + 1)) * (1 / (x + 1)) * clossHUM[chem];
      cplusCO2c[chem] := -(x / (x + 1)) * clossHUM[chem];
    end;
    {11.}
    with RothC do
    begin
      DPMpool[i] := DPMpool[i] + lossDPM;
      RPMpool[i] := RPMpool[i] + lossRPM;
      BIOpool[i] := BIOpool[i] + plusBIOa + lossBIO + plusBIOb + plusBIOc;
      {consider BIOpool as being partly biomass, partly DOC and consider DOC immobile
       then DOCa can be calculated using an emperical assessment by Zhang et al 2006
       via MBC=-477.99+8.14DOCa
       where DOCa, MBC are in mgC/kgSoil
       MBC=0.1*BIOpool*10^6/(DLZ*RHO)  (conversion BIOpool (ton C/ha) to mgC/kgSoil)
       DOCa=(MBC+467.99)/8.14
       DOC=DOCa*DLZ/(1000*RHO) (conversion mgC/kgSoil to mmol DOC-C/m2)
       Netto: DOC=((0.1*BIOpool*10^6/(DLZ*RHO)+467.99)/8.14)*DLZ/(1000*RHO)}
      DOC[i] := ((BIOpool[i] * 100000 / (DLZ * RHO[i]) + 467.99) / 8.14) * DLZ / (1000 * RHO[i]);
      HUMpool[i] := HUMpool[i] + plusHUMa + lossHUM + plusHUMb + plusHUMc;
      CO2pool[i] := CO2pool[i] + plusCO2a + plusCO2b + plusCO2c;
      DailyCO2PoolIncrease[i] := max(0, (plusCO2a + plusCO2b + plusCO2c));
      if OK14c then
      begin
        {11X: update of 14C ("X")pools}
        DPMXpool[i] := DPMXpool[i] + lossDPMX;
        RPMXpool[i] := RPMXpool[i] + lossRPMX;
        BIOXpool[i] := BIOXpool[i] + plusBIOaX + lossBIOX + plusBIObX + plusBIOcX;
        HUMXpool[i] := HUMXpool[i] + plusHUMaX + lossHUMX + plusHUMbX + plusHUMcX;
        CO2Xpool[i] := CO2Xpool[i] + plusCO2aX + plusCO2bX + plusCO2cX;
      end;
      {11a}
      for chem := 1 to nchem do
      begin
        CHEMDPMpool[chem, i] := CHEMDPMpool[chem, i] + clossDPM[chem];
        CHEMRPMpool[chem, i] := CHEMRPMpool[chem, i] + clossRPM[chem];
        CHEMBIOpool[chem, i] := CHEMBIOpool[chem, i] + cplusBIOa[chem] +
          clossBIO[chem] + cplusBIOb[chem] + cplusBIOc[chem];
        CHEMHUMpool[chem, i] := CHEMHUMpool[chem, i] + cplusHUMa[chem] +
          clossHUM[chem] + cplusHUMb[chem] + cplusHUMc[chem];
        CHEMCO2pool[chem, i] := cplusCO2a[chem] + cplusCO2b[chem] + cplusCO2c[chem];
         {nb: implicitly CHEMCO2pool[chem,i] has each day the initial value 0
              because it is assumed that mineralised chemicals are available immediately}
      end;
    end;
  end;
  {top compartment always in equilibrium with pCO2_r of 0.0003 bar;
   this value is used to reset the value for CO2pool[1]}
  RothC.pCO2_r[1] := untGUI.pCO2atm;
  pCO2[1] := untGUI.pCO2atm;
  {convert mineralization of ectorganic OM into chemicals to a fertilizer.
   conversion from mmol/m2,comp,day to mol/m2}
  for chem := 1 to nchem do
    FER[chem, ferno] := FER[chem, ferno] + CHEMCO2pool[chem, 1] / 1000;
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving RothC_Daily2');
end;

procedure CalcProtection(comp:integer);
{calculate effect of selected protection mechanisms on decay of OM (called from RothC_daily)}
begin
  {protection factors F1, F2, F3, F4 are calculated, if desired via settings in untGUI}
  if (untDecayProtection.frmDecayProtection.chckbxF1.checked)
  then Fp1[comp]:=untDecayProtection.A1+(1-untDecayProtection.A1)*minpools[10,comp]/
                (minpools[11,comp]+minpools[12,comp]+minpools[6,comp]+minpools[10,comp])
  else Fp1[comp]:=1;
  if (untDecayProtection.frmDecayProtection.chckbxF2.checked)
  then Fp2[comp]:=exp(-untDecayProtection.A2*(SE[12,comp]+SE[13,comp]+SE[14,comp]))
  else Fp2[comp]:=1;
  if (untDecayProtection.frmDecayProtection.chckbxF3.checked)
  then Fp3[comp]:=max(1-untDecayProtection.A3a*(XSE[1][comp]+XSE[2][comp]+XSE[3][comp]+XSE[4][comp])/
                                               (XSE[1][comp]+XSE[2][comp]+XSE[3][comp]+XSE[4][comp]+XSE[10][comp]+XSE[12][comp]),
                      1-untDecayProtection.A3b*CALCITEA[comp]/(RHO[comp]*DLZ))
  else Fp3[comp]:=1;
  if (untDecayProtection.frmDecayProtection.chckbxF4.checked)
  then Fp4[comp]:=untDecayProtection.A4+(1-untDecayProtection.A4)*pmes[comp]/(pmes[comp]+pmic[comp])
  else Fp4[comp]:=1;
  {the over-all protection factor is calculated as desired via user settings}
  case untDecayProtection.frmDecayProtection.rdgrpFcombi.ItemIndex of
    0: Fprot[comp]:=min(Fp1[comp],min(Fp2[comp],min(Fp3[comp],Fp4[comp])));
    1: Fprot[comp]:=max(Fp1[comp],max(Fp2[comp],max(Fp3[comp],Fp4[comp])));
    2: Fprot[comp]:=Fp1[comp]*Fp2[comp]*Fp3[comp]*Fp4[comp];
  end;
end;

procedure CO2_diffusion;
{redistribute produced and present CO2 over the profile by diffusion}
{Inputs are taken from the momentary value of the simulation
 input: * profile of air-filled porosity    : TG[2..kk-1]
        * boundary conditions               : pCO2[1] and pCO2[kk] = 0.0003 Bar
        * initial pCO2-profile              : pCO2[2..kk-1]
        * daily profile of C-CO2 production : DailyCO2PoolIncrease[2..kk-1]
          (calculated for day by RothC)       (value 0 is used for nodes 1 and kk)}
var Ds,P: Profile; i:integer; stable:boolean; dt,d0,campbelb,e,e100,temperature:real;
begin
  {if (TRACE=1) then wwwriteln} singletracewriteln(' Entering CO2_Diffusion');
  {0. Initialization}
  stable:=false;
  {for boundary nodes}
  Ds[1]:=(10000*3600)*0.0000139*power(((273.16+TEM[1])/273.16),1.75)*1;
  {Ds[kk]:= to be set in procedure CO2_diff_Eq; will be equal to Ds[kk-1]}
  P[1]:=0;  P[kk]:=0;
  RothC.pCO2_r[1]:=untGUI.pCO2atm;
  {RothC.pCO2_r[kk]:=to be set in procedure CO2_diff_Eq}
  {calculations}
  for i:=2 to kk-1 do
  begin
   {1. Calculate apparent diffusivity in (cm2/h) using the air-filled porosity.
       Option 1: Anlauf et al, 1990: Models for processes in the soil, p.61
       Ds[i]:=3.6*EXP(9*TG[i]); }
    {  Option 2: Moldrulp et al, 2000 is currently implemented:
       D0 = Dx *(T/Tx)^1.75 * (px/p) with d0 in m2/s.
       converting to cm2/h and assuming p constant, this gives:}
    d0:=(10000*3600)*0.0000139*power(((273.15+TEM[i])/273.16),1.75)*1;
    e100:=TS[i]-THETA(i,-1000,RG[i],TS[i],AEV[i],BCAM[i],L[i],TM[i],HM[i]);
    e100:=max(0.001,e100); {to prevent numerical problems at low porosities}
    {Ds=Do*(2e100 ^3 + 0.04e100)*(e/e100)^(2+3b)}
    Ds[i]:=d0*(2*e100*e100*e100+0.04*e100)*power((max(TG[i],0.001)/e100),(2+3/BCAM[i]));
    {Additionally the effect of bioturbation is included:
     Singer et al (2001) report that the diffusion coefficient increased
     by a factor 8.9 in case of bioturbation by earthworms.
     This finding is applied for the compartments affected by bioturbation only}
     if untGUI.bioturbfile then
       if i-1<untGUI.dbot/dlz then Ds[i]:=Ds[i]*8.9;
   {2. Calculate CO2-production term in cm3 CO2/ cm2.h.compartment
                from DailyCO2PoolIncrease (ton C/ ha,day,compartment)
                a. tonC/ha,d,comp *100000*44/(12*24) yields mgCO2/m2,h,comp
                b. mgCO2/m2,h,comp *12/(22.24*10000) yields cm3 CO2/cm2,comp,h
                netto: DailyCO2PoolIncrease[i]*10*44/(24*22.24)}
    P[i]:=DailyCO2PoolIncrease[i]*10*44/(24*22.24);
    {3. Initial pCO2: copy from SoilGen}
    RothC.pCO2_r[i]:=pCO2[i];
  end;
  dt:=0.020; {twice the initial time step in hours for numerical solution}
  {4. Calculate CO2 diffusion. Decrease timestep if necessary.}
  repeat
    dt:=dt/2;
    CO2_diff_Eq(Ds,P,RothC.pCO2_r,dt,stable);
    if dt<1E-8 then
    begin {risk for infinite loop due to small dt: bypass the rest of the routine}
      stable:=true;
      {...and limit pCO2 to range[untGUI.pCO2atm;maxpCO2]   }
      for i:=2 to kk-1 do RothC.pCO2_r[i]:=max(untGUI.pCO2atm,min(RothC.pCO2_r[i],maxpCO2));
    end;
  until stable;
  for i:=2 to kk-1 do {if high pCO2 are simulated, usually when very
                       low air-filled porosities are present,
                       due to very wet or frozen soils:
                       assume immediate convective gas transport
                       towards atmosphere and limit pCO2 to maxpCO2 (eg 0.2) Bar}
  begin
     RothC.pCO2_r[i]:=min(RothC.pCO2_r[i],maxpCO2);
  end;
  {if (TRACE=1) then wwwriteln} singletracewriteln(' Leaving CO2_Diffusion');
end;

procedure CO2_diff_Eq(var Ds,P,calcpCO2: profile; dt:real; var stable:boolean);
{solve CO2-diffusion equation (called from CO2-diffusion)}
var ok:boolean; t:real; i:integer; localpCO2,localDs,localTG,localP: array[1..LL] of real;
{air-filled pore volume is minimized to 0.001 to prevent numerical instability in wet soils
 if this is not done, dt becomes iteratively smaller and performance slows down.}
begin
  for i:=1 to kk do localpCO2[i]:=0; {initialization}
  {if (TRACE=1) then wwwriteln} singletracewriteln(' Entering CO2_Diff_Eq');
  ok:=true; t:=0;
  for i:=1 to kk-1 do
  begin
    localpCO2[i]:=calcpCO2[i]; localDs[i]:=Ds[i]; localP[i]:=P[i]; localTG[i]:=TG[i];
  end;
  {if lower boundary= no diffusion of CO2 then it should be reset here:}
  {localpCO2[kk]:=localpCO2[kk-1];}
  {else if atmospheric conditions are assumed:}
  {localpCO2[kk]:=0.0003;}

  {in this version, we connect the BBC for water flow to that of CO2-diffusion}
  if BBC=2 then localpCO2[kk]:=untGUI.pCO2atm;     {free drainage=atmospheric }
  if BBC=3 then localpCO2[kk]:=localpCO2[kk-1];            {0-drainage=no diffusion}

  localDs[kk]:=Ds[kk-1]; localP[kk]:=0; localTG[kk]:=TG[kk-1];
  repeat
    t:=t+dt;
    for i:=2 to kk-1 do
    begin
      localpCO2[i]:=localpCO2[i]+                             {a: initial concentration this timestep}
                    localDs[i]*(dt/max(localTG[i],0.001))*
                          (localpCO2[i+1]-2*localpCO2[i]+localpCO2[i-1])/
                          (dlz*dlz)+
                                                              {b: diffusion term}
                    localP[i]*dt/(max(localTG[i],0.001)*dlz); {c: production term}
      if (localpCO2[i]<0)or(localpCO2[i]>1) then ok:=false; {numerical instability}
    end;
    {localpCO2[kk] is not changed: stable pCO2}
    {localpCO2[kk]:=localpCO2[kk-1]; lower boundary: no diffusion}
  until (t>=24)or(not ok);
  stable:=ok;
  if stable then for i:=2 to kk-1 do calcpCO2[i]:=localpCO2[i];
  {if (TRACE=1) then wwwriteln} singletracewriteln(' Leaving CO2_Diff_Eq');
end;

procedure PassDataOUT;
{pass data from Processes unit to calling main program untGUI.Run at end of year via the record DataYearOut}
var
  i, compartment, m, chem: integer;  ftmp:textfile; txt:string;  tmp:real;
begin
  untGUI.frmMAIN.MVAR(GlobalTime*10000+Datayearin.nolayers*100,' start untProcesses.PassDataOut: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
  untGUI.frmMAIN.MVAR(GlobalTime*10000+Datayearin.nolayers*100,' start untProcesses.PassDataOut: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);

  {if (TRACE = 1) then}
    singletracewriteln(' Entering PassDataOut');
   {very first:
   1. annual average soil temperatures, and
   2. annual clay dispersion indicator
   by layer}
  if claycount < 1 then
    claycount := 1;  {only relevant when crash at first timestep}
  if tempcount < 1 then
    tempcount := 1;  {only relevant when crash at first timestep}
  for i := 1 to LL do
  begin
    TEMPGEM[i] := TEMPGEM[i] / TEMPCOUNT;
    ClayDispInd[i] := ClayDispInd[i] / ClayCount;
  end;
  {from node to layer:}
  for i := 2 to kk-1 do
  begin
    untGUI.DataYearIn.TEMPGEM[i - 1] := TEMPGEM[i];
    untGUI.DataYearIn.ClayDispInd[i - 1] := ClayDispInd[i];
  end;

{0. set data record DataYearOut as initialisation equal to DataYearIn
(=input LEACHC/untProcesses).  Relevant fields will be overwritten with data from LEACHC/untProcesses}
  untGUI.DataYearOut := untGUI.DataYearIn;
  {1 en 2}
  if (untGUI.DataYearIn.firstyear) and (untGUI.DataYearIn.nn1 <> 0) then
    {read the selectivity coefficients that were calculated during the first year}
    for i := 2 to kk-1 do
    begin
      compartment := i - 1;
      untGUI.DataYearOut.sc1[compartment] := sc1[i];
      untGUI.DataYearOut.sc2[compartment] := sc2[i];
      untGUI.DataYearOut.sc3[compartment] := sc3[i];
      untGUI.DataYearOut.sc4[compartment] := sc4[i];
      untGUI.DataYearOut.sc5[compartment] := sc5[i];
      untGUI.DataYearOut.sc6[compartment] := sc6[i];
      untGUI.DataYearOut.sc7[compartment] := sc7[i];
      untGUI.DataYearOut.sc8[compartment] := sc8[i];
      untGUI.DataYearOut.sc9[compartment] := sc9[i];
      untGUI.DataYearOut.sc10[compartment] := sc10[i];
      untGUI.DataYearOut.sc11[compartment] := sc11[i];
      untGUI.DataYearOut.sc12[compartment] := sc12[i];
      untGUI.DataYearOut.sc13[compartment] := sc13[i];
      untGUI.DataYearOut.sc14[compartment] := sc14[i];
      untGUI.DataYearOut.sc15[compartment] := sc15[i];
    end;
  if (untGUI.ClayMigrationAllowed) then
    Exit_Clay_Migration;
  for i := 2 to kk-1 do
  begin
    compartment := i - 1;
    {first series water, temperature anions,calcite, gypsum and pCO2}
    untGUI.DataYearOut.AW[compartment] := W[i];
    untGUI.DataYearOut.AH[compartment] := H[i] / 102.2;
    untGUI.DataYearOut.FLUX[compartment] := WFW[i - 1];
    untGUI.DataYearOut.ET[compartment] := CTRAN[i];
    untGUI.DataYearOut.TempGem[compartment] := TEMPGEM[i];
    untGUI.DataYearOut.ATEM[compartment] := TEM[i];
    untGUI.DataYearOut.SEA[5][compartment] := SE[5][i];     {is Cl}
    untGUI.DataYearOut.SEA[6][compartment] := SE[6][i];     {is SO4}
    untGUI.DataYearOut.HCO3[compartment] := SE[7][i];
    untGUI.DataYearOut.CO3[compartment] := SE[8][i];
    untGUI.DataYearOut.SEA[8][compartment] := SE[8][i];     {is CO3}
    untGUI.DataYearOut.SEA[13][compartment] := SE[13][i];   {was Org3-}
    untGUI.DataYearOut.SEA[14][compartment] := SE[14][i];   {was HOrg2-}
    untGUI.DataYearOut.SEA[15][compartment] := SE[15][i];   {was H2Org-}
    untGUI.DataYearOut.DOC[compartment] := DOC[i];          {S DOC}
    {take into account the effect of a changed RHO at this point}
    if not (untGUI.ClayMigrationAllowed) then
      newRHO[i] := RHO[i];
    untGUI.DataYearOut.DOCperc[compartment] := DOC[i] * 10 / (DLZ * 1 * newRHO[i]);
    {convert back from mmol/dm3 soil to mass per compartment}
    untGUI.DataYearOut.CALCITEA[compartment] := CALCITE[i] * 100 * DLZ / 1000000;
    untGUI.DataYearOut.GYPA[compartment] := GYP[i] * 172 * DLZ / 1000000;

    untGUI.DataYearOut.SAR[compartment] := SAR[i];
    untGUI.DataYearOut.ESP[compartment] := ESP[i];
    untGUI.DataYearOut.EC[compartment] := EC[i];
    untGUI.DataYearOut.PH[compartment] := PH[i];
    untGUI.phlab[compartment] := phlab[i];
    untGUI.DataYearOut.APCO2[compartment] := PCO2[i];
    untGUI.DataYearOut.WE.Gibbsitepool[compartment] := Gibbsitepool[i];
    untGUI.DataYearOut.SUM21[compartment] := SUM21[i];
    untGUI.DataYearOut.MM21[compartment] := MM21[i];
    {second series: root density, clay dispersion and (exch+sol) cations}
    untGUI.DataYearOut.RDF[compartment] := RDF[i];
    untGUI.DataYearOut.ClayDispInd[compartment] := ClayDispInd[i];
    untGUI.DataYearOut.AXSE[1][compartment] := XSE[1][i];    {X Ca}
    untGUI.DataYearOut.AXSE_oc[1][compartment] := XSE_oc[1][i];
    untGUI.DataYearOut.AXSE_clay[1][compartment] := XSE_clay[1][i];
    untGUI.DataYearOut.SEA[1][compartment] := SE[1][i];      {S Ca}
    untGUI.DataYearOut.AXSE[2][compartment] := XSE[2][i];    {X Mg}
    untGUI.DataYearOut.AXSE_oc[2][compartment] := XSE_oc[2][i];
    untGUI.DataYearOut.AXSE_clay[2][compartment] := XSE_clay[2][i];
    untGUI.DataYearOut.SEA[2][compartment] := SE[2][i];      {S Mg}
    untGUI.DataYearOut.AXSE[3][compartment] := XSE[3][i];    {X Na}
    untGUI.DataYearOut.AXSE_oc[3][compartment] := XSE_oc[3][i];
    untGUI.DataYearOut.AXSE_clay[3][compartment] := XSE_clay[3][i];
    untGUI.DataYearOut.SEA[3][compartment] := SE[3][i];      {S Na}
    untGUI.DataYearOut.AXSE[4][compartment] := XSE[4][i];    {X K}
    untGUI.DataYearOut.AXSE_oc[4][compartment] := XSE_oc[4][i];
    untGUI.DataYearOut.AXSE_clay[4][compartment] := XSE_clay[4][i];
    untGUI.DataYearOut.SEA[4][compartment] := SE[4][i];      {S K}
    untGUI.DataYearOut.AXSE[10][compartment] := XSE[10][i];  {X H}
    untGUI.DataYearOut.AXSE_oc[10][compartment] := XSE_oc[10][i];
    untGUI.DataYearOut.AXSE_clay[10][compartment] := XSE_clay[10][i];
    untGUI.DataYearOut.SEA[10][compartment] := SE[10][i];    {S H}
    untGUI.DataYearOut.AXSE[12][compartment] := XSE[12][i];  {X Al}
    untGUI.DataYearOut.AXSE_oc[12][compartment] := XSE_oc[12][i];
    untGUI.DataYearOut.AXSE_clay[12][compartment] := XSE_clay[12][i];
    untGUI.DataYearOut.SEA[12][compartment] := SE[12][i];    {S Al}
    untGUI.DataYearOut.ALKALINITY[compartment] :=
      SE[7][i] + 2 * SE[8][i] + SE[11][i] - SE[10][i];
    untGUI.DataYearOut.SEA[7][compartment] :=
      untGUI.DataYearOut.alkalinity[compartment];
    untGUI.DataYearOut.DISSOLVEDCATEQ[compartment] :=
      2 * SE[1][i] + 2 * SE[2][i] + SE[3][i] + SE[4][i];
    untGUI.DataYearOut.ACEC[compartment] :=
        XSE[1][i] + XSE[2][i] + XSE[3][i] + XSE[4][i] + XSE[10][i] + XSE[12][i];
    untGUI.DataYearOut.ACEC_clay[compartment] :=
        XSE_clay[1][i] + XSE_clay[2][i] + XSE_clay[3][i] + XSE_clay[4][i] + XSE_clay[10]
        [i] + XSE_clay[12][i];
    untGUI.DataYearOut.ACEC_oc[compartment] :=
        XSE_oc[1][i] + XSE_oc[2][i] + XSE_oc[3][i] + XSE_oc[4][i] + XSE_oc[10][i] + XSE_oc[12][i];
    untGUI.DataYearOut.BS[compartment] :=
      100 * (XSE[1][i] + XSE[2][i] + XSE[3][i] + XSE[4][i]) /
      untGUI.DataYearOut.ACEC[compartment];
  end;     
  for i:=2 to kk-1 do
  begin
    compartment := i - 1;
    {third series: clay,silt,sand,oc,rho}
    untGUI.DataYearOut.CLAY[compartment] := CLAY[i];
    untGUI.DataYearOut.CLAYm[compartment] := CLAYm[i];
    untGUI.DataYearOut.SILT[compartment] := SILT[i];
    untGUI.DataYearOut.SILTm[compartment] := SILTm[i];
    untGUI.DataYearOut.SAND[compartment] := SAND[i];
    untGUI.DataYearOut.SANDm[compartment] := SANDm[i];
    untGUI.DataYearOut.fineSANDm[compartment] := fineSANDm[i];
    untGUI.DataYearOut.GRAVEL[compartment]:= GRAVEL[i];
    untGUI.DataYearOut.Gravelm[compartment]:= GRAVELm[i];
    untGUI.DataYearOut.STONES[compartment]:= STONES[i];
    untGUI.DataYearOut.Stonesm[compartment]:= Stonesm[i];
    untGUI.DataYearOut.FreezeCounter[compartment]:= FreezeCounter[i];
    untGUI.DataYearOut.Rootcounter[compartment]:= Rootcounter[i]/3650;
    {update RHO with newRHO}
    untGUI.DataYearOut.RHO[compartment] := newRHO[i];
    untGUI.DataYearOut.OC[compartment] := OC[i];
    untGUI.DataYearOut.OCm[compartment] := OCm[i];
  end; 
  {fourth series: all data from RothC-module}
  untGUI.DataYearOut.RothC.vegetatie := untProcesses.RothC.vegetatie;
  untGUI.DataYearOut.RothC.plantressum := untProcesses.RothC.plantressum;
  untGUI.DataYearOut.RothC.manuresum := untProcesses.RothC.manuresum;
  untGUI.DataYearOut.RothC.dpm2rpm := untProcesses.RothC.dpm2rpm;
  untGUI.DataYearOut.RothC.frectolitter := untProcesses.RothC.frectolitter;
  untGUI.DataYearOut.RothC.frendolitter := untProcesses.RothC.frendolitter;
  untGUI.DataYearOut.RothC.ectoDPMpool := untProcesses.RothC.DPMpool[1];
  untGUI.DataYearOut.RothC.ectoRPMpool := untProcesses.RothC.RPMpool[1];
  untGUI.DataYearOut.RothC.ectoBIOpool := untProcesses.RothC.BIOpool[1];
  untGUI.DataYearOut.RothC.ectoHUMpool := untProcesses.RothC.HUMpool[1];
  untGUI.DataYearOut.RothC.ectoIOMpool := untProcesses.RothC.IOMpool[1];
  untGUI.DataYearOut.RothC.ectoDPMXpool := untProcesses.RothC.DPMXpool[1];
  untGUI.DataYearOut.RothC.ectoRPMXpool := untProcesses.RothC.RPMXpool[1];
  untGUI.DataYearOut.RothC.ectoBIOXpool := untProcesses.RothC.BIOXpool[1];
  untGUI.DataYearOut.RothC.ectoHUMXpool := untProcesses.RothC.HUMXpool[1];
  untGUI.DataYearOut.RothC.ectoIOMXpool := untProcesses.RothC.IOMXpool[1];
  untGUI.DataYearOut.RothC.totalMaxTSMD := untProcesses.RothC.totalMaxTSMD;
  untGUI.DataYearOut.RothC.ectoCO2pool := untProcesses.RothC.ectoCO2pool;
  for m := 1 to 12 do
  begin
    untGUI.DataYearOut.RothC.totalPlantResiduesIN[m] :=
      RothC.totalPlantResiduesIN[m];
    untGUI.DataYearOut.RothC.totalManureIN[m] := RothC.totalManureIN[m];
  end;
  {NB: the ones below are actually passed by reading the RC*.out file!}
  for i := 2 to kk-1 do
  begin
    compartment := i - 1;
    untGUI.DataYearOut.RothC.DPMpool[compartment] := untProcesses.RothC.DPMpool[i];
    untGUI.DataYearOut.RothC.RPMpool[compartment] := untProcesses.RothC.RPMpool[i];
    untGUI.DataYearOut.RothC.BIOpool[compartment] := untProcesses.RothC.BIOpool[i];
    untGUI.DataYearOut.RothC.HUMpool[compartment] := untProcesses.RothC.HUMpool[i];
    untGUI.DataYearOut.RothC.IOMpool[compartment] := untProcesses.RothC.IOMpool[i];
    untGUI.DataYearOut.RothC.CO2pool[compartment] := untProcesses.RothC.CO2pool[i];
    untGUI.DataYearOut.RothC.accTSMD[compartment] := untProcesses.RothC.accTSMD[i];
    untGUI.DataYearOut.RothC.pCO2[compartment] := untProcesses.RothC.pCO2_r[i];
    untGUI.DataYearOut.RothC.clayperc[compartment] := untProcesses.RothC.clayperc[i];
    untGUI.DataYearOut.RothC.DPMXpool[compartment] := untProcesses.RothC.DPMXpool[i];
    untGUI.DataYearOut.RothC.RPMXpool[compartment] := untProcesses.RothC.RPMXpool[i];
    untGUI.DataYearOut.RothC.BIOXpool[compartment] := untProcesses.RothC.BIOXpool[i];
    untGUI.DataYearOut.RothC.HUMXpool[compartment] := untProcesses.RothC.HUMXpool[i];
    untGUI.DataYearOut.RothC.IOMXpool[compartment] := untProcesses.RothC.IOMXpool[i];
    untGUI.DataYearOut.RothC.CO2Xpool[compartment] := untProcesses.RothC.CO2Xpool[i];
    untGUI.DataYearOut.RothC.x2[compartment] := untProcesses.RothC.x2[i];
    untGUI.DataYearOut.RothC.x3[compartment] := untProcesses.RothC.x3[i];
    for chem := 1 to nchem do
    begin
      untGUI.DataYearOut.CHEMDPMPool[chem, i] := untProcesses.CHEMDPMPool[chem, i];
      untGUI.DataYearOut.CHEMRPMPool[chem, i] := untProcesses.CHEMRPMPool[chem, i];
      untGUI.DataYearOut.CHEMBIOPool[chem, i] := untProcesses.CHEMBIOPool[chem, i];
      untGUI.DataYearOut.CHEMHUMPool[chem, i] := untProcesses.CHEMHUMPool[chem, i];
      untGUI.DataYearOut.CHEMIOMPool[chem, i] := untProcesses.CHEMIOMPool[chem, i];
      untGUI.DataYearOut.CHEMCO2Pool[chem, i] := untProcesses.CHEMCO2Pool[chem, i];
    end;
    for m := 1 to 12 do
    begin
      untGUI.DataYearOut.RothC.PlantResiduesIN[m, i] := RothC.PlantResiduesIN[m, i];
      untGUI.DataYearOut.RothC.ManureIN[m, i] := RothC.ManureIN[m, i];
    end;
  end;
  {fifth series: weathering data}
  for i := 2 to kk-1 do
  begin
    compartment := i - 1;
    untGUI.DataYearOut.WE.Gibbsitepool[compartment] := Gibbsitepool[i];
    untGUI.DataYearOut.WE.WE_Capool[compartment] := WE_Capool[i];
    untGUI.DataYearOut.WE.WE_Mgpool[compartment] := WE_Mgpool[i];
    untGUI.DataYearOut.WE.WE_Kpool[compartment] := WE_Kpool[i];
    untGUI.DataYearOut.WE.WE_Napool[compartment] := WE_Napool[i];
    untGUI.DataYearOut.WE.WE_Sipool[compartment] := WE_Sipool[i];
    untGUI.DataYearOut.WE.WE_Fepool[compartment] := WE_Fepool[i];
    untGUI.DataYearOut.WE.WE_Alpool[compartment] := WE_Alpool[i];
    untGUI.DataYearOut.WE.WE_FePedogenic[compartment] := WE_FePedogenic[i];
    untGUI.DataYearOut.WE.WE_SiPedogenic[compartment] := WE_SiPedogenic[i];
    for m := 1 to nrminerals do
    begin
      untGUI.DataYearOut.WE.MinPools[m, compartment] := minpools[m, i];
    end;
  end;
  {update the effect of physical weathering due to forst spalling and root action in the rock compartment, if any}
  if untGUI.rock.RockBottomLayers>0 { *Converted from FileExists*  }
  then
  begin
    {1. reduce the stone mass as a consequence of freezecounter: }
    untGUI.rock.saproStonesm:=untGUI.rock.saproStonesm-
                                      max(frStoneSplit*untGUI.rock.saproStonesm*untGUI.rock.saprofreezecounter/100,
                                          (frStoneSplit/100)*untGUI.rock.saproStonesm);
    {2. reduce the stone mass as a consequence of rootcounter: }
    untGUI.rock.saproStonesm:=untGUI.rock.saproStonesm-
                                      max(frStoneSplit*untGUI.rock.saproStonesm*untGUI.rock.saprorootcounter/(3650*100),
                                          (frStoneSplit/100)*untGUI.rock.saproStonesm);
    {3. update the number of stones and calc the mass percentage of stones lost by physical weathering for this year}
    untGUI.rock.saproPW_K[-4]:=round(100*rock.saproSTONESm/(power((PW_X[-4]/100000),3)*PW_PARDEN[-4]));
    untGUI.rock.phweathloss:=100*(untGUI.rock_original.saproStonesm-untGUI.rock.saproStonesm)/untGUI.rock_original.saproStonesm;
    {4. reset saprofreezecounter and saprorootcounter}
    untGUI.rock.saprofreezecounter:=0 ;
    untGUI.rock.saprorootcounter:=0;
    {5. update the texture fractions}
    {calculate mass in clay-silt-sand-gravel fraction using the physically weathered part}
    untGUI.rock.saproclaym:=0.5*(untGUI.rock.phweathloss/100)*(untGUI.rock.rockMontmorillonite[2]+
                                         untGUI.rock.rockGypsum[2]+untGUI.rock.rockChlorite[2]+untGUI.rock.rockKaolinite[2]+
                                         untGUI.rock.rockMuscovite[2]+untGUI.rock.rockBiotite[2]+untGUI.rock.rockIllite[2]+
                                         untGUI.rock.rockGibbsite[2]);
    untGUI.rock.saprosiltm:=0.5*(untGUI.rock.phweathloss/100)*(untGUI.rock.rockCalcite[2]+untGUI.rock.rockHornblende[2]+
                                         untGUI.rock.rockAugite[2]+untGUI.rock.rockKFeldspar[2]+
                                         untGUI.rock.rockAnorthite[2]+untGUI.rock.rockAmorphite[2]+untGUI.rock.rockOtherite[2]);
    untGUI.rock.saprosandm:=0.5*(untGUI.rock.phweathloss/100)*(untGUI.rock.rockFayalite[2]+untGUI.rock.rockAlbite[2]+
                                         untGUI.rock.rockQuartz[2]+untGUI.rock.rockForsterite[2]);
    untGUI.rock.saprogravelm:=untGUI.rock.saproclaym+untGUI.rock.saprosiltm+untGUI.rock.saprosandm;
  end;
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving PassDataOut');
    untGUI.frmMAIN.MVAR(GlobalTime*10000+Datayearin.nolayers*100,' einde untProcesses.PassDataOut: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
    untGUI.frmMAIN.MVAR(GlobalTime*10000+Datayearin.nolayers*100,' einde untProcesses.PassDataOut: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);

end;

procedure WE_init;
{initialization and read input file for Weathering submodel, status end previous year.
reads remaining cation pools in primary minerals and gets general data from calling program}
{Reads the pool-data and gets the constants from the calling program, pf08..pf14}
var
  fwe: textfile;
  i, j, idum: integer;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering WE_Init');
  {pool data are in fileWE: <WE*.>, made in calling program in procedure WriteDataWE}
  system.Assign(fwe, FileWE);
  system.reset(fwe);
  system.readln(fwe);
  system.readln(fwe);
  for i := 2 to kk - 1 do
  begin
    system.Read(fwe, idum);
    for j := 1 to nrminerals do
      system.Read(fwe, MinPools[j][i]);
    system.readln(fwe);
  end;
  system.readln(fwe);
  system.readln(fwe);
  for i := 2 to kk - 1 do
  begin
    system.readln(fwe, idum, Gibbsitepool[i],
      WE_Capool[i], WE_Mgpool[i],
      WE_Kpool[i], WE_Napool[i],
      WE_Sipool[i], WE_Fepool[i],
      WE_Alpool[i], WE_FePedogenic[i],
      WE_SiPedogenic[i]);
  end;
  Gibbsitepool[1] := 0;
  WE_Capool[1] := 0;
  WE_Mgpool[1] := 0;
  WE_Kpool[1] := 0;
  WE_Napool[1] := 0;
  WE_Sipool[1] := 0;
  WE_Fepool[1] := 0;
  WE_FePedogenic[1] := 0;
  WE_SiPedogenic[1] := 0;
  WE_Alpool[1] := 0;
  Gibbsitepool[kk] := 0;
  WE_Capool[kk] := 0;
  WE_Mgpool[kk] := 0;
  WE_Kpool[kk] := 0;
  WE_Napool[kk] := 0;
  WE_Sipool[kk] := 0;
  WE_Fepool[kk] := 0;
  WE_FePedogenic[kk] := 0;
  WE_SiPedogenic[kk] := 0;
  WE_Alpool[kk] := 0;
  closefile(fwe);
  {rate constants etc. were already read in calling program in procedure ReadDataWE,
   and need to be copied to local variables}
  for i := 1 to nrparams do
    for j := 1 to nrminerals do
      MinDisPar[i, j] := untGUI.DataInit.WEGen.MinDisPar[i, j];
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving WE_Init');
end;

procedure AdaptRjkRedfac(var redfacQtz,redfacChlor,redfacKaol,redfacIllit,redfacMontm:real;
                         var rjkqtz,rjkchlor,rjkkaol,rjkillit,rjkmontm:real;
                         rjkalb,rjkkfeld,rjkbio,rjkmusco,rjkanort,rjkForster,rjkaugit,rjkhornb,rjkfayal,rjkamorp,rjkother,
                         Aalb,Akfeld,Abio,Amusco,Aanort,AForster,Aaugit,Ahornb,Afayal,Aamorp,Aother,
                         Aqtz,Achlor,Akaol,Aillit,Amontm,
                         Moalb,Mokfeld,Momusco,Mobio,Moanort,MoForster,Moaugit,Mohornb,Mofayal,MoAmorp,MoOther,
                         Moqtz,Mochlor,Mokaol,Moillit,Momontm:real; ilayer:integer);
{reduce weathering rates for newforming minerals if the soil solution cannot deliver the cations (called from WE_daily)}
Var FCa1, FCa2, FNa1, FNa2, FK1, FK2, FSi1, FSi2, FAl1, FAl2, FMg1, FMg2, T,
    FSimols1, FSimols2, FSimols3, FAlmols1, FAlmols2, FAlmols3, FCamols1, FCamols2, FCamols3, FMgmols1, FMgmols2, FMgmols3,
    FKmols1, FKmols2, FKmols3, FNamols1, FNamols2, FNamols3, tmp: real;
    bQtz,bChlor,bKaol,bIllit,bMontm:boolean; txt:string;
Begin
    {bQtz:=false; bChlor:=false; bKaol:=false; bIllit:=false; bMontm:=false;}

    T := DLZ / 1000;

    FSi1 := T * (3 * Akfeld * rjkkfeld * Mokfeld + 3 * Amusco * rjkmusco * Momusco +
      3 * Aalb * rjkalb * Moalb + 3 * Abio * rjkbio * Mobio +  2 * Aanort * rjkanort * Moanort +
      1 * AForster * rjkForster * MoForster + 1 * Aaugit * rjkaugit * Moaugit +
      6.7 * Ahornb * rjkhornb * Mohornb + 1 * Afayal * rjkfayal * Mofayal +
      max(0,untGUI.amorp_Si) * Aamorp * rjkamorp * Moamorp +
      max(0,untGUI.other_Si) * Aother * rjkother * Moother);
    FSi2 := T * (1 * Aqtz * rjkqtz * Moqtz + 3 * Achlor * rjkchlor * Mochlor + 2 * Akaol * rjkkaol * Mokaol +
      3.4 * Aillit * rjkillit * Moillit + 4 * Amontm * rjkmontm * Momontm);
    FSimols1 := FSi1 * 60 * 60 * 24; {dissolving Si; converting flux (moles/m2/s) to moles/day; area is 1 m2}
    FSimols2 := FSi2 * 60 * 60 * 24; {precipitating Si}
    {Add Si to solution here}
    tmp:=SE[15,ilayer]*W[ilayer]*dlz+1000*(FSimols1+FSimols2);
    If tmp<1E-15 then {over-exhaustion of concentration. Can be due to precip of Qtz, Chlor, Kaol, Illit or Montm here}
    begin
      FSimols3:=1E-15-(SE[15,ilayer]*W[ilayer]*dlz+1000*FSimols1)/1000; {this is the max Si precip allowed}
      if FSimols2<>0 then redfacQtz:=redfacQtz*FSimols3/FSimols2;     rjkqtz:=redfacQtz*rjkqtz;       {bQtz:=true;}
      if FSimols2<>0 then redfacChlor:=redfacChlor*FSimols3/FSimols2; rjkchlor:=redfacChlor*rjkchlor; {bChlor:=true; }
      if FSimols2<>0 then redfacKaol:=redfacKaol*FSimols3/FSimols2;   rjkkaol:=redfacKaol*rjkkaol;    {bKaol:=true; }
      if FSimols2<>0 then redfacIllit:=redfacIllit*FSimols3/FSimols2; rjkillit:=redfacIllit*rjkillit; {bIllit:=true; }
      if FSimols2<>0 then redfacMontm:=redfacMontm*FSimols3/FSimols2; rjkmontm:=redfacMontm*rjkmontm; {bMontm:=true; }
    end;

    FAl1 := T * (1 * Akfeld * rjkkfeld * Mokfeld + 3 * Amusco * rjkmusco * Momusco +
      1 * Aalb * rjkalb * Moalb + 1 * Abio * rjkbio * Mobio + 2 * Aanort * rjkanort * Moanort +
      0 * AForster * rjkForster * MoForster + 0 * Aaugit * rjkaugit * Moaugit + 0 * Ahornb * rjkhornb * Mohornb +
      0 * Afayal * rjkfayal * Mofayal + max(0,untGUI.amorp_Al) * Aamorp * rjkamorp * Moamorp +
      max(0,untGUI.other_Al) * Aother * rjkother * Moother);
    FAl2 := T * (0 * Aqtz * rjkqtz * Moqtz + 2 * Achlor * rjkchlor * Mochlor +
      2 * Akaol * rjkkaol * Mokaol + 2.38 * Aillit * rjkillit * Moillit +
      1.67 * Amontm * rjkmontm * Momontm);
    FAlmols1 := FAl1 * 60 * 60 * 24; {converting flux (moles/m2/s) to moles/day; area is 1 m2}
    FAlmols2 := FAl2 * 60 * 60 * 24;
    {Add Al to solution and precip Gibbsite if necessary}
    tmp := SE[12, ilayer] * W[ilayer] * dlz + 1000 * (FAlmols1+FAlmols2);
    If tmp<1E-15 then {over-exhaustion of concentration. Can be due to precip of Chlor, Kaol, Illit or Montm here}
    begin
      FAlmols3:=1E-15-(SE[12,ilayer]*W[ilayer]*dlz+1000*FAlmols1)/1000; {this is the max Al precip allowed}
      if FAlmols2<>0 then redfacChlor:=redfacChlor*FAlmols3/FAlmols2; rjkchlor:=redfacChlor*rjkchlor; {bChlor:=true; }
      if FAlmols2<>0 then redfacKaol:=redfacKaol*FAlmols3/FAlmols2;   rjkkaol:=redfacKaol*rjkkaol;    {bKaol:=true; }
      if FAlmols2<>0 then redfacIllit:=redfacIllit*FAlmols3/FAlmols2; rjkillit:=redfacIllit*rjkillit; {bIllit:=true;}
      if FAlmols2<>0 then redfacMontm:=redfacMontm*FAlmols3/FAlmols2; rjkmontm:=redfacMontm*rjkmontm; {bMontm:=true;}
    end;

    FCa1 := T * (0 * Akfeld * rjkkfeld * Mokfeld + 0 * Amusco * rjkmusco * Momusco +
      0 * Aalb * rjkalb * Moalb + 0 * Abio * rjkbio * Mobio + 1 * Aanort * rjkanort * Moanort +
      0 * AForster * rjkForster * MoForster + 0 * Aaugit * rjkaugit * Moaugit + 2 * Ahornb * rjkhornb * Mohornb +
      0 * Afayal * rjkfayal * Mofayal + max(0,untGUI.amorp_Ca) * Aamorp * rjkamorp * Moamorp +
      max(0,untGUI.other_Ca) * Aother * rjkother * Moother);
    FCa2 := T * (0 * Aqtz * rjkqtz * Moqtz + 0 * Achlor * rjkchlor * Mochlor +
      0 * Akaol * rjkkaol * Mokaol + 0.01 * Aillit * rjkillit * Moillit +
      0.04135 * Amontm * rjkmontm * Momontm);
    FCamols1 := FCa1 * 60 * 60 * 24; {converting flux (moles/m2/s) to moles/day; area is 1 m2}
    FCamols2 := FCa2 * 60 * 60 * 24;
    {Add Ca to solution here}
    tmp := SE[1, ilayer] * W[ilayer] * dlz + 1000 * (FCamols1+FCamols2);
    If tmp<1E-15 then {over-exhaustion of concentration. Can be due to precip of Illit or Montm here}
    begin
      FCamols3:=1E-15-(SE[1,ilayer]*W[ilayer]*dlz+1000*FCamols1)/1000; {this is the max Ca precip allowed}
      if FCamols2<>0 then redfacIllit:=redfacIllit*FCamols3/FCamols2; rjkillit:=redfacIllit*rjkillit; {bIllit:=true;  }
      if FCamols2<>0 then redfacMontm:=redfacMontm*FCamols3/FCamols2; rjkmontm:=redfacMontm*rjkmontm; {bMontm:=true;  }
    end;

    FMg1 := T * (0 * Akfeld * rjkkfeld * Mokfeld + 0 * Amusco * rjkmusco * Momusco +
      0 * Aalb * rjkalb * Moalb + 1.5 * Abio * rjkbio * Mobio + 0 * Aanort * rjkanort * Moanort +
      2 * AForster * rjkForster * MoForster + 0 * Aaugit * rjkaugit * Moaugit +
      4 * Ahornb * rjkhornb * Mohornb + 0 * Afayal * rjkfayal * Mofayal +
      max(0,untGUI.amorp_Mg) * Aamorp * rjkamorp * Moamorp +
      max(0,untGUI.other_Mg) * Aother * rjkother * Moother);
    FMg2 := T * (0 * Aqtz * rjkqtz * Moqtz + 5 * Achlor * rjkchlor * Mochlor +
      0 * Akaol * rjkkaol * Mokaol + 0.02 * Aillit * rjkillit * Moillit +
      (0.33 + 0.04135) * Amontm * rjkmontm * Momontm);
    FMgmols1 := FMg1 * 60 * 60 * 24; {converting flux (moles/m2/s) to moles/day; area is 1 m2}
    FMgmols2 := FMg2 * 60 * 60 * 24;
    {Add Mg to solution here}
    tmp := SE[2, ilayer] * W[ilayer] * dlz + 1000 * (FMgmols1+FMgmols2);
    If tmp<1E-15 then {over-exhaustion of concentration. Can be due to precip of Chlor, Illit or Montm here}
    begin
      FMgmols3:=1E-15-(SE[13,ilayer]*W[ilayer]*dlz+1000*FMgmols1)/1000; {this is the max Mg precip allowed}
      if FMgmols2<>0 then redfacChlor:=redfacChlor*FMgmols3/FMgmols2; rjkchlor:=redfacChlor*rjkchlor;  {bChlor:=true; }
      if FMgmols2<>0 then redfacIllit:=redfacIllit*FMgmols3/FMgmols2; rjkillit:=redfacIllit*rjkillit;  {bIllit:=true; }
      if FMgmols2<>0 then redfacMontm:=redfacMontm*FMgmols3/FMgmols2; rjkmontm:=redfacMontm*rjkmontm;  {bMontm:=true; }
    end;

    FK1 := T * (1 * Akfeld * rjkkfeld * Mokfeld + 1 * Amusco * rjkmusco * Momusco +
      0 * Aalb * rjkalb * Moalb + 1 * Abio * rjkbio * Mobio + 0 * Aanort * rjkanort * Moanort +
      0 * AForster * rjkForster * MoForster + 0 * Aaugit * rjkaugit * Moaugit +
      0 * Ahornb * rjkhornb * Mohornb + 0 * Afayal * rjkfayal * Mofayal +
      max(0,untGUI.amorp_K) * Aamorp * rjkamorp * Moamorp +
      max(0,untGUI.other_K) * Aother * rjkother * Moother);
    FK2 := T * (0 * Aqtz * rjkqtz * Moqtz + 0 * Achlor * rjkchlor * Mochlor +
      0 * Akaol * rjkkaol * Mokaol + 0.44 * Aillit * rjkillit * Moillit +
      0.0825 * Amontm * rjkmontm * Momontm);
    FKmols1 := FK1 * 60 * 60 * 24; {converting flux (moles/m2/s) to moles/day; area is 1 m2}
    FKmols2 := FK2 * 60 * 60 * 24;
    {Add K to solution here}
    tmp := SE[4, ilayer] * W[ilayer] * dlz + 1000 * (FKmols1+FKmols2);
    If tmp<1E-15 then {over-exhaustion of concentration. Can be due to precip of Illit or Montm here}
    begin
      FKmols3:=1E-15-(SE[13,ilayer]*W[ilayer]*dlz+1000*FKmols1)/1000; {this is the max K precip allowed}
      if FKmols2<>0 then redfacIllit:=redfacIllit*FKmols3/FKmols2; rjkillit:=redfacIllit*rjkillit;   {bIllit:=true;  }
      if FKmols2<>0 then redfacMontm:=redfacMontm*FKmols3/FKmols2; rjkmontm:=redfacMontm*rjkmontm;   {bMontm:=true;  }
    end;

    FNa1 := T * (0 * Akfeld * rjkkfeld * Mokfeld + 0 * Amusco * rjkmusco * Momusco +
      1 * Aalb * rjkalb * Moalb + 0 * Abio * rjkbio * Mobio + 0 * Aanort * rjkanort * Moanort +
      0 * AForster * rjkForster * MoForster + 0 * Aaugit * rjkaugit * Moaugit +
      0 * Ahornb * rjkhornb * Mohornb + 0 * Afayal * rjkfayal * Mofayal +
      max(0,untGUI.amorp_Na) * Aamorp * rjkamorp * Moamorp +
      max(0,untGUI.other_Na) * Aother * rjkother * Moother);
    FNa2 := T * (0 * Aqtz * rjkqtz * Moqtz + 0 * Achlor * rjkchlor * Mochlor +
      0 * Akaol * rjkkaol * Mokaol + 0 * Aillit * rjkillit * Moillit + 0.0825 * Amontm * rjkmontm * Momontm);
    FNamols1 := FNa1 * 60 * 60 * 24; {converting flux (moles/m2/s) to moles/day; area is 1 m2}
    FNamols2 := FNa2 * 60 * 60 * 24;
    {Add Na to solution here}
    tmp := SE[3, ilayer] * W[ilayer] * dlz + 1000 * (FNamols1+FNamols2);
    If tmp<1E-15 then {over-exhaustion of concentration. Can be due to precip of Montm here}
    begin
      FNamols3:=1E-15-(SE[13,ilayer]*W[ilayer]*dlz+1000*FNamols1)/1000; {this is the max Na precip allowed}
      if FNamols2<>0 then redfacMontm:=redfacMontm*FNamols3/FNamols2; rjkmontm:=redfacMontm*rjkmontm;  {bMontm:=true; }
    end;
    {melding bw test}
    {if bQtz   then txt:='Qrtz ' else txt:=' '; untGUI.frmMain.lblNOTE.Caption:=txt;
    if bChlor then txt:='Chlor ' else txt:=' '; untGUI.frmMain.lblNOTE.Caption:=untGUI.frmMain.lblNOTE.Caption+txt;
    if bKaol  then txt:='Kaol ' else txt:=' '; untGUI.frmMain.lblNOTE.Caption:=untGUI.frmMain.lblNOTE.Caption+txt;
    if bIllit then txt:='Illi ' else txt:=' '; untGUI.frmMain.lblNOTE.Caption:=untGUI.frmMain.lblNOTE.Caption+txt;;
    if bMontm then txt:='Montm ' else txt:=' '; untGUI.frmMain.lblNOTE.Caption:=untGUI.frmMain.lblNOTE.Caption+txt;
    untGUI.frmMain.lblNOTE.refresh;}
    {eind melding bwv test}
End;


Procedure IAPOMEGA(var IAP,Omega: Mindat; var finh: Profile);
{calculates IonActivityProducts per mineral and the saturation degree Omega (called from WE_daily)}
var ilayer:Integer;
    silacid,Fe3pool,Fe2pool,Keq,convfac,a,b,c,d,e,f,g,h,i,tempHact,tempOHact:real;
begin
  convfac:=1000 {from mol to mmol vv.};
  for ilayer := 2 to kk - 1 do
  begin
    {1. calculate aluminum inhibition factor, with SE brought to mol/liter}
    finh[ilayer]:=power(1E-7/(1e-7+SE[13,ilayer]/convfac),0.33);

    {2. calculate IonActivityProduct of each mineral dissolving, using the current solution composition
    Mineral	Dissolution equation & IAP=
    Albite	NaAlSi3O8 + 4 H2O + 4 H+ = Na+ + Al+3 + 3 H4SiO4 & Al*Na*silacid3/H4
    K-Feldspar	KAlSi3O8 + 4 H2O + 4 H+ = K+ + Al+3 + 3 H4SiO4 & Al*K*silacid3/H4
    Muscovite	KAl3Si3O10(OH)2 + 10 H+ = K+ + 3 Al+3 + 3 H4SiO4 & Al3*K*silacid3/H10
    Biotite	KMg1.5Fe1.5AlSi3O10(OH)2 + 10 H+ = K+ + 1.5 Mg+2 + 1.5 Fe+2 + Al+3 + 3 H4SiO4 & Al*K*Mg1.5*Fe1.5*silacid3/H10
    Quartz	SiO2 + 2 H2O = H4SiO4 & silacid
    Chlorite	Mg5Al2Si3O10 (OH)8 + 16 H+ = 5 Mg+2 + 2 Al+3 + 3 H4SiO4 + 6H2O & Al2*Mg5*K*silacid3/H16
    Anorthite	CaAl2Si2O8 + 8 H+ = Ca+2 + 2 Al+3 + 2 H4SiO4 & Al2*Ca*silacid2/H8
    Forsterite	Mg2SiO4 + 4 H+ = 2 Mg+2 + H4SiO4 & Mg2*silacid/H4
    Augite	FeSiO3 + 2 H+ + H2O = Fe+2 + H4SiO4 & Fe*silacid/H2
    Kaolinite	Al2Si2O5(OH)4 + 6 H+ = H2O + 2 H4SiO4 + 2 Al+3 & Al2*silacid2/H6
    Illite	K0.6Mg0.25Al2.3Si3.5O10(OH)2 + 2 H2O + 8H+ = 0.6K+ + 0.25Mg+2 + 2.3Al+3 + 3.5 H4SiO4 & Al2.3*Mg0.25*K0.6*silacid3.5/H8
    Montmorillonite	Si4O10(OH)2Mg0.33Al1.67Ca0.04135Mg0.04135K0.0825Na0.0825 + 4 H2O + 6 H+ = 0.04135 Ca+2 + 0.37125 Mg+2 + 0.0825 K+ + 0.0825 Na+ + 1.67 Al+3 + 4 H4SiO4 & Al1.67*Mg0.37125*K0.0824*Ca0.04135*Na0.0824*silacid4/H6
    Hornblende	Ca2Mg4AlSi6.7AlO22.4 +4.4 H2O + 18 H+ = 2 Ca+2 + 4 Mg+2 + 2 Al+3 + 6.7 H4SiO4 & Al2*Mg4*Ca2*silacid6.7/H18
    Fayalite	Fe2SiO4 + 4 H+ = 2 Fe+2 + H4SiO4 & Fe2*silacid/H4
    Amorphite	composition not fixed, from amorphite.txt input file.
    Otherite	composition not fixed, from otherite.txt input file. 	 }

    {2a. preparing silicic acid, using Gunnarsson&Arnórsson,2000. https://doi.org/10.1016/S0016-7037(99)00426-3,
    where their equation predicts te max ppm SiO2 dissolving as a function of temperature,
    which we convert to mol/dm3 and then also is describing H4SiO4 being formed by SiO2+2H2O=H4SiO4
    if needed we reduced by the amount that can be delivered from pedogenic Si.
    At this moment no pH-effect included!}
    silacid:=-8.476-485.24/(273.15+TEM[ilayer])-0.000002268/power((273.15+TEM[ilayer]),2)+3.068*LOG10(273.15+TEM[ilayer]);
    silacid:=power(10,silacid);     {this is in ppm, or mg/kg, or mg/dm3water}
    silacid:=silacid*0.001/60.0843; {this is in mol/dm3}
    silacid:=min(silacid,max(WE_SiPedogenic[ilayer],1e-7));   {silacid=0 cannot be tolerated due to numerical issues on calc IAP later}

    tempHact := power(10, -pH[ilayer]);
    tempOHact := power(10, -(14 - pH[ilayer]));
    {2b. Fe++ (SE[14,ilayer]) and Fe+++ (SE[13,ilayer] are calculated here first. Based on:
    1. Fe(OH)3 = Fe+3 + 3 OH- with pS=38,                  so log Fe+3=4-3pH
    2. Fe(OH)2 = Fe+2 + 2 OH- with pS=15.1,                so log Fe+2=12.9-2pH
    3. At redox potential=0: 0=1.06+0.059 log(H^3/Fe+2),   so log Fe+2=(1.06/0.059)-3pH
    In the below calcs we assume immediate equilibria, no rates. Room for future improvement!}
    {2ba. For Fe+3 we take the result of 1, whch will be a very low amount, and limit it to the max present in pedogenic Fe}
    {First bring existing Fe+2 en Fe+3 into WE_FePedogenic, so that it can later be recalculated based on the pH and pS:}
    WE_FePedogenic[ilayer]:=WE_FePedogenic[ilayer]+W[ilayer]*dlz*(SE[13,ilayer]+SE[14,ilayer])/convfac;
    {now calc the FeIII-pool based on eq.1:}
    Fe3pool:=power(10,4-3*Ph[ilayer])*W[ilayer]*dlz; {Fe+3 in moles/m2,dlz mm compartment thickness}
    {check if there is enough pedogenic Fe to fill this FeIII pool}
    if Fe3pool>WE_FePedogenic[ilayer] then Fe3pool:=WE_FePedogenic[ilayer];
    WE_FePedogenic[ilayer]:=WE_FePedogenic[ilayer]-Fe3pool;
    {and transform the FeIII pool to FeIII-concentration:}
    SE[13,ilayer]:=max(1E-10,1000*Fe3pool/(W[ilayer]*dlz));
    {2bb. For Fe+2 we take the least of 2 and 3 and limit it to the max present in (pedogenic Fe - Fe+3)}
    Fe2pool:=min(power(10,12.9-2*Ph[ilayer]),power(10,(1.06/0.059)-3*pH[ilayer]))*W[ilayer]*dlz; {Fe+2 in moles/m2,dlz mm compartment thickness}
    if Fe2pool>WE_FePedogenic[ilayer] then Fe2pool:=WE_FePedogenic[ilayer];
    WE_FePedogenic[ilayer]:=WE_FePedogenic[ilayer]-Fe2pool;
    SE[14,ilayer]:=max(1E-10,convfac*Fe2pool/(W[ilayer]*dlz));
    {now the FeII+FeIII concentrations (mmol/dm3) are recalculated based on pH and pS and the pedogenic Fe pool (mol/m2,dlz mm) is adapted}

    {the rates below are not yet used but noted for later usage}
    {k_oxidationrate = 1.5 * EXP(-13); }
    {=oxidation rate of Fe2 in Litre2/MOL2/atm/minute; based on Stumm and Lee, 1961}
    {k_reductrate = 0.453;             }
    {=reduction rates of Fe3; units : per day; obtained from Roden and Urrutia, 1999}

    {2c. calculate IAP per mineral for layer ilayer}
    IAP[1]:=(SE[12,ilayer]/convfac)*(SE[3,ilayer]/convfac)*power(silacid,3)/power(tempHact,4); {1 IAP-Albite}
    IAP[2]:=(SE[12,ilayer]/convfac)*(SE[4,ilayer]/convfac)*power(silacid,3)/power(tempHact,4); {2 IAP-Kfeldspar}
    IAP[3]:=(power((SE[12,ilayer]/convfac),3)*(SE[4,ilayer]/convfac)*power(silacid,3)/power(tempHact,10)); {3 IAP-Muscovite}
    IAP[4]:=(SE[12,ilayer]/convfac)*(SE[4,ilayer]/convfac)*power((SE[2,ilayer]/convfac),1.5)*power((SE[13,ilayer]/convfac),1.5)*
            power(silacid,3)/power(tempHact,10); {4 IAP-Biotite}
    IAP[5]:=silacid; {5 IAP-Quartz}
    IAP[6]:=power((SE[12,ilayer]/convfac),2)*power((SE[2,ilayer]/convfac),5)*(SE[4,ilayer]/convfac)*power(silacid,3)/
            power(tempHact,16); {6 IAP-Chlorite}
    IAP[7]:=power((SE[12,ilayer]/convfac),2)*(SE[1,ilayer]/convfac)*power(silacid,2)/power(tempHact,8); {7 IAP-Anorthite}
    IAP[8]:=power((SE[2,ilayer]/convfac),2)*silacid/power(tempHact,4); {8 IAP-Fosterite}
    IAP[9]:=(SE[14,ilayer]/convfac)*silacid/power(tempHact,2); {9 IAP-Augite}
    IAP[10]:=power((SE[12,ilayer]/convfac),2)*power(silacid,2)/power(tempHact,6); {10 IAP-Kaolinite}
    IAP[11]:=power((SE[12,ilayer]/convfac),2.3)*power((SE[2,ilayer]/convfac),0.25)*power((SE[4,ilayer]/convfac),0.6)*
             power(silacid,3.5)/power(tempHact,8); {11 IAP-Illite}
    IAP[12]:=power((SE[12,ilayer]/convfac),11.67)*power((SE[2,ilayer]/convfac),0.37125)*power((SE[4,ilayer]/convfac),0.0824)*
             power((SE[1,ilayer]/convfac),0.04135)*power((SE[3,ilayer]/convfac),0.0824)*power(silacid,4)/
             power(tempHact,6); {12 IAP-Montmorillonite}
    IAP[13]:=power((SE[12,ilayer]/convfac),2)*power((SE[2,ilayer]/convfac),4)*power((SE[1,ilayer]/convfac),2)*
             power(silacid,6.7)/power(tempHact,18); {13 IAP-Hornblende}
    IAP[14]:=power((SE[14,ilayer]/convfac),2)*silacid/power(tempHact,4); {14 IAP-Fayalite}
    {IAP-Amorphite and IAP-Otherite: using a generic mineral composition (using a..i) we can derive a generic reaction equation
    Oa Alb Cac Mgd Ke Naf Feg Sih Hi + (4h-a)H2O + (4h-i)-2*(4h-a)H+ = bAl + cCa + dMg + eK + fNa + gFe + hH4SiO4
    and so a generic way to calculate the IAP
    Al^b * Mg^d * K^e * Ca^c * Na^f * (H4SiO4)^h * Fe^g / H^((4h-i)*2*(4h-a))
    For Amorphite and Otherite we know a..i because these are the stoichiometric coefficients in amorphite.txt and otherite.txt}
    {numerical safety:}
    if (SE[12,ilayer]=0)or(SE[2,ilayer]=0)or(SE[4,ilayer]=0)or(SE[1,ilayer]=0)or(SE[3,ilayer]=0)
       or(silacid=0)or(SE[14,ilayer]=0)or(SE[10,ilayer]=0)
    then begin IAP[15]:=0;  IAP[16]:=0; end
    else
    begin
      if MinPools[15][ilayer]<1E-10
      then IAP[15]:=0
      else
      begin
        a:=untGUI.amorp_O;  b:=untGUI.amorp_Al; c:=untGUI.amorp_Ca;
        d:=untGUI.amorp_Mg; e:=untGUI.amorp_K;  f:=untGUI.amorp_Na;
        g:=untGUI.amorp_Fe; h:=untGUI.amorp_Si; i:=untGUI.amorp_H;
        IAP[15]:=power((SE[12,ilayer]/convfac),b)*power((SE[2,ilayer]/convfac),d)*power((SE[4,ilayer]/convfac),e)*
                 power((SE[1,ilayer]/convfac),c)*power((SE[3,ilayer]/convfac),f)*power(silacid,h)*power((SE[14,ilayer]/convfac),g)/
                 power(tempHact,((4*h-i)*2*(4*h-a))); {15 IAP-Amorphite} ;
      end;
      if MinPools[16][ilayer]<1E-10
      then IAP[16]:=0
      else
      begin      ;
        a:=untGUI.other_O;  b:=untGUI.other_Al; c:=untGUI.other_Ca;
        d:=untGUI.other_Mg; e:=untGUI.other_K;  f:=untGUI.other_Na;
        g:=untGUI.other_Fe; h:=untGUI.other_Si; i:=untGUI.other_H;
        IAP[16]:=power((SE[12,ilayer]/convfac),b)*power((SE[2,ilayer]/convfac),d)*power((SE[4,ilayer]/convfac),e)*
                 power((SE[1,ilayer]/convfac),c)*power((SE[3,ilayer]/convfac),f)*power(silacid,h)*power((SE[14,ilayer]/convfac),g)/
                 power(tempHact,((4*h-i)*2*(4*h-a))); {16 IAP-Otherite} ;
      end;
    end;
    {4. Calculate the omega as ratio IAP/Keq, making sure that it cannot be >1 for primary minerals
        The if..then..else are to prevent floating point overflows}
    if (IAP[1]>0.0001*KAlbit)
    then Omega[1]:=1
    else if (IAP[1]<0.0001*KAlbit)
         then Omega[1]:=0
         else Omega[1]:=min(1,IAP[1]/KAlbit);  {primary mineral}
    if (IAP[2]>0.0001*KKFeld)
    then Omega[2]:=1
    else if (IAP[2]<0.0001*KKFeld)
         then Omega[2]:=0
         else Omega[2]:=min(1,IAP[2]/KKFeld);  {primary mineral}
    if (IAP[3]>0.0001*KMusco)
    then Omega[3]:=1
    else if (IAP[3]<0.0001*KMusco)
         then Omega[3]:=0
         else Omega[3]:=min(1,IAP[3]/KMusco);  {primary mineral}
    if (IAP[4]>0.0001*KBioti)
    then Omega[4]:=1
    else if (IAP[4]<0.0001*KBioti)
         then Omega[4]:=0
         else Omega[4]:=min(1,IAP[4]/KBioti);  {primary mineral}
    if IAP[5]>maxomega*KQuart  {newformation not excluded, but maximize Omega to maxomega}
    then omega[5]:=maxomega
    else if IAP[5]<0.0001*KQuart then omega[5]:=0
                            else Omega[5]:=min(1.1,IAP[5]/KQuart);
    if IAP[6]>maxomega*KChlor  {newformation not excluded, but maximize Omega to maxomega}
    then omega[6]:=maxomega
    else if IAP[6]<0.0001*KChlor then omega[6]:=0
                            else Omega[6]:=min(1.1,IAP[6]/KChlor);
    if (IAP[7]>0.0001*KAnort)
    then Omega[7]:=1
    else if (IAP[7]<0.0001*KAnort)
         then Omega[7]:=0
         else Omega[7]:=min(1,IAP[7]/KAnort);  {primary mineral}
    if (IAP[8]>0.0001*KForst)
    then Omega[8]:=1
    else if (IAP[8]<0.0001*KForst)
         then Omega[8]:=0
         else Omega[8]:=min(1,IAP[8]/KForst);  {primary mineral}
    if (IAP[9]>0.0001*KAugit)
    then Omega[9]:=1
    else if (IAP[9]<0.0001*KAugit)
         then Omega[9]:=0
         else Omega[9]:=min(1,IAP[9]/KAugit);  {primary mineral}
    if IAP[10]>maxomega*KKaoli  {newformation not excluded, but maximize Omega to maxomega}
    then omega[10]:=maxomega
    else if IAP[10]<0.0001*KKaoli then omega[10]:=0
                            else Omega[10]:=min(1.1,IAP[10]/KKaoli);
    if IAP[11]>maxomega*KIllit  {newformation not excluded, but maximize Omega to maxomega}
    then omega[11]:=maxomega
    else if IAP[11]<0.0001*KIllit then omega[11]:=0
                            else Omega[11]:=min(1.1,IAP[11]/KIllit);
    if IAP[12]>maxomega*KMontm  {newformation not excluded, but maximize Omega to maxomega}
    then omega[12]:=maxomega
    else if IAP[12]<0.0001*KMontm then omega[12]:=0
                            else Omega[12]:=min(1.1,IAP[12]/KMontm);
    if (IAP[13]>0.0001*KHornb)
    then Omega[13]:=1
    else if (IAP[13]<0.0001*KHornb)
         then Omega[13]:=0
         else Omega[13]:=min(1,IAP[13]/KHornb);{primary mineral}
    if (IAP[14]>0.0001*KFayal)
    then Omega[14]:=1
    else if (IAP[14]<0.0001*KFayal)
         then Omega[14]:=0
         else Omega[14]:=min(1,IAP[14]/KFayal);{primary mineral}
    if (IAP[15]>0.0001*KAmorp)
    then Omega[15]:=1
    else if (IAP[15]<0.0001*KAmorp)
         then Omega[15]:=0
         else Omega[15]:=min(1,IAP[15]/KAmorp);{primary mineral}
    if (IAP[16]>0.0001*KOther)
    then Omega[16]:=1
    else if (IAP[16]<0.0001*KOther)
         then Omega[16]:=0
         else Omega[16]:=min(1,IAP[16]/KOther);{primary mineral}
  end;
end;

procedure WE_Daily;
{weathering of soil minerals on a daily basis; calculates daily cation release by weathering and adjusts pools}
{calculates per soil compartment the daily release of Al, Ca, Mg, K, Na from
 hydrolysis of primary minerals and adds these to the soil solution}
var
  Aalb, Akfeld, Abio, Aqtz, Amusco, Achlor, Aanort, AForster, Aaugit, Akaol,
  Aillit, Amontm, Ahornb, Afayal, Aamorp, Aother, rjkalb, rjkkfeld, rjkbio, rjkqtz, rjkmusco,
  rjkchlor, rjkanort, rjkForster, rjkaugit, rjkkaol, rjkillit, rjkmontm, rjkhornb, rjkfayal,
  rjkamorp, rjkother,
  KHalb, KOHalb, KHkfeld, KOHkfeld, KHbio, KOHbio, KHqtz, KOHqtz, KHmusco,
  KOHmusco, KHchlor, KOHchlor, KHanort, KOHanort, KHForster, KOHForster,
  KHaugit, KOHaugit, KHkaol, KOHkaol, KHillit, KOHillit, KHmontm, KOHmontm,
  KHhornb, KOHhornb, KHfayal, KOHfayal, KHamorp, KOHamorp, KHother, KOHother,
  T, FCa, FNa, FK, FSi, FAl, FFe, FMg,
  lossAlbite, lossKfeldspar, lossMuscovite, lossBiotite, lossQuartz,
  lossChlorite, lossAnorthite, lossForsterite, lossAugite,
  lossKaolinite, lossIllite, lossMontmorillonite, lossHornblende, lossFayalite,
  lossAmorphite, lossOtherite,
  Moalb, Mokfeld, Momusco, Moqtz, Mobio, Mochlor, Moanort, MoForster, Moaugit,
  Mokaol, Moillit, Momontm, Mohornb, Mofayal, MoAmorp, MoOther, FNamols, FCamols, FKmols,
  FFemols, FSimols, FAlmols, FMgmols, Atot, MassTot,
  redfacQtz, redfacChlor, redfacKaol, redfacIllit, redfacMontm: real;
  MassFractions: array[1..NrMinerals] of Profile;
  {contain mass fractions summing to 1 of the nrminerals primary minerals}
  ilayer, imineral: integer;
  clay, silt, sand, Temp_diff: real;
  tmp, deltAl, tempHact, tempOHact: real;
  txt:string;
  code:integer;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering WE_Daily');
  T := DLZ / 1000; {T is compartment thickness in meter}
  for ilayer := 2 to kk - 1 do
  begin
    {extra reductionfactors for weathering rates only for the newforming minerals
    in case the solution is too much depleted of elements. Initially set to =1}
    redfacQtz:=1;  redfacChlor:=1;  redfacKaol:=1;  redfacIllit:=1;  redfacMontm:=1;

    {Area, A (m2/mol) calculated by multiplying area (m2/g) by the RFM of the mineral}
    {calculate mass fractions from current mass of the <nrminerals> minerals}
    MassTot := 0;
    for imineral := 1 to nrminerals do
      MassTot := MassTot + MinPools[imineral][ilayer];
    for imineral := 1 to nrminerals do
      MassFractions[imineral][ilayer] := MinPools[imineral][ilayer] / MassTot;

    {calculating current number of moles of each mineral, from current mass, Mo is moles/m3}
    Moalb := (MinPools[1][ilayer] / RFMalbite) * 1000;
    Mokfeld := (MinPools[2][ilayer] / RFMkfeldspar) * 1000;
    Momusco := (MinPools[3][ilayer] / RFMmuscovite) * 1000;
    Mobio := (MinPools[4][ilayer] / RFMbiotite) * 1000;
    Moqtz := (MinPools[5][ilayer] / RFMquartz) * 1000;
    Mochlor := (MinPools[6][ilayer] / RFMchlorite) * 1000;
    Moanort := (MinPools[7][ilayer] / RFManorthite) * 1000;
    MoForster := (MinPools[8][ilayer] / RFMForsterite) * 1000;
    Moaugit := (MinPools[9][ilayer] / RFMaugite) * 1000;
    Mokaol := (MinPools[10][ilayer] / RFMkaolinite) * 1000;
    Moillit := (MinPools[11][ilayer] / RFMillite) * 1000;
    Momontm := (MinPools[12][ilayer] / RFMmontmorillonite) * 1000;
    Mohornb := (MinPools[13][ilayer] / RFMhornblende) * 1000;
    Mofayal := (MinPools[14][ilayer] / RFMfayalite) * 1000;
    if RFMamorphite>0 then MoAmorp:= (MinPools[15][ilayer] / RFMamorphite) * 1000 else MoAmorp:=0;
    if RFMotherite>0 then MoOther:= (MinPools[16][ilayer] / RFMotherite) * 1000 else MoOther:=0;

    MassTot := SANDm[ilayer] + SILTm[ilayer] + CLAYm[ilayer];
    {MassTot now used for total mass over all texture fractions}
    clay := CLAYm[ilayer] / MassTot; {clay=clay fraction}
    silt := SILTm[ilayer] / MassTot;
    sand := fineSANDm[ilayer] / MassTot; {thus: only sand <256 microns!}

    {begin by calculating total mineral area, Atot}
    Atot := 8 * clay + 2.2 * silt + 0.3 * sand;
    {now calculate individual mineral areas and convert units from m2/g to m2/mol}

    Aalb := MassFractions[1][ilayer] * Atot * RFMalbite;
    Akfeld := MassFractions[2][ilayer] * Atot * RFMkfeldspar;
    Amusco := MassFractions[3][ilayer] * Atot * RFMmuscovite;
    Abio := MassFractions[4][ilayer] * Atot * RFMbiotite;
    Aqtz := MassFractions[5][ilayer] * Atot * RFMquartz;
    Achlor := MassFractions[6][ilayer] * Atot * RFMchlorite;
    Aanort := MassFractions[7][ilayer] * Atot * RFManorthite;
    AForster := MassFractions[8][ilayer] * Atot * RFMForsterite;
    Aaugit := MassFractions[9][ilayer] * Atot * RFMaugite;
    Akaol := MassFractions[10][ilayer] * Atot * RFMkaolinite;
    Aillit := MassFractions[11][ilayer] * Atot * RFMillite;
    Amontm := MassFractions[12][ilayer] * Atot * RFMmontmorillonite;
    Ahornb := MassFractions[13][ilayer] * Atot * RFMhornblende;
    Afayal := MassFractions[14][ilayer] * Atot * RFMfayalite;
    if RFMamorphite>0 then Aamorp:= MassFractions[15][ilayer] * Atot * RFMamorphite else Aamorp:=0;
    if RFMotherite>0 then Aother:= MassFractions[16][ilayer] * Atot * RFMotherite else Aother:=0;

    {begin correcting mineral dissolution rate constants for temperatures}
    Temp_diff := (1 / T_const - 1 / (TEM[ilayer] + 273.15));
    KHalb := MinDisPar[1, 1] * Exp((MinDisPar[2, 1] / R) * (Temp_diff));
    KOHalb := MinDisPar[4, 1] * Exp((MinDisPar[5, 1] / R) * (Temp_diff));
    KHkfeld := MinDisPar[1, 2] * Exp((MinDisPar[2, 2] / R) * (Temp_diff));
    KOHkfeld := MinDisPar[4, 2] * Exp((MinDisPar[5, 2] / R) * (Temp_diff));
    KHmusco := MinDisPar[1, 3] * Exp((MinDisPar[2, 3] / R) * (Temp_diff));
    KOHmusco := MinDisPar[4, 3] * Exp((MinDisPar[5, 3] / R) * (Temp_diff));
    KHbio := MinDisPar[1, 4] * Exp((MinDisPar[2, 4] / R) * (Temp_diff));
    KOHbio := MinDisPar[4, 4] * Exp((MinDisPar[5, 4] / R) * (Temp_diff));
    KHqtz := MinDisPar[1, 5] * Exp((MinDisPar[2, 5] / R) * (Temp_diff));
    KOHqtz := MinDisPar[4, 5] * Exp((MinDisPar[5, 5] / R) * (Temp_diff));
    KHchlor := MinDisPar[1, 6] * Exp((MinDisPar[2, 6] / R) * (Temp_diff));
    KOHchlor := MinDisPar[4, 6] * Exp((MinDisPar[5, 6] / R) * (Temp_diff));
    KHanort := MinDisPar[1, 7] * Exp((MinDisPar[2, 7] / R) * (Temp_diff));
    KOHanort := MinDisPar[4, 7] * Exp((MinDisPar[5, 7] / R) * (Temp_diff));
    KHForster := MinDisPar[1, 8] * Exp((MinDisPar[2, 8] / R) * (Temp_diff));
    KOHForster := MinDisPar[4, 8] * Exp((MinDisPar[5, 8] / R) * (Temp_diff));
    KHaugit := MinDisPar[1, 9] * Exp((MinDisPar[2, 9] / R) * (Temp_diff));
    KOHaugit := MinDisPar[4, 9] * Exp((MinDisPar[5, 9] / R) * (Temp_diff));
    KHkaol := MinDisPar[1, 10] * Exp((MinDisPar[2, 10] / R) * (Temp_diff));
    KOHkaol := MinDisPar[4, 10] * Exp((MinDisPar[5, 10] / R) * (Temp_diff));
    KHillit := MinDisPar[1, 11] * Exp((MinDisPar[2, 11] / R) * (Temp_diff));
    KOHillit := MinDisPar[4, 11] * Exp((MinDisPar[5, 11] / R) * (Temp_diff));
    KHmontm := MinDisPar[1, 12] * Exp((MinDisPar[2, 12] / R) * (Temp_diff));
    KOHmontm := MinDisPar[4, 12] * Exp((MinDisPar[5, 12] / R) * (Temp_diff));
    KHhornb := MinDisPar[1, 13] * Exp((MinDisPar[2, 13] / R) * (Temp_diff));
    KOHhornb := MinDisPar[4, 13] * Exp((MinDisPar[5, 13] / R) * (Temp_diff));
    KHfayal := MinDisPar[1, 14] * Exp((MinDisPar[2, 14] / R) * (Temp_diff));
    KOHfayal := MinDisPar[4, 14] * Exp((MinDisPar[5, 14] / R) * (Temp_diff));
    KHamorp := MinDisPar[1, 15] * Exp((MinDisPar[2, 15] / R) * (Temp_diff));
    KOHamorp := MinDisPar[4, 15] * Exp((MinDisPar[5, 15] / R) * (Temp_diff));
    KHother := MinDisPar[1, 16] * Exp((MinDisPar[2, 16] / R) * (Temp_diff));
    KOHother := MinDisPar[4, 16] * Exp((MinDisPar[5, 16] / R) * (Temp_diff));

    {PF2023: Insert here a call to IAPOmega to calcuate
     1. the relevant ion activity products (in array IAP)
     2. the Saturation degree Omega (array OMEGA)
     3. the Aluminum inhibition factor finh (array FINH)         }
    IAPOMEGA(IAP,Omega,finh);

    {calculation of mineral dissolution rates,as a function of pH and pOH}
    {denote that KH, KOH, MinDisPar[3,*] and MinDisPar[6,*] can have the value 0 in case of no effect or no data.
     PF2023: rjk* rates are adapted by a factor (1-omega[ispecies])*finh[ilayer].
     For Quartz,Chlorite, Kaolinite, Illite and Montmorillonite omega may be >1,
     so their rates may become negative (newformation). We prepared for such situation by
     adding a reduction factor redfacQtz, redfacChlor, redfacKaol, redfacIllit and redfacMontm
     These have initially the value 1}
    tempHact := power(10, -pH[ilayer]);
    tempOHact := power(10, -(14 - pH[ilayer]));
    rjkalb := (KHalb * power(tempHact, MinDisPar[3, 1]) + KOHalb *
      power(tempOHact, MinDisPar[6, 1]))*(1-omega[1])*finh[ilayer];
    rjkkfeld := (KHkfeld * power(tempHact, MinDisPar[3, 2]) + KOHkfeld *
      power(tempOHact, MinDisPar[6, 2]))*(1-omega[2])*finh[ilayer];
    rjkmusco := (KHmusco * power(tempHact, MinDisPar[3, 3]) + KOHmusco *
      power(tempOHact, MinDisPar[6, 3]))*(1-omega[3])*finh[ilayer];
    rjkbio := (KHbio * power(tempHact, MinDisPar[3, 4]) + KOHbio *
      power(tempOHact, MinDisPar[6, 4]))*(1-omega[4])*finh[ilayer];
    rjkanort := (KHanort * power(tempHact, MinDisPar[3, 7]) + KOHanort *
      power(tempOHact, MinDisPar[6, 7]))*(1-omega[7])*finh[ilayer];
    rjkForster := (KHForster * power(tempHact, MinDisPar[3, 8]) +
      KOHForster * power(tempOHact, MinDisPar[6, 8]))*(1-omega[8])*finh[ilayer];
    rjkaugit := (KHaugit * power(tempHact, MinDisPar[3, 9]) +
      KOHaugit * power(tempOHact, MinDisPar[6, 9]))*(1-omega[9])*finh[ilayer];
    rjkhornb := (KHhornb * power(tempHact, MinDisPar[3, 13]) + KOHhornb *
      power(tempOHact, MinDisPar[6, 13]))*(1-omega[13])*finh[ilayer];
    rjkfayal := (KHfayal * power(tempHact, MinDisPar[3, 14]) + KOHfayal *
      power(tempOHact, MinDisPar[6, 14]))*(1-omega[14])*finh[ilayer];
    rjkamorp:= (KHamorp * power(tempHact, MinDisPar[3, 15]) + KOHamorp *
      power(tempOHact, MinDisPar[6, 15]))*(1-omega[15])*finh[ilayer];
    rjkother:= (KHother * power(tempHact, MinDisPar[3, 16]) + KOHother *
      power(tempOHact, MinDisPar[6, 16]))*(1-omega[16])*finh[ilayer];
    {PF2023: now the secundary minerals that may precipitate.
    Adaptation of rjk* and redfac* is needed
    if updated concentrations of each element would become negative
    due to excessive precipitation}
    rjkqtz := (KHqtz * power(tempHact, MinDisPar[3, 5]) + KOHqtz *
      power(tempOHact, MinDisPar[6, 5]))*(1-omega[5])*finh[ilayer]*redfacQtz;
    rjkchlor := (KHchlor * power(tempHact, MinDisPar[3, 6]) + KOHchlor *
      power(tempOHact, MinDisPar[6, 6]))*(1-omega[6])*finh[ilayer]*redfacChlor;
    rjkkaol := (KHkaol * power(tempHact, MinDisPar[3, 10]) + KOHkaol *
      power(tempOHact, MinDisPar[6, 10]))*(1-omega[10])*finh[ilayer]*redfacKaol;
    rjkillit := (KHillit * power(tempHact, MinDisPar[3, 11]) + KOHillit *
      power(tempOHact, MinDisPar[6, 11]))*(1-omega[11])*finh[ilayer]*redfacIllit;
    rjkmontm := (KHmontm * power(tempHact, MinDisPar[3, 12]) + KOHmontm *
      power(tempOHact, MinDisPar[6, 12]))*(1-omega[12])*finh[ilayer]*redfacMontm;
    {indicator (test purpose)}
    {str(omega[5]:5:3,txt); untGUI.frmMain.lblNOTE.Caption:=txt ;
    str(omega[6]:5:3,txt); untGUI.frmMain.lblNOTE.Caption:=untGUI.frmMain.lblNOTE.Caption+'  '+txt ;
    str(omega[10]:5:3,txt); untGUI.frmMain.lblNOTE.Caption:=untGUI.frmMain.lblNOTE.Caption+'  '+txt ;
    str(omega[11]:5:3,txt); untGUI.frmMain.lblNOTE.Caption:=untGUI.frmMain.lblNOTE.Caption+'  '+txt ;
    str(omega[12]:5:3,txt); untGUI.frmMain.lblNOTE.Caption:=untGUI.frmMain.lblNOTE.Caption+'  '+txt ;
    untGUI.frmMain.lblNOTE.Refresh; }
    {end indicator (test)}
    {PF2023: here we update rjk* and redfac* so that concentrations of each element
     will remain positive after precipitation of a secundary mineral.}
    AdaptRjkRedfac(redfacQtz,redfacChlor,redfacKaol,redfacIllit,redfacMontm,
                   rjkqtz,rjkchlor,rjkkaol,rjkillit,rjkmontm,
                   rjkalb,rjkkfeld,rjkbio,rjkmusco,rjkanort,rjkForster,rjkaugit,rjkhornb,rjkfayal,rjkamorp,rjkother,
                   Aalb,Akfeld,Abio,Amusco,Aanort,AForster,Aaugit,Ahornb,Afayal,Aamorp,Aother,
                   Aqtz,Achlor,Akaol,Aillit,Amontm,
                   Moalb,Mokfeld,Momusco,Mobio,Moanort,MoForster,Moaugit,Mohornb,Mofayal,MoAmorp,MoOther,
                   Moqtz,Mochlor,Mokaol,Moillit,Momontm,ilayer);

  {calculation of fluxes (moles/m2/s) of elements into solution from dissolution of each mineral,
   as a function of mineral area, dissolution rates, mass and compartment thickness}
    FCa := T * (0 * Akfeld * rjkkfeld * Mokfeld + 0 * Amusco * rjkmusco * Momusco +
      0 * Aalb * rjkalb * Moalb + 0 * Abio * rjkbio * Mobio + 0 * Aqtz * rjkqtz * Moqtz +
      0 * Achlor * rjkchlor * Mochlor + 1 * Aanort * rjkanort * Moanort +
      0 * AForster * rjkForster * MoForster + 0 * Aaugit * rjkaugit * Moaugit +
      0 * Akaol * rjkkaol * Mokaol + 0.01 * Aillit * rjkillit * Moillit +
      0.04125 * Amontm * rjkmontm * Momontm + 2 * Ahornb * rjkhornb * Mohornb +
      0 * Afayal * rjkfayal * Mofayal + max(0,untGUI.amorp_Ca) * Aamorp * rjkamorp * Moamorp +
      max(0,untGUI.other_Ca) * Aother * rjkother * Moother);
    FCamols := FCa * 60 * 60 * 24; {converting flux (moles/m2/s) to moles/day; area is 1 m2}
    {Add Ca to solution here}
    tmp := SE[1, ilayer] * W[ilayer] * dlz + 1000 * FCamols;
    {total millimoles already present in solution and added}
    SE[1, ilayer] := tmp / (W[ilayer] * dlz);             {millimoles/dm3 in solution }
    {decrease CaPool with what has been weathered}
    WE_CaPool[ilayer] := WE_CaPool[ilayer] - FCamols;

    FMg := T * (0 * Akfeld * rjkkfeld * Mokfeld + 0 * Amusco * rjkmusco * Momusco +
      0 * Aalb * rjkalb * Moalb + 1.5 * Abio * rjkbio * Mobio + 0 * Aqtz * rjkqtz * Moqtz +
      5 * Achlor * rjkchlor * Mochlor + 0 * Aanort * rjkanort * Moanort +
      2 * AForster * rjkForster * MoForster + 0 * Aaugit * rjkaugit * Moaugit +
      0 * Akaol * rjkkaol * Mokaol + 0.02 * Aillit * rjkillit * Moillit +
      (0.33 + 0.04125) * Amontm * rjkmontm * Momontm + 4 * Ahornb * rjkhornb * Mohornb +
      0 * Afayal * rjkfayal * Mofayal + max(0,untGUI.amorp_Mg) * Aamorp * rjkamorp * Moamorp +
      max(0,untGUI.other_Mg) * Aother * rjkother * Moother);
    FMgmols := FMg * 60 * 60 * 24; {converting flux (moles/m2/s) to moles/day; area is 1 m2}
    {Add Mg to solution here}
    tmp := SE[2, ilayer] * W[ilayer] * dlz + 1000 * FMgmols;
    {total millimoles already present in solution and added}
    SE[2, ilayer] := tmp / (W[ilayer] * dlz);             {millimoles/dm3 in solution }
    {decrease MgPool with what has been weathered}
    WE_MgPool[ilayer] := WE_MgPool[ilayer] - FMgmols;

    FK := T * (1 * Akfeld * rjkkfeld * Mokfeld + 1 * Amusco * rjkmusco * Momusco +
      0 * Aalb * rjkalb * Moalb + 1 * Abio * rjkbio * Mobio + 0 * Aqtz * rjkqtz * Moqtz +
      0 * Achlor * rjkchlor * Mochlor + 0 * Aanort * rjkanort * Moanort +
      0 * AForster * rjkForster * MoForster + 0 * Aaugit * rjkaugit * Moaugit +
      0 * Akaol * rjkkaol * Mokaol + 0.44 * Aillit * rjkillit * Moillit +
      0.0825 * Amontm * rjkmontm * Momontm + 0 * Ahornb * rjkhornb * Mohornb +
      0 * Afayal * rjkfayal * Mofayal + max(0,untGUI.amorp_K) * Aamorp * rjkamorp * Moamorp +
      max(0,untGUI.other_K) * Aother * rjkother * Moother);
    FKmols := FK * 60 * 60 * 24; {converting flux (moles/m2/s) to moles/day; area is 1 m2}
    {Add K to solution here}
    tmp := SE[4, ilayer] * W[ilayer] * dlz + 1000 * FKmols;
    {total millimoles already present in solution and added}
    SE[4,ilayer]:=tmp/(W[ilayer]*dlz);{PB2023: uncommented! millimoles/dm3 in solution }
    {decrease KPool with what has been weathered}
    WE_KPool[ilayer] := WE_KPool[ilayer] - FKmols;

    FNa := T * (0 * Akfeld * rjkkfeld * Mokfeld + 0 * Amusco * rjkmusco * Momusco +
      1 * Aalb * rjkalb * Moalb + 0 * Abio * rjkbio * Mobio + 0 * Aqtz * rjkqtz * Moqtz +
      0 * Achlor * rjkchlor * Mochlor + 0 * Aanort * rjkanort * Moanort +
      0 * AForster * rjkForster * MoForster + 0 * Aaugit * rjkaugit * Moaugit +
      0 * Akaol * rjkkaol * Mokaol + 0 * Aillit * rjkillit * Moillit + 0.0825 * Amontm * rjkmontm * Momontm +
      0 * Ahornb * rjkhornb * Mohornb + 0 * Afayal * rjkfayal * Mofayal +
      max(0,untGUI.amorp_Na) * Aamorp * rjkamorp * Moamorp +
      max(0,untGUI.other_Na) * Aother * rjkother * Moother);
    FNamols := FNa * 60 * 60 * 24; {converting flux (moles/m2/s) to moles/day; area is 1 m2}
    {Add Na to solution here}
    tmp := SE[3, ilayer] * W[ilayer] * dlz + 1000 * FNamols;
    {total millimoles already present in solution and added}
    SE[3, ilayer] := tmp / (W[ilayer] * dlz);             {millimoles/dm3 in solution }
    {decrease NaPool with what has been weathered}
    WE_NaPool[ilayer] := WE_NaPool[ilayer] - FNamols;

    FSi := T * (3 * Akfeld * rjkkfeld * Mokfeld + 3 * Amusco * rjkmusco * Momusco +
      3 * Aalb * rjkalb * Moalb + 3 * Abio * rjkbio * Mobio + 1 * Aqtz * rjkqtz * Moqtz +
      3 * Achlor * rjkchlor * Mochlor + 2 * Aanort * rjkanort * Moanort +
      1 * AForster * rjkForster * MoForster + 1 * Aaugit * rjkaugit * Moaugit +
      2 * Akaol * rjkkaol * Mokaol + 3.4 * Aillit * rjkillit * Moillit + 4 * Amontm * rjkmontm * Momontm +
      6.7 * Ahornb * rjkhornb * Mohornb + 1 * Afayal * rjkfayal * Mofayal +
      max(0,untGUI.amorp_Si) * Aamorp * rjkamorp * Moamorp +
      max(0,untGUI.other_Si) * Aother * rjkother * Moother);
    FSimols := FSi * 60 * 60 * 24; {converting flux (moles/m2/s) to moles/day; area is 1 m2}
    {Add Si to solution here}
  {Si non-existing species until PF2023:}
   tmp:=SE[15,ilayer]*W[ilayer]*dlz+1000*FSimols;  {total millimoles already present in solution and added}
   SE[15,ilayer]:=tmp/(W[ilayer]*dlz);             {millimoles/dm3 in solution }
    {decrease SiPool with what has been weathered}
    WE_SiPool[ilayer] := WE_SiPool[ilayer] - FSimols;
    WE_SiPedogenic[ilayer]:=WE_SiPedogenic[ilayer]+FSimols;

    FFe := T * (0 * Akfeld * rjkkfeld * Mokfeld + 0 * Amusco * rjkmusco * Momusco +
      0 * Aalb * rjkalb * Moalb + 1.5 * Abio * rjkbio * Mobio + 0 * Aqtz * rjkqtz * Moqtz +
      0 * Achlor * rjkchlor * Mochlor + 0 * Aanort * rjkanort * Moanort +
      0 * AForster * rjkForster * MoForster + 1 * Aaugit * rjkaugit * Moaugit +
      0 * Akaol * rjkkaol * Mokaol + 0 * Aillit * rjkillit * Moillit + 0 * Amontm * rjkmontm * Momontm +
      0 * Ahornb * rjkhornb * Mohornb + 2 * Afayal * rjkfayal * Mofayal +
      max(0,untGUI.amorp_Fe) * Aamorp * rjkamorp * Moamorp +
      max(0,untGUI.other_Fe) * Aother * rjkother * Moother);
    FFemols := FFe * 60 * 60 * 24; {converting flux (moles/m2/s) to moles/day; area is 1 m2}
    {Add Fe to solution here}
  {Fe non-existing species until PF2023: I added all to the FeII pool! }
   tmp:=SE[14,ilayer]*W[ilayer]*dlz+1000*FFemols;  {total millimoles already present in solution and added  }
   SE[14,ilayer]:=tmp/(W[ilayer]*dlz);             {millimoles/dm3 in solution }
    {decrease FePool with what has been weathered}
    WE_FePool[ilayer] := WE_FePool[ilayer] - FFemols;
    WE_FePedogenic[ilayer]:=WE_FePedogenic[ilayer]+FFemols;

    FAl := T * (1 * Akfeld * rjkkfeld * Mokfeld + 3 * Amusco * rjkmusco * Momusco +
      1 * Aalb * rjkalb * Moalb + 1 * Abio * rjkbio * Mobio + 0 * Aqtz * rjkqtz * Moqtz +
      2 * Achlor * rjkchlor * Mochlor + 2 * Aanort * rjkanort * Moanort +
      0 * AForster * rjkForster * MoForster + 0 * Aaugit * rjkaugit * Moaugit +
      2 * Akaol * rjkkaol * Mokaol + 2.38 * Aillit * rjkillit * Moillit +
      1.67 * Amontm * rjkmontm * Momontm + 0 * Ahornb * rjkhornb * Mohornb +
      0 * Afayal * rjkfayal * Mofayal + max(0,untGUI.amorp_Al) * Aamorp * rjkamorp * Moamorp +
      max(0,untGUI.other_Al) * Aother * rjkother * Moother);
    FAlmols := FAl * 60 * 60 * 24; {converting flux (moles/m2/s) to moles/day; area is 1 m2}
    {Add Al to solution and precip Gibbsite if necessary}
    tmp := SE[12, ilayer] * W[ilayer] * dlz + 1000 * FAlmols;
    {total millimoles of Al already present in solution and added over the current day}
    SE[12, ilayer] := tmp / (W[ilayer] * dlz);
    {millimoles/dm3 of Al in solution if no gibbsite would precip}
    tmp := 1000 * KSP * tempHact * tempHact * tempHact;
    {max concentration in mmol/dm3 in equilibrium with Gibbsite at current pH}
    if SE[12, ilayer] > tmp then  {oversaturated: precip deltAl to Gibbsite}
    begin
      deltAl := (SE[12, ilayer] - tmp) * W[ilayer] * dlz;
      {delAl=millimoles Al that should precipitate into Gibbsite (in moles)}
      Gibbsitepool[ilayer] :=  Gibbsitepool[ilayer] + deltAl/1000;
      {Gibbsitepool in mol /m2 over dlz mm}
      SE[12, ilayer] := tmp;
    end;
    {decrease AlPool with what has been weathered}
    WE_AlPool[ilayer] := WE_AlPool[ilayer] - FAlmols;

    {calculation of fluxes (moles/m2/day) of minerals that dissolve}
    lossAlbite := Aalb * rjkalb * Moalb * 60 * 60 * 24 * T;
    {moles/m2,day from m2/mol * mol/m2,s * mol/m3soil * s/day * m } ;
    lossKfeldspar := Akfeld * rjkkfeld * Mokfeld * 60 * 60 * 24 * T;
    {moles/m2,day } ;
    lossMuscovite := Amusco * rjkmusco * Momusco * 60 * 60 * 24 * T;
    {moles/m2,day } ;
    lossBiotite := Abio * rjkbio * Mobio * 60 * 60 * 24 * T;
    {moles/m2,day } ;
    lossQuartz := Aqtz * rjkqtz * Moqtz * 60 * 60 * 24 * T;
    {moles/m2,day } ;
    lossChlorite := Achlor * rjkchlor * Mochlor * 60 * 60 * 24 * T;
    {moles/m2,day } ;
    lossAnorthite := Aanort * rjkanort * Moanort * 60 * 60 * 24 * T;
    {moles/m2,day } ;
    lossForsterite := AForster * rjkForster * MoForster * 60 * 60 * 24 * T;
    {moles/m2,day } ;
    lossAugite := Aaugit * rjkaugit * Moaugit * 60 * 60 * 24 * T;
    {moles/m2,day } ;
    lossKaolinite := Akaol * rjkkaol * Mokaol * 60 * 60 * 24 * T;
    {moles/m2,day } ;
    lossIllite := Aillit * rjkillit * Moillit * 60 * 60 * 24 * T;
    {moles/m2,day } ;
    lossMontmorillonite := Amontm * rjkmontm * Momontm * 60 * 60 * 24 * T;
    {moles/m2,day } ;
    lossHornblende := Ahornb * rjkhornb * Mohornb * 60 * 60 * 24 * T;
    {moles/m2,day } ;
    lossFayalite := Afayal * rjkfayal * Mofayal * 60 * 60 * 24 * T;
    {moles/m2,day } ;
    lossAmorphite := Aamorp * rjkamorp * Moamorp * 60 * 60 * 24 * T;
    {moles/m2,day } ;
    lossOtherite := Aother * rjkother * Moother * 60 * 60 * 24 * T;
    {moles/m2,day } ;

  {calculation of moles and mass of minerals, each time subtracting what goes to solution:
  moles/m2 of minerals remaining and kg of minerals remaining after 1 timestep of 1 day}
    Mokfeld := Mokfeld - lossKfeldspar;
    minpools[2][ilayer] := Mokfeld * RFMkfeldspar / 1000;
    {in kg again};
    Momusco := Momusco - lossMuscovite;
    minpools[3][ilayer] := Momusco * RFMmuscovite / 1000;
    Moqtz := Moqtz - lossQuartz;
    minpools[5][ilayer] := Moqtz * RFMquartz / 1000;
    Moalb := Moalb - lossAlbite;
    minpools[1][ilayer] := Moalb * RFMalbite / 1000;
    Mobio := Mobio - lossBiotite;
    minpools[4][ilayer] := Mobio * RFMbiotite / 1000;
    Mochlor := Mochlor - lossChlorite;
    minpools[6][ilayer] := Mochlor * RFMchlorite / 1000;
    Moanort := Moanort - lossAnorthite;
    minpools[7][ilayer] := Moanort * RFManorthite / 1000;
    MoForster := MoForster - lossForsterite;
    minpools[8][ilayer] := MoForster * RFMForsterite / 1000;
    Moaugit := Moaugit - lossAugite;
    minpools[9][ilayer] := Moaugit * RFMaugite / 1000;
    Mokaol := Mokaol - lossKaolinite;
    minpools[10][ilayer] := Mokaol * RFMkaolinite / 1000;
    Moillit := Moillit - lossIllite;
    minpools[11][ilayer] := Moillit * RFMillite / 1000;
    Momontm := Momontm - lossMontmorillonite;
    minpools[12][ilayer] := Momontm * RFMmontmorillonite / 1000;
    Mohornb := Mohornb - lossHornblende;
    minpools[13][ilayer] := Mohornb * RFMhornblende / 1000;
    Mofayal := Mofayal - lossFayalite;
    minpools[14][ilayer] := Mofayal * RFMfayalite / 1000;
    Moamorp := Moamorp - lossAmorphite;
    if RFMamorphite>0 then minpools[15][ilayer] := Moamorp * RFMamorphite / 1000;
    Moother := Moother - lossOtherite;
    if RFMotherite>0 then minpools[16][ilayer] := Moother * RFMotherite / 1000;
    {PF2023: Now dynamic recalculation of MM21 and SUM21 from clay mineralogy}
    if (MinPools[11,ilayer]+MinPools[12,ilayer]+MinPools[6,ilayer]+minpools[10][ilayer])=0
    then begin MM21[ilayer]:=0; SUM21[ilayer]:=0; end
    else
    begin
      {MM21=100*montmorillonite/(illite+montmorillonite+chlorite+kaolinite)}
      MM21[ilayer]:=100*MinPools[12,ilayer]/(MinPools[11,ilayer]+MinPools[12,ilayer]+MinPools[6,ilayer]+minpools[10][ilayer]) ;
      {SUM21=100*(illite+montmorillonite)/(illite+montmorillonite+chlorite+kaolinite), assuming Chlorite is 2:1 but has low CEC and does not disperse}
      SUM21[ilayer]:=100*(MinPools[11,ilayer]+MinPools[12,ilayer])/(MinPools[11,ilayer]+MinPools[12,ilayer]+MinPools[6,ilayer]+minpools[10][ilayer]) ;
    end;
  end; {for ilayer:=2 to kk-1}
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving WE_Daily');
end;

procedure WE_Yearly;
{void: calculates yearly cation release by weathering and adjusts pools; not used}
{calculates per soil compartment the annual release of Al, Ca, Mg, K, Na from
 hydrolysis of primary minerals and adds these to the soil solution}
begin
  {this routine would be copy of WE_daily but with an additional "365" rate modifying factor}
end;

procedure WE_Exit;
{write minerals and cations locked in minerals to WE*.out,
i.e. weathering data (size remaining cation pools in primary minerals)}
{Writes annual output data from the weathering routines; pf08}
var
  fwe: textfile;
  i, j: integer;
begin
  {if GUItrace = 1 then}
    singletracewriteln('Entering WriteDataWE');
  system.Assign(fwe, fileWE + '.out');
  system.rewrite(fwe);
  system.writeln(fwe, '[Mass of minerals (kg)]');
  system.writeln(fwe,
    '[Layer Albite K-Feldspar Muscovite Biotite Quartz Chlorite Anorthite' +
    'Forsterite Augite Kaolinite Illite Montmorillonite Hornblende Fayalite Gibbsite Amorphite Otherite]');
  for i := 2 to kk - 1 do
  begin
    system.Write(fwe, ' ', i: 2, ' ');
    for j := 1 to nrminerals do
      system.Write(fwe, MinPools[j][i], ' ');
    system.writeln(fwe);
  end;
  system.writeln(fwe, '[Cations still locked in minerals]');
  system.writeln(fwe, '[layer ',
    'Gibbsite Al (mol)          ',
    'Ca (mol)          ',
    'Mg (mol)          ',
    'K  (mol)          ',
    'Na (mol)          ',
    'Si (mol)          ',
    'Fe (mol)          ',
    'Al (mol)]         ',
    'PedogenicFe (mol) ',
    'PedogenicSi (mol) ');
  for i := 2 to kk - 1 do
  begin
    system.writeln(fwe, ' ', i: 2, ' ', Gibbsitepool[i], ' ',
      WE_Capool[i], ' ', WE_Mgpool[i], ' ',
      WE_Kpool[i], ' ', WE_Napool[i], ' ',
      WE_Sipool[i], ' ', WE_Fepool[i], ' ',
      WE_Alpool[i], ' ',WE_FePedogenic[i], ' ',
      WE_SiPedogenic[i]);
  end;
  closefile(fwe);
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving WE_Exit');
end;

procedure estimate_KatH(compnr: integer; var clay, silt, rho, oc: real;
  var ksat, kath, ath: real);
{estimate the hydraulic conductivity at input hydraulic head using the fitted hydraulic parameters;
 currently used to fill cmatch-hmatch per soil compartment
 when PTF for soil hydraulic characteristics are estimated}
var
  alfa, en, el: real;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering estimate_KatH');
  {  Mean particle density}
  PARDEN[compnr] := 100 / (clay / DataInit.RHOS2 +
    (100 - clay - 1.7 * oc) / DataInit.RHOS1 + 1.7 * oc / DataInit.RHOS3);
  predictVGN(compnr, clay, silt, oc, rho, PARDEN[compnr]);
  ksat := VGNks[compnr];
  if ksat < 10 then
    ksat := 10; {minimize to 10 mm/day to prevent waterlogging e.g. in case of high rho's}
  ath := 0; {in mm}
  kath := ksat;
  {if (TRACE = 1) then}
    singletracewriteln(' Leaving estimate_KatH');
end;

procedure AdaptConst(temperature: real);
{change reaction constants using current temperatures (called from StartProcesses + CHEM2)}
{This function adapts the default reaction constants (at 25oC)
 to values at the current temperature. These values were determined in the calling program
 for whole degrees between -10 and +40 oC and put in kvaluetable[reaction;temperature]}
var
  itemp: integer;
begin
  itemp := round(temperature);
  if itemp < -10 then
    itemp := -10
  else if itemp > 40 then
    itemp := 40;
  KH :=   DataInit.kvaluetable[1, itemp] {Henry's law};
  KW :=   DataInit.kvaluetable[2, itemp] {water stability};
  KA1 :=  DataInit.kvaluetable[3, itemp] {1st diss H2CO3};
  KA2 :=  DataInit.kvaluetable[4, itemp] {2nd diss H2CO3};
  KD1 :=  DataInit.kvaluetable[5, itemp] {stab CoCO3o};
  KD2 :=  DataInit.kvaluetable[6, itemp] {stab CaHCO3+};
  KD3 :=  DataInit.kvaluetable[7, itemp] {stab CaOH+};
  KD4 :=  DataInit.kvaluetable[8, itemp] {stab CaSO4o};
  KD5 :=  DataInit.kvaluetable[9, itemp] {stab MgCO3o};
  KD6 :=  DataInit.kvaluetable[10, itemp]{stab MgHCO3+};
  KD7 :=  DataInit.kvaluetable[11, itemp]{stab MgOH+};
  KD8 :=  DataInit.kvaluetable[12, itemp]{stab MgSO4o};
  KD9 :=  DataInit.kvaluetable[13, itemp]{stab NaSO4-};
  KD10 := DataInit.kvaluetable[14, itemp]{stabNaCO3-};
  KD11 := DataInit.kvaluetable[15, itemp]{stab KSO4-};
  SP1 :=  DataInit.kvaluetable[16, itemp]{solub gypsum};
  SP2 :=  DataInit.kvaluetable[17, itemp]{solub calcite};
  KSP :=  DataInit.kvaluetable[18, itemp]{KSP Solubility product of Gibbsite};
  KAl1 := DataInit.kvaluetable[19, itemp]{KAl1 Stability constant AlOH_2p};
  KAl2 := DataInit.kvaluetable[20, itemp]{KAl2 Stability constant AlOH2_p};
  KAl4 := DataInit.kvaluetable[21, itemp]{KAl4 Stability constant AlOH4_m };
  Kf1 :=  DataInit.kvaluetable[22, itemp]{Kf1 Stability constant AlF_2p};
  Kf2 :=  DataInit.kvaluetable[23, itemp]{Kf2 Stability constant AlF2_p};
  Kf3 :=  DataInit.kvaluetable[24, itemp]{Kf3 Stability constant AlF3};
  Kf4 :=  DataInit.kvaluetable[25, itemp]{Kf4 Stability constant AlF4_m};
  Kf5 :=  DataInit.kvaluetable[26, itemp]{Kf5 Stability constant AlF5_2m};
  Ko1 :=  DataInit.kvaluetable[27, itemp]{Ko1 Stability constant AlOrg};
  Ko2 :=  DataInit.kvaluetable[28, itemp]{Ko2 Stability constant AlHOrg_p};
  Kg1 :=  DataInit.kvaluetable[29, itemp]{Kg1 Stability constant H2Org_m};
  Kg2 :=  DataInit.kvaluetable[30, itemp]{Kg2 Stability constant HOrg_2m};
  Kg3 :=  DataInit.kvaluetable[31, itemp]{Kg3 Stability constant Org_3m};
  KHF :=  DataInit.kvaluetable[32, itemp]{KHF Dissociation constant HF};
  KCaF := DataInit.kvaluetable[33, itemp]{CaF2 solubility constant of CalciumFluoride};
  KAlbit:=DataInit.kvaluetable[34, itemp]{Solubility product of Albite};
  KKFeld:=DataInit.kvaluetable[35, itemp]{Solubility product of K-Feldspar};
  KMusco:=DataInit.kvaluetable[36, itemp]{Solubility product of Muscovite};
  KBioti:=DataInit.kvaluetable[37, itemp]{Solubility product of Biotite};
  KQuart:=DataInit.kvaluetable[38, itemp]{Solubility product of Quartz};
  KChlor:=DataInit.kvaluetable[39, itemp]{Solubility product of Chlorite};
  KAnort:=DataInit.kvaluetable[40, itemp]{Solubility product of Anorthite};
  KForst:=DataInit.kvaluetable[41, itemp]{Solubility product of Forsterite};
  KAugit:=DataInit.kvaluetable[42, itemp]{Solubility product of Augite};
  KKaoli:=DataInit.kvaluetable[43, itemp]{Solubility product of Kaolinite};
  KIllit:=DataInit.kvaluetable[44, itemp]{Solubility product of Illite};
  KMontm:=DataInit.kvaluetable[45, itemp]{Solubility product of Montmorillonite};
  KHornb:=DataInit.kvaluetable[46, itemp]{Solubility product of Hornblende};
  KFayal:=DataInit.kvaluetable[47, itemp]{Solubility product of Fayalite};
  KAmorp:=DataInit.kvaluetable[48, itemp]{Solubility product of Amorphite};
  KOther:=DataInit.kvaluetable[49, itemp]{Solubility product of Otherite};
  {added}
  KGibbs:=DataInit.kvaluetable[50, itemp]{Solubility product of Gibbsite};
  KGypsu:=DataInit.kvaluetable[51, itemp]{Solubility product of Gypsum};
  KCalci:=DataInit.kvaluetable[52, itemp]{Solubility product of Calcite};
end;

procedure Physical_Weathering(var deltheat: real);
{calculate effect of physical weathering on grain size distribution during 10% of day}
var
  i, L, j, kklocal: integer; txt:string; tmp:real;
begin
  untGUI.frmMAIN.MVAR(GlobalTime*100000+Datayearin.nolayers*1000+cumt,' start untProcesses.Physical_Weathering: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
  untGUI.frmMAIN.MVAR(GlobalTime*100000+Datayearin.nolayers*1000+cumt,' start untProcesses.Physical_Weathering: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);

  {Variables and constants; are all declared in untGUI and initialised there,
   some of these are updated in this procedure.
   particle siz classes: -4=stones; -3..0=gravel; 1..11=fine earth (11=clay)
   PW_X[-4..11] is the ribbon of the cube for 16 particle size classes;
   PW_G[-4..11,1..NoLayers] is the mass fraction (mineral material only) per psclass
   PW_S[-4..11,1..NoLayers] is the number of split particles per psclass during the timestep
   PW_K[-4..11,1..NoLayers] is the remaining number of grains per psclass at the start of the timestep
   PW_E is the expectation of the number of particles needed to split 1 particle during tstep
   PW_P is the actual probability to split 1 particle once (calculated)
   PW_M is the number of splits to have 1 particle turn into 8 particles of the smaller psclass (=7)
   PW_Pmax is the max. probability to split 1 particle once (=input)
   PW_dTdtPmax is the temperature gradient per tstep where P becomes Pmax (=input)   }
  {if (TRACE = 1) then}
    singletracewriteln(' Entering Physical_Weathering');
  {calc absolute change in temperature over deltheat time, where deltheat is fraction of day
   and tem_old contains temperatures at t=t-deltheat and tem has just been updated in HEAT}
  for i := 2 to kk - 1 do
  begin
    dTdt[i] := abs(tem_old[i] - tem[i]) / deltheat;
    if (tem_old[i]*tem[i]<0)and(w[i]/TS[i]>0.9) then freezecounter[i]:=freezecounter[i]+1;
    if RothC.RPMpool[i]>=untGUI.rootcounterthreshold then Rootcounter[i]:=Rootcounter[i]+1;
  end;
  kklocal:=kk-1;
  {calc actual P}

  for i := 2 to kklocal do
  begin
    L := i - 1; {L=layer number used in untGUI}
    {calc actual split probability}
    if dTdt[i] < untGUI.PW_dTdtPmax then
      untGUI.PW_P := dTdt[i] * untGUI.PW_Pmax / untGUI.PW_dTdtPmax
    else
      untGUI.PW_P := untGUI.PW_Pmax;
    {calc expectation of the number of particles needed to split 1 particle}
    if untGUI.PW_P > 0 then
      untGUI.PW_E := untGUI.PW_M / untGUI.PW_P
    else
      untGUI.PW_E := 0;
    {calc actual particles per psclass that will be split}
    for j := -3 to 11 do      {j=-4 = stones do not follow this weathering approach}
    begin
      {how many particles are split}
      if untGUI.PW_E > 0 then
        untGUI.PW_S[j, L] := round(
          min(untGUI.PW_K[j, L], untGUI.PW_K[j, L] / untGUI.PW_E))
      else
        untGUI.PW_S[j, L] := 0; {no particles split because probability = 0}
      {update amount of particles, including input from larger psclass}
      if (j > -3) and (j < 11) then
        untGUI.PW_K[j, L] := untGUI.PW_K[j, L] - untGUI.PW_S[j, L] + 8 *
          untGUI.PW_S[j - 1, L]
      else
      begin
        if (j = -3) then
        begin
          untGUI.PW_K[j, L] := untGUI.PW_K[j, L] - untGUI.PW_S[j, L];
          {str(untGUI.PW_S[j, L],txt);
          untGUI.frmMAIN.lblNote.caption:=txt; untGUI.frmMAIN.lblNote.refresh;  }
          {PF2023: note: perhaps additions from the stone fractions will be addedlater...}
        {j=-3 is the largest gravel fraction}
        end
        else if j=11 then {clay fraction j=11: add new grains to clay mass; also influenced by clay migration}
        begin
          CLAYm[i] := CLAYm[i] + 0.08 * untGUI.PW_S[10, L] *
            power((untGUI.PW_X[11] / 100000), 3) *
            untGUI.PW_PARDEN[11]; {for 1 m2 only!}
          untGUI.PW_K[j, L] :=
            round(CLAYm[i] * 100 / (untGUI.PW_PARDEN[11] *
            power((untGUI.PW_X[11] / 100000), 3)));
          untGUI.mass[11, L] := claym[i] * 100;         {for 100 m2}
        end;
      end;
    end;
    if freezecounter[i]>100 then
    begin
      freezecounter[i]:=0; {now 2% (input) of the stones enter the largest gravel class by physical weathering}
      tmp:=frStoneSplit*untGUI.PW_K[-4, L];
      untGUI.PW_K[-4, L] := round(untGUI.PW_K[-4, L] - tmp);
      untGUI.PW_K[-3, L] := round(untGUI.PW_K[-3, L] + tmp)
    end;
    if rootcounter[i]/3650>100 then
    begin
      rootcounter[i]:=0; {now 2% (input) of the stones enter the largest gravel class by physical weathering}
      tmp:=frStoneSplit*untGUI.PW_K[-4, L];
      untGUI.PW_K[-4, L] := round(untGUI.PW_K[-4, L] - tmp);
      untGUI.PW_K[-3, L] := round(untGUI.PW_K[-3, L] + tmp)
    end;
    SILTm[i] := (untGUI.PW_K[6, L] * power(
      (untGUI.PW_X[6] / 100000), 3) * untGUI.PW_PARDEN[6] / 100) + {eqv mass[6,i]}
      (untGUI.PW_K[7, L] * power(
      (untGUI.PW_X[7] / 100000), 3) * untGUI.PW_PARDEN[7] / 100) + {eqv mass[7,i]}
      (untGUI.PW_K[8, L] * power(
      (untGUI.PW_X[8] / 100000), 3) * untGUI.PW_PARDEN[8] / 100) + {eqv mass[8,i]}
      (untGUI.PW_K[9, L] * power(
      (untGUI.PW_X[9] / 100000), 3) * untGUI.PW_PARDEN[9] / 100) + {eqv mass[9,i]}
      (untGUI.PW_K[10, L] * power(
      (untGUI.PW_X[10] / 100000), 3) * untGUI.PW_PARDEN[10] / 100); {eqv mass[10,i]}
    SANDm[i] := (untGUI.PW_K[1, L] * power(
      (untGUI.PW_X[1] / 100000), 3) * untGUI.PW_PARDEN[1] / 100) + {eqv mass[1,i]}
      (untGUI.PW_K[2, L] * power(
      (untGUI.PW_X[2] / 100000), 3) * untGUI.PW_PARDEN[2] / 100) + {eqv mass[2,i]}
      (untGUI.PW_K[3, L] * power(
      (untGUI.PW_X[3] / 100000), 3) * untGUI.PW_PARDEN[3] / 100) + {eqv mass[3,i]}
      (untGUI.PW_K[4, L] * power(
      (untGUI.PW_X[4] / 100000), 3) * untGUI.PW_PARDEN[4] / 100) + {eqv mass[4,i]}
      (untGUI.PW_K[5, L] * power(
      (untGUI.PW_X[5] / 100000), 3) * untGUI.PW_PARDEN[5] / 100); {eqv mass[5,i]}
    fineSANDm[i] := (untGUI.PW_K[4, L] * power(
      (untGUI.PW_X[4] / 100000), 3) * untGUI.PW_PARDEN[4] / 100) + {eqv mass[4,i]}
      (untGUI.PW_K[5, L] * power(
      (untGUI.PW_X[5] / 100000), 3) * untGUI.PW_PARDEN[5] / 100); {eqv mass[5,i]}
    Gravelm[i] := (untGUI.PW_K[-3, L] * power(
      (untGUI.PW_X[-3] / 100000), 3) * untGUI.PW_PARDEN[-3] / 100) + {eqv mass[-3,i]}
      (untGUI.PW_K[-2, L] * power(
      (untGUI.PW_X[-2] / 100000), 3) * untGUI.PW_PARDEN[-2] / 100) + {eqv mass[-2,i]}
      (untGUI.PW_K[-1, L] * power(
      (untGUI.PW_X[-1] / 100000), 3) * untGUI.PW_PARDEN[-1] / 100) + {eqv mass[-1,i]}
      (untGUI.PW_K[0, L] * power(
      (untGUI.PW_X[0] / 100000), 3) * untGUI.PW_PARDEN[0] / 100);    {eqv mass[0,i]}
    Stonesm[i]:= (untGUI.PW_K[-4, L] * power(
      (untGUI.PW_X[-4] / 100000), 3) * untGUI.PW_PARDEN[-4] / 100);
    totalm[i] := CLAYm[i] + SILTm[i] + SANDm[i] + 1.7 * OCm[i] + Gravelm[i] + stonesm[i];
    gravel[i]:=100*gravelm[i]/totalm[i];
    stones[i]:=100*stonesm[i]/totalm[i];
  end;

  {In case of bedrock: the upper bedrock compartment is also subject to physical weathering.
   Assuming drivers for physical weathering in the "rock" are the same as in the deepest unconsolidated compartment}
  if untGUI.rock.RockBottomLayers>0 then
  begin
    {Only count the rootcounter and the freezecounter; values at the end of each year will be used to calculate mass lost from stone fraction to finer fractions,
     using the frStoneSplit to reduce the mass and nr of particles in the stone fraction, and to increase the fine earth fraction.
     This will be done in the routine PassDataOut}
     if (tem_old[kk-1]*tem[kk-1]<0)and(w[kk-1]/TS[kk-1]>0.9)
        then untGUI.rock.saproFreezecounter:=untGUI.rock.saproFreezecounter+1;
     {old: only raise rootcounter when RPM above threshold
     if RothC.RPMpool[kk-1]>=untGUI.rootcounterthreshold
        then untGUI.rock.saproRootcounter:=untGUI.rock.saproRootcounter+1;}
     {new: raise rootcounter with fraction of threshold, which will be 0 below rootzone}
     untGUI.rock.saproRootcounter:=untGUI.rock.saproRootcounter+min(1,round(RothC.RPMpool[kk-1]/untGUI.rootcounterthreshold))
  end;

  {if (TRACE = 1) then}
    singletracewriteln(' Leaving Physical_Weathering');
    untGUI.frmMAIN.MVAR(GlobalTime*100000+Datayearin.nolayers*1000+cumt,' einde untProcesses.Physical_Weathering: DataYearIn.RHO[nolayers]=',DataYearIn.RHO[Datayearin.nolayers]);
    untGUI.frmMAIN.MVAR(GlobalTime*100000+Datayearin.nolayers*1000+cumt,' einde untProcesses.Physical_Weathering: DataYearOut.RHO[nolayers]=',DataYearout.RHO[Datayearout.nolayers]);

end;

Procedure predictVGN(compnr:integer; cl,si,carbon,rho,PARDEN:real);
{predict Van Genuchten parameters using Hypres continuous functions (called from RETPRED + estimateKatH)}
var clay, silt, om, rho1: real;
topsoil: integer;
begin
if not (untGUI.DataYearIn.VGNmeasured[compnr-1]) then
begin {only if VanGenuchten parameters are not measured
      and supplied from an inputfile specified in <eventsINT.txt>:
      estimate VanGenuchten parameters using the Hypress functions
      for non-peat soils and the Staring Series PTF for peat soils.}
  {Calculate porosity}
  TS[compnr] := max(1 - RHO / PARDEN,0.05); {calculate using the bulkdensity but make it minimally 5%}
  {now correct for stoniness by reducing TS by (100-stonepercentage)/100, thus by a factor.
  Later, when theta-res is estimated, TS is checked and corrected so that it is always greater than theta-res}
  TS[compnr]:=TS[compnr]*(100-stoneperc[compnr])/100;
  clay := max(1, 100 * cl / (100 - 1.7 * carbon));
  silt := max(1, 100 * si / (100 - 1.7 * carbon));
  om := max(0.1, 1.7 * carbon);
  rho1 := min(1.9,max(0.8, rho)); {otherwise outside validity domain ptf}
  if clay < 1 then
    clay := 1;
  if carbon > 18 then   {class PTF for organic layers, Wosten et al 1999}
  begin
    VGNts[compnr] := TS[compnr];
    VGNtr[compnr] := 0.01;  {value from Wosten et al 1999, class PTF}
    VGNal[compnr] := 0.013;
    VGNen[compnr] := 1.2039;
    VGNel[compnr] := 0.4;
    VGNks[compnr] := 80;  {here in mm/day!}
    VGNem[compnr] := 1 - 1 / VGNen[compnr];
  end
  else
  begin
    {consider as topsoil the profile part with some bioturbation}
    if untGUI.dbot > untGUI.DataInit.DLZ * (compnr - 1) then
      topsoil := 1
    else
      topsoil := 0;
    VGNts[compnr] := TS[compnr];
    VGNtr[compnr] := 0.01;  {value from Wosten et al 1999, class PTF}
    VGNal[compnr] := -14.96 + 0.03135 * clay + 0.0351 * silt + 0.646 * om + 15.29 * RHO1 - 0.192 * topsoil -
      4.671 * RHO1 * RHO1 - 0.000781 * clay * clay - 0.00687 * om * om +
      0.0449 / om + 0.0663 * LN(silt) + 0.1482 * LN(om) - 0.04546 *
      RHO1 * silt - 0.4852 * RHO1 * om + 0.00673 * topsoil * clay;
    VGNen[compnr] := -25.23 - 0.02195 * clay + 0.0074 * silt - 0.194 * om + 45.5 * RHO1 - 7.24 * RHO1 * RHO1 +
      0.0003658 * clay * clay + 0.002885 * om * om - 12.81 / RHO1 -
      0.1524 / silt - 0.01958 / om - 0.2876 * ln(SILT) - 0.0709 * LN(OM) - 44.6 * LN(RHO1) -
      0.02264 * RHO1 * CLAY + 0.0896 * RHO1 * OM + 0.00718 * topsoil * clay;
    VGNel[compnr] := 0.0202 + 0.0006193 * clay * clay - 0.001136 * om * om - 0.2316 * ln(om) -
      0.03544 * RHO1 * clay + 0.00283 * RHO1 * silt + 0.0488 * RHO1 * om;
    VGNks[compnr] := 7.755 + 0.0352 * silt + 0.93 * topsoil - 0.967 * RHO1 * RHO1 - 0.000484 * clay * clay -
      0.000322 * silt * silt + 0.001 / silt - 0.0748 / om - 0.643 * ln(silt) -
      0.01398 * RHO1 * clay - 0.1673 * RHO1 * om + 0.02986 * topsoil *
      clay - 0.03305 * topsoil * silt;
    {transformations}
    VGNal[compnr] := exp(VGNal[compnr]);
    {VGNal[compnr]:=min(0.145,max(VGNal[compnr],0.008));} {if uncommented: restrict to domain 0.008=clay..0.145=sand}
    VGNen[compnr] := exp(VGNen[compnr]) + 1;
    {VGNen[compnr] :=min(2.68,max(VGNen[compnr],1.09));}  {if uncommented: restrict to domain 1.09=clay..2.68=sand}
    VGNel[compnr] := exp(VGNel[compnr]);
    VGNel[compnr] := (10 * VGNel[compnr] - 10) / (1 + VGNel[compnr]);
    VGNks[compnr] := max(5,(10 * exp(VGNks[compnr])));  {convert to mm/day but make it minimally 5 mm !}
    VGNem[compnr] := 1 - 1 / VGNen[compnr];
  end;
end
else
begin
  {Van Genuchten parameters were read from file in calling programme};
  VGNts[compnr] := untGUI.VGNts[compnr-1];
  {read value was already corrected to 1-RHO/PARDEN}
  VGNtr[compnr] := untGUI.VGNtr[compnr-1];
  VGNal[compnr] := untGUI.VGNal[compnr-1];
  VGNen[compnr] := untGUI.VGNen[compnr-1];
  VGNel[compnr] := untGUI.VGNel[compnr-1];
  VGNks[compnr] := untGUI.VGNks[compnr-1];
  VGNem[compnr] := untGUI.VGNem[compnr-1];
  TS[compnr] := min((max(1 - RHO / PARDEN,0.05)), VGNts[compnr]);
  {calculate using the bulkdensity but make it minimally 5%}
end;
if TS[compnr]<VGNtr[compnr]
then
begin
  TS[compnr]:=VGNtr[compnr]+0.05;
  VGNts[compnr]:=TS[compnr];
end;
{transformations done, now estimate equivalent Campbell parameters
 from the Van Genuchten parameters according to the paper
 Sommer, R. and Stöckle, C. 2010. Correspondence between the Campbell and
 van Genuchten Soil-Water-Retention models. Journal of irrigation and drainage
 engineering, 2010: 559-562.}
ABCAM[compnr] := 0.4183 + 1.008 * (1 - VGNem[compnr]) /
  (VGNem[compnr] * (1 - power(0.5, 1 / VGNem[compnr])) /
  (1 - (VGNtr[compnr] / (0.5 * (VGNtr[compnr] + VGNts[compnr])))));
ABCAM[compnr]:=max(1.1,min(16,ABCAM[compnr])); {restrict b for rho >1.95 and rho <0.5}
BCAM[compnr] := ABCAM[compnr];
AAEV[compnr] := -0.1576 + 0.7617 * (1 / VGNal[compnr]) *
  POWER(0.5 + VGNtr[compnr] / (2 * VGNts[compnr]), ABCAM[compnr]) *
  POWER((POWER(0.5, -1 / VGNem[compnr]) - 1), 1 - VGNem[compnr]);
{restrict to domain [5;100] as in paper Sommer, R. and Stöckle, C. 2010.}
AAEV[compnr]:=max(5,min(100,AAEV[compnr]));
{NB: This prediction for air entry value AAEV is in cm water and positive,
     it should be in kPa and negative for SoilGen}
{first limit predicted values within tolerable range}
{based on Jabro, jabro and Fox, J. Environ. Qual. 35:1227–1236 (2006)}
AAEV[compnr]:=max(AAEV[compnr],3);
AAEV[compnr] := -AAEV[compnr] / 10.22;
AEV[compnr] := AAEV[compnr];
END;

procedure tmpdmp(x: real; txt: string; y: real);
{void; currently inactive temporary write of 2 params to outputfile (called from clay_migration)}
begin
  system.writeln(ftmp, x: 10: 8, txt, y: 10: 8);
end;

procedure calcphlab(var RHO, PH, AW, PHLAB: rLL; DLZ: real);
{transform the calculated pH as if it were measured in a 1:5 water extract; "pH-H2O 1#5"}
var
  phfac, phgrwater, molh: real;
  i: integer;
begin
  for i := 2 to kk - 1 do
  begin
    phfac := 10 / (RHO[i] * dlz * 1000);
    phgrwater := phfac * dlz * AW[i] * 1000;
    molh := power(10, -PH[i]) * phgrwater / 1000;
    phlab[i] := -log10((1000 * molh + power(10, -5.6) * 50 / 1000) / (phgrwater + 50));
  end;
end;

procedure WriteDailyTDD;
{write a time-depth data table to file for (max 6) selected soil properties for each day dring the current year}
var img,i,j,itemp:integer;
    txt,name1,name2:string;
begin
  {If TRACE=1 then wwwriteln} singletracewriteln('Entering AssignDailyTDD (header of file with daily diagrams)');
  for img:=1 to untGUI.frmMAIN.lstbxScreenVariables.Items.Count do
  begin
    for i:=1 to untGUI.maxTDD do
    begin
    name1:=untGUI.frmMAIN.lstbxScreenVariables.Items[img-1];
    name2:=untGUI.frmMAIN.cmbbxScreenVariables.Items[i-1];
    if name1=name2
    then
    begin
      if img=1 then
      begin
       SelectDailyTimeDepth(name1,dailytdd1);
       for j:=1 to (untGUI.maxpluscompartments-(KK-2))
       do system.write(untGUI.fdaily1,'-999 ');
       for j:=2 to KK-1 do system.write(untGUI.fdaily1,dailytdd1[j],' ');
       system.writeln(untGUI.fdaily1); system.flush(untGUI.fdaily1);
      end;
      if img=2 then
      begin
       SelectDailyTimeDepth(name1,dailytdd2);
       for j:=1 to (untGUI.maxpluscompartments-(KK-2))
       do system.write(untGUI.fdaily2,'-999 ');
       for j:=2 to KK-1 do system.write(untGUI.fdaily2,dailytdd2[j],' ');
       system.writeln(untGUI.fdaily2); system.flush(untGUI.fdaily2);
      end;
      if img=3 then
      begin
       SelectDailyTimeDepth(name1,dailytdd3);
       for j:=1 to (untGUI.maxpluscompartments-(KK-2))
       do system.write(untGUI.fdaily3,'-999 ');
       for j:=2 to KK-1 do system.write(untGUI.fdaily3,dailytdd3[j],' ');
       system.writeln(untGUI.fdaily3); system.flush(untGUI.fdaily3);
      end;
      if img=4 then
      begin
       SelectDailyTimeDepth(name1,dailytdd4);
       for j:=1 to (untGUI.maxpluscompartments-(KK-2))
       do system.write(untGUI.fdaily4,'-999 ');
       for j:=2 to KK-1 do system.write(untGUI.fdaily4,dailytdd4[j],' ');
       system.writeln(untGUI.fdaily4); system.flush(untGUI.fdaily4);
      end;
      if img=5 then
      begin
       SelectDailyTimeDepth(name1,dailytdd5);
       for j:=1 to (untGUI.maxpluscompartments-(KK-2))
       do system.write(untGUI.fdaily5,'-999 ');
       for j:=2 to KK-1 do system.write(untGUI.fdaily5,dailytdd5[j],' ');
       system.writeln(untGUI.fdaily5); system.flush(untGUI.fdaily5);
      end;
      if img=6 then
      begin
       SelectDailyTimeDepth(name1,dailytdd6);
       for j:=1 to (untGUI.maxpluscompartments-(KK-2))
       do system.write(untGUI.fdaily6,'-999 ');
       for j:=2 to KK-1 do system.write(untGUI.fdaily6,dailytdd6[j],' ');
       system.writeln(untGUI.fdaily6); system.flush(untGUI.fdaily6);
      end;
    end;
    end;
  end;
  {If TRACE=1 then wwwriteln} singletracewriteln('Leaving AssignDailyTDD (header of file with daily diagrams)');
end;

procedure SelectDailyTimeDepth(name:string; var DailyTDD:Profile);
{choose selected soil variable for daily output (called from WriteDailyTimeDepth)}
var i,j:integer;
      ry1,ry2,ry3,ry4,rsy,p1,p2,m:real;
      tmpCLAY,tmpSAND,tmpSILT,tmpOC,tmpRHO,tmpPARDEN,tmpTS:Profile;
begin
    {If TRACE=1 then wwwriteln} singletracewriteln('Entering SelectDailyTimeDepth');
    for i:=2 to kk-1 do
    begin
      DailyTDD[i]:=-999;
      tmpRHO[i]:=(CLAYm[i]+SILTm[i]+1.7*((RothC.DPMpool[i]+RothC.RPMpool[i]+RothC.BIOpool[i]+RothC.HUMpool[i]+
                      RothC.IOMpool[i])/10)+SANDm[i])/(((100-datayearin.stoneperc[i])/100)*dlz);
      {undone: tmpRHO[i]:=untGUI.frmMAIN.HYPRESRHO(100*CLAYm[i]/
                              (CLAYm[i]+SILTm[i]+SANDm[i]+1.7*((RothC.DPMpool[i]+RothC.RPMpool[i]+RothC.BIOpool[i]+RothC.HUMpool[i]+RothC.IOMpool[i])/10)),
                              1.7*100*((RothC.DPMpool[i]+RothC.RPMpool[i]+RothC.BIOpool[i]+RothC.HUMpool[i]+RothC.IOMpool[i])/10))/
                              (CLAYm[i]+SILTm[i]+SANDm[i]+1.7*((RothC.DPMpool[i]+RothC.RPMpool[i]+RothC.BIOpool[i]+RothC.HUMpool[i]+RothC.IOMpool[i])/10));}
    end;
    if pos('Alkalinity (mmol- per l)',name)>0 then
    begin for i:=2 to kk-1 do DailyTDD[i]:=SE[7][i]+2*SE[8][i]+SE[11][i]-SE[10][i]; end;
    if pos('Base Saturation (%)',name)>0 then
    begin for i:=2 to kk-1 do DailyTDD[i]:=100*(XSE[1][i]+XSE[2][i]+XSE[3][i]+XSE[4][i])/(XSE[1][i]+XSE[2][i]+XSE[3][i]+XSE[4][i]+XSE[10][i]+XSE[12][i]); end;
    if pos('Ca-exch (mmol+ per kg soil)',name)>0 then
    begin for i:=2 to kk-1 do DailyTDD[i]:=XSE[1][i]; end;
    if pos('Calcite (mass fraction)',name)>0 then
    begin for i:=2 to kk-1 do DailyTDD[i]:=(CALCITE[i]*100*DLZ/1000000)/(tmpRHO[i]*DLZ); end;
    if pos('Ca-solution (mmol per l)',name)>0 then
    begin for i:=2 to kk-1 do DailyTDD[i]:=SE[1][i]; end;
    if pos('Cl- (mmol per l)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=SE[5][i]; end;
    if pos('CO3-- (mmol per l)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=SE[8][i]; end;
    if pos('Diss. Ca+Mg+Na+K (mmol+ per l)',name)>0 then
    begin for i:=2 to kk-1 do DailyTDD[i]:=2*SE[1][i]+2*SE[2][i]+SE[3][i]+SE[4][i]; end;
    if pos('EC (mS per m)',name)>0 then
    begin for i:=2 to kk-1 do DailyTDD[i]:=EC[i]; end;
    if pos('ESP (%)',name)>0 then
    begin for i:=2 to kk-1 do DailyTDD[i]:=ESP[i]; end;
    if pos('ET (mm)',name)>0 then
    begin for i:=2 to kk-1 do DailyTDD[i]:=CTRAN[i]; end;
    if pos('CEC (mmol+ per kg soil)',name)>0 then
    begin for i:=2 to kk-1 do DailyTDD[i]:=XSE[1][i]+XSE[2][i]+XSE[3][i]+XSE[4][i]+XSE[10][i]+XSE[12][i]; end;
    if pos('Flux (mm)',name)>0 then
    begin for i:=2 to kk do DailyTDD[i]:=WFW[i-1]; end;
    if pos('Gypsum (mass fraction)',name)>0 then
    begin for i:=2 to kk-1 do DailyTDD[i]:=(GYP[i]*172*DLZ/1000000)/(tmpRHO[i]*DLZ); end;
    if pos('HCO3- (mmol per l)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=SE[7][i]; end;
    if pos('K-exch (mmol+ per kg soil)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=XSE[4][i]; end;
    if pos('K-solution (mmol per l)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=SE[4][i]; end;
    if pos('Mg-exch (mmol+ per kg soil)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=XSE[2][i]; end;
    if pos('Mg-solution (mmol per l)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=SE[2][i]; end;
    if pos('Na-exch (mmol+ per kg soil)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=XSE[3][i]; end;
    if pos('Na-solution (mmol per l)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=SE[3][i]; end;
    if pos('pH    (-log mol per l)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=PH[i]; end;
    if pos('Potential (kPa)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=H[i]/102.2; end;
    if pos('Roots (fraction of total)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=RDF[i]; end;
    if pos('SAR  (-)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=SAR[i]; end;
    if pos('SO4-- (mmol per l)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=SE[6][i]; end;
    if pos('Theta (cm.3 per cm.3)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=W[i]; end;
{clay, silt, sand, oc are usually updated at the end of the year via ExitClayMigration;
 has now pro forma to be done here as well}
    for i:=2 to kk-1 do
    begin
     {determine mass% per compartment of clay, silt and OC by the remaining mass}
      OCm[i]:=(RothC.DPMpool[i]+RothC.RPMpool[i]+RothC.BIOpool[i]+RothC.HUMpool[i]+RothC.IOMpool[i])/10;
      tmpCLAY[i]:=100*CLAYm[i]/(CLAYm[i]+SILTm[i]+SANDm[i]+1.7*OCm[i]);
      {tmpCLAY[i]:=100*CLAYm[i]/(claym[i]+siltm[i]+sandm[i]+1.7*OCm[i]);}
      tmpSILT[i]:=100*SILTm[i]/(CLAYm[i]+SILTm[i]+SANDm[i]+1.7*OCm[i]);
      {tmpSILT[i]:=100*SILTm[i]/(tmpRHO[i]*DLZ);}  
      tmpSAND[i]:=100*SANDm[i]/(CLAYm[i]+SILTm[i]+SANDm[i]+1.7*OCm[i]);
      {tmpSAND[i]:=100*SANDm[i]/(tmpRHO[i]*DLZ); }
      tmpOC[i]:=100*OCm[i]/(CLAYm[i]+SILTm[i]+SANDm[i]+1.7*OCm[i]);
      {tmpOC[i]:=100*OCm[i]/(claym[i]+siltm[i]+sandm[i]+1.7*OCm[i]);}
     {  Mean particle density}
      tmpPARDEN[i]:=100/(tmpCLAY[I]/RHOS2+(100-tmpCLAY[I]-1.7*tmpOC[I])/RHOS1+1.7*tmpOC[I]/RHOS3);
     {  Calculate porosity}
      tmpTS[I]:=max(((100-stoneperc[i])/100)*(1-tmpRHO[I]/tmpPARDEN[i]),0.05);
    end;
    if pos('RHO (kg per dm.3)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=tmpRHO[i]; end;
    if pos('Clay (mass% fine earth)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=tmpCLAY[i]; end;
    if pos('Silt (mass% fine earth)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=tmpSILT[i]; end;
    if pos('Sand (mass% fine earth)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=tmpSAND[i]; end;
    if pos('OC (mass% solid fraction)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=tmpOC[i]; end;
    if pos('Porosity (volume fraction)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=tmpTS[i]; end;
    if pos('Clay (volume%)',name)>0 then
    begin
      for i:=2 to kk-1 do
      begin
         ry1:=1.7*tmpOC[i]/RHOS3;
         ry2:=tmpsilt[i]/RHOS1;
         ry3:=tmpclay[i]/RHOS2;
         ry4:=(100-1.7*tmpOC[i]-tmpsilt[i]-tmpclay[i])/RHOS1;
         rsy:=ry1+ry2+ry3+ry4;
         {calculate volumetric clay percentage}
         DailyTDD[i]:=(100-tmpTS[i])*ry3/rsy;
      end;
    end;
    if pos('ClayDispersionIndicator (-)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=ClayDispInd[i]; end;
    if pos('Temperature (oC)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=TEM[i]; end;
    if pos('DecomposablePlantMaterial (ton per ha)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=RothC.DPMpool[i]; end;
    if pos('ResistantPlantMaterial (ton per ha)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=RothC.RPMpool[i]; end;
    if pos('Biomass (ton per ha)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=RothC.BIOpool[i]; end;
    if pos('Humus (ton per ha)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=RothC.HUMpool[i]; end;
    if pos('InertOrganicMatter (ton per ha)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=RothC.IOMpool[i]; end;
    if pos('pCO2 (bar)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=PCO2[i]; end;
    if pos('Al-exch (mmol+ per kg soil)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=XSE[12][i]; end;
    if pos('Gibbsite (mol per m2)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=Gibbsitepool[i]/1000; end;
    if pos('Al-solution (mmol per l)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=SE[12][i]; end;
    if pos('delta14C(ppt)',name)>0 then
    begin
     for i:=2 to kk-1 do
     begin
      if (RothC.DPMpool[i]+RothC.RPMpool[i]+RothC.BIOpool[i]+RothC.HUMpool[i]+RothC.IOMpool[i])<=0
      then DailyTDD[i]:=0
      else
       DailyTDD[i]:=(RothC.DPMXpool[i]+RothC.RPMXpool[i]+RothC.BIOXpool[i]+RothC.HUMXpool[i]+RothC.IOMXpool[i])/
                 (RothC.DPMpool[i]+RothC.RPMpool[i]+RothC.BIOpool[i]+RothC.HUMpool[i]+RothC.IOMpool[i]);
     end;
    end;
    if pos('age(14C years before simulation year)',name)>0 then
    begin
      for i:=2 to kk-1 do DailyTDD[i]:=-999;   {not calculated}
    end;
    if pos('CIA (% Nesbitt+Young 1982)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=-999;   {not calculated} end;
    if pos('IndexB (- Kronberg+Nesbitt 1981)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=-999;   {not calculated} end;
    if pos('CIW (% Harnois 1988)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=-999;   {not calculated} end;
    if pos('PIA (% FedoEtAl 1995)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=-999;   {not calculated} end;
    if pos('WI (- PriceEtAl 1991)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=-999;   {not calculated} end;
    if pos('Fk-DPM (- 14CpermilOverC in DPM)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=-999;   {not calculated} end;
    if pos('Fk-RPM (- 14CpermilOverC in RPM)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=-999;   {not calculated} end;
    if pos('Fk-BIO (- 14CpermilOverC in BIO)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=-999;   {not calculated} end;
    if pos('Fk-HUM (- 14CpermilOverC in HUM)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=-999;   {not calculated} end;
    if pos('Fk-IOM (- 14CpermilOverC in IOM)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=-999;   {not calculated} end;
    if pos('CPA (% BuggleEtAl 2011)',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=-999;   {not calculated} end;
    if pos('PlantAvailableWater(-)',name)>0 then
    begin
      P1:=101.9744;    {equivalent in cm water of pF 2.0}
      P2:=16161.85781; {equivalent in cm water of pF4.2}
      for i:=2 to kk-1 do
      begin
        m:=1-(1/VGNen[i]);
        DailyTDD[i]:=(VGNtr[i]+(VGNts[i]-VGNtr[i])/(power((1+power((VGNal[i]*P1),VGNen[i])),m)))-
                     (VGNtr[i]+(VGNts[i]-VGNtr[i])/(power((1+power((VGNal[i]*P2),VGNen[i])),m)));
      end;
    end;
    if pos('RateModifierMoistureTemperature',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=RothC.x2[i]; end;
    if pos('RateModifierGeochemistry',name)>0 then begin for i:=2 to kk-1 do DailyTDD[i]:=RothC.x3[i]; end;
    {insert more outputvariables here, if desired:
    Ca-in-minerals (mol per m2)
    Mg-in-minerals (mol per m2)
    Na-in-minerals (mol per m2)
    K-in-minerals (mol per m2)
    Fe-in-minerals (mol per m2)
    Si-in-minerals (mol per m2)
    Al-in-minerals (mol per m2)
    Albite (kg NaAlSi3O8 per m2)
    K-feldspar (kg KAlSi3O8 per m2)
    Muscovite (kg KAl3Si3O10(OH)2 per m2)
    Biotite (kg KMg1.5Fe1.5AlSi3O10(OH)2 per m2)
    Quartz (kg SiO2 per m2)
    ChloriteAsClinochlore (kg Mg5Al2Si3O10 (OH)8 per m2)
    Anorthite (kg CaAl2Si2O8 per m2)
    Fosterite (kg Mg2SiO4 per m2)
    AugiteAsOrthoferrosilite (kg FeSiO3 per m2)
    Kaolinite (kg Al2Si2O5(OH)4 per m2)
    Illite (kg Si3.4O10(OH)2Mg0.02Al2.38Ca0.01K0.44 per m2)
    MontmorilloniteAverageComposition (kg Si4O10(OH)2Mg0.33Al1.67Ca0.04125Mg0.04125K0.0825Na0.0825 per m2)
    Hornblende (kg Ca2Mg4AlSi6.7AlO22.4 per m2)
    Fayalite (kg Fe2SiO4 per m2)
    pH-H2O_lab (-log mol per l)
    }
    {If TRACE=1 then wwwriteln} singletracewriteln('Leaving SelectDailyTimeDepth');
  end;

procedure WE_init_sapro;
{bedrock layer: initialization weathering routines}
{status: NOT NEEDED}
{Reads the pool-data and gets the constants from the calling program, pf08..pf14}
var i,j:integer;
begin
  {if (TRACE = 1) then}
  singletracewriteln(' Entering WE_Init_sapro');
  {if (TRACE = 1) then}
  singletracewriteln(' Leaving WE_Init_sapro');
end;


Procedure IAPOMEGA_sapro(var saprOmega: Mindat; var saprfinh: Profile; var OmegaGibb,OmegaGyps,OmegaCalc:real);
{bedrock layer: calculates IonActivityProducts per mineral and the saturation degree Omega (called from WE_daily_sapro).
At present:  weathering under low ionic concentrations assumed,
no newformation calcite, gypsum, gibbsite and clay minerals, no Al inhibition}
var ilayer,j:Integer;
    convfac:real;
{this routine is comparable to that of the "regular" IAPOMEGA for the soil layers, but with some adaptations:
  1: it is only applied using the soil solution concentration SE[xxx] values of the bottom "soil" compartment, thus ilayer=kk-1
  2: for the saprolite we also include Gibbsite, Gypsum and Calcite assuming the Transition State Theory approach,
     thus:
     a. added procedure arguments (DONE) and calculations (DONE) for IAP (Ion Activity Products) of Gibbsite, Gypsum and Calcite
     b. added procedure arguments (DONE) and calculations for Omega of Gibbsite, Gypsum and Calcite (DONE)
        This required the calculation elsewhere of the KGibbs, KGypsu and KCalci (in adaptconst (DONE) using K and dHr (ReadDataEQ, DONE))}
begin
    ilayer:=kk - 1;convfac:=1000;
    {1. calculate aluminum inhibition factor, with SE brought to mol/liter}
    saprfinh[ilayer]:=power(1E-7/(1e-7+SE[13,ilayer]/convfac),0.33);
    {For the time being: weathering under low ionic concentrations assumed, no newformation calcite, gypsum, gibbsite and clay minerals,
    no Al inhibition}
    saprfinh[ilayer]:=1;
    for j:=1 to nrminerals do saprOmega[j]:=0;
    OmegaCalc:=0; OmegaGyps:=0; OmegaGibb:=0;
end;

procedure WE_Daily_sapro;
{bedrock layer: weathering of minerals on a daily basis}
{calculates per soil compartment the daily release of Al, Ca, Mg, K, Na from
 hydrolysis of primary minerals and adds these to the soil solution}
var
  Aalb, Akfeld, Abio, Aqtz, Amusco, Achlor, Aanort, AForster, Aaugit, Akaol,
  Aillit, Amontm, Ahornb, Afayal, Aamorp, Aother, AGibbs, AGypsu, ACalci, rjkalb, rjkkfeld, rjkbio, rjkqtz, rjkmusco,
  rjkchlor, rjkanort, rjkForster, rjkaugit, rjkkaol, rjkillit, rjkmontm, rjkhornb, rjkfayal,
  rjkamorp, rjkother,rjkcalc,rjkgyps,rjkgibb,OmegaGibb,OmegaGyps,OmegaCalc,
  KHalb, KOHalb, KHkfeld, KOHkfeld, KHbio, KOHbio, KHqtz, KOHqtz, KHmusco,
  KOHmusco, KHchlor, KOHchlor, KHanort, KOHanort, KHForster, KOHForster,
  KHaugit, KOHaugit, KHkaol, KOHkaol, KHillit, KOHillit, KHmontm, KOHmontm,
  KHhornb, KOHhornb, KHfayal, KOHfayal, KHamorp, KOHamorp, KHother, KOHother,
  KHCalci,KOHCalci,KHGypsu,KOHGypsu,KHGibbsi,KOHGibbsi,
  T,
  lossAlbite, lossKfeldspar, lossMuscovite, lossBiotite, lossQuartz,
  lossChlorite, lossAnorthite, lossForsterite, lossAugite,
  lossKaolinite, lossIllite, lossMontmorillonite, lossHornblende, lossFayalite,
  lossAmorphite, lossOtherite,lossCalcite,lossGypsum,lossGibbsite,
  Moalb, Mokfeld, Momusco, Moqtz, Mobio, Mochlor, Moanort, MoForster, Moaugit,
  Mokaol, Moillit, Momontm, Mohornb, Mofayal, MoAmorp, MoOther, MoGibbsi,MoCalci,MoGypsu,Atot: real;
  {contain mass fractions summing to 1 of the nrminerals primary minerals}
  saprOmega: Mindat;
  saprfinh: Profile;
  ilayer: integer;
  clay, silt, sand, Temp_diff, secperday: real;
  tempHact, tempOHact: real;
  code:integer;
begin
  {if (TRACE = 1) then}
    singletracewriteln(' Entering WE_Daily_sapro');
  T := DLZ / 1000; {T is compartment thickness in meter}
  secperday:=60*60*24; {seconds per day}
  ilayer:=kk-1; {only using the data from the lowermost soil layer and the bedrock data}

    {extra reductionfactors for weathering rates only for the newforming minerals
    in case the solution is too much depleted of elements. Initially set to =1}

    {Area, A (m2/mol) is for the rock calculated by the Rock porosity (fraction) multiplied by the RFM of the mineral}
    {Mass per mineral comes from the record Rock:
     rockAlbite[2]+rockKFeldspar[2]+rockMuscovite[2]+rockBiotite[2]+
         rockAnorthite[2]+rockForsterite[2]+rockAugite[2]+rockHornblende[2]+
         rockFayalite[2]+rockQuartz[2]+rockChlorite[2]+rockKaolinite[2]+
         rockIllite[2]+rockMontmorillonite[2]+rockAmorphite[2]+rockOtherite[2]+
         rockGibbsite[2]+rockCalcite[2]+rockGypsum[2]}
    {calculating current number of moles of each mineral, from current mass, Mo is moles/m3}
    {changed to use Rock data!}
    Moalb :=   (untGUI.Rock.rockAlbite[2]/ RFMalbite)* 1000;
    Mokfeld := (untGUI.Rock.rockKFeldspar[2]/ RFMkfeldspar) * 1000;
    Momusco := (untGUI.Rock.rockMuscovite[2]/ RFMmuscovite) * 1000;
    Mobio :=   (untGUI.Rock.rockBiotite[2]/ RFMbiotite) * 1000;
    Moqtz :=   (untGUI.Rock.rockQuartz[2]/ RFMquartz) * 1000;
    Mochlor := (untGUI.Rock.rockChlorite[2]/ RFMchlorite) * 1000;
    Moanort := (untGUI.Rock.rockAnorthite[2]/ RFManorthite) * 1000;
    MoForster := (untGUI.Rock.rockForsterite[2]/ RFMForsterite) * 1000;
    Moaugit := (untGUI.Rock.rockAugite[2]/ RFMaugite) * 1000;
    Mokaol :=  (untGUI.Rock.rockKaolinite[2]/ RFMkaolinite) * 1000;
    Moillit := (untGUI.Rock.rockIllite[2]/ RFMillite) * 1000;
    Momontm := (untGUI.Rock.rockMontmorillonite[2]/ RFMmontmorillonite) * 1000;
    Mohornb := (untGUI.Rock.rockHornblende[2]/ RFMhornblende) * 1000;
    Mofayal := (untGUI.Rock.rockFayalite[2]/ RFMfayalite) * 1000;
    if RFMamorphite>0 then MoAmorp:= (untGUI.Rock.rockAmorphite[2]/ RFMamorphite) * 1000 else MoAmorp:=0;
    if RFMotherite>0  then MoOther:= (untGUI.Rock.rockOtherite[2]/ RFMotherite) * 100 else MoOther:=0;
 {added}
    MoGibbsi:= (untGUI.Rock.rockGibbsite[2]/ RFMGibbsite) * 1000;
    MoCalci := (untGUI.Rock.rockCalcite[2]/ RFMCalcite) * 1000;
    MoGypsu := (untGUI.Rock.rockGypsum[2]/ RFMGypsum) * 1000;

    {begin by calculating total mineral area, Atot: estimated by rock porosity fraction!}
    Atot := untGUI.Rock.rockporosityperc/100;
    {But Atot can be adapted using the clay-silt-sand now present from physical weathering, expressed as mass fraction from the total mineral mass.
    This means that these masses must be updated during the physical weathering process, or at least during the annual update
     in Passdataout!
     Then take for Atot becomes a number >=the untGUI.Rock.rockporosityperc/100}
    if (untGUI.rock.saproclaym+untGUI.rock.saprosiltm+
                                       untGUI.rock.saprosandm+untGUI.rock.saprogravelm+
                                       untGUI.rock.saprostonesm)>0
    then
    begin
      clay:=untGUI.rock.saproclaym/(untGUI.rock.saproclaym+untGUI.rock.saprosiltm+
                                            untGUI.rock.saprosandm+untGUI.rock.saprogravelm+
                                            untGUI.rock.saprostonesm);
      silt:=untGUI.rock.saprosiltm/(untGUI.rock.saproclaym+untGUI.rock.saprosiltm+
                                            untGUI.rock.saprosandm+untGUI.rock.saprogravelm+
                                            untGUI.rock.saprostonesm);
      sand:=0.4*
            untGUI.rock.saprosandm/(untGUI.rock.saproclaym+untGUI.rock.saprosiltm+
                                            untGUI.rock.saprosandm+untGUI.rock.saprogravelm+
                                            untGUI.rock.saprostonesm);
    end
    else begin clay:=0; silt:=0;sand:=0; end;
    Atot := max(untGUI.Rock.rockporosityperc/100,8*clay+2.2*silt+0.3*sand);

    {update mass fractions: first the sum of all mineral masses}
    untGUI.rock.rockmasssum:=0;
    untGUI.rock.rockmasssum:=untGUI.rock.rockmasssum+untGUI.rock.rockAlbite[2];
    untGUI.rock.rockmasssum:=untGUI.rock.rockmasssum+untGUI.rock.rockKFeldspar[2];
    untGUI.rock.rockmasssum:=untGUI.rock.rockmasssum+untGUI.rock.rockMuscovite[2];
    untGUI.rock.rockmasssum:=untGUI.rock.rockmasssum+untGUI.rock.rockBiotite[2];
    untGUI.rock.rockmasssum:=untGUI.rock.rockmasssum+untGUI.rock.rockAnorthite[2];
    untGUI.rock.rockmasssum:=untGUI.rock.rockmasssum+untGUI.rock.rockForsterite[2];
    untGUI.rock.rockmasssum:=untGUI.rock.rockmasssum+untGUI.rock.rockAugite[2];
    untGUI.rock.rockmasssum:=untGUI.rock.rockmasssum+untGUI.rock.rockHornblende[2];
    untGUI.rock.rockmasssum:=untGUI.rock.rockmasssum+untGUI.rock.rockFayalite[2];
    untGUI.rock.rockmasssum:=untGUI.rock.rockmasssum+untGUI.rock.rockQuartz[2];
    untGUI.rock.rockmasssum:=untGUI.rock.rockmasssum+untGUI.rock.rockChlorite[2];
    untGUI.rock.rockmasssum:=untGUI.rock.rockmasssum+untGUI.rock.rockKaolinite[2];
    untGUI.rock.rockmasssum:=untGUI.rock.rockmasssum+untGUI.rock.rockIllite[2];
    untGUI.rock.rockmasssum:=untGUI.rock.rockmasssum+untGUI.rock.rockMontmorillonite[2];
    untGUI.rock.rockmasssum:=untGUI.rock.rockmasssum+untGUI.rock.rockAmorphite[2];
    untGUI.rock.rockmasssum:=untGUI.rock.rockmasssum+untGUI.rock.rockOtherite[2];
    untGUI.rock.rockmasssum:=untGUI.rock.rockmasssum+untGUI.rock.rockGibbsite[2];
    untGUI.rock.rockmasssum:=untGUI.rock.rockmasssum+untGUI.rock.rockCalcite[2];
    untGUI.rock.rockmasssum:=untGUI.rock.rockmasssum+untGUI.rock.rockGypsum[2];

    {use effectively mass fractions bij /rock.rockmasssum}
    {now calculate individual mineral areas and convert units from m2/g to m2/mol using the mass per mineral in rock}
    Aalb :=   untGUI.rock.rockAlbite[2] * Atot * RFMalbite/untGUI.rock.rockmasssum;
    Akfeld := untGUI.rock.rockKFeldspar[2] * Atot * RFMkfeldspar/untGUI.rock.rockmasssum;
    Amusco := untGUI.rock.rockMuscovite[2] * Atot * RFMmuscovite/untGUI.rock.rockmasssum;
    Abio :=   untGUI.rock.rockBiotite[2] * Atot * RFMbiotite/untGUI.rock.rockmasssum;
    Aqtz :=   untGUI.rock.rockQuartz[2] * Atot * RFMquartz/untGUI.rock.rockmasssum;
    Achlor := untGUI.rock.rockChlorite[2] * Atot * RFMchlorite/untGUI.rock.rockmasssum;
    Aanort := untGUI.rock.rockAnorthite[2] * Atot * RFManorthite/untGUI.rock.rockmasssum;
    AForster := untGUI.rock.rockForsterite[2] * Atot * RFMForsterite/untGUI.rock.rockmasssum;
    Aaugit := untGUI.rock.rockAugite[2] * Atot * RFMaugite/untGUI.rock.rockmasssum;
    Akaol :=  untGUI.rock.rockKaolinite[2] * Atot * RFMkaolinite/untGUI.rock.rockmasssum;
    Aillit := untGUI.rock.rockIllite[2] * Atot * RFMillite;
    Amontm := untGUI.rock.rockMontmorillonite[2] * Atot * RFMmontmorillonite/untGUI.rock.rockmasssum;
    Ahornb := untGUI.rock.rockHornblende[2] * Atot * RFMhornblende/untGUI.rock.rockmasssum;
    Afayal := untGUI.rock.rockFayalite[2] * Atot * RFMfayalite/untGUI.rock.rockmasssum;
    if RFMamorphite>0 then Aamorp:= untGUI.rock.rockAmorphite[2] * Atot * RFMamorphite/untGUI.rock.rockmasssum else Aamorp:=0;
    if RFMotherite>0 then Aother:= untGUI.rock.rockOtherite[2] * Atot * RFMotherite/untGUI.rock.rockmasssum else Aother:=0;
    {added:}
    AGibbs:=untGUI.rock.rockGibbsite[2] * Atot * RFMGibbsite/untGUI.rock.rockmasssum;
    ACalci:=untGUI.rock.rockCalcite[2] * Atot * RFMcalcite/untGUI.rock.rockmasssum;
    AGypsu:=untGUI.rock.rockGypsum[2] * Atot * RFMGypsum/untGUI.rock.rockmasssum;

    {begin correcting mineral dissolution rate constants for temperatures}
    Temp_diff := (1 / T_const - 1 / (TEM[ilayer] + 273.15));
    KHalb := MinDisPar[1, 1] * Exp((MinDisPar[2, 1] / R) * (Temp_diff));
    KOHalb := MinDisPar[4, 1] * Exp((MinDisPar[5, 1] / R) * (Temp_diff));
    KHkfeld := MinDisPar[1, 2] * Exp((MinDisPar[2, 2] / R) * (Temp_diff));
    KOHkfeld := MinDisPar[4, 2] * Exp((MinDisPar[5, 2] / R) * (Temp_diff));
    KHmusco := MinDisPar[1, 3] * Exp((MinDisPar[2, 3] / R) * (Temp_diff));
    KOHmusco := MinDisPar[4, 3] * Exp((MinDisPar[5, 3] / R) * (Temp_diff));
    KHbio := MinDisPar[1, 4] * Exp((MinDisPar[2, 4] / R) * (Temp_diff));
    KOHbio := MinDisPar[4, 4] * Exp((MinDisPar[5, 4] / R) * (Temp_diff));
    KHqtz := MinDisPar[1, 5] * Exp((MinDisPar[2, 5] / R) * (Temp_diff));
    KOHqtz := MinDisPar[4, 5] * Exp((MinDisPar[5, 5] / R) * (Temp_diff));
    KHchlor := MinDisPar[1, 6] * Exp((MinDisPar[2, 6] / R) * (Temp_diff));
    KOHchlor := MinDisPar[4, 6] * Exp((MinDisPar[5, 6] / R) * (Temp_diff));
    KHanort := MinDisPar[1, 7] * Exp((MinDisPar[2, 7] / R) * (Temp_diff));
    KOHanort := MinDisPar[4, 7] * Exp((MinDisPar[5, 7] / R) * (Temp_diff));
    KHForster := MinDisPar[1, 8] * Exp((MinDisPar[2, 8] / R) * (Temp_diff));
    KOHForster := MinDisPar[4, 8] * Exp((MinDisPar[5, 8] / R) * (Temp_diff));
    KHaugit := MinDisPar[1, 9] * Exp((MinDisPar[2, 9] / R) * (Temp_diff));
    KOHaugit := MinDisPar[4, 9] * Exp((MinDisPar[5, 9] / R) * (Temp_diff));
    KHkaol := MinDisPar[1, 10] * Exp((MinDisPar[2, 10] / R) * (Temp_diff));
    KOHkaol := MinDisPar[4, 10] * Exp((MinDisPar[5, 10] / R) * (Temp_diff));
    KHillit := MinDisPar[1, 11] * Exp((MinDisPar[2, 11] / R) * (Temp_diff));
    KOHillit := MinDisPar[4, 11] * Exp((MinDisPar[5, 11] / R) * (Temp_diff));
    KHmontm := MinDisPar[1, 12] * Exp((MinDisPar[2, 12] / R) * (Temp_diff));
    KOHmontm := MinDisPar[4, 12] * Exp((MinDisPar[5, 12] / R) * (Temp_diff));
    KHhornb := MinDisPar[1, 13] * Exp((MinDisPar[2, 13] / R) * (Temp_diff));
    KOHhornb := MinDisPar[4, 13] * Exp((MinDisPar[5, 13] / R) * (Temp_diff));
    KHfayal := MinDisPar[1, 14] * Exp((MinDisPar[2, 14] / R) * (Temp_diff));
    KOHfayal := MinDisPar[4, 14] * Exp((MinDisPar[5, 14] / R) * (Temp_diff));
    KHamorp := MinDisPar[1, 15] * Exp((MinDisPar[2, 15] / R) * (Temp_diff));
    KOHamorp := MinDisPar[4, 15] * Exp((MinDisPar[5, 15] / R) * (Temp_diff));
    KHother := MinDisPar[1, 16] * Exp((MinDisPar[2, 16] / R) * (Temp_diff));
    KOHother := MinDisPar[4, 16] * Exp((MinDisPar[5, 16] / R) * (Temp_diff));
    {added}
    KHCalci:=power(10,-1.5)*Exp((14.4/R)*(Temp_diff));
    KOHCalci:=power(10,-5.0)*Exp((35.4/R)*(Temp_diff));
    KHGypsu:=power(10,-4.58)*Exp((-0.46/R)*(Temp_diff));
    KOHGypsu:=power(10,-4.58)*Exp((-0.46/R)*(Temp_diff));
    KHGibbsi:=power(10,-7.65)*Exp((47.5/R)*(Temp_diff));
    KOHGibbsi:=power(10,-16.65)*Exp((80.1/R)*(Temp_diff));


    {PF2023: Here a pro-forma call to IAPOMEGA_sapro to calcuate
     1. the Saturation degree Omega (array OMEGA+OmegaGibb,OmegaGyps,OmegaCalc)
     2. the Aluminum inhibition factor finh (FINH)         }
    IAPOMEGA_sapro(saprOmega,saprfinh,OmegaGibb,OmegaGyps,OmegaCalc);

    {calculation of mineral dissolution rates,as a function of pH and pOH}
    {denote that KH, KOH, MinDisPar[3,*] and MinDisPar[6,*] can have the value 0 in case of no effect or no data.
     PF2023: rjk* rates are adapted by a factor (1-omega[ispecies])*finh[ilayer].
     For Quartz,Chlorite, Kaolinite, Illite, Montmorillonite, Gibbsite, Gypsum and Calcite omega may be >1,
     so their rates may become negative (newformation). We prepared for such situation by
     adding a reduction factor redfacQtz, redfacChlor, redfacKaol, redfacIllit, redfacMontm
     These have initially the value 1}
    tempHact := power(10, -pH[ilayer]);
    tempOHact := power(10, -(14 - pH[ilayer]));
    rjkalb := (KHalb * power(tempHact, MinDisPar[3, 1]) + KOHalb *
      power(tempOHact, MinDisPar[6, 1])){*(1-saprOmega[1])*saprfinh[ilayer]};
    rjkkfeld := (KHkfeld * power(tempHact, MinDisPar[3, 2]) + KOHkfeld *
      power(tempOHact, MinDisPar[6, 2])){*(1-saprOmega[2])*saprfinh[ilayer]};
    rjkmusco := (KHmusco * power(tempHact, MinDisPar[3, 3]) + KOHmusco *
      power(tempOHact, MinDisPar[6, 3])){*(1-saprOmega[3])*saprfinh[ilayer]};
    rjkbio := (KHbio * power(tempHact, MinDisPar[3, 4]) + KOHbio *
      power(tempOHact, MinDisPar[6, 4])){*(1-saprOmega[4])*saprfinh[ilayer]};
    rjkanort := (KHanort * power(tempHact, MinDisPar[3, 7]) + KOHanort *
      power(tempOHact, MinDisPar[6, 7])){*(1-saprOmega[7])*saprfinh[ilayer]};
    rjkForster := (KHForster * power(tempHact, MinDisPar[3, 8]) +
      KOHForster * power(tempOHact, MinDisPar[6, 8])){*(1-saprOmega[8])*saprfinh[ilayer]};
    rjkaugit := (KHaugit * power(tempHact, MinDisPar[3, 9]) +
      KOHaugit * power(tempOHact, MinDisPar[6, 9])){*(1-saprOmega[9])*saprfinh[ilayer]};
    rjkhornb := (KHhornb * power(tempHact, MinDisPar[3, 13]) + KOHhornb *
      power(tempOHact, MinDisPar[6, 13])){*(1-saprOmega[13])*saprfinh[ilayer]};
    rjkfayal := (KHfayal * power(tempHact, MinDisPar[3, 14]) + KOHfayal *
      power(tempOHact, MinDisPar[6, 14])){*(1-saprOmega[14])*saprfinh[ilayer]};
    rjkamorp:= (KHamorp * power(tempHact, MinDisPar[3, 15]) + KOHamorp *
      power(tempOHact, MinDisPar[6, 15])){*(1-saprOmega[15])*saprfinh[ilayer]};
    rjkother:= (KHother * power(tempHact, MinDisPar[3, 16]) + KOHother *
      power(tempOHact, MinDisPar[6, 16])){*(1-saprOmega[16])*saprfinh[ilayer]};
    {PF2023: now the secundary minerals that may precipitate.
    Adaptation of rjk* and redfac* is needed if updated concentrations of each element would become negative
    due to excessive precipitation. However, we set all saprOmega to 0, thus only weathering, no newformation}
    rjkqtz := (KHqtz * power(tempHact, MinDisPar[3, 5]) + KOHqtz *
      power(tempOHact, MinDisPar[6, 5])){*(1-saprOmega[5])*saprfinh[ilayer]*redfacQtz};
    rjkchlor := (KHchlor * power(tempHact, MinDisPar[3, 6]) + KOHchlor *
      power(tempOHact, MinDisPar[6, 6])){*(1-saprOmega[6])*saprfinh[ilayer]*redfacChlor};
    rjkkaol := (KHkaol * power(tempHact, MinDisPar[3, 10]) + KOHkaol *
      power(tempOHact, MinDisPar[6, 10])){*(1-saprOmega[10])*saprfinh[ilayer]*redfacKaol};
    rjkillit := (KHillit * power(tempHact, MinDisPar[3, 11]) + KOHillit *
      power(tempOHact, MinDisPar[6, 11])){*(1-saprOmega[11])*saprfinh[ilayer]*redfacIllit};
    rjkmontm := (KHmontm * power(tempHact, MinDisPar[3, 12]) + KOHmontm *
      power(tempOHact, MinDisPar[6, 12])){*(1-saprOmega[12])*saprfinh[ilayer]*redfacMontm};
      {added}
    rjkGibb := (KHGibbsi * power(tempHact, 0.99) + KOHGibbsi *
      power(tempOHact, -0.78)){*(1-OmegaGibb)*saprfinh[ilayer]*redfacGibb};
    {Gypsum: no pH-dependent weathering, so just effect of solution in water: exponents n and m are=0}
    rjkGyps := (KHGypsu * power(tempHact, 0) + KOHGypsu *
      power(tempOHact, 0)){*(1-OmegaGyps)*saprfinh[ilayer]*redfacGyps};
    rjkCalc := (KHCalci * power(tempHact, 0.7) + KOHCalci *
      power(tempOHact, 0.1)){*(1-OmegaCalc)*saprfinh[ilayer]*redfacCalc};

    {indicator (test purpose)}
    {str(omega[5]:5:3,txt); untGUI.frmMain.lblNOTE.Caption:=txt ;
    str(omega[6]:5:3,txt); untGUI.frmMain.lblNOTE.Caption:=untGUI.frmMain.lblNOTE.Caption+'  '+txt ;
    str(omega[10]:5:3,txt); untGUI.frmMain.lblNOTE.Caption:=untGUI.frmMain.lblNOTE.Caption+'  '+txt ;
    str(omega[11]:5:3,txt); untGUI.frmMain.lblNOTE.Caption:=untGUI.frmMain.lblNOTE.Caption+'  '+txt ;
    str(omega[12]:5:3,txt); untGUI.frmMain.lblNOTE.Caption:=untGUI.frmMain.lblNOTE.Caption+'  '+txt ;
    untGUI.frmMain.lblNOTE.Refresh; }
    {end indicator (test)}
    {PF2023: here we COULD update rjk* and redfac* by AdaptRjkRedfac_sapro
     so that concentrations of each element will remain positive after precipitation of a secundary mineral.}
    {calculation of fluxes (moles/m2/day) of minerals that dissolve}
    lossAlbite := Aalb * rjkalb * Moalb * secperday * T;
    {moles/m2,day from m2/mol * mol/m2,s * mol/m3soil * s/day * m } ;
    lossKfeldspar := Akfeld * rjkkfeld * Mokfeld * secperday * T;
    {moles/m2,day } ;
    lossMuscovite := Amusco * rjkmusco * Momusco * secperday * T;
    {moles/m2,day } ;
    lossBiotite := Abio * rjkbio * Mobio * secperday * T;
    {moles/m2,day } ;
    lossQuartz := Aqtz * rjkqtz * Moqtz * secperday * T;
    {moles/m2,day } ;
    lossChlorite := Achlor * rjkchlor * Mochlor * secperday * T;
    {moles/m2,day } ;
    lossAnorthite := Aanort * rjkanort * Moanort * secperday * T;
    {moles/m2,day } ;
    lossForsterite := AForster * rjkForster * MoForster * secperday * T;
    {moles/m2,day } ;
    lossAugite := Aaugit * rjkaugit * Moaugit * secperday * T;
    {moles/m2,day } ;
    lossKaolinite := Akaol * rjkkaol * Mokaol * secperday * T;
    {moles/m2,day } ;
    lossIllite := Aillit * rjkillit * Moillit * secperday * T;
    {moles/m2,day } ;
    lossMontmorillonite := Amontm * rjkmontm * Momontm * secperday * T;
    {moles/m2,day } ;
    lossHornblende := Ahornb * rjkhornb * Mohornb * secperday * T;
    {moles/m2,day } ;
    lossFayalite := Afayal * rjkfayal * Mofayal * secperday * T;
    {moles/m2,day } ;
    lossAmorphite := Aamorp * rjkamorp * Moamorp * secperday * T;
    {moles/m2,day } ;
    lossOtherite := Aother * rjkother * Moother * secperday * T;
    {moles/m2,day } ;
    {added}
    lossCalcite  := ACalci * rjkcalc * MoCalci * secperday * T;
    lossGypsum   := AGypsu * rjkgyps * MoGypsu * secperday * T;
    lossGibbsite := AGibbs * rjkGibb * MoGibbsi * secperday * T;
  {calculation of moles and mass of minerals, each time subtracting what goes to solution:
  moles/m2 of minerals remaining and kg of minerals remaining after 1 timestep of 1 day}

    Mokfeld := max(0,Mokfeld - lossKfeldspar);
    untGUI.rock.rockKFeldspar[2] := Mokfeld * RFMkfeldspar / 1000;
    {in kg again};
    Momusco := max(0,Momusco - lossMuscovite);
    untGUI.rock.rockMuscovite[2] := Momusco * RFMmuscovite / 1000;
    Moqtz := max(0,Moqtz - lossQuartz);
    untGUI.rock.rockQuartz[2] := Moqtz * RFMquartz / 1000;
    Moalb := max(0,Moalb - lossAlbite);
    untGUI.rock.rockAlbite[2] := Moalb * RFMalbite / 1000;
    Mobio := max(0,Mobio - lossBiotite);
    untGUI.rock.rockBiotite[2] := Mobio * RFMbiotite / 1000;
    Mochlor := max(0,Mochlor - lossChlorite);
    untGUI.rock.rockChlorite[2] := Mochlor * RFMchlorite / 1000;
    Moanort := max(0,Moanort - lossAnorthite);
    untGUI.rock.rockAnorthite[2] := Moanort * RFManorthite / 1000;
    MoForster := max(0,MoForster - lossForsterite);
    untGUI.rock.rockForsterite[2] := MoForster * RFMForsterite / 1000;
    Moaugit := max(0,Moaugit - lossAugite);
    untGUI.rock.rockAugite[2] := Moaugit * RFMaugite / 1000;
    Mokaol := max(0,Mokaol - lossKaolinite);
    untGUI.rock.rockKaolinite[2] := Mokaol * RFMkaolinite / 1000;
    Moillit := max(0,Moillit - lossIllite);
    untGUI.rock.rockIllite[2] := Moillit * RFMillite / 1000;
    Momontm := max(0,Momontm - lossMontmorillonite);
    untGUI.rock.rockMontmorillonite[2] := Momontm * RFMmontmorillonite / 1000;
    Mohornb := max(0,Mohornb - lossHornblende);
    untGUI.rock.rockHornblende[2] := Mohornb * RFMhornblende / 1000;
    Mofayal := max(0,Mofayal - lossFayalite);
    untGUI.rock.rockFayalite[2] := Mofayal * RFMfayalite / 1000;
    Moamorp := max(0,Moamorp - lossAmorphite);
    if RFMamorphite>0 then untGUI.rock.rockAmorphite[2] := Moamorp * RFMamorphite / 1000;
    Moother := max(0,Moother - lossOtherite);
    if RFMotherite>0 then untGUI.rock.rockOtherite[2] := Moother * RFMotherite / 1000;
    {added}
    MoCalci := max(0,MoCalci - lossCalcite);
    untGUI.rock.rockCalcite[2] := MoCalci * RFMCalcite / 1000;
    MoGypsu := max(0,MoGypsu - lossGypsum);
    untGUI.rock.rockGypsum[2] := MoGypsu * RFMGypsum / 1000;
    MoGibbsi := max(0,MoGibbsi - lossGibbsite);
    untGUI.rock.rockGibbsite[2] := MoGibbsi * RFMGibbsite / 1000;

    {calculate mass lost relative to the initial amount}
    {mass lost up to this day during this year}
    untGUI.rock.rockAlbite[4]:=    untGUI.Rock_original.rockAlbite[2]-untGUI.rock.rockAlbite[2];
    untGUI.rock.rockKFeldspar[4]:= untGUI.Rock_original.rockKFeldspar[2]-untGUI.rock.rockKFeldspar[2];
    untGUI.rock.rockMuscovite[4]:= untGUI.Rock_original.rockMuscovite[2]-untGUI.rock.rockMuscovite[2];
    untGUI.rock.rockBiotite[4]:=   untGUI.Rock_original.rockBiotite[2]-untGUI.rock.rockBiotite[2] ;
    untGUI.rock.rockQuartz[4]:=    untGUI.Rock_original.rockQuartz[2]-untGUI.rock.rockQuartz[2] ;
    untGUI.rock.rockChlorite[4]:=  untGUI.Rock_original.rockChlorite[2]-untGUI.rock.rockChlorite[2] ;
    untGUI.rock.rockAnorthite[4]:= untGUI.Rock_original.rockAnorthite[2]-untGUI.rock.rockAnorthite[2] ;
    untGUI.rock.rockForsterite[4]:=untGUI.Rock_original.rockForsterite[2]-untGUI.rock.rockForsterite[2] ;
    untGUI.rock.rockForsterite[4]:=untGUI.Rock_original.rockForsterite[2]-untGUI.rock.rockForsterite[2] ;
    untGUI.rock.rockKaolinite[4]:= untGUI.Rock_original.rockKaolinite[2]-untGUI.rock.rockKaolinite[2] ;
    untGUI.rock.rockIllite[4]:=    untGUI.Rock_original.rockIllite[2]-untGUI.rock.rockIllite[2] ;
    untGUI.rock.rockMontmorillonite[4]:= untGUI.Rock_original.rockMontmorillonite[2]-untGUI.rock.rockMontmorillonite[2] ;
    untGUI.rock.rockHornblende[4]:=untGUI.Rock_original.rockHornblende[2]-untGUI.rock.rockHornblende[2] ;
    untGUI.rock.rockFayalite[4]:=  untGUI.Rock_original.rockFayalite[2]-untGUI.rock.rockFayalite[2] ;
    untGUI.rock.rockAmorphite[4]:= untGUI.Rock_original.rockAmorphite[2]-untGUI.rock.rockAmorphite[2] ;
    untGUI.rock.rockOtherite[4]:=  untGUI.Rock_original.rockOtherite[2]-untGUI.rock.rockOtherite[2] ;
    untGUI.rock.rockGibbsite[4]:=  untGUI.Rock_original.rockGibbsite[2]-untGUI.rock.rockGibbsite[2] ;
    untGUI.rock.rockCalcite[4]:=   untGUI.Rock_original.rockCalcite[2]-untGUI.rock.rockCalcite[2] ;
    untGUI.rock.rockGypsum[4]:=    untGUI.Rock_original.rockGypsum[2]-untGUI.rock.rockGypsum[2] ;

  {if (TRACE = 1) then}
    singletracewriteln(' Leaving WE_Daily_sapro');
end;

end.
